#include "cbes.h"

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *  Basis package for Bessel functions J0, J1, Y0, Y1, I0, I1, K0, K1.
 *  Bruce Langdon, LLNL, May 1989; last changed August 1991.
 *   (The J and Y functions were added by someone else and I haven't
 *    fully fixed them.)
 *  This is done as "built-in" functions, instead of just directly accessing
 *  i0 etc as compiled functions, so I can use vector arguments.
 *  With an example handy, it's not all that hard to do, so why not?
 */

Integer Besbfcn(Integer nargs, char *f, Integer *sx)
{
    float *x;
    Integer i, tc, length;
    static char
        msg1[] = "bes: illegal argument type.",
        msg2[] = "k0: argument <= 0.0.",
        msg3[] = "k1: argument <= 0.0.",
        msg4[] = "y0: argument <= 0.0.",
        msg5[] = "y1: argument <= 0.0.",
        msg6[] = "bes: function not implemented.";

    Fcnargb(1, sx);             /* sx is now data descriptor of the argument */
/*
 *
 *  We allow integer argument, e.g. i0(0), but not complex.
 *
 *  On the Sun, Basis now treats floating point constants as double's.
 *  Thus j0(1.2) started giving "Illegal argument" errors. This is
 *  now "fixed" to allow double arguments. These are coerced to real, which
 *  isn't really the way it should be done, but it works.
 *
 *    Jim (JAC); 7/16/91
 *
 *  Reprogrammed in C (ZCM 4/13/2000)
 *
 */

    tc = Utstrcod("real");
    if (sx[SS_TC - 1] == Utstrcod("integer") ||
        sx[SS_TC - 1] == Utstrcod("double"))
        i = Parcoerc(sx, &tc);
    if (sx[SS_TC - 1] != tc) {
        BasError(msg1, strlen(msg1));
        return ERR;
    }
/*
 *  Output has same type and dimension as input, and input is not actually the
 *  user's storage, so we can simply overwrite input with output.
 */
    length = Parlen(sx);
    x = (float *) Paraddr(sx);

/* All that's left is a case select for all the functions. */
    if (!strncmp(f, "i0", 2))
        for (i = 0; i < length; i++)
            x[i] = I0(x + i);
    else if (!strncmp(f, "i1", 2))
        for (i = 0; i < length; i++)
            x[i] = I1(x + i);
    else if (!strncmp(f, "k0", 2))
        for (i = 0; i < length; i++) {
            if (x[i] <= 0.) {
                BasError(msg2, strlen(msg2));
                return ERR;
            }
            x[i] = K0(x + i);
    } else if (!strncmp(f, "k1", 2))
        for (i = 0; i < length; i++) {
            if (x[i] <= 0.) {
                BasError(msg3, strlen(msg3));
                return ERR;
            }
            x[i] = K1(x + i);
    } else if (!strncmp(f, "j0", 2))
        for (i = 0; i < length; i++)
            x[i] = J0(x + i);
    else if (!strncmp(f, "j1", 2))
        for (i = 0; i < length; i++)
            x[i] = J1(x + i);
    else if (!strncmp(f, "y0", 2))
        for (i = 0; i < length; i++) {
            if (x[i] <= 0.) {
                BasError(msg4, strlen(msg4));
                return ERR;
            }
            x[i] = Y0(x + i);
    } else if (!strncmp(f, "y1", 2))
        for (i = 0; i < length; i++) {
            if (x[i] <= 0.) {
                BasError(msg5, strlen(msg5));
                return ERR;
            }
            x[i] = Y1(x + i);
    } else {
        BasError(msg6, strlen(msg6));
        return ERR;
    }
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
