	subroutine ezcapsfx(namer,ftype,fnsfx,fname,succ)
        character*(*) namer, ftype, fname
	integer fnsfx
	logical succ
# Subroutine to generate a unique filename with proper extension.
#
#   The returned fname will have the form  namer.namesfx.ext, where
#       namer    is the input "root" of the filename,
#       namesfx  is a unique 3-digit file suffix, and
#       ext      is an extension based on ftype.
#   In addition to namer, ftype, and fname, the arguments to ezcapsfx are:
#       fnsfx    is the trial value for namesfx on input.
#                If succ=.true., the returned value is one greater than
#                the actual suffix in the returned fname; i.e., the value
#                appropriate for the next call to ezcapsfx.
#                If fnsfx(in) was acceptable, fnsfx(out)=fnsfx(in)+1.
#       suff     is the output logical success/fail flag.
#                If suff=.false., then fname will be blank and fnsfx will
#                remain unchanged.
#   The ftype-to-ext mapping is done in subroutine ezcsfname.
#
#   NOTE: If fnsfx(in) >= 1000 (so not representable as a 3-digit number)
#         the generated name will have form  namer.1.namesfx.ext.  This
#         means that a total of 1999 filenames can be generated for a
#         given namer and ftype.  (Anything > 1999 will give error.)
#
#  Change record:
#    970304 (FNF)
#       Modified to always return file suffix.  Previous version had
#       if (ftype == "cgmlog" .or. ftype == "pslog") fnsfx = namesfx
#    970306 (FNF)
#       1. Added this preamble and some internal documentation.
#       2. Corrected coding to handle case fnsfx(in)>=1000 properly.
#    970310 (FNF)
#       1. Modified so that the filename after namer.999.ext is
#          namer.1.000.ext, rather than namer.999.001.ext.
#       2. Added code to check for filename truncation.
#    970311 (FNF)
#       Made namei,nameo,namew type Filename.
#    970312 (FNF)
#       Changed namer,fname from character*(80) to character*(*).
#    970520 (FNF)
#       Added auxiliary variable msg to avoid compilation error on Cray's.
# -------------------------------------------------------------------------
	Filename  namei, nameo, namew
	character*(3) cnamesfx  #character equivalent of namesfx
        character*(120) msg
	integer namesfx  #current trial suffix
	integer savesfx
	integer errsvl	#error severity level
	logical fex

	integer ezcgetcl
        logical ezccklen
	external ezccklen, ezcerror, ezcgetcl, ezcsfname
Use(Devices)

	succ = .false.
	namesfx = fnsfx
        do
           namei = namer(1:ezcgetcl(namer))
	   savesfx = namesfx
           if (namesfx>=1000) then
              namesfx = namesfx-1000
	      if (namesfx>=1000) goto 200  # Assume error if still too large.
	      namei = namer(1:ezcgetcl(namer))//"."//"1"
	      if (~ezccklen(namer,namei,2)) goto 200
           endif
           write(cnamesfx,'(i3.3)') namesfx
           namew = namei(1:ezcgetcl(namei))//"."//cnamesfx

	   # Following test is to guard against possible infinite loop
	   # in case of extremely long namei causing namew truncation.
           if (~ezccklen(namei,namew,4)) goto 200

	   # Append proper extension, based on ftype.
	   call ezcsfname(namew,ftype,nameo)
	   # Check for filename truncation.
	   if (ezcgetcl(nameo) <= ezcgetcl(namew)) goto 210

	   # Check whether a file by this name already exists.
           inquire(FILE=nameo,EXIST=fex)
           namesfx = namesfx + 1
           if (savesfx>=1000) namesfx = namesfx+1000
        until (~fex)
	# Assert: namesfx is one greater than the suffix used in nameo
	#        (or 1000 greater than that if suffix has rolled over).

	fname = nameo(1:ezcgetcl(nameo))
	fnsfx = namesfx
	succ = .true.
	return

#      Error exits
 200	continue
        msg = "ezcapsfx: Could not build unique suffix for "\
	    //ftype(1:ezcgetcl(ftype))//" file."
        errsvl = 0
        call ezcerror(msg,errsvl)
        goto 290

 210	continue
        msg = "ezcapsfx: Could not add proper extension to "\
	    //ftype(1:ezcgetcl(ftype))//" filename."
        errsvl = 0
        call ezcerror(msg,errsvl)

 290	continue
        fname = " "
        return
	end
