	integer function ezccoltb(indlo,indhi,red,green,blue)
c----------------------------------------------------------------------------
c*s
c	NAME OF ROUTINE... ezccoltb
c
c	AUTHOR OF ROUTINE...Steve Langer...May 13, 1992
c
c	PURPOSE...
c		Sets the RGB color corresponding to indexes "indlo" through
c		"indhi" for all active color windows and meta files.
c	
c	RETURNS...
c		zero for success, non-zero for bad arguments
c
c	GLOSSARY OF IMPORTANT VARIABLES...
c		indlo := Logical indices for low color ranges
c		indhi := Logical indices for high color ranges 
c		red =: Array of red color range
c		green =: Array of green color range
c		blue =: Array of blue color range	
c
c	OUTLINE...
c	  - Red, green, and blue are fractions of full intensity (0 through 1).
c
c     NOTE: This routine modifies color table entries with
c     logical indices indlo thru indhi. It thus adds an
c     offset based on indbase to the index before calling
c     the GKS set colour representation routine. The input
c     arrays red, green, and blue are indhi-indlo+1 elements
c     long, and are declared as having a range indlo:indhi to
c     avoid the need for yet another offset.
c
      integer indlo, indhi, cgmon, pson
      real(Size4) red(indlo:indhi), green(indlo:indhi), blue(indlo:indhi)
#
#  Revision history:
# (FNF 8/12/97) Changed "call remark" statements to "call ezcremrk".

      integer ncoli,cola
                # ncoli: number of colors available
                # cola: color availability indicator (GMONOC=0,GCOLOR=1)
#
Use(Colors)
Use(Devices)
c
      integer ilow, ihi, ii, jj, iwin
      character*80 msg
c
c	This function sets color index "indx" to a color with
c	fractional components red, green, blue.
c
c	be sure the requested indices are in range
c	NOTE: the input indices are indices into a
c	Fortran array, so they start at 1
	if(indlo .gt. indhi) then
	    ezccoltb= -10
	    return
	endif
	ilow= indlo
	if(ilow .lt. 1) then
	    ezccoltb= -1
	    return
	endif
	if(ilow .gt. numcol) then
	    ezccoltb= -2
	    return
	endif
	ihi= indhi
	if(ihi .lt. ilow) then
	    ezccoltb= -3
	    return
	endif
	if(ihi .gt. numcol) then
	    ezccoltb= -4
	    return
	endif
	ezccoltb= 0
c
c	the indices are legal, so go ahead and use them
c	for all active GKS color devices.
c
c       X11 workstation
        do iwin=1,iwinop
c          set colors only for open windows
           if(winstate(iwin)) then
             if(debcolr .ne. 0) then
                call ezcremrk(" setting colors for X11 window")
		call ezcremrk(winame(iwin))
             endif
c	check for monochrome X11 window
        # IF this is a monochrome X11 window, do not set any additional colors.
             cola = wincola(iwin)
             ncoli = winncoli(iwin)
             if (cola > 0) then
c            loop over all colors
                do ii=ilow,ihi
c               adjust so that a 1 becomes the base index
c               in the available part of the GKS color table
                   jj= ii-1+indbase
		   if (jj <= ncoli) then
                      call GSCR(iwsid(iwin), jj, red(ii), green(ii), blue(ii))
                      if (debcolr <> 0) then
                         write(msg,'(a,i8)') 'in ezccoltb: set indx = ',jj
                         call ezcremrk(msg)
                      endif
		   endif
                enddo
	     endif
           endif
        enddo
c
c       CGM workstation
        if(iezccgma >= 0) then
           if (debcolr .ne. 0) then
              call ezcremrk(" setting colors for CGM")
           endif
	   # skip the color table setting for mono CGM
           if (cgmclrmap <> "mono") then
c	     set flag to indicate whether the cgm file was on or off
	     if(iezccgma == 0) then
c               inactive cgm file so turn it on
	        cgmon= 0
	        call ezccgm("on"," ")
	     else
	        cgmon= 1
	     endif
c            loop over all colors
             do ii=ilow,ihi
c               adjust so that a 1 becomes the base index
c               in the available part of the GKS color table
                jj= ii-1+indbase
                call GSCR(CGMWKID, jj, red(ii), green(ii), blue(ii))
                if (debcolr <> 0) then
                   write(msg,'(a,i8)') 'in ezccoltb: set indx = ',jj
                   call ezcremrk(msg)
                endif
             enddo
c	     if inactive cgm file, turn it back off
	     if(cgmon == 0) call ezccgm("off"," ")
           endif
        endif
c
c       Postscript workstation
        if(iezcpsa >= 0) then
           if(debcolr .ne. 0) then
             call ezcremrk(" setting colors for PS")
           endif
	   # skip the color table setting for mono PS 
           if (psclrmap <> "mono") then
c	      set flag to indicate whether the postscript file was on or off
	      if(iezcpsa == 0) then
c             inactive ps file so turn it on
	         pson= 0
	         call ezcps("on","color")
	      else
	         pson= 1
	      endif
c             loop over all colors
              do ii=ilow,ihi
c                adjust so that a 1 becomes the base index
c                in the available part of the GKS color table
                 jj= ii-1+indbase
                 call GSCR(PSWKID, jj, red(ii), green(ii), blue(ii))
                 if (debcolr <> 0) then
                    write(msg,'(a,i8)') 'in ezccoltb: set indx = ',jj
                    call ezcremrk(msg)
                 endif
              enddo
c	      if inactive ps file, turn it back off
	      if(pson == 0) call ezcps("off"," ")
	   endif
        endif
	return
	end

        integer function ctablist()
c-------------------------------------------------------------------------------
c*s
c	NAME OF ROUTINE...ctablist 
c
c	AUTHOR OF ROUTINE...Steve Langer...May 13, 1992
c
c	PURPOSE...
c		Prints the current color table for the first window.
c
c	RETURNS...
c		zero for success, non-zero for X11 workstation not open
c
c	GLOSSARY OF IMPORTANT VARIABLES...
c
c	OUTLINE...
c
c
      real(Size4) red, green, blue
      integer errind, ii, jj, iwin
#
Use(Colors)
Use(Devices)
# (FNF 7/30/97) Changed "write(STDOUT,..." statements to "call remark".
# (FNF 8/12/97) Changed "call remark" statements to "call ezcremrk".
        character*80 msg
c
c	list all colors in the current color table for the first
c       active window
c
c       Return error if X11 workstation is not active
        if(debcolr .ne. 0) then
           write(msg,'(a,i8)') "in ctablist: numcol= ",numcol
           call ezcremrk(msg)
        endif
	if(iwinop == 0) then	# no open window exists
           ctablist = -1
           return
        endif
	ctablist= 0
c       loop over all windows
        do iwin=1,iwinop
c          show colors only for open windows
           if(winstate(iwin)) then
c	     loop over all colors
	     do ii=1,numcol
c               adjust so that a 1 becomes the base index
c               in the available part of the GKS color table
                jj= ii-1+indbase
                call GQCR(iwsid(iwin), jj, GREALI, errind, red, green, blue)
                write(msg,75) ii,red,green,blue
                call ezcremrk(msg)
75              format(' index=',i3,',  red=',f6.3,',  green=',f6.3,
     *	              ',  blue=',f6.3)
             enddo
c            done, so return
             return
           endif
         enddo
c        no active windows, so flag error
 	 ctablist= -10
	 return
	 end

	integer function ezcgetcol(indx)
c-------------------------------------------------------------------------------
c*s
c	NAME OF ROUTINE...ezcgetcol
c
c	AUTHOR OF ROUTINE...Steve Langer...May 13, 1992
c
c	PURPOSE...
c		Prints the color components for color index "indx"
c               for all open windows.
c
c	RETURNS...
c		zero for success, non-zero for bad arguments
c               or X11 workstation not open
c
c	GLOSSARY OF IMPORTANT VARIABLES...
c		indx =: Color index
c
c (FNF 1/08/98)  Corrected rgb output formats from i8 to f6.3.
c
	integer indx
	real(Size4) red, green, blue
        integer trundx, errind, iwin
#
Use(Colors)
Use(Devices)
# (FNF 7/30/97) Changed "write(STDOUT,..." statements to "call remark".
# (FNF 8/12/97) Changed "call remark" statements to "call ezcremrk".
        character*80 msg
c
c	This function prints the color components for color index "indx".
c	The use of GREALI means return the colors actually
c	used for this index, not what the last "set" call requested.
c
c       Return error if X11 workstation is not active
        if(debcolr .ne. 0) then
           write(msg,'(a,i8)') "in ezcgetcol: iwinop= ",iwinop
           call ezcremrk(msg)
        endif
	if(iwinop == 0) then	# no open window exists
           ezcgetcol = -2
           return
        endif
c       adjust the requested index to reflect presence of named colors
	trundx= indx-1+indbase
        if(debcolr .ne. 0) then
           write(msg,'(a,i8)') "trundx=",trundx
           call ezcremrk(msg)
        endif
	if(trundx .lt. 0 .or. trundx .gt. numcol-1+indbase) then
	    ezcgetcol= -1
	    return
	endif
        ezcgetcol= 0
c
c       loop over all windows
c       NOTE: will return red, green, and blue for first active window
        do iwin=1,iwinop
           if(debcolr .ne. 0) then
              write(msg,'(a,i8,a,i8)') "iwin=",iwin,", winstate=",winstate(iwin)
              call ezcremrk(msg)
           endif
c          show colors only for open windows
           if(winstate(iwin)) then
              call GQCR(iwsid(iwin), trundx, GREALI, errind, red, green, blue)
              if(debcolr .ne. 0) then
                 write(msg,'(a,i8,a,i8)') "in ezcgetcol: trundx= ",trundx,
                                          ", window= ",iwsid(iwin)
                 call ezcremrk(msg)
              endif
              write(msg,'(a,f6.3,a,f6.3,a,f6.3)') "red=",red,", green=",green,", blue=",blue
              call ezcremrk(msg)
              return
           endif
        enddo
c       No active window, so flag failure
	ezcgetcol= -10
	return
	end

	subroutine reqcolr(nreq)
c------------------------------------------------------------------------------
c*s
c	NAME OF ROUTINE...reqcolr
c
c	AUTHOR OF ROUTINE...Steve Langer...May 14, 1992
c
c	PURPOSE... set the number of colors in the color table
c                  (could be overridden due to device limits)
c		
c
        integer nreq
#
Use(Colors)
Use(Devices)
# (FNF 7/30/97) Changed "write(STDOUT,..." statement to "call remark".
# (FNF 8/12/97) Changed "call remark" statement to "call ezcremrk".
# (FNF 5/17/2000) Removed code related to (unsupported) tek output.
        character*80 msg
        if(debcolr .ne. 0) then
           write(msg,'(a,i8)') "in reqcolr: nreq= ",nreq
           call ezcremrk(msg)
        endif
c       do not do anything if there is a workstation of any kind open
        if(iwinop > 0 .or. iezccgma >= 0 .or. iezcpsa >= 0) return
	numcol= nreq
        if(numcol .lt. 0) numcol= 0
        if(numcol .gt. 240) numcol= 240
	return
	end
	subroutine backcolr(bkcolr)
c------------------------------------------------------------------------------
c*s
c	NAME OF ROUTINE...backcolr
c
c	AUTHOR OF ROUTINE...Steve Langer...Nov. 7, 1992
c
c	PURPOSE... use black background for windows if 0 and
c                  white background otherwise
c		
c
        integer bkcolr
#
Use(Colors)
Use(Devices)
# (FNF 7/30/97) Changed "write(STDOUT,..." statement to "call remark".
# (FNF 8/12/97) Changed "call remark" statement to "call ezcremrk".
        character*80 msg
# (FNF 5/17/2000) Removed code related to (unsupported) tek output.
        if(debcolr .ne. 0) then
           write(msg,'(a,i8)') "in backcolr: bkcolr= ",bkcolr
           call ezcremrk(msg)
        endif
c       do not do anything if there is a workstation of any kind open
        if(iwinop > 0 .or. iezccgma >= 0 .or. iezcpsa >= 0) return
	bakcol= bkcolr
        if(bakcol .ne. 0) bakcol= 1
	return
	end

