      subroutine ezdprobe (px, py, upx, upy, qflag)
#
#   Probe currently active window and return coordinates of point user
#   clicked on.
#
#   If the return value of qflag is zero, then:
#     px, py  are the screen coordinates of the point, and
#     upx,upy are the result of translating this to user coordinates.
#   Otherwise, the click was outside the plot frame.
#
# Change record:
# (FNF 7/23/97) Changed "write(STDERR,..." to "call ezcerror".
# (FNF 7/30/97) Changed "write(STDOUT,..." statement to "call remark".
# (FNF 8/12/97) Changed "call remark" statement to "call ezcremrk".
# (FNF 3/19/98) Modified to be active for NCAR GKS, too.
# (FNF 6/16/98) Minor clean-up to make work with J. Rowe's routines.
# (FNF 12/22/98) Added check for non-existent window.
# (FNF 1/14/99) Modified to ignore expose events.
# (FNF 3/17/99) Moved code that scans active window list to separate
#               routine, so calling routine can inform user of multiple
#               open windows, if appropriate.
# (FNF 5/17/2000) Removed code for ATC GKS, which is no longer supported.
# ----------------------------------------------------------------------
Use(Devices)

      integer stat # status
      integer tnr  # transformation number (probably useless)
      real(Size4) px, py  # position x,y
      real(Size4) upx, upy  # user coordinates of position x,y

      integer errind  #error index
      integer ldr
      integer qflag   #quit probe flag
      character*80 dr # datarecord
      character*80 errmsg

      integer wson         #Number of active windows
      integer wsid         #ID of active window
      character*16 wsname  #Name of active window
      character*32 wsdisp  #Display of active window

      integer ezcgetcl
      external ezcgetcl

      px = -1.0
      py = -1.0
      upx = 0.0
      upy = 0.0

      call ezdckwin (wson, wsid, wsname, wsdisp)

      if (wson < 1) then
         call ezcerror('No apparently open windows.',0)
	 qflag = 1
         return
      endif
      qflag = 0

      # empty data array
      errind = 0
      call gprec(0,1,0,1,0,0,'a',80,errind,ldr,dr)
      if (errind > 1) then
         write (errmsg,'(A,I10)') 'ezdprb: gprec returned errind =', errind
         call ezcerror(errmsg,0)
         qflag = 1
         return
      endif

      # init  page 9-3
#     don't trust this call; it can mess up subsequent windows
#      call ginlc(wsid,1,0,0.0,0.0,1,0.5,0.0,1.0,1.0,ldr,dr)

      # request locator position
  100 continue
      call grqlc(wsid, 1, stat, tnr, px,py)
      if (stat > 1) then
#         Ignore expose event; probe again.
         if (stat = 5) goto 100
         write (errmsg,'(A,I8)') 'ezdprb: grqlc returned stat =', stat
         call ezcerror(errmsg,0)
         write (errmsg,'(3A)') '        Window ', wsname(1:ezcgetcl(wsname)),
                               ' missing or inactive??'
         call ezcerror(errmsg,0)
         qflag = 1
         return
      endif


      # stat = GNONE can be produced by middle mouse in window
      #if (stat = GNONE) then
      #	  call ezcerror('NO valid point selected',0)
      #   return  # no valid point selected
      #endif

# to actual user coordinates
      call ezcprobe(px,py,upx,upy,qflag)

      return
      end
      
