        subroutine ezcps(iflag,istring)
        character*(*) iflag,istring
#     Routine to process command "ps iflag istring".
#
# Change record:
# (yymmdd means 19yy/mm/dd)
#  970305  (FNF)
#     1. Modified to make sure that ps and pslog file get same suffix.
#     2. Changed "write(STDOUT,..." statements to "call remark".
#     3. Additional minor housekeeping.
#  970311  (FNF)
#     Changed character*(80) declaration for fnameroot to Filename.
#  970516  Made "open" a synonym for "on".  (FNF)
#  970806  (FNF)
#     Moved declarations inside conditional compile block to eliminate
#     compiler warning messages.  Also improved NCAR error message.
#  970812  Changed "call remark" statements to "call ezcremrk".  (FNF)
#  971223  Implemented NCAR GKS option. (FNF)
#  971229  Cosmetic changes to make ezccgm and ezcps more similar. (FNF)
#  980107  Added gclrwk calls (NCAR GKS only). (FNF)
#  980626  Eliminated extra blank frame from "ps send" files. (FNF)
# (yymmdd means 20yy/mm/dd)
#  000517  Removed code for ATC GKS, which is no longer supported. (FNF)
#  000517  Removed code related to (unsupported) tek output. (FNF)
# -------------------------------------------------------------------------
Use(Devices)
Use(Colors)
        Filename  fnameroot
        character*(16) filetype
        character*(16) oldcmap    #Temp store of old colormap name
        integer iws, iwstype      #Integers for workstation id and type
        integer i, j, k
        integer verbfl
        logical winrvs(NWINDOWS), cgmrvs
        integer suffile, sufflog  #Temporary file suffix values
        logical fnsucc            #Flag for successful file name assignment
        character*(120) msg       #Error message

        integer rtgetverb
        external rtgetverb

        integer ezcgetcl
        external ezcgetcl

        integer idum

        idum = -1
        verbfl = rtgetverb()

	iws = PSWKID
	#Set postscript workstation type for NCAR GKS:
        if (istring <> "color" & psclrmap == "mono") then
           iwstype = 23		#portrait monochrome PS for NCAR GKS
        else
           iwstype = 20		#portrait color PS for NCAR GKS
        endif
        if (debcolr <> 0) then
           write (msg,'(a,i8)') "ezcps: set iwstype =",iwstype
           call ezcremrk (msg)
        endif
        call ezcp(1)
        if (iflag == "on" | iflag == "open") then
	   # Open the PS device.
           if (iezcpsa < 0) then
	     # Set the name of the PS file.
              if (fnroot == " ") fnroot = "Problem"
              fnameroot = fnroot(1:ezcgetcl(fnroot))

              #Append suffix, making sure ps and pslog get same one.
              # This takes care of case user has moved either old ps
              # files or old pslog files (but not both).
              # Note that this is not foolproof if there are gaps in the
              # suffixes on existing files.  (FNF 3/05/97)
              filetype = "ps"
              suffile = psfsfx
              call ezcapsfx(fnameroot,filetype,suffile,psfile,fnsucc)
              if (~fnsucc) goto 210
              filetype = "pslog"
              sufflog = psfsfx
              call ezcapsfx(fnameroot,filetype,sufflog,iezcpsf,fnsucc)
              if (~fnsucc) goto 200
              psfsfx = max(suffile, sufflog)
              if (suffile < psfsfx) then
                 filetype = "ps"
                 psfsfx = psfsfx-1  # Adjust to suffix actually used
                 call ezcapsfx(fnameroot,filetype,psfsfx,psfile,fnsucc)
                 if (~fnsucc) goto 210
              else if (sufflog < psfsfx) then
                 filetype = "pslog"
                 psfsfx = psfsfx-1  # Adjust to suffix actually used
                 call ezcapsfx(fnameroot,filetype,psfsfx,iezcpsf,fnsucc)
                 if (~fnsucc) goto 200
              endif

              # Now actually create/open the PS file.
	      # Set up NCAR GKS open workstation call.
              call ngsetc('ME', psfile(1:ezcgetcl(psfile)))
              call gopwk(iws,idum,iwstype)
	      # Notify user of the PS file name.
              if (verbfl .eq. 1) then
                 write(msg,105) psfile(1:ezcgetcl(psfile))
 105             format('Beginning PS File ',a)
                 call ezcremrk(msg)
              endif

              # Create/open PS log file and notify user.
              open(unit=PSLOGUNIT,file=iezcpsf(1:ezcgetcl(iezcpsf)),\
                    status='new',err=200)
              if (verbfl .eq. 1) then
                 write(msg,100) iezcpsf(1:ezcgetcl(iezcpsf))
 100             format('Beginning PS Log ',a)
                 call ezcremrk(msg)
              endif

              # Finish PS device initialization.
              call gacwk(iws)
              iezcpsa = 1
              iezcdev = iezcdev + 1
	      # Initialize color indices (for color PS).
              if (istring == "color" | psclrmap <> "mono") then
                 call ezccidx(iws,iwstype,psclrmap)
              else
                 psclrmap = "mono"
                 call ezcmono(iws,iwstype)
              endif
           endif

	   # Check for previously opened PS file.
           if (iezcpsa == 0) then
              call gacwk(iws)
              iezcdev = iezcdev + 1
              iezcpsa = 1
           endif

        elseif (iflag == "off") then
           if (iezcpsa > 0) then
              # Make sure picture properly terminated.
              call gclrwk(iws,0)
              # Now deactivate the device.
              call gdawk(iws)
              iezcdev = iezcdev - 1
              iezcpsa = 0
           endif

        elseif (iflag == "close") then
           if (iezcpsa >= 0) then
              if (iezcpsa>0) then
                 # Make sure picture properly terminated.
                 call gclrwk(iws,0)
                 # Now deactivate the device.
                 call gdawk(iws)
                 iezcdev = iezcdev - 1
              endif
              call gclwk(iws)
              iezcpsa = -1
              if (verbfl .eq. 1) then
                 write(msg,110) psfile(1:ezcgetcl(psfile)),iezcpsn
 110             format('Closed PS File ',a,', ',i5,' frames.')
                 call ezcremrk(msg)
              endif
              iezcpsn = 0
              close(PSLOGUNIT)
              if (verbfl .eq. 1) then
                 write(msg,505) iezcpsf(1:ezcgetcl(iezcpsf))
 505             format('Closed PS Log File ',a)
                 call ezcremrk(msg)
              endif
           endif

        elseif (iflag == "send") then
           # Deactivate CGM if it is on.
	   cgmrvs = FALSE
	   if (iezccgma > 0) then
	      call ezccgm("off"," ")
	      cgmrvs = TRUE
           endif

	   # Deactivate windows, if any.
	   if (iwinop > 0) then
              #initialize the window reverse state array 
	      do k=1,NWINDOWS
	         winrvs(k) = FALSE
	      enddo	
	      do i=1,iwinop
	         if (winstate(i)) then
	            winrvs(i) = TRUE
		    call ezcwin("off",winame(i))
		 endif
              enddo
           endif

           # Open PS and write frame to it.
           if (iezcpsa <= 0) then
               call ezcpsr("on",istring)
           endif
           ezcsend = TRUE
           call ezcshowg
           ezcsend = FALSE
           # Turn it off.
           call ezcpsr("off"," ")

           # Restore the orginal states before "send".
           if (cgmrvs) call ezccgm("on"," ")
	   if (iwinop >0) then
	      do j=1,iwinop
                 if (winrvs(j)) call ezcwin("on",winame(j))
	      enddo
           endif

        elseif (iflag == "colormap") then
           oldcmap = psclrmap
           psclrmap = istring
           if (debcolr <> 0) then
              msg = "PS colormap has been set to "//psclrmap(1:ezcgetcl(psclrmap))
              call ezcremrk(msg)
           endif
           if (iezcpsa < 0) then
	      call ezcpsr("on","color")
           else
	      if (oldcmap <> psclrmap) then
                 if (oldcmap == "mono" | psclrmap == "mono") then
                    call ezcremrk("Color and mono PS files are not switchable.")
                    call ezcremrk("Please close the current PS file and open a new one.")
                 elseif (iezcpsa == 0) then
                    call ezcpsr("on","color")
                    call ezccidx(iws,iwstype,psclrmap)
                    call ezcpsr("off"," ")
                 else
                    call ezccidx(iws,iwstype,psclrmap)
                 endif
              endif
           endif

        else
	   msg = "ezcps: ignoring unknown command- "//iflag(1:ezcgetcl(iflag))
           call ezcerror(msg,0)
        endif

        return

#      Error exits
200     msg = "Could not create PS log file."
	call ezcerror(msg,1)
	return
210     msg = "Could not create PS graphics file."
	call ezcerror(msg,1)
	return

        end

        subroutine ezcpsr(iflag,istring)
c recursion intermediary
        character*(*) iflag,istring
        call ezcps(iflag,istring)
        return
        end
