        subroutine ezcwin(iflag,istring)
        character*(*) iflag,istring
#     Routine to process command "win iflag istring".
#
# Change record:
# (yymmdd means 19yy/mm/dd)
#  970227  (FNF)
#     Fixed problem that allowed only first window to use colors when
#     loaded with NCAR-GKS.
#  970410  (FNF)
#     Numerous cosmetic changes to reduce unnecessary differences
#     between ATC and NCAR sections.
#  970516  Made "open" a synonym for "on". (FNF)
#  970723  Changed "write(STDERR,..." statement to "call ezcerror".
#  970730  Changed "write(STDOUT,..." statement to "call remark".
#  970812  Changed "call remark" statements to "call ezcremrk".  (FNF)
#  980107  Added "colormap" command to NCAR section. (FNF)
#  980107  Consolidated the ATC and NCAR sections, since they now have
#          only relatively small differences. (FNF)
#  980108  Added optional background/foreground color queries. (FNF)
#  980109  Eliminated redundant increment of iwsidset. (FNF)
#  980116  Implemented new commands "list", "slist", "close all". (FNF)
#  980319  Added call to Jeff Rowe's new NCAR X-window labeler. (FNF)
#  980612  Replaced that with a call to Jeff Rowe's new NCAR X-window
#          creation routine gxwin. (FNF)
#  980615  Added matching call to gdwin. (FNF)
#  980617  Minor corrections to list commands. (FNF)
#  980619  Changes for Jeff Rowe's alterations to gxwin. (FNF)
#  980624  Corrected bug in "win close all". (FNF)
#  980626  Corrected bugs when ezcwinlb is nonblank. (FNF)
#  981204  Implemented user-specifiable window size for NCAR GKS. (FNF)
#          This is done via two new global variables:
#             ezcwinht is requested window height in inches, if >0;
#             ezcwinwd is requested window width in inches, if >0.
#  981211  Modified so ezcwinht,ezcwinwd also affect ATC GKS. (FNF)
#  981222  Added workstation ID to gxwin call sequence. (FNF)
#  990115  Changed 'Window Name' to 'EZC Window' in window labels. (FNF)
#  990125  Added new routine ezcwoff to deactivate a window by table
#          index.  This eliminates a redundant table search and avoids
#          a potential recursion bug. (FNF)
#  990316  Added dispnam to gidpy and gxwin call lists for Jeff Rowe's
#          990308 update. (FNF)
#  990326  Corrected indexing error in loop that deactivates other
#          windows when an existing window hs been activated. (FNF)
#  991103  Changed idwind and iwscn from real(Size8) to integer(2). (FNF)
#  991104  Modified to work on IRIX64 system. (FNF)
#  991108  Moved idwind declaration to ezd.v in order to take advantage
#          of AXP "-align dcommons" to cure alignment problem. (FNF)
# (yymmdd means 20yy/mm/dd)
#  000210  Modified to account for change to type of idwind in ezd.v
#          and associate iconn with correct half of idwind. (FNF)
#  000222  Changed local iconn (now back in ezd.v) to iconloc. (FNF)
#  000517  Removed code for ATC GKS, which is no longer supported. (FNF)
#  000517  Removed code related to (unsupported) tek output. (FNF)
#  000518  Changed iconloc to more descriptive iconn32, and removed
#          code to save it in iconn (which is no longer defined).  (FNF)
# ---------------------------------------------------------------------
Use(Devices)
Use(Colors)

	character*120 msg	#error message
        integer ezcgetcl 
        external ezcgetcl
	integer errsvl
        integer errind
	integer iwstype		# workstation type 

        integer iconn32         # 32-bit copy of connection ID, for NCAR
#       ---------------
#         The following strange-looking set of ifelse's is there due to
#         the fact that the Solaris compilers f77 and f90 are mutually
#         antagonistic.  We have forced all variables to be AUTOMATIC on
#         Solaris platforms, in order to allow recursion, by use of the
#         -stackvar compiler option.  However, Solaris f77 doesn't allow
#         equivalencing of an automatic variable to one in common.  The
#         SAVE declaration negates AUTOMATIC, and the compiler is happy.
#         On the other hand, the Solaris f90 compiler cannot stand this,
#         complaining that an object with the SAVE attribute cannot be
#         equivalenced to an object in a common block.  Go figure!
#            FNF - 10/02/2000
#         NOTE: none of the other compilers we have tested object to the
#         equivalence.
#       ---------------
ifelse(SYSTEM,SOL,[
ifelse(COMPILER,F77,[
        save iconn32
])
])
#       Set up association between iconn32 and idwind.
#       ---------------
#         The following is due to an "endian" issue with respect to
#         IRIX64.  The problem is that idwind is a 64-bit quantity
#         (which is actually a pointer to a Window in C) that needs to
#         be passed on to the NCAR routine gopwk in order to set up a
#         proper connection.  However, NCAR assumes 32-bit integers, so
#         it is important that the proper part of idwind is picked up
#         for the call.  This is accomplished by the following rather
#         convoluted set of system-dependent equivalences, in which the
#         left half is picked up on most platforms, but the right half
#         on IRIX64.
#            FNF - 10/02/2000
#       ---------------
ifelse(SYSTEM,IRIX64,[
        integer idwind64(2)
        equivalence (idwind,idwind64)
        equivalence (idwind64(2),iconn32)
],[
        equivalence (idwind,iconn32)
])
	integer idx,indx,j,i
	integer idxfd,iwsidfd
	logical found,cgmrvs,psrvs
        character*32 display, dispnam
        # variables needed for colormap id query
        character*16 namestore	#Truncated version of name (as stored)
        character*16 oldcmap    #temp store of old colormap name
        # variables needed for bg/fgcolor query
        real(Size4) cr, cg, cb
        # variables for window labeling, etc.
        real(Size4) winht, winhtdf, winwd, winwddf
	character*34 winlabl	#Must be 18 longer than namestore
	character*32 winlbpx

        character*17 iconlabl	#Must be one longer than winame.

        integer htinmm, htinpx, iwinht, iwinwd, wdinmm, wdinpx

        iwstype=7   # window workstation type for NCAR (per J. Rowe)
	winlbpx = 'EZC Window: '
        winhtdf = 7.           #Default window height (inches)
        winwddf = 7.           #Default window width (inches)

	if (ezcdisp <> " ") then
	   display = ezcdisp
           call ezcsdisp("DISPLAY="//ezcdisp)
	else 
           display = " "
	   call getenv("DISPLAY",display)
	   if (display == " ") then
              call ezcremrk("You must set environment variable DISPLAY")
              call ezcremrk("or variable ezcdisp")
              call kaboom(0)
           endif
	endif

        call ezcp(1)
	if (iwinop == 0) then	# no open window exists
	  found = FALSE
          if (iflag <> "on" & iflag <> "open" & iflag <> "colormap") then
	     msg = "No open windows, ignoring command- "\
			//iflag(1:ezcgetcl(iflag))
	     errsvl = 0
	     call ezcerror(msg,errsvl)
	     return
	  endif
	else	# open window exists
		# try to find existing open window with the given name
	   found = FALSE	# initialize found
           namestore = istring	# Truncate to length of stored names.
	   do idx = 1, iwinop
	      if( (winame(idx) == namestore) .or. \
                 ((namestore==" ").and.(winame(idx)==ezcwinlb)) )  then
		found = TRUE
		iwsidfd = iwsid(idx)	# workstation id found
		idxfd = idx		# index in the list of open windows
		break			# break the do loop
	      endif
	   enddo
         endif

        # check the assciated command for the window
	if (iflag == "on" | iflag == "open") then
	   if ( .not. found) then	#open a new window
	      #increment iwinop only after success.
	      iwsidset = iwsidset + 1
	      
	      #assign the proper name to the window
     	      if (istring <> " ") then
                 namestore = istring(1:ezcgetcl(istring))
	      elseif (ezcwinlb <> " ") then
		 namestore = ezcwinlb(1:ezcgetcl(ezcwinlb))
	      else
	         namestore = " "
	      endif
              winame(iwinop+1) = namestore

              iwsid(iwinop+1) = iwsidset
	      windisp(iwinop+1) = display	
	      #record the associating display with window

	      #Set up for open workstation call.

	      # set up window label (with terminal null character)
	      if (namestore <> " ") then
		 winlabl = winlbpx(1:ezcgetcl(winlbpx))//" "\
                           //namestore(1:ezcgetcl(namestore)) // char(0)
	      else
                 winlabl = "EZC NCAR Xgks" // char(0)
	      endif

              # set up window size
              if (ezcwinht > 0) then
                 winht = ezcwinht
              else
                 winht = winhtdf
              endif
              if (ezcwinwd > 0) then
                 winwd = ezcwinwd
              else
                 winwd = winwddf
              endif
#
#             Get the size parameters for the display:
#               wdinpx = width  in pixels; wdinmm = width  in millimeters;
#               htinpx = height in pixels; htinmm = height in millimeters.
#             First, add null character to display name for C.
                dispnam = display(1:ezcgetcl(display))//char(0)
              call gidpy (dispnam(1:ezcgetcl(dispnam)),
                          wdinpx, htinpx, wdinmm, htinmm, idum, istat)
              if (istat > 0) then
                write (msg,'(a,i5)') "ezcwin: GIDPY returned istat =", istat
                call ezcerror(msg,0)
                return
              endif
#
#             Convert window size specifications from inches to pixels.
#               (Note: 25.4 mm = 1 inch.)
              iwinht = 25.4*(real(htinpx)/real(htinmm))*winht
              iwinht = min(iwinht, htinpx)  # Make sure it fits.
              iwinwd = 25.4*(real(wdinpx)/real(wdinmm))*winwd
              iwinwd = min(iwinwd, wdinpx)  # Make sure it fits.

#        Open new window -- NCAR GKS version ---------
	      #Call Jeff Rowe's NCAR X-Window Creater (June 1998)
             if (winame(iwinop+1) <> " ") then
                # Append a null character for C.
                iconlabl = winame(iwinop+1)(1:ezcgetcl(winame(iwinop+1))) \
                           // char(0)
             else
                iconlabl = "Xgks" // char(0)
             endif
             errind = 0
#               Note that the following sets idwind.
#               Open window with mixed color model.
             call gxwin (iwsid(iwinop+1), dispnam(1:ezcgetcl(dispnam)),
                         0,0, iwinwd,iwinht, winlabl(1:ezcgetcl(winlabl)),
                         iconlabl(1:ezcgetcl(iconlabl)), 2, errind, idwind)
             if (errind > 0) then
                call ezcerror('Failed to open X11 window',0)
                write (msg,'(a,i2)') 'GXWIN returned error status:',errind
                call ezcerror(msg,0)
                return
             endif

#              Note that iconn32 is equivalenced to (part of) idwind.
	     call gopwk(iwsid(iwinop+1),iconn32,iwstype)
	     iwinop = iwinop + 1  # only increment after success 

#              Save connection id (needed for gdwin call).
             iwscn(iwinop) = idwind
	     call gacwk(iwsid(iwinop))
	     winstate(iwinop) = TRUE
             #deactivate all other windows in the same display
             if (iwinop > 1) then
      	        do i = 1,iwinop-1
		   if (windisp(i) == windisp(iwinop) & winstate(i)) then
	              call ezcwoff(i)
		   endif
		enddo
	     endif

             #initialize color indices
      #Note: For NCAR-GKS prior to release 4.0 it may be necessary to add
      #      coding to cause the following to be executed only once, to avoid
      #      code crashes on opening second window.
             if (winclrmap == "mono") then
                call ezcmono(iwsid(iwinop),iwstype)
             else
                call ezccidx(iwsid(iwinop),iwstype,winclrmap)
             endif
             if (debcolr <> 0) then
               # Check background color settings.
                call gqcr(iwsid(iwinop),0,0,errind,cr,cg,cb)
                write (msg,'(a,3f6.3)') ' bgcolor requested was: ', cr,cg,cb
                call ezcremrk(msg)

                call gqcr(iwsid(iwinop),0,1,errind,cr,cg,cb)
                write (msg,'(a,3f6.3)') ' bgcolor actually used: ', cr,cg,cb
                call ezcremrk(msg)

               # Check foreground color settings.
                call gqcr(iwsid(iwinop),1,0,errind,cr,cg,cb)
                write (msg,'(a,3f6.3)') ' fgcolor requested was: ', cr,cg,cb
                call ezcremrk(msg)

                call gqcr(iwsid(iwinop),1,1,errind,cr,cg,cb)
                write (msg,'(a,3f6.3)') ' fgcolor actually used: ', cr,cg,cb
                call ezcremrk(msg)
             endif
		
	     iezcdev = iezcdev + 1
		
	  else	# found existing window; perform the command "on"
	     if (.not.winstate(idxfd)) then	#window not active
	        call gacwk(iwsidfd)
	        winstate(idxfd) = TRUE
	        iezcdev = iezcdev + 1

	        #deactivate all other windows in the same display
	        if (iwinop > 1) then
		   do i = 1,iwinop
		      if (i <> idxfd) then
		         if (windisp(i) == windisp(idxfd) & winstate(i)) then
		            call ezcwoff(i)
		         endif
		      endif
		   enddo
	        endif
	     else	# window already open
		msg = "Window already open, ignoring command- "\
			//iflag(1:ezcgetcl(iflag))
	   	errsvl = 0
		call ezcerror(msg,errsvl)
	     endif
	  endif

       elseif (iflag == "off") then 	#for "off" command
          if (found) then	#for existing window
	     if (winstate(idxfd)) then	#deactivate the WS
                call ezcwoff(idxfd)
             else	#window already inactive
		msg = "Window is inactive, ignoring command- "\
			//iflag(1:ezcgetcl(iflag))
		errsvl = 0
		call ezcerror(msg,errsvl)
	     endif
	  else	#off command issued to non-existing window
	     msg = "Window does not exist, ignoring command- "\
			//iflag(1:ezcgetcl(iflag)) //" "\
                        //istring(1:ezcgetcl(istring))
	     errsvl = 0
	     call ezcerror(msg,errsvl)
	  endif
	 
       elseif (iflag == "send") then	# for "send" command
          #turn off CGM temporarily
          cgmrvs = FALSE
          if (iezccgma >0) then
  	     call ezccgm("off"," ")
	     cgmrvs = TRUE
	  endif	
	  #turn off PS temporarily
          psrvs = FALSE
          if (iezcpsa >0) then
             call ezcps("off"," ")
             psrvs = TRUE
          endif

	  call ezcshowg

	  #restore the original state of other devices
	  if (cgmrvs) call ezccgm("on"," ")
	  if (psrvs) call ezcps("on"," ")

       elseif (iflag == "close") then 	#for "close" command
          if (found) then
	     if (winstate(idxfd)) then	#deactivate WS first
	        call gdawk(iwsidfd)
	        iezcdev = iezcdev - 1
	     endif
	     call gclwk(iwsidfd)
#             Close window opened by gxwin.
             idwind = iwscn(idxfd)
             call gdwin(idwind, errind)
             if (errind > 0) then
                write (msg,'(a,i3)') 'Trouble closing window.  '//\
                   'GDWIN returned:', errind
	        call ezcerror(msg,errsvl)
             endif

	     #clean the open window table
	     if (idxfd < iwinop) then	#shift table upwards
	        do j = idxfd+1,iwinop
	           winame(j-1) = winame(j)
	           winstate(j-1) = winstate(j)
		   iwsid(j-1) = iwsid(j)
		   iwscn(j-1) = iwscn(j)
		   windisp(j-1) = windisp(j)
                   winncoli(j-1) = winncoli(j)
                   wincola(j-1) = wincola(j)
		enddo
	     endif

	     iwinop = iwinop -1	#decrease the number of open windows
	     if (iwinop == 1) then	# automatically activate the only window
                if (.not.winstate(1)) then
		   call ezcwinr("on",winame(1))
		endif
	     endif

	  elseif (iwinop == 1) then 	# close the only window as default
	     call ezcwinr("close",winame(1))

          elseif (istring == "all") then # close all windows
             do indx = iwinop, 1, -1  #Backward loop avoids recursion problem.
                call ezcwinr("close",winame(indx))
             enddo

	  else	#could not find right window to close
	     msg = "Could not find window to close, ignoring command- "\
			//iflag(1:ezcgetcl(iflag)) //" "\
                        //istring(1:ezcgetcl(istring))
	     errsvl = 0
	     call ezcerror(msg,errsvl)
	  endif

       elseif (iflag == "colormap") then 	#for "colormap" command
          oldcmap = winclrmap
          winclrmap = istring
          if (debcolr <> 0) then
             msg = "Window colormap requested: "\
                     //winclrmap(1:ezcgetcl(winclrmap))
             call ezcremrk(msg)
          endif
	  # open window if non-exist
          if (iwinop == 0) then
             call ezcwinr("on"," ")
          elseif (iwinop == 1) then
             if (oldcmap <> winclrmap) then
                namestore = winame(1)
                call ezcwinr("close",namestore)
                call ezcremrk("Closed the current window to reset the colormap.")
                call ezcwinr("on",namestore)
                if (namestore <> " ") then
                    msg = "New window has same name: "\
                           // namestore(1:ezcgetcl(namestore))
                    call ezcremrk(msg)
                endif
             endif
          else 
             call ezcremrk("Multiple windows are open.")
             call ezcremrk("Please close all windows, then open a new one.")
             call ezcremrk("The new colormap will be assigned to the new window.")
             call ezcremrk("Use 'win list' or 'win slist' to list windows,")
             call ezcremrk(" or 'win close all' to close all windows.")
          endif

       elseif (iflag == "list") then    #for "list" command
          call ezcwinll(namestore)

       elseif (iflag == "slist") then   #for "slist" command
          call ezcwinsl

       else	#unknown command for windows
	  msg = "ezcwin: Unknown command, ignoring- "\
			//iflag(1:ezcgetcl(iflag))
	  errsvl = 0
	  call ezcerror(msg,errsvl)
       endif

        return
        end

        subroutine ezcwinr(iflag,istring)
c recursion intermediary
        character*(*) iflag,istring
        call ezcwin(iflag,istring)
        return
        end

c ---------------------------------
        subroutine ezcwoff (idx)
        integer idx
#
#  Deactivate window whose index is idx.
#
#    Note that there is no error checking here. It is assumed that
#    ezcwoff is only called by ezcwin with a valid index.
#
# Change record:
#  990125  Created by Fred N. Fritsch.
# ---------------------------------------------------------------------
Use(Devices)
      call gdawk(iwsid(idx))
      winstate(idx) = FALSE
      iezcdev = iezcdev - 1
      return
      end
