# EZD window listing routines:
#	ezcwinll = long-list
#       ezcwinsl = short-list

      subroutine ezcwinll(name)
      character*(*) name
#   EZD window long-list routine.
#     Implements command "win list name".
#
#   If name is omitted (blank), print entire contents of windows arrays.
#   For nonblank name, search for that name and print only its info.
#   If no windows are open, report that fact.
#
# Change record:
# (yymmdd means 19yy/mm/dd)
#  980116  Written by Fred N. Fritsch.
#  980612  Corrected format for logical variable winstate. (FNF)
#  980615  Modified to print new iwscn and omit wincola, winncoli. (FNF)
#  980617  Move position of winstate in output. (FNF)
#  980619  Changed iwscn to integer*8. (FNF)
#  991103  Changed iwscn from integer*8 to integer(2). (FNF)
# (yymmdd means 20yy/mm/dd)
#  000219  Changed iwscn from integer(2) to Address, which effectively
#          returned it to the state before the 991103 change. (FNF)
# ---------------------------------------------------------------------
      Use(Devices)

      character*120 msg
      integer ezcgetcl
      external ezcgetcl
      integer indx, idx
      logical found

      if (iwinop > 0) then
         if (name <> " ") then
            found = FALSE
            do idx = 1, iwinop
               if (winame(idx) == name) then
                  found = TRUE
                  indx = idx
                  break
               endif
            enddo

            if (found) then
               write (msg,100) indx, winstate(indx), winame(indx),
                    iwsid(indx), iwscn(indx), windisp(indx)
               call ezcremrk(msg)
            else
               msg = 'Window name not found: ' // name(1:ezcgetcl(name))
               call ezcerror(msg,0)
            endif

         else
            call ezcremrk(' idx st  name              wsid     wsconn (hex)    display')
            call ezcremrk(' -------------------------------------------------------------------')
            do idx = 1, iwinop
               write (msg,100) idx, winstate(idx), winame(idx),
                    iwsid(idx), iwscn(idx), windisp(idx)
               call ezcremrk(msg)
            enddo
 100        format (i4,l3,2x,a,i6,2x,z16,2x,a)
         endif

      else
#        Should never get here (see ezcwin).
         call ezcerror('No windows are open.', 0)
      endif

      return
      end

      subroutine ezcwinsl
#
#   EZD window short-list routine.
#     Implements command "win slist name".
#
#   Print window names and ids only.
#   If no windows are open, report that fact.
#   Argument name, if given on command, is ignored.
#
# Change record:
#  980116  Written by Fred N. Fritsch.
#  980617  Omitted unused argument name.  Modified to print winstate
#          instead of iwsid, so this just gives first 3 columns of long
#          list. (FNF)
# ---------------------------------------------------------------------
      Use(Devices)

      character*120 msg
      integer ezcgetcl
      external ezcgetcl
      integer idx

      if (iwinop > 0) then
         call ezcremrk(' idx st  name')
         call ezcremrk(' -------------------------')
         do idx = 1, iwinop
            write (msg, 200) idx, winstate(idx), winame(idx)
            call ezcremrk(msg)
         enddo
 200     format (i4,l3,2x,a)

      else
#        Should never get here (see ezcwin).
         call ezcerror('No windows are open.', 0)
      endif

      return
      end
