/*  Fortran externals are name_
   C  externals are name
 */
#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>

static Display *display_ptr = NULL;
static char *vendor, *display_name;
static Window Xwindow = 0, top_window_id = 0, root_id;
char *getenv();
int debug = 0;


int
g_(int *x_win, int *y_win, int *width_win, int *height_win, char *name,
   char *icname, int *colmodel, int *stat, Window * wind_id)
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  Routine:  g_
  Author:  Jeff Rowe,
           Lawrence Livermore National Lab,
           Livermore Computing Center,
           Information Management & Graphics Group
           Tel: (510) 422-4042, email: jrowe@llnl.gov
  Modifications:
    Created: Jun 98

  Description: Create an X11 Window. Fortran callable.

  Parameters:
	int *x_win ,*y_win   <  upper left pixel of new window's border.  
	int *width_win, height_win < width and height, in pixels, of new window.
	int *name < pointer to window name string.
	int *icname  < pointer to window icon name string.
	int *colmodel  < Color model to use.   Reserved for future use.
	int *stat   > 0 if no error, 1 if X display not initialized.
	Window *wind_id   > X Window ID. 
	  This can be used in NCAR's open  workstation
          if NCAR should use an existing X window (workstation type 7) :
          xwindow_id = llnlgetxwin();
	  or
	  xwindow_id = g_(.....);
	  sprintf(&wid, "%d", xwindow_id);
   	  gopen_ws(X_WIN, &wid, 7);

	int g_   > same as stat.
  Procedure: Setup for XCreateWindow. 

=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
{
    static XWMHints xwmh = {
        (InputHint | StateHint),        /* flags                       */
        True,                   /* input                        */
        NormalState,            /* initial_state                */
        0,                      /* icon pixmap                  */
        0,                      /* icon window                  */
        0, 0,                   /* icon location                */
        0,                      /* icon mask                    */
        0                       /* Window group                 */
    };
    static XClassHint xch = {
        "xgks",                 /* resource name                */
        "Xgks"                  /* class name                   */
    };
    static XSizeHints xsh = {   /* Size hints for window manager */
        (PMinSize),
        0, 0,                   /* obsolete ????                */
        600,                    /* obsolete ????                */
        400,                    /* obsolete ????                */
        256, 256,               /* minimum usefull win dim      */
        0, 0,                   /* max dim (not used)           */
        0, 0,                   /* not used                     */
        {0, 0},                 /* not used                     */
        {0, 0},                 /* not used                     */
        0,
        0,                      /* dimensions of window         */
        0
    };
    XSetWindowAttributes xswa;  /* Set Window Attribute struct  */
    XTextProperty window_name, icon_name;
    unsigned long bw = 0;       /* Border width                 */
    XEvent event;               /* Event received               */
    XSizeHints *xshptr = &xsh;



    static int xw_size, yw_size, xw_mm, yw_mm, w_depth;
    static unsigned long display, parent_id, width, height, border_width,
        border_height, border_id, background_id;
    static long x_coord, y_coord;
    static int no_planes, release_no, no_cells, no_screens;

#define MONOCHROME (no_cells <= 2)      /* monochrome screen */
    static unsigned long gc_mask;


/* these are the screen variables */
    /* figure out the display string */
    Screen *screen_id;
    static Visual *my_visual;
    static Colormap my_colormap;
    XColor colval[300];
    XColor oldcolval[300];      /* used to store away old color values */
    int screen_num, default_depth;
    Visual *default_visual;
    XVisualInfo visual_info;

    *stat = 0;
    *wind_id = 0;

    if (debug)
        printf
            ("g_: xw %d yw %d xs %d ys %d name %s ic %s stat %d winid %d\n",
             *x_win, *y_win, *width_win, *height_win, name, icname, *stat);

    {
        display_name = (char *) NULL;   /* default causes XOpen Display to use the DISPLAY env var */
        if (display_name == NULL)
            display_name = getenv("DISPLAY");

        /* open up the display */
        if (NULL == (display_ptr = XOpenDisplay(display_name))) {
            if (display_name == NULL)
                printf
                    ("Error: Couldn't open display.\n g_ at IP: %s \n  A LLNL private NCAR routine. Contact J.Rowe, LC/IMG, x 2-4042\n",
                     XDisplayName(display_name));

            *stat = 1;
            return 1;
        }
    }

    /* get info on the display and screen */

    screen_id = XDefaultScreenOfDisplay(display_ptr);
    no_screens = XScreenCount(display_ptr);
    root_id = XRootWindowOfScreen(screen_id);
    no_planes = XPlanesOfScreen(screen_id);
    xw_size = XWidthOfScreen(screen_id);
    yw_size = XHeightOfScreen(screen_id);
    xw_mm = XWidthMMOfScreen(screen_id);
    yw_mm = XHeightMMOfScreen(screen_id);
    release_no = XVendorRelease(display_ptr);
    vendor = XServerVendor(display_ptr);
    no_cells = XCellsOfScreen(screen_id);
    /*
     * Ensure that the window's colormap field points to the default
     * colormap,  so that the window manager knows the correct 
     * colormap to use for the window.  
     */
    xswa.bit_gravity = CenterGravity;
    xswa.backing_store = WhenMapped;
    xswa.background_pixel =
        BlackPixel(display_ptr, DefaultScreen(display_ptr));
    xswa.border_pixel =
        BlackPixel(display_ptr, DefaultScreen(display_ptr));

    xsh.x = *x_win;
    xsh.y = *y_win;
    xsh.width = *width_win;
    xsh.height = *height_win;

    /* 
     * Create the Window with the information in the XSizeHints, the
     * border width, and the border & background pixels.
     */
    Xwindow =
        XCreateWindow(display_ptr, root_id, xsh.x, xsh.y, xsh.width,
                      xsh.height, bw, CopyFromParent, InputOutput,
                      CopyFromParent,
                      (CWBitGravity | CWBackingStore | CWBackPixel |
                       CWBorderPixel), &xswa);

    /*
     * Set the standard properties for the window managers. 
     */
    window_name.encoding = XA_STRING;
    window_name.format = 8;
    window_name.value = (unsigned char *) name;
    window_name.nitems = strlen((char *) window_name.value);
    icon_name.encoding = XA_STRING;
    icon_name.format = 8;
    icon_name.value = (unsigned char *) icname;
    icon_name.nitems = strlen((char *) icon_name.value);

    XSetWMProperties(display_ptr, Xwindow, &window_name, &icon_name, NULL,
                     0, xshptr, &xwmh, &xch);

    /* 
     * Select notification of Expose event that is generated when
     * the window is first mapped (becomes visible) to the screen.
     */
    XSelectInput(display_ptr, Xwindow, ExposureMask);

    /*
     * Map the window to make it visible.
     */
    XMapWindow(display_ptr, Xwindow);

    /*
     *    get expose event as window becomes visible. we can't
     *      draw until after this 
     */
    while (1) {
        /* get next event       */
        XNextEvent(display_ptr, &event);

        /* 
         * find the last expose event on the event queue.
         */
        if (event.type == Expose && event.xexpose.count == 0) {

            /*
             * Remove any other pending Expose events from 
             * the queue to avoid multiple repaints. 
             */
            /*SUPPRESS570 */
            while (XCheckTypedEvent(display_ptr, Expose, &event)) ;

            break;
        }
    }


    /*
     * Select input for "pause"
     */
    XSelectInput(display_ptr, Xwindow,
                 (ExposureMask | ButtonPressMask | KeyPressMask |
                  ButtonReleaseMask | StructureNotifyMask));

    *stat = 0;
    *wind_id = Xwindow;
    return 0;
}

/*------------------------------------------------------------------------*/
/* Change to return window and display so mouse locator input can be implemented
   J. Rowe,  LLNL,Livermore Computing, 510.422.4042, jrowe*llnl.gov 11 Dec 97  
 */

Window llnlgetxwin()
{
    if (debug)
        printf("llnlgetwin: int: %d , str: %s \n", Xwindow, &Xwindow);
    return (Window) Xwindow;
}

Display *llnlgetxdpy()
{
    if (debug)
        printf("llnlgetxdpy: int: %d , str: %s \n", display_ptr,
               XDisplayName(display_name));
    return (Display *) display_ptr;
}

/*  of change for LLNL mouse locator input */
/*------------------------------------------------------------------------*/
