/* This is a template for gdwind.c.
Copy Gdwind.t to gdwind.c, then replace pattern Gdwind.
 Fortran externals are name, name_, or NAME (platform dependent);
 C  externals are name
 Same as gdwindc.c. Cp Gdwind.t to gdwindc.c, replace Gdwind.
 */
#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>

int
    Gdwind(Window * wind_id, int *stat)
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  Routine:  Gdwind
  Author:  Jeff Rowe,
           Lawrence Livermore National Lab,
           Livermore Computing Center,
           Information Management & Graphics Group
           Tel: (510) 422-4042, email: jrowe@llnl.gov
  Modifications:
    Created: Jun 98

  Description: Destroy an X11 Window. Fortran callable.

  Parameters:
	Window *wind_id   > X Window ID. 
	int *stat   > same as Gdwind.
	Gdwind   > 0 if no error, 1 if X display not initialized.
			    2 if window not initiailized.
			    3 if can't destroy window.

  Procedure: Setup for XDetroyWindow. 

=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
{
    Display *llnlgetwdpy();
    char *llnlgetdpyn();
    Display *display_ptr = NULL;
    Window llnlgetxwin();
    Window Xwindow = 0;
    char *display_name = NULL;
    int debug = 0;

    *stat = 0;

    Xwindow = *wind_id;

    if (debug)
        printf("Gdwind: stat %d winid %d\n", *stat, *wind_id);

    display_name = llnlgetdpyn(Xwindow);        /* default causes XOpen Display to use the DISPLAY env var */

    if (display_name == NULL) {
        printf
            ("Error: Couldn't open display.\n Gdwind at IP: %s \n  A LLNL private NCAR routine. Contact J.Rowe, LC/IMG, x 2-4042\n",
             XDisplayName(display_name));

        *stat = 1;
        return 1;
    }

    display_ptr = llnlgetwdpy(Xwindow);
    if (display_ptr == NULL) {
        printf
            ("Error: Couldn't open display.\n Gdwind at IP: %s display_ptr %x \n  A LLNL private NCAR routine. Contact J.Rowe, LC/IMG, x 2-4042\n",
             XDisplayName(display_name), display_ptr);

        *stat = 1;
        return 1;
    }

    if (wind_id == NULL) {
        *stat = 2;
        return 2;
    }

    if (debug)
        printf("Gdwind: XDestroy Window: display_ptr %d winid %d\n",
               display_ptr, *wind_id);
    XDestroyWindow(display_ptr, Xwindow);
#if 0
    if (!XDestroyWindow(display_ptr, Xwindow)) {
        *stat = 3;
        /*return 3; */
    }
#endif
/* Wait until all events have been received and processed by X Server. Discards
all events on input queue.
*/
    XSync(display_ptr, True);
    *stat = 0;
    return 0;
}
