/*This is a template for gidisp.c.
Copy Gidisp.t to gidisp.c, then replace pattern Gidisp.
Fortran externals are name, name_, or NAME (platform dependent);
    C  externals are name.

 */
#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>

static Display *display_ptr = NULL;
static char *vendor, *display_name;
static Window Xwindow = 0, top_window_id = 0, root_id;
char *getenv();


int
Gidisp(char *disp_name, int *xw_size, int *yw_size, int *xmm_size,
       int *ymm_size, int *no_planes, int *stat)
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  Routine:  Gidisp
  Author:  Jeff Rowe,
           Lawrence Livermore National Lab,
           Livermore Computing Center,
           Information Management & Graphics Group
           Tel: (510) 422-4042, email: jrowe@llnl.gov
  Modifications:
    Created: Jun 98

  Description: Inquiry X11 display sizes. Fortran callable.

  Parameters:
  char *disp_name < If NULL, uses environment DISPLAY. 
			       Otherwise string of form "workstation_IP_address:0"

	int *xw_size, *yw_size  >  width and height of screen in pixels..  
	int *xmm_size, *ymm_size  >  width and height of screen in millimeters..  
	int *no_planes > number of planes (colors)
	int *stat   > 0 if no error, 1 if X display not initialized.
	int Gidisp   > same as stat.
  Procedure: Setup for XCreateWindow. 

=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
{
    XSetWindowAttributes xswa;  /* Set Window Attribute struct  */
    XTextProperty window_name, icon_name;
    unsigned long bw = 0;       /* Border width                 */
    XEvent event;               /* Event received               */

    static unsigned long display, parent_id, width, height, border_width,
        border_height, border_id, background_id;
    static long x_coord, y_coord;
    static int release_no, no_cells, no_screens;
    int debug = 0;


/* these are the screen variables */
    /* figure out the display string */
    Screen *screen_id;
    int screen_num, default_depth;

    *stat = 0;


    {
        display_name = disp_name;       /* default causes XOpen Display to use the DISPLAY env var */

        if (display_name == NULL) {
            /* default causes XOpen Display to use the DISPLAY env var */
            display_name = getenv("DISPLAY");
        }
        if (display_name == NULL) {
            printf
                ("Gidisp error: DISPLAY not set: %s \n  A LLNL private NCAR routine. Contact J.Rowe, LC/IMG, x 2-4042\n",
                 XDisplayName(display_name));

            *stat = 1;
            return 1;
        }
        /* open up the display */
        if (NULL == (display_ptr = XOpenDisplay(display_name))) {
            printf
                ("Gidisp error: Couldn't open display at IP: %s \n  A LLNL private NCAR routine. Contact J.Rowe, LC/IMG, x 2-4042\n",
                 XDisplayName(display_name));
            *stat = 1;
            return 1;
        }
    }

    /* get info on the display and screen */

    screen_id = XDefaultScreenOfDisplay(display_ptr);
    no_screens = XScreenCount(display_ptr);
    root_id = XRootWindowOfScreen(screen_id);
    *no_planes = XPlanesOfScreen(screen_id);
    *xw_size = XWidthOfScreen(screen_id);
    *yw_size = XHeightOfScreen(screen_id);
    *xmm_size = XWidthMMOfScreen(screen_id);
    *ymm_size = XHeightMMOfScreen(screen_id);
    release_no = XVendorRelease(display_ptr);
    vendor = XServerVendor(display_ptr);
    no_cells = XCellsOfScreen(screen_id);

    XCloseDisplay(display_ptr);

    if (debug)
        printf("Gidisp: disp_name: %s xw %d yw %d xs %d  w_color %d\n",
               display_name, *xw_size, *yw_size, no_screens, *no_planes);
    *stat = 0;
    return 0;
}
