/* This is a template for grqlcf.c.
   Copy Grqlcf.t to grqlcf.c, then replace pattern Grqlcf.

   Fortran externals are name, name_, or NAME (platform dependent);
   C  externals are name.
 */

/* Mangle names of FORTRAN routines if necessary */
#if defined(CRAY2) || defined(XMP) || defined(YMP) || defined(C90) || defined(J90)
#   define gqwks_ GQWKS
#   define gqwkc_ GQWKC
#elif ! defined(SUN4) && ! defined(SOL) && ! defined(SGI) && ! defined(CS2) && ! defined(AXP) && ! defined(LINUX) && ! defined(IRIX64)
#   define gqwks_ gqwks
#   define gqwkc_ gqwkc
#endif

#include <stdio.h>

int
Grqlcf(int *wsid, int *lcdnr, int *stat, int *tnr, float *px, float *py)

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  Routine:  grqlcf
  Author:  Jeff Rowe,
           Lawrence Livermore National Lab,
           Livermore Computing Center,
           Information Management & Graphics Group
           Tel: (510) 422-4042, email: jrowe@llnl.gov
  Modifications:
    Created: Dec 97

  Description: GKS Request locator. C implementation of Fortran binding. 
	       Calls Fortran GKS routines.

  Parameters:
	int *wsid  < workstation identifier
	int *lcdnr < locator device number (ignored)
	int *stat < pointer to status
	int *tnr  < pointer to normalization transformation number (ignored)
	float *px < pointer to locator x position
	float *py < pointer to locator y position
	int *stat > 0 for none, 1 for ok, or Grqlcf.
	float *px > NDC (0-1.) x position
	float *py > NDC (0-1.) y position
	Grqlcf   > 0 if no error, 1 if X display not initialized.
			    2 if window not initialized/user destroyed.
			    3 if wsid not active.
			    4 if wsid wrong type (must be 7 or 8 for NCAR).
			    5 if window expose event.  Redraw and or recall.
			    7 if window doesn't match wsid.
  Procedure: call request_locator. normalize results.

=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
{
#define debug 0
    int xloc, yloc, width, height, locrtn;      /* locator output */
    int state = -1;
    int err_ind = 0;
    int conid = 0, wtype = 0;


/* Get locator input  */

    xloc = yloc = 0;
    width = 0;
    height = 0;
    *stat = 0;


/*  See if workstation of correct type (for NCAR this is 7 or 8) */

    gqwkc_(wsid, &err_ind, &conid, &wtype);

    if ((wtype == 7) || (wtype == 8)) {


/* See if workstation in correct state: open, active */

/* Call Fortan version */
        gqwks_(wsid, &err_ind, &state);

        if (state == 1) {

            locrtn = request_locator(wsid, &xloc, &yloc, &width, &height);

            if (locrtn == 0) {
                if (height <= 0)
                    height = 1;
                *px = (float) xloc / (float) width;
                *py = (float) 1.0 - ((float) yloc / (float) height);
                *stat = 1;
            }
            if (debug) {
                printf
                    ("Grqlcf: request_locator wsid: returned xloc %d, yloc %d width %d height %d error: %d \n",
                     wsid, xloc, yloc, width, height, locrtn);
            }
            *stat = locrtn;
            return locrtn;
        } else {
            printf
                ("Error: Workstation not active or wrong state.\n Grqlcf: gqwks: wsid %d err_ind %d state %d\n  A LLNL private NCAR routine. Contact J.Rowe, LC/IMG, x 2-4042\n",
                 wsid, err_ind, state);
            *stat = 3;
            locrtn = 3;
            return 3;
        }
    } else {
        printf
            ("Error: Workstation  wrong type.\n Grqlcf: gqwkc: wsid %d wtype %d, err_ind %d state %d\n  A LLNL private NCAR routine. Contact J.Rowe, LC/IMG, x 2-4042\n",
             *wsid, wtype, err_ind, state);
        *stat = 4;
        locrtn = 4;
        return 4;
    }
}
