      SUBROUTINE GXWIN(IWKID, IDPNAM, LOCX, LOCY, IXSIZE, IYSIZE, NAME,
     x                 ICNAME, ICOLMOD, ISTAT, IDWIND) 
C     (Assumes Fortran default typing will work OK here.)

C*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
C  Routine:  GXWIN
C  Author:  Jeff Rowe,
C           Lawrence Livermore National Lab,
C           Livermore Computing Center,
C           Information Management & Graphics Group
C           Tel: (925) 422-4042, email: jrowe@llnl.gov
C  Modifications:
C    Created: Jun 98 
C    Modified:
C    3 Mar 99: Added DPNAME.
C   16 Mar 99: Changed DPNAME to IDPNAM (implicit integer).
C    3 Nov 99: Corrected usage instructions + minor cosmetics. (FNF)
C
C  Description: Create an X11 Window.
C
C  Parameters:
C       IWKID>  workstation id.
C       IDPNAM < DISPLAY name (valid workstation name or IP address
C                If null, uses DISPLAY environment variable.
C	LOCX, LOCY   <  upper left pixel of new window's border.  
C	IXSIZE, IYSIZE < width and height, in pixels, of new window.
C	NAME < pointer to window name string.
C	ICNAME  < pointer to window icon name string.
C	ICOLMOD  < Color model to use: 
C        =0 for Shared X Color Model 
C         If the X Workstation is directed to use the Shared X Color Model,
C         then that X window will use the default color table shared by all
C         applications. If you request more colors, via calls to GSCR, than there
C         are available to the given window, then a color matching algorithm is
C         employed. The idea of the algorithm is that the color in the current
C         color table that is closest to the requested color will be selected.
C         Closest is defined in terms of the normal distance metric on the RGB
C         cube. If the closest color is farther away than the percentage error
C         allowed times the length of the diagonal of the RGB color cube, then
C         error number -220 is issued together with a warning. The closest color
C         is still assigned for the requested color. The Shared X Color Model is
C         the default for workstations of type 7. 
C        =1 for Private X Color Model 
C         Sometimes you might need to guarantee that 256 distinct colors are 
C         available to a given X window. If an X Workstation is directed to use
C         the Private X Color Model, then that X Workstation will install its
C         own X Color table for the X window. This means that X window will have
C         a different Color table than all the other windows on the screen.
C         Therefore, you usually have to have the mouse pointer in the window for
C         the correct color table to be installed. One disadvantage of this Model
C         is that there is usually a color flashing effect on the screen, since a
C         different color table will be installed for all the other windows on the
C         screen. 
C        =2 for Mixed X Color Model 
C         The Mixed X Color Model attempts to take the best of both of the
C         previous models. It starts out behaving like the Shared X Color Model,
C         in that it uses the default color table for the screen. It differs in
C         that, once it can't allocate any more colors from the default color
C         table, it allocates its own private color table and starts using it.
C         This way, color flashing is only present if it absolutely needs to be
C         so that the X Workstation can display the correct color. 
C         The Mixed X Color Model is the default for Workstations of type 8. 
C
C	ISTAT   > 0 if no error, 1 if X display not initialized.
C	IDWIND   > X Window ID. This can be used in NCAR's open  workstation
C          if NCAR should use an existing X window (workstation type 7).
C          Note usage from following example:
C
C         Leave space for large window id's of up to 64bits,
C         assuming Fortran INTEGER's are at least 32 bits.
C         INTEGER IDWIND(2)
C         CALL GXWIN(0,0,800,800,"MY WINDOW NAME", "MYICON",
C        x 0,ISTAT,IDWIND(1) )
C   
C         IF (ISTAT .GT. 0) THEN 
C            WRITE (*, 1003) ISTAT
C    1003 FORMAT('My program cant create X window', 'ISTAT' i4)
C            CALL EXIT
C         END IF
C      
C         LUNIT = IDWIND(1)
C         May need to change 1 to 2 on SGI IRIX64 systems.
C         CALL GOPWK (IWKID, LUNIT, 7)
C
C
C  Procedure: Call C version gxwind_. 
C
C=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
C This calls gxwind_  ...

      CALL GXWIND(IWKID, IDPNAM, LOCX, LOCY, IXSIZE, IYSIZE, NAME,
     x            ICNAME, ICOLMOD, ISTAT, IDWIND) 

C Make sure color model is same as that of default NCAR
C Namely, Mixed color model. See GKS Escape Functions for X Output
C The default for wktype 7 is shared color model. This could gen
C -220 errors from gscr if requested color can't be found or dithered.
C To apply to next workstation created, use -1 for wkid

ifelse(SYSTEM,SUN4, [
# Following doesn't work for NCAR4.0, only NCAR4.0.1
],[
       IF(ICOLMOD .EQ. 0)CALL NGSETI ('SC', -1)
       IF(ICOLMOD .EQ. 1)CALL NGSETI ('PC', -1)
       IF(ICOLMOD .EQ. 2)CALL NGSETI ('MC', -1)
])
       RETURN
       END
