/* This is template for gxwind.c. cp Gxwind.c to gxwind.c, then
    replace pattern Gxwind.
    
    Fortran externals are name, name_, or NAME (platform dependent);
   C  externals are name

   2 Mar 99: Added dpy_ptr to sWinDat so different DISPLAY's env var's can
   be attached to windows.
   28 Dec 98
   Fixed request_locator.c to process only window specific events. Also
   disables manual window exit or close. 
   14 Dec 98
   changed backing store from whenmapped, to always.
   16 Dec 98
   Added window delete protcol- so if user closes or exits a window manually, the
   app doesn't exit, but waits to process the client message event. See request_
   locator.c. 

 */
#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>

static Display *display_ptr = NULL;
static char *vendor;
static char *display_name = NULL;
static int num_wsid = 0;

#define MaxW 12
static Window Xwindow;
static Window root_id;
static struct sWinDat {
    Window WindowID;
    Window top_window_id;
    Window root_id;
    Display *dpy_ptr;
    char *dpy_name;
    int WSID;
};
static struct sWinDat WinDat[MaxW];
char *getenv();
int debug = 0;
char *strcpy();


int
Gxwind(int *wsid, char *disp_name, int *x_win, int *y_win, int *width_win,
       int *height_win, char *name, char *icname, int *colmodel, int *stat,
       Window * wind_id)
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  Routine:  Gxwind
  Author:  Jeff Rowe,
           Lawrence Livermore National Lab,
           Livermore Computing Center,
           Information Management & Graphics Group
           Tel: (510) 422-4042, email: jrowe@llnl.gov
  Modifications:
    Created: Jun 98

  Description: Create an X11 Window. Fortran callable.

  Parameters:
 	int *wsid  < workstation identifier
	char *disp_name < If NULL, uses environment DISPLAY. 
			     Otherwise string of form "workstation_IP_address:0"
	int *x_win ,*y_win   <  upper left pixel of new window's border.  
	int *width_win, height_win < width and height, in pixels, of new window.
	int *name < pointer to window name string.
	int *icname  < pointer to window icon name string.
	int *colmodel  < Color model to use.   Reserved for future use.
	int *stat   > 0 if no error, 1 if X display not initialized.
	Window *wind_id   > X Window ID. 
	This can be used in NCAR's open  workstation
          if NCAR should use an existing X window (workstation type 7) :
          xwindow_id = llnlgetxwin(wsid);
	  or
	  xwindow_id = Gxwind(.....);
	  sprintf(&wid, "%d", xwindow_id);
   	  gopen_ws(X_WIN, &wid, 7);

	int Gxwind   > same as stat.
  Procedure: Setup for XCreateWindow. 

=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
{
    static XWMHints xwmh = {
        (InputHint | StateHint),        /* flags                       */
        True,                   /* input                        */
        NormalState,            /* initial_state                */
        0,                      /* icon pixmap                  */
        0,                      /* icon window                  */
        0, 0,                   /* icon location                */
        0,                      /* icon mask                    */
        0                       /* Window group                 */
    };
    static XClassHint xch = {
        "xgks",                 /* resource name                */
        "Xgks"                  /* class name                   */
    };
    static XSizeHints xsh = {   /* Size hints for window manager */
        (PMinSize),
        0, 0,                   /* obsolete ????                */
        600,                    /* obsolete ????                */
        400,                    /* obsolete ????                */
        256, 256,               /* minimum usefull win dim      */
        0, 0,                   /* max dim (not used)           */
        0, 0,                   /* not used                     */
        {0, 0},                 /* not used                     */
        {0, 0},                 /* not used                     */
        0,
        0,                      /* dimensions of window         */
        0
    };
    XSetWindowAttributes xswa;  /* Set Window Attribute struct  */
    XTextProperty window_name, icon_name;
    unsigned long bw = 0;       /* Border width                 */
    XEvent event;               /* Event received               */
    XSizeHints *xshptr = &xsh;
    int myXerr_handler();
    Atom wm_delete_proto;

    static int xw_size, yw_size, xw_mm, yw_mm, w_depth, i;
    static unsigned long display, parent_id, width, height, border_width,
        border_height, border_id, background_id;
    static long x_coord, y_coord;
    static int no_planes, release_no, no_cells, no_screens;

#define MONOCHROME (no_cells <= 2)      /* monochrome screen */
    static unsigned long gc_mask;


/* these are the screen variables */
    /* figure out the display string */
    Screen *screen_id;
    static Visual *my_visual;
    static Colormap my_colormap;
    XColor colval[300];
    XColor oldcolval[300];      /* used to store away old color values */
    int screen_num, default_depth;
    Visual *default_visual;
    XVisualInfo visual_info;

    *stat = 0;
    *wind_id = 0;

    if (debug)
        printf
            ("Gxwind:disp_name %s xw %d yw %d xs %d ys %d name %s ic %s col %d stat %d ",
             disp_name, *x_win, *y_win, *width_win, *height_win, name,
             icname, *colmodel, *stat);
    /* Make a copy of user's data  */
    display_name = (char *) calloc(64, sizeof(char));

    if (disp_name != NULL)
        strcpy(display_name, disp_name);

    if (display_name == NULL) {
        /* default causes XOpen Display to use the DISPLAY env var */
        display_name = getenv("DISPLAY");
    }
    if (display_name == NULL) {
        printf
            ("Gxwind error: DISPLAY not set: %s \n  A LLNL private NCAR routine. Contact J.Rowe, LC/IMG, x 2-4042\n",
             XDisplayName(display_name));

        *stat = 1;
        return 1;
    }
    /* open up the display */
    if (NULL == (display_ptr = XOpenDisplay(display_name))) {
        printf
            ("Gxwind error: Couldn't open display at IP: %s \n  A LLNL private NCAR routine. Contact J.Rowe, LC/IMG, x 2-4042\n",
             XDisplayName(display_name));
        *stat = 1;
        return 1;
    }


    /* get info on the display and screen */

    screen_id = XDefaultScreenOfDisplay(display_ptr);
    no_screens = XScreenCount(display_ptr);
    root_id = XRootWindowOfScreen(screen_id);
    no_planes = XPlanesOfScreen(screen_id);
    xw_size = XWidthOfScreen(screen_id);
    yw_size = XHeightOfScreen(screen_id);
    xw_mm = XWidthMMOfScreen(screen_id);
    yw_mm = XHeightMMOfScreen(screen_id);
    release_no = XVendorRelease(display_ptr);
    vendor = XServerVendor(display_ptr);
    no_cells = XCellsOfScreen(screen_id);
    /*
     * Ensure that the window's colormap field points to the default
     * colormap,  so that the window manager knows the correct 
     * colormap to use for the window.  
     */
    xswa.bit_gravity = CenterGravity;
    xswa.backing_store = WhenMapped;
    xswa.background_pixel =
        BlackPixel(display_ptr, DefaultScreen(display_ptr));
    xswa.border_pixel =
        BlackPixel(display_ptr, DefaultScreen(display_ptr));
    xswa.backing_pixel =
        BlackPixel(display_ptr, DefaultScreen(display_ptr));
    xswa.backing_planes = no_planes;

    xsh.x = *x_win;
    xsh.y = *y_win;
    xsh.width = *width_win;
    xsh.height = *height_win;

    /* 
     * Create the Window with the information in the XSizeHints, the
     * border width, and the border & background pixels.
     */
    Xwindow =
        XCreateWindow(display_ptr, root_id, xsh.x, xsh.y, xsh.width,
                      xsh.height, bw, CopyFromParent, InputOutput,
                      CopyFromParent,
                      (CWBitGravity | CWBackingStore | CWBackPixel |
                       CWBackingPlanes | CWBorderPixel), &xswa);

    /*
     * Set the standard properties for the window managers. 
     */
    window_name.encoding = XA_STRING;
    window_name.format = 8;
    window_name.value = (unsigned char *) name;
    window_name.nitems = strlen((char *) window_name.value);
    icon_name.encoding = XA_STRING;
    icon_name.format = 8;
    icon_name.value = (unsigned char *) icname;
    icon_name.nitems = strlen((char *) icon_name.value);

    XSetWMProperties(display_ptr, Xwindow, &window_name, &icon_name, NULL,
                     0, xshptr, &xwmh, &xch);
/* Set WM Protocols- in particular the WM_DELETE_Window  so that client
   is not disconnected from the server if the user asks for or deletes a 
   window. 22 DEc 98.
 */
    wm_delete_proto = XInternAtom(display_ptr, "WM_DELETE_WINDOW", False);

    if (!XSetWMProtocols(display_ptr, Xwindow, &wm_delete_proto, 1)) {
        printf
            ("Gxwind error: Couldn't set delete protocol for window %x .\n",
             Xwindow);

    }
    /* 
     * Select notification of Expose event that is generated when
     * the window is first mapped (becomes visible) to the screen.
     */
    XSelectInput(display_ptr, Xwindow, ExposureMask);

    /*
     * Map the window to make it visible.
     */
    XMapWindow(display_ptr, Xwindow);

    /*
     *    get expose event as window becomes visible. we can't
     *      draw until after this 
     */
    while (1) {
        /* get next event       */
        XNextEvent(display_ptr, &event);

        /* 
         * find the last expose event on the event queue.
         */
        if (event.type == Expose && event.xexpose.count == 0) {

            /*
             * Remove any other pending Expose events from 
             * the queue to avoid multiple repaints. 
             */
            /*SUPPRESS570 */
            while (XCheckTypedEvent(display_ptr, Expose, &event)) ;

            break;
        }
    }


    /*
       Select input for "pause"
       If this is done here, then this window responds to mouse- even
       though another window may be thinking it should ...??
       Hence, I put the XSelectInput in request_locator.c
     */

    XSelectInput(display_ptr, Xwindow,
                 (ExposureMask | ButtonPressMask | KeyPressMask |
                  ButtonReleaseMask | StructureNotifyMask));

/* Notify NCAR with display_name via llnl_setdpyname in xcontrol.c  */

    llnl_setdpyname(display_name);


    *stat = 0;
    *wind_id = Xwindow;
    for (i = 0; i < MaxW; i++) {
        if ((WinDat[i].WSID == 0) || (WinDat[i].WSID == *wsid)) {
            WinDat[i].WSID = *wsid;
            WinDat[i].WindowID = Xwindow;
            WinDat[i].root_id = 0;
            WinDat[i].dpy_ptr = display_ptr;
            WinDat[i].dpy_name = display_name;
            break;
        }
    }

/* Set my error handler so XGetWindowAttributes in llnlgetxwin returns an error
   if window destroyed by user.
 */
    XSetErrorHandler(myXerr_handler);

    if (debug)
        printf(" Gxwind: windid %x display_ptr %x\n", Xwindow,
               display_ptr);

    return 0;
}

/*------------------------------------------------------------------------*/
/* Change to return window and display so mouse locator input can be implemented
   J. Rowe,  LLNL,Livermore Computing, 510.422.4042, jrowe*llnl.gov 11 Dec 97  
 */

Window llnlgetxwin(int wsid)
/*Given workstaton id wsid, return Xwindow winid  or null.
   or BadWindow  (e.g window closed).
   Also, note that WM_DELETE_WINDOW protocol is being used to get client_messages
   and prevent the X server from causing the application to exit if user
   closes/exits a window. See gxwind above.
   This could pick up exited window in the request_locator event loop, but an
   explicit call to
   llnlgetxwin can also determine if window exitied.
 */
{
    int i, xgetwa;
    XWindowAttributes xwa;      /* Get window attributes        */


    for (i = 0; i < MaxW; i++) {
        if (debug)
            printf("  llnlgetwin: wsid: %x window:  %x  \n",
                   WinDat[i].WSID, WinDat[i].WindowID);
        if (WinDat[i].WSID == wsid) {
            Xwindow = WinDat[i].WindowID;
            display_ptr = WinDat[i].dpy_ptr;
            display_name = WinDat[i].dpy_name;
            /* See if window still around... */
            xgetwa = XGetWindowAttributes(display_ptr, Xwindow, &xwa);
            if (debug)
                printf
                    ("llnlgetwin: wsid: %d Xwindow: %x display %xxgetws %d \n",
                     wsid, Xwindow, display_ptr, xgetwa);
            if (xgetwa == 0) {  /* Window probably destroyed by user  */
                WinDat[i].WindowID = 0;
                WinDat[i].WSID = 0;
                WinDat[i].dpy_ptr = 0;
/*        Destroy now done in request_locator upon
   receipt of client message via llnldelxwin below.
   if (debug)
   printf ("llnlgetwin: XDestroyWindow  wsid: %x Xwindow: %x display: %x \n", wsid,  Xwindow, display_ptr);

   XDestroyWindow(display_ptr,Xwindow);
 */
                return (Window) NULL;
            } else
                return (Window) Xwindow;
        }                       /* of if (WinDat[i].WSID == wsid) */
    }

    if (debug)
        printf("llnlgetwin: cant find wsid: %x \n", wsid);
    return (Window) NULL;
}

/*------------------------------------------------------------------------*/
int llnldelxwin(Window xwin)
/* Remove  window  xwin from session. */
{
    int i;

    for (i = 0; i < MaxW; i++) {
        /* Window probably destroyed by user  */
        if (WinDat[i].WindowID == xwin) {
            display_ptr = WinDat[i].dpy_ptr;
            display_name = WinDat[i].dpy_name;
            XDestroyWindow(display_ptr, xwin);
            WinDat[i].WindowID = 0;
            WinDat[i].dpy_ptr = 0;
            /* Leave WSID on in case llnlgetxwin called by app 
               WinDat[i].WSID = 0;
             */
            return 0;
        }
    }
    return 1;
}

Display *llnlgetxdpy(int wsid)
/* Only valid after llnlgetxwin called. */
{
    if (debug)
        printf("llnlgetxdpy: %x , str: %s \n", display_ptr,
               XDisplayName(display_name));
    llnlgetxwin(wsid);          /* Get display_ptr 3mar99 */

    return (Display *) display_ptr;
}

/*  of change for LLNL mouse locator input */
/*------------------------------------------------------------------------*/

Display *llnlgetwdpy(Window xwin)
/* return  display.ptr for Window  xwin from session. */
{
    int i;

    for (i = 0; i < MaxW; i++) {
        if (WinDat[i].WindowID == xwin) {
            return WinDat[i].dpy_ptr;
        }
    }
    return NULL;
}

/*------------------------------------------------------------------------*/
char *llnlgetdpyn(Window xwin)
/* return  DISPLAY for Window  xwin from session. */
{
    int i;

    for (i = 0; i < MaxW; i++) {
        if (WinDat[i].WindowID == xwin) {
            return WinDat[i].dpy_name;
        }
    }
    return NULL;
}


/*------------------------------------------------------------------------*/

char *llnlgetxdpyname(int wsid)
/* Only valid after llnlgetxwin called. */
{
    if (debug)
        printf("llnlgetxdpyname: %x , str: %s \n", display_ptr,
               XDisplayName(display_name));
    llnlgetxwin(wsid);          /* Get display_name 3mar99 */

    return (char *) display_name;
}

/*  of change for LLNL mouse locator input */
/*------------------------------------------------------------------------*/

int myXerr_handler(display, myerr)
Display *display;
XErrorEvent *myerr;
{
    char msg[80];
    int length = 0;
    char *pmsg = msg;

    if (debug)
        fprintf(stderr, "-->>myXerr_handler called: \n");
#if 0
    XGetErrorText(display, myerr->error_code, pmsg, length);
    fprintf(stderr, "-->myXerr_handler error code: %s\n", pmsg);
    fprintf(stderr, "-->myXerr_handler error code: %s by %s\n",
            myerr->error_code, myerr->request_code);
#endif

    return 0;
}
