      subroutine ebilin(r,z,rp,zp,c)

c     originally written by D. Kershaw
c     reduced to one zone and added to lip by J. Harte
c     
c     Given point (rp,zp) in the zone, jz, bounded by the 4 (r,z)
c     coordinates, specified in counter clockwise order,
c     starting at the NE corner in logical space,
c     
c                    2 ____________ 1
c                     |            |
c                     |            |
c                     | *(rp,zp)   |
c                     |            |
c                     |            |
c                    3 ____________ 4
c     
c     return the bi-linear interpolation coefficients, ai and bi, 
c     such that, vp, the value at that point, is a function
c     of pq, the physical quantity, specified at the 4 vertices,
c 
c        vp=ai*bi*pq(jz)+ai*(1-bi)*pq(jz-1)+
c           (1-ai)*(1-bi)*pq(jz-kmax-1)+(1-ai)*bi*pq(jz-kmax)
c     
c     Here return the 4-d array 
c     
c        c=[ai*bi, ai*(1-bi), (1-ai)*(1-bi), (1-ai)*bi]
c     
c     so that 
c     
c        vp = c![pq(jz), pq(jz-1), pq(jz-kmax-1), pq(jz-kmax)]
c     
c        
c     
c     
c     Input
c     	r(4)    = r-coords, in counter clockwise order
c     	z(4)    = z-coords, in counter clockwise order
c       (rp,zp) = point in the zone
c     
c     Output
c     
c       c(4) = Bi-linear interpolation coefficients
c            = [ai*bi, ai*(1-bi), (1-ai)*(1-bi), (1-ai)*bi]
c     
      real(Size8) r(4), z(4), c(4), rp,zp,
          r1,r2,r3,r4,z1,z2,z3,z4,ax,bx,cx,ai,bi,dr,dz
c....       point in a zone interior - use bilinear interpolation

            r1=r(1)-rp
            r2=r(2)-rp
            r3=r(3)-rp   
            r4=r(4)-rp   
            z1=z(1)-zp   
            z2=z(2)-zp   
            z3=z(3)-zp   
            z4=z(4)-zp   
            ax=r3*z1-r4*z1-r3*z2+r4*z2-r1*z3+r2*z3+r1*z4-r2*z4
            bx=-r3*z1 + 2*r3*z2 - r4*z2 + r1*z3 - 2*r2*z3 + r2*z4
            cx=-r3*z2 + r2*z3
            bi=2*cx/(-bx-sqrt(bx**2-4*ax*cx))
            dr=(bi*r1 + r2 - bi*r2 - r3 + bi*r3 - bi*r4)
            dz=(bi*z1 + z2 - bi*z2 - z3 + bi*z3 - bi*z4)
            if(abs(dr)>abs(dz))then
              ai=-(r3 - bi*r3 + bi*r4)/dr
            else
              ai=-(z3 - bi*z3 + bi*z4)/dz
            endif
            c(1) = ai*bi
            c(2) = ai*(1-bi)
            c(3) = (1-ai)*(1-bi)
            c(4) = (1-ai)*bi

      return
      end
 
