      subroutine ezcaddp (x, y, nx1, nx2)
# Add a polyplot object to the display list.
#
#     Real arrays x and y contain the x- and y-coordinates of the
#     polygon(s) to be plotted.  They are both assumed dimensioned 
#     (nx1,nx2).  If either of these numbers is 1, a single polygon
#     is to be be plotted; otherwise, npoly=nx2 polygons are to be
#     plotted.  In the latter case npts=nx1>2 is the number of sides
#     of the polygons.
#
# Change record:
#  yymmdd means day dd of month mm of year 20yy.
#  000330  Created by Fred N. Fritsch (A-Program Code Group).
#  000403  Minor correction. (FNF)
#  000404  Modified to allow x and y to be arrays of polygons.  This
#          required changing final argument npts to nx1, nx2.  (FNF)
#  000405  Minor cosmetic changes. (FNF)
#  000410  Modified to allow data to affect plot extrema. (FNF)
#  000515  Changed osallot calls to osallotn. (FNF)
#  000516  Corrected ezcsdata calls. (FNF)
# --------------------------------------------------------------------

#   Declare arguments
      integer nx1, nx2
      real(Size8) y(nx1,nx2), x(nx1,nx2)

Use(Ezcurve)
Use(Collections)
Use(Default_attributes)
Use(Extrema)

#   Declare local variables
      Dynamic(xs,real(Size4),1)
      Dynamic(ys,real(Size4),1)
      integer i, j, k, numdata, npoly, npts
      logical xexlarge, xexsmall, yexlarge, yexsmall
      character*80 errmsg

#   Declare externals
      integer utsizeof
      external utsizeof
      logical ezcnopl
      external ezcnopl
      real(Size4) r1mach
      external r1mach

#   Declare inline functions
      real(Size4) store, floor
      real(Size8) anum
      logical arequal
      real(Size4) arg1,arg2

      store(anum)=real(sign(min(ceiling,abs(anum)),anum),Size4)
      floor(anum)=real(sign(max(realsmall,abs(anum)),anum),Size4)

#       The following assumes arg2>0.
      arequal(arg1,arg2) = abs(arg1-arg2)/max(abs(arg1),arg2) .lt. r1mach(4)

    
#   First executable statement.
      if (ezcnopl()) return  #For no-plot option
#      write (*,*) "Debug: eznaddp called with nx1, nx2=", nx1, nx2

#   Initialize local variables.
      xexlarge = .false.
      yexlarge = .false.
      xexsmall = .false.
      yexsmall = .false.

#   Start filling in the display list.
      call ezcxpnd
      namec(nc) = PolyFill

#   Distinguish between one and multiple polygons.
      if (nx1 == 1) then   # Treat (1,nx2) array as (nx2,1).
         npts = nx2
         npoly = 1
      else
         npts = nx1
         npoly = nx2
      endif
#      write (*,*) "Debug:   npts, npoly=", npts, npoly

#   Save the data, safely converting to real(Size4) for NCAR.
      if (npts > 2) then
         numdata = utsizeof (nx1*nx2, type4)
         call osallotn(Point(xs),numdata,"EZNADDP:xs ")
         call osallotn(Point(ys),numdata,"EZNADDP:ys ")

         k = 0
         do j = 1, nx2
            do i = 1, nx1
               k = k + 1
               xs(k) = store(x(i,j))
               if ( arequal(abs(xs(k)), real(ceiling,Size4)) ) then
                  xexlarge = .true.
               else if (x(i,j) <> 0.) then
                  xs(k) = floor(x(i,j))
                  if ( arequal(abs(xs(k)), real(realsmall,Size4)) ) \
                     xexsmall = .true.
               endif

               ys(k) = store(y(i,j))
               if ( arequal(abs(ys(k)), real(ceiling,Size4)) ) then
                  yexlarge =.true.
               else if (y(i,j) <> 0.) then
                  ys(k) = floor(y(i,j))
                  if ( arequal(abs(ys(k)), real(realsmall,Size4)) ) \
                     yexsmall = .true.
               endif
#               write (*,*) "Debug: i,j,k, x,y=", i,j,k, xs(k), ys(k)
            enddo
         enddo
         if (ezcdebug & k<>numdata) then
            call ezcremrk('WARNING!! plotp consistency check failed:')
            write(errmsg,'(10x,a,i8,a,i8)') \\
                'Number of points stored =', k, ';  expected ', numdata
            call ezcremrk(errmsg)
         endif
         call ezcsdata(Point(xs),npoly,npts,xi(nc),TRUE)
         call ezcsdata(Point(ys),npoly,npts,yi(nc),TRUE)

      else
         call ezcremrk('plotp: Must supply at least 3 points.')
         go to 900
      endif

#   Optional debug printouts.
      if (ezcdebug) then
         if (xexlarge) then
            write(errmsg,'(a,1p,e11.3)') 'WARNING!! plotp x data exceeds plot limit ',ceiling
            call ezcremrk(errmsg)
         endif
         if (xexsmall) then
            write(errmsg,'(a,1p,e11.3)') 'WARNING!! plotp x data is below plot limit ',realsmall
            call ezcremrk(errmsg)
         endif
         if (yexlarge) then
            write(errmsg,'(a,1p,e11.3)') 'WARNING!! plotp y data exceeds plot limit ',ceiling
            call ezcremrk(errmsg)
         endif
         if (yexsmall) then
            write(errmsg,'(a,1p,e11.3)') 'WARNING!! plotp y data is below plot limit ',realsmall
            call ezcremrk(errmsg)
         endif
      endif

      pending = TRUE
      if (ezcshow) call ezcshowf
      return

#   Error return.
900   continue
      call ezcremrk(legenda)
      nc = nc - 1
      return
      end
