      subroutine ezcaddpf (x, y, z, nx1, nx2, cidxflag)
# Add a polyfunc object to the display list.
#
#     Real arrays x and y contain the x- and y-coordinates of the
#     polygon(s) to be treated.  They are both assumed dimensioned 
#     (nx1,nx2).  If either of these numbers is 1, there is a single
#     polygon (npoly=1); otherwise, there are npoly=nx2 polygons.  In
#     the latter case npts=nx1>2 is the number of sides of the polygons.
#
#     In either case, there are npoly elements in the z-array.
#
#     The flag cidxflag is nonzero if z contains color indices.
#
# Change record:
#  yymmdd means day dd of month mm of year 20yy.
#  000412-13  Created by Fred N. Fritsch (A-Program Code Group).
#  000426  Modified to use the new ezcmp8 routines instead of modified
#          ezclr8 routines. (FNF)
#  000515  Changed osallot/osfree calls to osallotn/osfreen. (FNF)
#  000516  Corrected ezcsdata calls. (FNF)
#  010629  Added new cscale options rlin, rlog, rnormal. (FNF)
# --------------------------------------------------------------------

#   Declare arguments
      integer nx1, nx2, cidxflag
      real(Size8) y(nx1,nx2), x(nx1,nx2), z(*)

Use(Ezcurve)
Use(Collections)
Use(Default_attributes)
Use(Extrema)
Use(EzcurveDefaults)

#   Declare local variables
      real(Size8) rco, zlim(2), zz(5)
      Dynamic(xs,real(Size4),1)
      Dynamic(ys,real(Size4),1)
      Dynamic(zexts,real(Size4),5)  #zmin,zmax,zbar,zsigma and zmap for z
      Dynamic(zci,integer,1)        #temporary color indices array
      Dynamic(zs,integer,1)         #stored color indices array

      integer i, j, k, numdata, npoly, npts
      integer nclrs                 #number of colors allocated in ezd
      integer itemp                 #temp index

      logical xexlarge, xexsmall, yexlarge, yexsmall
      character*80 errmsg           #for error messages

#   Declare externals
      integer utsizeof
      external utsizeof
      integer ibasis
      external ibasis
      logical ezcnopl
      external ezcnopl
      real(Size4) r1mach
      external r1mach

#   Declare inline functions
      real(Size4) store,floor
      real(Size8) anum
      logical arequal
      real(Size4) arg1,arg2

      store(anum)=real(sign(min(ceiling,abs(anum)),anum),Size4)
      floor(anum)=real(sign(max(realsmall,abs(anum)),anum),Size4)

#       The following assumes arg2>0.
      arequal(arg1,arg2) = abs(arg1-arg2)/max(abs(arg1),arg2) .lt. r1mach(4)


#   First executable statement.
      if (ezcnopl()) return  #Implement no-plot option
#      write (*,*) "Debug: eznaddpf called with nx1, nx2=", nx1, nx2

#   Initialize local variables.
      xexlarge = .false.
      xexsmall = .false.
      yexlarge = .false.
      yexsmall = .false.

#   Check for user-supplied limits.
      if (nzlima > 0) then
         zlim(1) = zlimin(1)
         zlim(2) = zlimin(2)
      else
         zlim(1) = 0
         zlim(2) = 0
      endif

#   Start filling in the display list.
      call ezcxpnd
      namec(nc)=PolyFunc

#   Distinguish between one and multiple polygons.
      if (nx1 == 1) then   # Treat (1,nx2) array as (nx2,1).
         npts = nx2
         npoly = 1
      else
         npts = nx1
         npoly = nx2
      endif
#      write (*,*) "Debug:   npts, npoly=", npts, npoly

#   Convert to color indices from physics quantities.
      nclrs = ibasis("ezd.numcol")
      call osallotn(Point(zci),npoly,"EZNADDPF:zci ")
      if (cidxflag == 1) then
         zz(5) = 3
         do k = 1, npoly
            itemp = int(z(k))
            if (itemp > 0 & itemp <= nclrs) then
               zci(k) = itemp
            else
               call ezcremrk("plotpf: bad index, negative or exceeds limit.")
               call osfreen(Point(zci),"ezn.zci")
               go to 900
            endif
         enddo
      else
         zz(5) = 1  !Set for normal color order.
         rco = 0
         if (cscalea == "rlog" | cscalea == "rnormal" | \
             cscalea == "rlin")  then  !Reverse color order.
              rco = 4
              zz(5) = -1
#           Note that the correct output value of zz(5)
#             zz(5) = 0 : linear mapping
#             zz(5) = 1 : logarithmic mapping
#             zz(5) = 2 : normal mapping
#           will be set by the ezcmp8-routine called below.
         endif
         if (cscalea == "log" | cscalea == "rlog") then
            call ezcmp8ln(nclrs,npoly,z,zci,zz,zlim)
            if (zz(1) <= 0 | zz(2) <= 0) then
#               Bad data message already printed by ezcmp8ln.
               call osfreen(Point(zci),"ezn.zci")
               go to 900
            endif
         elseif (cscalea == "normal" | cscalea == "rnormal") then
            call ezcmp8nml(nclrs,npoly,z,zci,zz,zlim)
         else
            call ezcmp8(nclrs,npoly,z,zci,zz,zlim)
         endif
#         Add rco so that zz(5) is in range 0-2 for normal color order,
#         and in range 4-6 for reverse color order.  (Note that zz(5)=3
#         has the special meaning that user has supplied color indices.)
#         This is used in ezcdispl to get correct annotation. (FNF)
         zz(5) = zz(5) + rco
      endif

#   Save color indices for display list.
      call osallotn(Point(zs),npoly,"EZNADDPF:zs ")
      do k = 1, npoly
         zs(k) = zci(k) + ibasis("ezd.indbase") - 1
      enddo
      call osfreen(Point(zci),"ezn.zci")

#   Save contents of zz array.
      call osallotn(Point(zexts),5,"EZNADDPF:zexts ")
      zexts(1) = store(zz(1))
      zexts(2) = store(zz(2))
      zexts(3) = store(zz(3))
      zexts(4) = store(zz(4))
      zexts(5) = store(zz(5))

#   Save the polygon data, safely converting to real(Size4) for NCAR.
      if (npts > 2) then
         numdata = utsizeof (nx1*nx2, type4)
         call osallotn(Point(xs),numdata,"EZNADDPF:xs ")
         call osallotn(Point(ys),numdata,"EZNADDPF:ys ")

         k = 0
         do j = 1, nx2
            do i = 1, nx1
               k = k + 1
               xs(k) = store(x(i,j))
               if ( arequal(abs(xs(k)), real(ceiling,Size4)) ) then
                  xexlarge = .true.
               else if (x(i,j) <> 0.) then
                  xs(k) = floor(x(i,j))
                  if ( arequal(abs(xs(k)), real(realsmall,Size4)) ) \
                     xexsmall = .true.
               endif

               ys(k) = store(y(i,j))
               if ( arequal(abs(ys(k)), real(ceiling,Size4)) ) then
                  yexlarge =.true.
               else if (y(i,j) <> 0.) then
                  ys(k) = floor(y(i,j))
                  if ( arequal(abs(ys(k)), real(realsmall,Size4)) ) \
                     yexsmall = .true.
               endif
#               write (*,*) "Debug: i,j,k, x,y=", i,j,k, xs(k), ys(k)
            enddo
         enddo
         if (ezcdebug & k<>numdata) then
            call ezcremrk('WARNING!! plotpf consistency check failed:')
            write(errmsg,'(10x,a,i8,a,i8)') \\
                'Number of points stored =', k, ';  expected ', numdata
            call ezcremrk(errmsg)
         endif

         call ezcsdata(Point(xs),npoly,npts,xi(nc),TRUE)
         call ezcsdata(Point(ys),npoly,npts,yi(nc),TRUE)
         call ezcsdata(Point(zs),1,npoly,zi(nc),FALSE)
         call ezcsdata(Point(zexts),1,5,zexti(nc),FALSE)

      else
         call ezcremrk('plotpf: Must supply at least 3 points.')
         go to 900
      endif

#   Optional debug printouts.
      if (ezcdebug) then
         if (xexlarge) then
            write(errmsg,'(a,1p,e11.3)') 'WARNING!! x data exceeds plot limit ',ceiling
            call ezcremrk(errmsg)
         endif
         if (xexsmall) then
            write(errmsg,'(a,1p,e11.3)') 'WARNING!! x data is below plot limit ',realsmall
            call ezcremrk(errmsg)
         endif
         if (yexlarge) then
            write(errmsg,'(a,1p,e11.3)') 'WARNING!! y data exceeds plot limit ',ceiling
            call ezcremrk(errmsg)
         endif
         if (yexsmall) then
            write(errmsg,'(a,1p,e11.3)') 'WARNING!! y data is below plot limit ',realsmall
            call ezcremrk(errmsg)
         endif
      endif

      pending = TRUE
      if (ezcshow) call ezcshowf
      return

#   Error return.
900   continue
      call ezcremrk(legenda)
      nc = nc - 1
      return
      end
