c==============================================================================
c
c  ezclr4ln
c
c  purpose: convert an array of reals into an array of indices into a
c     colormap using a logarithmic mapping.
c
c  created: bert still, 930406
c
c  revision history:
c     - renamed from r2clrln (ezcr2cln), [chs] 931031.
c     - add check for zmin .eq. zmax  YHC 950407
c     - Corrected formula for ndx.  (FNF 961226)
c     - As in ezclr8ln, added working array ztmp to avoid trashing z.
c       (FNF 961226)
c     - Improved main loop to avoid problems with large values and to make
c       mapping scale-independent.  Also added test for nonpositive data.
c       (FNF 961227)
c     - Removed unused variable epsilon.  (FNF 970103)
c     - Changed "call remark" statement to "call ezcremrk".  (FNF 970813)
c
c==============================================================================


      subroutine ezclr4ln(ncol, n, z, ndx)
c
      integer        ncol, n, ndx(n)
      real(Size4)    z(n)
c
      integer        i, ncm1
      real(Size4)    zmin, zmax
c
      integer utsizeof, utstrcod
      external utsizeof, utstrcod
      integer numdata, type4
      Dynamic(ztmp,real(Size4),1)

c   Define function to map f (in [0.,1.]) to color index (in [0,nind]),
c   mapping any negative f's to 0, f > 1 to nind.  (FNF)
      integer      nind, colmap
      real(Size4)  f
      colmap(f,nind) = max(0, min( int(f*nind), nind) )

c find range of data
      zmin = z(1)
      zmax = z(1)
      do i=2, n
         if (z(i) > zmax) zmax = z(i)
         if (z(i) < zmin) zmin = z(i)
      enddo

c check for valid data
      if (zmin <= 0 | zmax <= 0) then
         call ezcremrk ("ezclr4ln: nonpositive data")
         return
      endif
      zmin = log(zmin)
      zmax = log(zmax)

c for the case of zmin .eq. zmax
      if (zmax == zmin) then
         do i=1,n
            ndx(i) = 1
         enddo
c
         return
      endif

c allocate work array
      type4 = utstrcod(Quote(real(Size4)))
      numdata = utsizeof(n,type4)
      call osallotn(Point(ztmp),numdata,"EZCCLR4LN:ztmp ")

c take logs of data (safe, because of previous tests)
      do i=1, n
         ztmp(i) = log(z(i))
      enddo
c     (Recall that zmin,zmax are already log-mapped.)

c convert data to color indices
      ncm1 = ncol - 1
      do i=1, n
         ndx(i) = colmap( (ztmp(i) - zmin)/(zmax - zmin), ncm1) + 1
      enddo
c
      call osfreen(Point(ztmp),"ezn.ztmp")
      return
      end
