c==============================================================================
c
c  ezclr8
c
c  Purpose: Convert an array of real8's into an array of indices into a
c     colormap using a linear mapping.
c
c  Arguments:
c     ncol  Number of colors in colormap.
c     n     Number of elements in array z.
c     z     The array to be mapped.
c     ndx   The output array of color indices, in range [1,ncol].
c     zz    Auxiliary output array.  This routine returns zz(1)=zmin,
c           zz(2)=zmax, zz(5)=zmap=0 (to signal linear mapping).
c
c  Routines called: ezcmp8.
c
c  Created: Bert Still, 931031
c
c  Revision History:
c     - Developed from clr4.
c     - Add argument zz(1:5) to return zmin, zmax, zbar, zsigma, zmap 
c	(YHC 940816)
c     - Add check for zz(1) .eq. zz(2)   (YHC 950407)
c     - Corrected formula for ndx.  (FNF 961226)
c     - Modified to allow user-supplied limits.  (FNF 000414)
c     - Modified to call new ezcmp8 to do color mapping.  This restores
c       zz to output only and eliminates duplicate coding.  (FNF 000426)
c
c==============================================================================

      subroutine ezclr8 (ncol, n, z, ndx, zz)

c Declare arguments.
      integer        ncol, n, ndx(n)
      real(Size8)    z(n), zz(5)

c Declare local variables.
      real(Size8)    zlim(2)

c Initialize zlim to cause computation of limits by ezcmp8.
      zlim(1) = 0
      zlim(2) = 0

c Do the color mapping.
      call ezcmp8 (ncol, n, z, ndx, zz, zlim)
 
      return
      end
 
         
