c==============================================================================
c
c  ezclr8ln
c
c  Purpose: Convert an array of real8's into an array of indices into a
c     colormap using a logarithmic mapping.
c
c  Arguments:
c     ncol  Number of colors in colormap.
c     n     Number of elements in array z.
c     z     The array to be mapped.
c     ndx   The output array of color indices, in range [1,ncol].
c     zz    Auxiliary output array.  This routine returns zz(1)=zmin,
c           zz(2)=zmax, zz(5)=zmap=1 (to signal logarithmic mapping).
c
c  Routines called: ezcmp8ln.
c
c  Created: Bert Still, 931031
c
c  Revision History:
c     - Developed from clr4ln.
c     - Add ztmp working array to avoid trash z (YHC 940812)
c     - Add arguments zz(1:5) to return zmin, zmax, zbar, zsigma, zmap
c	(YHC 940816)
c     - Add check for zz(1) .eq. zz(2)  (YHC 950407)
c     - Corrected formula for ndx.  Also moved ztmp allocation so it
c       gets properly deallocated.  (FNF 961226)
c     - Improved main loop to avoid problems with large values and to make
c       mapping scale-independent.  Also added test for nonpositive data.
c       (FNF 961227)
c     - Removed unused variable epsilon.  (FNF 970103)
c     - Changed "call remark" statement to "call ezcremrk".  (FNF 970813)
c     - Modified to allow user-supplied limits.  (FNF 000414)
c     - Modified to call new ezcmp8ln to do color mapping.  This restores
c       zz to output only and eliminates duplicate coding.  (FNF 000426)
c
c==============================================================================

      subroutine ezclr8ln (ncol, n, z, ndx, zz)

c Declare arguments.
      integer        ncol, n, ndx(n)
      real(Size8)    z(n), zz(5)

c Declare local variables.
      real(Size8)    zlim(2)

c Initialize zlim to cause computation of limits by ezcmp8ln.
      zlim(1) = 0
      zlim(2) = 0

c Do the color mapping.
      call ezcmp8ln (ncol, n, z, ndx, zz, zlim)
 
      return
      end
 
         
