c==============================================================================
c
c  ezclr8nml
c
c  Purpose: Convert an array of real8's into an array of indices into a
c     colormap using a normal mapping.
c
c  Arguments:
c     ncol  Number of colors in colormap.
c     n     Number of elements in array z.
c     z     The array to be mapped.
c     ndx   The output array of color indices, in range [1,ncol].
c     zz    Auxiliary output array.  This routine returns zz(1)=zmin,
c           zz(2)=zmax, zz(3)=zbar, zz(4)=zsigma, zz(5)=zmap=2 (to signal
c           normal mapping).
c
c  Routines called: ezcmp8nml.
c
c  Created: Yu-Hsing Chiu, 940819
c
c  Revision History:
c     - Add check for zz(1) .eq. zz(2)  (YHC 950407)
c     - Move location of ztmp allocation to assure deallocation.  (FNF 961230)
c     - Modified to allow user-supplied limits.  (FNF 000414)
c     - Added code to handle special case all data equal.  (FNF 000420)
c     - Modified to call new ezcmp8nml to do color mapping.  This restores
c       zz to output only and eliminates duplicate coding.  (FNF 000426)
c
c==============================================================================

      subroutine ezclr8nml (ncol, n, z, ndx, zz)

c Declare arguments.
      integer        ncol, n, ndx(n)
      real(Size8)    z(n), zz(5)

c Declare local variables.
      real(Size8)    zlim(2)

c Initialize zlim to cause computation of limits by ezcmp8nml.
      zlim(1) = 0
      zlim(2) = 0

c Do the color mapping.
      call ezcmp8nml (ncol, n, z, ndx, zz, zlim)
 
      return
      end
 
         
