c==============================================================================
c
c  ezcmp8
c
c  Purpose: Convert an array of real8's into an array of indices into a
c     colormap using a linear mapping, with optional user-supplied limits.
c
c  Arguments:
c     ncol  Number of colors in colormap.
c     n     Number of elements in array z.
c     z     The array to be mapped.
c     ndx   The output array of color indices, in range [1,ncol].
c     zz    Auxiliary output array.  This routine returns zz(1)=zmin,
c           zz(2)=zmax, zz(5)=zmap=0 (to signal linear mapping).
c           If the input value of zz(5) is negative, reverse the mapping.
c     zlim  If the input values zlim(1), zlim(2) are not equal, they are
c           treated as user-supplied limits.  (Ordering not assumed.)
c
c  Routines called: eznminmx.
c
c  Created: Fred N. Fritsch, 26 April 2000
c
c  Revision History:
c     - Created from 000414 version of ezclr8, as modified to call
c       eznmimmx to compute array limits.  Note that ANY two equal
c       values cause computation of the actual array limits.
c     - Made zz(5) an input, to determine order of color map. (FNF 010702)
c
c==============================================================================

      subroutine ezcmp8 (ncol, n, z, ndx, zz, zlim)

c Declare arguments.
      integer        ncol, n, ndx(n)
      real(Size8)    z(n), zz(5), zlim(2)

c Declare local variables.
      integer i, imax, imin, ncm1

c Define function to map f (in [0.,1.]) to color index (in [0,nind]),
c mapping any negative f's to 0, f > 1 to nind.  (FNF)
      integer      nind, colmap
      real(Size8)  f
      colmap(f,nind) = max(0, min( int(f*nind), nind) )

c Find range of data.
      if (zlim(1) <> zlim(2)) then
         zz(1) = min( zlim(1), zlim(2) )
         zz(2) = max( zlim(1), zlim(2) )
      else
         call eznminmx (z, 1, n, 1, zz(1), zz(2), imin, imax)
      endif

c Check for constant data.
      if ( zz(1) == zz(2)) then
         do i=1,n
            ndx(i) = 1
         enddo
c       
         go to 30
      endif

c Convert data to color indices.
      ncm1 = ncol - 1
      do i=1, n
         ndx(i) = colmap( (z(i) - zz(1))/(zz(2) - zz(1)), ncm1) + 1
c         Check for reverse order desired.
         if (zz(5) < 1) ndx(i) = ncol+1 - ndx(i)
      enddo
c
   30 continue
      zz(3) = 0
      zz(4) = 0
      zz(5) = 0    #linear mapping
      return
      end
 
         
