      subroutine ezcdefb(ireg,kmax,lmax,kpoint,lpoint,kseg,lseg)
#*s
#     NAME OF ROUTINE...ezcdefb
#
#     AUTHOR OF ROUTINE...Janet Takemoto  04/14/88
#
#     PURPOSE...
#
#        To create the mesh connectivity arrays needed to plot the 
#        boundaries.  This is the default version.
#
#     GLOSSARY OF IMPORTANT VARIABLES...
#
#        kmax                 = the maximum k value for the r and z arrays.
#        lmax                 = the maximum l value for the r and z arrays.
#        krangea               = user supplied rangea for the k-lines
#                               krangea(1) is the minimum value for k.
#                               krangea(2) is the maximum value for k.
#        lrangea               = user supplied rangea for the l-lines
#                               lrangea(1) is the minimum value for l.
#                               lrangea(2) is the maximum value for l.
#        ireg                 = zonal region numbers array.  it is
#                               dimensional, kmax by lmax.  It is assumed
#                               that the values of the first row and first
#                               column are zero.

#        mconn        =  connectivity array holding indices into the r
#                               and z arrays.  line segments are drawn between
#                               (r(mconn(i)),z(mconn(i))) and 
#                                (r(mconn(i+1)),z(mconn(i+1)).
#        npoint               = the number of elements in connectivity arrays.
#
#     COMMENTS...
#
#        Here, the mesh is assumed to be quadrilateral.
#
#/c
Use(Ezcurve)
Use(Collections)
Use(Default_attributes)
      integer kmax,lmax,kpoint,lpoint, kseg, lseg
      integer ireg(kmax,lmax)
      integer k, l, plkmin, plkmax, pllmin, pllmax
      integer nks, nls, maxpoint, maxcrvk, maxcrvl
      logical kanchor, lanchor
      integer utsizeof
      external utsizeof

      if(krangea(1) == DEFAULT) then
         plkmin = 1
      else
         plkmin = max(1,krangea(1))
      endif
      if(lrangea(1) == DEFAULT) then
         pllmin = 1
      else
         pllmin = max(1,lrangea(1))
      endif
      if(krangea(2) == DEFAULT) then
         plkmax = kmax
      else
         plkmax = min(kmax,krangea(2))
      endif
      if(lrangea(2) == DEFAULT) then
         pllmax = lmax
      else
         pllmax = min(lmax,lrangea(2))
      endif
      plkmax=max(plkmin,plkmax)
      pllmax=max(pllmin,pllmax)
#note; increments ignored for doing the boundary
      nks = 1+(plkmax-plkmin)
      nls = 1+(pllmax-pllmin)
      maxpoint = 2*nks*nls
      call osallotn(Point(mconn),maxpoint,"EZCDEFB:mconn ")
      maxcrvl = nls*(nks/2)+1
      call osallotn(Point(llen),maxcrvl,"EZCDEFB:llen ")
      maxcrvk = nks*(nls/2)+1
      call osallotn(Point(klen),maxcrvk,"EZCDEFB:klen ")

      kpoint = 0      
      kseg = 0
#     -- Do for horizontal lines.
      if (kstylea <> "none") then
         do k=plkmin,plkmax
            lanchor = FALSE
            do l=pllmin+1,pllmax
               if(excludea(ireg(k,l))) then
                  if(k<kmax) then
                     if(excludea(ireg(k+1,l))) then
                        lanchor = FALSE
                        next
                     endif
                  else
                     lanchor = FALSE
                     next
                  endif
               endif
               if(k < kmax) then
                  if(ireg(k,l) == ireg(k+1,l)) then
                     lanchor = FALSE
                     next
                  endif
               endif
               if(lanchor) then
                  kpoint = kpoint + 1
                  mconn(kpoint)=(l-1)*kmax+k
                  klen(kseg) = klen(kseg) + 1
               else
                  kpoint = kpoint + 2
                  kseg = kseg+1
                  klen(kseg)=2
                  mconn(kpoint)=(l-1)*kmax+k
                  mconn(kpoint-1)=(l-2)*kmax+k
                  lanchor = TRUE
               endif
            enddo
         enddo
      endif
#     -- Do for vertical lines.
      lpoint = kpoint
      lseg = 0
      if (lstylea <> "none") then
         do l=pllmin,pllmax
            kanchor = FALSE
            do k=plkmin+1,plkmax
               if(excludea(ireg(k,l))) then
                  if(l<lmax) then
                     if(excludea(ireg(k,l+1))) then
                        kanchor = FALSE
                        next
                     endif
                  else
                     kanchor = FALSE
                     next
                  endif
               endif
               if(l < lmax) then
                  if(ireg(k,l) == ireg(k,l+1)) then
                     kanchor = FALSE
                     next
                  endif
               endif
               if(kanchor) then
                  lpoint = lpoint + 1
                  mconn(lpoint)=(l-1)*kmax+k
                  llen(lseg)=llen(lseg) + 1
               else
                  lpoint = lpoint + 2
                  lseg = lseg + 1
                  llen(lseg) = 2
                  mconn(lpoint)=(l-1)*kmax+k
                  mconn(lpoint-1)=(l-1)*kmax+k-1
                  kanchor = TRUE
               endif
            enddo
         enddo
      endif
      lpoint = lpoint - kpoint

      return
      end
