c==============================================================================
c
c  ezcfmcln
c
c  Purpose: Convert an array of real8's into an array of indices into a
c     colormap using a logarithmic mapping with user supplied zmin and zmax.
c
c  Arguments:
c     ncol  Number of colors in colormap.
c     n     Number of elements in array z.
c     z     The array to be mapped.
c     ndx   The output array of color indices, in range [1,ncol].
c     zlim  Input array containing limits: zlim(1)=zmin, zlim(2)=zmax.
c
c  Routines called: ezcmp8ln.
c
c  Created: Yu-Hsing Chiu, 940811
c
c  Revision History:
c     - Corrected formula for ndx.  (FNF 961226)
c     - Improved main loop to avoid problems with large values and to make
c       mapping scale-independent.  Also added test for nonpositive limits.
c       (FNF 961227)
c     - Removed unused variable epsilon.  (FNF 970103)
c     - Changed "call remark" statement to "call ezcremrk".  (FNF 970813)
c     - Corrected function name in error message.
c       Also made some cosmetic changes.  (FNF 000420)
c     - Modified to call new ezcmp8ln to do color mapping, which elimi-
c       nates duplicate coding.  (FNF 000426)
c
c==============================================================================

      subroutine ezcfmcln (ncol, n, z, ndx, zlim)

c Declare arguments.
      integer        ncol, n, ndx(n)
      real(Size8)    z(n), zlim(2)

c Declare local variables.
      real(Size8)    zz(5)  #For auxiliary output from ezcmp8ln

c Do the color mapping.
      call ezcmp8ln (ncol, n, z, ndx, zz, zlim)

      return
      end
