      integer function ezcplotp (nargs, nargs1)
#
#  Process the command: plotp x y keylist
#       where
#          x, y  contain the x-, y-coordinates of the polygon(s) to
#                be plotted.  (Both are required.)
#
#       nargs1 is the number of arguments found prior to "keylist".
#
# Change record:
#  yymmdd means day dd of month mm of year 20yy.
#  000330  Created by Fred N. Fritsch (A-Program Code Group).
#  000404  Modified to allow x and y to be arrays of polygons. (FNF)
#  000515  Removed ss and sp as arguments. (FNF)
# --------------------------------------------------------------------

#   Declare arguments
      integer nargs, nargs1

Use(Keywords)      
Use(Collections)
Use(Default_attributes)
Use(EzcurveDefaults)

#   Declare general local variables directly related to arguments.
      integer sx(SS_WIDTH), sy(SS_WIDTH)
      integer nx1, nx2, ny1, ny2

      Dynamic(x,real(Size8),1)
      Dynamic(y,real(Size8),1)

#   Declare other needed local variables.
      character*80 msg     # for constructing error messages.

#   Declare externals.
      Address  paraddr
      external paraddr
      integer  parcoerc, parlen
      external parcoerc, parlen
#      Omit following if not needed.
      integer utsizeof
      external utsizeof

#   First executable statement.
#      write (*,*) "Debug: Have entered ezcplotp"
#   Get argument list.
      call ezcseta(nargs,nargs1,LOCAL)

#   Check for valid number of arguments.
      if (nargs1 <> 2) then
         write(msg,100) nargs1
         call ezcremrk(msg)
100      format('plotp: there must be 2 numerical arguments, not',i5)
         go to 950   # Skip over parkill calls--not appropriate here.
      endif

#   Pick up contents of arguments.
      call fcnarga(1,sx)
      call pardim(sx) 

      call fcnarga(2,sy)
      call pardim(sy) 

#   Process arguments and check for validity.

      if (parcoerc(sx,type8)==ERR) go to 900
      Point(x) = paraddr(sx)
      nx1 = sx(SS_N1)
      if (sx(SS_N) == 1) then        #single polygon
         nx2 = 1
      else if (sx(SS_N) == 2) then   #array of polygons
         nx2 = sx(SS_N1+1)
      else
         call ezcremrk('plotp: 1- or 2-dimensional arrays required.')
         go to 900
      endif
#      write (*,*) "Debug: sx(SS_N), sx(SS_N1), sx(SS_N1+1) = ",
#                          sx(SS_N), sx(SS_N1), sx(SS_N1+1)

      if (parcoerc(sy,type8)==ERR) go to 900
      Point(y) = paraddr(sy)
      ny1 = sy(SS_N1)
      if (sy(SS_N) == 1) then        #single polygon
         ny2 = 1
      else if (sy(SS_N) == 2) then   #array of polygons
         ny2 = sy(SS_N1+1)
      else
         call ezcremrk('plotp: 1- or 2-dimensional arrays required.')
         go to 900
      endif
#      write (*,*) "Debug: sy(SS_N), sy(SS_N1), sy(SS_N1+1) = ",
#                          sy(SS_N), sy(SS_N1), sy(SS_N1+1)

      if (nx1 <> ny1 | nx2 <> ny2) then
         call ezcremrk('plotp: argument size mismatch.')
         go to 900
      endif

#   Add this to the collection.
      call ezcaddp (x, y, nx1, nx2)

      call parkill(sx)
      call parkill(sy)
      call ezcrda
      return(OK)

#   Error returns.
900   continue
      call parkill(sx)
      call parkill(sy)
950   continue
      call ezcrda
      call ezcremrk('plotp command ignored.')
      call ezcremrk(legenda)   # Echo the command to the terminal.
      return(OK)   # OK just ignores command, doesn't kill collection.
      end
