      integer function ezcplotpf (nargs, nargs1)
#
#  Process the command: plotpf z x y keylist
#       where
#          z     is an array of function values on a set of polygons,
#                or optionally an array of color indices;
#          x, y  contain the x-, y-coordinates of the polygon(s).
#       (All three arguments are required.)
#
#       nargs1 is the number of arguments found prior to "keylist".
#
# Change record:
#  yymmdd means day dd of month mm of year 20yy.
#  000412  Created by Fred N. Fritsch (A-Program Code Group).
#  000515  Removed ss and sp as arguments. (FNF)
# --------------------------------------------------------------------

#   Declare arguments
      integer nargs, nargs1

Use(Keywords)      
Use(Collections)
Use(Default_attributes)
Use(EzcurveDefaults)

#   Declare general local variables directly related to arguments.
      integer sy(SS_WIDTH), sx(SS_WIDTH), sz(SS_WIDTH)
      integer nx1, nx2, ny1, ny2, nz

      Dynamic(x,real(Size8),1)
      Dynamic(y,real(Size8),1)
      Dynamic(z,real(Size8),1)		#physics quantity to be color coded

#   Declare other needed local variables.
      integer cidxflag			#flag for z value as color indices
      character*80 msg

#   Declare externals.
      integer utsizeof
      external utsizeof
      Address  paraddr
      external paraddr
      integer  parcoerc, parlen, partypea
      external parcoerc, parlen, partypea

#   First executable statement.
#      write (*,*) "Debug: Have entered ezcplotpf"
      cidxflag = 0			#initialize to 0

#   Get argument list.
      call ezcseta(nargs,nargs1,LOCAL)

#   Check for valid number of arguments.
      if (nargs1 <> 3) then
         write(msg,100) nargs1
         call ezcremrk(msg)
100      format('plotpf: there must be 3 numerical arguments, not',i5)
         go to 950   # Skip over parkill calls--not appropriate here.
      endif

# Note: The following if-then-else is to process the first argument,
# which can either be an array of physics variables (or color indices)
# or a function that produces same.  It first checks whether it is a
# function.  If so, it has the parser invoke the function and put the
# result in variable ezc.ezccin.  Since we already know that nargs1=3,
# the 4 in the fcnargd call will cause sz to take the value of ezc.ezccin.
# The else branch merely picks up the value of the argument in sz.
# Since no default is appropriate, fcnarga is called instead of fcnargd.
      if ( partypea(1) == typef) then
        call parsestr(
"ezc chameleon ezccin = "//symbols(1)(1:lsymbols(1))//"()")
        call fcnargd(nargs1,4,sz,"ezc.ezccin")
      else
        call fcnarga(1,sz)
      endif
      call pardim(sz) 
#      write (*,*) "Debug: sz(SS_N), sz(SS_N1) = ", sz(SS_N), sz(SS_N1)

# Simply pick up arguments 2 and 3. (Again, no defaults.)
      call fcnarga(2,sx)
      call pardim(sx) 
#      write (*,*) "Debug: sx(SS_N), sx(SS_N1), sx(SS_N1+1) = ",
#                          sx(SS_N), sx(SS_N1), sx(SS_N1+1)

      call fcnarga(3,sy)
      call pardim(sy) 
#      write (*,*) "Debug: sy(SS_N), sy(SS_N1), sy(SS_N1+1) = ",
#                          sy(SS_N), sy(SS_N1), sy(SS_N1+1)


# Check for type of z input, to distinguish between color index input
# and physics quantity input.  (cidxflag is passed to ezcaddpf.)
      if (sz(SS_TC) == typeint) cidxflag = 1

      if (parcoerc(sz,type8)==ERR) go to 900
      Point(z) = paraddr(sz)
      if (sz(SS_N) == 0) then        # scalar
         nz = 1
      else if (sz(SS_N) == 1) then   # 1-D array of values
         nz = sz(SS_N1)
      else
         call ezcremrk('plotpf: z must be a scalar or 1-dimensional array.')
         go to 900
      endif

      if (parcoerc(sx,type8)==ERR) go to 900
      Point(x) = paraddr(sx)
      nx1 = sx(SS_N1)
      if (sx(SS_N) == 1) then        #single polygon
         nx2 = 1
      else if (sx(SS_N) == 2) then   #array of polygons
         nx2 = sx(SS_N1+1)
      else
         call ezcremrk('plotpf: 1- or 2-dimensional x,y arrays required.')
         go to 900
      endif

      if (parcoerc(sy,type8)==ERR) go to 900
      Point(y) = paraddr(sy)
      ny1 = sy(SS_N1)
      if (sy(SS_N) == 1) then        #single polygon
         ny2 = 1
      else if (sy(SS_N) == 2) then   #array of polygons
         ny2 = sy(SS_N1+1)
      else
         call ezcremrk('plotpf: 1- or 2-dimensional x,y arrays required.')
         go to 900
      endif

      if ((nx1 == 1 & nz <> 1)  |     #single polygon case
          (nx1 > 1 & nz <> nx2) |     #array of polygons case
           nx1 <> ny1 | nx2 <> ny2) then
         call ezcremrk('plotpf: argument size mismatch.')
         go to 900
      endif

#   Add this to the collection.
      call ezcaddpf (x, y, z, nx1, nx2, cidxflag)

      call parkill(sz)
      call parkill(sy)
      call parkill(sx)
      call ezcrda
      return(OK)

#   Error returns.
900   continue
      call parkill(sz)
      call parkill(sy)
      call parkill(sx)
950   continue
      call ezcrda
      call ezcremrk("plotpf command ignored.")
      call ezcremrk(legenda)   # Echo the command to the terminal.
      return(OK)   # OK just ignores command, doesn't kill collection.
      end
