# This file is intended to provide a general skeleton for the ezcplotX
# family of functions.  Replace X with the appropriate letter(s) and
# customize as appropriate for the command being processed.  Of course,
# you will need to add an appropriate mcchrdef call to ezcinit, write
# a routine ezcaddX to add the necessary data to the database for the
# plot display list, and add code to ezcdisp to plot the data.  You must
# also add information about ezcplotX and associated variables to ezn.v.
# (See file README in this directory for more details.)
#
# This pattern is for a mesh-oriented command with a first argument z
# followed by x,y,ireg (a.k.a. zt,rt,ireg).
#
# Change record:
#   970724  Created by Fred Fritsch.
#   970813  Changed "call remark" statements to "call ezcremrk".  (FNF)
#   990813  Removed ss and sp as arguments.  (ZCM)
#   000512  Made above instructions more complete.  (FNF)
#
#--------------------------( cut here )---------------------------------
      integer function ezcplotX (nargs, nargs1)
      integer nargs, nargs1
#
#       Process the command: plotX z x y ireg keylist
#       where
#          z        is required, but
#          x,y,ireg can be defaulted to ezcx,ezcy,ezcireg respectively.
#
#       nargs1 is the number of arguments found prior to "keylist".


Use(Keywords)      
Use(Collections)
Use(Default_attributes)
Use(EzcurveDefaults)

#   Declare general local variables directly related to arguments.
      integer sz(SS_WIDTH), sx(SS_WIDTH), sy(SS_WIDTH), sireg(SS_WIDTH)
      integer nz1, nz2, nx1, nx2, ny1, ny2, ni1, ni2

      Dynamic(z,real(Size8),1)
      Dynamic(x,real(Size8),1)
      Dynamic(y,real(Size8),1)
      Dynamic(ireg,integer,1)

#   Declare other needed local variables.
      character*80 msg     # for constructing error messages.

#   Declare externals.
      Address  paraddr
      external paraddr
      integer  parcoerc, parlen
      external parcoerc, parlen
#      Omit following if not needed.
      integer utsizeof
      external utsizeof

#   Get argument list.
      call ezcseta(nargs,nargs1,LOCAL)

#   Check for valid number of arguments.
#    (The number here and in format 100 will depend on the command.)
      if (nargs1 > 4) then
         write(msg,100) nargs1
         call ezcremrk(msg)
100      format('plotX: there must be 0-4 numerical arguments, not',i5)
         go to 950   # Skip over parkill calls--not appropriate here.
      endif

#   Pick up contents of arguments.
      call fcnarga(1,sz)
#   (The first argument has no default, so fcnargd is not appropriate. )
#   (See ezcplotf for an example of how to allow this to be a function.)
      call fcnargd(nargs1,2,sx,ezcx)
      call fcnargd(nargs1,3,sy,ezcy)
      call fcnargd(nargs1,4,sireg,ezcireg)

      call pardim(sz) 
      call pardim(sy) 
      call pardim(sx) 
      call pardim(sireg)

#   Process arguments and check for validity.
#   (Note: if the first argument may be either integer or real8, and   )
#   (      different treatment applies to the two cases, add something )
#   (      like the following here:                                    )

##    Check for type of input z.
#      if(sz(SS_TC) == typeint) intflag = 1

#   (    Pass intflag as an additional argument to ezcaddX, and then   )
#   (    process the data accordingly there.  See ezcaddfm for example.)

      if(parcoerc(sz,type8)==ERR) go to 900
      Point(z) = paraddr(sz)
      if(sz(SS_N) <> 2) go to 900
      nz1 = sz(SS_N1)
      nz2 = sz(SS_N1+1)

      if(parcoerc(sy,type8)==ERR) go to 900
      Point(y) = paraddr(sy)
      if(sy(SS_N) <> 2) go to 900
      ny1 = sy(SS_N1)
      ny2 = sy(SS_N1+1)

      if(parcoerc(sx,type8)==ERR) go to 900
      Point(x) = paraddr(sx)
      if(sx(SS_N) <> 2) go to 900
      nx1 = sx(SS_N1)
      nx2 = sx(SS_N1+1)

      if(parcoerc(sireg,typeint) == ERR) go to 900
      Point(ireg) = paraddr(sireg)
      if(sireg(SS_N) <> 2) go to 900
      ni1 = sireg(SS_N1)
      ni2 = sireg(SS_N1+1)
      if(ni1 <> nz1 | ni2 <> nz2) then
         call ezcremrk('plotX: ireg the wrong size.')
         go to 900
      endif
      if(nz1 <> ny1 | nz2 <> ny2 | nz1 <> nx1 | nz2 <> nx2) then
         call ezcremrk('plotX: argument size mismatch.')
         go to 900
      endif

#   Add this to the collection.
      call ezcaddX(x,y,z,ireg,nz1,nz2)

      call parkill(sz)
      call parkill(sy)
      call parkill(sx)
      call parkill(sireg)
      call ezcrda
      return(OK)

#   Error returns.
900   continue
      call parkill(sz)
      call parkill(sy)
      call parkill(sx)
      call parkill(sireg)
950   continue
      call ezcrda
      call ezcremrk("plotX command ignored.")
      call ezcremrk(legenda)   # Echo the command to the terminal.
      return(OK)   # OK just ignores command, doesn't kill collection.
      end
