c=======================================================================
c
c  isosurface plotting: plot all t(i,j,k) = c0
c
c  Notes:
c   o this routine is not designed to stand alone.  it will only work
c     with the basis system.
c
c   o the scratch array should be at least (max(nx,ny,nz)+2)^2 words
c     long, ie. at least twice the size of z(); we get the scratch array
c     dynamically.
c
c   o it is recommended that the viewpoint be outside the plotted
c     box by a factor of 2-5 on the horizontal and 5-10 on the vertical
c     and the point should be well outside the box on the z-coordinate.
c     since i am using perspective viewing, we don't want the object
c     to be too small, so we might have to use smaller multipliers.
c     the point we will look towards is the origin: by design, the
c     origin will be the center of the plotted surface.
c
c   o we accept any size arrays on input now, and provide data thinning
c     as necessary to produce a good plot.  the number of data points
c     used in the plot is user-controllable via basis.  the actual arrays
c     used in plotting will be allocated and freed within this routine.
c
c  Created:   Bert Still [chs], 950503
c  Revision history:
c     Modified srfplot.m to work for isosurface routine instead, 950503 [chs]
c     Made xlab,ylab local variables, so isoxtle,isoytle will not be
c          modified by isol2u, 970430 (FNF)
c     Modified axis labelling to be more reasonable.  The labels are fixed
c          and rotate with the axes as view changes.  [FNF 971028]
c
c  Known problems:
c   o The action of view(2) does not seem to be quite as advertised.
c     [FNF 970430]
c   o The axis label positioning isn't quite right for certain values
c     of view(2).  [FNF 971028]
c
c=======================================================================

      subroutine isoplot(t,in_x,in_y,in_z,c0,view)

Use(Isosurface_Plot)
Use(Iso_Internal_Parms)

c Arguments
      real(Size4)    t(in_x,in_y,in_z)
      INTEGER        in_x, in_y, in_z
      real(Size4)    c0, view(2)
      
c Local variables
      real(Size4)    viewscale
      INTEGER        nx, ny, nz, utgetcl, ichht
      INTEGER        ixflip, iyflip
      INTEGER        ixswap, iyswap
      INTEGER        muvwp2
      CHARACTER*80   xlab, ylab
      INTEGER        xlablen, ylablen
      real(Size4)    sight(3)
      integer        i

c  initialize sight to zeros.
      do i=1,3
         sight(i) = 0.0_Size4
      enddo

c  put data into plotting arrays, and scale it to prevent pancake plots
      nx = min(in_x, isonpx)
      ny = min(in_y, isonpy)
      nz = min(in_z, isonpz)
      call allot("isotplot", nx*nx*nz)
      call isoload(t, in_x, in_y, in_z, isotplot, nx, ny, nz)

c  set up viewport information
      call isovpt(view, ixflip, iyflip, ixswap, iyswap)
c    NOTE: This version ignores the values of ixflip,iyflip,ixswap,iyswap.

c  other initialization
      viewscale = isoview * sqrt(25*nx*nx + 12.25*ny*ny + 4*nz*nz)
      sight(1) = viewscale * cos(view(2)) * cos(view(1))
      sight(2) = viewscale * cos(view(2)) * sin(view(1))
      sight(3) = viewscale * (sin(view(2)) + 1.0_Size4)

c  allocate the work array
      muvwp2 = max(nx,ny,nz)+2
      call allot("isoscr", muvwp2*muvwp2)

c  check to see that a graphics device exists and fire up a new frame
      call pltstart
      call ezcnf

c  plot the title: center string, set height (30pt), and write text
c      using:  AGPWRT(x, y, msg, len, ichsize, angle, center)
      call gselnt(0)
      ichht = min0(800 / utgetcl(isotitle), 30)
      call agpwrt(0.5_Size4, 0.95_Size4, isotitle, utgetcl(isotitle), ichht, 0, 0)

c  get true label lengths
      xlablen = utgetcl(isoxtle)
      ylablen = utgetcl(isoytle)

c  do the plot: ISOSRF(T, LU, MU, LV, MV, MW, EYE, MP2, ISOSCR, C0, ISOFLG)
      call isosrf(isotplot, nx, nx, ny, ny, nz, sight, muvwp2, isoscr, c0, isoflg)

c  convert lowercase to uppercase in labels because PWRZI can't
c       process lowercase
      xlab = isoxtle
      ylab = isoytle
      call isol2u(xlab)
      call isol2u(ylab)

c  label the axes
      if (isolabel == 1) then
         ichht = min0(630 / max0(xlablen,ylablen), 30)
         call pwrzi(1.0_Size4, -0.5_Size4, 0.0_Size4, xlab, xlablen, ichht, 
                    -1, -2, 1)
         call pwrzi(-0.5_Size4, 1.0_Size4, 0.0_Size4, ylab, ylablen, ichht,
                    2, -1, -1)
      endif

      call sflush       #????
c  call the external notification routine
      if (isoxn == 1) then
         call ezcsetxn
      endif

c  cleanup and exit
      call basfree("isoscr")
      call basfree("isotplot")

      return
 101  format(A, G10.3, A, G10.3)
      end

