c=======================================================================
c
c  isovpt: viewport calculations
c
c  Notes:
c   o this routine will determine whether X- and Y-axis labels need
c       to swap sides or be flipped (for readability) and will move
c       extreme viewports back into the usual range [-180,180]
c
c  Created:   Bert Still [chs], 930519
c  Revision history:
c     Modified srfvpt() for use with isosurfaces, 950503 [chs]
c     Corrected viewpoint computation to allow full 360-degee rotation.
c         Also corrected internal comments. (970429 FNF)
c     Revised viewpoint to be consistent with old version. (971028 FNF)
c
c=======================================================================
      subroutine isovpt(view, ixflip, iyflip, ixswap, iyswap)

Use(Isosurface_Plot)
Use(Iso_Internal_Parms)

      real(Size4)    PIBY180
      PARAMETER     (PIBY180=1.74532925199e-2_Size4)
      INTEGER        ixflip, iyflip, ixswap, iyswap
      real(Size4)    view(2)

C  Move viewpoint to standard position and normalize angles.
C
      view(1) = mod(view(1), 360._Size4)
#       Negative numbers give values in (-360,0).
      if (view(1) < 0.) view(1) = view(1) + 360._Size4
      view(2) = mod(view(2), 360._Size4)
#       Negative numbers give values in (-360,0).
      if (view(2) < 0.) view(2) = view(2) + 360._Size4
C  At this point, both angles are in range [0,360].

C  Figure out label orientation based on viewpoint.
      if (view(1) == 0.0_Size4) then
         ixflip = 1
         iyflip = -1
         ixswap = -1
         iyswap = -1
      else if (view(1) > 0.0_Size4  &  view(1) < 90._Size4) then
         ixflip = 1
         iyflip = -1
         ixswap = 1
         iyswap = -1
      else if (view(1) == 90._Size4) then
         ixflip = 1
         iyflip = 1
         ixswap = 1
         iyswap = -1
      else if (view(1) > 90._Size4  &  view(1) <= 180._Size4) then
         ixflip = 1
         iyflip = 1
         ixswap = 1
         iyswap = 1
      else if (view(1) > 180._Size4  &  view(1) < 270._Size4) then
         ixflip = -1
         iyflip = 1
         ixswap = -1
         iyswap = 1
      else if (view(1) >= 270._Size4  &  view(1) < 360._Size4) then
         ixflip = -1
         iyflip = -1
         ixswap = -1
         iyswap = -1
      endif

C  Map angles to standard range [-180,180].  (Is this necessary?)
      if (view(1) > 180._Size4)  view(1) = view(1) - 360._Size4
      if (view(2) > 180._Size4)  view(2) = view(2) - 360._Size4

C  Convert angles to radians before return.
      view(1) = view(1) * PIBY180
      view(2) = view(2) * PIBY180

      return
      end
