c=======================================================================
c
c  srfscal: data scaling
c
c  notes:
c   o arrays x(nx), y(ny), and z(nx,ny) are scaled to lie in
c       the unit interval [-1,1] or box [-1,1]X[-1,1].
c
c  created:   bert still [chs], 930519
c  revision history:
c     Extracted from showsrf() to simplify that routine, 930519 [chs]
c     Changed "call remark" statements to "call ezcremrk", 970813 [FNF]
c
c=======================================================================
      integer function srfscal(x, y, z, nx, ny, low, high)

Use(Surface_Plot)
Use(Srf_Internal_Parms)

      INTEGER        nx, ny, i, j
      real(Size4)    x(nx), y(ny), z(nx,ny)
      real(Size4)    xhi, xlo, yhi, ylo, zhi, zlo
      real(Size4)    xspread, yspread, zspread
      real(Size4)    low(3), high(3)

c  find the range of the data
      if (srfautoscal == 1) then
         xhi = x(1)
         xlo = x(1)
         zhi = z(1,1)
         zlo = z(1,1)
         do i=2, nx
            xhi = amax1(x(i), xhi)
            xlo = amin1(x(i), xlo)
            do j=1, ny
               zhi = amax1(z(i,j), zhi)
               zlo = amin1(z(i,j), zlo)
            enddo
         enddo
         yhi = y(1)
         ylo = y(1)
         do j=2, ny
            yhi = amax1(y(j), yhi)
            ylo = amin1(y(j), ylo)
         enddo
      else
         xhi = srfxhi
         xlo = srfxlo
         yhi = srfyhi
         ylo = srfylo
         zhi = srfzhi
         zlo = srfzlo
      endif

      xspread = xhi - xlo
      if (xspread < srfspmin) then
         call ezcremrk("srfplot: cannot plot constant x data.")
         return(ERR)
      endif

      yspread = yhi - ylo
      if (yspread < srfspmin) then
         call ezcremrk("srfplot: cannot plot constant y data.")
         return(ERR)
      endif

      zspread = zhi - zlo
      if (zspread < srfspmin) then
         call ezcremrk("srfplot: cannot plot constant z data.")
         return(ERR)
      endif

c  scale the x-, y-, and z-data to [-1,1]  (prevents pancake plots)
      do i=1, nx
         x(i) = 2.0_Size4 * (x(i) - xlo) / xspread - 1.0_Size4
         do j=1, ny
            z(i,j) = 2.0_Size4 * (z(i,j) - zlo) / zspread - 1.0_Size4
         enddo
      enddo
      do j=1, ny
         y(j) = 2.0_Size4 * (y(j) - ylo) / yspread - 1.0_Size4
      enddo

c  return the range of the data
      low(1) = xlo
      low(2) = ylo
      low(3) = zlo
      high(1) = xhi
      high(2) = yhi
      high(3) = zhi

      return(OK)
      end

