/*
 * fft.c
 */

#include "fft.h"

#ifdef HAS_DOUBLE_COMPLEX
static Integer complex = COMPLEX;
#endif

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

Integer Fftbfcn(Integer nargs, char *fname, Integer *sx)
{
    float *x,
        *y,
        *z,
        *xc,
        *zc;
    real8 *xd,
        *yd,
        *zd;
    Integer i,
        idim1,
        dim,
        dimlen,
        npoints,
        istride,
        izindx,
        offset,
        dimsize,
        j,
        sy[SS_WIDTH],
        sz[SS_WIDTH];
    static char
        msg1[] = "fft or ffti: transform which direction?",
        msg2[] = "fft or ffti: arg2 is out of range.",
        msg3[] = "fft or ffti: illegal argument type";

    if (!strcmp(fname, "fft") || !strcmp(fname, "ffti")) {
        Fcnargb(1, sx);         /* return in argument's storage */
        npoints = Parlen(sx);
        x = (float *) Paraddr(sx);
        xd = (real8 *) x;
        xc = (float *) x;
        idim1 = 0;
        for (i = 0; i < sx[SS_N - 1]; i++)
            if (sx[SS_N1 + i - 1] == 1)
                idim1++;
        if (nargs == 1) {
            dim = 1;
            if (sx[SS_N - 1] - idim1 > 1) {
                BasError(msg1, strlen(msg1));
                return ERR;
            }
            dimlen = npoints;
        } else if (nargs == 2) {
            Fcnargb(2, sy);
            if (Parint(sy, &dim) == ERR)
                return (ERR);
            Parrel(sy);         /* release space for argument */
            if (dim > sx[SS_N - 1] || dim < 1) {
                BasError(msg2, strlen(msg2));
                return ERR;
            }
            istride = 1;
            for (i = 0; i <= dim - 2; i++)
                istride *= sx[SS_N1 + i - 1];
            dimlen = sx[SS_N1 + dim - 2];       /* length of transform direction */
            dimsize = istride * dimlen;
        }
        sx[SS_M1 + dim - 2] = 0;        /* starting subscript */
        sy[SS_TC - 1] = REAL;
        sy[SS_N - 1] = 1;       /* one dimensional scratch */
        sy[SS_M1 - 1] = 1;      /* starting subscript */
        if (sx[SS_TC - 1] == REAL) {
            sy[SS_N1 - 1] = 2 * dimlen + 15;
            Parget(sy);
            y = (float *) Paraddr(sy);
            Rffti(&dimlen, y);
        } else if (sx[SS_TC - 1] == DOUBLE) {
            sy[SS_N1 - 1] = 2 * dimlen + 15;
            sy[SS_TC - 1] = DOUBLE;
            Parget(sy);
            yd = (real8 *) Paraddr(sy);
            Dffti(&dimlen, yd);
        } else if (sx[SS_TC - 1] == COMPLEX || sx[SS_TC - 1] == DOUBLE_C) {
#      ifdef HAS_DOUBLE_COMPLEX
            if (sx[SS_TC - 1] == DOUBLE_C)      /* coerce to complex, anyhow */
                if (Parcoerc(sx, &complex) == ERR) {
                    BasError(msg3, strlen(msg3));
                    return ERR;
                }
#      endif
            sy[SS_N1 - 1] = 4 * dimlen + 15;
            Parget(sy);
            y = (float *) Paraddr(sy);
            Cffti(&dimlen, y);
        } else {
            BasError(msg3, strlen(msg3));
            return ERR;
        }
        if (sx[SS_N - 1] - idim1 < 2) { /* scalar or 1 dimensional */
            if (sx[SS_TC - 1] == REAL) {
                if (!strcmp(fname, "fft"))
                    Rfftf(&npoints, x, y);
                else
                    Rfftb(&npoints, x, y);
            } else if (sx[SS_TC - 1] == DOUBLE) {
                if (!strcmp(fname, "fft"))
                    Dfftf(&npoints, xd, yd);
                else
                    Dfftb(&npoints, xd, yd);
            } else {            /* COMPLEX */
                if (!strcmp(fname, "fft"))
                    Cfftf(&npoints, xc, y);
                else
                    Cfftb(&npoints, xc, y);
            }
        } else {                /* data dimensionality > 1. */
            /* SLATEC fft's don't take strides, so... */
            sz[SS_N - 1] = 1;   /* one dimensional scratch */
            sz[SS_TC - 1] = sx[SS_TC - 1];      /* same type as data */
            sz[SS_M1 - 1] = 1;
            sz[SS_N1 - 1] = dimlen;
            Parget(sz);
            if (sx[SS_TC - 1] == REAL) {
                z = Paraddr(sz);
                for (offset = 0; offset <= npoints - 1; offset += dimsize)
                    for (i = 0; i < istride; i++) {
                        izindx = i + offset;    /* copy it out */
                        for (j = 0; j < dimlen; j++) {
                            z[j] = x[izindx];
                            izindx += istride;
                        }
                        if (!strcmp(fname, "fft"))
                            Rfftf(&dimlen, z, y);
                        else
                            Rfftb(&dimlen, z, y);
                        izindx = i + offset;    /* copy it back */
                        for (j = 0; j < dimlen; j++) {
                            x[izindx] = z[j];
                            izindx += istride;
                        }
                    }
            } else if (sx[SS_TC - 1] == DOUBLE) {
                zd = Paraddr(sz);
                for (offset = 0; offset <= npoints - 1; offset += dimsize)
                    for (i = 0; i < istride; i++) {
                        izindx = i + offset;    /* copy it out */
                        for (j = 0; j < dimlen; j++) {
                            zd[j] = xd[izindx];
                            izindx += istride;
                        }
                        if (!strcmp(fname, "fft"))
                            Dfftf(&dimlen, zd, yd);
                        else
                            Dfftb(&dimlen, zd, yd);
                        izindx = i + offset;    /* copy it back */
                        for (j = 0; j < dimlen; j++) {
                            xd[izindx] = zd[j];
                            izindx += istride;
                        }
                    }
            } else {            /* COMPLEX */
                zc = Paraddr(sz);
                for (offset = 0; offset <= npoints - 1; offset += dimsize)
                    for (i = 0; i < istride; i++) {
                        izindx = i + offset;    /* copy it out */
                        for (j = 0; j < dimlen; j++) {
                            zc[j] = xc[izindx];
                            izindx += istride;
                        }
                        if (!strcmp(fname, "fft"))
                            Cfftf(&dimlen, zc, y);
                        else
                            Cfftb(&dimlen, zc, y);
                        izindx = i + offset;    /* copy it back */
                        for (j = 0; j < dimlen; j++) {
                            xc[izindx] = zc[j];
                            izindx += istride;
                        }
                    }
            }
            Parrel(sz);         /* release scratch */
        }
        Parrel(sy);             /* release scratch */
    }
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
