#include "fit.h"
#include <stdio.h>

#if (WORD_BIT == 64)
static Integer t8 = REAL;
#else
static Integer t8 = DOUBLE;
#endif

static Integer one = 1;

static Integer fitn;
static real8
   *fitc = (real8 *) NOTSET,            /* fitc(0:fitn) _real */
   *fitworka = (real8 *) NOTSET,        /* fitworka(fitnx,0:fitn) _real */
   *fitworkb = (real8 *) NOTSET,        /* fitworkb(fitworkl) _real */
   *fitwork = (real8 *) NOTSET;         /* fitwork(fitlwork) _real */
static Integer ratio = sizeof(real8) / sizeof(Integer);

extern void Osfreen(Address *ptr, char *name);
extern void Osallotn(Address *ptr, Integer n, char *name);

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* FITALLOT - allocate the space for the above variables */

static Integer Fitallot(void)
{

    if (fitc != (real8 *) NOTSET)
        Osfreen((Address *) (&fitc), "FITALLOT:fitc");

    if (fitworka != (real8 *) NOTSET)
        Osfreen((Address *) (&fitworka), "FITALLOT:fitworka");

    if (fitworkb != (real8 *) NOTSET)
        Osfreen((Address *) (&fitworkb), "FITALLOT:fitworkb");

    if (fitwork != (real8 *) NOTSET)
        Osfreen((Address *) (&fitwork), "FITALLOT:fitwork");

    Osallotn((Address *) (&fitc), (fitn + 1) * ratio, "FITALLOT:fitc");
    Osallotn((Address *) (&fitworka), (fitn + 1) * fitnx * ratio,
             "FITALLOT:fitworka");
    Osallotn((Address *) (&fitworkb), fitworkl * ratio,
             "FITALLOT:fitworkb");
    Osallotn((Address *) (&fitwork), fitlwork * ratio, "FITALLOT:fitwork");

    if (fitc == (real8 *) NULL || fitworka == (real8 *) NULL ||
        fitworkb == (real8 *) NULL || fitwork == (real8 *) NULL)
        return ERR;
    else
        return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* FITFIN - free the space for these variables */

Integer Fitfin(Integer *dummy1, Integer *dummy2)
{

    if (fitc != (real8 *) NOTSET) {
        Osfreen((Address *) (&fitc), "FITFREE:fitc");
        fitc = (real8 *) NOTSET;
    }

    if (fitworka != (real8 *) NOTSET) {
        Osfreen((Address *) (&fitworka), "FITFREE:fitworka");
        fitworka = (real8 *) NOTSET;
    }

    if (fitworkb != (real8 *) NOTSET) {
        Osfreen((Address *) (&fitworkb), "FITFREE:fitworkb");
        fitworkb = (real8 *) NOTSET;
    }

    if (fitwork != (real8 *) NOTSET) {
        Osfreen((Address *) (&fitwork), "FITFREE:fitwork");
        fitwork = (real8 *) NOTSET;
    }

    return 0;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

static Integer Fittempfree(void)
{
    Osfreen((Address *) (&fitworka), "FITALLOT:fitworka");
    Osfreen((Address *) (&fitworkb), "FITALLOT:fitworkb");
    Osfreen((Address *) (&fitwork), "FITALLOT:fitwork");
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

static real8 Fitvalue(real8 x, real8 *c, Integer ndegree)
{
    int i;
    real8 result;

    result = 0.;
    for (i = ndegree; i >= 0; i--)
        result = result * x + c[i];
    return result;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

static void Fitfilla(Integer n1, Integer n2, real8 *a, real8 *b,
                     real8 *x, real8 *y)
{
    int i, j;

    for (i = 0; i < n1; i++) {
        a[i] = 1.;
        b[i] = y[i];
        for (j = n1; j <= n1 * n2; j += n1)
            a[i + j] = a[i + j - n1] * x[i];
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

Integer Fitbfcn(Integer nargs, char *fname, Integer *sx)
{
    Integer i, j, info, nx, nc, sy[SS_WIDTH], sz[SS_WIDTH], sw[SS_WIDTH];
    static char
        msg1[] = "1st argument to fit is wrong type.",
        msg2[] = "2nd argument to fit is wrong type.",
        msg3[] = "3rd argument to fit is wrong type.",
        msg4[] = "Impossible error in fit.",
        msg5[] = "y and x not the same length in fit.",
        msg6[] = "Failure of sgels or dgels called by fit.",
        msg7[] = "fitvalue: 1st argument of bad type.",
        msg8[] = "fitvalue: 2nd argument of bad type.",
        msg9[] = "fitvalue: null second argument; no call to fit?",
        msg10[] = "fitc: subscript must be integer or range.",
        msg11[] = "fitc: integer subscript must be scalar or 1d array.",
        msg12[] = "fit: Problem allocating space.",
        msg13[] = "fit: Problem freeing space.",
        msg14[] = "fitc: subscript is out of range.",
        msg15[] = "fitc: dynamic variable with unset pointer.",
        msg16[] = "fitc: range stride is 0??";
    real8 *x, *y, *c;
    Integer *n, fitn1, *subscripts, sub1, sub2;

    sx[SS_N - 1] = 0;
    sx[SS_TC - 1] = 0;
    if (!strcmp(fname, "fit")) {
        Fcnargb(1, sw);
        Fcnargb(2, sy);
        Fcnargb(3, sz);
        if (sw[SS_TC - 1] < INTEGER || sw[SS_TC - 1] > DOUBLE) {
            BasError(msg1, strlen(msg1));
            return ERR;
        }
        if (sy[SS_TC - 1] < INTEGER || sy[SS_TC - 1] > DOUBLE) {
            BasError(msg2, strlen(msg2));
            return ERR;
        }
        if (sz[SS_TC - 1] != INTEGER) {
            BasError(msg3, strlen(msg3));
            return ERR;
        }
        if (Parcoerc(sw, &t8) == ERR) {
            BasError(msg4, strlen(msg4));
            return ERR;
        }
        if (Parcoerc(sy, &t8) == ERR) {
            BasError(msg4, strlen(msg4));
            return ERR;
        }
        x = (real8 *) Paraddr(sw);
        y = (real8 *) Paraddr(sy);
        n = (Integer *) Paraddr(sz);
        fitn = n[0];
        fitnx = Parlen(sw);
        if (fitnx != Parlen(sy)) {
            BasError(msg5, strlen(msg5));
            return ERR;
        }
        fitworkl = max(fitnx, fitn + 1);
        fitlwork = 10 * fitworkl;
        if (Fitallot() == ERR) {
            BasError(msg12, strlen(msg12));
            return ERR;
        }
        Fitfilla(fitnx, fitn, fitworka, fitworkb, x, y);
        Parkill(sw);
        Parkill(sy);
        Parkill(sz);
        fitn1 = fitn + 1;
#   if (WORD_BIT == 64)
        Sgels("N", &fitnx, &fitn1, &one, fitworka, &fitnx, fitworkb,
              &fitworkl, fitwork, &fitlwork, &info, 1);
#   else
        Dgels("N", &fitnx, &fitn1, &one, fitworka, &fitnx, fitworkb,
              &fitworkl, fitwork, &fitlwork, &info, 1);
#   endif
        fitlwork = (Integer) (fitwork[0]);
        if (info == 0)
            for (i = 0; i <= fitn; i++)
                fitc[i] = fitworkb[i];
        if (!fitsave)
            if (Fittempfree() == ERR) {
                BasError(msg13, strlen(msg13));
                return ERR;
            }
        if (info != 0) {
            BasError(msg6, strlen(msg6));
            return ERR;
        }
        /* return fitc as the result */
        sx[SS_N - 1] = 1;
        sx[SS_N1 - 1] = fitn + 1;
        sx[SS_M1 - 1] = 0;
        sx[SS_I1 - 1] = 1;
        sx[SS_TC - 1] = t8;
        Parget(sx);
        x = (real8 *) Paraddr(sx);
        for (i = 0; i <= fitn; i++)
            x[i] = fitc[i];
    } else if (!strcmp(fname, "fitvalue")) {
        Fcnargb(1, sw);
        if (Parcoerc(sw, &t8) == ERR) {
            BasError(msg7, strlen(msg7));
            return ERR;
        }
        x = (real8 *) Paraddr(sw);
        nx = Parlen(sw);
        if (nargs == 1) {
            nc = fitn;
            c = fitc;
        } else {
            Fcnargb(2, sz);
            if (Parcoerc(sz, &t8) == ERR) {
                BasError(msg8, strlen(msg8));
                return ERR;
            }
            nc = Parlen(sz) - 1;
            c = (real8 *) Paraddr(sz);
        }
        if (nc < 0) {
            BasError(msg9, strlen(msg9));
            return ERR;
        }
        Parcopy(sw, sx);
        Parget(sx);
        y = (real8 *) Paraddr(sx);
        for (i = 0; i < nx; i++)
            y[i] = Fitvalue(x[i], c, nc);
        Parkill(sw);
        if (nargs == 2)
            Parkill(sz);
    } else if (!strcmp(fname, "fitc")) {
        if (fitc == (real8 *) NOTSET) {
            BasError(msg15, strlen(msg15));
            return ERR;
        }
        if (nargs == 0) {
            /* return fitc as the result */
            sx[SS_N - 1] = 1;
            sx[SS_N1 - 1] = fitn + 1;
            sx[SS_M1 - 1] = 0;
            sx[SS_I1 - 1] = 1;
            sx[SS_TC - 1] = t8;
            Parget(sx);
            x = (real8 *) Paraddr(sx);
            for (i = 0; i <= fitn; i++)
                x[i] = fitc[i];
        } else {
            Fcnargb(1, sw);
            if (sw[SS_TC - 1] == RANGE) {
                subscripts = (Integer *) Paraddr(sw);
                if (subscripts[0] == DEFAULT)
                    subscripts[0] = 0;
                else if (subscripts[0] < 0 || subscripts[0] > fitn) {
                    BasError(msg14, strlen(msg14));
                    return ERR;
                }
                if (subscripts[1] == DEFAULT)
                    subscripts[1] = fitn;
                else if (subscripts[1] < 0 || subscripts[1] > fitn) {
                    BasError(msg14, strlen(msg14));
                    return ERR;
                }
                if (subscripts[2] == DEFAULT)
                    subscripts[2] = subscripts[0] < subscripts[1] ? 1 : -1;
                else if (subscripts[2] == 0) {
                    BasError(msg16, strlen(msg16));
                    return ERR;
                }
                sx[SS_N - 1] = 1;
                sx[SS_TC - 1] = t8;
                sx[SS_N1 - 1] =
                    (subscripts[1] - subscripts[0]) / subscripts[2] + 1;
                sx[SS_M1 - 1] = 1;
                sx[SS_I1 - 1] = 1;
                Parget(sx);
                x = (real8 *) Paraddr(sx);
                if (subscripts[2] > 0)
                    for (j = 0, i = subscripts[0]; i <= subscripts[1];
                         j++, i += subscripts[2])
                        x[j] = fitc[i];
                else
                    for (j = 0, i = subscripts[0]; i >= subscripts[1];
                         j++, i += subscripts[2])
                        x[j] = fitc[i];
            } else if (sw[SS_TC - 1] == INTEGER) {
                if (sw[SS_N - 1] > 1) {
                    BasError(msg11, strlen(msg11));
                    return ERR;
                }
                subscripts = (Integer *) Paraddr(sw);
                if (sw[SS_N - 1] == 0) {        /* scalar subscript */
                    if (subscripts[0] < 0 || subscripts[0] > fitn) {
                        BasError(msg14, strlen(msg14));
                        return ERR;
                    }
                    sx[SS_N - 1] = 0;
                    sx[SS_TC - 1] = t8;
                    Parget(sx);
                    x = (real8 *) Paraddr(sx);
                    x[0] = fitc[subscripts[0] - 1];
                } else {        /* array subscript */
                    sub1 = Ivecmin(subscripts, 1, sw[SS_N1 - 1]);
                    sub2 = Ivecmax(subscripts, 1, sw[SS_N1 - 1]);
                    if (sub1 < 0 || sub2 > fitn) {
                        BasError(msg14, strlen(msg14));
                        return ERR;
                    }
                    sx[SS_N - 1] = 1;
                    sx[SS_TC - 1] = t8;
                    sx[SS_N1 - 1] = sw[SS_N1 - 1];
                    sx[SS_M1 - 1] = 1;
                    sx[SS_I1 - 1] = 1;
                    Parget(sx);
                    x = (real8 *) Paraddr(sx);
                    for (i = 0; i < sw[SS_N1 - 1]; i++)
                        x[i] = fitc[subscripts[i]];
                }
            } else {
                /* Has to be INTEGER or RANGE */
                BasError(msg10, strlen(msg10));
                return ERR;
            }
            Parkill(sw);
        }
    } else if (!strcmp(fname, "fitn")) {
        sx[SS_N - 1] = 0;
        sx[SS_TC - 1] = INTEGER;
        sx[SS_PTR - 1] = Partempc(&fitn);
    }
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
