/*
 * hstprint.c
 */

#include "hst.h"

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* HSTPRINT - print out a summary of the status of the tag */

void Hstprint(char *tag, Integer iunit)
{
    Integer unit, itag, icond, mode, ifind, item, lname, icount, lcat;
    Integer nc, nf, nm, it;
    int nc1;
    char v, thisitem[MAXITEMSIZE + 1], thisname[VARNAMESIZE + 1];
    char msg[FILENAMESIZE + 1], ff[FILENAMESIZE + 1];
    char *pt;
    static char
        msg1[] = "Histories being kept in memory.",
        msg2[] = "Tag is inactive, will not be collected.",
        msg3[] = "Tag data structure is damaged, bad mode.",
        msg4[] = "Before collecting this tag, will execute: ",
        msg5[] = "*Storage Name             Item",
        msg6[] = "-------------------------------------------------";

    nc = VARNAMESIZE;
    nf = FILENAMESIZE;
    nm = MAXITEMSIZE;

    if (Gmemtr() != 0) {
        Remark("Check literals: start hstprint.", 31);
        Baslt10();
    }

    unit = iunit;
    if (unit == DEFAULT)
        unit = STDERR;

    itag = Hsttnum(tag, strlen(tag));
    if (itag == 0) {
        sprintf(msg, "hstprint: No such tag, %s.", tag);
        Baswline(&unit, msg, strlen(msg));
        return;
    }

    it = itag - 1;

    Baswline(&unit, " ", 1);

    lname = StrGetCl((Address) tagname, it * nc, nc);
    strcpy(msg, "Tag ");
    strncat(msg, tagname + it * nc, lname);

    strcat(msg, " status is:");
    Baswline(&unit, msg, strlen(msg));

    strncpy(ff, tagfile + it * nf, nf);
    ff[nf] = '\0';
    ff[strcspn(ff, " ")] = '\0';
    if (ff[0] == '\0') {
        Baswline(&unit, msg1, strlen(msg1));
    } else {
        sprintf(msg,
                "Histories being written to a sequence file with prefix %s.",
                ff);
        Baswline(&unit, msg, strlen(msg));
    }

    /* watch ot for the accumulating lengths of string catenations into msg */

    strcpy(msg, "Cycle counter is: ");
    lcat = StrGetCl((Address) tagccn, it * nc, nc);
    strncat(msg, (char *) tagccn + it * nc, lcat);
    strcat(msg, " Independent variable is: ");
    lcat = StrGetCl((Address) tagivn, it * nc, nc);
    strncat(msg, (char *) tagivn + it * nc, lcat);
    if (tagdoub[it]) {
        strcat(msg, "(double precision variable)");
    }
    Baswline(&unit, msg, strlen(msg));

    if (tagrec[it] == 0) {
        strcpy(msg, "This tag has never been collected.");
    } else {
        sprintf(msg, "Generation %ld last collected at cycle %ld.",
                (long) tagrec[it], (long) tagclc[it]);
    }
    Baswline(&unit, msg, strlen(msg));

    if (tagrecrn[it] == 0) {
        strcpy(msg, "This tag has not been collected this session.");
    } else {
        sprintf(msg,
                "Current Session Generation %ld last collected at cycle %ld.",
                (long) tagrecrn[it], (long) tagclc[it]);
    }
    Baswline(&unit, msg, strlen(msg));

    if (strncmp((char *) taglogic + it * 80, "true", 4)) {
        strcpy(msg, "Tag logical condition is: ");
        strncat(msg, (char *) taglogic + it * 80, 80);
        Baswline(&unit, msg, strlen(msg));
    }

    if (tagcond[it] == 0) {
        Baswline(&unit, msg2, strlen(msg2));
        icond = 0;
    } else {
        icond = tagcond[it];
    }

    icount = 0;
    while (icond != 0) {
        mode = hstmode[icond - 1];
        icount += 1;
        if (mode == CYCLESPEC) {
            sprintf(msg,
                    "Cycle interval condition %ld: pending cycle = %ld.",
                    (long) icount, (long) hstcyc[icond - 1]);
        } else if (mode == CYCLELIST) {
            sprintf(msg, "Cycle list condition %ld: pending cycle = %ld.",
                    (long) icount, (long) hstcyc[icond - 1]);
        } else if (mode == VALUESPEC) {
            sprintf(msg,
                    "Time interval condition %ld: pending time = %14.6e.",
                    (long) icount, hstvalue[icond - 1]);
        } else if (mode == VALUELIST) {
            sprintf(msg, "Time list condition %ld: pending time = %14.6e.",
                    (long) icount, hstvalue[icond - 1]);
        } else {
            strcpy(msg, msg3);
        }

        Baswline(&unit, msg, strlen(msg));
        strcpy(msg, "Condition values are stored in the variable named ");
        lcat = StrGetCl((Address) hstcond, (icond - 1) * nc, nc);
        strncat(msg, (char *) hstcond + (icond - 1) * nc, lcat);
        Baswline(&unit, msg, strlen(msg));
        icond = hstnext[icond - 1];
    }

    pt = ((char *) tagact) + it * 500;

    if (*pt != ' ') {
        strcpy(msg, msg4);
        strncat(msg, pt, nf - strlen(msg4));
        Baswline(&unit, msg, strlen(msg));
    }

    ifind = 0;
    for (item = 0; item < numitems; item++) {
        if (hstown[item] != itag)
            continue;
        ifind += 1;
    }

    if (ifind > 0) {
        Baswline(&unit, msg5, strlen(msg5));
        Baswline(&unit, msg6, strlen(msg6));
        for (item = 0; item < numitems; item++) {
            if (hstown[item] != itag)
                continue;

            strncpy(thisitem, (char *) hstitms + item * nm, nm);
            thisitem[nm] = '\0';
            thisitem[strcspn(thisitem, " ")] = '\0';

            v = (hstvol[item] > 0) ? '*' : ' ';

/* mark as static file variable */
            if (ff[0] != '\0' && hststatc[item] == 1)
                v = '+';

            strncpy(thisname, (char *) hstname + item * nc, nc);

            /* remove trailing blanks, but leave at least 24 characters */
            for (nc1 = nc - 1; nc1 >= 24; nc1--) {
                if (thisname[nc1] != ' ') {
                    break;
                }
            }
            thisname[nc1 + 1] = '\0';

            sprintf(msg, "%c%s %s", v, thisname, thisitem);
            Baswline(&unit, msg, strlen(msg));
        }
    }

    if (Gmemtr() != 0) {
        Remark("Check literals: exit hstprint.", 30);
        Baslt10();
    }

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
