#include <pdb.h>
#include <Basis.h>

#define MAXLINE1 256

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* _PD_WRITE_AUX - write a variable out via PD_write_as_alt but map the
 *               - indexing information to the correct type first
 */

static FIXNUM _PD_write_aux(PDBfile * file, char *name, char *typi,
                            char *typo, byte * space, FIXNUM nd,
                            FIXNUM *dims)
{
    long i, ne, *pi;
    FIXNUM ret;

    ne = 3 * nd;
    pi = FMAKE_N(long, ne, "_PD_WRITE_AUX:pi");

    for (i = 0; i < ne; i++)
        pi[i] = dims[i];

    ret = PD_write_as_alt(file, name, typi, typo, space, nd, pi);

    SFREE(pi);

    return (ret);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* _PD_READ_AUX - read a variable in via _PD_indexed_read_as but map the
 *              - indexing information to the correct type first
 *              - NOTE: this would be symmetric with _PD_WRITE_AUX if
 *              - the read calls required the number of dimensions as the
 *              - write calls do!!!!
 */

static FIXNUM _PD_read_aux(PDBfile *file, char *name, char *type, byte *vr,
                           FIXNUM *ind)
{
    char fullpath[MAXLINE1];
    dimdes *pd, *dims;
    syment *ep;
    long *pi;
    FIXNUM i, ne, nd, ret;

#ifdef _SC_T_INDEX
    _SC_T_INDEX(_t_index);
#else
#ifdef SC_THREAD_ID
    SC_THREAD_ID(_t_index);
#else
    int _t_index = 0;
#endif
#endif

    switch (setjmp(_PD_READ_ERR(_t_index))) {
    case ABORT:
        return (FALSE);
    case ERR_FREE:
        return (TRUE);
    default:
        memset(PD_ERR(_t_index), 0, MAXLINE1);
        break;
    };

/* look up the variable name and return FALSE if it is not there */
    ep = _PD_effective_ep(file, name, TRUE, fullpath);
    if (ep == NULL)
        PD_error("ENTRY NOT IN SYMBOL TABLE - _PD_READ_AUX", PD_READ);

    dims = PD_entry_dimensions(ep);
    for (nd = 0, pd = dims; pd != NULL; pd = pd->next, nd++) ;

    ne = 3 * nd;

    pi = FMAKE_N(long, ne, "_PD_READ_AUX:pi");

    for (i = 0; i < ne; i++)
        pi[i] = ind[i];

    ret = _PD_indexed_read_as(file, fullpath, type, vr, nd, pi, ep);

    SFREE(pi);

    return (ret);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* The following routines accept the Address string paradigm. */

FIXNUM MyPfWrtd(integer fileid, integer nch1, char *name, integer
                nc2, char *typ, byte * space, integer ndims, integer *dims)
{
    PDBfile *file;

    file = SC_GET_POINTER(PDBfile, fileid);
    return _PD_write_aux(file, name, typ, typ, space, ndims, dims);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

FIXNUM MyPfRead(integer fileid, integer nchar, char *name, byte * space)
{
    PDBfile *file;

    file = SC_GET_POINTER(PDBfile, fileid);
    if (!(PD_read(file, name, space))) {
        BasError(PD_err, strlen(PD_err));
        return FALSE;
    } else
        return TRUE;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

FIXNUM MyPfPtRd(integer fileid, integer nchar, char *name, byte * space,
                integer *ind)
{
    PDBfile *file;

    file = SC_GET_POINTER(PDBfile, fileid);

    return (_PD_read_aux(file, name, (char *) NULL, space, ind));
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
