c
c EPDEV.M - device handlers
c

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EZCDODEV - main device routine
c          - Processes command:  devtype arg1 arg2
c          -   devtype is the device type. Valid values are:
c          -      [cgm, ps, win]
c          -   arg1 is the device action command.  Valid values are:
c          -      [on, off, close, send, colormap]
c          -      Defaults to "on" of omitted.
c          -   arg2 contains device attributes. 
c          -      win has a string for window name
c          -      cgm, ps has color or mono
c          -      colormap has idl1, idl2, .., idl16, mycolormap

      subroutine ezcdodev(devtype, arg1, arg2)
      implicit none
      character*(*) devtype, arg1, arg2

Use(Devices)

      character*(8) command
      character*(120) msg
      integer errsvl

      integer ezcgetcl
      external ezcgetcl
      logical ezcnopl
      external ezcnopl

      if (.not. ezcnopl()) then
         command = "on"
         if (arg1 .ne. " ") command = arg1

         if (devtype .eq. "cgm") then
            call ezccgm(command, arg2)

         elseif (devtype .eq. "ps") then
            call ezcps(command, arg2)

         elseif (devtype .eq. "jpeg") then
            call ezcjpeg(command, arg2)

         elseif (devtype .eq. "mpeg") then
            call ezcmpeg(command, arg2)

         elseif (devtype .eq. "tv" | devtype .eq. "win") then  
            call ezcwin(command, arg2)

         else
            msg = devtype(1:ezcgetcl(devtype))//" graphic device not available"
            errsvl = 0
            call ezcerror(msg, errsvl)
         endif
      endif

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EZCP - device control

      subroutine ezcp(iflagin)
      integer iflagin

Use(Devices)

      integer iflag, ibufsz, ezcgetcl, i, ilimit, ezcxn, iquit, nc
      external ezcgetcl, ezcxn
      character*80 msg

      iflag = iflagin
      iquit = 0

c ... emergency shutdown
      if (iflag .eq. -1) then
         iflag = 0
         iquit = 1
      endif

c ... start up graphics file if we haven't already
      if (iflag .eq. 1) then
         if (codeplot .eq. NO) then
            codeplot = YES
            call ezcada 
            ibufsz = 4096
c            call gopks(STDERR, ibufsz) # ibufsz GKS-version dependent
         endif   

      elseif (codeplot .eq. YES) then
         if (iezcdev .eq. 0) then
            codeplot = NO
            return
         else

c ... get last frame done
            if (iquit.eq.0) then
               call ezcnf

c ... can't call nf for emergency shutdown, might get an error
            else
               call frame
            endif

            if (iezccgma .ge. 0) call ezccgm("close", " ")
            if (iezcpsa .ge. 0) call ezcps("close", " ")
            if (iezcjpega .ge. 0) call ezcjpeg("close", " ")
            if (iezcmpega .ge. 0) call ezcmpeg("close", " ")
            if (iwinop .gt. 0) then
               ilimit = iwinop
               do i = 1, ilimit 
                  call ezcwin("close", winame(1))
               enddo
            endif               

c            call gclks

            if (iezccgma .ge. 0) then           
	       nc = ezcgetcl(iezccgmf)
               call utsprint(1, 1, 512, msg,
     &                       25, "Closing CGM Log File %.*s",
     &                       nc, iezccgmf)
               call ezcremrk(msg)
               close(CGMLOGUNIT)
            endif

            if (iezcpsa .ge. 0) then
               nc = ezcgetcl(iezcpsf)
               call utsprint(1, 1, 512, msg,
     &                       24, "Closing PS Log File %.*s",
     &                       nc, iezcpsf)
               call ezcremrk(msg)
               close(PSLOGUNIT)
            endif

            if (iezcjpega .ge. 0) then
               nc = ezcgetcl(iezcjpegf)
               call utsprint(1, 1, 512, msg,
     &                       26, "Closing JPEG Log File %.*s",
     &                       nc, iezcjpegf)
               call ezcremrk(msg)
               close(JPEGLOGUNIT)
            endif

            if (iezcmpega .ge. 0) then
               nc = ezcgetcl(iezcmpegf)
               call utsprint(1, 1, 512, msg,
     &                       26, "Closing MPEG Log File %.*s",
     &                       nc, iezcmpegf)
               call ezcremrk(msg)
               close(MPEGLOGUNIT)
            endif

            codeplot = NO
         endif
      endif

      return
      end

c--------------------------------------------------------------------------

c                        SCREEN DEVICE ROUTINES

c--------------------------------------------------------------------------

c EZCWIN - routine to process command "win iflag istring".

      subroutine ezcwin(iflag, istring)
      character*(*) iflag, istring

Use(Devices)
Use(Colors)

      character*120 msg	#error message
      integer errsvl
      integer errind
      integer iws, iwstype		# workstation type 
      integer idx, j, i
      integer idxfd, iwsidfd
      integer nwt, nwm, nwl
      logical found
      character*32 display
      character*255 wtype, wmode

      real(Size8) x1, y1, x2, y2

c ... variables needed for colormap id query
      character*16 namestore	#Truncated version of name (as stored)

c ... variables for window labeling, etc.
      real(Size8) winht, winhtdf, winwd, winwddf
      character*34 winlabl	#Must be 18 longer than namestore
      character*32 winlbpx

      integer idwind(2)	#Replaces iconn for GXWIN (must be 64 bits).
      character*17 iconlabl	#Must be one longer than winame.

      integer ezcgetcl 
      external ezcgetcl

      integer pgmkdv, pgopen, pgclos, pgsbkc, pgspal
      external pgmkdv, pgopen, pgclos, pgsbkc, pgspal

      iwstype = 7             # window workstation type for NCAR (per J. Rowe)
      winlbpx = 'EZC Window: '
      display = ' '

      winhtdf = 7.0           # Default window height (inches)
      winwddf = 7.0           # Default window width (inches)

c       if (ezcdisp .ne. " ") then
c          display = ezcdisp
c          call ezcsdisp("DISPLAY="//ezcdisp)
c       else 
c          display = " "
c          call getenv("DISPLAY", display)
c          if (display .eq. " ") then
c             call ezcremrk("You must set environment variable DISPLAY")
c             call ezcremrk("or variable ezcdisp")
c             call kaboom(0)
c          endif
c       endif

      call ezcp(1)
      if (iwinop .eq. 0) then	# no open window exists
         found = FALSE
         if (iflag .ne. "on" & iflag .ne. "open" & iflag .ne. "colormap") then
            msg = "No open windows, ignoring command- "\
			//iflag(1:ezcgetcl(iflag))
            errsvl = 0
            call ezcerror(msg, errsvl)
            return
         endif

c ... open window exists
c ... try to find existing open window with the given name
      else
         found = FALSE	# initialize found
         namestore = istring	# Truncate to length of stored names.
         do idx = 1, iwinop
            if ((winame(idx) .eq. namestore) .or. \
                ((namestore.eq." ").and.(winame(idx).eq.ezcwinlb)))  then
               found = TRUE
               iwsidfd = iwsid(idx)	# workstation id found
               idxfd = idx		# index in the list of open windows
               break			# break the do loop
            endif
         enddo
      endif

c ... check the assciated command for the window
      if (iflag .eq. "on" | iflag .eq. "open") then

c ... open a new window
         if (.not. found) then

c ... increment iwinop only after success
            iwsidset = iwsidset + 1
	      
c ... assign the proper name to the window
            if (istring .ne. " ") then
               namestore = istring(1:ezcgetcl(istring))
            elseif (ezcwinlb .ne. " ") then
               namestore = ezcwinlb(1:ezcgetcl(ezcwinlb))
            else
               namestore = " "
            endif
            winame(iwinop+1) = namestore

            iwsid(iwinop+1) = iwsidset
            windisp(iwinop+1) = display	

c ... record the associating display with window

c ... set up for open workstation call.

c ... set up window label (with terminal null character)
            if (namestore .ne. " ") then
               winlabl = winlbpx(1:ezcgetcl(winlbpx))//" "\
                           //namestore(1:ezcgetcl(namestore)) // char(0)
	    else
               winlabl = "EZC PGS" // char(0)
            endif

c ... set up window size
            if (ezcwinht > 0) then
               winht = ezcwinht
            else
               winht = winhtdf
            endif
            if (ezcwinwd > 0) then
               winwd = ezcwinwd
            else
               winwd = winwddf
            endif

c c ... get the size parameters for the display:
c c ... wdinpx = width  in pixels; wdinmm = width  in millimeters;
c c ... htinpx = height in pixels; htinmm = height in millimeters.
c c ... first, add null character to display name for C.
c             dispnam = display(1:ezcgetcl(display))//char(0)
c             call gidpy (dispnam(1:ezcgetcl(dispnam)), 
c      &                  wdinpx, htinpx, wdinmm, htinmm, idum, istat)
c             if (istat > 0) then
c                write (msg, '(a, i5)') "ezcwin: GIDPY returned istat =", istat
c                call ezcerror(msg, 0)
c                return
c             endif
c 
c c ... convert window size specifications from inches to pixels.
c c ... (Note: 25.4 mm = 1 inch.)
c             iwinht = 25.4*(real(htinpx)/real(htinmm))*winht
c             iwinht = min(iwinht, htinpx)  # Make sure it fits.
c             iwinwd = 25.4*(real(wdinpx)/real(wdinmm))*winwd
c             iwinwd = min(iwinwd, wdinpx)  # Make sure it fits.

c ... open new window - PGS version
             if (winame(iwinop+1) .ne. " ") then

c ... append a null character for C
                iconlabl = winame(iwinop+1)(1:ezcgetcl(winame(iwinop+1))) \
                           // char(0)
             else
                iconlabl = "Xgks" // char(0)
             endif
             errind = 0

c ... note that the following sets idwind.
c ... open window with mixed color model.
             wtype = 'SCREEN'
             wmode = 'COLOR'
             nwt = ezcgetcl(wtype)
             nwm = ezcgetcl(wmode)
             nwl = ezcgetcl(winlabl)

             iws = pgmkdv(nwt, wtype, nwm, wmode, nwl, winlabl)

             errind = pgsbkc(iws, bakcol)

             x1 = 0.40
             y1 = 0.05
             x2 = 0.55
             y2 = 0.55

             errind = pgopen(iws, x1, y1, x2, y2)
             if (errind .ne. 1) then
                call ezcerror('Failed to open X11 window', 0)
                call utsprint(1, 1, 512, msg,
     &                        32, "PGOPEN returned error status: %d",
     &                        errind)
                return
             endif

             errind = pgspal(iws, 3, 'rgb')

             idwind(1) = iws
             idwind(2) = 0

             iconn  = idwind(1)
	     iwinop = iwinop + 1  # only increment after success 

c ... save connection id (needed for gdwin call).
             iwscn(1, iwinop) = idwind(1)
             iwscn(2, iwinop) = idwind(2)

	     winstate(iwinop) = TRUE

c ... deactivate all other windows in the same display
             if (iwinop > 1) then
      	        do i = 1, iwinop-1
		   if (windisp(i) .eq. windisp(iwinop) & winstate(i)) then
	              call ezcwoff(i)
		   endif
		enddo
	     endif

c c ... initialize color indices
c c ... Note: For NCAR-GKS prior to release 4.0 it may be necessary to add
c c ... coding to cause the following to be executed only once, to avoid
c c ... code crashes on opening second window.
c              if (winclrmap .eq. "mono") then
c                 call ezcmono(iwsid(iwinop), iwstype)
c              else
c                 call ezccidx(iwsid(iwinop), iwstype, winclrmap)
c              endif
c              if (debcolr .ne. 0) then
c 
c c ... check background color settings.
c                 call gqcr(iwsid(iwinop), 0, 0, errind, cr, cg, cb)
c                 write (msg, '(a, 3f6.3)') ' bgcolor requested was: ', cr, cg, cb
c                 call ezcremrk(msg)
c 
c                 call gqcr(iwsid(iwinop), 0, 1, errind, cr, cg, cb)
c                 write (msg, '(a, 3f6.3)') ' bgcolor actually used: ', cr, cg, cb
c                 call ezcremrk(msg)
c 
c c ... check foreground color settings.
c                 call gqcr(iwsid(iwinop), 1, 0, errind, cr, cg, cb)
c                 write (msg, '(a, 3f6.3)') ' fgcolor requested was: ', cr, cg, cb
c                 call ezcremrk(msg)
c 
c                 call gqcr(iwsid(iwinop), 1, 1, errind, cr, cg, cb)
c                 write (msg, '(a, 3f6.3)') ' fgcolor actually used: ', cr, cg, cb
c                 call ezcremrk(msg)
c              endif
		
	     iezcdev = iezcdev + 1
             ezcdevs(iezcdev) = iws
		
c ... found existing window; perform the command "on"
	  else
	     if (.not.winstate(idxfd)) then	#window not active
c 	        call gacwk(iwsidfd)
	        winstate(idxfd) = TRUE
	        iezcdev = iezcdev + 1

c ... deactivate all other windows in the same display
	        if (iwinop > 1) then
		   do i = 1, iwinop
		      if (i .ne. idxfd) then
		         if (windisp(i) .eq. windisp(idxfd) & winstate(i)) then
		            call ezcwoff(i)
		         endif
		      endif
		   enddo
	        endif

c ... window already open
	     else
		msg = "Window already open, ignoring command- "\
			//iflag(1:ezcgetcl(iflag))
	   	errsvl = 0
		call ezcerror(msg, errsvl)
	     endif
	  endif

c ... for "off" command
       elseif (iflag .eq. "off") then

c ... for existing window
          if (found) then

c ... deactivate the WS
	     if (winstate(idxfd)) then
                call ezcwoff(idxfd)

c ... window already inactive
             else
		msg = "Window is inactive, ignoring command- "\
			//iflag(1:ezcgetcl(iflag))
		errsvl = 0
		call ezcerror(msg, errsvl)
	     endif

c ... off command issued to non-existing window
	  else
	     msg = "Window does not exist, ignoring command- "\
			//iflag(1:ezcgetcl(iflag)) //" "\
                        //istring(1:ezcgetcl(istring))
	     errsvl = 0
	     call ezcerror(msg, errsvl)
	  endif
	 
c c ... for "send" command
c        elseif (iflag .eq. "send") then
c 
c c ... turn off CGM temporarily
c           cgmrvs = FALSE
c           if (iezccgma .gt. 0) then
c   	     call ezccgm("off", " ")
c 	     cgmrvs = TRUE
c 	  endif	
c 
c c ... turn off PS temporarily
c           psrvs = FALSE
c           if (iezcpsa >0) then
c              call ezcps("off", " ")
c              psrvs = TRUE
c           endif
c 
c 	  call ezcshowg
c 
c c ... restore the original state of other devices
c 	  if (cgmrvs) call ezccgm("on", " ")
c 	  if (psrvs) call ezcps("on", " ")

c ... for "close" command
       elseif (iflag .eq. "close") then
          if (found) then
	     if (winstate(idxfd)) then	#deactivate WS first
c 	        call gdawk(iwsidfd)
	        iezcdev = iezcdev - 1
	     endif

c ... Close window opened by gxwin.
             idwind(1) = iwscn(1, idxfd)
             idwind(2) = iwscn(2, idxfd)
	     errind = pgclos(idwind)
c              call gdwin(idwind, errind)
             if (errind .ne. 1) then
                call utsprint(1, 1, 512, msg,
     &                        44, "Trouble closing window.\nPGCLOS returned: %d",
     &                        errind)
	        call ezcerror(msg, errsvl)
             endif

c ... clean the open window table
             if (idxfd < iwinop) then	#shift table upwards
                do j = idxfd+1, iwinop
                   winame(j-1) = winame(j)
                   winstate(j-1) = winstate(j)
                   iwsid(j-1) = iwsid(j)
                   iwscn(1, j-1) = iwscn(1, j)
                   iwscn(2, j-1) = iwscn(2, j)
                   windisp(j-1) = windisp(j)
                   winncoli(j-1) = winncoli(j)
                   wincola(j-1) = wincola(j)
                enddo
             endif

c ... decrease the number of open windows
             iwinop = iwinop - 1

c c ... automatically activate the only window
c              if (iwinop .eq. 1) then
c                 if (.not.winstate(1)) then
c                    call ezcwinr("on", winame(1))
c                 endif
c              endif
c 
c c ... close the only window as default
c           elseif (iwinop .eq. 1) then
c              call ezcwinr("close", winame(1))
c 
c c ... close all windows
c           elseif (istring .eq. "all") then
c 
c c ... backward loop avoids recursion problem.
c              do indx = iwinop, 1, -1
c                 call ezcwinr("close", winame(indx))
c              enddo
c 
c c ... could not find right window to close
c 	  else
c 	     msg = "Could not find window to close, ignoring command- "\
c 			//iflag(1:ezcgetcl(iflag)) //" "\
c                         //istring(1:ezcgetcl(istring))
c 	     errsvl = 0
c 	     call ezcerror(msg, errsvl)

	  endif

c c ... for "colormap" command
c        elseif (iflag .eq. "colormap") then
c           oldcmap = winclrmap
c           winclrmap = istring
c           if (debcolr .ne. 0) then
c              msg = "Window colormap requested: "\
c                    //winclrmap(1:ezcgetcl(winclrmap))
c              call ezcremrk(msg)
c           endif
c 
c c ... open window if non-exist
c           if (iwinop .eq. 0) then
c              call ezcwinr("on", " ")
c           elseif (iwinop .eq. 1) then
c              if (oldcmap .ne. winclrmap) then
c                 namestore = winame(1)
c                 call ezcwinr("close", namestore)
c                 call ezcremrk("Closed the current window to reset the colormap.")
c                 call ezcwinr("on", namestore)
c                 if (namestore .ne. " ") then
c                    msg = "New window has same name: "\
c                            // namestore(1:ezcgetcl(namestore))
c                    call ezcremrk(msg)
c                 endif
c              endif
c           else 
c              call ezcremrk("Multiple windows are open.")
c              call ezcremrk("Please close all windows, then open a new one.")
c              call ezcremrk("The new colormap will be assigned to the new window.")
c              call ezcremrk("Use 'win list' or 'win slist' to list windows, ")
c              call ezcremrk(" or 'win close all' to close all windows.")
c           endif

c ... for "list" command
       elseif (iflag .eq. "list") then
          call ezcwinll(namestore)

c ... for "slist" command
       elseif (iflag .eq. "slist") then
          call ezcwinsl

c ... unknown command for windows
       else
	  msg = "ezcwin: Unknown command, ignoring- "\
			//iflag(1:ezcgetcl(iflag))
	  errsvl = 0
	  call ezcerror(msg, errsvl)
       endif

       return
       end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EZCWINR - recursion intermediary

        subroutine ezcwinr(iflag, istring)
        character*(*) iflag, istring

        call ezcwin(iflag, istring)

        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EZCWOFF - deactivate window whose index is idx.
c         - Note that there is no error checking here. It is assumed that
c         - ezcwoff is only called by ezcwin with a valid index.

        subroutine ezcwoff(idx)
        integer idx

c Use(Devices)
c       call gdawk(iwsid(idx))
c 
c       winstate(idx) = FALSE
c       iezcdev = iezcdev - 1

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EZCWINLL - EZD window long-list routine
c          - implements command "win list name"
c          - if name is omitted (blank), print entire contents of windows arrays.
c          - For nonblank name, search for that name and print only its info.
c          - If no windows are open, report that fact.

      subroutine ezcwinll(name)
      character*(*) name

c       Use(Devices)
c 
c       character*120 msg
c       integer ezcgetcl
c       external ezcgetcl
c       integer indx, idx
c       logical found
c 
c       if (iwinop > 0) then
c          if (name .ne. " ") then
c             found = FALSE
c             do idx = 1, iwinop
c                if (winame(idx) .eq. name) then
c                   found = TRUE
c                   indx = idx
c                   break
c                endif
c             enddo
c 
c             if (found) then
c                write (msg, 100) indx, winstate(indx), winame(indx), 
c                     iwsid(indx), iwscn(1, indx), iwscn(2, indx), windisp(indx)
c                call ezcremrk(msg)
c             else
c                msg = 'Window name not found: ' // name(1:ezcgetcl(name))
c                call ezcerror(msg, 0)
c             endif
c 
c          else
c             call ezcremrk(' idx st  name              wsid     wsconn (hex)    display')
c             call ezcremrk(' -------------------------------------------------------------------')
c             do idx = 1, iwinop
c                write (msg, 100) idx, winstate(idx), winame(idx), 
c                     iwsid(idx), iwscn(1, idx), iwscn(2, idx), windisp(idx)
c                call ezcremrk(msg)
c             enddo
c  100        format (i4, l3, 2x, a, i6, 2x, 2z8, 2x, a)
c          endif
c 
c       else
c 
c c ... should never get here (see ezcwin)
c          call ezcerror('No windows are open.', 0)
c       endif

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EZCWINSL - window short-list routine.
c          - implements command "win slist name".
c          - print window names and ids only.
c          - If no windows are open, report that fact.
c          - argument name, if given on command, is ignored.

      subroutine ezcwinsl

c       Use(Devices)
c 
c       character*120 msg
c       integer ezcgetcl
c       external ezcgetcl
c       integer idx
c 
c       if (iwinop > 0) then
c          call ezcremrk(' idx st  name')
c          call ezcremrk(' -------------------------')
c          do idx = 1, iwinop
c             write (msg, 200) idx, winstate(idx), winame(idx)
c             call ezcremrk(msg)
c          enddo
c  200     format (i4, l3, 2x, a)
c 
c       else
c 
c c ... should never get here (see ezcwin)
c          call ezcerror('No windows are open.', 0)
c       endif

      return
      end

c--------------------------------------------------------------------------

c                    POSTSCRIPT DEVICE ROUTINES

c--------------------------------------------------------------------------

c EZCPS - routine to process command "ps iflag istring".

      subroutine ezcps(iflag, istring)
      implicit none
      character*(*) iflag, istring

Use(Devices)

      character*32 type, mode

      type = "PS"
      mode = "COLOR"

      call ezcgen(iflag, istring, psfile, psfsfx,
     &            2, type, mode, psclrmap, PSLOGUNIT,
     &            iezcpsa, iezcpsf, iezcpsn, iezcpsid, ezcpsc,
     &            0.0, 0.0, 0.0, 0.0)

      return
      end

c--------------------------------------------------------------------------

c                        CGM DEVICE ROUTINES

c--------------------------------------------------------------------------

c EZCCGM - process command "cgm iflag istring"

      subroutine ezccgm(iflag, istring)
      implicit none
      character*(*) iflag, istring

Use(Devices)

      character*32 type, mode

      type = "CGM"
      mode = "COLOR"

      call ezcgen(iflag, istring, cgmfile, cgmfsfx,
     &            3, type, mode, cgmclrmap, CGMLOGUNIT,
     &            iezccgma, iezccgmf, iezccgmn, iezccgmid, ezccgmc,
     &            0.0, 0.0, 0.0, 0.0)

      return
      end

c--------------------------------------------------------------------------

c                        JPEG DEVICE ROUTINES

c--------------------------------------------------------------------------

c EZCJPEG - process command "jpeg iflag istring"

      subroutine ezcjpeg(iflag, istring)
      implicit none
      character*(*) iflag, istring

Use(Devices)

      character*32 type, mode

      type = "JPEG"
      mode = "COLOR"

      call ezcgen(iflag, istring, jpegfile, jpegfsfx,
     &            4, type, mode, jpegclrmap, JPEGLOGUNIT,
     &            iezcjpega, iezcjpegf, iezcjpegn, iezcjpegid, ezcjpegc,
     &            0.0, 0.0, 400.0, 400.0)

      return
      end

c--------------------------------------------------------------------------

c                        MPEG DEVICE ROUTINES

c--------------------------------------------------------------------------

c EZCMPEG - process command "mpeg iflag istring"

      subroutine ezcmpeg(iflag, istring)
      implicit none
      character*(*) iflag, istring

Use(Devices)

      character*32 type, mode

      type = "MPEG"
      mode = "COLOR"

      call ezcgen(iflag, istring, mpegfile, mpegfsfx,
     &            4, type, mode, mpegclrmap, MPEGLOGUNIT,
     &            iezcmpega, iezcmpegf, iezcmpegn, iezcmpegid, ezcmpegc,
     &            0.0, 0.0, 400.0, 400.0)

      return
      end

c--------------------------------------------------------------------------

c                        GENERIC DEVICE ROUTINE

c--------------------------------------------------------------------------

c EZCGEN - process command "xxx iflag istring"

      subroutine ezcgen(iflag, istring, file, fsfx,
     &                  nwt, wtype, wmode, clrmap, wunit,
     &                  iezca, iezcf, iezcn, iezcid, ezcc,
     &                  x1, y1, x2, y2)
      character*(*) iflag, istring
      Filename file
      integer fsfx, nwt
      character*255 wtype, wmode, clrmap
      integer wunit, iezca
      Filename iezcf
      integer iezcn, iezcid, ezcc
      real(Size8) x1, y1, x2, y2

Use(Devices)
Use(Colors)

      integer iws, nc, nwm, nie, nfr, nwf
      integer suffile, sufflog  #  Temporary file suffix values
      integer fnsucc, errind    #  Flag for successful file name assignment
      integer verbfl
      character*(512) msg       #  Error message
      character*8 ftype
      Filename  fnameroot

      integer rtgetverb
      external rtgetverb

      integer ezcgetcl, ezcgfnam
      external ezcgetcl, ezcgfnam

      integer pgmkdv, pgopen, pgclos, pgsbkc, pgspal
      external pgmkdv, pgopen, pgclos, pgsbkc, pgspal

      ftype = wtype(1:nwt)
      nwm   = ezcgetcl(wmode)
      nie   = ezcgetcl(iezcf)
      nwf   = ezcgetcl(file)

      verbfl = rtgetverb()

      call ezcp(1)
      if ((iflag .eq. "on") .or. (iflag .eq. "open")) then

c ... open the device
         if (iezca .lt. 0) then

c ... set the name of the file
            if (fnroot .eq. " ") fnroot = "Problem"
            nfr = ezcgetcl(fnroot)

            fnameroot = fnroot(1:nfr)

c ... append suffix, making sure file and log get same one.
c ... This takes care of case user has moved either old
c ... files or old log files (but not both).
c ... Note that this is not foolproof if there are gaps in the
c ... suffixes on existing files.  (FNF 3/05/97)
            suffile = fsfx
            fnsucc  = ezcgfnam(nfr, fnameroot, nwt, ftype, 
     &                         nwf, file, suffile, 0)
            if (fnsucc .eq. 0) goto 210

            sufflog = fsfx
            fnsucc  = ezcgfnam(nfr, fnameroot, nwt, ftype,
     &                         nie, iezcf, sufflog, 1)
            if (fnsucc .eq. 0) goto 200

            fsfx = max(suffile, sufflog)
            if (suffile < fsfx) then
               fsfx   = fsfx - 1          # adjust to suffix actually used
               fnsucc = ezcgfnam(nfr, fnameroot, nwt, ftype,
     &                           nwf, file, fsfx, 0)
               if (fnsucc .eq. 0) goto 210

            else if (sufflog < fsfx) then
               fsfx   = fsfx - 1          # adjust to suffix actually used
               fnsucc = ezcgfnam(nfr, fnameroot, nwt, ftype,
     &                           nie, iezcf, fsfx, 1)
               if (fnsucc .eq. 0) goto 200
            endif

c ... now actually create/open the file
            iws = pgmkdv(nwt, ftype, nwm, wmode, nwf, file)

            errind = pgsbkc(iws, bakcol)

            errind = pgopen(iws, x1, y1, x2, y2)

            errind = pgspal(iws, 3, 'rgb')

c ... notify user of the file name
            if (verbfl .eq. 1) then
               call utsprint(1, 1, 512, msg,
     &                       24, "Beginning %.*s File %.*s",
     &                       nwt, ftype, nwf, file)
               call ezcremrk(msg)
            endif

c ... create/open the log file and notify user
            open(unit=wunit, file=iezcf(1:nie),
     &           status='new', err=200)

            if (verbfl .eq. 1) then
               call utsprint(1, 1, 512, msg,
     &                       23, "Beginning %.*s Log %.*s",
     &                       nwt, ftype, nie, iezcf)
               call ezcremrk(msg)
            endif

c ... finish device initialization
            iezca  = 1
            iezcid = iws
            iezcdev   = iezcdev + 1
            ezcdevs(iezcdev) = iws
         endif

c ... check for previously opened file
         if (iezca .eq. 0) then
c            call gacwk(iws)
            iezcdev  = iezcdev + 1
            iezca = 1
         endif

      elseif (iflag .eq. "off") then
         if (iezca .gt. 0) then
c            call gdawk(iws)
            iezcdev = iezcdev - 1
            iezca = 0
         endif

      elseif (iflag .eq. "close") then
         if (iezca .ge. 0) then
            if (iezca .gt. 0) then
c               call gdawk(iws)
               iezcdev = iezcdev - 1
            endif
            errind = pgclos(iezcid)

            iezca = -1

            if (verbfl .eq. 1) then
               call utsprint(1, 1, 512, msg,
     &                       32, "Closed %.*s File %.*s, %d frames",
     &                       nwt, ftype, nwf, file, iezcn)
               call ezcremrk(msg)
            endif

            iezcn = 0
            close(wunit)

            if (verbfl .eq. 1) then
               call utsprint(1, 1, 512, msg,
     &                       25, "Closed %.*s Log File %.*s",
     &                       nwt, ftype, nie, iezcf)
               call ezcremrk(msg)
            endif
         endif

      elseif (iflag .eq. "colormap") then
         nc     = ezcgetcl(clrmap)
         errind = pgspal(iezcid, nc, clrmap)
         if (debcolr .ne. 0) then
            call utsprint(1, 1, 512, msg,
     &                    34, "%.*s colormap has been set to %.*s",
     &                    nwt, ftype, nc, clrmap)
            call ezcremrk(msg)
         endif

      else
         msg = "ezcgen: ignoring unknown command- "//iflag(1:ezcgetcl(iflag)) 
         call ezcerror(msg, 0)
      endif
      return

c ... error exits

 200  call utsprint(1, 1, 512, msg,
     &              30, "Could not create %.*s log file",
     &              nwt, ftype)
      call ezcerror(msg, 1)
      return

 210  call utsprint(1, 1, 512, msg,
     &              35, "Could not create %.*s graphics file",
     &              nwt, ftype)
      call ezcerror(msg, 1)

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

