c
c EPMISC.M - miscellaneous graphics things
c

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EZCSCM - create and initialize a colormap to all devices

      subroutine ezcscm(idepth)
      integer idepth

      real(Size8)  red(256), green(256), blue(256)
      integer i, idepth2, istat, ezccoltb
      character*80 msg

c ... create the colormap
      idepth2 = idepth/2
      do i = 1, idepth2
         red(i)   = i * 1.0/idepth
         blue(i)  = 1.0 - red(i)
         green(i) = i * 1.0/idepth2
      enddo

      do i = idepth2+1, idepth
         red(i)   = i * 1.0/idepth
         blue(i)  = 1.0 - red(i)
         green(i) = 1.0 - (i-idepth2) * 1.0/(idepth-idepth2)
      enddo

c ... check to see that a graphics device exists and fire up a new frame
      call pltstart

c ... download the colormap to all devices
c      istat = ezccoltb(1, idepth, red, green, blue)
      istat = 0

c ... print error message if warranted
      if (istat .ne. 0) then
         call utsprint(1, 1, 80, msg,
     &                 34, "Colormap not loaded.  Status is %d",
     &                 istat)
         call ezcerror(msg,0)
      endif

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EZCLR4 - convert an array of reals into an array of indices into a colormap
c        - using a linear mapping

      subroutine ezclr4(ncol, n, z, ndx)
      integer ncol, n
      real(Size8) z(n)
      integer ndx(n)

      integer i, ncm1, nind, colmap
      real(Size8) f, zmin, zmax

c ... define function to map f (in [0.,1.]) to color index (in [0,nind]),
c ... mapping any negative f's to 0, f > 1 to nind
      colmap(f,nind) = max(0, min(int(f*nind), nind))

c ... find range of data
      zmin = z(1)
      zmax = z(1)
      do i=2, n
         if (z(i) > zmax) zmax = z(i)
         if (z(i) < zmin) zmin = z(i)
      enddo

c ... convert data to color indices
      if (zmax == zmin) then
         do i=1,n
            ndx(i) = 1
         enddo
      else
         ncm1 = ncol - 1
         do i=1, n
            ndx(i) = colmap((z(i) - zmin)/(zmax - zmin), ncm1) + 1
         enddo
      endif

      return
      end
 
c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c ISOPLOT - isosurface plotting
c         - usage: plot all t(i,j,k) = c0
c
c  Notes:
c   o this routine is not designed to stand alone.  it will only work
c     with the basis system.
c
c   o the scratch array should be at least (max(nx,ny,nz)+2)^2 words
c     long, ie. at least twice the size of z(); we get the scratch array
c     dynamically.
c
c   o it is recommended that the viewpoint be outside the plotted
c     box by a factor of 2-5 on the horizontal and 5-10 on the vertical
c     and the point should be well outside the box on the z-coordinate.
c     since i am using perspective viewing, we don't want the object
c     to be too small, so we might have to use smaller multipliers.
c     the point we will look towards is the origin: by design, the
c     origin will be the center of the plotted surface.
c
c   o we accept any size arrays on input now, and provide data thinning
c     as necessary to produce a good plot.  the number of data points
c     used in the plot is user-controllable via basis.  the actual arrays
c     used in plotting will be allocated and freed within this routine.

      subroutine isoplot(t,in_x,in_y,in_z,c0,view)
      INTEGER in_x, in_y, in_z
      real(Size8) t(in_x,in_y,in_z), c0, view(2)

Use(Isosurface_Plot)
Use(Iso_Internal_Real_Scalar)

      real(Size8) viewscale
      Integer nx, ny, nz, utgetcl, ichht
      Integer ixflip, iyflip
      Integer ixswap, iyswap
      Integer muvwp2
      Integer xlablen, ylablen
      integer i
      character*80 xlab, ylab
      real(Size8) sight(3)

c ... initialize sight to zeros.
      do i = 1, 3
         sight(i) = 0.0_Size8
      enddo

c ... put data into plotting arrays, and scale it to prevent pancake plots
      nx = min(in_x, isonpx)
      ny = min(in_y, isonpy)
      nz = min(in_z, isonpz)
      call allot("isotplot", nx*nx*nz)
c      call isoload(t, in_x, in_y, in_z, isotplot, nx, ny, nz)

c ... set up viewport information
c      call isovpt(view, ixflip, iyflip, ixswap, iyswap)

c ... other initialization
      viewscale = isoview * sqrt(25*nx*nx + 12.25*ny*ny + 4*nz*nz)
      sight(1) = viewscale * cos(view(2)) * cos(view(1))
      sight(2) = viewscale * cos(view(2)) * sin(view(1))
      sight(3) = viewscale * (sin(view(2)) + 1.0_Size8)

c ... allocate the work array
      muvwp2 = max(nx,ny,nz)+2
      call allot("isoscr", muvwp2*muvwp2)

c ... check to see that a graphics device exists and fire up a new frame
      call pltstart
      call ezcnf

c ... plot the title: center string, set height (30pt), and write text
c      using:  AGPWRT(x, y, msg, len, ichsize, angle, center)
c      call gselnt(0)
      ichht = min0(800 / utgetcl(isotitle), 30)
c      call agpwrt(0.5_Size8, 0.95_Size8, isotitle, utgetcl(isotitle), ichht, 0, 0)

c ... get true label lengths
      xlablen = utgetcl(isoxtle)
      ylablen = utgetcl(isoytle)

c ... do the plot: ISOSRF(T, LU, MU, LV, MV, MW, EYE, MP2, ISOSCR, C0, ISOFLG)
c      call isosrf(isotplot, nx, nx, ny, ny, nz, sight, muvwp2, isoscr, c0, isoflg)

c ... convert lowercase to uppercase in labels because PWRZI can't
c ... process lowercase
      xlab = isoxtle
      ylab = isoytle
c      call isol2u(xlab)
c      call isol2u(ylab)

c ... label the axes
      if (isolabel == 1) then
         ichht = min0(630 / max0(xlablen,ylablen), 30)
c         call pwrzi(1.0_Size8, -0.5_Size8, 0.0_Size8, xlab, xlablen, ichht, 
c                    -1, -2, 1)
c         call pwrzi(-0.5_Size8, 1.0_Size8, 0.0_Size8, ylab, ylablen, ichht,
c                    2, -1, -1)
      endif

c      call sflush

c ... call the external notification routine
      if (isoxn == 1) then
         call ezcsetxn
      endif

c ... cleanup and exit
      call basfree("isoscr")
      call basfree("isotplot")

      return
 101  format(A, G10.3, A, G10.3)
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c SRFPLOT - surface plotting z(i,j) = F(x(i), y(j)).
c
c  Notes:
c   o this routine is not designed to stand alone.  it will only work
c     with the basis system.
c
c   o the scratch array should be at least 2*nx*ny words long, ie.
c     at least twice the size of z(); we get the scratch array
c     dynamically.
c
c   o it is recommended that the viewpoint be outside the plotted
c     box by a factor of 2-5 on the horizontal and 5-10 on the vertical
c     and the point should be well outside the box on the z-coordinate.
c     since i am using perspective viewing, we don't want the object
c     to be too small, so we might have to use smaller multipliers.
c     the point we will look towards is the origin: by design, the
c     origin will be the center of the plotted surface.
c
c   o we accept any size arrays on input now, and provide data thinning
c     as necessary to produce a good plot.  the number of data points
c     used in the plot is user-controllable via basis.  the actual arrays
c     used in plotting will be allocated and freed within this routine.
c
c   o CAUTION: Input array view is changed internally by srfplot.

      subroutine srfplot(x, y, z, in_x, in_y, view)
      Integer in_x, in_y
      real(Size8) x(in_x), y(in_y), z(in_x,in_y)
      real(Size8) view(2)

Use(Surface_Plot)

      Integer i, nx, ny, utgetcl, ichht
      Integer ixflip, iyflip, ixswap, iyswap
      Integer xlablen, ylablen, zlablen, srfscal
      real(Size8) low(3), high(3), sight(6), r0
      real(Size8) viewscale, picscale
      character*128  msgx, msgy, msgz
      character*80 xlab, ylab

      viewscale = 12.5_Size8
      picscale  = 1.1_Size8

c ... initialize sight to zeros
      do i=1,6
         sight(i) = 0.0_Size8
      enddo

c ... set some common block values (like FRAME off)
      ifr_n = 0
      if (srfiskrt == 1) then
         iskrt_n = 1

c ... the following assumes z-data scaled to [-1,1].
         hskrt_n = -(1.0_Size8 + srfhskrt)
      else
         iskrt_n = 0
      endif

c ... put data into plotting arrays, and scale it to prevent pancake plots
      nx = min(in_x, srfnpx)
      ny = min(in_y, srfnpy)
      call allot("srfxplot", nx)
      call allot("srfyplot", ny)
      call allot("srfzplot", nx*ny)
c      call srfload(x, y, z, in_x, in_y, srfxplot, srfyplot, srfzplot, nx, ny)

c      if (srfscal(srfxplot, srfyplot, srfzplot, nx, ny, low, high) <> OK)
c     &    goto 900

c ... set up viewport information
c      call srfvpt(view, ixflip, iyflip, ixswap, iyswap)

c ... other initialization
      sight(1) = viewscale * cos(view(2)) * cos(view(1))
      sight(2) = viewscale * cos(view(2)) * sin(view(1))
      sight(3) = viewscale * (sin(view(2)) + 1.0_Size8)
      r0 = sqrt(sight(1)*sight(1) + sight(2)*sight(2) + sight(3)*sight(3))

c ... allocate the work array
      call allot("srfscr", 2*nx*ny)

c ... check to see that a graphics device exists and fire up a new frame
      call pltstart
      call ezcnf

c ... plot the title: center string, set height (30pt), and write text
c      call gselnt(0)
      ichht = min0(800 / utgetcl(srftitle), 30)
c      call agpwrt(0.5_Size8, 0.95_Size8, srftitle, utgetcl(srftitle), ichht, 0, 0)

c ... get true label lengths, and put legends into the plotting
c ... window using:  AGPWRT(x, y, msg, len, ichsize, angle, center)
      xlablen = utgetcl(srfxtle)
      ylablen = utgetcl(srfytle)
      zlablen = utgetcl(srfztle)
      call utsprint(1, 1, 128, msgx,
     &              16, "%.*s in %d to %d",
     &              xlablen, srfxtle, low(1), high(1))
      call utsprint(1, 1, 128, msgy,
     &              16, "%.*s in %d to %d",
     &              ylablen, srfytle, low(2), high(2))
      call utsprint(1, 1, 128, msgz,
     &              16, "%.*s in %d to %d",
     &              zlablen, srfztle, low(3), high(3))

 101  format(A, G10.3, A, G10.3)

      if (srftopln == 1) then
c         call agpwrt(1.0_Size8, 0.98_Size8, msgx, utgetcl(msgx), 2, 0, 1)
c         call agpwrt(1.0_Size8, 0.96_Size8, msgy, utgetcl(msgy), 2, 0, 1)
c         call agpwrt(1.0_Size8, 0.94_Size8, msgz, utgetcl(msgz), 2, 0, 1)
      else
c         call agpwrt(0.98_Size8, 0.09_Size8, msgx, utgetcl(msgx), 2, 0, 1)
c         call agpwrt(0.98_Size8, 0.07_Size8, msgy, utgetcl(msgy), 2, 0, 1)
c         call agpwrt(0.98_Size8, 0.05_Size8, msgz, utgetcl(msgz), 2, 0, 1)
      endif

c ... set the parameters: SETR (XMIN, XMAX, YMIN, YMAX, ZMIN, ZMAX, R0)
c      call setr(-picscale, picscale, -picscale, picscale,
c     &          -picscale, picscale, r0)

c ... do the plot: SRFACE(X, Y, Z, WORK, MX, NX, NY, SIGHT, STEREO) 
c      call srface(srfxplot, srfyplot, srfzplot, srfscr, nx, nx, ny, sight, 0._4)

c ... convert lowercase to uppercase in labels because PWRZS can't
c ... process lowercase
      xlab = srfxtle
      ylab = srfytle
c      call srfl2u(xlab)
c      call srfl2u(ylab)

c ... label the axes
      if (srflabel == 1) then
         ichht = min0(630 / max0(xlablen,ylablen), 30)
c         call pwrzs(0.0_Size8, 1.3_Size8*ixswap, 0.0_Size8, xlab, xlablen, ichht, 
c                    -1*ixflip, -2*ixflip, 0)
c         call pwrzs(-1.3_Size8*iyswap, 0.0_Size8, 0.0_Size8, ylab, ylablen, ichht,
c                    -2*iyflip, 1*iyflip, 0)
      endif

c      call sflush

c ... call the external notification routine
      if (srfxn == 1) then
         call ezcsetxn
      endif

c ... cleanup and exit
 900  continue
      call basfree("srfscr")
      call basfree("srfzplot")
      call basfree("srfyplot")
      call basfree("srfxplot")

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------
