c
c EPREND.M - plot handlers
c

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EZCPLOTX - this function is intended to provide a general skeleton
c          - for the ezcplotX family of functions
c          - Replace X with the appropriate letter(s) and
c          - customize as appropriate for the command being processed.  Of course, 
c          - you will need to add an appropriate mcchrdef call to ezcinit, and
c          - write a routine ezcaddX to process the data as appropriate.
c          - this pattern is for a mesh-oriented command with a first argument z
c          - followed by x, y, ireg (a.k.a. zt, rt, ireg).

      integer function ezcplotX (nargs, nargs1)
      integer nargs, nargs1

c       Process the command: plotX z x y ireg keylist
c       where
c          z        is required, but
c          x, y, ireg can be defaulted to ezcx, ezcy, ezcireg respectively.
c
c       nargs1 is the number of arguments found prior to "keylist".

Use(Keywords)      
Use(Collections)
Use(Default_attributes)
Use(EzcurveDefaults)

c ... declare general local variables directly related to arguments
      integer sz(SS_WIDTH), sx(SS_WIDTH), sy(SS_WIDTH), sireg(SS_WIDTH)
      integer nz1, nz2, nx1, nx2, ny1, ny2, ni1, ni2

      Dynamic(z, real(Size8), 1)
      Dynamic(x, real(Size8), 1)
      Dynamic(y, real(Size8), 1)
      Dynamic(ireg, integer, 1)

c ... declare other needed local variables
      character*80 msg     # for constructing error messages.

c ... declare externals
      Address  paraddr
      external paraddr
      integer  parcoerc, parlen
      external parcoerc, parlen

c ... omit following if not needed
      integer utsizeof
      external utsizeof

c ... get argument list
      call ezcseta(nargs, nargs1, LOCAL)

c ... check for valid number of arguments
c ... (the number here and in format 100 will depend on the command)
      if (nargs1 > 4) then
         write(msg, 100) nargs1
         call ezcremrk(msg)
100      format('plotX: there must be 0-4 numerical arguments, not', i5)
         go to 950
      endif

c ... pick up contents of arguments
      call fcnarga(1, sz)

c ... (the first argument has no default, so fcnargd is not appropriate)
c ... (see ezcplotf for an example of how to allow this to be a function)
      call fcnargd(nargs1, 2, sx, ezcx)
      call fcnargd(nargs1, 3, sy, ezcy)
      call fcnargd(nargs1, 4, sireg, ezcireg)

      call pardim(sz) 
      call pardim(sy) 
      call pardim(sx) 
      call pardim(sireg)

c ... process arguments and check for validity
c ... (Note: if the first argument may be either integer or real8, and   )
c ... (      different treatment applies to the two cases, add something )
c ... (      like the following here:                                    )

c ... check for type of input z
c ... if (sz(SS_TC) .eq. typeint) intflag = 1

c ... (Pass intflag as an additional argument to ezcaddX, and then  )
c ... (process the data accordingly there.  See ezcaddfm for example)

      if (parcoerc(sz, type8).eq.ERR) go to 900
      Point(z) = paraddr(sz)
      if (sz(SS_N) .ne. 2) go to 900

      nz1 = sz(SS_N1)
      nz2 = sz(SS_N1+1)

      if (parcoerc(sy, type8).eq.ERR) go to 900
      Point(y) = paraddr(sy)
      if (sy(SS_N) .ne. 2) go to 900

      ny1 = sy(SS_N1)
      ny2 = sy(SS_N1+1)

      if (parcoerc(sx, type8).eq.ERR) go to 900
      Point(x) = paraddr(sx)
      if (sx(SS_N) .ne. 2) go to 900

      nx1 = sx(SS_N1)
      nx2 = sx(SS_N1+1)

      if (parcoerc(sireg, typeint) .eq. ERR) go to 900

      Point(ireg) = paraddr(sireg)
      if (sireg(SS_N) .ne. 2) go to 900

      ni1 = sireg(SS_N1)
      ni2 = sireg(SS_N1+1)
      if (ni1 .ne. nz1 | ni2 .ne. nz2) then
         call ezcremrk('plotX: ireg the wrong size.')
         go to 900
      endif

      if (nz1 .ne. ny1 | nz2 .ne. ny2 | nz1 .ne. nx1 | nz2 .ne. nx2) then
         call ezcremrk('plotX: argument size mismatch.')
         go to 900
      endif

c ... add this to the collection
c      call ezcaddX(x, y, z, ireg, nz1, nz2)

      call parkill(sz)
      call parkill(sy)
      call parkill(sx)
      call parkill(sireg)
      call ezcrda
      return(OK)

c ... error returns
900   continue
      call parkill(sz)
      call parkill(sy)
      call parkill(sx)
      call parkill(sireg)

950   continue
      call ezcrda
      call ezcremrk("plotX command ignored.")
      call ezcremrk(legenda)   # Echo the command to the terminal.

      return(OK)   # OK just ignores command, doesn't kill collection.
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EZCPLOT - curve plotter

      integer function ezcplot(nargs, nargs1)
      integer nargs, nargs1

      integer sy(SS_WIDTH), sx(SS_WIDTH)
      Dynamic(x, real(Size8), 1)
      Dynamic(y, real(Size8), 1)
      integer m, nyc, ilow, istridey, nx, nxc, i, mnx

      Address  paraddr
      external paraddr
      integer  parcoerc
      external parcoerc

Use(Keywords)      
Use(Collections)
Use(Default_attributes)

      call ezcseta(nargs, nargs1, LOCAL)

      if (nargs1 >=3) then 
         call ezcremrk("plot: incorrect keyword or more than two arguments.")
         go to 900
      endif

      if (nargs1 .eq. 0) then
          call ezcshowf
          go to 800
      endif

      call fcnarga(1, sy)
      call pardim(sy) 
      if (parcoerc(sy, type8).eq.ERR) go to 900

      Point(y) = paraddr(sy)
      if (sy(SS_N) .eq. 0) then
         m = 1
         nyc = 1
         ilow = 1
         istridey = 1
      elseif (sy(SS_N) .eq. 1) then
         m = sy(SS_N1)
         nyc = 1
         ilow = sy(SS_M1)
         istridey = sy(SS_I1)
      elseif (sy(SS_N) >= 2) then
         m = sy(SS_N1)
         nyc = sy(SS_N1+1)
         ilow = 1
         istridey = 1
         do i = 3, sy(SS_N)
            nyc = nyc*sy(SS_N1-1+i)
         enddo
      endif

c ... only y value specified
      if (nargs1 .eq. 1) then
	 Point(x)=Point(y)	#never used
         nx = m
         nxc = 0

c ... nargs1 .eq. 2
      else
         call fcnarga(2, sx)
         if (sx(SS_TC)=  0) then
            nx = m
            nxc = 0
            go to 200
         endif

         call pardim(sx) 
         if (parcoerc(sx, type8).eq.ERR) go to 900

         Point(x) = paraddr(sx)
	 if (sx(SS_N) .eq. 0) then
            nx = 1
            nxc = 1

         elseif (sx(SS_N) .eq. 1) then
            nx = sx(SS_N1)
            nxc = 1

         elseif (sx(SS_N) >= 2) then
            nx = sx(SS_N1)
            nxc = sx(SS_N1+1)
            do i = 3, sx(SS_N)
               nxc = nxc*sx(SS_N1-1+i)
            enddo
         endif
      endif

200   continue

c ... check for zero-length arrays
      mnx = min(m, nx)

c ... add this to the collection
      if (mnx > 0) call ezcaddc(y, m, nyc, ilow, istridey, x, nx, nxc)

      call parkill(sy)
      if (nargs1 > 1) call parkill(sx)

      if (mnx <= 0) then
         call ezcremrk("plot: zero-length array(s).")
         go to 900
      endif

800   continue
      call ezcrda
      return(OK)

900   continue
      call ezcremrk(legenda)
      call ezcrda
      call ezcremrk("plot command ignored.")

      return(OK)
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EZCPLOTC - a contour plotter

      integer function ezcplotc(nargs, nargs1)
      integer nargs, nargs1

      integer sy(SS_WIDTH), sx(SS_WIDTH), 
        sz(SS_WIDTH), nz1, nz2, nx1, nx2, ny1, ny2, ni1, ni2

      integer utsizeof
      external utsizeof

Use(Keywords)      
Use(Collections)
Use(Default_attributes)
Use(EzcurveDefaults)

      integer sireg(SS_WIDTH)
      Dynamic(z, real(Size8), 1)
      Dynamic(y, real(Size8), 1)
      Dynamic(x, real(Size8), 1)
      Dynamic(ireg, integer, 1)
c      Dynamic(zonpt, real(Size8), 1)
      character*80 msg

      Address  paraddr
      external paraddr
      integer  partypea, parcoerc, parlen
      external partypea, parcoerc, parlen

      call ezcseta(nargs, nargs1, LOCAL)

      if (partypea(1) .eq. typef) then
        call parsestr("ezc chameleon ezccin = "//symbols(1)(1:lsymbols(1))//"()")
        call fcnargd(nargs1, 5, sz, "ezc.ezccin")
      else
        call fcnarga(1, sz)
      endif

      call fcnargd(nargs1, 2, sx, ezcx)
      call fcnargd(nargs1, 3, sy, ezcy)
      call fcnargd(nargs1, 4, sireg, ezcireg)

      call pardim(sz) 
      call pardim(sy) 
      call pardim(sx) 
      call pardim(sireg)

      if (nargs1 > 4) then
         write(msg, 100) nargs1
         call ezcremrk(msg)
100   format('plotc: there must be 0-4 numerical arguments, not', i5)
         go to 900
      endif

      if (parcoerc(sz, type8).eq.ERR) go to 900

      Point(z) = paraddr(sz)

      if (sz(SS_N) .ne. 2) then
         call ezcremrk("plotc: z not two-dimensional")
         go to 900
      endif

      nz1 = sz(SS_N1)
      nz2 = sz(SS_N1+1)

      if (parcoerc(sy, type8).eq.ERR) go to 900

      Point(y) = paraddr(sy)

      if (sy(SS_N) .ne. 2) then
         call ezcremrk("plotc: y not two-dimensional")
         go to 900
      endif

      ny1 = sy(SS_N1)
      ny2 = sy(SS_N1+1)

      if (parcoerc(sx, type8).eq.ERR) go to 900

      Point(x) = paraddr(sx)

      if (sx(SS_N) .ne. 2) then
         call ezcremrk("plotc: x not two-dimensional")
         go to 900
      endif

      nx1 = sx(SS_N1)
      nx2 = sx(SS_N1+1)

      if (parcoerc(sireg, typeint) .eq. ERR) go to 900

      Point(ireg) = paraddr(sireg)

      if (sireg(SS_N) .ne. 2) then
         call ezcremrk("plotc: ireg not two-dimensional")
         go to 900
      endif

      ni1 = sireg(SS_N1)
      ni2 = sireg(SS_N1+1)

      if (ni1 .ne. nz1 | ni2 .ne. nz2) then
         call ezcremrk('plotc: ireg size not the same as z.')
         go to 900
      endif

      if (nz1 .ne. ny1 | nz2 .ne. ny2 | nz1 .ne. nx1 | nz2 .ne. nx2) then
         call ezcremrk('plotc: size mismatch, ignored.')
         go to 900
      endif

c ... add this to the collection
c      if (centera .eq. NO) then
c         call memtemp(Point(zonpt), Quote(real8), nz1*nz2,
c     &                Quote(EZCPLOTC:zonpt ))
c         call ezctopt(z, zonpt, ireg, nz1, nz2)         
c         call ezcaddmc(x, y, zonpt, ireg, nz1, nz2)
c         call memfree(Point(zonpt))
c      else
c         call ezcaddmc(x, y, z, ireg, nz1, nz2)
c      endif
      call ezcaddmc(x, y, z, ireg, nz1, nz2)

      call parkill(sz)
      call parkill(sy)
      call parkill(sx)
      call parkill(sireg)

800   continue
      call ezcrda
      return(OK)

900   continue
      call parkill(sz)
      call parkill(sy)
      call parkill(sx)
      call parkill(sireg)
      call ezcrda
      call ezcremrk("plotc: data not suitable, ignored.")
      call ezcremrk(legenda)

      return(OK)
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EZCPLOTF - poly fill plotter

      integer function ezcplotf(nargs, nargs1)
      integer nargs, nargs1, sy(SS_WIDTH), sx(SS_WIDTH), 
        sz(SS_WIDTH), nz1, nz2, nx1, nx2, ny1, ny2, ni1, ni2

      integer partypea, utsizeof
      external partypea, utsizeof

Use(Keywords)      
Use(Collections)
Use(Default_attributes)
Use(EzcurveDefaults)

      integer sireg(SS_WIDTH)
      Dynamic(z, real(Size8), 1)		#physics quantities for color coded
      Dynamic(y, real(Size8), 1)
      Dynamic(x, real(Size8), 1)
      Dynamic(ireg, integer, 1)
      character*80 msg

      Address  paraddr
      external paraddr
      integer  parcoerc, parlen
      external parcoerc, parlen
      integer cidxflag			#flag for z value as color indices

      cidxflag = 0			#initialize to 0

      call ezcseta(nargs, nargs1, LOCAL)
      if (nargs1 > 4) then
         write(msg, 100) nargs1
100      format('plotf: there must be 0-4 numerical arguments, not', i5)
         call ezcremrk(msg)
         go to 950
      endif

c ... note: The following if-then-else is to process the first argument,
c ... which can either be an array of physics variables (or color indices)
c ... or a function that produces same.  It first checks whether it is a
c ... function.  If so, it has the parser invoke the function and put the
c ... result in variable ezc.ezccin.  Since we already know than nargs1<=4, 
c ... the 5 in the fcnargd call will cause sz to take the value of ezc.ezccin.
c ... the else branch merely picks up the value of the argument in sz.
c ... since no default is appropriate, fcnarga is called instead of fcnargd.
      if (partypea(1) .eq. typef) then
        call parsestr(
"ezc chameleon ezccin = "//symbols(1)(1:lsymbols(1))//"()")
        call fcnargd(nargs1, 5, sz, "ezc.ezccin")

      else
        call fcnarga(1, sz)
      endif

      call fcnargd(nargs1, 2, sx, ezcx)
      call fcnargd(nargs1, 3, sy, ezcy)
      call fcnargd(nargs1, 4, sireg, ezcireg)

      call pardim(sz) 
      call pardim(sy) 
      call pardim(sx) 
      call pardim(sireg)

c ... check for type of z input, to distinguish between color index input
c ... and physics quantity input.  (cidxflag is passed to ezcaddfm.)
      if (sz(SS_TC) .eq. typeint) cidxflag = 1

      if (parcoerc(sz, type8).eq.ERR) go to 900
      Point(z) = paraddr(sz)
      if (sz(SS_N) .ne. 2) go to 900

      nz1 = sz(SS_N1)
      nz2 = sz(SS_N1+1)

      if (parcoerc(sy, type8).eq.ERR) go to 900
      Point(y) = paraddr(sy)
      if (sy(SS_N) .ne. 2) go to 900

      ny1 = sy(SS_N1)
      ny2 = sy(SS_N1+1)

      if (parcoerc(sx, type8).eq.ERR) go to 900
      Point(x) = paraddr(sx)
      if (sx(SS_N) .ne. 2) go to 900

      nx1 = sx(SS_N1)
      nx2 = sx(SS_N1+1)

      if (parcoerc(sireg, typeint) .eq. ERR) go to 900
      Point(ireg) = paraddr(sireg)

      if (sireg(SS_N) .ne. 2) go to 900

      ni1 = sireg(SS_N1)
      ni2 = sireg(SS_N1+1)
      if (ni1 .ne. nz1 | ni2 .ne. nz2) then
         call ezcremrk('plotf: ireg the wrong size.')
         go to 900
      endif

      if (nz1 .ne. ny1 | nz2 .ne. ny2 | nz1 .ne. nx1 | nz2 .ne. nx2) then
         call ezcremrk('plotf: size mismatch, ignored.')
         call ezcremrk(legenda)
         go to 900
      endif

c ... add this to the collection
      call ezcaddfm(x, y, z, ireg, nz1, nz2, cidxflag)

      call parkill(sz)
      call parkill(sy)
      call parkill(sx)
      call parkill(sireg)
      call ezcrda
      return(OK)

900   continue
      call parkill(sz)
      call parkill(sy)
      call parkill(sx)
      call parkill(sireg)

950   continue
      call ezcrda
      call ezcremrk("plotf command ignored.")
      call ezcremrk(legenda)

      return(OK)
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EZCPLOTI - image plotter

      integer function ezcploti(nargs, nargs1)
      integer nargs, nargs1

      integer sicell(SS_WIDTH), sx(SS_WIDTH)
      Dynamic(icell, integer)
      Dynamic(x, real(Size8))
      real(Size8) x1, x2, y1, y2
      integer m, n, i

      Address  paraddr
      external paraddr
      integer  parcoerc
      external parcoerc

Use(Keywords)      
Use(Collections)
Use(Default_attributes)

      call ezcseta(nargs, nargs1, LOCAL)
      call fcnarga(1, sicell)
      call pardim(sicell)

      if (parcoerc(sicell, typeint).eq.ERR) go to 900

      Point(icell) = paraddr(sicell)
      if (sicell(SS_N) .eq. 0) then
         m = 1
         n = 1
      elseif (sicell(SS_N) .eq. 1) then
         m = sicell(SS_N1)
         n =1
      elseif (sicell(SS_N) >= 2) then
         m = sicell(SS_N1)
         n = sicell(SS_N1+1)
         do i = 3, sicell(SS_N)
            n = n*sicell(SS_N1-1+i)
         enddo
      endif
      x1 = 0._Size8
      x2 = 1._Size8
      y1 = 0._Size8
      y2 = 1._Size8
      if (nargs1 >= 2) then
         call fcnarga(2, sx)
         if (sx(SS_TC) > 0) then
            if (parcoerc(sx, type8).eq.ERR) go to 900
            Point(x)=paraddr(sx)
            x1 = x
         endif
         call parkill(sx)
      endif

      if (nargs1 >= 3) then
         call fcnarga(3, sx)
         if (sx(SS_TC) > 0) then
            if (parcoerc(sx, type8).eq.ERR) go to 900
            Point(x)=paraddr(sx)
            x2 = x
         endif
         call parkill(sx)
      endif

      if (nargs1 >= 4) then
         call fcnarga(4, sx)
         if (sx(SS_TC) > 0) then
            if (parcoerc(sx, type8).eq.ERR) go to 900
            Point(x)=paraddr(sx)
            y1 = x
         endif
         call parkill(sx)
      endif

      if (nargs1 >= 5) then
         call fcnarga(5, sx)
         if (sx(SS_TC) > 0) then
            if (parcoerc(sx, type8).eq.ERR) go to 900
            Point(x)=paraddr(sx)
            y2 = x
         endif
         call parkill(sx)
      endif

c ... add this to the collection
      call ezcaddi(icell, m, n, x1, x2, y1, y2)
      call parkill(sicell)
      call ezcrda
      return(OK)

900   continue
      call ezcremrk(legenda)
      call ezcrda
      call ezcremrk("plot command ignored.")

      return(OK)
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EZCPLOTM - mesh plotter

      integer function ezcplotm(nargs, nargs1)
      integer nargs, nargs1

      integer sy(SS_WIDTH), sx(SS_WIDTH), sireg(SS_WIDTH)
      Dynamic(x, real(Size8), 1)
      Dynamic(y, real(Size8), 1)
      Dynamic(ireg, integer, 1)
      character*80 msg

      Address  paraddr
      external paraddr
      integer  parcoerc, parlen
      external parcoerc, parlen
      integer kmax, lmax

Use(Keywords)      
Use(Collections)
Use(Default_attributes)
Use(EzcurveDefaults)

      call ezcseta(nargs, nargs1, LOCAL)
      if (nargs1 > 3) then
         write(msg, 100) nargs1
         call ezcremrk(msg)
100   format('plotm: there must be 0-3 numerical arguments, not', i5)
         go to 900
      endif

      call fcnargd(nargs1, 1, sx, ezcx)
      call fcnargd(nargs1, 2, sy, ezcy)
      call fcnargd(nargs1, 3, sireg, ezcireg)

      call pardim(sy) 
      if (parcoerc(sy, type8).eq.ERR) go to 900

      Point(y) = paraddr(sy)

      if (sy(SS_N) .ne. 2) go to 900

      kmax = sy(SS_N1)
      lmax = sy(SS_N1+1)

      call pardim(sx) 
      if (parcoerc(sx, type8).eq.ERR) go to 900

      Point(x) = paraddr(sx)

      if (sx(SS_N) .ne. 2) go to 900
      if (kmax.ne. sx(SS_N1) ) go to 900
      if (lmax.ne. sx(SS_N1+1) ) go to 900
      
      call pardim(sireg)
      if (parcoerc(sireg, typeint).eq.ERR) go to 900

      Point(ireg) = paraddr(sireg)

      if (sireg(SS_N) .ne. 2) go to 900
      if (kmax.ne. sireg(SS_N1) ) go to 900
      if (lmax.ne. sireg(SS_N1+1) ) go to 900

c ... add this to the collection
      call ezcaddm(x, y, ireg, kmax, lmax)
      call parkill(sx)
      call parkill(sy)
      call parkill(sireg)

800   continue
      call ezcrda
      return(OK)

900   continue
      call ezcrda
      call ezcremrk("plotm: argument sizes incorrect, command ignored.")
      call ezcremrk(legenda)

      return(OK)
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EZCPLOTP - draw array of polygon points
c
c  Process the command: plotp x y keylist
c       where
c          x, y  contain the (x, y) coordinates of the polygon(s) to
c                be plotted.  (Both are required.)
c
c       nargs1 is the number of arguments found prior to "keylist".
c

      integer function ezcplotp(nargs, nargs1)

c ... declare arguments
      integer nargs, nargs1

Use(Keywords)      
Use(Collections)
Use(Default_attributes)
Use(EzcurveDefaults)

c ... declare general local variables directly related to arguments.
      integer sx(SS_WIDTH), sy(SS_WIDTH)
      integer nx1, nx2, ny1, ny2

      Dynamic(x, real(Size8), 1)
      Dynamic(y, real(Size8), 1)

c ... declare other needed local variables.
      character*80 msg     # for constructing error messages.

c ... declare externals.
      Address  paraddr
      external paraddr
      integer  parcoerc, parlen
      external parcoerc, parlen

c ... omit following if not needed.
      integer utsizeof
      external utsizeof

      call ezcseta(nargs, nargs1, LOCAL)
      if (nargs1 .ne. 2) then
         write(msg, 100) nargs1
         call ezcremrk(msg)
100      format('plotp: there must be 2 numerical arguments, not', i5)
         go to 950   # Skip over parkill calls--not appropriate here.
      endif

c ... pick up contents of arguments
      call fcnarga(1, sx)
      call pardim(sx) 

      call fcnarga(2, sy)
      call pardim(sy) 

c ... process arguments and check for validity
      if (parcoerc(sx, type8) .eq. ERR) go to 900

      Point(x) = paraddr(sx)
      nx1      = sx(SS_N1)
      if (sx(SS_N) .eq. 1) then        #single polygon
         nx2 = 1
      else if (sx(SS_N) .eq. 2) then   #array of polygons
         nx2 = sx(SS_N1+1)
      else
         call ezcremrk('plotp: 1- or 2-dimensional arrays required.')
         go to 900
      endif

      if (parcoerc(sy, type8) .eq. ERR) go to 900
      Point(y) = paraddr(sy)

      ny1 = sy(SS_N1)
      if (sy(SS_N) .eq. 1) then        #single polygon
         ny2 = 1
      else if (sy(SS_N) .eq. 2) then   #array of polygons
         ny2 = sy(SS_N1+1)
      else
         call ezcremrk('plotp: 1- or 2-dimensional arrays required.')
         go to 900
      endif

      if ((nx1 .ne. ny1) .or. (nx2 .ne. ny2)) then
         call ezcremrk('plotp: argument size mismatch.')
         go to 900
      endif

c ... add this to the collection
      call ezcaddp(x, y, nx1, nx2)

      call parkill(sx)
      call parkill(sy)
      call ezcrda
      return(OK)

900   continue
      call parkill(sx)
      call parkill(sy)

950   continue
      call ezcrda
      call ezcremrk('plotp command ignored.')
c      call ezcremrk(legenda)

      return(OK)
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EZCPLOTPF - draw and fill array of polygons
c
c  Process the command: plotpf z x y keylist
c       where
c          z     is an array of function values on a set of polygons, 
c                or optionally an array of color indices;
c          x, y  contain the x-, y-coordinates of the polygon(s).
c       (All three arguments are required.)
c
c       nargs1 is the number of arguments found prior to "keylist".
c

      integer function ezcplotpf(nargs, nargs1)

c ... declare arguments
      integer nargs, nargs1

Use(Keywords)      
Use(Collections)
Use(Default_attributes)
Use(EzcurveDefaults)

c ... declare general local variables directly related to arguments.
      integer sy(SS_WIDTH), sx(SS_WIDTH), sz(SS_WIDTH)
      integer nx1, nx2, ny1, ny2, nz

      Dynamic(x, real(Size8), 1)
      Dynamic(y, real(Size8), 1)
      Dynamic(z, real(Size8), 1)		#physics quantity to be color coded

c ... declare other needed local variables.
      integer cidxflag			#flag for z value as color indices
      character*80 msg

      integer utsizeof
      external utsizeof
      Address  paraddr
      external paraddr
      integer  parcoerc, parlen, partypea
      external parcoerc, parlen, partypea

      cidxflag = 0			#initialize to 0

c ... get argument list.
      call ezcseta(nargs, nargs1, LOCAL)
      if (nargs1 .ne. 3) then
         write(msg, 100) nargs1
         call ezcremrk(msg)
100      format('plotpf: there must be 3 numerical arguments, not', i5)
c         go to 950   # Skip over parkill calls--not appropriate here.
      endif

c ... note: The following if-then-else is to process the first argument, 
c ... which can either be an array of physics variables (or color indices)
c ... or a function that produces same.  It first checks whether it is a
c ... function.  If so, it has the parser invoke the function and put the
c ... result in variable ezc.ezccin.  Since we already know that nargs1=3, 
c ... the 4 in the fcnargd call will cause sz to take the value of ezc.ezccin.
c ... the else branch merely picks up the value of the argument in sz.
c ... since no default is appropriate, fcnarga is called instead of fcnargd.
      if ( partypea(1) .eq. typef) then
        call parsestr(
"ezc chameleon ezccin = "//symbols(1)(1:lsymbols(1))//"()")
        call fcnargd(nargs1, 4, sz, "ezc.ezccin")
      else
        call fcnarga(1, sz)
      endif

      call pardim(sz) 

c ... simply pick up arguments 2 and 3. (Again, no defaults.)
      call fcnarga(2, sx)
      call pardim(sx) 

      call fcnarga(3, sy)
      call pardim(sy) 

c ... check for type of z input, to distinguish between color index input
c ... and physics quantity input.  (cidxflag is passed to ezcaddpf.)
      if (sz(SS_TC) .eq. typeint) cidxflag = 1

      if (parcoerc(sz, type8) .eq. ERR) go to 900
      Point(z) = paraddr(sz)

      if (sz(SS_N) .eq. 0) then        # scalar
         nz = 1
      else if (sz(SS_N) .eq. 1) then   # 1-D array of values
         nz = sz(SS_N1)
      else
         call ezcremrk('plotpf: z must be a scalar or 1-dimensional array.')
         go to 900
      endif

      if (parcoerc(sx, type8) .eq. ERR) go to 900
      Point(x) = paraddr(sx)

      nx1 = sx(SS_N1)
      if (sx(SS_N) .eq. 1) then        #single polygon
         nx2 = 1
      else if (sx(SS_N) .eq. 2) then   #array of polygons
         nx2 = sx(SS_N1+1)
      else
         call ezcremrk('plotpf: 1- or 2-dimensional x, y arrays required.')
         go to 900
      endif

      if (parcoerc(sy, type8) .eq. ERR) go to 900
      Point(y) = paraddr(sy)

      ny1 = sy(SS_N1)
      if (sy(SS_N) .eq. 1) then        #single polygon
         ny2 = 1
      else if (sy(SS_N) .eq. 2) then   #array of polygons
         ny2 = sy(SS_N1+1)
      else
         call ezcremrk('plotpf: 1- or 2-dimensional x, y arrays required.')
         go to 900
      endif

      if (((nx1 .eq. 1) .and. (nz .ne. 1))  .or.     # single polygon case
     &    ((nx1 .gt. 1) .and. (nz .ne. nx2)) .or.    # array of polygons case
     &     (nx1 .ne. ny1) .or. (nx2 .ne. ny2)) then
         call ezcremrk('plotpf: argument size mismatch.')
         go to 900
      endif

c ... add this to the collection.
      call ezcaddpf(x, y, z, nx1, nx2, cidxflag)

      call parkill(sz)
      call parkill(sy)
      call parkill(sx)
      call ezcrda

      return(OK)

900   continue
      call parkill(sz)
      call parkill(sy)
      call parkill(sx)

950   continue
      call ezcrda
      call ezcremrk("plotpf command ignored.")
      call ezcremrk(legenda)   # Echo the command to the terminal.

      return(OK)
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EZCPLOTR - ray plotter

      integer function ezcplotr(nargs, nargs1)
      integer nargs, nargs1

      integer sl(SS_WIDTH)
      integer laser1, i		
      Dynamic(laser, integer, 1)

      Address  paraddr
      external paraddr
      integer  parcoerc, parlen
      external parcoerc, parlen
      logical exists
      external parsestr, exists
      integer utsizeof, ibasis
      external utsizeof, ibasis

      character*(200) raypow, raycall

Use(Keywords)      
Use(Collections)
Use(Default_attributes)
Use(EzcurveDefaults)
Use(Ezcurve)

      call ezcseta(nargs, nargs1, LOCAL)
      if (nargs1 .eq.0) then
         Point(laser) = loc(laser1)
         laser1 = 1
         ezcnlaser = 1
         call allot("ezc.ezclaser", 0)
         ezclaser(1) = laser(1)

      elseif (nargs1 .eq. 1) then
         call fcnarga(1, sl)
         if (parcoerc(sl, typeint).eq.ERR) go to 900
         Point(laser) = paraddr(sl)
         ezcnlaser = parlen(sl)
         call allot("ezc.ezclaser", 0)
         do i = 1, ezcnlaser
            ezclaser(i) = laser(i)
         enddo

      else
         go to 900
      endif

c ... add this to the collection
      if (.not.exists("rayppow")) then	#for backwards compatible
	 raypow = "global real8 rayppow(length(raypr))=1."
	 call parsestr(raypow)
      endif

      raycall = "call ezcaddr(ezc.ezclaser, ezc.ezcnlaser, raypr, raypz, "//\
	        "rayppow, jrp, "//\
                "kptryp, ndx, mxp, idmpr, rypltf, irayl, irayu, nsray)"
      call parsestr(raycall)
      if (nargs1 .ne. 0) call parkilla(1)
      call basfree("ezc.ezclaser")

      call ezcrda
      return(OK)

900   continue
      call parkilla(1)
      call basfree("ezc.ezclaser")

      call ezcrda
      call ezcremrk("plotr: data not suitable, ignored.")
      call ezcremrk(legenda)

      return(OK)
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EZCPLOTV - vector plotter

      integer function ezcplotv(nargs, nargs1)
      integer nargs, nargs1

      integer sy(SS_WIDTH), sx(SS_WIDTH), 
        sxv(SS_WIDTH), syv(SS_WIDTH) , sireg(SS_WIDTH), m, kmax, lmax

      Dynamic(x, real(Size8), 1)
      Dynamic(y, real(Size8), 1)
      Dynamic(xv, real(Size8), 1)
      Dynamic(yv, real(Size8), 1)
      Dynamic(ireg, integer, 1)
      character*80 msg

      Address  paraddr
      external paraddr
      integer  parcoerc, parlen
      external parcoerc, parlen

Use(Keywords)      
Use(Default_attributes)
Use(Collections)
Use(EzcurveDefaults)

      call ezcseta(nargs, nargs1, LOCAL)
      if (nargs1 .gt. 5) then
         write(msg, 100) nargs1
         call ezcremrk(msg)
100   format('plotv: there must be 0-5 numerical arguments, not', i5)
         go to 900
      endif

      call fcnargd(nargs1, 1, sx, ezcx)
      call fcnargd(nargs1, 2, sy, ezcy)
      call fcnargd(nargs1, 3, sxv, ezcxv)
      call fcnargd(nargs1, 4, syv, ezcyv)
      call fcnargd(nargs1, 5, sireg, ezcireg)

      call pardim(sy) 
      if (parcoerc(sy, type8).eq.ERR) go to 900
      Point(y) = paraddr(sy)

      call pardim(sx) 
      if (parcoerc(sx, type8).eq.ERR) go to 900
      Point(x) = paraddr(sx)

      call pardim(syv) 
      if (parcoerc(syv, type8).eq.ERR) go to 900
      Point(yv) = paraddr(syv)

      call pardim(sxv) 
      if (parcoerc(sxv, type8).eq.ERR) go to 900
      Point(xv) = paraddr(sxv)

c      call pardim(sireg)   #Don't squeeze out dimension 1.
      if (parcoerc(sireg, typeint).eq.ERR) go to 900

      Point(ireg) = paraddr(sireg)

      m = parlen(sy)
      if (m .ne. parlen(sx)) then
          call ezcremrk('plotv: length of x does not match that of y')
          go to 900
      endif

      if (m .ne. parlen(syv)) then
          call ezcremrk('plotv: length of yv does not match that of y')
          go to 900
      endif

      if (m .ne. parlen(sxv)) then
          call ezcremrk('plotv: length of xv does not match that of y')
          go to 900
      endif

      if (sireg(SS_N) .eq. 2) then
         kmax = sireg(SS_N1)
         lmax = sireg(SS_N1+1)

      elseif (sireg(SS_N) .eq. 1) then
         kmax = 1
         lmax = sireg(SS_N1)

      else
         kmax = 0
         lmax = 0
      endif

      if (m .ne. kmax*lmax) then
          call ezcremrk('plotv: size of ireg does not match that of y')
          go to 900
      endif

c ... add this to the collection
      call ezcaddv(x, y, xv, yv, ireg, kmax, lmax)
      call parkill(sy)
      call parkill(sx)
      call parkill(syv)
      call parkill(sxv)
      call parkill(sireg) # ZCM 11/14/95 this was ireg for years

800   continue
      call ezcrda
      return(OK)

900   continue
      call ezcrda
      call ezcremrk(legenda)
      call ezcremrk("plotv command ignored.")

      call parkill(sy)
      call parkill(sx)
      call parkill(syv)
      call parkill(sxv)
      call parkill(sireg) # ZCM 11/14/95 this was ireg for years

      return(OK)
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EZCPLOTZ - another contour plotter

      integer function ezcplotz(nargs, nargs1)
      integer nargs, nargs1

      integer sy(SS_WIDTH), sx(SS_WIDTH), 
         sz(SS_WIDTH), nz1, nz2, nx1, nx2, ny1, ny2

      integer utsizeof
      external utsizeof

      Use(Keywords)
      Use(Collections)
      Use(Default_attributes)
      Use(EzcurveDefaults)

      Dynamic(z, real(Size8), 1)
      Dynamic(y, real(Size8), 1)
      Dynamic(x, real(Size8), 1)
      character*80 msg

      Address  paraddr
      external paraddr
      integer  parcoerc, parlen
      external parcoerc, parlen

      call ezcseta(nargs, nargs1, LOCAL)

      call fcnarga(1, sz)
      call fcnarga(2, sx)
      call fcnarga(3, sy)

      call pardim(sz)
      call pardim(sy)
      call pardim(sx)

      if (nargs1 .gt. 3) then
         write(msg, 100) nargs1
         call ezcremrk(msg)
100   format('plotz: there must be 0-3 numerical arguments, not', i5)
         go to 900
      endif
      if (parcoerc(sz, type8).eq.ERR) go to 900
      Point(z) = paraddr(sz)

      if (parcoerc(sx, type8).eq.ERR) go to 900
      Point(x) = paraddr(sx)

      if (parcoerc(sy, type8).eq.ERR) go to 900
      Point(y) = paraddr(sy)

      if (parlen(sx) <=1 | parlen(sy) <= 1 | parlen(sz) <= 1) then
         call ezcremrk('plotz: Some input has length <=1.')
         go to 900
      endif

      nx1 = sx(SS_N1)
      if (sx(SS_N) .eq. 1) then
         nx2 = 1
      else
         nx2 = sx(SS_N1+1)
      endif

      ny1 = sy(SS_N1)
      if (sy(SS_N) .eq. 1) then
         ny2 = 1
      else
         ny2 = sy(SS_N1+1)
      endif

      nz1 = sz(SS_N1)
      if (sz(SS_N) .eq. 1) then
         nz2 = 1
      else
         nz2 = sz(SS_N1+1)
      endif

      if ((nz1 .eq. nx1) .and. (nz2 .eq. nx2) .and.
     &    (nz1 .eq. ny1) .and. (nz2 .eq. ny2)) then
      elseif ((nz1 .eq. nx1) .and. (nz2 .eq. ny1) .and.
     &        (nx2 .eq. 1) .and. (ny2 .eq. 1)) then
      elseif ((nz1*nz2 .eq. nx1*nx2) .and. (nz1*nz2 .eq. ny1*ny2)) then
      else
         call ezcremrk("plotz: input sizes don't match.")
         write(msg, 200) nz1, nz2, nx1, nx2, ny1, ny2
         call ezcremrk(msg)
  200    format(6i6)
         go to 900
      endif

      call ezcaddz(z, nz1, nz2, x, nx1, nx2, y, ny1, ny2)
      call parkill(sz)
      call parkill(sy)
      call parkill(sx)
      call ezcrda
      return(OK)

  900 continue
      call parkill(sz)
      call parkill(sy)
      call parkill(sx)

      call ezcrda
      call ezcremrk("plotz command ignored.")
      call ezcremrk(legenda)

      return(OK)
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EZCTEXT - text command handler

      integer function ezctext(nargs,nargs1)
      integer nargs, nargs1

      integer sy(SS_WIDTH), sx(SS_WIDTH), sang(SS_WIDTH), snsize(SS_WIDTH),
        stext(SS_WIDTH),ncs, salg(SS_WIDTH)
      Dynamic(x,real(Size8))
      Dynamic(y,real(Size8))
      Dynamic(angle,real(Size8))
      Dynamic(align,real(Size8))
      Dynamic(nsize,integer)
      integer ns
      real(Size8) ang,algn
      Address sget
      character*(LSIZE) s
      character*80 msg

      Address  paraddr
      external paraddr
      integer  parcoerc, utgetcl
      external parcoerc, utgetcl

Use(Keywords)      
Use(Default_attributes)
Use(Collections)

      call pltstart
      call ezcseta(nargs,nargs1,LOCAL)
      if (nargs1 .gt. 6) then

         write(msg,100) nargs1
         call ezcremrk(msg)
100   format('text: there must be 0-6 numerical arguments, not',i5)
         go to 900
      endif

      call fcnarga(1,stext)
      call fcnarga(2,sx)
      call fcnarga(3,sy)
      ncs = -stext(SS_TC)
      if(ncs <=0) then
         s = symbols(1)
         ncs=utgetcl(s)
      else
         sget = paraddr(stext)
         call smovatof (s, 0, sget, 0, min (ncs, len (s)))
         if (ncs < len (s)) s (ncs:) = " "
      endif

      if(parcoerc(sx,type8)==ERR) go to 900
      Point(x) = paraddr(sx)

      if(parcoerc(sy,type8)==ERR) go to 900
      Point(y) = paraddr(sy)

      if(nargs1.ge.4) then
         call fcnarga(4,snsize)
         if(snsize(SS_TC)==0) then
            call ezciminsz(ns)
         else
            if(parcoerc(snsize,typeint)==ERR) go to 900
            Point(nsize) = paraddr(snsize)
            ns = nsize
            call parkill(snsize)
         endif
      else
         call ezciminsz(ns)
      endif
      if(nargs1.ge.5) then
         call fcnarga(5,sang)
         if(sang(SS_TC)==0) then
            ang = 0._Size8
         else
            if(parcoerc(sang,type8)==ERR) go to 900
            Point(angle) = paraddr(sang)
            ang = angle
            call parkill(sang)
         endif
      else
         ang = 0._Size8
      endif
      if(nargs1.ge.6) then
         call fcnarga(6,salg)
         if(salg(SS_TC)==0) then
            algn = -1._Size8
         else
            if(parcoerc(salg,type8)==ERR) go to 900
            Point(align) = paraddr(salg)
            algn = align
            call parkill(salg)
         endif
      else
         algn = -1._Size8
      endif

c ... add this to the collection
      call ezcaddt(s(1:ncs),x,y,ns,ang,algn)

      call parkill(stext)
      call parkill(sx)
      call parkill(sy)
800   continue

      call ezcrda
      return(OK)

900   continue
      call ezcrda

      call parkill(stext)
      call parkill(sx)
      call parkill(sy)
      call parkill(snsize)
      call parkill(sang)
      call parkill(salg)
      call ezcremrk("text command ignored.")

      return(OK)
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EZCFTEXT - ftext command handler

      integer function ezcftext(nargs, nargs1)
      integer nargs, nargs1

      integer sy(SS_WIDTH), sx(SS_WIDTH), sang(SS_WIDTH), snsize(SS_WIDTH), 
              stext(SS_WIDTH), ncs, salg(SS_WIDTH)
      Dynamic(x, real(Size8))
      Dynamic(y, real(Size8))
      Dynamic(angle, real(Size8))
      Dynamic(align, real(Size8))
      Dynamic(nsize, integer)
      integer ns
      real(Size8) ang, algn
      Address sget
      character*(LSIZE) s
      character*80 msg

      Address  paraddr
      external paraddr
      integer  parcoerc, utgetcl
      external parcoerc, utgetcl

Use(Keywords)      
Use(Default_attributes)
Use(Collections)

      call pltstart
      call ezcseta(nargs, nargs1, LOCAL)
      if (nargs1 .gt. 6) then
         write(msg, 100) nargs1
         call ezcremrk(msg)
 100	 format('ftext: there must be 0-6 numerical arguments, not', i5)
         go to 900
      endif

      call fcnarga(1, stext)
      call fcnarga(2, sx)
      call fcnarga(3, sy)
      ncs = -stext(SS_TC)
      if (ncs <=0) then
         s = symbols(1)
         ncs=utgetcl(s)
      else
         sget = paraddr(stext)
         call smovatof (s, 0, sget, 0, min (ncs, len (s)))
         if (ncs < len (s)) s (ncs:) = " "
      endif

      if (parcoerc(sx, type8).eq.ERR) go to 900
      Point(x) = paraddr(sx)

      if (parcoerc(sy, type8).eq.ERR) go to 900
      Point(y) = paraddr(sy)

      if (nargs1.ge.4) then
         call fcnarga(4, snsize)
         if (snsize(SS_TC).eq.0) then
            call ezciminsz(ns)
         else
            if (parcoerc(snsize, typeint).eq.ERR) go to 900
            Point(nsize) = paraddr(snsize)
            ns = nsize
            call parkill(snsize)
         endif
      else
         call ezciminsz(ns)
      endif
      if (nargs1.ge.5) then
         call fcnarga(5, sang)
         if (sang(SS_TC).eq.0) then
            ang = 0._Size8
         else
            if (parcoerc(sang, type8).eq.ERR) go to 900
            Point(angle) = paraddr(sang)
            ang = angle
            call parkill(sang)
         endif
      else
         ang = 0._Size8
      endif
      if (nargs1.ge.6) then
         call fcnarga(6, salg)
         if (salg(SS_TC).eq.0) then
            algn = -1._Size8
         else
            if (parcoerc(salg, type8).eq.ERR) go to 900
            Point(align) = paraddr(salg)
            algn = align
            call parkill(salg)
         endif
      else
         algn = -1._Size8
      endif

c ... add this to the collection
      call ezcaddft(s(1:ncs), x, y, ns, ang, algn)
      call parkill(stext)
      call parkill(sx)
      call parkill(sy)

800   continue
      call ezcrda
      return(OK)

900   continue
      call ezcrda
      call parkill(stext)
      call parkill(sx)
      call parkill(sy)
      call parkill(snsize)
      call parkill(sang)
      call parkill(salg)
      call ezcremrk("ftext command ignored.")

      return(OK)
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EZCTITLS - 

        subroutine ezctitls(t1,t2,t3,t4)
        character*(*) t1,t2,t3,t4

Use(Ezcurve)
Use(Default_attributes)

        integer utgetcl
        external utgetcl

        titlet = t1
        titleb = t2
        titlel = t3
        titler = t4
        pending = TRUE

        if (ezcshow) call ezcshowf

        return
        end

c--------------------------------------------------------------------------

c                          HELPERS

c--------------------------------------------------------------------------

c EZCTOPT - 

      subroutine ezctopt(pq, pqp, ireg, kmax, lmax)
      integer kmax, lmax, ireg(kmax, lmax)
      real(Size8) pq(kmax, lmax), pqp(kmax, lmax)

      integer k, l

Use(Collections)

      Dynamic(wtsum, integer, [kmax, lmax])

      call memtemp(Point(wtsum), Quote(integer), kmax*lmax,
     &             Quote(EZCOPT:wtsum ))

      do k = 1, kmax
         do l = 1, lmax
            pqp(k, l)  =  0._Size8
            wtsum(k, l) = 0
         enddo
      enddo

      do k = 2, kmax
         do l = 2, lmax
            if (ireg(k, l) .eq. 0) next
            pqp(k, l)       = pqp(k, l)+pq(k, l)
            wtsum(k, l)     = wtsum(k, l)+1
            pqp(k-1, l)     = pqp(k-1, l)+pq(k, l)
            wtsum(k-1, l)   = wtsum(k-1, l)+1
            pqp(k, l-1)     = pqp(k, l-1)+pq(k, l)
            wtsum(k, l-1)   = wtsum(k, l-1)+1
            pqp(k-1, l-1)   = pqp(k-1, l-1)+pq(k, l)
            wtsum(k-1, l-1) = wtsum(k-1, l-1)+1
         enddo
      enddo
 
      do k = 1, kmax
         do l = 1, lmax
            pqp(k, l) = pqp(k, l)/(wtsum(k, l)+1.e-100_Size8)
         enddo
      enddo

      call memfree(Point(wtsum))

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EZCRDA - restore the default style attributes

      subroutine ezcrda

Use(Saved_attributes)
Use(Default_attributes)

      integer i

      fonta = fonts
      stylea	 = styles
      thicka	 = thicks
      arrowa	 = arrows
      marka	 = marks
      marksa = markss

      do i = 1, 3
        krangea(i) = kranges(i)
        lrangea(i) = lranges(i)
      enddo

      kstylea	 = kstyles
      lstylea	 = lstyles
      kcolora = kcolors
      lcolora = lcolors

      do i = 1, MAXREG
         excludea(i) = excludes(i)
      enddo

      bnda	 = bnds
#      zfactora	 = zfactors
#      zcentera	 = zcenters
      vscalea	 = vscales
      colora = colors
      cscalea = cscales
      centera = centers
      nlevela = nlevels
      levela = levels
      rsqa = rsqs

#optional items
      labela = labels
      zlimin(1) = zlimins(1)
      zlimin(2) = zlimins(2)
      nzlima = nzlims

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

