/*
 * Basis.h - C include file for C programs that want to interact with Basis
 *
 */

#ifndef HDR_BASIS
#define HDR_BASIS

#include <basconf.h>
#include <Fcc.h>

/* A few constants that may have to be coordinated with mppl.BASIS etc. */

#if defined(C90)|defined(CRAY2)|defined(J90)|defined(NLTSS)|defined(UNICOS)\
    |defined(XMP)|defined(YMP)
# define LOCS_PER_WORD (1)
#else
# define LOCS_PER_WORD (4)
#endif

#if defined(AIX)|defined(AXP)|defined(CS2)|defined(DEC3100)|defined(HP700)\
      |defined(HPUX)|defined(IRIX64)|defined(LINUX)|defined(MIPS)|defined(RS6000)\
      |defined(SGI)|defined(SOL)|defined(SUN4)|defined(SUNOS)|defined(ULTRIX)\
      |defined(VP2000)
#   define HAS_DOUBLE_COMPLEX
#else
#   undef HAS_DOUBLE_COMPLEX
#endif

#if defined(GENERIC)|defined(HP700)|defined(HPUX)
#   define STDERR 7
#else
#if defined(NLTSS)
#   define STDERR 6
#else
#   define STDERR 0
#endif
#endif

/* The following has to agree with mppl.BASIS: */
#define OK 0
#define ERR -1
#define DONE 1
#define SLEEPING 0
#define GENERATE 1
#define UP 2
#define FINISHED 3

/* mangle the name Basmain and others as appropriate for platforms */
#if defined(CRAY2) || defined(XMP) || defined(YMP) || defined(C90) || defined(J90)
#   define Remark RemarkCray
#   define BasError BasErrCray
#   undef WORKSTATION
#else
#   define Remark   FF_ID(remark, REMARK)
#   define BasError FF_ID(baserror, BASERROR)
#   define WORKSTATION
#endif

#define Usrmain   FF_ID(usrmain, USRMAIN)
#define Gchange   FF_ID(gchange, GCHANGE)
#define Fpullapt  FF_ID(fpullapt, FPULLAPT)
#define Glbxdb    FF_ID(glbxdb, GLBXDB)
#define Rtufnam   FF_ID(rtufnam, RTUFNAM)
#define Changej   FF_ID(changej, CHANGEJ)
#define Getnumf   FF_ID(getnumf, GETNUMF)
#define Glbninit  FF_ID(glbninit, GLBNINIT)
#define Glbfind   FF_ID(fglbfind, FGLBFIND)
#define Parsestr  FF_ID(parsestr_, PARSESTR_)
#define Parasgn   FF_ID(parasgn, PARASGN)
#define Baspline  FF_ID(baspline, BASPLINE)
#define Cpow      FF_ID(cpow, CPOW)
#define Dcpow     FF_ID(dcpow, DCPOW)
#define Baderr    FF_ID(baderr, BADERR)
#define Sgesv     FF_ID(sgesv, SGESV)
#define Sgesv8    FF_ID(sgesv8, SGESV8)
#define Mcnest    FF_ID(mcnest, MCNEST)
#define Mcunnest  FF_ID(mcunnest, MCUNNEST)
#define Parfetch  FF_ID(parfetch, PARFETCH)
#define Lrxact    FF_ID(lrxact, LRXACT)
#define Lrsize    FF_ID(lrsize, LRSIZE)
#define Parcopy   FF_ID(parcopy, PARCOPY)
#define Mcname    FF_ID(mcname, MCNAME)
#define Lrvlup    FF_ID(lrvlup, LRVLUP)
#define Lxvcons   FF_ID(lxvcons, LXVCONS)
#define Basclose  FF_ID(basclose, BASCLOSE)
#define Baswline  FF_ID(baswline, BASWLINE)
#define Basnxtsq  FF_ID(basnxtsq, BASNXTSQ)
#define Ffopen    FF_ID(ffopen, FFOPEN)
#define Lxgptr    FF_ID(lxgptr, LXGPTR)
#define Mcnum     FF_ID(mcnum, MCNUM)
#define Osprompt  FF_ID(osprompt, OSPROMPT)
#define Utsizeof  FF_ID(utsizeof, UTSIZEOF)
#define Parint    FF_ID(parint, PARINT)
#define Parreal   FF_ID(parreal, PARREAL)
#define Pardble   FF_ID(pardble, PARDBLE)
#define Parsubc   FF_ID(parsubc, PARSUBC)
#define Parcoerc  FF_ID(parcoerc, PARCOERC)
#define Parcfcn1  FF_ID(parcfcn1, PARCFCN1)
#define Pardim    FF_ID(pardim, PARDIM)
#ifdef HAS_DOUBLE_COMPLEX
#define Prdcfcn1  FF_ID(prdcfcn1, PRDCFCN1)
#endif
#define WORKSTATION
#define Sorti     FF_ID(sorti, SORTI)
#define Pullapt2  FF_ID(pullapt2, PULLAPT2)
#define Parmdexpn FF_ID(parmdexpn, PARMDEXPN)
#define Basloc    FF_ID(basloc, BASLOC)
#define Basnxtsq  FF_ID(basnxtsq, BASNXTSQ)
#define Partrans  FF_ID(partrans, PARTRANS)
#define Mcsdlim   FF_ID(mcsdlim, MCSDLIM)
#define Mcsdlimn  FF_ID(mcsdlimn, MCSDLIMN)
#define Fcndomax  FF_ID(fcndomax, FCNDOMAX)
#define Fcndomin  FF_ID(fcndomin, FCNDOMIN)
#define Prratan2  FF_ID(prratan2, PRRATAN2)
#define Prdatan2  FF_ID(prdatan2, PRDATAN2)
#define Freeout   FF_ID(freeout, FREEOUT)
#define Glbfin    FF_ID(glbfin, GLBFIN)

extern void Usrmain(const char *,const char *, const Integer, const Integer);
extern Integer Gchange(const char *, const Integer, const Integer);
extern void Remark(const char *, const Integer);
extern void BasError(const char *, const Integer);
extern integer Fpullapt(const Integer *, const Integer *, const Integer *, 
                        Integer *, Integer *, Integer *, Integer *);
extern void Glbxdb(const Integer *, const Integer *, Address *, Integer *,
                   Integer *, Integer *, Integer *);
extern void Rtufnam(Address *);
extern Integer Changej(Integer *, Integer *, Integer *);
extern Integer Getnumf(void);
extern Integer Glbfind(Integer *, Integer *, Integer *);
extern void Glbninit(Integer *);
extern void Parsestr(Address *, Integer *);
extern Integer Parasgn(Integer *, Integer *);
extern void Baspline(Integer *, char *, Integer);
extern Integer Cpow(float *c, float *a, float *b, Integer *n);
extern Integer Dcpow(double *c, double *a, double *b, Integer *n);
extern void Baderr(const char *, const Integer);
extern void Sgesv(Integer *, Integer *, float *, Integer *, Integer *,
                  float *, Integer *, Integer *);
extern void Sgesv8(Integer *, Integer *, Real8 *, Integer *, Integer *,
                   Real8 *, Integer *, Integer *);
extern void Mcnest(void);
extern void Mcunnest(void);
extern void Parfetch(Integer *, char *, Integer *, Integer*, Integer *,
                     Integer);
extern void Lrxact(Integer *, Integer *);
extern Integer Lrsize(Integer *);
extern void Parcopy(Integer *, Integer *);
extern Integer Mcname(void);
extern void Lrvlup(Integer *, char *, Integer *);
extern Integer Lxvcons(Integer *, char *, Integer *, Integer *);
extern void Ruthere(void);
extern void Basclose(Integer *);
extern void Baswline(Integer *, char *, Integer);
extern void Basnxtsq(char *, char *, Integer, Integer);
extern void Ffopen(Integer *);
extern void Lxgptr(Integer *);
extern Integer Mcnum(void);
extern void Osprompt(char *, Integer);
extern Integer Utsizeof(Integer *, Integer *);
extern Integer Parint(Integer *, Integer *);
extern Integer Parreal(Integer *, float *);
extern Integer Pardble(Integer *, real8 *);
extern void Parsubc(Integer *);
extern Integer Parcoerc(Integer *, Integer *);

extern void Pardim(Integer *);
extern Integer Fcndomax(Integer *, Integer *);
extern Integer Fcndomin(Integer *, Integer *);
extern Integer Prratan2(float *rx, float *ry, float *rvalue, Integer *nx,
                        Integer *ny, Integer *n);
extern Integer Prdatan2(real8 *dx, real8 *dy, real8 *dvalue, Integer *nx,
                        Integer *ny, Integer *n);
extern void Mcsdlim(char *mcudlim, Integer *i, Integer len);
extern void Mcsdlimn(Integer *nargs);
extern void Partrans(Integer *sxi);
extern void Basnxtsq(char *f, char *g, Integer lenf, Integer leng);
extern Address Basloc(char *vname, Integer namelen);
extern Integer Parmdexpn(Integer *tc, Integer *mdc, Integer *mdp,
                         Integer *mdv, Integer *sxi);
extern void Pullapt2(Integer *ndb, char *dimstr, Integer *jvar,
                     Integer *nlow, Integer *nhi, Integer *ndim,
                     Integer *ierr, Integer len);
extern void Sorti(Integer *isortary, Integer *isortidx, Integer *len);

extern void Freeout(void);
extern Integer Glbfin(Integer *, Integer *, Integer *);

#endif /* HDR_BASIS */
