# Basis Code Development System
# Copyright 1986 The Regents of the University of California
# All rights reserved

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PARASGN - perform assignment of stack element s to var t
c         -    si, ti, stack elements describing source, target
c         -    ti is of type LHS, we hope
c         -    si is an exp
c
c       OUTLINE
c 1. if the target is a non-subscripted undeclared variable, and the
c    flag autovar is on, then build a database entry for it and assign
c    it the size and type of the source.
c 2. If the target is an non-subscripted register variable, create it at
c    the size and type of the source.
c 3. Get the attributes of the target
c 4. Call parasgn2 to finish the job

      integer function parasgn(ti, si)

Use(Actions)
Use(Stacks)
Use(Controls)
Use(Functions)

      integer ndb, opasgn, si(SSWIDTH), ti(SSWIDTH), actstr, rtgnastr
      integer tc,ndim,ilow(7),ihi(7),icol(7)
      Address fwa
      Varname name
      character*DIMSTRSIZE action
      logical rtischam
      integer parname, parasgn2, utgetcl, naml
      external parname, rtischam,parasgn2, utgetcl, rtgnastr
      
#0. get the source
      call parfetch(si,nxp(rsp),lxp(rsp),COPY,FALSE)  #need the source now
      entry opasgn(ti, si)

#0. fix to avoid blowup on multiply-subscripted <lhs>'s
      if (ti(TC) ~= LHS) then
         call remark("parasgn: incorrect <lhs> for assignment.")
         go to 900
      endif

#0.5 check the source
      if (si(TC) == FUNCTION_TYPE) then
         call remark("parasgn: function name on rhs of assignment.")
         go to 900
      endif

#1. autovar check 
      call parautov(ti,si)
      jvar = ti(PTR)
      tc   = ti(TCSAVE)
      ndb  = ti(N)

# 2. If the target is a non-subscipted chameleon variable, create it at
#    the size and type of the source.
      if (ti(NS) == 0) then
         if (rtischam(jvar,ndb)) then
            call parendow(jvar,ndb,si)
            tc = si(TC)
         endif
      endif

# 3. Get the other attributes of the target -- tc already set
# if asgnchk is YES, use LIMITED; if NO, use FULL.
# For the following to work, need FULL=LIMITED+1
      call rtxdb(jvar, ndb, fwa, ndim, ilow, ihi ,icol,FULL-asgnchek)
      if (partrace &(memtrace .eq. YES)) then
         write(6, 90909) fwa
90909    format("Target of assignment: ", i10)
      endif

      if (fwa == NOTSET) then
         call remark(
         "Target of assignment is dynamic variable with unset pointer.")
         go to 900
      endif

# 4. call parasgn2 to finish the job -- pardodi goes right to parasgn2
      if (parasgn2(tc,fwa,ndim,ilow,ihi,icol,ti(NS),ti(N1),ti(M1),ti(I1),si)
     &     = ERR) go to 900

      if (semerr == OK) then
         inassign = FALSE           # done with assignment
         assignsp = 0
         symbolic = TRUE
      endif

      if (jvar <= 0) return(OK)  #no foreign
      actstr = rtgnastr( jvar , ndb )
      
      if (actstr <> 0) then
         call strgget(actstr,action)
         call parsestr(action)
      endif
      return(OK)

c ... error, print name of target
        
 900  continue
      if (ti(TC) == LHS | ti(TC) == NAME | ti(TC) == FUNCTION_TYPE | 
     &     ti(TC) == NULL_TYPE | ti(TC) < 0) then
         if (ti(PTR) >= 0) then
            naml = parname(ti,name)
            call baserror("parasgn: error in assignment to variable "// \
            "named '" // name(1:naml) // "'.")
         else
            call baserror("parasgn: attempted assignment to foreign "// \
            "variable.")
            call baserror("         pdb variables are read-only.")
         endif
      else
         call remark("parasgn: attempted assignment to expression, "// \
         "not variable.")
      endif
      
      return(ERR)
      end
      
c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PARASGN2 - to perform broadcast assignment of scalars to subscripted arguments
c          - OUTLINE
c          - 1. Get the true shape of the source; if of zero length return
c          - 2. Check the target subscripts for appropriateness
c          - 3. Check the shapes of source and target.
c          - 4. Coerce the source to the target's type.
c          - 5. Store the values
c          - 6. Release storage for source
c
c          -       tc        = type of target
c          -       fwa       = first word address of target
c          -       ndim      = number of dimensions of target
c          -       ilow, ihi = arrays of low, hi subscripts for target
c          -                   target column lengths in memory
c          -       ns        = number of subscripts supplied on target
c          -       tl, th    = subscripts
c          -       ti        = subscript increment
c          -       si is an exp
c          -       n = number of dimensions of source

      integer function parasgn2(tc, fwa, ndim, ilow, ihi, icol, ns, tl, th, ti, si)

Use(Stacks)

      character*120 errmsg, errm1, errm2
      integer si(SSWIDTH)
      integer tc,ndim,ns,tl(7),th(7),ti(7),ilow(7),ihi(7),icol(7)
      Address fwa # ZCM 10/3/94
      integer cl(7)
      integer nns,nnt,strue(7),ttrue(7),itrue(7),jtrue(7)
      logical scalar
      integer parcoerc,parstore, parstors
      Address paraddr
      external parcoerc,parstore, parstors, paraddr
      Address subarr(7)
      integer lsubarr(7), imin, imax
integer i, j, k, n
      
# 1. Get the true shape of the source; if of zero length return
      n   = si(N)
      nns = 0
      do i = 1, n
         j = si(N1-1+i)
         if (j .gt. 1) then
            nns = nns + 1
            strue(nns) = j

# jtrue notes the index in si corresponding to the i'th true dimension
            jtrue(nns) = i

         elseif (j .eq. 0) then
#GOTCHA: must release bassubs space before returning too (else it can
#GOTCHA: grow when functions like "where" evaluate to 0-shape for the source)
# First get rid of any array subscripts
            do k = 1, ndim
               if (tl(k) == DEFAULT + 1) then
               isub = th(k)
               bassubs(N1 + 1, isub) = bassubs(N1 + 1, isub) - 1
                  if (bassubs(N1 + 1, isub) == 0) then
# reference count reached 0, so release
                     call parrel(bassubs(1, isub))
                     bassubs(TC, isub) = NULL_TYPE
                  endif
               endif
           enddo
#Then get rid of the source
            call parkill(si)
            return(OK)  #source has length 0, do nothing
         endif
      enddo

      scalar = nns .eq. 0

# Get addresses of subscript arrays for parstore and parstors, if any
      do i = 1, 7
         if ((i .le. ns) .and.(tl(i) .eq. DEFAULT + 1)) then
            subarr(i)  = paraddr(bassubs(1, th(i)))
            lsubarr(i) = bassubs(N1, th(i))
         else
            subarr(i)  = 0
            lsubarr(i) = 0
         endif
      enddo

# 2. Check the target subscripts for appropriateness
      if ((ns .ne. 0) &(ns .gt. ndim)) then
         if ((ns .eq. 1) &(ndim .eq. 0)) then
            ndim    = 1
            ilow(1) = 1
            ihi(1)  = 1
            icol(1) = 1
         else
            call remark("parasgn: too many subscripts on target ")
            call remark("of assignment statement.")
            return(ERR)
         endif
      endif

# fill out unspecified subscripts
      do i= ns + 1, ndim
         tl(i) = ilow(i)
         th(i) = ihi(i)
         ti(i) = 1
      enddo

# take care of DEFAULTs and bounds checks
      do i = 1, ndim
         if (lsubarr(i) ~= 0) then

# make sure array subscripts are in range.
            call iaminmax(subarr(i), lsubarr(i), imin, imax)

            if (imin < ilow(i) | imin > ihi(i)) then
               call utsprint(1, 1, 120, errmsg,
     &         67, "parasgn: subscript %d(min. value %d) is out of the range %d to %d.",
     &         i, imin, ilow(i), ihi(i))
               call remark(errmsg)
               return(ERR)
            endif

            if (imax < ilow(i) | imax > ihi(i)) then
               call utsprint(1, 1, 120, errmsg,
     &         67, "parasgn: subscript %d(max. value %d) is out of the range %d to %d.",
     &         i, imax, ilow(i), ihi(i))
               call remark(errmsg)
               return(ERR)
            endif

         else
            if (tl(i) == DEFAULT) tl(i) = ilow(i)
            if (th(i) == DEFAULT) th(i) = ihi(i)
            if (ti(i) == DEFAULT) ti(i) = 1
            
            if (tl(i)*ti(i) > th(i)*ti(i)) then
               call utsprint(1, 1, 120, errmsg,
     &              57, "parasgn: assignment target %d has lower %d * %d > %d * %d",
     &              i, tl(i), ti(i), th(i), ti(i))
               call remark(errmsg)
               return(ERR)
            endif

            if (ti(i) > 0) then
               if (ilow(i) > tl(i) | ihi(i) < th(i)) then
                  call remark("Error in assignment --subscript is out of bounds")
                  return(ERR)
               endif

            else
               if (ilow(i) > th(i) | ihi(i) < tl(i)) then
                  call remark("Error in assignment --subscript is out of bounds")
                  return(ERR)
               endif
            endif
         endif
      enddo

# 3. check that the shape of the subscripted target and the source
#      are compatible.
#        At this point the default subscripts have been
#        replaced and subscripts have been checked for low <=high, etc.
#        and the number of subscripts is on the target is ndim.
#
#        If the true dimension of the source is less than that of the
#        target, adjust subscript ranges on the target to default them
#        to their lower bound.
#
#        If the column lengths of the source are less than that of the
#        target, adjust the subscript upper bounds on the target.
#
#      (ZCM 9/1/1999) In the case where a subscript is an arbitrary
#        array, we try to do as close to the above as possible.
#
# calculate column lengths of t
      nnt = 0
      do i=1,ndim
         if (lsubarr(i) == 0) then
            cl(i) = (th(i) - tl(i))/ti(i) + 1
         else
            cl(i) = lsubarr(i)
         endif
         if (cl(i) > 1) then
            nnt = nnt + 1
            ttrue(nnt) = cl(i)

# itrue notes the index in t corresponding to the i'th true dimension
            itrue(nnt) = i
         endif
      enddo

c ... check shapes
        if (~ scalar) then

# conform the dimensions of target DOWN to dimensions of source UNLESS
# source is a scalar. --ZCM 8/88
           if (nns > nnt)  go to 900
           do i=1, nns   # note that we only check the nns ones.
              
# if the target is smaller than the source, we
# truly don't know what part of the source to assign.
              if (ttrue(i) < strue(i)) then
                 go to 900
              else

# set target upper subscript to match length of source
                 if (lsubarr(itrue(i)) == 0) then
                    th(itrue(i)) = tl(itrue(i))  +
                  (si(N1-1+jtrue(i))-1)*ti(itrue(i))

                 else
# Only use the first strue elements of the array
                    lsubarr(itrue(i)) = strue(i)
                 endif
              endif
           enddo

# for the extra dimensions of the target, set upper subscript to lower
           do i=nns+1,nnt
              if (lsubarr(itrue(i)) == 0) then
                 th(itrue(i)) = tl(itrue(i))
                 ti(itrue(i)) = 1

              else
# Use only the first array element
                 lsubarr(itrue(i)) = 1
              endif
           enddo
        endif

# 4. Coerce the source to the target's type.
        if (parcoerc(si, tc) == ERR) return(ERR)

# 5. Store the values
        if (~ scalar)then
           if (parstore(fwa,tc,ndim,ilow,ihi,icol,tl,th,ti,si,\
           subarr, lsubarr)== ERR) return(ERR)

# separate routine for scalar broadcast
        else
           if (parstors(fwa,tc,ndim,ilow,ihi,icol,tl,th,ti,si,\
           subarr, lsubarr)== ERR) return(ERR)
        endif

# 6. release storage

# First get rid of any array subscripts
        do i = 1, nnt
           if (tl(i) == DEFAULT + 1) then
              isub = th(i)
              bassubs(N1 + 1, isub) = bassubs(N1 + 1, isub) - 1
              if (bassubs(N1 + 1, isub) == 0) then
# reference count reached 0, so release
                 call parrel(bassubs(1, isub))
                 bassubs(TC, isub) = NULL_TYPE
              endif
           endif
        enddo

# Next get rid of src
        call parkill(si)
        return(OK)

c ... error diagnostics
 900    continue
        call remark(
        "parasgn2: Shape mismatch between source and target in assignment or append.")

        call utsprint(1, 0, 120, errm1,
     &                51, "Right side(source) dimension = %d shape = ",
     &                nns)
        call utsprint(1, 0, 120, errm2,
     &                50, "Left side(target) dimension = %d shape = ",
     &                nnt)
        last = 0
        do i = 1, nns
           if (i .eq. nns) then
              last = 1
           endif
           call utsprint(0, last, 120, errm1,
     &                   3, " %d", strue(i))
        enddo

        last = 0
        do i = 1, nnt
           if (i .eq. nnt) then
              last = 1
           endif
           call utsprint(0, last, 120, errm2,
     &                   3, " %d", ttrue(i))
        enddo

        call baserror(errm1)
        call baserror(errm2)

        return(ERR)
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PAROPASN - Perform op(|, &, +, -, *, /, or **) on runtime stack
c          - elements at rsp-1 and rsp, then assign the result to
c          - the element at rsp-1. This implements the
c          - operator assignments |=, &=, +=, -=, *=, /=, and **=.
c          -   op, the operation to be performed

      integer function paropasn(op)

Use(Stacks)

      integer pardobin, parasgn, opasgn
      external pardobin, parasgn, parcopy, opasgn
      integer op, iop
      
      iop = op
        
# Make sure there's room on the stack
      if (rsp + 1 > SSLENGTH) then
         call baserror("paropasn: runtime stack overflow doing op=....possible runaway recursion?")
         call kaboom(1)
      endif

# copy the rhs stack elt to use as second source
      call parcopy(ss(1,rsp),ss(1,rsp+1))
      nxp(rsp + 1) = nxp(rsp)
      lxp(rsp + 1) = lxp(rsp)

# copy the lhs stack elt to use as first source
      call parcopy(ss(1,rsp-1),ss(1,rsp))
      nxp(rsp) = nxp(rsp-1)
      lxp(rsp) = lxp(rsp-1)
      do i = 1, ss(NS,rsp-1)
         if (ss(N1+i-1,rsp-1) == DEFAULT + 1) then

# increase reference count to bassubs
            bassubs(N1+1, ss(M1+i-1,rsp-1)) = bassubs(N1+1, ss(M1+i-1,rsp-1)) + 1
         endif
        enddo

# pardobin does a parfetch on si1, which will decrement
# the reference count by 1
        rsp = rsp + 1 # pardobin operates on the top two elements

# Fool pardobin into doing the right operation
        if (op == OREQ | op == ANDEQ) then
           iop = iop - 9
        else
           iop = iop - 19
        endif
        if (pardobin(iop) == ERR) return(ERR)
        rsp = rsp - 1 # done with second source
        
# ss(.,rsp) now contains the result which must be assigned to ss(.,rsp-1)
        if (opasgn(ss(1,rsp-1), ss(1,rsp)) == ERR) return(ERR)
        
# Note: opasgn releases bassubs if the lhs has an array index.
        
        return(OK)
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PARDODI - assign exp at stack position rsp to variable "$"
c         - Then print $ using nxp(rsp) as a comment

      integer function pardodi()

      integer len
      integer ilow(7), ihi(7), icol(7), ndim, t(SSWIDTH),tc
      Address fwa
      integer parasgn2, prtvar, getius, lnam
      external parasgn2, prtvar, getius
      Varname nam
      external rtgvname

Use(Controls)
Use(Stacks)

      lnam = len(nam)
      call parfetch(ss(1,rsp),nam,lnam,COPY,FALSE)
      tc = ss(TC,rsp)

# don't display a NULL_TYPE
      if (tc == NULL_TYPE) return(OK)

# assign matrix at rsp to variable $ in the par package ... or maybe, $a, etc.
      if (autohist == 0) then
         jvar = jbucks
      else
         jvar = jbucks + 1 + abs(mod(autohist-1,26))
         autohist = autohist + 1
      endif

      call parendow(jvar,jbucksp,ss(1,rsp)) #jbucksp is the parser package #

      tc = ss(TC,rsp)
      t(NS) = 0
      call rtxdb(jvar,jbucksp,fwa,ndim,ilow,ihi,icol,FULL)
      if (parasgn2(tc,fwa,ndim,ilow,ihi,icol,
     &             t(NS),t(N1),t(M1),t(I1),ss(1,rsp)) = ERR)
         return(ERR)

      call rtcomchv( jvar, jbucksp, nxp(rsp))
      call rtgvname(jvar, jbucksp, nam)

      return(prtvar(nam,jvar,jbucksp,ss(TC,rsp),ss(I1,rsp),getius()))
      end
      
c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PARENDOW - take the data base entry described by jvar and db and
c          - endow it with the same dimensions and type as s
c          -(used for assignments to chameleons and to undeclared
c          - variables when autovar is on)
c          -   jvar  index into the target database
c          -   db    the target database
c          -   s     stack element for the source

        subroutine parendow(jvar,ndb,s)
        integer ndb,jvar,s(*)

	call rtstc(jvar, ndb, s(TC))
        call rtstgget(jvar,ndb,s(N),s(N1),s(M1),0)

        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PARCOERC - coerce matrix described by stack element s to type newtype
c          -   si       stack element describing a matrix
c          -   newtype  type code of desired type

      function parcoerc(si,newtype)

      integer parcoerc,si(*),newtype,t(SSWIDTH)
      integer parlen,utgetcl,ccoerce
      Address paraddr
      external parlen, paraddr, utgetcl,ccoerce,smovatoa,smovftoa
      Address chs, cht
      Ch(MAXSTRING) blanks
      integer n,lx
      Address sloc # ZCM 3/7/95
      integer nt,nc,ns,os,ot
      character*TYPESIZE argtype,argtype1
      character*COMMENTSIZE nx

Use(Stacks)

      if (si(TC) == LHS)then
         lx = 18
         nx = "***NAME UNKNOWN***"
         call parfetch(si, nx, lx, ACTUAL,FALSE)
      endif

      if (si(TC) == FUNCTION_TYPE) then
         call remark('Function used without args but expects some.')
         call parxferr(si(N),si(N2),si(PTR),si(N2)) #never returns, error
      endif

      if (si(TC) == newtype) return(OK)

ifelse(HAS_DOUBLE_COMPLEX,NO,[
      # GOTCHA : mppl requires this to be on one line
      if ((si(TC) .eq. DOUBLE_C) .and. (newtype .eq. COMPLEX) .or. (si(TC) .eq. COMPLEX) .and. (newtype .eq. DOUBLE_C)) return(OK)
],[])

      if (si(TC) < 0 | newtype < 0) go to 500
      if (si(TC) > RANGE) go to 900

      call parcopy(si,t)
      n    = parlen(si)
      sloc = paraddr(si)

ifelse(HAS_DOUBLE_COMPLEX,NO,[
      if (newtype .eq. DOUBLE_C) newtype = COMPLEX
],[])

# should not call parget with newtype = FUNCTION_TYPE

      if (newtype .eq. FUNCTION_TYPE) go to 900

      t(TC) = newtype
      call parget(t)
      if (ccoerce(sloc, paraddr(t), n, si(TC), newtype) .eq. ERR) go to 900

      call parrel(si)
      call parcopy(t,si)
      return(OK)

# one or both is character
 500  continue
      if (si(TC) >=0 | newtype >=0) then
         call remark("parcoerc: can't mix character and other types.")
         return(ERR)
      endif

# both are character type
      blanks=" "    #a big array full of blanks
      call parcopy(si,t)
      t(TC) = newtype
      t(NAML) = 1 # since it's to be in the literal table(ZCM 4/10/95)
      call parget(t)
      chs= paraddr(si)
      cht= paraddr(t)
      ns=abs(si(TC))
      nt=abs(newtype)
      os=0
      ot=0
      n=parlen(si)
      nc = min(ns,nt)
      do i=1,n
         call smovatoa(cht,ot,chs,os,nc)
         if (ns < nt) then
            call smovftoa(cht,ot+ns,blanks,0,nt-ns)
         endif
         os=os+ns;ot=ot+nt
      enddo

      call parrel(si)
      call parcopy(t,si)

      return(OK)

 900  continue
      if (newtype .eq. FUNCTION_TYPE) then
         call remark(
         "Tried to use a function name without arguments in an expression")
         call remark("but the function requires arguments.")          
         return(ERR)
      endif

      call remark("parcoerc: Impossible conversion of types needed.")
      call utcodstr(si(TC),argtype)
      call utcodstr(newtype,argtype1)
      call remark("Trying to convert type "//argtype(1:utgetcl(argtype))// \
      " into type "//argtype1(1:utgetcl(argtype1))//".")

      return(ERR)
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PARFWA - calculate loc(x(tl(1),tl(2),...tl(ndim)) where fwa = loc(x)
c        - and ilow, ihi contain the upper and lower subscripts of x

        Address function parfwa(fwa,tc,ndim,ilow,ihi,cl,tl)
        Address fwa
        integer tc,ndim,ilow(7),ihi(7),tl(7)
        integer cl(7),utsizeof,i,offset,istep
        external utsizeof

        select ndim
        case 0: return(fwa)
        case 1: return(LOCS_PER_WORD*utsizeof(tl(1)-ilow(1), tc) + fwa)
        case 2: return(LOCS_PER_WORD*utsizeof(tl(1)-ilow(1)+cl(1)*(tl(2)-ilow(2)),tc) + fwa)
        case 3,4,5,6,7:
             offset = 0
             istep = 1
             do i=1,ndim
                offset = offset +(tl(i) - ilow(i)) * istep
                istep = istep * cl(i)
             enddo
             return(LOCS_PER_WORD*utsizeof(offset,tc) + fwa)
        default: call baderr('Impossible dimension in parfwa, call authors.')
        endselect

        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PARCHO - calculate character offset of x(tl(1),tl(2),...tl(ndim)) from loc(x)
c        - ilow, ihi contain the upper and lower dimensions of x
c        - tc is the number of characters per element

      integer function parcho(tc,ndim,ilow,ihi,cl,tl)
      integer tc,ndim,ilow(*),ihi(*),tl(*)
      integer cl(*),i,offset

      if ( ndim = 0) return(0)
      offset = 0
      istep = 1
      do i=1,ndim
         offset = offset +(tl(i) - ilow(i)) * istep
         istep = istep*cl(i)
      enddo

      return(-tc*offset)
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PARAPP - Perform append of stack elt s to var t
c        -    s,t stack elements describing source, target
c        -    t is of type LHS, we hope
c        -    s is an exp

        function parapp()

Use(Actions)
Use(Stacks)
Use(Controls)
Use(Functions)

      integer ndb,parapp
      integer tc,ndim,ilow(7),ihi(7),icol(7), actstr, rtgnastr
      Address fwa
      integer nt
      Varname name
      character*DIMSTRSIZE action
      logical rtischam
      integer parname, parasgn2, utgetcl, naml, rtsetdl
      external parname, rtischam,parasgn2, utgetcl,rtsetdl
      integer parasgn, parlen
      external parasgn, parlen, rtgnastr
      
#0. fix to avoid blowup on multiply-subscripted <lhs>'s
      if (ss(TC,rsp-1) ~= LHS)then
         call remark("parapp: incorrect <lhs> for append.")
         go to 900
      endif

      if (ss(NS,rsp-1) <> 0)then
         call remark("parapp: incorrect <lhs> for append.")
         go to 900
      endif

      if (ss(N,rsp-1) .eq. UNKNOWN) then
# try parasgn, maybe autovar is on
         return(parasgn(ss(1,rsp-1),ss(1,rsp)))
      endif

#1 get the source
      if (ss(TC,rsp) .eq. FUNCTION_TYPE) then
         call remark("parapp: function name on rhs of append.")
         go to 900
      endif
      call parfetch(ss(1,rsp),nxp(rsp),lxp(rsp),COPY,FALSE)  #need the source now
      call pardim(ss(1,rsp))  #reduce source to true shape
      ns = parlen(ss(1,rsp))

      if (ns .eq.0) return(OK)

#2. Get the LIMITED attributes of the target -- tc already set
      jvar = ss(PTR,rsp-1)
      tc = ss(TCSAVE,rsp-1)
      ndb = ss(N,rsp-1)
      call rtxdb(jvar, ndb, fwa, ndim, ilow, ihi ,icol,LIMITED)
      if (fwa .eq. NOTSET) then
         call remark(
         "Target of append is dynamic variable with unset pointer.")
         go to 900
      endif

#3.  If the target is currently a scalar, make it 1-D
      if (ndim.eq.0) then
         call rtup1d(jvar,ndb)
         ndim = 1
         ilow(1)=1
         ihi(1)=1
         icol(1) = 1
      endif

# 4. make the limited target big enough to hold the result
      ksave = ihi(ndim)-ilow(ndim)+1
      klo = ihi(ndim)+1
      if (ss(N,rsp) .eq. 0) then
         khi = klo
      else
         nt = 1
         do i = 1, ndim - 1
            if (ilow(i) >= ihi(i)) next
            nt = nt + 1
         enddo
         if (nt > ss(N,rsp)) then
            khi = klo
         elseif (nt .eq. ss(N,rsp)) then
            khi = klo - 1 + ss(N1-1+ss(N,rsp),rsp)
         else  #this is going to fail when we call parasgn2 !

# proceed anyway to get error diagnosis
            khi = klo 
         endif
      endif
      if ( rtsetdl(jvar,ndb,khi-ilow(ndim)+1) .eq. ERR) go to 900

#5.  set up for an assignment
      ss(NS,rsp-1)=ndim
      do i=1, ndim - 1
         ss(N1-1+i,rsp-1) = ilow(i)
         ss(M1-1+i,rsp-1) = ihi(i)
         ss(I1-1+i,rsp-1) = 1
      enddo
      ss(N1-1+ndim,rsp-1) = klo
      ss(M1-1+ndim,rsp-1) = khi
      ss(I1-1+ndim,rsp-1) = 1

# 6. call parasgn2 to finish the job 
# careful, things have changed!
      call rtxdb(jvar, ndb, fwa, ndim, ilow, ihi ,icol,LIMITED)
      if (parasgn2(tc,fwa,ndim,ilow,ihi,icol,
     &             ss(NS,rsp-1),ss(N1,rsp-1),ss(M1,rsp-1),
     &             ss(I1,rsp-1),ss(1,rsp)) = ERR) then

         junk=rtsetdl(jvar,ndb,ksave)
         go to 900
      endif

      if (semerr .eq. OK) then
         inassign = FALSE           # done with append
         assignsp = 0
         symbolic = TRUE
      endif

      if (jvar <= 0) return(OK)  #no foreign
      actstr = rtgnastr(jvar, ndb)

      if (actstr <> 0) then
         call strgget(actstr,action)
         call parsestr(action)
      endif
      return(OK)

c ... error, print name of target

 900  continue
      if (ss(TC,rsp-1) .eq. LHS | ss(TC,rsp-1) .eq. NAME | ss(TC,rsp-1) .eq. FUNCTION_TYPE | 
     &    ss(TC,rsp-1) .eq. NULL_TYPE | ss(TC,rsp-1) < 0) then
         naml = parname(ss(1,rsp-1),name)
         call baserror("parapp: error in append to variable named '"// \
         name(1:naml) // "'.")
      else
         call remark("parapp: attempted append to expression, "// \
         "not variable.")
      endif

      return(ERR)
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PARMDEXPN - expand matrix to fill the minus dimensions

        function parmdexpn(tc,mdc,mdp,mdv,sxi)

Use(Controls)
Use(Functions)

      integer parmdexpn
      integer parstore
      Address paraddr
      external parget,parrel,paraddr,parstore

      integer sxi(SSWIDTH), swi(SSWIDTH)
      integer i, j, ia(7)
      Address ib(7)
      integer ilow(7),ihi(7),icol(7),sl(7),sh(7),si(7)
      Address fwa
      integer tc,ndim
      integer mdc,mdp(7),mdv(7),imdc
      integer icur
      data ia /7*0/, ib /7*0/
 
# expand one dimension at a time
# get data descriptors including the minus dimension
      swi(TC) = tc
      
      do imdc = 1,mdc
         swi(N) =  sxi(N) + 1
         icur = 0
         do i = 1,swi(N)
            if (mdp(imdc) <> i) then
               swi(N1-1+i) = sxi(N1+icur)
               swi(M1-1+i) = 1
               icur = icur + 1
            else
               swi(N1-1+i) = mdv(imdc)        #insert neg dim
               swi(M1-1+i) = 1        
            endif
         enddo
         
         call parget(swi)		#allocate space for the expaned
         
# expand minus dimensions based on the base matrix
# use the parstore(fwa,tc,ndim,ilow,ihi,icol,sl,sh,si,s)
# to assign the entries of the expanded matrix
         fwa = paraddr(swi)
         ndim = swi(N)
         do j=1,swi(N1-1+mdp(imdc))
            do i=1,ndim
               si(i) = 1
               ilow(i) = 1
               ihi(i) = swi(N1-1+i)
               icol(i) = ihi(i)
               if (mdp(imdc) <> i) then
                  sl(i) = ilow(i)
                  sh(i) = ihi(i)
               else
                  sl(i) = j
                  sh(i) = j
               endif
            enddo     #i loop
            
            if (parstore(fwa,tc,ndim,ilow,ihi,icol,sl,sh,si,sxi,ib,ia)=ERR) then
               call remark('parstore failure')
               return(ERR)
            endif
         enddo        #j loop
         
# now swap swi to sxi to make it as a base matrix
         call parrel(sxi)
         call parcopy(swi, sxi)
      enddo   #mdc(minus dimensions)
      
      return(OK)
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

