/*
 * attr.c - handle group attributes
 *
 */

#include "rt.h"


static char
    attribs[MAXATTR][VARNAMESIZE + 1];

static Integer
    natable = 0,
    readonly,
    attcodln = 0,
    attcodsz = 0,
    *codeatt;

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

Integer getreadonly()
{
    return readonly;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

logical Bastestattr(Integer m, Integer n)
{
    char *t;
    logical rval;

    t = FccMakeString(NULL, natable, 1);
    StrgGet(m, t);
    rval = t[n - 1] == 'y';
    FccFree(t);
    return rval;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

 /* returns TRUE if variable jvar has attribute nattr=rtattrlu("attribute")
  * Revision 9/89 (ZCM) if ndb <0, jvar is really a group number. See if
  * the group has the attribute.
  * Revision 7/91 (ZCM) remove direct references to Macros group
  * Revision 10/93 return false for foreign variables
  * Revision 5/96 (FNF) Fix incorrect integer declaration for bastestattr.
  * Rewritten 9/23/1999 (ZCM) in C.
  */
logical Rtdattr(Integer jvar, Integer ndb, Integer nattr)
{
    if (jvar < 0)
        return 0;
    if (nattr == DYNATT) {
        if (ndb == MACRODB)
            return 0;
        if (ndb <= 0)
            return 0;
        return Rtisdyn(jvar, ndb);
    }
    if (ndb == MACRODB)
        return Bastestattr(Rtmacat(jvar), nattr);
    if (ndb > 0)
        return Bastestattr(Rtgvattr(jvar, ndb), nattr);
    else
        return Bastestattr(Rtggattr(jvar, ndb), nattr);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

 /* returns the attribute number of attribute attr (1-based)
  * returns 0 if attr not in the attribute table
  *   Arguments:
  *         attr     =:  name of attribute to look for
  *         idx      =:  where to start looking in the table
  *         partfind =:  if FALSE, then only an exact match with attr will be
  *                      found, else if ending in ":" then check for partial
  *                      matches, i.e. "w:" will match both "w:mass" and "w:ions".
  *         xactmtch  := if TRUE, then an exact match was found, else a partial
  *                      match was found (i.e an attribute was found which begins
  *                      with string ATTR). */
Integer Rtattrlu(char *attr, Integer idx, Integer partfind,
                 Integer *xactmtch)
{
    int i, idxcolon, lenattr;

    *xactmtch = 1;
    if (!strcmp(attr, "dynamic"))
        return (DYNATT);
    idxcolon = 0;
    lenattr = strlen(attr);
    if (lenattr >= VARNAMESIZE)
        return 0;

    idxcolon = strcspn(attr, ":");
    if (idxcolon == lenattr)
        idxcolon = 0;
    for (i = idx - 1; i < natable; i++) {
        if (!strcmp(attr, attribs[i]))
            return i + 1;
        if (idxcolon > 0) {
            if (partfind && !strncmp(attr, attribs[i], idxcolon)) {
                *xactmtch = 0;
                return i + 1;
            }
        }
    }
    return 0;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 * Set an attribute.
 */

void Rtsattr(Integer *m, Integer n)
{
    /* PFD change the meaning of m from the actual bits to a string index */
    char *t;

    t = FccMakeString(NULL, natable, 1);
    StrgGet(*m, t);
    t[n - 1] = 'y';
    *m = StrgFind(t);
    FccFree(t);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 * Remove an attribute
 */

static void Rtrattr(Integer *m, Integer n)
{
    char *t;

    t = FccMakeString(NULL, natable, 1);
    StrgGet(*m, t);
    t[n - 1] = ' ';
    *m = StrgFind(t);
    FccFree(t);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 * Add attr to table, return number.
 */

Integer Rtattrin(char *attr)
{
    /* returns the attribute number of attribute attr, adding it to the table
     * if necessary. */
    Integer nattr;
    logical partfind, xactmtch;
    static char msg1[] = "rtattrin: Too many attributes defined.";

    /* do NOT allow partial searches.  MUST match exactly */
    partfind = (logical) 0;
    nattr = Rtattrlu(attr, 1, partfind, &xactmtch);
    if (nattr != 0)
        return nattr;
    if (natable >= MAXATTR) {
        BasError(msg1, strlen(msg1));
        Kaboom(1);
    }
    natable += 1;
    strcpy(attribs[natable - 1], attr);
    return natable;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 * Modify the attribute list.
 *
 * Created (SMH) 10/89 -- extracted portion of old subroutin RTVATTRE
 * Changed (SMH) 01/90 -- allow attr to contain a list of attributes
 *                        (both add and subtract requests)
 * Changed (ZCM) 10/1999 -- reprogrammed in C
 * This routine modifies atrlist to reflect the addition or subtraction
 *       of the attribute given in argument ATTR.
 * Arguments:
 *       atrlist   := list (1 bit/attribute) of attributes in which to
 *                    turn on or off a given attribute.
 *       attr     =:  name(s) of the given attribute(s) to turn on or off.
 */
void Rtchattr(Integer *atrlist, char *attr)
{
    char c, ct[2];
    char lattr[VARNAMESIZE + 1];
    Integer idx, idxlen, n, lenstr;

    idx = 0;
    lenstr = strlen(attr);
    while (idx < lenstr) {
        idx += strspn(attr + idx, " ");
        c = attr[idx];
        if (c == '+' || c == '-') {
            ct[0] = c;
            ct[1] = '\0';
            if (!strcmp(attr, ct)) {
                break;
            }
            idx += 1;
            idx += strspn(attr + idx, " ");
        }

        /* find end of attribute name, terminated by blank, +, -, or EOL */

        idxlen = strcspn(attr + idx, " +-");
        if (idxlen == 0) {
            break;
        }
        strncpy(lattr, attr + idx, idxlen);
        lattr[idxlen] = '\0';
        n = Rtattrin(lattr);
        if (c == '-')
            Rtrattr(atrlist, n);
        else
            Rtsattr(atrlist, n);
        idx += idxlen;
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* change the attribute to the variable jvar in database ndb
 * Revision (ZCM)  9/89 -- if ndb < 0 this is a group attribute to edit.
 *       Revision (SMH) 10/89 -- calls newly created routine RTCHATTR
 *       Revision (ZCM)  10/1999 -- rewritten in C
 */

void Rtvattre(Integer jvar, Integer ndb, char *attr)
{
    Integer iattr;
    static char
        msg[] = "rtvattre: Cannot set attribute of foreign variable";

    if (jvar <= 0) {
        BasError(msg, strlen(msg));
        Kaboom(0);
    }
    if (ndb > 0) {
        iattr = Rtgvattr(jvar, ndb);
        Rtchattr(&iattr, attr);
        Rtsvattr(jvar, ndb, iattr);
    } else {
        iattr = Rtggattr(jvar, ndb);
        Rtchattr(&iattr, attr);
        Rtsgattr(jvar, abs(ndb), iattr);
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 * Does 'name' have attribute 'attr'?
 * REVISION 01/90 (SMH) allows for partial searches on attributes if attr ends
 *                     in a ":",  i.e. "w:" will match both "w:mass" and "w:ions"
 * REWRITTEN in C: ZCM 10/06/1999
 */

static Integer Rtdbattr(Integer jvar, Integer ndb, char *attr,
                        Integer warnflg)
{
    Integer idxattr, nattr;
    logical partfind, xactmtch, foundflg;
    static char msg[] = "caution, no such attribute:";

    if (!strcmp(attr, "dynamic")) {
        if (ndb == MACRODB || ndb <= 0)
            return 0;
        return (Integer) Rtisdyn(jvar, ndb);
    }

    partfind = (logical) 1;
    idxattr = 1;
    nattr = Rtattrlu(attr, idxattr, partfind, &xactmtch);
    if (nattr == 0) {
        if (warnflg) {
            BasError(msg, strlen(msg));
            BasError(attr, strlen(attr));
        }
        return 0;
    }

    if (xactmtch) {
        return (Integer) Rtdattr(jvar, ndb, nattr);
    } else {
        /* look at all partial matches until found or done */
        while (1) {
            foundflg = Rtdattr(jvar, ndb, nattr);
            if (foundflg || xactmtch)
                return (Integer) foundflg;
            idxattr = nattr + 1;
            nattr = Rtattrlu(attr, idxattr, partfind, &xactmtch);
            if (nattr == 0)
                /* exhausted all matches, none found */
                return 0;
        }
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 * Does 'name' have attribute 'attr'?
 *
 * This routine mostly replaces rtattr, which is now just a shell.
 */

Integer Crtattr(char *name, char *attr)
{
    char newname[VARNAMESIZE + 1];
    Integer jvar, ndb, itype, macindx, okflg, namlen1, namlen2;

    Rtismac(name, newname, &macindx);
    if (macindx > 0) {
        return Rtdbattr(macindx, MACRODB, attr, 1);

    } else {
        /* try to avoid using newname in case name is longer than a Varnam */
        namlen1 = strlen(newname);
        namlen2 = strlen(name);
        if (!strncmp(name, newname, min(namlen1, namlen2))) {
            okflg = Rtfinder(name, &jvar, &ndb, &itype, "rtattr");
        } else {
            okflg = Rtfinder(newname, &jvar, &ndb, &itype, "rtattr");
        }
        if (okflg == OK) {
            if (itype != GROUP) {
                return Rtdbattr(jvar, ndb, attr, 1);
            } else {
                return Rtdbattr(jvar, -ndb, attr, 1);
            }
        } else {
            return 0;
        }
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 * Change a name's attributes.
 *
 * This routine mostly replaces rtcattr, which is now just a shell.
 */

void Crtcattr(char *name, char *attr)
{
    char newname[MAXOUT];
    Integer jvar, ndb, itype, macindx, okflg, namlen1, namlen2;
    static char msg[] = "rtcattr: cannot locate variable requested.";

    Rtismac(name, newname, &macindx);
    if (macindx > 0) {
        Rtchattr(Gmacatra(macindx), attr);
    } else {
        namlen1 = strlen(newname);
        namlen2 = strlen(name);
        if (!strncmp(name, newname, min(namlen1, namlen2))) {
            okflg = Rtfinder(name, &jvar, &ndb, &itype, "rtcattr");
        } else {
            okflg = Rtfinder(newname, &jvar, &ndb, &itype, "rtcattr");
        }
        if (okflg == OK) {
            if (itype != GROUP) {
                Rtvattre(jvar, ndb, attr);
            } else {
                Rtvattre(jvar, -ndb, attr);
            }
        } else {
            BasError(msg, strlen(msg));
            Kaboom(0);
        }
    }
}

/*--------------------------------------------------------------------------*/
/*                     FUNCTIONS FOR ACCESSING                              */
/*                     THE ATTRIBUTES DATABASE                              */
/*--------------------------------------------------------------------------*/

Integer Rtnattr()
{
    return natable;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

void Rtxattr(Integer i, char *attr)
{
    strcpy(attr, attribs[i - 1]);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

Integer Rtlattr(Integer i)
{
    return strlen(attribs[i - 1]);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

void Setrdo(Integer val)
{
    readonly = val;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

void Rtinscod(Integer codewrd)
{
    /* Moved here from srv.m to isolate references to dynamics */
    /*          (Zane C. Motteler 2/16/2000)                   */
    if (attcodln >= attcodsz) {
        if (attcodsz == 0) {
            Osallotn((Address *) &codeatt, ATCODEINC, "RTINSCOD:codeatt");
        } else {
            Oschangn((Address *) &codeatt, attcodsz + ATCODEINC, attcodsz,
                     "codeatt");
        }
        attcodsz += ATCODEINC;
    }
    codeatt[attcodln++] = codewrd;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

void Rtdelcod(void)
{
    Osfreen((Address *) &codeatt, "RTDELCOD:codeatt");
    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Get the idx'th codeatt (ZCM 2/16/2000) */

Integer Gattcode(Integer idx)
{
    return codeatt[idx - 1];
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Initialize the codeatt block (ZCM 2/16/2000) */

void Iattcdln(void)
{
    attcodln = 0;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
