       subroutine basnxtsq(prev, f)
# set f to name of next file after prev
# ok if prev and f are the same
# Policy from Dave Munro
#       (1) Is the final character of the filename a digit?
#           If so, go to step (3).
#       (2) Scan backwards through the filename looking for a
#           period ("."), and make the character before the period
#           the current character.  If there is no period, or if the
#           period is the first character, make the final character
#           of the filename the current character again.
#       (3) If the current character is not A-z, a-z, or 0-9, scan
#           backwards through the name until a character in one of these
#           ranges is encountered, making that the current character.
#           If no such character is present, prepend an "A" to the name.
#       (4) Increment the current character in the following sense:
#           A-Y become B-Z, a-y become b-z, 0-8 become 1-9, and z
#           or Z becomes 0.  If the current character is a 9, it
#           becomes a 0, the preceding character becomes the current
#           character, and we loop to step (3).  Otherwise, done.
#
#The following family sequences are all handled correctly by this
#algorithm (and I think any of them is a perfectly reasonable choice):
#
#       prob01fa prob01fb prob01fc prob01fd ...
#       prob02q00 prob02q01 prob02q02 prob02q03 ... prob02q43 ...
#       prob03q00.pdb prob03q01.pdb prob03q02.pdb prob03q03.pdb ...
#       prob03q00.cdf prob03q01.cdf prob03q02.cdf prob03q03.cdf ...
#
#The algorithm also has the virtue that it never does anything
#involving outlandish characters, nor can it fail to produce a name
#under any circumstance (not counting trying to increment eight 9s
#under LTSS).
# m is the index of the "current character" in the above description
       character*(*) prev, f
       Filename g
       character c
       integer utgetcl, m, n
       external utgetcl
       g = prev  #save for later in case f, prev the same
       f = prev
1      continue
       n = utgetcl(f)
       m = n
       c = f(m:m)
       if(c < "0" | c > "9") then
# 2
          while(m > 0) then
             c = f(m:m)
# This routine assumed file-not complete paths--would be used as arguments.
# Thus the sequencing gets very messed up if a '.' appears as a directory in
# the path name. So we simply stop looking for '.' if we are in part of the 
# path chain, assumed delimited by '/'.
             if(c=="/") then
                m = n
                go to 3
             elseif(c==".") then
                m = m - 1
                go to 3
             endif
             m = m - 1
          endwhile
          m = n
       endif
3      continue
       while(m > 0) 
          c = f(m:m)
          if((c>="0" & c<="9") |
             (c>="a" & c<="z") |
             (c>="A" & c<="Z") ) then
              break
          endif
          m = m - 1
       endwhile
       if(m == 0) then
          g = "A"//f  #use g not prev here in case f, prev the same
          f = g
          if(utgetcl(f) == n) then
             call remark("basnxtsq: no room in file name to sequence.")
             call remark(g)
             call kaboom(0)
          endif
          return
       endif
4      continue
       c = f(m:m)
       if(c == "z"|c=="Z") then
          f(m:m) = "0"
       elseif(c=="9" ) then
          f(m:m) = "0"
          m = m - 1
          go to 3
       else
          f(m:m) = char(ichar(c)+1)
       endif
       return
       end
