/*
 * CEXEC.C 
 *
 */

#include "rt.h"

Integer oldpc, nsp;

/************** EXTERNALS **************/

#define Ruthere FF_ID(ruthere, RUTHERE)
extern void Ruthere(void);

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* LRSETDO - Executes the SET1 instruction.
 *              1) Assigns <first> to <lhs>, the loop control variable
 *              2) Makes sure <lhs> is an integer scalar(n. b. <first>,
 *                 <last>, and <incr> have already been checked and are OK
 *                 if we made it this far.
 *              3) moves the values of <last> and <incr> to s(N1) and S(M1)
 *                 respectively, for subsequent runs through the loop.
 *              4) releases their stack elements ss(1,rsp-1) and ss(1,rsp).
 * No arguments are necessary; rsp-3 points at lhs stack element; rsp-2 points
 * at first; rsp-1 points at last; rsp points at incr.
 */

void Lrsetdo(void)
{
    Integer jvar, ndb, tc, ns;
    int error = 0;
    char name[256], msg[80];
    static char
        msg1[] = "lrsetdo: do variable not of type integer.",
        msg2[] = "lrsetdo: illegal use of array name as do loop control.",
        msg3[] = "lrsetdo: do control specification is not scalar.";

    INIT_BLANK_STRING(name, 256);

    jvar = ssl(PTR, rsp - 3);
    ndb = ssl(N, rsp - 3);
    tc = ssl(TCSAVE, rsp - 3);
    ns = ssl(NS, rsp - 3);

    if (ndb != UNKNOWN) {
        if (!Rtischam(jvar, ndb)) {
            if (tc != INTEGER) {
                BasError(msg1, strlen(msg1));
                error = 1;
            } else if (ns == 0 && Rtgndim(jvar, ndb) > 0) {
                BasError(msg2, strlen(msg2));
                error = 1;
            } else if (ns > 0 && Lrsize(&(ssl(1, rsp - 3))) > 1) {
                BasError(msg3, strlen(msg3));
                error = 1;
            }
        }
    }

    if (!error) {
        if (Parasgn(&(ssl(1, rsp - 3)), &(ssl(1, rsp - 2))))
            error = 1;
        else {
            ssl(N1, rsp - 3) = *(int *) (Paraddr(&(ssl(1, rsp - 1))));
            ssl(M1, rsp - 3) = *(int *) (Paraddr(&(ssl(1, rsp))));
            loopctl[nestlevl - 1] = Partempc(dofwa);
            set1flg = (logical) 1;
            looptyp[nestlevl - 1] = DO_DEF;
            Parrel(&(ssl(1, rsp - 1)));
            Parrel(&(ssl(1, rsp)));
        }
    }

    if (error) {
        Rtgvname(jvar, ndb, name);
        sprintf(msg, "lrsetdo: error in do loop on variable '%s'.", name);
        BasError(msg, strlen(msg));
        Kaboom(0);
    }

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* LRDOLOOP - Test and increment do loop variable; values have been
 *          - placed on stack dynamically by SET1 instruction.
 *          - This routine executes the DOTEST instruction.
 */

static void Lrdoloop(integer *pc)
{
    integer ilit, index;
    Address aindex;

/* ilit is an index into the literal table */
    ilit = loopctl[nestlevl - 1];
    aindex = Getaddr(ilit);
    index = *((int *) aindex);

/* increment variable prior to test */
    if (!set1flg) {
        index += ssl(M1, rsp);
        memmove(aindex, (void *) (&index), sizeof(integer));
    }

    set1flg = (logical) 0;

    if ((ssl(M1, rsp) > 0 && index > ssl(N1, rsp)) ||
        (ssl(M1, rsp) < 0 && index < ssl(N1, rsp)))
        *pc += ic[*pc];         /* exit loop */
    else
        *pc += 2;               /* continue */

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* LRPFLOW - Execute flow of control instructions which occur
 *         - in the intermediate code for loops and if-structures.
 */

static void Lrpflow(integer *pc)
{
    integer op;
    char msg[80];
    static char
        msg1[] =
        "lrpflow: jump to negative location counter. Call Z. Motteler.",
        msg2[] = "lrpflow: if control must be a scalar of logical type.";
    static integer actual = ACTUAL, false = 0;
    logical lvalue;

    op = ic[*pc - 1];

    switch (op) {
    case JMP:                  /* Unconditional jump: */
        *pc += ic[*pc];
        break;
    case INT:                  /* break--goes to address AFTER unnest */
        *pc += ic[*pc] + 2;
        break;
    case DOTEST:
        Lrdoloop(pc);
        break;
    case JF:
        Parfetch(&(ssl(1, rsp)), (char *) nxp + (rsp - 1) * COMMENTSIZE,
                 &(lxp[rsp - 1]), &actual, &false, COMMENTSIZE);
        if (ssl(TC, rsp) != LOGICAL || Parlen(&(ssl(1, rsp))) != 1) {
            BasError(msg2, strlen(msg2));
            semerr = ERR;
            return;
        }
        lvalue = *((logical *) Paraddr(&(ssl(1, rsp))));
        if (!lvalue)
            *pc += ic[*pc];
        else
            *pc += 2;
        Parrel(&(ssl(1, rsp))); /* release space for if control */
        rsp--;
        break;
    default:
        sprintf(msg, "lrpflow: impossible flow of control operator: %ld",
                (long) op);
        BasError(msg, strlen(msg));
        semerr = ERR;
    }

    if (*pc < 0) {
        BasError(msg1, strlen(msg1));
        semerr = ERR;
    }

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* LRYACT - This subroutine actually carries out the actions associated with
 *        - terminal symbols.
 */

void Lryact(integer *pc)
{
    static char
        msg[] =
        "Runtime stack overflow pushing literal...possible runaway recursion.";
    char msg1[80];
    integer action, ltokn, token, ic3;
    Address attokn;

    if (semerr == ERR)
        return;
    if (rsp + 1 > SSLENGTH) {
        BasError(msg, strlen(msg));
        Kaboom(0);
    }

    action = -ic[*pc - 1];
    token = ic[*pc];
    ic3 = ic[*pc + 1];
    ltokn = StrgLen(ic3);
    attokn = StrgAddr(ic3);

    rsp++;
    ssl(N, rsp) = 0;

/* names treated differently in this version */
    if (action == 100) {        /* a name */
        ssl(TC, rsp) = NAME;

/* # ss(PTR,.) is the subscript into the literal table
 # it used to be the string table, but this avoids
 # redundant lookup(ZCM, 7/28/95)
 */
        ssl(PTR, rsp) = Partempc(attokn);
        ssl(NAML, rsp) = -ltokn;        /* ss(NAML,.) < 0 for this type of NAME */

/* 7/31/95(ZCM) N1 entry used to signal not yet looked
 * up in string table
 */
        ssl(N1, rsp) = 0;
        *pc += 3;
        ssl(NS, rsp) = 0;
        if (symbolic) {
            memset((void *) (nxp + (rsp - 1)), (int) ' ', COMMENTSIZE);
            strncpy((char *) nxp + (rsp - 1) * COMMENTSIZE,
                    (char *) attokn, ltokn);
            lxp[rsp - 1] = ltokn;
        }
        return;
    }

/* literals get converted into stack elements */
    ssl(PTR, rsp) = 0;
    ssl(N2, rsp) = 0;

    /* save in nxp */
    if (symbolic) {
        memset((void *) (nxp + (rsp - 1)), (int) ' ', COMMENTSIZE);
        if (action == 106) {    /* restick the quotes */
            if (ltokn == 0)
                lxp[rsp - 1] = 0;
            else {
                nxp[rsp - 1][0] = '"';
                strncpy((char *) nxp + (rsp - 1) * COMMENTSIZE + 1,
                        (char *) attokn, ltokn);
                nxp[rsp - 1][ltokn + 1] = '"';
                lxp[rsp - 1] = ltokn + 2;
            }
        } else {
            strncpy((char *) nxp + (rsp - 1) * COMMENTSIZE,
                    (char *) attokn, ltokn);
            lxp[rsp - 1] = ltokn;
        }
    }

    switch (action) {

/* Note that in the case of integer, real, double, and complex, the
 * token has already been converted to binary and placed in the hash
 * table values.  Token has been set to index of the constant into
 * the hash table, so it can simply be looked up.
 */
    case 101:
        ssl(TC, rsp) = INTEGER;
        ssl(N1, rsp) = values[token - 1];
        break;
    case 102:
        ssl(TC, rsp) = REAL;
        ssl(N1, rsp) = values[token - 1];
        break;
    case 103:
#  ifdef HAS_DOUBLE_COMPLEX
        ssl(TC, rsp) = DOUBLE_C;
        ssl(N1, rsp) = 0;
        ssl(N2, rsp) = 0;
        memmove((void *) (&(ssl(N3, rsp))),
                (void *) (&(dvalues[token - 1])), sizeof(double));
#  else
        ssl(TC, rsp) = COMPLEX;
        ssl(N1, rsp) = 0;
        ssl(N2, rsp) = values[token - 1];
#  endif
        break;
    case 104:                  /* double precision */
        ssl(TC, rsp) = DOUBLE;
        memmove((void *) (&(ssl(N1, rsp))),
                (void *) (&(dvalues[token - 1])), sizeof(double));
        break;
    case 105:                  /* group name */
        ssl(TC, rsp) = -ltokn;
        ssl(PTR, rsp) = StrgFina(attokn, ltokn);
        ssl(NAML, rsp) = 0;     /* signals string literal in string table */
        break;

/* special for fcommand: if argument omitted, string will have length 0
 * in that case put NULL on the stack
 */
    case 106:
        ssl(TC, rsp) = -ltokn;
        if (ltokn > 0)
            ssl(PTR, rsp) = StrgFina(attokn, ltokn);
        ssl(NAML, rsp) = 0;     /* NAML == 0 signals string literal */
        break;
    default:
        sprintf(msg1, "Impossible action in lryact: %ld.\n",
                (long) action);
        BasError(msg1, strlen(msg1));
        semerr = ERR;
    }
    *pc += 3;

    return;
}

/*--------------------------------------------------------------------------*/

/*                         MAIN EXECUTION ENGINE                            */

/*--------------------------------------------------------------------------*/

/* LREXEC - To execute intermediate code by calling appropriate
 *        - action routines:
 *        - lryact for basic token actions(stack-building)
 *        - lrpflow for flow of control operators(jumps)
 *        - lrxact for all other actions
 */

integer Lrexec(void)
{
    integer pc, oldpc, oldrsp, pcnext, oldop, ius;
    int idummy;

    char msg[80];
    static char
        msg1[] = "lrexec: Stack pointer clobbered?",
        msg2[] = "After execution of the following instruction:",
        msg3[] = "Oops! The link pointer is 1 after executing:",
        msg4[] = "Oops! The stack pointer is negative after executing:",
        msg5[] = "Oops! The stack pointer is zero after executing:";

/* pc is the program counter for intermediate code during execution
 # rsp is the 'runtime' stack pointer
 # Don't execute if:
 # the nesting level nestlevl is greater than when this compile started(xuser)
 # normally xuser is zero but parstr and execuser set it.
 */
    if (nestlevl != xuser)
        return OK;
    if (nestlevl == 0)
        argfwa[0] = ufcnptr;
    execflag = (logical) 1;
    pc = codebgn;

/* When lrexec is called, we are always executing in intcode: */
    ic = intcode;

/* Address into funpar(NameType) array */
    nb = (char *) funpar;
    tb = (Integer *) funpar;

/* execute the code
 # Note: we are executing code at the outermost, global, or "main program"
 # level if ic == intcode. Whenever a user function has been
 # called, however, the two pointers will be different. Thus in this case
 # PC comparisons are worthless, since the two PC's are not related(each
 # is relative to the block within which it is valid).
 # Changed the semerr test to OK to catch bad value immediately.(Z 8/97)
 */
    while ((ic != intcode || (pc < nextcode && pc > 0)) && semerr == OK) {
        oldpc = pc;
        oldrsp = rsp;
        oldop = ic[pc - 1];
        if (pc == 1 && rsp != 0) {
            sprintf(msg,
                    "--stack pointer is %ld and should be 0, when pc is 1.",
                    (long) rsp);
            BasError(msg1, strlen(msg1));
            BasError(msg, strlen(msg));
            Kaboom(0);
        }

        if (oldop < 0) {        /* token action */
            Lryact(&pc);
        } else if (oldop < JMP) {       /* basic code actions */
            Lrxact(&pc, &pcnext);
            pc = pcnext;        /* pc + 1 unless something happened */
        } else {                /* flow of control and such */
            Lrpflow(&pc);       /* includes DOTEST instruction now */
        }

        if (rlinkp == 1) {
            BasError(msg3, strlen(msg3));
            ius = iusout;
            Dbgtrace(ius, oldpc, oldrsp, rsp, 1, 0);
            Kaboom(0);
        }

        if (rsp < 0) {
            BasError(msg4, strlen(msg4));
            ius = iusout;
            if (rlinkp != 0)
                Dbgtrace(ius, oldpc, oldrsp, rsp, rlinkp, ssl(1, rlinkp));
            else
                Dbgtrace(ius, oldpc, oldrsp, rsp, 0, 0);
            Kaboom(0);
        }

        if (rsp == 0 && oldop == LINE) {
            BasError(msg5, strlen(msg5));
            ius = iusout;
            if (rlinkp != 0)
                Dbgtrace(ius, oldpc, oldrsp, rsp, rlinkp, ssl(1, rlinkp));
            else
                Dbgtrace(ius, oldpc, oldrsp, rsp, 0, 0);
            Kaboom(0);
        }

        nsp = rsp;
        if (semerr != OK && semerr != ERR) {
            sprintf(msg, "lrexec: bad value of semerr: %ld.",
                    (long) semerr);
            BasError(msg, strlen(msg));
            BasError(msg2, strlen(msg2));
            if (!partrace || tracedone) {
                ius = iusout;
                if (rlinkp != 0)
                    Dbgtrace(ius, oldpc, oldrsp, rsp, rlinkp,
                             ssl(1, rlinkp));
                else
                    Dbgtrace(ius, oldpc, oldrsp, rsp, 0, 0);
                Kaboom(0);
            }
        }

        if (partrace) {
            if (!tracedone) {
                ius = iusout;
                if (rlinkp != 0 && oldop != 136)
                    Dbgtrace(ius, oldpc, oldrsp, rsp, rlinkp,
                             ssl(1, rlinkp));
                else
                    Dbgtrace(ius, oldpc, oldrsp, rsp, 0, 0);
            } else
                tracedone = (logical) 0;
        }

        if (memtrace == YES) {
            idummy = SC_mem_chk(3);
            if (idummy < 0) {
                sprintf(msg,
                        "lrexec: memory just went bad; memcheck = %d.",
                        idummy);
                BasError(msg, strlen(msg));
                Kaboom(0);
            }
        }
        Ruthere();
    }

/* clean up */
    if (semerr == ERR)
        Kaboom(1);

    if (semerr != OK) {
        sprintf(msg, "lrexec: bad value of semerr: %ld.", (long) semerr);
        BasError(msg, strlen(msg));
        Kaboom(1);
    }

    nextcode = codebgn;
    execflag = (logical) 0;
    nestlevl = xuser;
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
