/*
 * cfcn.c
 */

#include "cfcn.h"

/*
 * HOW TO ADD A BUILTIN FUNCTION
 *
 * (1) Write the function
 *
 * (2) Add a call to Parinstall_fcn in the appropriate section
 *     of Bfcninit
 */

logical
    false = (logical) 0;

Integer
    copy     = COPY,
    nodata   = NODATA,
    actual   = ACTUAL,
    myrng    = RANGE,
    myint    = INTEGER,
    myreal   = REAL,
    mydble   = DOUBLE,
    mycmplx  = COMPLEX,
    mydcmplx = DOUBLE_C,
    two      = 2,
    nine     = 9,
    ATAN2    = 0,
    BASNXTSQ = 1,
    INF      = 2,
    MAX      = 3,
    MIN      = 4,
    MNX      = 5,
    MXX      = 6,
    PSUM     = 7,
    RNGINC   = 8,
    RSUM     = 9,
    STRLEN   = 10,
    SUP      = 11,
    TOLOWER  = 12,
    TOUPPER  = 13,
    TRIM     = 14,
    TRIML    = 15,
    TRIMR    = 16;

static HASHTAB
   *Par_fcn_tab = NULL;

/* N. B. In the following table we use CINTEGER, CLOGICAL, and CFWA
 * to avoid conflict with the par.v(par_.h) values INTEGER, LOGICAL, and FWA
 */

static Integer
    CINTEGER       =  0,        /* integer only */
    I_TO_D         =  1,        /* integer, real, or double */
    R_AND_D        =  2,        /* real or double only */
    RN             =  3,        /* range */
    I_TO_D_AND_RN  =  4,        /* I_TO_D and RN */
    C_AND_DC       =  5,        /* complex and double complex */
    I_TO_DC        =  6,        /* I_TO_D plus C+AND_DC */
    I_TO_DC_AND_RN =  7,        /* I_TO_DC plus RN */
    CLOGICAL       =  8,        /* logical */
    I_TO_L         =  9,        /* I_TO_DC and CLOGICAL */
    I_TO_L_AND_RN  = 10,        /* I_TO_L plus RN */
    STR1           = 11,        /* character * 1 */
    STR            = 12,        /* character * N */
    CFWA           = 13,        /* Address */
    ANY            = 14,        /* any type accepted */
    IGNORE         = 15;        /* ignore type for now */

/*--------------------------------------------------------------------------*/

/*                       FUNCTION TABLE ROUTINES                            */

/*--------------------------------------------------------------------------*/

/* BFCNINIT - initializes the builtin function table */

void Bfcninit(void)
{
/* Bit functions */
    Parinstall_fcn("btest",  Cbtest,  CINTEGER, 0);
    Parinstall_fcn("iand",   Ciand,   CINTEGER, 0);
    Parinstall_fcn("ibclr",  Cibclr,  CINTEGER, 0);
    Parinstall_fcn("ibits",  Cibits,  CINTEGER, 0);
    Parinstall_fcn("ibset",  Cibset,  CINTEGER, 0);
    Parinstall_fcn("ieor",   Cieor,   CINTEGER, 0);
    Parinstall_fcn("ior",    Cior,    CINTEGER, 0);
    Parinstall_fcn("ishft",  Cishft,  CINTEGER, 0);
    Parinstall_fcn("ishftc", Cishftc, CINTEGER, 0);
    Parinstall_fcn("not",    Cunot,   CINTEGER, 0);

/* Logical functions */
    Parinstall_fcn("land", Cland, CLOGICAL, 0);
    Parinstall_fcn("lor",  Clor,  CLOGICAL, 0);

/* Type functions */
    Parinstall_fcn("char",   Cchar,   CINTEGER, 0);
    Parinstall_fcn("cmplx",  Ccmplx,   I_TO_DC, 0);
    Parinstall_fcn("dble",   Cdble,    I_TO_DC, 0);
    Parinstall_fcn("dcmplx", Cdcmplx,  I_TO_DC, 0);
    Parinstall_fcn("float",  Cfloat,   I_TO_DC, 0);
    Parinstall_fcn("ichar",  Cichar,      STR1, 0);
    Parinstall_fcn("int",    Ciint,    I_TO_DC, 0);
    Parinstall_fcn("sngl",   Csngl,    I_TO_DC, 0);
    Parinstall_fcn("struct", Cstruct,      ANY, 0);
    Parinstall_fcn("type",   Ctype,        ANY, 0);
    Parinstall_fcn("utype",  Cutype,       STR, 0);

/* Standard Math functions */
    Parinstall_fcn("abs",    Par_srarg, I_TO_DC, 2, fabs,    madcabs);
    Parinstall_fcn("acos",   Par_sarg,  I_TO_DC, 2, maacos,  madcacos);
    Parinstall_fcn("aint",   Par_srarg, I_TO_DC, 2, maaint,  madcaint);
    Parinstall_fcn("alog",   Par_sarg,  I_TO_DC, 2, malog,   madclog);
    Parinstall_fcn("alog10", Par_sarg,  I_TO_DC, 2, log10,   madclog10);
    Parinstall_fcn("anint",  Par_srarg, I_TO_DC, 2, maanint, madcanint);
    Parinstall_fcn("asin",   Par_sarg,  I_TO_DC, 2, maasin,  madcasin);
    Parinstall_fcn("atan",   Par_sarg,  I_TO_DC, 2, atan,    madcatan);
    Parinstall_fcn("cos",    Par_sarg,  I_TO_DC, 2, cos,     madccos);
    Parinstall_fcn("cosh",   Par_sarg,  I_TO_DC, 2, cosh,    madccosh);
    Parinstall_fcn("cot",    Par_sarg,  I_TO_DC, 2, macot,   madccot);
    Parinstall_fcn("coth",   Par_sarg,  I_TO_DC, 2, macoth,  madccoth);
    Parinstall_fcn("exp",    Par_sarg,  I_TO_DC, 2, exp,     madcexp);
    Parinstall_fcn("log",    Par_sarg,  I_TO_DC, 2, malog,   madclog);
    Parinstall_fcn("log10",  Par_sarg,  I_TO_DC, 2, malog10, madclog10);
    Parinstall_fcn("ranf",   Par_zarg,  I_TO_DC, 2, Ranf,    madcranf);
    Parinstall_fcn("sin",    Par_sarg,  I_TO_DC, 2, sin,     madcsin);
    Parinstall_fcn("sinh",   Par_sarg,  I_TO_DC, 2, sinh,    madcsinh);
    Parinstall_fcn("sqrt",   Par_sarg,  I_TO_DC, 2, masqrt,  madcsqrt);
    Parinstall_fcn("tan",    Par_sarg,  I_TO_DC, 2, tan,     madctan);
    Parinstall_fcn("tanh",   Par_sarg,  I_TO_DC, 2, tanh,    madctanh);

    Parinstall_fcn("aimag",    Caimag,    C_AND_DC, 0);
    Parinstall_fcn("atan2",    Catan2,      I_TO_D, 1, &ATAN2);
    Parinstall_fcn("conjg",    Cconjg,    C_AND_DC, 0);
    Parinstall_fcn("legendre", Clegendre,   I_TO_D, 0);
    Parinstall_fcn("max",      Cminmaxb,    I_TO_D, 1, &MAX);
    Parinstall_fcn("min",      Cminmaxb,    I_TO_D, 1, &MIN);
    Parinstall_fcn("mod",      Cmod,        I_TO_D, 0);
    Parinstall_fcn("nint",     Cnint,      I_TO_DC, 0);
    Parinstall_fcn("sign",     Csign,       I_TO_D, 0);

/* Vector, Matrix or Geometrical Math functions */
    Parinstall_fcn("ave",       Cave,       I_TO_DC, 0);
    Parinstall_fcn("cross",     Ccross,      I_TO_D, 0);
    Parinstall_fcn("cumaddin",  Ccumaddin,     CFWA, 0);
    Parinstall_fcn("diag",      Cdiag,      I_TO_DC, 0);
    Parinstall_fcn("inf",       Cinfsup,     I_TO_D, 1, &INF);
    Parinstall_fcn("outer",     Couter,     I_TO_DC, 0);
    Parinstall_fcn("sum",       Csum,       I_TO_DC, 0);
    Parinstall_fcn("sup",       Cinfsup,     I_TO_D, 1, &SUP);
    Parinstall_fcn("transpose", Ctranspose,     ANY, 0);
    Parinstall_fcn("vmax",      Cvminvmax,   I_TO_D, 1, &MAX);
    Parinstall_fcn("vmin",      Cvminvmax,   I_TO_D, 1, &MIN);

/* String functions */
    Parinstall_fcn("strchpat", Cstrchpat,     STR, 0);
    Parinstall_fcn("strlen",   Cstrlen,       STR, 1, &STRLEN);
    Parinstall_fcn("substr",   Csubstr,       STR, 0);
    Parinstall_fcn("tolower",  Ctolowerupper, STR, 1, &TOLOWER);
    Parinstall_fcn("toupper",  Ctolowerupper, STR, 1, &TOUPPER);
    Parinstall_fcn("trim",     Ctrim,         STR, 1, &TRIM);
    Parinstall_fcn("triml",    Ctrim,         STR, 1, &TRIML);
    Parinstall_fcn("trimr",    Ctrim,         STR, 1, &TRIMR);

/* Range functions */
    Parinstall_fcn("rangex",    Crangex,               ANY, 0);
    Parinstall_fcn("rngbeg",    Crngbeg,     I_TO_D_AND_RN, 0);
    Parinstall_fcn("rngend",    Crngend,     I_TO_D_AND_RN, 0);
    Parinstall_fcn("rngsetdf",  Crngsetdf,  I_TO_DC_AND_RN, 0);
    Parinstall_fcn("shape",     Cshape,                ANY, 0);
    Parinstall_fcn("truerange", Ctruerange,            STR, 0);
    Parinstall_fcn("trueshape", Ctrueshape,            STR, 0);

/* Miscellaneous functions */
    Parinstall_fcn("basloc",   Cbasloc,             ANY, 0);
    Parinstall_fcn("basnxtsq", Cbasnxtsq,           STR, 1, &BASNXTSQ);
    Parinstall_fcn("checksum", Cchecksum,           ANY, 0);
    Parinstall_fcn("flist",    Cflist,              STR, 0);
    Parinstall_fcn("format",   Cformat,          I_TO_D, 0);
    Parinstall_fcn("fromone",  Cfromone,            ANY, 0);
    Parinstall_fcn("gather",   Cgather,         I_TO_DC, 0);
    Parinstall_fcn("index",    Cindex,              STR, 0);
    Parinstall_fcn("iota",     Ciota,     I_TO_L_AND_RN, 0);
    Parinstall_fcn("length",   Clength,             ANY, 0);
    Parinstall_fcn("load",     Cload,          CINTEGER, 0);
    Parinstall_fcn("mnx",      Cmnxmxx,          I_TO_D, 1, &MNX);
    Parinstall_fcn("mxx",      Cmnxmxx,          I_TO_D, 1, &MXX);
    Parinstall_fcn("ones",     Cones,           I_TO_DC, 0);
    Parinstall_fcn("psum",     Cpsumrsum,       I_TO_DC, 1, &PSUM);
    Parinstall_fcn("ptp",      Cptp,             I_TO_D, 0);
    Parinstall_fcn("rmsdv",    Crmsdv,           I_TO_D, 0);
    Parinstall_fcn("rnginc",   Crnginc,          IGNORE, 0);
    Parinstall_fcn("rsum",     Cpsumrsum,       I_TO_DC, 1, &RSUM);
    Parinstall_fcn("sort",     Csort,              CFWA, 0);
    Parinstall_fcn("spanl",    Cspanl,           I_TO_D, 0);
    Parinstall_fcn("squeeze",  Csqueeze,            ANY, 0);
    Parinstall_fcn("where",    Cwhere,         CLOGICAL, 0);
    Parinstall_fcn("zcen",     Czcen,            I_TO_D, 0);

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* PARINSTALL_FCN - install a built in function */

void Parinstall_fcn(char *fname, PFHand hand, Integer atyp, Integer ni,
                    ...)
{
    int n;
    f_entry *fe;
    void **info;

    if (Par_fcn_tab == NULL)
        Par_fcn_tab = SC_make_hash_table(HSZLARGE, NODOC);

    if (ni > 0)
        info = FMAKE_N(void *, ni, "PARINSTALL_FCN:info");

    else
        info = NULL;

    SC_VA_START(ni);

    for (n = 0; n < ni; n++)
        info[n] = SC_VA_ARG(void *);

    SC_VA_END;

    fe = FMAKE(f_entry, "PARINSTALL_FCN:fe");

    fe->fname   = SC_strsavef(fname, "PARINSTALL_FCN:fname");
    fe->arg1typ = atyp;
    fe->handler = hand;
    fe->ni      = n;
    fe->info    = info;

    SC_install(fname, fe, "f_entry *", Par_fcn_tab);

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* PARRLFE - releas a function table entry */

static void Parrlfe(f_entry *fe)
{
    SFREE(fe->info);
    SFREE(fe->fname);
    SFREE(fe);

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* PARCLRFT - release the function table */

void Parclrft(void)
{
    SC_hash_clr_table(Par_fcn_tab, Parrlfe);

    Par_fcn_tab = NULL;

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* PARLOOKUP_FCN - lookup a built in function */

f_entry *Parlookup_fcn(char *fname)
{
    f_entry *thisf;
    char errmsg[MAXOUT + 1];

    thisf = (f_entry *) SC_def_lookup(fname, Par_fcn_tab);

    if (thisf == NULL) {
        sprintf(errmsg, "Parlookup_fcn: builtin function %s not found.",
                fname);
        BasError(errmsg, strlen(errmsg));
    }

    return (thisf);
}

/*--------------------------------------------------------------------------*/

/*                    FUNCTION CALL ARGUMENT ROUTINES                       */

/*--------------------------------------------------------------------------*/

/* FCNARGA - This is used by routines which don't need the
 *         - a copy of the data
 */

void Fcnarga(Integer iarg, Integer *s)
{
    int i;

/* get iarg'th argument off stack into s */
    i = rlinkp + iarg;
    if (ssl(TC, i) == LHS)

/* do lazy evaluation of argument(LHS means not yet evaluated) */
        Parfetch(&(ssl(1, i)), nxp[i - 1], &(lxp[i - 1]), &actual,
                 &false, COMMENTSIZE);

    Parcopy(&(ssl(1, i)), s);

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* FCNARGB - put call to parfetch to aid in lazy evaluation */

void Fcnargb(Integer iarg, Integer *s)
{
    int i;

/* get iarg'th argument off stack into s */
    i = rlinkp + iarg;
    if (ssl(TC, i) == LHS)

/* do lazy evaluation of argument(LHS means not yet evaluated) */
        Parfetch(&(ssl(1, i)), nxp[i - 1], &(lxp[i - 1]), &copy,
                 &false, COMMENTSIZE);

    Parcopy(&(ssl(1, i)), s);

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* FCNARGBP - This is used by routines which don't need the data such
 *          - as 'shape', 'length', and the like
 */

void Fcnargbp(Integer iarg, Integer *s)
{
    int i;

/* get iarg'th argument off stack into s */
    i = rlinkp + iarg;
    if (ssl(TC, i) == LHS)

/* do lazy evaluation of argument(LHS means not yet evaluated) */
        Parfetch(&(ssl(1, i)), nxp[i - 1], &(lxp[i - 1]), &nodata,
                 &false, COMMENTSIZE);

    Parcopy(&(ssl(1, i)), s);

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* FCNARGS - return symbolic representation of a stack element
 *         - does this for the iarg'th value in the stack frame.
 */

void Fcnargs(Integer iarg, char *nx, Integer *lx)
{
    int jarg, nxlen;

    nxlen = strlen(nx);
    jarg = iarg;
    if (iarg == 0)
        jarg = -1;              /* function name is below link pointer */

    *lx = lxp[rlinkp - 1 + jarg];
    if (*lx == 0) {
        memset(nx, (int) ' ', nxlen);
        return;
    }

    strncpy(nx, nxp[rlinkp - 1 + jarg], min(nxlen, COMMENTSIZE));

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* FCNARGD - return iarg'th argument, defaulting to name; result to sdi
 *         - stack and stack pointer are no longer necessary.
 *         - position of stack frame is known. arguments
 *         - run from rlinkp+1 to rsp.
 */

void Fcnargd(Integer n, Integer iarg, Integer *sdi, char *name)
{
    Integer ilow[7], ihi[7], icol[7];
    Integer ndim, tc, nwords, jvar, ndb, len, lentemp;
    Integer *idata, *jdata;
    char tempname[65];
    char *ptempname = tempname; /* needed to pass a static char array to Fortran */
    logical gottemp;
    int i;
    Address fwa;
    static char
        errmsg1[] = "Internal error in fcnargd, please report this.";

    if (iarg <= n) {
        Fcnarga(iarg, sdi);
        if (sdi[TC - 1] != 0)
            return;
    }

    if (Rtfinder(name, &jvar, &ndb, &tc, "fcnargd") == OK) {
        gottemp = (logical) 0;
        if (tc == FUNCTION_TYPE) {
            strcpy(tempname, "par.$fcnargd=");
            strcat(tempname, name);
            lentemp = strlen(tempname);
            gottemp = (logical) 1;
            Parsestr((Address *) &ptempname, &lentemp);
            if (Rtfinder("par.$fcnargd", &jvar, &ndb, &tc, "fcnargd") ==
                ERR) {
                BasError(errmsg1, strlen(errmsg1));
                Kaboom(0);
            }
        }

        Rtxdb(jvar, ndb, &fwa, &ndim, ilow, ihi, icol, 2);
        sdi[N - 1] = ndim;
        sdi[TC - 1] = tc;
        for (i = 0; i < ndim; i++) {
            sdi[N1 - 1 + i] = icol[i];
            sdi[M1 - 1 + i] = ilow[i];
            sdi[I1 - 1 + i] = 1;
        }

        Parget(sdi);
        idata = (Integer *) fwa;
        jdata = (Integer *) Paraddr(sdi);
        len = Parlen(sdi);
        nwords = Utsizeof(&len, &tc);
        memcpy(jdata, idata, nwords * sizeof(Integer));
        if (gottemp == (logical) 1)
            Rtstgdel(jvar, ndb);
    } else
        Kaboom(0);

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* PARCHECK_FIRST_ARG - prepare a stack element SYI and check the
 *                    - first argument of a function call for integrity
 */

static int Parcheck_first_arg(Integer *syi, char *fname, Integer nargs,
                              f_entry *thisf)
{
    int i;
    Integer atyp;
    char errmsg[MAXOUT + 1];
    char expected_type[80];
    Integer *code;
    static char
        errmsg1[] = "Something is seriously wrong.";

    if (nargs == 0)
        for (i = 0; i < SSWIDTH; i++)
            syi[i] = 0;

    else if (nargs > 0) {

        atyp = thisf->arg1typ;
        code = NULL;
        if (thisf->info != NULL)
            code = (Integer *) thisf->info[0];

/* pick up the first argument */
        if (code == &BASNXTSQ)
            Fcnarga(1, syi);

        else if (code == &STRLEN)
            Fcnargbp(1, syi);

        else
            Fcnargb(1, syi);

/* check it for integrity unless we don't care */
        if ((atyp != ANY) && (atyp != IGNORE)) {
            errmsg[0] = '\0';
            expected_type[0] = '\0';

            if (atyp == CINTEGER) {
                if (syi[TC - 1] != INTEGER)
                    strcpy(expected_type, "integer");
            } else if (atyp == I_TO_D) {
                if (syi[TC - 1] < INTEGER || syi[TC - 1] > DOUBLE)
                    strcpy(expected_type, "integer, real, or double");
            } else if (atyp == R_AND_D) {
                if (syi[TC - 1] != REAL && syi[TC - 1] != DOUBLE)
                    strcpy(expected_type, "real or double");
            } else if (atyp == RN) {
                if (syi[TC - 1] != RANGE)
                    strcpy(expected_type, "range");
            } else if (atyp == I_TO_D_AND_RN) {
                if (syi[TC - 1] < INTEGER ||
                    (syi[TC - 1] > DOUBLE && syi[TC - 1] != RANGE))
                    strcpy(expected_type,
                           "integer, real, double, or range");
            } else if (atyp == C_AND_DC) {
                if (syi[TC - 1] != COMPLEX && syi[TC - 1] != DOUBLE_C)
                    strcpy(expected_type, "complex or double complex");
            } else if (atyp == I_TO_DC) {
                if (syi[TC - 1] < INTEGER || syi[TC - 1] > DOUBLE_C)
                    strcpy(expected_type, "numerical type");
            } else if (atyp == I_TO_DC_AND_RN) {
                if (syi[TC - 1] < INTEGER ||
                    (syi[TC - 1] > DOUBLE_C && syi[TC - 1] != RANGE))
                    strcpy(expected_type, "numerical type or range");
            } else if (atyp == CLOGICAL) {
                if (syi[TC - 1] != LOGICAL)
                    strcpy(expected_type, "logical");
            } else if (atyp == I_TO_L) {
                if (syi[TC - 1] < INTEGER || syi[TC - 1] > LOGICAL)
                    strcpy(expected_type, "numerical or logical type");
            } else if (atyp == I_TO_L_AND_RN) {
                if (syi[TC - 1] < INTEGER || syi[TC - 1] > RANGE)
                    strcpy(expected_type, "numerical, logical, or range");
            } else if (atyp == STR1) {
                if (syi[TC - 1] != -1)
                    strcpy(expected_type, "character *1");
            } else if (atyp == STR) {
                if (syi[TC - 1] > 0)
                    strcpy(expected_type, "character");
            } else if (atyp == CFWA) {
                if (syi[TC - 1] != FWA)
                    strcpy(expected_type, "address");
            } else {
                sprintf(errmsg,
                        "parbfcn: unknown first argument type %ld!",
                        (long) atyp);
                BasError(errmsg, strlen(errmsg));
                BasError(errmsg1, strlen(errmsg1));
                return (ERR);
            }

            if (expected_type[0] != '\0') {
                sprintf(errmsg,
                        "parbfcn: %s was expected as type of first argument to %s.",
                        expected_type, fname);
                BasError(errmsg, strlen(errmsg));
                return (ERR);
            }
        }
    }

    return (OK);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* PARBFCN - reduces the production
 *         -     <rid>::= reference <args>
 *         - for the builtin functions
 */

Integer Parbfcn(Integer nargs, char *fname, Integer *sxi)
{
    int ierr, i;
    Integer syi[SSWIDTH];
    f_entry *thisf;
    Integer (*hand) (Integer *sxi, f_entry *thisf,
                     Integer nargs, Integer *syi);
    char errmsg[MAXOUT + 1];

    thisf = Parlookup_fcn(fname);

    if (Parcheck_first_arg(syi, fname, nargs, thisf) == ERR)
        return (ERR);

/* At this point, if there is a first argument, it has been
 * checked for integrity, and we are now ready to proceed
 * to the computational routines
 */

    hand = thisf->handler;
    if (hand != NULL) {
        ierr = (*hand) (sxi, thisf, nargs, syi);
    } else {
        sprintf(errmsg, "parbfcn: unknown function %s in table",
                thisf->fname);
        BasError(errmsg, strlen(errmsg));
        return ERR;
    }

    for (i = 0; i < 7; i++)
        sxi[I1 + i - 1] = 1;

    return ierr;
}

/*--------------------------------------------------------------------------*/

/*                           MATHLIB ROUTINES                               */

/*--------------------------------------------------------------------------*/

/* RVECMIN - this subroutine(partially) implements the Mathlib routine
 *         - minaf(LCSD-422)
 *         - input: rarray= real array
 *         -        ifirst= the first element in the array
 *         -        ilast= the last element in the array
 *         - function output: the minimum number in the array
 */

float Rvecmin(float *rarray, Integer ifirst, Integer ilast)
{
    float rmin;
    int i;

    rmin = rarray[ifirst - 1];

    for (i = ifirst; i < ilast; i++)
        if (rarray[i] < rmin)
            rmin = rarray[i];

    return rmin;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* DVECMIN - this subroutine(partially) implements the Mathlib routine
 *         - minaf(LCSD-422)
 *         - based on Mike McNeill 3/86 version of aminmx
 *         - input: darray= double array
 *         -        ifirst= the first element in the array
 *         -        ilast= the last element in the array
 *         - function output: the minimum number in the array
 */

real8 Dvecmin(real8 *darray, Integer ifirst, Integer ilast)
{
    real8 dmin;
    int i;

    dmin = darray[ifirst - 1];

    for (i = ifirst; i < ilast; i++)
        if (darray[i] < dmin)
            dmin = darray[i];

    return dmin;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* IVECMIN - this subroutine(partially) implements the Mathlib routine
 *         - minaf(LCSD-422)
 *         - based on Mike McNeill 3/86 version of aminmx
 *         - input: iarray= integer array
 *         -        ifirst= the first element in the array
 *         -        ilast= the last element in the array
 *         - function output: the minimum number in the array
 */

Integer Ivecmin(Integer *iarray, Integer ifirst, Integer ilast)
{
    Integer imin;
    int i;

    imin = iarray[ifirst - 1];

    for (i = ifirst; i < ilast; i++)
        if (iarray[i] < imin)
            imin = iarray[i];

    return imin;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* RVECMAX - this subroutine(partially) implements the Mathlib routine
 *         - maxaf(LCSD-422)
 *         - input: rarray= real array
 *         -        ifirst= the first element in the array
 *         -        ilast= the last element in the array
 *         - function output: the maximum number in the array
 */

float Rvecmax(float *rarray, Integer ifirst, Integer ilast)
{
    float rmax;
    int i;

    rmax = rarray[ifirst - 1];

    for (i = ifirst; i < ilast; i++)
        if (rarray[i] > rmax)
            rmax = rarray[i];

    return rmax;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* DVECMAX - this subroutine(partially) implements the Mathlib routine
 *         - maxaf(LCSD-422)
 *         - input: darray= double array
 *         -        ifirst= the first element in the array
 *         -        ilast= the last element in the array
 *         - function output: the maximum number in the array
 */

real8 Dvecmax(real8 *darray, Integer ifirst, Integer ilast)
{
    real8 dmax;
    int i;

    dmax = darray[ifirst - 1];

    for (i = ifirst; i < ilast; i++)
        if (darray[i] > dmax)
            dmax = darray[i];

    return dmax;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* IVECMAX - this subroutine(partially) implements the Mathlib routine
 *         - maxaf(LCSD-422)
 *         - input: iarray= integer array
 *         -        ifirst= the first element in the array
 *         -        ilast= the last element in the array
 *         - function output: the maximum number in the array
 */

Integer Ivecmax(Integer *iarray, Integer ifirst, Integer ilast)
{
    Integer imax;
    int i;

    imax = iarray[ifirst - 1];

    for (i = ifirst; i < ilast; i++)
        if (iarray[i] > imax)
            imax = iarray[i];

    return imax;
}

/*--------------------------------------------------------------------------*/

/*                              MISCELLANEOUS                               */

/*--------------------------------------------------------------------------*/

/* PAR2GET - ? */

Integer Par2get(char *fname, Integer *syi, Integer nargs,
                Integer *dim, Integer *sxi, Integer *dimlens,
                Integer *dimstride)
{
    Integer szi[SSWIDTH], npoints, i;
    char errmsg[OUTLINE];

    npoints = Parlen(syi);

/* this case is detected as a special case by the caller */
    if (npoints == 0)
        return 0;

    if (nargs == 1) {
        sxi[N - 1] = 0;
        sxi[TC - 1] = syi[TC - 1];
        *dim = 0;
        dimlens[0] = 0;
        dimstride[0] = 0;
    } else if (nargs == 2) {
        Fcnargb(2, szi);
        if (Parint(szi, dim) == ERR)
            return ERR;

/* release space for argument */
        Parrel(szi);
        if (*dim > syi[N - 1] || *dim < 1) {
            sprintf(errmsg, "%s: the second argument is out of range.",
                    fname);
            BasError(errmsg, strlen(errmsg));
            Kaboom(0);
        }

        sxi[N - 1] = syi[N - 1] - 1;
        sxi[TC - 1] = syi[TC - 1];
        dimstride[0] = 1;

        for (i = 0; i < syi[N - 1]; i++) {
            dimlens[i] = syi[N1 + i - 1];
            dimstride[i + 1] = dimstride[i] * dimlens[i];
            if (i < *dim - 1) {
                sxi[N1 + i - 1] = syi[N1 + i - 1];
                sxi[M1 + i - 1] = syi[M1 + i - 1];
            } else if (i > *dim - 1) {
                sxi[N1 + i - 2] = syi[N1 + i - 1];
                sxi[M1 + i - 2] = syi[M1 + i - 1];
            }
        }
    }

/* set up space for the answer */
    Parget(sxi);

    return npoints;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
