/*
 * CFCN1.C
 *
 */

#include "cfcn.h"

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CATAN2 - handler for atan2 function */

Integer Catan2(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer npointsx, npointsy, npoints, idummy, ierr;
    float *rx, *ry, rvalue;
    real8 *dx, *dy, dvalue;
    static char
        errmsg27[] = "atan2: argument lengths incompatible.",
        errmsg28[] = "atan2: bad argument type.";

    Fcnargb(2, sxi);
    npointsy = Parlen(syi);
    npointsx = Parlen(sxi);
    if (npointsx != npointsy && npointsx != 1 && npointsy != 1) {
        BasError(errmsg27, strlen(errmsg27));
        return ERR;
    }
    if (sxi[TC - 1] > DOUBLE || sxi[TC - 1] < 0) {
        BasError(errmsg28, strlen(errmsg28));
        return ERR;
    }
    if (sxi[TC - 1] == INTEGER)
        idummy = Parcoerc(sxi, &myreal);
    if (syi[TC - 1] == INTEGER)
        idummy = Parcoerc(syi, &myreal);
    if (sxi[TC - 1] > syi[TC - 1])
        idummy = Parcoerc(syi, &(sxi[TC - 1]));
    if (syi[TC - 1] > sxi[TC - 1])
        idummy = Parcoerc(sxi, &(syi[TC - 1]));
    if (npointsx == 1) {
        if (syi[TC - 1] == REAL) {
            rx = (float *) Paraddr(sxi);
            rvalue = rx[0];
        } else if (syi[TC - 1] == DOUBLE) {
            dx = (real8 *) Paraddr(sxi);
            dvalue = dx[0];
        }
        sxi[N1 - 1] = npointsy;
        sxi[N - 1] = 1;
        /* ZCM 11/13/95 discovered by basltest: */
        Parrel(sxi);
        Parget(sxi);
    }
    if (syi[TC - 1] == REAL) {
        rx = (float *) Paraddr(sxi);
        ry = (float *) Paraddr(syi);
        ierr = Prratan2(rx, ry, &rvalue, &npointsx, &npointsy, &npoints);
        if (ierr == ERR)
            return (ERR);
    } else if (syi[TC - 1] == DOUBLE) {
        dx = (real8 *) Paraddr(sxi);
        dy = (real8 *) Paraddr(syi);
        ierr = Prdatan2(dx, dy, &dvalue, &npointsx, &npointsy, &npoints);
        if (ierr == ERR)
            return (ERR);
    }
    Parrel(syi);
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CDCMPLX - handler for the dcmplx builtin */

Integer Cdcmplx(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer szi[SSWIDTH];
    Integer npointsx, npointsy, npointsz, idummy;
    real8 *dx, *dy, *dcx;
    int i;
    static char
        errmsg29[] = "cmplx, dcmplx: argument lengths incompatible.",
        errmsg30[] = "dcmplx: cannot convert a double complex argument.",
        errmsg31[] = "dcmplx: cannot convert a complex argument.";

    Parcopy(syi, sxi);

    if (nargs == 1) {

/* we have a copy of the stack element, but we need a copy of the space
 * so that the Parrel below doesn't kill us
 */
        Cobjcpy(sxi, syi);

        if (Parcoerc(sxi, &mydcmplx) == ERR)
            return ERR;
    } else if (nargs == 2) {
        Fcnargb(2, szi);
        npointsy = Parlen(syi);
        npointsz = Parlen(szi);
        npointsx = npointsy > npointsz ? npointsy : npointsz;

        if (syi[TC - 1] == DOUBLE_C || szi[TC - 1] == DOUBLE_C) {
            BasError(errmsg29, strlen(errmsg29));
            return ERR;
        } else if (syi[TC - 1] == COMPLEX || szi[TC - 1] == COMPLEX) {
            BasError(errmsg30, strlen(errmsg30));
            return ERR;
        }

        idummy = Parcoerc(syi, &mydble);
        if (Parcoerc(szi, &mydble) == ERR) {
            BasError(errmsg31, strlen(errmsg31));
            return ERR;
        }

        if (npointsy <= npointsz)
            Parcopy(szi, sxi);
        else
            Parcopy(syi, sxi);
        sxi[TC - 1] = DOUBLE_C;
        Parget(sxi);

        dcx = (real8 *) Paraddr(sxi);
        dy = (real8 *) Paraddr(szi);
        dx = (real8 *) Paraddr(syi);
        if (npointsy == 1)
            for (i = 0; i < npointsx; i++) {
                dcx[2 * i] = dx[0];
                dcx[2 * i + 1] = dy[i];
        } else if (npointsz == 1 && npointsy != 1)
            for (i = 0; i < npointsx; i++) {
                dcx[2 * i] = dx[i];
                dcx[2 * i + 1] = dy[0];
        } else
            for (i = 0; i < npointsx; i++) {
                dcx[2 * i] = dx[i];
                dcx[2 * i + 1] = dy[i];
            }
    }
    Parrel(szi);
    Parrel(syi);

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CCMPLX - handler for the cmplx builtin
 *        - Coerce data to complex type
 */

Integer Ccmplx(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer npointsx, npointsy, npointsz, typ1, typ2;
    Integer szi[SSWIDTH];
    char rout[7], errmsg[MAXOUT];
    float *cx, *rx, *ry;
    double *dcx, *dx, *dy;
    int i;
    static char
        errmsg29[] = "cmplx, dcmplx: argument lengths incompatible.",
        errmsg31[] = "dcmplx: cannot convert a complex argument.";

    Parcopy(syi, sxi);

    if (nargs == 1) {

/* we have a copy of the stack element, but we need a copy of the space
 * so that the Parrel below doesn't kill us
 */
        Cobjcpy(sxi, syi);

/* force double complex if argument is double
 * and platform supports double complex
 */
        if (sxi[TC - 1] == DOUBLE) {
            if (Parcoerc(sxi, &mydcmplx) == ERR)
                return ERR;
        } else if (Parcoerc(sxi, &mycmplx) == ERR)
            return ERR;

    } else if (nargs == 2) {
        Fcnargb(2, szi);
        npointsy = Parlen(syi);
        npointsz = Parlen(szi);
        npointsx = npointsy > npointsz ? npointsy : npointsz;
        if (npointsy != npointsz)
            if (npointsy != 1 && npointsz != 1) {
                BasError(errmsg29, strlen(errmsg29));
                return ERR;
            }

        if (syi[TC - 1] == COMPLEX || szi[TC - 1] == COMPLEX ||
            syi[TC - 1] == DOUBLE_C || szi[TC - 1] == DOUBLE_C) {
            BasError(errmsg31, strlen(errmsg31));
            return ERR;
        }

/* force double complex if either argument is double
 * and platform supports double complex
 */
        if (syi[TC - 1] == DOUBLE || szi[TC - 1] == DOUBLE) {
            typ1 = DOUBLE;
            typ2 = DOUBLE_C;
            strcpy(rout, "dcmplx");
        } else {
            typ1 = REAL;
            typ2 = COMPLEX;
            strcpy(rout, "cmplx");
        }

        if (Parcoerc(syi, &typ1) == ERR) {
            sprintf(errmsg, "%s: first argument has illegal type.", rout);
            BasError(errmsg, strlen(errmsg));
            return ERR;
        }

        if (Parcoerc(szi, &typ1) == ERR) {
            sprintf(errmsg, "%s: second argument has illegal type.", rout);
            BasError(errmsg, strlen(errmsg));
            return ERR;
        }

        if (npointsy <= npointsz)
            Parcopy(szi, sxi);
        else
            Parcopy(syi, sxi);

        sxi[TC - 1] = typ2;
        Parget(sxi);

        if (typ2 == COMPLEX) {
            cx = (float *) Paraddr(sxi);
            ry = (float *) Paraddr(szi);
            rx = (float *) Paraddr(syi);
            if (npointsy == 1)
                for (i = 0; i < npointsx; i++) {
                    cx[2 * i] = rx[0];
                    cx[2 * i + 1] = ry[i];
            } else if (npointsz == 1 && npointsy != 1)
                for (i = 0; i < npointsx; i++) {
                    cx[2 * i] = rx[i];
                    cx[2 * i + 1] = ry[0];
            } else
                for (i = 0; i < npointsx; i++) {
                    cx[2 * i] = rx[i];
                    cx[2 * i + 1] = ry[i];
                }

        } else {                /* ASSERT: typ2==DOUBLE_C */
            dcx = (real8 *) Paraddr(sxi);
            dy = (real8 *) Paraddr(szi);
            dx = (real8 *) Paraddr(syi);
            if (npointsy == 1)
                for (i = 0; i < npointsx; i++) {
                    dcx[2 * i] = dx[0];
                    dcx[2 * i + 1] = dy[i];
            } else if (npointsz == 1 && npointsy != 1)
                for (i = 0; i < npointsx; i++) {
                    dcx[2 * i] = dx[i];
                    dcx[2 * i + 1] = dy[0];
            } else
                for (i = 0; i < npointsx; i++) {
                    dcx[2 * i] = dx[i];
                    dcx[2 * i + 1] = dy[i];
                }
        }
    }

    Parrel(szi);
    Parrel(syi);

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CWHERE - handler for the where builtin */

Integer Cwhere(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer szi[SSWIDTH], szzi[SSWIDTH], npointsy, npointsz, npointszz;
    Integer *ix, *iz, *izz;
    float *rx, *rz, *rzz, *cx, *cz, *czz;
    real8 *dx, *dz, *dzz, *dcx, *dcz, *dczz;
    logical *ly;
    int i, j, k;
    static char
        errmsg33[] = "where: lengths of arg1 and arg2 don't match.",
        errmsg34[] = "where: lengths of arg1 and arg3 don't match.",
        errmsg35[] = "where: arg2 not coerceable to arg1 type.",
        errmsg36[] = "where: arg1 not coerceable to arg2 type.",
        errmsg37[] = "where: arg2, arg3 are incorrect types.";

    ly = (logical *) Paraddr(syi);      /* Type already checked */
    Fcnargb(2, szi);
    npointsy = Parlen(syi);
    npointsz = Parlen(szi);
    if (npointsz != 1 && npointsz != npointsy) {
        BasError(errmsg33, strlen(errmsg33));
        return ERR;
    }
    if (nargs == 2) {
        sxi[N - 1] = 1;
        sxi[TC - 1] = szi[TC - 1];
        sxi[M1 - 1] = 1;
        sxi[N1 - 1] = 0;
        for (i = 0; i < npointsy; i++)
            if (ly[i])
                sxi[N1 - 1]++;
    } else if (nargs == 3) {
        Fcnargb(3, szzi);
        npointszz = Parlen(szzi);
        if (npointszz != 1 && npointszz != npointsy) {
            BasError(errmsg34, strlen(errmsg34));
            return ERR;
        }
        if (szzi[TC - 1] < szi[TC - 1]) {
            if (Parcoerc(szzi, &(szi[TC - 1])) == ERR) {
                BasError(errmsg35, strlen(errmsg35));
                return ERR;
            }
        } else if (Parcoerc(szi, &(szzi[TC - 1])) == ERR) {
            BasError(errmsg36, strlen(errmsg36));
            return ERR;
        }
        Parcopy(syi, sxi);
        sxi[TC - 1] = szi[TC - 1];
    }
    /* #### set up space for the answer */
    Parget(sxi);
    if (szi[TC - 1] == INTEGER) {
        ix = (Integer *) Paraddr(sxi);
        iz = (Integer *) Paraddr(szi);
        if (nargs == 2) {
            for (i = 0, j = 0; i < npointsy; i++)
                if (ly[i]) {
                    k = min(i, (int) npointsz - 1);
                    ix[j] = iz[k];
                    j += 1;
                }
        } else {
            izz = (Integer *) Paraddr(szzi);
            for (i = 0; i < npointsy; i++)
                if (ly[i]) {
                    k = min(i, (int) npointsz - 1);
                    ix[i] = iz[k];
                } else {
                    k = min(i, (int) npointszz - 1);
                    ix[i] = izz[k];
                }
        }
    } else if (szi[TC - 1] == REAL) {
        rx = (float *) Paraddr(sxi);
        rz = (float *) Paraddr(szi);
        if (nargs == 2) {
            for (i = 0, j = 0; i < npointsy; i++)
                if (ly[i]) {
                    k = min(i, (int) npointsz - 1);
                    rx[j] = rz[k];
                    j += 1;
                }
        } else {
            rzz = (float *) Paraddr(szzi);
            for (i = 0; i < npointsy; i++)
                if (ly[i]) {
                    k = min(i, (int) npointsz - 1);
                    rx[i] = rz[k];
                } else {
                    k = min(i, (int) npointszz - 1);
                    rx[i] = rzz[k];
                }
        }
    } else if (szi[TC - 1] == DOUBLE) {
        dx = (real8 *) Paraddr(sxi);
        dz = (real8 *) Paraddr(szi);
        if (nargs == 2) {
            for (i = 0, j = 0; i < npointsy; i++)
                if (ly[i]) {
                    k = min(i, (int) npointsz - 1);
                    dx[j] = dz[k];
                    j += 1;
                }
        } else {
            dzz = (real8 *) Paraddr(szzi);
            for (i = 0; i < npointsy; i++)
                if (ly[i]) {
                    k = min(i, (int) npointsz - 1);
                    dx[i] = dz[k];
                } else {
                    k = min(i, (int) npointszz - 1);
                    dx[i] = dzz[k];
                }
        }
    }
#ifdef HAS_DOUBLE_COMPLEX
    else if (szi[TC - 1] == COMPLEX) {
#else
    else if (szi[TC - 1] == COMPLEX || szi[TC - 1] == DOUBLE_C) {
#endif
        cx = (float *) Paraddr(sxi);
        cz = (float *) Paraddr(szi);
        if (nargs == 2)
            for (i = 0, j = 0; i < npointsy; i++) {
                if (ly[i]) {
                    k = min(i, (int) npointsz - 1);
                    cx[2 * j] = cz[2 * k];
                    cx[2 * j + 1] = cz[2 * k + 1];
                    j += 1;
                }
        } else {
            czz = (float *) Paraddr(szzi);
            for (i = 0, j = 0; i < npointsy; i++)
                if (ly[i]) {
                    k = min(i, (int) npointsz - 1);
                    cx[2 * i] = cz[2 * k];
                    cx[2 * i + 1] = cz[2 * k + 1];
                } else {
                    k = min(i, (int) npointsz - 1);
                    cx[2 * i] = czz[2 * k];
                    cx[2 * i + 1] = czz[2 * k + 1];
                }
        }
    }
#ifdef HAS_DOUBLE_COMPLEX
    else if (szi[TC - 1] == DOUBLE_C) {
        dcx = (real8 *) Paraddr(sxi);
        dcz = (real8 *) Paraddr(szi);
        if (nargs == 2)
            for (i = 0, j = 0; i < npointsy; i++) {
                if (ly[i]) {
                    k = min(i, (int) npointsz - 1);
                    dcx[2 * j] = dcz[2 * k];
                    dcx[2 * j + 1] = dcz[2 * k + 1];
                    j += 1;
                }
        } else {
            dczz = (real8 *) Paraddr(szzi);
            for (i = 0, j = 0; i < npointsy; i++)
                if (ly[i]) {
                    k = min(i, (int) npointsz - 1);
                    dcx[2 * i] = dcz[2 * k];
                    dcx[2 * i + 1] = dcz[2 * k + 1];
                } else {
                    k = min(i, (int) npointsz - 1);
                    dcx[2 * i] = dczz[2 * k];
                    dcx[2 * i + 1] = dczz[2 * k + 1];
                }
        }
    }
#endif
    else {
        BasError(errmsg37, strlen(errmsg37));
        return ERR;
    }
    /* release space for arguments */
    Parrel(syi);
    Parrel(szi);
    if (nargs >= 3)
        Parrel(szzi);
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
