/*
 * cfcn11.c
 *
 */

#include "cfcn.h"

extern Integer
    F77_ID(list_, list, LIST) (Integer *fd, char *name, Integer n);

extern void
    F77_ID(pardoluf_, pardoluf, PARDOLUF) (Integer *ndb),
    F77_ID(edtmcnam_, edtmcnam, EDTMCNAM) (Integer *fd);

#define fgetius F77_ID(getius_, getius, GETIUS)

extern Integer getius(void);

extern void
    Glblpack(void),
    Glbpknam(Integer i, char *pknam),
    Rtggname(Integer jgrp, Integer ndb, char *name);

extern Integer
    Rtgtvar(void),
    Rtgngrp(Integer ndb),
    Rtxcomg(Integer jgrp, Integer ndb, Integer icom, char *comment),
    Rtgnvar1(Integer jgrp, Integer ndb), Rtgtc(Integer jvar, Integer ndb);

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CMINMAXB - handler for the min and max builtins */

Integer Cminmaxb(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer fcode;

    fcode = GET_FCODE(thisf);

    if (nargs > 1) {
        if (fcode == MAX) {
            if (Fcndomax(&nargs, sxi) == ERR)
                return ERR;
        } else {
            if (Fcndomin(&nargs, sxi) == ERR)
                return ERR;
        }
    } else
        Cvminvmax(sxi, thisf, nargs, syi);

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CTYPE - handler for the type builtin */

Integer Ctype(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    int i;
    char *chx;
    char typestr[VARNAMESIZE + 1];

/* 1st arg hasn't been fetched yet */
    Fcnargbp(1, sxi);

/* Utcodstr returns "character*3" for string of length 3. */
    Utcodstr(sxi[TC - 1], typestr);
    if (strlen(typestr) >= 9 && !strncmp(typestr, "character", 9))
        strcpy(typestr, "character");

    i = strlen(typestr);

    Parkill(sxi);

    sxi[N - 1] = 0;
    sxi[TC - 1] = -i;

/* so it knows address is in literal table.(ZCM 4/10/95): */
    sxi[NAML - 1] = 1;
    Parget(sxi);

    chx = (char *) Paraddr(sxi);
    strncpy(chx, typestr, i);

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CUTYPE - handler for the utype builtin */

Integer Cutype(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer i, idummy, ivalue;
    char mcudlim[VARNAMESIZE + 1];
    static char
        errmsg51[] = "utype: no more than 10 user delimiters allowed.";

/* collect the user-defined delimiters */
    if (nargs > 10) {
        BasError(errmsg51, strlen(errmsg51));
        Kaboom(0);
    }

    for (i = 1; i <= nargs; i++) {
        if (i > 1)
            Fcnargb(i, sxi);
        else
            Parcopy(syi, sxi);

        INIT_BLANK_STRING(mcudlim, VARNAMESIZE);

        idummy = Parname(sxi, mcudlim);
        ivalue = (Integer) i;
        Mcsdlim(mcudlim, &ivalue, idummy);

        if (i != nargs)
            Parkill(sxi);
    }

    Mcsdlimn(&nargs);

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CFROMONE - handler for the fromone builtin */

Integer Cfromone(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    int i, ndim;
    static char errmsg40[] = "fromone: argument is FUNCTION_TYPE?!";

    Parcopy(syi, sxi);

    if (sxi[TC - 1] == FUNCTION_TYPE) {
        BasError(errmsg40, strlen(errmsg40));
        return (ERR);
    }

    ndim = sxi[N - 1];
    for (i = 0; i < ndim; i++) {
        sxi[M1 + i - 1] = 1;
        sxi[I1 + i - 1] = 1;
    }

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CCHECKSUM - handler for checksum builtin */

Integer Cchecksum(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer ni, bpi, nbts;
    Integer *ix;
    char *arr;

    nbts = 29;

    Fcnargb(1, syi);

    arr = (char *) Paraddr(syi);
    ni  = Parlen(syi);
    bpi = Utbytesin(syi[TC-1]);

    sxi[N - 1]  = 0;
    sxi[TC - 1] = INTEGER;
    Parget(sxi);

    ix = (Integer *) Paraddr(sxi);
    ix[0] = PM_checksum_array(arr, ni, bpi, nbts);

    Parrel(syi);

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CLENGTH - handler for the length builtin */

Integer Clength(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer npoints, *ix;
    static char errmsg52[] = "length: illegal argument type.";

/* data is not actually needed for this function */
    Fcnargbp(1, syi);

    if (syi[TC - 1] == FUNCTION_TYPE) {
        BasError(errmsg52, strlen(errmsg52));
        return ERR;
    }

    npoints = Parlen(syi);
    Parkill(syi);

    sxi[N - 1] = 0;
    sxi[TC - 1] = INTEGER;
    Parget(sxi);

    ix = (Integer *) Paraddr(sxi);
    ix[0] = npoints;

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CSTRLEN - handler for the strlen builtin */

Integer Cstrlen(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer *ix;
    static char errmsg53[] = "strlen:  scaler argument expected.";

    if (Parlen(syi) != 1) {
        BasError(errmsg53, strlen(errmsg53));
        return ERR;
    }

    sxi[N - 1] = 0;
    sxi[TC - 1] = INTEGER;
    Parget(sxi);

    ix = (Integer *) Paraddr(sxi);
    ix[0] = -syi[TC - 1];
    Parkill(syi);

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CSQUEEZE - handler for the squeeze builtin */

Integer Csqueeze(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Parcopy(syi, sxi);
    Pardim(sxi);

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CDBLE - handler for the dble builtin */

Integer Cdble(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer idummy;

    Parcopy(syi, sxi);
    idummy = Parcoerc(sxi, &mydble);

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CTRANSPOSE - handler for the transpose builtin */

Integer Ctranspose(Integer *sxi, f_entry *thisf,
                   Integer nargs, Integer *syi)
{
    Fcnargb(1, sxi);
    Partrans(sxi);

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CSTRUCT - handler for the struct builtin */

Integer Cstruct(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer i, idummy;

    Parstru0(sxi);

    for (i = 1; i <= nargs; i++) {
        Fcnargb(i, syi);
        idummy = Struadd(sxi, syi, 0);
        Parrel(syi);
    }

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CIINT - handler for the iint builtin */

Integer Ciint(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    static char errmsg54[] = "int: illegal argument type.";

    Parcopy(syi, sxi);
    if (Parcoerc(sxi, &myint) == ERR) {
        BasError(errmsg54, strlen(errmsg54));
        return ERR;
    }

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CSNGL - handler for the sngl builtin */

Integer Csngl(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer idummy;

    Parcopy(syi, sxi);
    idummy = Parcoerc(sxi, &myreal);

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*  EDTVNAM1 - do the real work for EDTVNAME */

static void edtvnam1(Integer fd, Integer ndb, Integer iflag)
{
    Integer jlo, jhi, i, j, ngrp, icol, nnext;
    char pkg[6], msg[OUTLINE];
    char comment[REMARKSIZE], vnam[VARNAMESIZE];
    char bf[MAXLINE];
    char *ps;

    if (Rtgnvar(ndb) != 0) {
        Glbpknam(ndb, pkg);

        sprintf(bf, "\n----- Package %s -----", pkg);
        Baswline(&fd, bf, strlen(bf));

        ngrp = Rtgngrp(ndb);
        for (i = 1; i <= ngrp; i++) {
            INIT_BLANK_STRING(msg, OUTLINE);

            Rtggname(i, ndb, msg);

            ps = strchr(msg, ' ');
            if (ps != NULL)
                ps[0] = '\0';

            if (Rtxcomg(i, ndb, 1, comment) == ERR)
                comment[0] = '\0';

/* GOTCHA: don't have an example that ever depends on this code
            ifun = index(comment, "and functions");
            if (ifun > 0)
                comment(ifun:ifun+12) = " ";
*/
            sprintf(bf, "\n%s: %s", msg, comment);
            Baswline(&fd, bf, strlen(bf));

            jlo = Rtgnvar1(i, ndb);
            jhi = Rtgnvar1(i + 1, ndb) - 1;
            if ((jhi < jlo) || (iflag == 0))
                continue;

            icol = 0;
            INIT_BLANK_STRING(msg, OUTLINE);
            for (j = jlo; j <= jhi; j++) {
                if (!Rtisdead(j, ndb)) {
                    INIT_BLANK_STRING(vnam, VARNAMESIZE);
                    Rtgvname(j, ndb, vnam);

                    ps = strchr(vnam, ' ');
                    if (ps != NULL)
                        ps[0] = '\0';

                    nnext = strlen(vnam);
                    nnext = min(nnext, 8);

                    if (Rtgtc(j, ndb) != FUNCTION_TYPE) {
                        strncpy(msg + 9 * icol, vnam, nnext);
                        icol++;
                    }

                    if (icol > 8) {
                        Baswline(&fd, msg, 9 * (icol - 1) + nnext);
                        INIT_BLANK_STRING(msg, OUTLINE);
                        icol = 0;
                    }
                }
            }

            if (icol > 0)
                Baswline(&fd, msg, 9 * (icol - 1) + nnext);
        }
    }

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* EDTVNAME - print out all the variables, by group, if iflag = YES
 *          - print out just group names, if iflag = NO
 */

static void edtvname(Integer fd, Integer ndb, Integer iflag)
{
    char bf[MAXLINE];

    if (ndb == 0) {
        if (Rtgtvar() == 0) {
            sprintf(bf, "There are no User or Local variables.");
            Baswline(&fd, bf, strlen(bf));
        } else {
            edtvnam1(fd, LOCALDB, iflag);
            edtvnam1(fd, USERDB, iflag);
        }

    } else {
        edtvnam1(fd, ndb, iflag);
    }

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* LIST_C - edit list requests */

static Integer list_c(Integer fd, char *name)
{
    Integer rv;

    rv = F77_ID(list_, list, LIST) (&fd, name, strlen(name));

    return (rv);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* PARDOLUF_C - list package functions */

static void pardoluf_c(Integer ndb)
{
    F77_ID(pardoluf_, pardoluf, PARDOLUF) (&ndb);

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* EDTMCNAM_C - list package macros */

static void edtmcnam_c(Integer fd)
{
    F77_ID(edtmcnam_, edtmcnam, EDTMCNAM) (&fd);

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CFLIST - handler for the flist builtin */

Integer Cflist(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer i, nc, rv, ndb, no, yes, fd;
    Integer vnc, first;
    char name[VARNAMESIZE], lname[VARNAMESIZE], pkg[VARNAMESIZE];
    char bf[MAXLINE], vname[VARNAMESIZE];
    char *pv;

    INIT_BLANK_STRING(name, VARNAMESIZE);

    nc = Parname(syi, name);
    name[nc] = '\0';

    strcpy(lname, name);
    SC_str_lower(lname);

    rv = OK;
    no = 0;
    yes = 1;
    ndb = syi[N - 1];

    fd = 6;

/* list packages */
    if (strcmp(lname, "packages") == 0) {
        if (nargs < 2)
            Glblpack();
        else {
            first = 1;
            for (i = 2; i <= nargs; i++) {
                INIT_BLANK_STRING(vname, VARNAMESIZE);

                Fcnargs(i, vname, &vnc);
                pv = strtok(vname, "\" \n");
                ndb = Glbpknux(pv);
                if (ndb != 0) {
                    Glblpackn(ndb, first);
                    first = 0;
                }
            }
        }
    }

/* list groups */
    else if (strcmp(lname, "groups") == 0) {
        if (nargs < 2)
            edtvname(fd, 0, no);
        else {
            for (i = 2; i <= nargs; i++) {
                INIT_BLANK_STRING(vname, VARNAMESIZE);

                Fcnargs(i, vname, &vnc);
                pv = strtok(vname, "\" \n");
                ndb = Glbpknux(pv);
                if (ndb != 0)
                    edtvname(fd, ndb, no);
            }
        }
    }

/* list variables */
    else if (strcmp(lname, "variables") == 0) {
        if (nargs < 2)
            edtvname(fd, 0, yes);
        else {
            for (i = 2; i <= nargs; i++) {
                INIT_BLANK_STRING(vname, VARNAMESIZE);

                Fcnargs(i, vname, &vnc);
                pv = strtok(vname, "\" \n");
                ndb = Glbpknux(pv);
                if (ndb != 0)
                    edtvname(fd, ndb, yes);
            }
        }
    }

/* list functions */
    else if (strcmp(lname, "functions") == 0) {
        if (nargs < 2)
            pardoluf_c(0);
        else {
            for (i = 2; i <= nargs; i++) {
                INIT_BLANK_STRING(vname, VARNAMESIZE);

                Fcnargs(i, vname, &vnc);
                pv = strtok(vname, "\" \n");
                ndb = Glbpknux(pv);
                if (ndb != 0)
                    pardoluf_c(ndb);
            }
        }
    }

/* list macros */
    else if (strcmp(lname, "macros") == 0)
        edtmcnam_c(fd);

    else if (ndb == 0)
        rv = list_c(fd, name);

    else {
        Glbpknam(ndb, pkg);
        sprintf(bf, "%s.%s", pkg, name);
        rv = list_c(fd, bf);
    }

/* cleanup */
    Parkill(syi);

    return (OK);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
