/* 
 * cfcn2.c
 *
 */

#include "cfcn.h"

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
static void starfill(char *cp)
{
    while (*cp) {
        *cp++ = '*';
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CNINT - handle the nint builtin function */

Integer Cnint(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer i, npoints;
    Integer *ix;
    float *ry;

    i = Parcoerc(syi, &myreal); /* type already checked */

    Parcopy(syi, sxi);

    sxi[TC - 1] = INTEGER;
    Parget(sxi);

    ix = (Integer *) Paraddr(sxi);
    ry = (float *) Paraddr(syi);

    npoints = Parlen(syi);
    for (i = 0; i < npoints; i++)
        ix[i] = floor(ry[i] + 0.5);

    Parrel(syi);

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CSIGN - handle the sign builtin function */

Integer Csign(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer i, npointsy, npointsx;
    Integer *ix, *iy;
    float *rx, *ry, rmin;
    real8 *dx, *dy, dmin;
    static char
        errmsg38[] = "sign: second argument has illegal type.",
        errmsg39[] = "sign: lengths do not match.";

    Fcnargb(2, sxi);
    npointsx = Parlen(sxi);
    npointsy = Parlen(syi);

/* error check */
    if (syi[TC - 1] < INTEGER || syi[TC - 1] > DOUBLE) {
        BasError(errmsg38, strlen(errmsg38));
        return ERR;
    }

/* if the arguments have different types, force them to agree. */
    if (sxi[TC - 1] > syi[TC - 1]) {
        if (Parcoerc(syi, &(sxi[TC - 1])) == ERR)
            return ERR;
    } else if (Parcoerc(sxi, &(syi[TC - 1])) == ERR)
        return ERR;

/* if both arguments are scalars, or if the first argument is a
 * scalar, do usual  sign  call.  If they are arrays, they must
 * both be the same length
 */
    if ((npointsy != 1) && (npointsx != npointsy)) {
        BasError(errmsg39, strlen(errmsg39));
        return ERR;
    }

    if (sxi[TC - 1] == INTEGER) {
        ix = (Integer *) Paraddr(sxi);
        iy = (Integer *) Paraddr(syi);
        if (npointsy == 1) {
            for (i = 0; i < npointsx; i++) {
                ix[i] = SIGNI(iy[0], ix[i]);
            }
        } else {
            for (i = 0; i < npointsx; i++) {
                ix[i] = SIGNI(iy[i], ix[i]);
            }
        }
    } else if (sxi[TC - 1] == REAL) {
        rx = (float *) Paraddr(sxi);
        ry = (float *) Paraddr(syi);

        if (npointsy == 1) {
            rmin = ry[0];
            for (i = 0; i < npointsx; i++)
                rx[i] = masign(rmin, rx[i]);
        } else
            for (i = 0; i < npointsx; i++)
                rx[i] = masign(ry[i], rx[i]);
    } else if (sxi[TC - 1] == DOUBLE) {
        dx = (real8 *) Paraddr(sxi);
        dy = (real8 *) Paraddr(syi);

        if (npointsy == 1) {
            dmin = dy[0];
            for (i = 0; i < npointsx; i++)
                dx[i] = masign(dmin, dx[i]);
        } else
            for (i = 0; i < npointsx; i++)
                dx[i] = masign(dy[i], dx[i]);
    }

    Parrel(syi);

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CFORMAT - handler for the format builtin */

Integer Cformat(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer sui[SSWIDTH], svi[SSWIDTH], szi[SSWIDTH];
    Integer nd, ts, fw, len, newlen, minlen, zerof, ivalue;
    float rvalue;
    real8 dvalue;
    char str[33], fstr[33];
    char *chx;
    int start, i;
    static char
        errmsg15[] = "FORMAT: number of decimal places must be of type integer.",
        errmsg16[] = "FORMAT: number of decimal places must be positive.",
        errmsg17[] = "FORMAT: type specification must be integer",
        errmsg18[] = "FORMAT: field width not integer.",
        errmsg19[] = "FORMAT: field width too large.",
        errmsg20[] = "FORMAT: conversion of integer to string requires exactly two arguments.",
        errmsg21[] = "FORMAT: conversion of integer to string requires integer second argument.",
        errmsg22[] = "FORMAT: conversion of real to string requires four arguments",
        errmsg23[] = "FORMAT: type specification must be 0 or 1.",
        errmsg24[] = "FORMAT: conversion of double to string requires four arguments.",
        errmsg25[] = "FORMAT: type specification must be 0 , 1, or 2.",
        errmsg26[] = "FORMAT: illegal argument type.";

/* Get next argument, field width */
    Fcnargb(2, szi);

/* revision by ZCM 9/15/95: if 4th argument is 2,
 * coerce the number to real and print it out with e format.
 */
    if (nargs >= 3) {

/* get and check third argument */
        Fcnargb(3, sui);
        if (Parint(sui, &nd) != OK) {
            BasError(errmsg15, strlen(errmsg15));
            return ERR;
        } else if (nd < 0) {
            BasError(errmsg16, strlen(errmsg16));
            return ERR;
        }
    }

    if (nargs >= 4) {
/* get and check fourth argument */
        Fcnargb(4, svi);
        if (Parint(svi, &ts) != OK) {
            BasError(errmsg17, strlen(errmsg17));
            return ERR;
        }
        if (ts == 2 && Pardble(syi, &dvalue) == ERR)
            return ERR;
    }

    /* check that field width is legal */
    if (Parint(szi, &fw) != OK) {
        BasError(errmsg18, strlen(errmsg18));
        return ERR;
    }

    zerof = 0;
    if (labs(fw) > 32) {
        BasError(errmsg19, strlen(errmsg19));
        return ERR;
    } else if (fw == 0)         /* will skip leading blanks */
        len = 32;
    else if (fw < 0) {          /* zero-fill to n digits */
        fw = labs(fw);
        len = fw;
        zerof = 1;
    } else
        len = fw;

/* call subroutines to do conversion */
    if (syi[TC - 1] == INTEGER) {
        if (nargs != 2) {
            BasError(errmsg20, strlen(errmsg20));
            return ERR;
        } else if (Parint(syi, &ivalue) != OK) {
            BasError(errmsg21, strlen(errmsg21));
            return ERR;
        }
        sprintf(fstr, "%s%ld%s", "%", (long) len, "d");
        sprintf(str, fstr, ivalue);
        /* Emulate fortran string formatting behavior. */
        if (strlen(str) > len) {
            starfill(str);
        }
    } else if (syi[TC - 1] == REAL) {
        char *fch;

        if (nargs != 4) {
            BasError(errmsg22, strlen(errmsg22));
            return ERR;
        }

/* convert to real number */
        if (Parreal(syi, &rvalue) == ERR)
            return (ERR);
        if ((ts != 0) && (ts != 1)) {
            BasError(errmsg23, strlen(errmsg23));
            return ERR;
        }
        fch = (ts == 0) ? "e" : "f";
        sprintf(fstr, "%s%ld.%ld%s", "%", (long) len, (long) nd, fch);
        sprintf(str, fstr, rvalue);
        /* Emulate fortran string formatting behavior. */
        if (strlen(str) > len) {
            starfill(str);
        }

/* release storage for args 3 and 4 */
        Parrel(sui);
        Parrel(svi);
    } else if (syi[TC - 1] == DOUBLE) {
        char *fch;

        if (nargs != 4) {
            BasError(errmsg24, strlen(errmsg24));
            return ERR;
        }

/* convert to double precision number */
        if (Pardble(syi, &dvalue) == ERR)
            return (ERR);
        if ((ts != 0) && (ts != 1) && (ts != 2)) {
            BasError(errmsg25, strlen(errmsg25));
            return ERR;
        }
        fch = (ts == 1) ? "f" : "e";
        sprintf(fstr, "%s%ld.%ld%s", "%", (long) len, (long) nd, fch);
        sprintf(str, fstr, dvalue);
        /* Emulate fortran string formatting behavior. */
        if (strlen(str) > len) {
            starfill(str);
        }

/* change 'e' to 'd' in str, unless ts was 2 */
        chx = strchr(str, 'e');
        if (chx && (ts != 2))
            *chx = 'd';

/* release storage for args 3 and 4 */
        Parrel(sui);
        Parrel(svi);
    } else {
        BasError(errmsg26, strlen(errmsg26));
        return ERR;
    }

/* release storage for args 3 and 4 */
    Parrel(syi);
    Parrel(szi);

/* create resultant stack element */
    if (fw != 0) {
        start = 0;
        newlen = len;
        if (zerof) {            /* zero-fill it */
            chx = strpbrk(str, "-ed");
            if (chx) {

/* Check if minus occurs before exponent */
                if (*chx == '-') {

/* If so, move it to first position */
                    *chx = '0';
                    str[0] = '-';
                }
            }
            for (i = str[0] == '-' ? 1 : 0; i < newlen - 1; i++)
                if (str[i] != ' ')
                    break;
                else
                    str[i] = '0';
        }
    } else {
        for (start = 0; start < len - 1; start++)
            if (str[start] != ' ')
                break;
        newlen = len - start;
    }

    sxi[TC - 1] = -newlen;
    sxi[N - 1] = 0;

/* forces string to be in literal table: */
    sxi[NAML - 1] = 1;

/* of course it has no subscripts(ZCM 8/3/95) */
    sxi[NS - 1] = 0;
    Parget(sxi);

    chx = (char *) Paraddr(sxi);
    minlen = min(newlen, len - start + 1);
    strncpy(chx, str + start, minlen);

    if (minlen < newlen)
        memset((void *) (chx + minlen), (int) ' ', newlen - minlen);

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CMNXMXX - handler for mnx and mxx builtins */

Integer Cmnxmxx(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer fcode, npoints, ilbase, dim, dimlens[7], dimstride[8];
    Integer *ix, szi[SSWIDTH], idummy;
    float *ry, rvalue;
    real8 *dy, dvalue;
    int newindx, offset, izindx, i, j;
    char errmsg[MAXOUT];
    char *fname;

    fname = thisf->fname;
    fcode = GET_FCODE(thisf);

    if (syi[N - 1] == 0) {
        syi[N - 1] = 1;
        syi[N1 - 1] = 1;
        syi[M1 - 1] = 1;
    }

    sxi[TC - 1] = INTEGER;

    if (nargs == 1) {
        npoints = Parlen(syi);
        sxi[N - 1] = 0;
        ilbase = syi[N - 1] == 1 ? syi[M1 - 1] - 1 : 0;
        dim = 0;
        dimlens[0] = 0;
        dimstride[0] = 0;
    } else if (nargs == 2) {
        Fcnargb(2, szi);
        if (Parint(szi, &dim) == ERR)
            return ERR;
        Parrel(szi);            /* release space for argument */
        if (dim > syi[N - 1] || dim < 1) {
            sprintf(errmsg, "%s: arg2 is out of range", fname);
            BasError(errmsg, strlen(errmsg));
            return ERR;
        }
        sxi[N - 1] = syi[N - 1] - 1;
        ilbase = (sxi[N - 1] <= 1) ? syi[M1 + dim - 2] - 1 : 0;
        dimstride[0] = 1;
        for (i = 0; i < syi[N - 1]; i++) {
            dimlens[i] = syi[N1 + i - 1];
            dimstride[i + 1] = dimstride[i] * dimlens[i];
            if (i < dim) {
                sxi[N1 + i - 1] = syi[N1 + i - 1];
                sxi[M1 + i - 1] = syi[M1 + i - 1];
            } else {
                sxi[N1 + i - 2] = syi[N1 + i - 1];
                sxi[M1 + i - 2] = syi[M1 + i - 1];
            }
        }
    }

/* set up space for the answer */
    Parget(sxi);
    ix = (Integer *) Paraddr(sxi);
    if (syi[TC - 1] == INTEGER)
        idummy = Parcoerc(syi, &myreal);
    if (syi[TC - 1] == REAL) {
        ry = (float *) Paraddr(syi);
        if (nargs == 1) {
            ix[0] = 1 + ilbase;
            rvalue = ry[0];
            if (fcode == MNX) {
                for (i = 1; i < npoints; i++)
                    if (ry[i] < rvalue) {
                        ix[0] = i + ilbase + 1;
                        rvalue = ry[i];
                    }
            } else {            /* MXX */

                for (i = 1; i < npoints; i++)
                    if (ry[i] > rvalue) {
                        ix[0] = i + ilbase + 1;
                        rvalue = ry[i];
                    }
            }
        } else {
            newindx = 1;
            for (offset = 0; offset <= dimstride[syi[N - 1]] - 1;
                 offset += dimstride[dim]) {
                for (i = 0; i < dimstride[dim - 1]; i++) {
                    izindx = i + offset + 1;
                    ix[newindx - 1] = 1 + ilbase;
                    rvalue = ry[izindx - 1];
                    if (fcode == MNX) {
                        for (j = 1; j < dimlens[dim - 1]; j++) {
                            izindx += dimstride[dim - 1];
                            if (ry[izindx - 1] < rvalue) {
                                ix[newindx - 1] = j + 1 + ilbase;
                                rvalue = ry[izindx - 1];
                            }
                        }
                    } else {    /* MXX */

                        for (j = 1; j < dimlens[dim - 1]; j++) {
                            izindx += dimstride[dim - 1];
                            if (ry[izindx - 1] > rvalue) {
                                ix[newindx - 1] = j + 1 + ilbase;
                                rvalue = ry[izindx - 1];
                            }
                        }
                    }
                    newindx += 1;
                }
            }
        }
    } else if (syi[TC - 1] == DOUBLE) {
        dy = (real8 *) Paraddr(syi);
        if (nargs == 1) {
            ix[0] = 1 + ilbase;
            dvalue = dy[0];
            if (fcode == MNX) {
                for (i = 1; i < npoints; i++)
                    if (dy[i] < dvalue) {
                        ix[0] = i + ilbase + 1;
                        dvalue = dy[i];
                    }
            } else {            /* MXX */

                for (i = 1; i < npoints; i++)
                    if (dy[i] > dvalue) {
                        ix[0] = i + ilbase + 1;
                        dvalue = dy[i];
                    }
            }
        } else {
            newindx = 1;
            for (offset = 0; offset <= dimstride[syi[N - 1]] - 1;
                 offset += dimstride[dim]) {
                for (i = 0; i < dimstride[dim - 1]; i++) {
                    izindx = i + offset + 1;
                    ix[newindx - 1] = 1 + ilbase;
                    dvalue = dy[izindx - 1];
                    if (fcode == MNX) {
                        for (j = 1; j < dimlens[dim - 1]; j++) {
                            izindx += dimstride[dim - 1];
                            if (dy[izindx - 1] < dvalue) {
                                ix[newindx - 1] = j + 1 + ilbase;
                                dvalue = dy[izindx - 1];
                            }
                        }
                    } else {    /* MXX */

                        for (j = 1; j < dimlens[dim - 1]; j++) {
                            izindx += dimstride[dim - 1];
                            if (dy[izindx - 1] > dvalue) {
                                ix[newindx - 1] = j + 1 + ilbase;
                                dvalue = dy[izindx - 1];
                            }
                        }
                    }
                    newindx += 1;
                }
            }
        }
    }

    Parrel(syi);

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
