/*
 * cfcn3.c
 *
 */

#include "cfcn.h"

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CDIAG - handler for the diag builtin
 *       - create a matrix whose k'th diagonal is the first argument, where
 *       - k is the second argument...k = 0 default.
 *       - result has type of first argument.
 */

Integer Cdiag(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer szi[SSWIDTH];
    Integer *ix, *iy;
    float *rx, *ry, *cx, *cy;
    real8 *dx, *dy;
    int ndiag, n, ny, k, i, nskip;
    static char errmsg41[] = "diag: bad type for second argument.";

    ndiag = 0;
    if (nargs == 2) {
        Fcnargb(2, szi);
        if (Parcoerc(szi, &myint) == ERR) {
            BasError(errmsg41, strlen(errmsg41));
            return ERR;
        }
        ix = (Integer *) Paraddr(szi);
        ndiag = *ix;
        Parrel(szi);
    }
    ny = Parlen(syi);
    n = ny + abs(ndiag);
    sxi[N - 1] = 2;
    sxi[N1 - 1] = n;
    sxi[M1 - 1] = 1;
    sxi[I1 - 1] = 1;
    sxi[N2 - 1] = n;
    sxi[M2 - 1] = 1;
    sxi[I2 - 1] = 1;
    sxi[TC - 1] = syi[TC - 1];
    Parget(sxi);
    k = (ndiag >= 0) ? ndiag * n : abs(ndiag);
    nskip = n + 1;              /* plus one since it's diagonal */
    if (sxi[TC - 1] == INTEGER) {
        iy = (Integer *) Paraddr(syi);
        ix = (Integer *) Paraddr(sxi);
        for (i = 0; i < n * n; i++)
            ix[i] = 0;
        for (i = 0; i < ny; i++)
            ix[k + i * nskip] = iy[i];
    } else if (sxi[TC - 1] == REAL) {
        ry = (float *) Paraddr(syi);
        rx = (float *) Paraddr(sxi);
        for (i = 0; i < n * n; i++)
            rx[i] = 0.;
        for (i = 0; i < ny; i++)
            rx[k + i * nskip] = ry[i];
    } else if (sxi[TC - 1] == DOUBLE) {
        dy = (real8 *) Paraddr(syi);
        dx = (real8 *) Paraddr(sxi);
        for (i = 0; i < n * n; i++)
            dx[i] = 0.e0;
        for (i = 0; i < ny; i++)
            dx[k + i * nskip] = dy[i];
    }
#ifdef HAS_DOUBLE_COMPLEX
    else if (sxi[TC - 1] == COMPLEX) {
#else
    else if (sxi[TC - 1] == COMPLEX || sxi[TC - 1] == DOUBLE_C) {
#endif
        cy = (float *) Paraddr(syi);
        cx = (float *) Paraddr(sxi);
        for (i = 0; i < n * n; i++) {
            cx[2 * i] = 0.;
            cx[2 * i + 1] = 0.;
        }
        for (i = 0; i < ny; i++) {
            cx[2 * (k + i * nskip)] = cy[2 * i];
            cx[2 * (k + i * nskip) + 1] = cy[2 * i + 1];
        }
    }
#ifdef HAS_DOUBLE_COMPLEX
    else if (sxi[TC - 1] == DOUBLE_C) {
        dy = (real8 *) Paraddr(syi);
        dx = (real8 *) Paraddr(sxi);
        for (i = 0; i < n * n; i++) {
            dx[2 * i] = 0.e0;
            dx[2 * i + 1] = 0.e0;
        }
        for (i = 0; i < ny; i++) {
            dx[2 * (k + i * nskip)] = dy[2 * i];
            dx[2 * (k + i * nskip) + 1] = dy[2 * i + 1];
        }
    }
#endif
    Parrel(syi);

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CPSUMRSUM - handler for the psum and rsum builtins */

Integer Cpsumrsum(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer szi[SSWIDTH], fcode, npoints, dim, istride, dimlen, dimsize;
    Integer *ix, *iy;
    int i, j, offset, icount, izindx;
    float *rx, *ry, *cx, *cy;
    real8 *dx, *dy, *dcx, *dcy;
    static char errmsg43[] = "rsum: arg2 is out of range.";

    fcode = GET_FCODE(thisf);

    npoints = Parlen(syi);
    Parcopy(syi, sxi);
    if (nargs == 2) {
        Fcnargb(2, szi);
        if (Parint(szi, &dim) == ERR)
            return ERR;
        if (dim > syi[N - 1] || dim < 1) {
            BasError(errmsg43, strlen(errmsg43));
            return ERR;
        }
        istride = 1;
        for (i = 0; i <= dim - 2; i++)
            istride *= syi[N1 + i - 1];
        dimlen = syi[N1 + dim - 2];
        dimsize = istride * dimlen;
    }
    /* #### set up space for the answer */
    Parget(sxi);
    if (syi[TC - 1] == INTEGER) {
        ix = (Integer *) Paraddr(sxi);
        iy = (Integer *) Paraddr(syi);
        if (nargs == 1) {
            if (fcode == RSUM) {
                ix[npoints - 1] = iy[npoints - 1];
                for (i = npoints - 2; i >= 0; i--)
                    ix[i] = iy[i] + ix[i + 1];
            } else if (fcode == PSUM) {
                ix[0] = iy[0];
                for (i = 1; i < npoints; i++)
                    ix[i] = iy[i] + ix[i - 1];
            }
        } else {
            if (fcode == RSUM) {
                for (offset = npoints - 1; offset >= 0; offset -= dimsize) {
                    i = 0;
                    icount = 0;
                    while (icount < istride) {
                        izindx = i + offset;
                        ix[izindx] = iy[izindx];
                        for (j = 1; j < dimlen; j++) {
                            izindx -= istride;
                            ix[izindx] = iy[izindx] + ix[izindx + istride];
                        }
                        i -= 1;
                        icount += 1;
                    }
                }
            } else if (fcode == PSUM) {
                for (offset = 0; offset < npoints; offset += dimsize)
                    for (i = 0; i < istride; i++) {
                        izindx = i + offset;
                        ix[izindx] = iy[izindx];
                        for (j = 1; j < dimlen; j++) {
                            izindx += istride;
                            ix[izindx] = iy[izindx] + ix[izindx - istride];
                        }
                    }
            }
        }
    } else if (syi[TC - 1] == REAL) {
        rx = (float *) Paraddr(sxi);
        ry = (float *) Paraddr(syi);
        if (nargs == 1) {
            if (fcode == RSUM) {
                rx[npoints - 1] = ry[npoints - 1];
                for (i = npoints - 2; i >= 0; i--)
                    rx[i] = ry[i] + rx[i + 1];
            } else if (fcode == PSUM) {
                rx[0] = ry[0];
                for (i = 1; i < npoints; i++)
                    rx[i] = ry[i] + rx[i - 1];
            }
        } else {
            if (fcode == RSUM) {
                for (offset = npoints - 1; offset >= 0; offset -= dimsize) {
                    i = 0;
                    icount = 0;
                    while (icount < istride) {
                        izindx = i + offset;
                        rx[izindx] = ry[izindx];
                        for (j = 1; j < dimlen; j++) {
                            izindx -= istride;
                            rx[izindx] = ry[izindx] + rx[izindx + istride];
                        }
                        i -= 1;
                        icount += 1;
                    }
                }
            } else if (fcode == PSUM) {
                for (offset = 0; offset < npoints; offset += dimsize)
                    for (i = 0; i < istride; i++) {
                        izindx = i + offset;
                        rx[izindx] = ry[izindx];
                        for (j = 1; j < dimlen; j++) {
                            izindx += istride;
                            rx[izindx] = ry[izindx] + rx[izindx - istride];
                        }
                    }
            }
        }
    } else if (syi[TC - 1] == DOUBLE) {
        dx = (real8 *) Paraddr(sxi);
        dy = (real8 *) Paraddr(syi);
        if (nargs == 1) {
            if (fcode == RSUM) {
                dx[npoints - 1] = dy[npoints - 1];
                for (i = npoints - 2; i >= 0; i--)
                    dx[i] = dy[i] + dx[i + 1];
            } else if (fcode == PSUM) {
                dx[0] = dy[0];
                for (i = 1; i < npoints; i++)
                    dx[i] = dy[i] + dx[i - 1];
            }
        } else {
            if (fcode == RSUM) {
                for (offset = npoints - 1; offset >= 0; offset -= dimsize) {
                    i = 0;
                    icount = 0;
                    while (icount < istride) {
                        izindx = i + offset;
                        dx[izindx] = dy[izindx];
                        for (j = 1; j < dimlen; j++) {
                            izindx -= istride;
                            dx[izindx] = dy[izindx] + dx[izindx + istride];
                        }
                        i -= 1;
                        icount += 1;
                    }
                }
            } else if (fcode == PSUM) {
                for (offset = 0; offset < npoints; offset += dimsize)
                    for (i = 0; i < istride; i++) {
                        izindx = i + offset;
                        dx[izindx] = dy[izindx];
                        for (j = 1; j < dimlen; j++) {
                            izindx += istride;
                            dx[izindx] = dy[izindx] + dx[izindx - istride];
                        }
                    }
            }
        }
    }
#ifdef HAS_DOUBLE_COMPLEX
    else if (syi[TC - 1] == COMPLEX) {
#else
    else if (syi[TC - 1] == COMPLEX || syi[TC - 1] == DOUBLE_C) {
#endif
        cx = (float *) Paraddr(sxi);
        cy = (float *) Paraddr(syi);
        if (nargs == 1) {
            if (fcode == RSUM) {
                cx[2 * (npoints - 1)] = cy[2 * (npoints - 1)];
                cx[2 * (npoints - 1) + 1] = cy[2 * (npoints - 1) + 1];
                for (i = npoints - 2; i >= 0; i--) {
                    cx[2 * (i)] = cy[2 * (i)] + cx[2 * (i + 1)];
                    cx[2 * (i) + 1] = cy[2 * i] + cx[2 * (i + 1) + 1];
                }
            } else if (fcode == PSUM) {
                cx[0] = cy[0];
                cx[1] = cy[1];
                for (i = 1; i < npoints; i++) {
                    cx[2 * i] = cy[2 * i] + cx[2 * (i - 1)];
                    cx[2 * i + 1] = cy[2 * i + 1] + cx[2 * (i - 1) + 1];
                }
            }
        } else {
            if (fcode == RSUM) {
                for (offset = npoints - 1; offset >= 0; offset -= dimsize) {
                    i = 0;
                    icount = 0;
                    while (icount < istride) {
                        izindx = i + offset;
                        cx[2 * izindx] = cy[2 * izindx];
                        cx[2 * izindx + 1] = cy[2 * izindx + 1];
                        for (j = 1; j < dimlen; j++) {
                            izindx -= istride;
                            cx[2 * izindx] = cy[2 * izindx] +
                                cx[2 * (izindx + istride)];
                            cx[2 * izindx + 1] = cy[2 * izindx + 1] +
                                cx[2 * (izindx + istride) + 1];
                        }
                        i -= 1;
                        icount += 1;
                    }
                }
            } else if (fcode == PSUM) {
                for (offset = 0; offset < npoints; offset += dimsize)
                    for (i = 0; i < istride; i++) {
                        izindx = i + offset;
                        cx[2 * izindx] = cy[2 * izindx];
                        cx[2 * izindx + 1] = cy[2 * izindx + 1];
                        for (j = 1; j < dimlen; j++) {
                            izindx += istride;
                            cx[2 * izindx] =
                                cy[2 * izindx] +
                                cx[2 * (izindx - istride)];
                            cx[2 * izindx + 1] =
                                cy[2 * izindx + 1] +
                                cx[2 * (izindx - istride) + 1];
                        }
                    }
            }
        }
    }
#ifdef HAS_DOUBLE_COMPLEX
    else if (syi[TC - 1] == DOUBLE_C) {
        dcx = (real8 *) Paraddr(sxi);
        dcy = (real8 *) Paraddr(syi);
        if (nargs == 1) {
            if (fcode == RSUM) {
                dcx[2 * (npoints - 1)] = dcy[2 * (npoints - 1)];
                dcx[2 * (npoints - 1) + 1] = dcy[2 * (npoints - 1) + 1];
                for (i = npoints - 2; i >= 0; i--) {
                    dcx[2 * i] = dcy[2 * i] + dcx[2 * (i + 1)];
                    dcx[2 * i + 1] = dcy[2 * i] + dcx[2 * (i + 1) + 1];
                }
            } else if (fcode == PSUM) {
                dcx[0] = dcy[0];
                dcx[1] = dcy[1];
                for (i = 1; i < npoints; i++) {
                    dcx[2 * i] = dcy[2 * i] + dcx[2 * (i - 1)];
                    dcx[2 * i + 1] = dcy[2 * i + 1] + dcx[2 * (i - 1) + 1];
                }
            }
        } else {
            if (fcode == RSUM) {
                for (offset = npoints - 1; offset >= 0; offset -= dimsize) {
                    i = 0;
                    icount = 0;
                    while (icount < istride) {
                        izindx = i + offset;
                        dcx[2 * izindx] = dcy[2 * izindx];
                        dcx[2 * izindx + 1] = dcy[2 * izindx + 1];
                        for (j = 1; j < dimlen; j++) {
                            izindx -= istride;
                            dcx[2 * izindx] = dcy[2 * izindx] +
                                dcx[2 * (izindx + istride)];
                            dcx[2 * izindx + 1] = dcy[2 * izindx + 1] +
                                dcx[2 * (izindx + istride) + 1];
                        }
                        i -= 1;
                        icount += 1;
                    }
                }
            } else if (fcode == PSUM) {
                for (offset = 0; offset < npoints; offset += dimsize)
                    for (i = 0; i < istride; i++) {
                        izindx = i + offset;
                        dcx[2 * izindx] = dcy[2 * izindx];
                        dcx[2 * izindx + 1] = dcy[2 * izindx + 1];
                        for (j = 1; j < dimlen; j++) {
                            izindx += istride;
                            dcx[2 * izindx] =
                                dcy[2 * izindx] +
                                dcx[2 * (izindx - istride)];
                            dcx[2 * izindx + 1] =
                                dcy[2 * izindx + 1] +
                                dcx[2 * (izindx - istride) + 1];
                        }
                    }
            }
        }
    }
#endif

    Parrel(syi);
    if (nargs == 2)
        Parrel(szi);
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CPTP - handler for the ptp builtin */

Integer Cptp(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer dim, dimlens[7], dimstride[8], npoints;
    Integer *ix, *iy;
    Integer iminval, imaxval;
    int newindx, offset, i, izindx, j;
    float *rx, *ry, rminval, rmaxval;
    real8 *dx, *dy, dminval, dmaxval;
    char *fname;
    static char errmsg45[] = "ptp: max - min of 0 items is meaningless.";

    fname = thisf->fname;

    npoints = Par2get(fname, syi, nargs, &dim, sxi, dimlens, dimstride);
    if (npoints == 0) {
        BasError(errmsg45, strlen(errmsg45));
        return ERR;
    }
    if (syi[TC - 1] == INTEGER) {
        ix = (Integer *) Paraddr(sxi);
        iy = (Integer *) Paraddr(syi);
        if (nargs == 1) {
            iminval = iy[0];
            imaxval = iy[0];
            for (i = 1; i < npoints; i++)
                if (iy[i] < iminval)
                    iminval = iy[i];
                else if (iy[i] > imaxval)
                    imaxval = iy[i];
            ix[0] = imaxval - iminval;
        } else {
            newindx = 0;
            for (offset = 0; offset < dimstride[syi[N - 1]];
                 offset += dimstride[dim])
                for (i = 0; i < dimstride[dim - 1]; i++) {
                    izindx = i + offset;
                    iminval = iy[izindx];
                    imaxval = iy[izindx];
                    for (j = 1; j < dimlens[dim - 1]; j++) {
                        izindx += dimstride[dim - 1];
                        if (iy[izindx] < iminval)
                            iminval = iy[izindx];
                        else if (iy[izindx] > imaxval)
                            imaxval = iy[izindx];
                    }
                    ix[newindx] = imaxval - iminval;
                    newindx += 1;
                }
        }
    } else if (syi[TC - 1] == REAL) {
        rx = (float *) Paraddr(sxi);
        ry = (float *) Paraddr(syi);
        if (nargs == 1) {
            rminval = ry[0];
            rmaxval = ry[0];
            for (i = 1; i < npoints; i++)
                if (ry[i] < rminval)
                    rminval = ry[i];
                else if (ry[i] > rmaxval)
                    rmaxval = ry[i];
            rx[0] = rmaxval - rminval;
        } else {
            newindx = 0;
            for (offset = 0; offset < dimstride[syi[N - 1]];
                 offset += dimstride[dim])
                for (i = 0; i < dimstride[dim - 1]; i++) {
                    izindx = i + offset;
                    rminval = ry[izindx];
                    rmaxval = ry[izindx];
                    for (j = 1; j < dimlens[dim - 1]; j++) {
                        izindx += dimstride[dim - 1];
                        if (ry[izindx] < rminval)
                            rminval = ry[izindx];
                        else if (ry[izindx] > rmaxval)
                            rmaxval = ry[izindx];
                    }
                    rx[newindx] = rmaxval - rminval;
                    newindx += 1;
                }
        }
    } else if (syi[TC - 1] == DOUBLE) {
        dx = (real8 *) Paraddr(sxi);
        dy = (real8 *) Paraddr(syi);
        if (nargs == 1) {
            dminval = dy[0];
            dmaxval = dy[0];
            for (i = 1; i < npoints; i++)
                if (dy[i] < dminval)
                    dminval = dy[i];
                else if (dy[i] > dmaxval)
                    dmaxval = dy[i];
            dx[0] = dmaxval - dminval;
        } else {
            newindx = 0;
            for (offset = 0; offset < dimstride[syi[N - 1]];
                 offset += dimstride[dim])
                for (i = 0; i < dimstride[dim - 1]; i++) {
                    izindx = i + offset;
                    dminval = dy[izindx];
                    dmaxval = dy[izindx];
                    for (j = 1; j < dimlens[dim - 1]; j++) {
                        izindx += dimstride[dim - 1];
                        if (dy[izindx] < dminval)
                            dminval = dy[izindx];
                        else if (dy[izindx] > dmaxval)
                            dmaxval = dy[izindx];
                    }
                    dx[newindx] = dmaxval - dminval;
                    newindx += 1;
                }
        }
    }
    Parrel(syi);
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CRNGEND - handler for the rngend builtin */

Integer Crngend(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer npointsy, npointsz, szi[SSWIDTH], *ix, *iy, *iz;
    int i;
    char errmsg[MAXOUT];

    npointsy = Parlen(syi);
    Fcnargb(2, szi);
    npointsz = Parlen(szi);
    errmsg[0] = '\0';
    if (Parcoerc(szi, &myint) == ERR)
        strcpy(errmsg, "rngend: second argument not of type INTEGER.");
    if (syi[TC - 1] == NULL_TYPE) {
        Parcopy(szi, sxi);
        return OK;
    }
    if (npointsy != npointsz && npointsz != 1)
        strcpy(errmsg, "rngend: argument1 and argument2 size mismatch.");
    if (Parcoerc(syi, &myrng) == ERR)
        strcpy(errmsg, "rngend: first argument not of type RANGE.");
    if (errmsg[0] != '\0') {
        BasError(errmsg, strlen(errmsg));
        Parrel(syi);
        Parrel(szi);
        return ERR;
    }
    Parcopy(syi, sxi);
    sxi[TC - 1] = INTEGER;
    Parget(sxi);
    ix = (Integer *) Paraddr(sxi);
    iy = (Integer *) Paraddr(syi);
    iz = (Integer *) Paraddr(szi);
    for (i = 0; i < npointsy; i++) {
        ix[i] = iy[3 * i + 1];
        if (ix[i] == DEFAULT)
            ix[i] = npointsz == 1 ? iz[0] : iz[i];
    }
    Parrel(syi);
    Parrel(szi);
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
