/*
 * cfcn4.c
 *
 */

#include "cfcn.h"

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CRNGINC - handler for the rnginc builtin */

Integer Crnginc(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer npointsy, *ix, *iy, szi[SSWIDTH];
    int i;
    static char errmsg3[] = "rnginc: first argument not of type RANGE.";

    npointsy = Parlen(syi);
    if (nargs == 2) {
        Fcnargb(2, szi);
        Parrel(szi);
    }
    if (syi[TC - 1] == 0) {
        sxi[TC - 1] = INTEGER;
        sxi[N - 1] = 0;
        Parget(sxi);
        ix = (Integer *) Paraddr(sxi);
        ix[0] = 1;
        return OK;
    }
    if (Parcoerc(syi, &myrng) == ERR) {
        BasError(errmsg3, strlen(errmsg3));
        Parrel(syi);
        return ERR;
    }
    Parcopy(syi, sxi);
    sxi[TC - 1] = INTEGER;
    Parget(sxi);
    ix = (Integer *) Paraddr(sxi);
    iy = (Integer *) Paraddr(syi);
    for (i = 0; i < npointsy; i++)
        ix[i] = iy[3 * i + 2];
    Parrel(syi);
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CTOLOWERUPPER - handler for the tolower and tupper builtin */

Integer Ctolowerupper(Integer *sxi, f_entry *thisf, Integer nargs,
                      Integer *syi)
{
    Integer nc, fcode;
    int i;
    char *chx, *chy;
    static char errmsg4[] = "tolower, toupper:  scalar argument expected";

    fcode = GET_FCODE(thisf);

    if (Parlen(syi) != 1) {
        BasError(errmsg4, strlen(errmsg4));
        return ERR;
    }

    nc = -syi[TC - 1];
    chy = (char *) Paraddr(syi);
    sxi[N - 1] = 0;
    sxi[TC - 1] = -nc;

/* so it knows address is in literal table.(ZCM 4/10/95): */
    sxi[NAML - 1] = 1;
    Parget(sxi);
    chx = (char *) Paraddr(sxi);

    if (fcode == TOLOWER)
        for (i = 0; i < nc; i++)
            chx[i] = (char) tolower((int) chy[i]);
    else
        for (i = 0; i < nc; i++)
            chx[i] = (char) toupper((int) chy[i]);

    Parrel(syi);

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CSTRCHPAT - handler for the strchpat builtin */

Integer Cstrchpat(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    BasError("Cstrchpat: strchpat has never been implemented.", 47);

    return ERR;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CSUBSTR - handler for the substr builtin */

Integer Csubstr(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer len, szi[SSWIDTH], *iz, start, *ix, sslen, sublen;
    char *chx, *chy;
    static char
        errmsg5[] = "substr: scalar argument expected",
        errmsg6[] = "substr: bad type for index of first character.",
        errmsg7[] = "substr: bad type for length of substring.",
        errmsg8[] = "substr: length of substring must be > 0",
        errmsg9[] = "substr: starting index is out of range.";

    /* Get the first argument. Verify that it is a scalar. */
    if (Parlen(syi) != 1) {
        BasError(errmsg5, strlen(errmsg5));
        return ERR;
    }
    chy = (char *) Paraddr(syi);
    len = -syi[TC - 1];
    /* Get the 1-origin index of the starting character. */
    Fcnargb(2, szi);
    if (Parcoerc(szi, &myint) == ERR) {
        BasError(errmsg6, strlen(errmsg6));
        return ERR;
    }
    iz = (Integer *) Paraddr(szi);
    start = iz[0];
    Parrel(szi);
    /* Get the length of the substring. */
    Fcnargb(3, sxi);
    if (Parcoerc(sxi, &myint) == ERR) {
        BasError(errmsg7, strlen(errmsg7));
        return ERR;
    }
    ix = (Integer *) Paraddr(sxi);
    sslen = ix[0];
    Parrel(sxi);
    if (sslen <= 0) {
        BasError(errmsg8, strlen(errmsg8));
        return ERR;
    }
    if (start > len || start <= 0) {
        BasError(errmsg9, strlen(errmsg9));
        return ERR;
    }
    sublen = (Integer) min((int) sslen, (int) (len - start + 1));
    sxi[N - 1] = 0;
    sxi[TC - 1] = -sublen;
    /* so it knows address is in literal table.(ZCM 4/10/95): */
    sxi[NAML - 1] = 1;
    Parget(sxi);
    chx = (char *) Paraddr(sxi);
    strncpy(chx, chy + start - 1, sublen);
    Parrel(syi);
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CONES - handler for the ones builtin */

Integer Cones(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer *ndimen, *ix, parl, nmin;
    int i, j, iskip, itemp;
    static char
        errmsg10[] = "ones: illegal argument type.",
        errmsg11[] = "ones: argument not scalar.",
        errmsg12[] = "ones: argument negative or 0.";

    Parcopy(syi, sxi);
    syi[TC - 1] = INTEGER;
    syi[N1 - 1] = nargs;
    syi[M1 - 1] = 1;
    syi[N - 1] = 1;
    Parget(syi);
    ndimen = (Integer *) Paraddr(syi);
    for (i = 0; i < nargs; i++) {
        if (i > 0)              /* 1st argument already checked */
            Fcnargb(i + 1, sxi);
        if (Parcoerc(sxi, &myint) == ERR) {
            BasError(errmsg10, strlen(errmsg10));
            return ERR;
        }
        if (Parlen(sxi) != 1) {
            BasError(errmsg11, strlen(errmsg11));
            return ERR;
        }
        ix = (Integer *) Paraddr(sxi);
        if (ix[0] < 1) {
            BasError(errmsg12, strlen(errmsg12));
            return ERR;
        }
        ndimen[i] = ix[0];
        Parrel(sxi);
    }
    /* get space for array */
    sxi[TC - 1] = INTEGER;
    sxi[N - 1] = nargs;
    for (i = 0; i < nargs; i++) {
        sxi[N1 + i - 1] = ndimen[i];
        sxi[M1 + i - 1] = 1;
    }
    Parget(sxi);
    ix = (Integer *) Paraddr(sxi);
    /* if one dimensional, put 1's in each location */
    if (nargs == 1) {
        for (i = 0; i < ndimen[0]; i++)
            ix[i] = 1;
    }
    /* in multi-dimensional case, put 1's on diagonal elements.   */
    /* This is equivalent to a multi-dimensional Kronecker delta. */
    else {
        parl = Parlen(sxi);
        for (i = 0; i < parl; i++)
            ix[i] = 0;
        /* determine skip size */
        nmin = Ivecmin(ndimen, 1, nargs);
        iskip = 1;
        for (i = 0; i < nargs - 1; i++) {
            itemp = 1;
            for (j = 0; j <= i; j++)
                itemp *= ndimen[j];
            iskip += itemp;
        }
        for (i = 0; i < nmin * iskip; i += iskip)
            ix[i] = 1;
    }
    Parrel(syi);
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CMOD - handler for the mod builtin */

Integer Cmod(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer npointsx, npointsy, tc, *ix, *iy;
    int i;
    float *rx, *ry;
    real8 *dx, *dy;
    static char errmsg13[] = "mod: cannot match lengths.";

    Parcopy(syi, sxi);
    Fcnargb(2, syi);
    npointsx = Parlen(sxi);
    npointsy = Parlen(syi);
    /* if the arguments have different types, force them to agree. */
    if (sxi[TC - 1] != syi[TC - 1]) {
        tc = (Integer) max(sxi[TC - 1], syi[TC - 1]);
        if (Parcoerc(sxi, &tc) == ERR || Parcoerc(syi, &tc) == ERR)
            return ERR;
    }
    /* if second argument is a scalar, do usual mod call.  If
     * it's a vector or array, it must be the same length as the first
     * argument in order to do a point by point mod call.
     */
    if (npointsy != 1 && npointsy != npointsx) {
        BasError(errmsg13, strlen(errmsg13));
        return ERR;
    }
    if (sxi[TC - 1] == REAL) {
        ry = (float *) Paraddr(syi);
        rx = (float *) Paraddr(sxi);
        if (npointsy == 1)
            for (i = 0; i < npointsx; i++)
                rx[i] = rx[i] - (float) ((int) (rx[i] / ry[0])) * ry[0];
        else
            for (i = 0; i < npointsx; i++)
                rx[i] = rx[i] - (float) ((int) (rx[i] / ry[i])) * ry[i];
    } else if (sxi[TC - 1] == DOUBLE) {
        dy = (real8 *) Paraddr(syi);
        dx = (real8 *) Paraddr(sxi);
        if (npointsy == 1)
            for (i = 0; i < npointsx; i++)
                dx[i] = dx[i] - (real8) ((int) (dx[i] / dy[0])) * dy[0];
        else
            for (i = 0; i < npointsx; i++)
                dx[i] = dx[i] - (real8) ((int) (dx[i] / dy[i])) * dy[i];
    } else {                    /* it is INTEGER */
        iy = (Integer *) Paraddr(syi);
        ix = (Integer *) Paraddr(sxi);
        if (npointsy == 1)
            for (i = 0; i < npointsx; i++)
                ix[i] = ix[i] % iy[0];
        else
            for (i = 0; i < npointsx; i++)
                ix[i] = ix[i] % iy[i];
    }
    Parrel(syi);
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CLAND - handler for the land builtin */

Integer Cland(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    logical lm, *ly, *lx, npoints;
    int i;
    static char errmsg14[] = "lor, land arguments must be logical type.";

    lm = (logical) 1;
    for (i = 0; i < nargs; i++) {
        if (i != 0)
            Fcnargb(i + 1, syi);
        if (i != 0 && syi[TC - 1] != LOGICAL) {
            BasError(errmsg14, strlen(errmsg14));
            return ERR;
        } else if (lm != (logical) 0) {
            npoints = Parlen(syi);
            ly = (logical *) Paraddr(syi);
            for (i = 0; i < npoints; i++) {
                if (!ly[i]) {
                    lm = (logical) 0;
                    break;
                }
            }
        }
        Parrel(syi);
    }
    sxi[TC - 1] = LOGICAL;
    sxi[N1 - 1] = 1;
    sxi[M1 - 1] = 1;
    sxi[N - 1] = 0;
    Parget(sxi);
    lx = (logical *) Paraddr(sxi);
    lx[0] = lm;
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CTRIM - handler for the trim builins
 *       -    TRIM  trim blanks from both ends of string
 *       -    TRIML trim blanks from left end of string
 *       -    TRIMR trim blanks from right end of string
 */

Integer Ctrim(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer fcode, len, ileft, iright;
    char *chx, *chy;

    fcode = GET_FCODE(thisf);

    chy = (char *) Paraddr(syi);
    len = -syi[TC - 1];
    ileft = 0;
    iright = len - 1;
    if (fcode == TRIM || fcode == TRIML) {
        /* skip over blanks at the beginning: */
        ileft = strspn(chy, " ");
    }
    if (fcode == TRIM || fcode == TRIMR) {
        /* then skip over nonblanks to first blank at end: */
        for (; iright >= ileft && chy[iright] == ' '; iright--) ;
    }

/* special cases for an all-blank string: return a single blank */
    if (ileft == len || iright == -1) {
        ileft = 0;
        iright = ileft;
    }
    sxi[N - 1] = 0;
    sxi[TC - 1] = -(iright - ileft + 1);
    /* needs to be nonzero for name in literal table: */
    sxi[NAML - 1] = 1;
    Parget(sxi);
    chx = (char *) Paraddr(sxi);
    strncpy(chx, chy + ileft, -sxi[TC - 1]);
    Parrel(syi);
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CRANGEX - handler for the rangex builtin */

Integer Crangex(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer npointsy, *ix;
    int i;
    static char errmsg44[] = "rangex: illegal argument type.";

    /* data is not actually needed for this function */
    Fcnargbp(1, syi);
    if (syi[TC - 1] == FUNCTION_TYPE || syi[TC - 1] == NULL_TYPE) {
        BasError(errmsg44, strlen(errmsg44));
        return ERR;
    }
    npointsy = Parlen(syi);
    if (npointsy == 1) {
        syi[N - 1] = 1;
        syi[N1 - 1] = 1;
        syi[M1 - 1] = 1;
    } else if (npointsy == 0) {
        syi[N - 1] = 1;
        syi[N1 - 1] = 0;
        syi[M1 - 1] = 1;
    }
    sxi[N - 1] = 2;
    sxi[TC - 1] = INTEGER;
    sxi[N1 - 1] = syi[N - 1];
    sxi[N2 - 1] = 2;
    sxi[M1 - 1] = 1;
    sxi[M2 - 1] = 1;
    Parget(sxi);
    ix = (Integer *) Paraddr(sxi);
    for (i = 0; i < syi[N - 1]; i++) {
        ix[i] = syi[M1 + i - 1];        /* low index */
        ix[i + sxi[N1 - 1]] = syi[N1 + i - 1] + syi[M1 + i - 1] - 1;    /* high */
    }
    Parkill(syi);
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CRNGBEG - handler for the rngbeg builtin */

Integer Crngbeg(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer npointsy, szi[SSWIDTH], npointsz, *ix, *iy, *iz;
    int i;
    char errmsg[MAXOUT];
    static char
        errmsg46[] = "rngbeg: second argument not of type INTEGER.",
        errmsg47[] = "rngbeg: argument1 and argument2 size mismatch.",
        errmsg48[] = "rngbeg: first argument not of type RANGE.";

    npointsy = Parlen(syi);
    Fcnargb(2, szi);
    npointsz = Parlen(szi);
    errmsg[0] = '\0';
    if (syi[TC - 1] == NULL_TYPE) {
        Parcopy(szi, sxi);
        return OK;
    }
    if (Parcoerc(szi, &myint) == ERR)
        strcpy(errmsg, errmsg46);
    else if (npointsy != npointsz && npointsz != 1)
        strcpy(errmsg, errmsg47);
    else if (Parcoerc(syi, &myrng) == ERR)
        strcpy(errmsg, errmsg48);
    if (errmsg[0] != '\0') {
        BasError(errmsg, strlen(errmsg));
        Parrel(syi);
        Parrel(szi);
        return ERR;
    }
    Parcopy(syi, sxi);
    sxi[TC - 1] = INTEGER;
    Parget(sxi);
    ix = (Integer *) Paraddr(sxi);
    iy = (Integer *) Paraddr(syi);
    iz = (Integer *) Paraddr(szi);
    for (i = 0; i < npointsy; i++) {
        ix[i] = iy[3 * i];
        if (ix[i] == DEFAULT)
            ix[i] = npointsz == 1 ? iz[0] : iz[i];
    }
    Parrel(syi);
    Parrel(szi);
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CTRUESHAPE - handler for the trueshape builtin */

Integer Ctrueshape(Integer *sxi, f_entry *thisf,
                   Integer nargs, Integer *syi)
{
    Integer len, jvar, ndb, tc, nlow[7], nhi[7], ndim, ierr, *ix;
    int nameend, nameend2, namebeg, i;
    char *chy,
        errmsg[MAXOUT],
        aryname[MAXSTRING + 1],
        tmpname[MAXSTRING + 1], dimstr[MAXSTRING + 1];
    Address fwa;
    static char errmsg49[] = "trueshape: name is too long.";

    len = -syi[TC - 1];
    if (len > MAXSTRING) {
        BasError(errmsg49, strlen(errmsg49));
        return ERR;
    }
    chy = (char *) Paraddr(syi);
    strncpy(tmpname, chy, len);
    tmpname[len] = '\0';
    nameend = strcspn(tmpname, "(");
    tmpname[nameend] = '\0';
    /* Now remove leading and trailing blanks */
    namebeg = strspn(tmpname, " ");
    nameend2 = strcspn(tmpname + namebeg, " ");
    *(tmpname + namebeg + nameend2) = '\0';
    strcpy(aryname, tmpname + namebeg);
    if (nameend < len) {
        strncpy(dimstr, chy + nameend, len - nameend);
        if (Rtfinder(aryname, &jvar, &ndb, &tc, "parbfcn") == ERR) {
            sprintf(errmsg, "trueshape: unknown name '%s'", aryname);
            BasError(errmsg, strlen(errmsg));
            return ERR;
        }
        Pullapt2(&ndb, dimstr, &jvar, nlow, nhi, &ndim, &ierr,
                 len - nameend);
    } else {
        if (Rtgetvar(aryname, &fwa, &tc, &ndim, nlow, nhi) == ERR) {
            sprintf(errmsg, "trueshape: unknown array '%s'", aryname);
            BasError(errmsg, strlen(errmsg));
            return ERR;
        }
    }
    sxi[N - 1] = 1;
    sxi[M1 - 1] = 1;
    sxi[N1 - 1] = max((int) ndim, 1);
    sxi[TC - 1] = INTEGER;
    Parget(sxi);
    ix = (Integer *) Paraddr(sxi);
    ix[0] = 1;                  /* for scalar case (ndim = 0) */
    for (i = 0; i < ndim; i++)
        ix[i] = nhi[i] - nlow[i] + 1;
    Parrel(syi);
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
