/*
 * CFCN5.C
 *
 */

#include "cfcn.h"

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CSUM - handler for the sum builtin */

Integer Csum(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer npoints, *ix, *iy, dimstride[8], dimlens[7], dim;
    float *rx, *cx, *ry, *cy;
    real8 *dx, *dy, *dcx, *dcy;
    int i, j, newindx, offset, izindx;
    char *fname;

    fname = thisf->fname;

    npoints = Par2get(fname, syi, nargs, &dim, sxi, dimlens, dimstride);
    if (npoints == 0) {
        /* return a value of 0. */
        sxi[N - 1] = 0;
        sxi[TC - 1] = syi[TC - 1];
        Parget(sxi);
        if (syi[TC - 1] == INTEGER) {
            ix = (Integer *) Paraddr(sxi);
            ix[0] = 0;
        } else if (syi[TC - 1] == REAL) {
            rx = (float *) Paraddr(sxi);
            rx[0] = 0.e0;
        } else if (syi[TC - 1] == DOUBLE) {
            dx = (real8 *) Paraddr(sxi);
            dx[0] = 0.e0;
        }
#   ifdef HAS_DOUBLE_COMPLEX
        else if (syi[TC - 1] == COMPLEX) {
#   else
        else if (syi[TC - 1] == COMPLEX || syi[TC - 1] == DOUBLE_C) {
#   endif
            cx = (float *) Paraddr(sxi);
            cx[0] = 0.e0;
            cx[1] = 0.e0;
        }
#   ifdef HAS_DOUBLE_COMPLEX
        else if (syi[TC - 1] == DOUBLE_C) {
            dcx = (real8 *) Paraddr(sxi);
            dcx[0] = 0.e0;
            dcx[1] = 0.e0;
        }
#   endif
        Parrel(syi);
        return OK;
    }
    if (syi[TC - 1] == INTEGER) {
        ix = (Integer *) Paraddr(sxi);
        iy = (Integer *) Paraddr(syi);
        if (nargs == 1) {
            ix[0] = 0;
            for (i = 0; i < npoints; i++)
                ix[0] += iy[i];
        } else {
            newindx = 0;
            for (offset = 0; offset < dimstride[syi[N - 1]];
                 offset += dimstride[dim])
                for (i = 0; i < dimstride[dim - 1]; i++) {
                    izindx = i + offset;
                    ix[newindx] = iy[izindx];
                    for (j = 1; j < dimlens[dim - 1]; j++) {
                        izindx += dimstride[dim - 1];
                        ix[newindx] += iy[izindx];
                    }
                    newindx += 1;
                }
        }
    } else if (syi[TC - 1] == REAL) {
        rx = (float *) Paraddr(sxi);
        ry = (float *) Paraddr(syi);
        if (nargs == 1) {
            rx[0] = 0;
            for (i = 0; i < npoints; i++)
                rx[0] += ry[i];
        } else {
            newindx = 0;
            for (offset = 0; offset < dimstride[syi[N - 1]];
                 offset += dimstride[dim])
                for (i = 0; i < dimstride[dim - 1]; i++) {
                    izindx = i + offset;
                    rx[newindx] = ry[izindx];
                    for (j = 1; j < dimlens[dim - 1]; j++) {
                        izindx += dimstride[dim - 1];
                        rx[newindx] += ry[izindx];
                    }
                    newindx += 1;
                }
        }
    } else if (syi[TC - 1] == DOUBLE) {
        dx = (real8 *) Paraddr(sxi);
        dy = (real8 *) Paraddr(syi);
        if (nargs == 1) {
            dx[0] = 0;
            for (i = 0; i < npoints; i++)
                dx[0] += dy[i];
        } else {
            newindx = 0;
            for (offset = 0; offset < dimstride[syi[N - 1]];
                 offset += dimstride[dim])
                for (i = 0; i < dimstride[dim - 1]; i++) {
                    izindx = i + offset;
                    dx[newindx] = dy[izindx];
                    for (j = 1; j < dimlens[dim - 1]; j++) {
                        izindx += dimstride[dim - 1];
                        dx[newindx] += dy[izindx];
                    }
                    newindx += 1;
                }
        }
    }
#ifdef HAS_DOUBLE_COMPLEX
    else if (syi[TC - 1] == COMPLEX) {
#else
    else if (syi[TC - 1] == COMPLEX || syi[TC - 1] == DOUBLE_C) {
#endif
        cx = (float *) Paraddr(sxi);
        cy = (float *) Paraddr(syi);
        if (nargs == 1) {
            cx[0] = 0;
            cx[1] = 0;
            for (i = 0; i < npoints; i++) {
                cx[0] += cy[2 * i];
                cx[1] += cy[2 * i + 1];
            }
        } else {
            newindx = 0;
            for (offset = 0; offset < dimstride[syi[N - 1]];
                 offset += dimstride[dim])
                for (i = 0; i < dimstride[dim - 1]; i++) {
                    izindx = i + offset;
                    cx[2 * newindx] = cy[2 * izindx];
                    cx[2 * newindx + 1] = cy[2 * izindx + 1];
                    for (j = 1; j < dimlens[dim - 1]; j++) {
                        izindx += dimstride[dim - 1];
                        cx[2 * newindx] += cy[2 * izindx];
                        cx[2 * newindx + 1] += cy[2 * izindx + 1];
                    }
                    newindx += 1;
                }
        }
    }
#ifdef HAS_DOUBLE_COMPLEX
    else if (syi[TC - 1] == DOUBLE_C) {
        dcx = (real8 *) Paraddr(sxi);
        dcy = (real8 *) Paraddr(syi);
        if (nargs == 1) {
            dcx[0] = 0;
            dcx[1] = 0;
            for (i = 0; i < npoints; i++) {
                dcx[0] += dcy[2 * i];
                dcx[0] += dcy[2 * i + 1];
            }
        } else {
            newindx = 0;
            for (offset = 0; offset < dimstride[syi[N - 1]];
                 offset += dimstride[dim])
                for (i = 0; i < dimstride[dim - 1]; i++) {
                    izindx = i + offset;
                    dcx[2 * newindx] = dcy[2 * izindx];
                    dcx[2 * newindx + 1] = dcy[2 * izindx + 1];
                    for (j = 1; j < dimlens[dim - 1]; j++) {
                        izindx += dimstride[dim - 1];
                        dcx[2 * newindx] += dcy[2 * izindx];
                        dcx[2 * newindx + 1] += dcy[2 * izindx + 1];
                    }
                    newindx += 1;
                }
        }
    }
#endif
    Parrel(syi);
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CAVE - handler for the ave builtin */

Integer Cave(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer npoints, dim, dimlens[7], dimstride[8];
    int i, j, newindx, offset, izindx;
    float *rx, *ry, rsum, *cx, *cy, csum[2];
    real8 *dx, *dy, dsum, *dcx, *dcy, dcsum[2];
    char *fname;
    static char errmsg50[] = "ave: can't take the average of 0 items.";

    fname = thisf->fname;

    npoints = Par2get(fname, syi, nargs, &dim, sxi, dimlens, dimstride);
    if (npoints == 0) {
        BasError(errmsg50, strlen(errmsg50));
        return ERR;
    }
    if (syi[TC - 1] == INTEGER || syi[TC - 1] == REAL) {
        if (syi[TC - 1] == INTEGER) {
            if (Parcoerc(syi, &myreal) == ERR)
                return ERR;
            sxi[TC - 1] = REAL;
        }
        rx = (float *) Paraddr(sxi);
        ry = (float *) Paraddr(syi);
        if (nargs == 1) {
            rsum = 0.e0;
            for (i = 0; i < npoints; i++)
                rsum += ry[i];
            rx[0] = rsum / npoints;
        } else {
            newindx = 0;
            for (offset = 0; offset < dimstride[syi[N - 1]];
                 offset += dimstride[dim])
                for (i = 0; i < dimstride[dim - 1]; i++) {
                    izindx = i + offset;
                    rsum = ry[izindx];
                    for (j = 1; j < dimlens[dim - 1]; j++) {
                        izindx += dimstride[dim - 1];
                        rsum += ry[izindx];
                    }
                    rx[newindx] = rsum / dimlens[dim - 1];
                    newindx += 1;
                }
        }
    } else if (syi[TC - 1] == DOUBLE) {
        dx = (real8 *) Paraddr(sxi);
        dy = (real8 *) Paraddr(syi);
        if (nargs == 1) {
            dsum = 0.e0;
            for (i = 0; i < npoints; i++)
                dsum += dy[i];
            dx[0] = dsum / npoints;
        } else {
            newindx = 0;
            for (offset = 0; offset < dimstride[syi[N - 1]];
                 offset += dimstride[dim])
                for (i = 0; i < dimstride[dim - 1]; i++) {
                    izindx = i + offset;
                    dsum = dy[izindx];
                    for (j = 1; j < dimlens[dim - 1]; j++) {
                        izindx += dimstride[dim - 1];
                        dsum += dy[izindx];
                    }
                    dx[newindx] = dsum / dimlens[dim - 1];
                    newindx += 1;
                }
        }
    }
#ifdef HAS_DOUBLE_COMPLEX
    else if (syi[TC - 1] == COMPLEX) {
#else
    else if (syi[TC - 1] == COMPLEX || syi[TC - 1] == DOUBLE_C) {
#endif
        cx = (float *) Paraddr(sxi);
        cy = (float *) Paraddr(syi);
        if (nargs == 1) {
            csum[0] = 0.e0;
            csum[1] = 0.e0;
            for (i = 0; i < npoints; i++) {
                csum[0] += cy[2 * i];
                csum[1] += cy[2 * i + 1];
            }
            cx[0] = csum[0] / npoints;
            cx[1] = csum[1] / npoints;
        } else {
            newindx = 0;
            for (offset = 0; offset < dimstride[syi[N - 1]];
                 offset += dimstride[dim])
                for (i = 0; i < dimstride[dim - 1]; i++) {
                    izindx = i + offset;
                    csum[0] = cy[2 * izindx];
                    csum[1] = cy[2 * izindx + 1];
                    for (j = 1; j < dimlens[dim - 1]; j++) {
                        izindx += dimstride[dim - 1];
                        csum[0] += cy[2 * izindx];
                        csum[1] += cy[2 * izindx + 1];
                    }
                    cx[2 * newindx] = csum[0] / dimlens[dim - 1];
                    cx[2 * newindx + 1] = csum[1] / dimlens[dim - 1];
                    newindx += 1;
                }
        }
    }
#ifdef HAS_DOUBLE_COMPLEX
    else if (syi[TC - 1] == DOUBLE_C) {
        dcx = (real8 *) Paraddr(sxi);
        dcy = (real8 *) Paraddr(syi);
        if (nargs == 1) {
            dcsum[0] = 0.e0;
            dcsum[1] = 0.e0;
            for (i = 0; i < npoints; i++) {
                dcsum[0] += dcy[2 * i];
                dcsum[1] += dcy[2 * i + 1];
            }
            dcx[0] = dcsum[0] / npoints;
            dcx[1] = dcsum[1] / npoints;
        } else {
            newindx = 0;
            for (offset = 0; offset < dimstride[syi[N - 1]];
                 offset += dimstride[dim])
                for (i = 0; i < dimstride[dim - 1]; i++) {
                    izindx = i + offset;
                    dcsum[0] = dcy[2 * izindx];
                    dcsum[1] = dcy[2 * izindx + 1];
                    for (j = 1; j < dimlens[dim - 1]; j++) {
                        izindx += dimstride[dim - 1];
                        dcsum[0] += dcy[2 * izindx];
                        dcsum[1] += dcy[2 * izindx + 1];
                    }
                    dcx[2 * newindx] = dcsum[0] / dimlens[dim - 1];
                    dcx[2 * newindx + 1] = dcsum[1] / dimlens[dim - 1];
                    newindx += 1;
                }
        }
    }
#endif
    Parrel(syi);
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CINFSUP - handler for the inf and sup builtins */

Integer Cinfsup(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer itemp, imaxval, imaxval1, iminval, iminval1, icount;
    Integer fcode, ircount, idcount, npoints, *ix;
    float rmaxval, rmaxval1, rminval, rminval1, *rx;
    real8 dmaxval, dmaxval1, dminval, dminval1, *dx;
    int i;
    char errmsg[MAXOUT];

    fcode = GET_FCODE(thisf);

    itemp = INTEGER;
    if (fcode == MAX || fcode == SUP) {
        imaxval = 0;
        rmaxval = 0.e0;
        dmaxval = 0.e0;
    } else if (fcode == MIN || fcode == INF) {
        iminval = 0;
        rminval = 0.e0;
        dminval = 0.e0;
    }
    icount = 0;
    ircount = 0;
    idcount = 0;
    for (i = 1; i <= nargs; i++) {
        if (i > 1) {
            Fcnargb(i, syi);
            if (syi[TC - 1] < INTEGER || syi[TC - 1] > DOUBLE) {
                sprintf(errmsg,
                        "sup (or inf): argument %d has illegal type.", i);
                BasError(errmsg, strlen(errmsg));
                return ERR;
            }
        }
        itemp = (Integer) (max((int) itemp, (int) (syi[TC - 1])));
        npoints = Parlen(syi);
        if (npoints == 0 && nargs == 1) {
            sxi[TC - 1] = syi[TC - 1];
            sxi[N - 1] = 0;
            Parget(sxi);
            if (syi[TC - 1] == REAL) {
                rx = (float *) Paraddr(sxi);
                rx[0] = (fcode == MAX || fcode == SUP) ? -R1mach(&two) :
                    R1mach(&two);
            } else if (syi[TC - 1] == DOUBLE) {
                dx = (real8 *) Paraddr(sxi);
                dx[0] = (fcode == MAX || fcode == SUP) ? -D1mach(&two) :
                    D1mach(&two);
            } else {
                ix = (Integer *) Paraddr(sxi);
                ix[0] = (fcode == MAX ||
                         fcode == SUP) ? I1mach(&nine) + 1 : I1mach(&nine);
            }
            Parrel(syi);
            return OK;
        }
        if (syi[TC - 1] == REAL) {
            ircount += 1;
            rx = (float *) Paraddr(syi);
            if (fcode == MAX || fcode == SUP) {
                rmaxval1 = Rvecmax(rx, 1, npoints);
                rmaxval =
                    (ircount == 1) ? rmaxval1 : max(rmaxval, rmaxval1);
            } else {
                rminval1 = Rvecmin(rx, 1, npoints);
                rminval =
                    (ircount == 1) ? rminval1 : min(rminval, rminval1);
            }
        } else if (syi[TC - 1] == DOUBLE) {
            idcount += 1;
            dx = (real8 *) Paraddr(syi);
            if (fcode == MAX || fcode == SUP) {
                dmaxval1 = Dvecmax(dx, 1, npoints);
                dmaxval =
                    (idcount == 1) ? dmaxval1 : max(dmaxval, dmaxval1);
            } else {
                dminval1 = Dvecmin(dx, 1, npoints);
                dminval =
                    (idcount == 1) ? dminval1 : min(dminval, dminval1);
            }
        } else {
            icount += 1;
            ix = (Integer *) Paraddr(syi);
            if (fcode == MAX || fcode == SUP) {
                imaxval1 = Ivecmax(ix, 1, npoints);
                imaxval =
                    (icount == 1) ? imaxval1 : max(imaxval, imaxval1);
            } else {
                iminval1 = Ivecmin(ix, 1, npoints);
                iminval =
                    (icount == 1) ? iminval1 : min(iminval, iminval1);
            }
        }
        Parrel(syi);
    }
    sxi[TC - 1] = itemp;
    sxi[N1 - 1] = 1;
    sxi[M1 - 1] = 1;
    sxi[N - 1] = 0;
    Parget(sxi);
    if (itemp == REAL) {
        rx = (float *) Paraddr(sxi);
        if (fcode == MAX || fcode == SUP) {
            rx[0] = rmaxval;
            if (icount > 0)
                rx[0] = max(rx[0], (float) imaxval);
        } else {
            rx[0] = rminval;
            if (icount > 0)
                rx[0] = min(rx[0], (float) iminval);
        }
    } else if (itemp == DOUBLE) {
        dx = (real8 *) Paraddr(sxi);
        if (fcode == MAX || fcode == SUP) {
            dx[0] = dmaxval;
            if (icount > 0)
                dx[0] = max(dx[0], (real8) imaxval);
            if (ircount > 0)
                dx[0] = max(dx[0], (real8) rmaxval);
        } else {
            dx[0] = dminval;
            if (icount > 0)
                dx[0] = min(dx[0], (real8) iminval);
            if (ircount > 0)
                dx[0] = min(dx[0], (real8) rminval);
        }
    } else {
        ix = (Integer *) Paraddr(sxi);
        if (fcode == MAX || fcode == SUP) {
            ix[0] = imaxval;
        } else {
            ix[0] = iminval;
        }
    }
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CLOR - handler for lor builtin */

Integer Clor(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    logical lm, *ly, *lx, npoints;
    int i;
    static char errmsg14[] = "lor, land arguments must be logical type.";

    lm = (logical) 0;
    for (i = 0; i < nargs; i++) {
        if (i != 0)
            Fcnargb(i + 1, syi);
        if (i != 0 && syi[TC - 1] != LOGICAL) {
            BasError(errmsg14, strlen(errmsg14));
            return ERR;
        } else if (lm != (logical) 1) {
            npoints = Parlen(syi);
            ly = (logical *) Paraddr(syi);
            for (i = 0; i < npoints; i++) {
                if (ly[i]) {
                    lm = (logical) 1;
                    break;
                }
            }
        }
        Parrel(syi);
    }
    sxi[TC - 1] = LOGICAL;
    sxi[N1 - 1] = 1;
    sxi[M1 - 1] = 1;
    sxi[N - 1] = 0;
    Parget(sxi);
    lx = (logical *) Paraddr(sxi);
    lx[0] = lm;
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
