/*
 * CFCN6.C
 *
 */

#include "cfcn.h"

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CSHAPE - handler for the shape command */

Integer Cshape(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer *ix, *iy, n, m, i, j, tc, bdfin, leny, mdc, mdp[7], mdv[7];
    float *rx, *ry, *cx, *cy;
    real8 *dx, *dy;
    logical *lx, *ly;
    int icur, jindex, kindex;
    char *chx, *chy;
    static char
        errmsg1[] = "shape: illegal argument type.",
        errmsg2[] = "shape: null is an illegal 1st argument.",
        errmsg3[] = "shape: zero length 1st argument.",
        errmsg4[] = "shape: second argument not an integer vector.",
        errmsg5[] = "shape: argument not an integer scalar.",
        errmsg6[] = "shape: size doesn't match arguments.",
        errmsg7[] = "shape:  shape: invalid type for arg1.",
        errmsg8[] = "shape: minus dim expansion error.";
    char errmsg[MAXOUT];

    /* if only one argument, return shape, else set it. */
    /* for "y=shape(z)" call                            */
    if (nargs == 1) {           /* return info about size */
        /* data is not actually needed for this function */
        Fcnargbp(1, syi);
        if (syi[TC - 1] == FUNCTION_TYPE || syi[TC - 1] == NULL_TYPE) {
            BasError(errmsg1, strlen(errmsg1));
            return ERR;
        }
        sxi[N - 1] = 1;
        sxi[N1 - 1] = max(syi[N - 1], 1);
        sxi[M1 - 1] = 1;
        sxi[TC - 1] = INTEGER;
        Parget(sxi);
        ix = (Integer *) Paraddr(sxi);
        ix[0] = 1;
        for (i = 0; i < syi[N - 1]; i++)
            ix[i] = syi[N1 + i - 1];
        Parkill(syi);
    } else {                    /* more than one argument, reshape */
        Fcnargb(1, sxi);
        if (sxi[TC - 1] == NULL_TYPE) {
            BasError(errmsg2, strlen(errmsg2));
            return ERR;
        }
        n = Parlen(sxi);
        if (n == 0) {
            BasError(errmsg3, strlen(errmsg3));
            return ERR;
        }
        m = 1;
        if (nargs == 2) {
            /* for "y=shape(x,shape(z))" call */
            /* this including the case of 2nd argument a linear vector;           */
            /* the expansion to minus dimension may be applied for this situation */
            bdfin = (n == 1);
            sxi[N - 1] = 0;
            mdc = 0;
            icur = 0;
            Fcnargb(2, syi);
            if (syi[TC - 1] != INTEGER || syi[N - 1] > 1) {
                BasError(errmsg4, strlen(errmsg4));
                return ERR;
            }
            iy = (Integer *) Paraddr(syi);
            leny = Parlen(syi);
            for (i = 0; i < leny; i++) {
                if (bdfin)
                    iy[i] = -abs(iy[i]);
                if (iy[i] < 0) {
                    mdc += 1;
                    mdp[mdc - 1] = i + 1;
                    mdv[mdc - 1] = -iy[i];
                } else {
                    sxi[N - 1] += 1;
                    sxi[N1 + icur - 1] = iy[i];
                    sxi[M1 + icur - 1] = 1;
                    icur += 1;
                    m *= iy[i];
                    if (m == n)
                        bdfin = 1;
                }
            }
            Parrel(syi);
        } else {
            /* for "y=shape(x,i,j,k,...)" call */
            bdfin = (n == 1);   /* base dims done */
            sxi[N - 1] = 0;     /* minus dimension count */
            mdc = 0;
            icur = 0;
            for (i = 2; i <= nargs; i++) {
                Fcnargb(i, syi);
                if (syi[TC - 1] != INTEGER || Parlen(syi) != 1) {
                    BasError(errmsg5, strlen(errmsg5));
                    sprintf(errmsg, "Argument number is %ld.", (long) i);
                    BasError(errmsg, strlen(errmsg));
                    return ERR;
                }
                iy = (Integer *) Paraddr(syi);
                if (bdfin)      /* backwards compatible */
                    iy[0] = -abs(iy[0]);
                if (iy[0] < 0) {        /* minus dimesion spec */
                    mdp[mdc] = i - 1;   /* minus dim position */
                    mdv[mdc] = -iy[0];  /* minus dim true value */
                    mdc += 1;
                } else {
                    sxi[N - 1] += 1;
                    sxi[N1 + icur - 1] = iy[0];
                    sxi[M1 + icur - 1] = 1;
                    icur += 1;
                    m *= iy[0];
                    if (m == n)
                        bdfin = 1;
                }
                Parrel(syi);
            }
        }
        /* the size of m and n should be equal */
        /* for positive dimension list         */
        if (m != n) {
            BasError(errmsg6, strlen(errmsg6));
            return ERR;
        } else {
            /* build base matrix with assigned positive dimensions */
            /* for "y=shape(x,i,j,k,...)" call                     */
            /* the call fcnargb is to identify the type of x       */
            /* note the iarg = 1 in y=shape(x,i,j,k,...) call      */
            Fcnargb(1, syi);
            Parget(sxi);
            if (syi[TC - 1] == INTEGER) {
                ix = (Integer *) Paraddr(sxi);
                iy = (Integer *) Paraddr(syi);
                for (j = 0; j < n; j++)
                    ix[j] = iy[j];
            } else if (syi[TC - 1] == REAL) {
                rx = (float *) Paraddr(sxi);
                ry = (float *) Paraddr(syi);
                for (j = 0; j < n; j++)
                    rx[j] = ry[j];
            } else if (syi[TC - 1] == DOUBLE) {
                dx = (real8 *) Paraddr(sxi);
                dy = (real8 *) Paraddr(syi);
                for (j = 0; j < n; j++)
                    dx[j] = dy[j];
            }
#      ifdef HAS_DOUBLE_COMPLEX
            else if (syi[TC - 1] == COMPLEX) {
#      else
            else if (syi[TC - 1] == COMPLEX || syi[TC - 1] == DOUBLE_C) {
#      endif
                cx = (float *) Paraddr(sxi);
                cy = (float *) Paraddr(syi);
                for (j = 0; j < n; j++) {
                    cx[2 * j] = cy[2 * j];
                    cx[2 * j + 1] = cy[2 * j + 1];
                }
            }
#      ifdef HAS_DOUBLE_COMPLEX
            else if (syi[TC - 1] == DOUBLE_C) {
                dx = (real8 *) Paraddr(sxi);
                dy = (real8 *) Paraddr(syi);
                for (j = 0; j < n; j++) {
                    dx[2 * j] = dy[2 * j];
                    dx[2 * j + 1] = dy[2 * j + 1];
                }
            }
#      endif
            else if (syi[TC - 1] == LOGICAL) {
                lx = (logical *) Paraddr(sxi);
                ly = (logical *) Paraddr(syi);
                for (j = 0; j < n; j++)
                    lx[j] = ly[j];
            } else if (syi[TC - 1] < 0) {
                chx = (char *) Paraddr(sxi);
                chy = (char *) Paraddr(syi);
                jindex = 0;
                kindex = 0;
                for (j = 0; j < n; j++) {
                    strncpy(chx + jindex, chy + kindex, -syi[TC - 1]);
                    jindex -= syi[TC - 1];
                    kindex -= syi[TC - 1];
                }
            } else {
                BasError(errmsg7, strlen(errmsg7));
                return ERR;
            }
            /* expand for minus dimensions */
            if (mdc > 0) {
                tc = syi[TC - 1];
                if (Parmdexpn(&tc, &mdc, mdp, mdv, sxi) == ERR) {
                    BasError(errmsg8, strlen(errmsg8));
                    return ERR;
                }
            }
            Parrel(syi);
        }
    }
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CAIMAG - handler for the aimag builtin */

Integer Caimag(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    float *rx, *cx;
    real8 *dx, *dcx;
    int i;
    Integer npoints;

#ifdef HAS_DOUBLE_COMPLEX
    if (syi[TC - 1] == COMPLEX) {
#else
    if (syi[TC - 1] == COMPLEX || syi[TC - 1] == DOUBLE_C) {
#endif
        Parcopy(syi, sxi);
        sxi[TC - 1] = REAL;
        Parget(sxi);
        cx = (float *) Paraddr(syi);
        rx = (float *) Paraddr(sxi);
        npoints = Parlen(syi);
        for (i = 0; i < npoints; i++)
            rx[i] = cx[2 * i + 1];
    }
#ifdef HAS_DOUBLE_COMPLEX
    else if (syi[TC - 1] == DOUBLE_C) {
        Parcopy(syi, sxi);
        sxi[TC - 1] = DOUBLE;
        Parget(sxi);
        dcx = (real8 *) Paraddr(syi);
        dx = (real8 *) Paraddr(sxi);
        npoints = Parlen(syi);
        for (i = 0; i < npoints; i++)
            dx[i] = dcx[2 * i + 1];
    }
#endif
    Parrel(syi);
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CSPANL - handler for the spanl builtin */

Integer Cspanl(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    float rstart, rend, ratio, *rx;
    real8 dstart, dend, dratio, *dx;
    Integer npoints, szi[SSWIDTH];
    int i;
    static char
        errmsg1[] = "spanl: illegal argument type.",
        errmsg2[] = "spanl: illegal argument length",
        errmsg3[] = "spanl: scale must have at least 2 points.",
        errmsg4[] = "spanl: illegal input.";

    /* get the starting value */
    if (Parlen(syi) > 1) {
        BasError(errmsg2, strlen(errmsg2));
        return ERR;
    }
    if (syi[TC - 1] == INTEGER) {
        if (Parcoerc(syi, &myreal) == ERR) {
            BasError(errmsg1, strlen(errmsg1));
            return ERR;
        }
    }
    /* get the stopping value */
    Fcnargb(2, szi);
    if (Parlen(szi) > 1) {
        BasError(errmsg2, strlen(errmsg2));
        return ERR;
    }
    if (szi[TC - 1] == INTEGER) {
        if (Parcoerc(szi, &myreal) == ERR) {
            BasError(errmsg1, strlen(errmsg1));
            return ERR;
        }
    }
    /* check type */
    if (syi[TC - 1] == REAL && szi[TC - 1] == REAL) {
        if (Parreal(syi, &rstart) == ERR)
            return ERR;
        if (Parreal(szi, &rend) == ERR)
            return ERR;
        sxi[TC - 1] = REAL;
    } else if (syi[TC - 1] == DOUBLE && szi[TC - 1] == DOUBLE) {
        if (Pardble(syi, &dstart) == ERR)
            return ERR;
        if (Pardble(szi, &dend) == ERR)
            return ERR;
        sxi[TC - 1] = DOUBLE;
    } else if (syi[TC - 1] == REAL && szi[TC - 1] == DOUBLE) {
        if (Parreal(syi, &rstart) == ERR)
            return ERR;
        if (Parcoerc(szi, &myreal) == ERR) {
            BasError(errmsg1, strlen(errmsg1));
            return ERR;
        }
        if (Parreal(szi, &rend) == ERR)
            return ERR;
        sxi[TC - 1] = REAL;
    } else if (syi[TC - 1] == DOUBLE && szi[TC - 1] == REAL) {
        if (Parcoerc(syi, &myreal) == ERR) {
            BasError(errmsg1, strlen(errmsg1));
            return ERR;
        }
        if (Parreal(syi, &rstart) == ERR)
            return ERR;
        if (Parreal(szi, &rend) == ERR)
            return ERR;
        sxi[TC - 1] = REAL;
    } else {
        BasError(errmsg1, strlen(errmsg1));
        return ERR;
    }
    Parrel(syi);
    Parrel(szi);
    /* get the length */
    Fcnargb(3, syi);
    if (Parcoerc(syi, &myint) == ERR) {
        BasError(errmsg1, strlen(errmsg1));
        return ERR;
    }
    if (Parlen(syi) != 1) {
        BasError(errmsg2, strlen(errmsg2));
        return ERR;
    }
    if (Parint(syi, &npoints) == ERR)
        return ERR;
    if (npoints < 2) {
        BasError(errmsg3, strlen(errmsg3));
        return ERR;
    }
    Parrel(syi);
    /* get space for array */
    sxi[N - 1] = 1;
    sxi[M1 - 1] = 1;
    sxi[N1 - 1] = npoints;
    Parget(sxi);
    if (sxi[TC - 1] == REAL) {
        if (rstart == 0.)
            rstart = rend * 1.e-7;
        /* Corrected to trap if both arguments negative (FNF 12/96/96) */
        if (rstart <= 0. && rend <= 0.) {
            BasError(errmsg4, strlen(errmsg4));
            return ERR;
        }
        ratio = (float) (log((double) rend) - log((double) rstart));
        rx = (float *) Paraddr(sxi);
        rx[0] = rstart;
        for (i = 1; i < npoints - 1; i++)
            rx[i] = rstart *
                (float) (exp
                         ((double)
                          (ratio * ((i + 1) - 1.) / (npoints - 1.))));
        rx[npoints - 1] = rend;
    } else {                    /* DOUBLE */
        if (dstart == 0.e0)
            dstart = dend * 1.e-7;
        /* Corrected to trap if both arguments negative (FNF 12/96/96) */
        if (dstart <= 0. && dend <= 0.) {
            BasError(errmsg4, strlen(errmsg4));
            return ERR;
        }
        dratio = (real8) (log((double) dend) - log((double) dstart));
        dx = (real8 *) Paraddr(sxi);
        dx[0] = dstart;
        for (i = 1; i < npoints - 1; i++)
            dx[i] = dstart *
                (real8) (exp
                         ((double)
                          (dratio * ((i + 1) - 1.e0) / (npoints - 1.e0))));
        dx[npoints - 1] = dend;
    }
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CLOAD - handler for the load builtin
 * Reprogrammed by Zane C. Motteler (3/21/2000) to use type FWA for the
 * first argument. I am unable to figure out whether, if this routine ever
 * worked, how it did. My understanding of it is as follows:
 * load (address, nitems): address is the address of a real array
 * of length nitems. load returns a new array of the same length
 * containing the contents of the one addressed by address.
 * When I tried to run this in the old Basis, I couldn't get
 * it to accept either ar integer (as advertised) or an address
 * as its first argument. The function may never have been used.
 * I am unclear as to the utility of this function, since
 *       real x = load (& y, 12)
 * is equivalent to
 *       real x = y (1:12)
 */

Integer Cload(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer szi[SSWIDTH], *ix;
    float *rx, *ry;
    float **ay;
    static char errmsg1[] = "load: second argument not an integer scalar.";

    Fcnargb(2, szi);
    if (Parlen(szi) != 1 || szi[TC - 1] != INTEGER) {
        BasError(errmsg1, strlen(errmsg1));
        return ERR;
    }
    ix = (Integer *) Paraddr(szi);
    sxi[TC - 1] = REAL;
    sxi[N - 1] = 1;
    sxi[N1 - 1] = ix[0];
    sxi[M1 - 1] = 1;
    Parget(sxi);
    ay = (float **) Paraddr(syi);
    ry = *ay;
    rx = Paraddr(sxi);
    memcpy((void *) rx, (void *) ry, sxi[N1 - 1] * sizeof(float));
    Parrel(syi);
    Parrel(szi);

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
