/*
 * cfcn7.c
 *
 */

#include "cfcn.h"

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* creates an empty structure, sarg */

void Parstru0(Integer *sarg)
{
    Integer *iparg;

    /* 1. set properties, get space for sarg */
    sarg[N - 1] = 1;
    sarg[TC - 1] = STRUCTURE;
    sarg[N1 - 1] = 3;
    sarg[M1 - 1] = 1;
    Parget(sarg);
    /* 2. initialize sarg as empty structure */
    iparg = (Integer *) Paraddr(sarg);
    iparg[0] = STRUCTURE;       /* type code for structure */
    iparg[1] = 3;               /* length of structure     */
    iparg[2] = 0;               /* number of elements      */
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CCROSS ::: handler for the cross builtin */

Integer Ccross(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer szi[SSWIDTH];
    float *rx, *ry, *rz;
    real8 *dx, *dy, *dz;
    static char
        errmsg1[] = "cross: illegal argument type.",
        errmsg2[] = "cross: vectors not of length = 3.";

    Fcnargb(2, szi);
    if (szi[TC - 1] < INTEGER || szi[TC - 1] > DOUBLE) {
        BasError(errmsg1, strlen(errmsg1));
        return ERR;
    }
    if (Parlen(syi) != 3 || Parlen(szi) != 3) {
        BasError(errmsg2, strlen(errmsg2));
        return ERR;
    }
    Parcopy(syi, sxi);
    /* new vector must be type real or double */
    sxi[TC - 1] = max(2, max(sxi[TC - 1], szi[TC - 1]));
    if (Parcoerc(syi, &(sxi[TC - 1])) == ERR)
        return ERR;
    if (Parcoerc(szi, &(sxi[TC - 1])) == ERR)
        return ERR;
    /* get space for the array, and do cross product(s) */
    Parget(sxi);
    if (sxi[TC - 1] == REAL) {
        rx = (float *) Paraddr(sxi);
        ry = (float *) Paraddr(syi);
        rz = (float *) Paraddr(szi);
        rx[0] = ry[1] * rz[2] - ry[2] * rz[1];
        rx[1] = ry[2] * rz[0] - ry[0] * rz[2];
        rx[2] = ry[0] * rz[1] - ry[1] * rz[0];
    } else if (sxi[TC - 1] == DOUBLE) {
        dx = (real8 *) Paraddr(sxi);
        dy = (real8 *) Paraddr(syi);
        dz = (real8 *) Paraddr(szi);
        dx[0] = dy[1] * dz[2] - dy[2] * dz[1];
        dx[1] = dy[2] * dz[0] - dy[0] * dz[2];
        dx[2] = dy[0] * dz[1] - dy[1] * dz[0];
    }
    Parrel(syi);
    Parrel(szi);
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Ctruerange ::: handler for the truerange builtin */

Integer Ctruerange(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer len, jvar, ndb, tc, nlow[7], nhi[7], ndim, ierr, *ix;
    char *chy,
        dimstr[MAXSTRING + 1], aryname[MAXSTRING + 1], errmsg[MAXOUT];
    int nameend, nameend2, namebeg, i;
    Address fwa;
    static char errmsg1[] = "truerange: name is too long.";

    len = -syi[TC - 1];
    if (len > MAXSTRING) {
        BasError(errmsg1, strlen(errmsg1));
        return ERR;
    }
    chy = (char *) Paraddr(syi);
    strncpy(dimstr, chy, len);
    dimstr[len] = '\0';
    /* name ends at first "(" or at end, whichever comes first. */
    nameend = strcspn(dimstr, "(");
    dimstr[nameend] = '\0';
    /* Skip over leading blanks */
    namebeg = strspn(dimstr, " ");
    /* find first trailing blank, if any */
    nameend2 = strcspn(dimstr + namebeg, " ");
    *(dimstr + namebeg + nameend2) = '\0';
    strcpy(aryname, dimstr + namebeg);
    if (nameend < len) {
        strncpy(dimstr, chy + nameend, len - nameend);
        dimstr[len - nameend] = '\0';
        if (Rtfinder(aryname, &jvar, &ndb, &tc, "parbfcn") == ERR) {
            sprintf(errmsg, "truerange: unknown name '%s'.", aryname);
            BasError(errmsg, strlen(errmsg));
            return ERR;
        }
        Pullapt2(&ndb, dimstr, &jvar, nlow, nhi, &ndim, &ierr,
                 strlen(dimstr));
    } else {
        if (Rtgetvar(aryname, &fwa, &tc, &ndim, nlow, nhi) == ERR) {
            sprintf(errmsg, "truerange: unknown array %s.", aryname);
            BasError(errmsg, strlen(errmsg));
            return ERR;
        }
    }
    sxi[N - 1] = 2;
    sxi[M1 - 1] = 1;
    sxi[M2 - 1] = 1;
    sxi[N1 - 1] = max(ndim, 1);
    sxi[N2 - 1] = 2;
    sxi[TC - 1] = INTEGER;
    Parget(sxi);
    ix = (Integer *) Paraddr(sxi);
    ix[0] = 1;                  /* for scalar case (ndim = 0) */
    ix[1] = 1;
    for (i = 0; i < ndim; i++) {
        ix[i] = nlow[i];
        ix[i + sxi[N1 - 1]] = nhi[i];
    }
    Parrel(syi);
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Zcen2d ::: zone center the array specified.
 *       ycen     =:  array to apply centering to
 *       nrows    =:  number of rows in the array
 *       ncols    =:  number of columns in the array
 *       xcen      := resultant array
 *
 *       This routine allows the arrays to be accessed with
 *       two subscripts.  Currently, the zone centering
 *       value = 2.  If additional values are needed, an
 *       extra argument may be added to the builtin function.
 */

static void Zcen2d(float *ycen, Integer nrows, Integer ncols, float *xcen)
{
    int i, j, k;

    for (i = 0; i < nrows; i++)
        xcen[i] = ycen[i];
    for (j = 0, k = 0; j < ncols; j++, k += nrows)
        xcen[k] = ycen[k];
    for (j = 1, k = nrows; j < ncols; j++, k += nrows)
        for (i = 1; i < nrows; i++)
            xcen[i+k] = (ycen[i+k] + ycen[i+k-nrows] +
                         ycen[i-1+k-nrows] + ycen[i-1+k]) / 4.0;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Dzcen2d ::: zone center the double precision array specified.
 *       ycen     =:  array to apply centering to
 *       nrows    =:  number of rows in the array
 *       ncols    =:  number of columns in the array
 *       xcen      := resultant array
 *
 *       This routine allows the arrays to be accessed with
 *       two subscripts.  Currently, the zone centering
 *       value = 2.  If additional values are needed, an
 *       extra argument may be added to the builtin function.
 */

static void Dzcen2d(real8 *ycen, Integer nrows, Integer ncols, real8 *xcen)
{
    int i, j, k;

    for (i = 0; i < nrows; i++)
        xcen[i] = ycen[i];
    for (j = 0, k = 0; j < ncols; j++, k += nrows)
        xcen[k] = ycen[k];
    for (j = 1, k = nrows; j < ncols; j++, k += nrows)
        for (i = 1; i < nrows; i++)
            xcen[i+k] = (ycen[i+k] + ycen[i+k-nrows] +
                         ycen[i-1+k-nrows] + ycen[i-1+k]) / 4.0;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CZCEN - handler for zcen builtin */

Integer Czcen(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer npoints, nrows, ncols;
    float *rx, *ry;
    real8 *dx, *dy;
    int i;
    static char
        errmsg1[] = "zcen:  illegal argument type.",
        errmsg2[] = "zcen:  no data available.",
        errmsg3[] = "zcen:  only takes 1 and 2-D arrays.";

    /* -- Make a copy of the data descriptor for the input array. */
    /* -- Get space for the resulting array.                      */
    Parcopy(syi, sxi);
    Parget(sxi);
    Pardim(syi);                /* must do after parcopy, so output sxi has same */
    /* original shape as input syi.                  */
    /* -- Coerce an integer array to type REAL. */
    if (syi[TC - 1] == INTEGER) {
        if (Parcoerc(syi, &myreal) == ERR) {
            BasError(errmsg1, strlen(errmsg1));
            return ERR;
        }
        sxi[TC - 1] = REAL;
    }
    if (syi[TC - 1] == REAL) {
        rx = (float *) Paraddr(sxi);
        ry = (float *) Paraddr(syi);
        /* -- Zone center 1-dimensional array. */
        if (syi[N - 1] == 1) {
            npoints = Parlen(syi);
            if (npoints <= 0) {
                BasError(errmsg2, strlen(errmsg2));
                return ERR;
            }
            rx[0] = ry[0];
            for (i = 1; i < npoints; i++)
                rx[i] = (ry[i] + ry[i - 1]) / 2;
        }
        /* -- Zone center 2-dimensional array. */
        else if (syi[N - 1] == 2) {
            nrows = syi[N1 - 1];
            ncols = syi[N2 - 1];
            Zcen2d(ry, nrows, ncols, rx);
        } else {
            BasError(errmsg3, strlen(errmsg3));
            return ERR;
        }
    } else if (syi[TC - 1] == DOUBLE) {
        dx = (real8 *) Paraddr(sxi);
        dy = (real8 *) Paraddr(syi);
        /* -- Zone center 1-dimensional array. */
        if (syi[N - 1] == 1) {
            npoints = Parlen(syi);
            if (npoints <= 0) {
                BasError(errmsg2, strlen(errmsg2));
                return ERR;
            }
            dx[0] = dy[0];
            for (i = 1; i < npoints; i++)
                dx[i] = (dy[i] + dy[i - 1]) / 2;
        }
        /* -- Zone center 2-dimensional array. */
        else if (syi[N - 1] == 2) {
            nrows = syi[N1 - 1];
            ncols = syi[N2 - 1];
            Dzcen2d(dy, nrows, ncols, dx);
        } else {
            BasError(errmsg3, strlen(errmsg3));
            return ERR;
        }
    }

    Parrel(syi);
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Cfloat ::: handler for float builtin */

Integer Cfloat(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer npoints;
    float *rx, *cx;
    real8 *dx, *dcx;
    int i;

    static char errmsg1[] = "float: illegal argument type.";

#ifdef HAS_DOUBLE_COMPLEX
    if (syi[TC - 1] == COMPLEX) {
#else
    if (syi[TC - 1] == COMPLEX || syi[TC - 1] == DOUBLE_C) {
#endif
        Parcopy(syi, sxi);
        sxi[TC - 1] = REAL;
        Parget(sxi);
        cx = (float *) Paraddr(syi);
        rx = (float *) Paraddr(sxi);
        npoints = Parlen(syi);
        for (i = 0; i < npoints; i++)
            rx[i] = cx[2 * i];
    }
#ifdef HAS_DOUBLE_COMPLEX
    else if (syi[TC - 1] == DOUBLE_C) {
        Parcopy(syi, sxi);
        sxi[TC - 1] = DOUBLE;
        Parget(sxi);
        dcx = (real8 *) Paraddr(syi);
        dx = (real8 *) Paraddr(sxi);
        npoints = Parlen(syi);
        for (i = 0; i < npoints; i++)
            dx[i] = dcx[2 * i];
    }
#endif
    else {
        Parcopy(syi, sxi);
        Cobjcpy(sxi, syi);
        if (Parcoerc(sxi, &myreal) == ERR) {
            BasError(errmsg1, strlen(errmsg1));
            return ERR;
        }
    }
    Parrel(syi);
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Cichar ::: handler for ichar builtin
 *         works on arbitrary array of character * 1
 */

Integer Cichar(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer npoints;
    char *chx;
    Integer *ix;
    int i;

    Parcopy(syi, sxi);
    sxi[TC - 1] = INTEGER;
    Parget(sxi);
    chx = (char *) Paraddr(syi);
    ix = (Integer *) Paraddr(sxi);
    npoints = Parlen(syi);

    for (i = 0; i < npoints; i++)
        ix[i] = (Integer) chx[i];

    Parrel(syi);

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Crmsdv ::: handler for the rmsdv builtin */

Integer Crmsdv(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer npoints, dim, dimstride[8], dimlens[7], szi[SSWIDTH];
    int i, j, newindx, offset, izindx;
    float *rx, *ry, rmsdv, rmssq, rdiff, rvalue;
    real8 *dx, *dy, drmsdv, drmssq, ddiff, dvalue;
    static char
        errmsg1[] = "rmsdv: arg2 is out of range.",
        errmsg2[] = "rmsdv: illegal argument type";

    if (nargs == 1) {
        npoints = Parlen(syi);
        sxi[N - 1] = 0;
        sxi[TC - 1] = syi[TC - 1];
    } else if (nargs == 2) {
        Fcnargb(2, szi);
        if (Parint(szi, &dim) == ERR)
            return ERR;
        Parrel(szi);            /* release space for argument */
        if (dim > syi[N - 1] || dim < 1) {
            BasError(errmsg1, strlen(errmsg1));
            return ERR;
        }
        sxi[N - 1] = syi[N - 1] - 1;
        sxi[TC - 1] = syi[TC - 1];
        dimstride[0] = 1;
        for (i = 0; i < syi[N - 1]; i++) {
            dimlens[i] = syi[N1 + i - 1];
            dimstride[i + 1] = dimstride[i] * dimlens[i];
            if (i < dim - 1) {
                sxi[N1 + i - 1] = syi[N1 + i - 1];
                sxi[M1 + i - 1] = syi[M1 + i - 1];
            } else if (i > dim - 1) {
                sxi[N1 + i - 2] = syi[N1 + i - 1];
                sxi[M1 + i - 2] = syi[M1 + i - 1];
            }
        }
    }
    /* set up space for the answer */
    Parget(sxi);
    if (syi[TC - 1] == INTEGER) {
        if (Parcoerc(syi, &myreal) == ERR) {
            BasError(errmsg2, strlen(errmsg2));
            return ERR;
        }
        sxi[TC - 1] = REAL;
    }
    if (syi[TC - 1] == REAL) {
        rx = (float *) Paraddr(sxi);
        ry = (float *) Paraddr(syi);
        if (nargs == 1) {
            rmsdv = 0.;
            rmssq = 0.;
            for (i = 0; i < npoints; i++) {
                rdiff = ry[i] - ry[0];
                rmsdv += rdiff;
                rmssq += rdiff * rdiff;
            }
            rx[0] = (float) sqrt((double) (max(0.,
                    rmssq / npoints - rmsdv * rmsdv / (npoints * npoints))));
        } else {
            newindx = 0;
            for (offset = 0; offset < dimstride[syi[N - 1]];
                 offset += dimstride[dim])
                for (i = 0; i < dimstride[dim - 1]; i++) {
                    izindx = i + offset;
                    rmsdv = 0.;
                    rmssq = 0.;
                    rvalue = ry[izindx];
                    for (j = 1; j < dimlens[dim - 1]; j++) {
                        izindx += dimstride[dim - 1];
                        rdiff = ry[izindx] - rvalue;
                        rmsdv += rdiff;
                        rmssq += rdiff * rdiff;
                    }
                    rx[newindx] = (float) sqrt((double) (max(0.,
                            rmssq / dimlens[dim-1] -
                            rmsdv * rmsdv /(dimlens[dim-1] * dimlens[dim-1]))));
                    newindx += 1;
                }
        }
    } else if (syi[TC - 1] == DOUBLE) {
        dx = (real8 *) Paraddr(sxi);
        dy = (real8 *) Paraddr(syi);
        if (nargs == 1) {
            drmsdv = 0.;
            drmssq = 0.;
            for (i = 0; i < npoints; i++) {
                ddiff = dy[i] - dy[0];
                drmsdv += ddiff;
                drmssq += ddiff * ddiff;
            }
            dx[0] = (float) sqrt((double) (max(0.e0,
                drmssq / npoints - drmsdv * drmsdv / (npoints * npoints))));
        } else {
            newindx = 0;
            for (offset = 0; offset < dimstride[syi[N - 1]];
                 offset += dimstride[dim])
                for (i = 0; i < dimstride[dim - 1]; i++) {
                    izindx = i + offset;
                    drmsdv = 0.;
                    drmssq = 0.;
                    dvalue = dy[izindx];
                    for (j = 1; j < dimlens[dim - 1]; j++) {
                        izindx += dimstride[dim - 1];
                        ddiff = dy[izindx] - dvalue;
                        drmsdv += ddiff;
                        drmssq += ddiff * ddiff;
                    }
                    dx[newindx] = (float) sqrt((double) (max(0.e0,
                         drmssq / dimlens[dim-1] -
                         drmsdv * drmsdv / (dimlens[dim-1] * dimlens[dim-1]))));
                    newindx += 1;
                }
        }
    }

    Parrel(syi);
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
