/*
 * cfcn9.c
 *
 */

#include "cfcn.h"

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* COUTER - handler for outer builtin */

Integer Couter(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer szi[SSWIDTH], npoints, npointsy, *ix, *iy;
    float *rx, *ry, *cx, *cy;
    real8 *dx, *dy, *dcx, *dcy;
    int i, j, iskip, nskip;
    static char
        errmsg1[] = "outer: illegal second argument type.",
        errmsg2[] = "outer: array will exceed dimension limit.";
    Fcnargb(2, szi);
    Pardim(syi);
    Pardim(szi);
    if (szi[TC - 1] < INTEGER || szi[TC - 1] > DOUBLE_C) {
        BasError(errmsg1, strlen(errmsg1));
        return ERR;
    }
    if (Parlen(syi) == 1) {
        syi[N - 1] = 1;
        syi[N1 - 1] = 1;
        syi[M1 - 1] = 1;
    }
    if (Parlen(szi) == 1) {
        szi[N - 1] = 1;
        szi[N1 - 1] = 1;
        szi[M1 - 1] = 1;
    }
/*
 * ####################################################################
 * ############  DETERMINE CHARACTERISTICS OF NEW MATRIX ##############
 * ####################################################################
 */
    sxi[N - 1] = syi[N - 1] + szi[N - 1];
    if (sxi[N - 1] > 7) {
        BasError(errmsg2, strlen(errmsg2));
        return ERR;
    }
    /* new array must be same type as the highest argument. */
    sxi[TC - 1] = max(syi[TC - 1], szi[TC - 1]);
    if (Parcoerc(syi, &(sxi[TC - 1])) == ERR)
        return ERR;
    if (Parcoerc(szi, &(sxi[TC - 1])) == ERR)
        return ERR;
    /* go through dimensions of syi, make dimensions of sxi the same */
    for (i = N1 - 1; i < N1 + syi[N - 1] - 1; i++)
        sxi[i] = syi[i];
    for (i = M1 - 1; i < M1 + syi[N - 1] - 1; i++)
        sxi[i] = syi[i];
    /* ..... repeat for szi */
    for (i = N1 - 1; i < N1 + szi[N - 1] - 1; i++) {
        iskip = i + syi[N - 1];
        sxi[iskip] = szi[i];
    }
    for (i = M1 - 1; i < M1 + szi[N - 1] - 1; i++) {
        iskip = i + syi[N - 1];
        sxi[iskip] = szi[i];
    }
    /* get space for the array, and do outer product(s) */
    Parget(sxi);
    if (sxi[TC - 1] == INTEGER) {
        ix = (Integer *) Paraddr(sxi);
        npoints = Parlen(sxi);
        iy = (Integer *) Paraddr(syi);
        npointsy = Parlen(syi);
        for (i = 0; i < npoints - npointsy + 1; i += npointsy)
            for (j = 0; j <= npointsy - 1; j++)
                ix[i + j] = iy[j];
        iy = (Integer *) Paraddr(szi);
        npointsy = Parlen(szi);
        nskip = npoints / npointsy;
        for (i = 0; i < npointsy; i++)
            for (j = i * nskip; j < (i + 1) * nskip; j++)
                ix[j] *= iy[i];
    } else if (sxi[TC - 1] == REAL) {
        rx = (float *) Paraddr(sxi);
        npoints = Parlen(sxi);
        ry = (float *) Paraddr(syi);
        npointsy = Parlen(syi);
        for (i = 0; i < npoints - npointsy + 1; i += npointsy)
            for (j = 0; j <= npointsy - 1; j++)
                rx[i + j] = ry[j];
        ry = (float *) Paraddr(szi);
        npointsy = Parlen(szi);
        nskip = npoints / npointsy;
        for (i = 0; i < npointsy; i++)
            for (j = i * nskip; j < (i + 1) * nskip; j++)
                rx[j] *= ry[i];
    } else if (sxi[TC - 1] == DOUBLE) {
        dx = (real8 *) Paraddr(sxi);
        npoints = Parlen(sxi);
        dy = (real8 *) Paraddr(syi);
        npointsy = Parlen(syi);
        for (i = 0; i < npoints - npointsy + 1; i += npointsy)
            for (j = 0; j <= npointsy - 1; j++)
                dx[i + j] = dy[j];
        dy = (real8 *) Paraddr(szi);
        npointsy = Parlen(szi);
        nskip = npoints / npointsy;
        for (i = 0; i < npointsy; i++)
            for (j = i * nskip; j < (i + 1) * nskip; j++)
                dx[j] *= dy[i];
    }
#ifdef HAS_DOUBLE_COMPLEX
    else if (sxi[TC - 1] == COMPLEX) {
#else
    else if (sxi[TC - 1] == COMPLEX || sxi[TC - 1] == DOUBLE_C) {
#endif
        cx = (float *) Paraddr(sxi);
        npoints = Parlen(sxi);
        cy = (float *) Paraddr(syi);
        npointsy = Parlen(syi);
        for (i = 0; i < npoints - npointsy + 1; i += npointsy)
            for (j = 0; j <= npointsy - 1; j++) {
                cx[2 * (i + j)] = cy[2 * j];
                cx[2 * (i + j) + 1] = cy[2 * j + 1];
            }
        cy = (float *) Paraddr(szi);
        npointsy = Parlen(szi);
        nskip = npoints / npointsy;
        for (i = 0; i < npointsy; i++)
            for (j = i * nskip; i < (i + 1) * nskip; i++) {
                cx[2 * j] =
                    cx[2 * j] * cy[2 * i] - cx[2 * j + 1] * cy[2 * i + 1];
                cx[2 * j + 1] =
                    cx[2 * j] * cy[2 * i + 1] + cx[2 * j + 1] * cy[2 * i];
            }
    }
#ifdef HAS_DOUBLE_COMPLEX
    else if (sxi[TC - 1] == DOUBLE_C) {
        dcx = (real8 *) Paraddr(sxi);
        npoints = Parlen(sxi);
        dcy = (real8 *) Paraddr(syi);
        npointsy = Parlen(syi);
        for (i = 0; i < npoints - npointsy + 1; i += npointsy)
            for (j = 0; j <= npointsy - 1; j++) {
                dcx[2 * (i + j)] = dcy[2 * j];
                dcx[2 * (i + j) + 1] = dcy[2 * j + 1];
            }
        dcy = (real8 *) Paraddr(szi);
        npointsy = Parlen(szi);
        nskip = npoints / npointsy;
        for (i = 0; i < npointsy; i++)
            for (j = i * nskip; i < (i + 1) * nskip; i++) {
                dcx[2 * j] =
                    dcx[2 * j] * dcy[2 * i] - dcx[2 * j + 1] * dcy[2 * i +
                                                                   1];
                dcx[2 * j + 1] =
                    dcx[2 * j] * dcy[2 * i + 1] + dcx[2 * j +
                                                      1] * dcy[2 * i];
            }
    }
#endif

    Parrel(syi);
    Parrel(szi);
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CGATHER - handler for gather builtin */

Integer Cgather(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer startss, endss, lenz, *iz, *ix, *iy, szi[SSWIDTH];
    float *rx, *ry, *cx, *cy;
    real8 *dx, *dy, *dcx, *dcy;
    int i;
    static char
        errmsg1[] = "gather: source array must be a 1-D array.",
        errmsg2[] = "gather: index array must be a 1-D array.",
        errmsg3[] = "gather: illegal argument type for index array.",
        errmsg4[] = "gather: index outside range of source array.";
    Pardim(syi);
    if (syi[N - 1] == 0) {
        /* scalar, but fool into thinking 1-D array. */
        syi[N - 1] = 1;
        syi[N1 - 1] = 1;
        syi[M1 - 1] = 1;
        syi[I1 - 1] = 1;
    }
    if (syi[N - 1] != 1) {
        BasError(errmsg1, strlen(errmsg1));
        return ERR;
    }
    sxi[N - 1] = 1;
    sxi[TC - 1] = syi[TC - 1];
    startss = syi[M1 - 1];
    endss = startss + syi[N1 - 1] - 1;
    /* get the index array */
    Fcnargb(2, szi);
    Pardim(szi);
    if (szi[N - 1] == 0) {
        /* scalar, but fool into thinking 1-D array. */
        szi[N - 1] = 1;
        szi[N1 - 1] = 1;
        szi[M1 - 1] = 1;
        szi[I1 - 1] = 1;
    }
    if (szi[N - 1] != 1) {
        BasError(errmsg2, strlen(errmsg2));
        return ERR;
    }
    if (Parcoerc(szi, &myint) == ERR) {
        BasError(errmsg3, strlen(errmsg3));
        return ERR;
    } else {
        iz = (Integer *) Paraddr(szi);
        lenz = Parlen(szi);
        sxi[N1 - 1] = szi[N1 - 1];
        sxi[M1 - 1] = szi[M1 - 1];
    }
    /* set up space for the answer */
    Parget(sxi);
    if (syi[TC - 1] == REAL) {
        rx = (float *) Paraddr(sxi);
        ry = (float *) Paraddr(syi);
        for (i = 0; i < lenz; i++)
            if (iz[i] >= startss && iz[i] <= endss)
                rx[i] = ry[iz[i] - startss];
            else {
                BasError(errmsg4, strlen(errmsg4));
                return ERR;
            }
    } else if (syi[TC - 1] == DOUBLE) {
        dx = (real8 *) Paraddr(sxi);
        dy = (real8 *) Paraddr(syi);
        for (i = 0; i < lenz; i++)
            if (iz[i] >= startss && iz[i] <= endss)
                dx[i] = dy[iz[i] - startss];
            else {
                BasError(errmsg4, strlen(errmsg4));
                return ERR;
            }
    } else if (syi[TC - 1] == INTEGER) {
        ix = (Integer *) Paraddr(sxi);
        iy = (Integer *) Paraddr(syi);
        for (i = 0; i < lenz; i++)
            if (iz[i] >= startss && iz[i] <= endss)
                ix[i] = iy[iz[i] - startss];
            else {
                BasError(errmsg4, strlen(errmsg4));
                return ERR;
            }
    }
#ifdef HAS_DOUBLE_COMPLEX
    else if (syi[TC - 1] == COMPLEX) {
#else
    else if (syi[TC - 1] == COMPLEX || syi[TC - 1] == DOUBLE_C) {
#endif
        cx = (float *) Paraddr(sxi);
        cy = (float *) Paraddr(syi);
        for (i = 0; i < lenz; i++)
            if (iz[i] >= startss && iz[i] <= endss) {
                cx[2 * i] = cy[2 * (iz[i] - startss)];
                cx[2 * i + 1] = cy[2 * (iz[i] - startss) + 1];
            } else {
                BasError(errmsg4, strlen(errmsg4));
                return ERR;
            }
    }
#ifdef HAS_DOUBLE_COMPLEX
    else if (syi[TC - 1] == DOUBLE_C) {
        dcx = (real8 *) Paraddr(sxi);
        dcy = (real8 *) Paraddr(syi);
        for (i = 0; i < lenz; i++)
            if (iz[i] >= startss && iz[i] <= endss) {
                dcx[2 * i] = dcy[2 * (iz[i] - startss)];
                dcx[2 * i + 1] = dcy[2 * (iz[i] - startss) + 1];
            } else {
                BasError(errmsg4, strlen(errmsg4));
                return ERR;
            }
    }
#endif

    Parrel(syi);
    Parrel(szi);

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CRNGSETDF - handler for rngsetdf builtin */

Integer Crngsetdf(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer npointsx, npointsy, *ix, *iy;
    int i;
    static char
        errmsg1[] = "rngsetdf: argument2 not of type RANGE.",
        errmsg2[] = "rngsetdf: argument1 and argument2 size mismatch.";

    Parcopy(syi, sxi);
    npointsx = Parlen(sxi);
    Fcnargb(2, syi);
    npointsy = Parlen(syi);
    if (Parcoerc(syi, &myrng) == ERR) {
        BasError(errmsg1, strlen(errmsg1));
        return ERR;
    }
    if (sxi[TC - 1] == NULL_TYPE) {
        Parcopy(syi, sxi);
        return OK;
    }
    if (npointsx != npointsy && npointsy != 1) {
        BasError(errmsg2, strlen(errmsg2));
        return ERR;
    }
    /* -- IF type RANGE replace defaults,                                      */
    /* -- but if type INTEGER, REAL, DOUBLE, or COMPLEX, return data unchanged */
    if (sxi[TC - 1] == RANGE) {
        ix = (Integer *) Paraddr(sxi);
        iy = (Integer *) Paraddr(syi);
        for (i = 0; i < 3 * npointsx; i++)
            if (ix[i] == DEFAULT) {
                if (npointsy == 1)
                    ix[i] = iy[i % 3];
                else
                    ix[i] = iy[i];
            }
    }

    Parrel(syi);
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CINDEX - handler for index builtin */

Integer Cindex(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer szi[SSWIDTH], len, sublen, *ix;
    char *chx, *chy;
    static char
        errmsg1[] = "index: scalar arguments expected.",
        errmsg2[] = "index: second argument not character.";

    Fcnargb(2, szi);
    if (szi[TC - 1] >= 0) {
        BasError(errmsg2, strlen(errmsg2));
        return ERR;
    }
    if (Parlen(syi) != 1 || Parlen(szi) != 1) {
        BasError(errmsg1, strlen(errmsg1));
        return ERR;
    }
    chy = (char *) Paraddr(syi);
    len = -syi[TC - 1];
    chx = (char *) Paraddr(szi);
    sublen = -szi[TC - 1];
    sxi[N - 1] = 0;
    sxi[TC - 1] = INTEGER;
    Parget(sxi);
    ix = (Integer *) Paraddr(sxi);
    ix[0] = StrIndex(chy, 0, len, chx, 0, sublen);

    Parrel(syi);
    Parrel(szi);

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

static void Iadjust(Integer *jx, Integer *ix, int i, Integer n)
{
    /* Note: indices are 1-based, so subtract 1 for C indexing. */
    int j;
    Integer jtemp;
    Integer itemp;

    jtemp = jx[i - 1];
    itemp = ix[i - 1];
    j = 2 * i;
    while (j <= n) {
        /* Find max of left and right child: */
        if (j < n && jx[j - 1] < jx[j])
            j++;
        /* compare max. child with jtemp; if jtemp is max, done. */
        if (jtemp >= jx[j - 1]) {
            jx[j / 2 - 1] = jtemp;
            ix[j / 2 - 1] = itemp;
            return;
        }
        /* move jx [j] up the tree */
        jx[j / 2 - 1] = jx[j - 1];
        ix[j / 2 - 1] = ix[j - 1];
        j = 2 * j;
    }
    jx[j / 2 - 1] = jtemp;
    ix[j / 2 - 1] = itemp;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

static void Radjust(float *rx, Integer *ix, int i, Integer n)
{
    /* Note: indices are 1-based, so subtract 1 for C indexing. */
    int j;
    float rtemp;
    Integer itemp;

    rtemp = rx[i - 1];
    itemp = ix[i - 1];
    j = 2 * i;
    while (j <= n) {
        /* Find max of left and right child: */
        if (j < n && rx[j - 1] < rx[j])
            j++;
        /* compare max. child with rtemp; if rtemp is max, done. */
        if (rtemp >= rx[j - 1]) {
            rx[j / 2 - 1] = rtemp;
            ix[j / 2 - 1] = itemp;
            return;
        }
        /* move rx [j] up the tree */
        rx[j / 2 - 1] = rx[j - 1];
        ix[j / 2 - 1] = ix[j - 1];
        j = 2 * j;
    }
    rx[j / 2 - 1] = rtemp;
    ix[j / 2 - 1] = itemp;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

static void Dadjust(real8 *dx, Integer *ix, int i, Integer n)
{
    /* Note: indices are 1-based, so subtract 1 for C indexing. */
    int j;
    real8 dtemp;
    Integer itemp;

    dtemp = dx[i - 1];
    itemp = ix[i - 1];
    j = 2 * i;
    while (j <= n) {
        /* Find max of left and right child: */
        if (j < n && dx[j - 1] < dx[j])
            j++;
        /* compare max. child with dtemp; if dtemp is max, done. */
        if (dtemp >= dx[j - 1]) {
            dx[j / 2 - 1] = dtemp;
            ix[j / 2 - 1] = itemp;
            return;
        }
        /* move dx [j] up the tree */
        dx[j / 2 - 1] = dx[j - 1];
        ix[j / 2 - 1] = ix[j - 1];
        j = 2 * j;
    }
    dx[j / 2 - 1] = dtemp;
    ix[j / 2 - 1] = itemp;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CSORT - handler for sort command */

Integer Csort(Integer *sxi, f_entry *thisf, Integer nargs, Integer *syi)
{
    Integer szi[SSWIDTH], szzi[SSWIDTH], *iy, *iz, itmp, len;
    float *ry, rtmp;
    real8 *dy, dtmp;
    int i, j, swapflg;

    static char
        errmsg1[] =
        "sort: first argument must be integer, real, or double.",
        errmsg2[] =
        "sort: second argument must be an address.", errmsg3[] =
        "sort: second argument must point to an integer array.",
        errmsg4[] = "sort: second argument is too short.", errmsg5[] =
        "sort: third argument must be integer length.", errmsg6[] =
        "sort: third argument must be integer scalar.", errmsg7[] =
        "sort: third argument < 0 or > length of array!";

    if (nargs == 3) {
        Fcnargb(3, szzi);
        if (szzi[TC - 1] != INTEGER) {
            BasError(errmsg5, strlen(errmsg5));
            return ERR;
        } else if (szzi[N - 1] > 0) {
            BasError(errmsg6, strlen(errmsg6));
            return ERR;
        }
        if (Parint(szzi, &len) == ERR)
            return ERR;
        if (len < 0 || len > syi[N1 - 1]) {
            BasError(errmsg7, strlen(errmsg7));
            return ERR;
        }
        Parrel(szzi);
    } else {
        /* paramb has computed the actual length and put it in syi [N1 - 1] */
        len = syi[N1 - 1];
    }
    if (len == 0)
        return OK;              /*nothing to sort */
    if (syi[TCSAVE - 1] < INTEGER || syi[TCSAVE - 1] > DOUBLE) {
        BasError(errmsg1, strlen(errmsg1));
        return ERR;
    }
    Fcnargb(2, szi);
    if (szi[TC - 1] != FWA) {
        BasError(errmsg2, strlen(errmsg2));
        return ERR;
    }
    if (szi[TCSAVE - 1] != INTEGER) {
        BasError(errmsg3, strlen(errmsg3));
        return ERR;
    }
    if (szi[N1 - 1] < len) {
        BasError(errmsg4, strlen(errmsg4));
        return ERR;
    }
    iz = (Integer *) Paraddr(szi);
    if (syi[TCSAVE - 1] == INTEGER) {
        iy = (Integer *) Paraddr(syi);
    } else if (syi[TCSAVE - 1] == REAL) {
        ry = (float *) Paraddr(syi);
    } else if (syi[TCSAVE - 1] == DOUBLE) {
        dy = (real8 *) Paraddr(syi);
    }
    for (i = 1; i <= szi[N1 - 1]; i++)
        iz[i - 1] = i;
    if (len <= 25) {            /* Do bubble sort */
        swapflg = 1;
        if (syi[TCSAVE - 1] == INTEGER)
            Sorti(iy, iz, &len);
        else if (syi[TCSAVE - 1] == REAL) {
            for (i = len - 1; i >= 0; i--) {
                if (!swapflg)
                    break;
                for (j = 1; j <= i; j++) {
                    if (ry[j - 1] > ry[j]) {
                        rtmp = ry[j - 1];
                        ry[j - 1] = ry[j];
                        ry[j] = rtmp;

                        itmp = iz[j - 1];
                        iz[j - 1] = iz[j];
                        iz[j] = itmp;
                        swapflg = 1;
                    }
                }
            }
        } else {
            for (i = len - 1; i >= 0; i--) {
                if (!swapflg)
                    break;
                for (j = 1; j <= i; j++) {
                    if (dy[j - 1] > dy[j]) {
                        dtmp = dy[j - 1];
                        dy[j - 1] = dy[j];
                        dy[j] = dtmp;

                        itmp = iz[j - 1];
                        iz[j - 1] = iz[j];
                        iz[j] = itmp;
                        swapflg = 1;
                    }
                }
            }
        }
    } else {                    /* Do heap sort */
        if (syi[TCSAVE - 1] == INTEGER) {
            for (i = len / 2; i >= 1; i--)
                Iadjust(iy, iz, i, len);
            for (i = len - 1; i >= 1; i--) {
                itmp = iy[i];
                iy[i] = iy[0];
                iy[0] = itmp;
                itmp = iz[i];
                iz[i] = iz[0];
                iz[0] = itmp;
                Iadjust(iy, iz, 1, i);
            }

        } else if (syi[TCSAVE - 1] == REAL) {
            for (i = len / 2; i >= 1; i--)
                Radjust(ry, iz, i, len);
            for (i = len - 1; i >= 1; i--) {
                rtmp = ry[i];
                ry[i] = ry[0];
                ry[0] = rtmp;
                itmp = iz[i];
                iz[i] = iz[0];
                iz[0] = itmp;
                Radjust(ry, iz, 1, i);
            }
        } else {
            for (i = len / 2; i >= 1; i--)
                Dadjust(dy, iz, i, len);
            for (i = len - 1; i >= 1; i--) {
                dtmp = dy[i];
                dy[i] = dy[0];
                dy[0] = dtmp;
                itmp = iz[i];
                iz[i] = iz[0];
                iz[0] = itmp;
                Dadjust(dy, iz, 1, i);
            }
        }
    }
    /* Takes the addresses out of the literal table, does not release storage. */
    Parrel(syi);
    Parrel(szi);
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
