/*
 * CMAC.C
 *
 * This file contains the dynamic variables formerly used in mac.m.
 * Zane C. Motteler, 2/17/2000
 */

#include "rt.h"

typedef struct {
    Integer dlimtype;
    Integer comtype;
    logical mactype;
} MacType;

typedef struct {
    Integer macatr;
    logical parnchk;
} DefM;

static MacType *mactype;
static DefM *defm;

static Integer mactypesize = sizeof(MacType) / sizeof(Integer) +
    (sizeof(MacType) % sizeof(Integer) ? 1 : 0);
static Integer defmsize = sizeof(DefM) / sizeof(Integer) +
    (sizeof(DefM) % sizeof(Integer) ? 1 : 0);

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

void Mccomchg(Integer minsiz)
{
    Integer oxcomsz;

    oxcomsz = mxcomsz;
    while (minsiz > mxcomsz)
        mxcomsz += COMINC;
    Oschangn((Address *) &mactype, mxcomsz * mactypesize,
             oxcomsz * mactypesize, "PERM|MCCOMCHG:mactype");
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

static Integer delimdef;
static logical defflg;
static Integer *dtemp;

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Clear delimdef prior to parsing command string */

void Mccomclr(void)
{
    delimdef = 0;
    defflg = (logical) 0;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Default the first Comflg entries of mactype
 * the token is 'command' alone, so we default to white space and comma
 * as delimiters, and expressions with macro expansion as arguments
 */
void Mccomdef(Integer Comflg)
{
    int i;

    delimdef = 0;
    delimdef |= (1 << (DLIM_WS - 1));
    delimdef |= (1 << (DLIM_CM - 1));
    for (i = 0; i < Comflg; i++) {
        mactype[i].dlimtype = delimdef;
        mactype[i].comtype = EXPRCOM;
        mactype[i].mactype = (logical) 1;
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Set the ith entry of mactype */
void Smactyp(Integer dlim, Integer com, logical mac, Integer i)
{
    mactype[i - 1].dlimtype = dlim;
    mactype[i - 1].comtype = com;
    mactype[i - 1].mactype = mac;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

logical Gmactype(Integer i)
{
    return mactype[i - 1].mactype;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* An argument is coming along */
void Mcarg(Integer *argcnt, Integer Comflg, Integer ch)
{
    int k;
    static char msg[] = "mclrv122: too many command arguments specified.";

    if (*argcnt == 0) {
        if (!defflg) {
            defflg = (logical) 1;
            delimdef |= (1 << (DLIM_WS - 1));
            delimdef |= (1 << (DLIM_CM - 1));
        }
        for (k = 0; k < Comflg; k++)
            mactype[k].dlimtype = delimdef;
    }
    *argcnt += 1;
    if (*argcnt > mxcomsz) {
        BasError(msg, strlen(msg));
        Kaboom(0);
    }
    mactype[*argcnt - 1].mactype = (logical) (ch > 0 ? 1 : 0);
    mactype[*argcnt - 1].comtype = (ch > 0 ? ch : -ch) - 17;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

void Mctloop(Integer lparen, Integer argcnt)
{
    int k, l;

    if (lparen == 0)
        for (k = argcnt; k < mxcomsz; k++) {
            mactype[k].comtype = mactype[argcnt - 1].comtype;
            mactype[k].mactype = mactype[argcnt - 1].mactype;
    } else
        for (k = argcnt; k < mxcomsz;)
            for (l = lparen - 1; l < argcnt; l++) {
                mactype[k] = mactype[l];
                k += 1;
                if (k >= mxcomsz)
                    break;
            }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

void Cldefflg(void)
{
    defflg = (logical) 0;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* setting delimiters */
void Mccomdel(Integer argcnt, Integer ch)
{
    if (argcnt == 0) {
        /* setting defaults */
        defflg = (logical) 1;
        dtemp = &delimdef;
    } else
        dtemp = &(mactype[argcnt - 1].dlimtype);
    if (ch > 0)
        *dtemp |= 1 << (ch - 1);
    else
        *dtemp &= ~(1 << ((-ch) - 1));
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Initialize the defm block */
void Defminit(void)
{
    int i;

    Osallotn((Address *) &defm, defmptr * defmsize, "PERM|DEFMINIT:defm");
    for (i = 0; i < defmptr; i++) {
        defm[i].macatr = 0;
        defm[i].parnchk = (logical) 0;
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Add something to the defm block, and increase its size if necessary */
void Defmchk(Integer jname, logical argflg)
{
    int i;

    if (jname - 1 >= defmptr) {
        Oschangn((Address *) &defm, (jname + 100) * defmsize,
                 defmptr * defmsize, "defm");
        for (i = defmptr; i < jname + 100; i++) {
            defm[i].macatr = 0;
            defm[i].parnchk = (logical) 0;
        }
        defmptr = jname + 100;
    }
    defm[jname - 1].macatr = 0;
    defm[jname - 1].parnchk = argflg;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Clear a specific entry in the defm block */
void Clrdefm(Integer jentry)
{
    defm[jentry - 1].macatr = 0;
    defm[jentry - 1].parnchk = (logical) 0;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Tell whether a macro has parameters ZCM 9/12/94
 * called from pfb
 */
logical Mcparck(Integer jentry)
{
    return defm[jentry - 1].parnchk;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* return the attribute word for macro k */
Integer Rtmacat(Integer k)
{
    return defm[k - 1].macatr;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

void Rcomtyp(Integer Comarg)
{
    switch (mactype[Comarg - 1].comtype) {
    case STRCOM:
        mactype[Comarg - 1].comtype = EXPRCOM;
        return;
    case EXPRCOM:
        mactype[Comarg - 1].comtype = STRCOM;
    default:
        return;
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

Integer Gcomtyp(Integer i)
{
    return mactype[i - 1].comtype;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

Integer Gdlimtyp(Integer i)
{
    return mactype[i - 1].dlimtype;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* return the address of the ith macatr */
Integer *Gmacatra(Integer i)
{
    return &(defm[i - 1].macatr);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
