/*
 * code.c
 *
 * These routines were moved here and recoded in C from lr.m
 * because the extensive access to dynamic integer arrays can
 * better be handled in C than by means of access functions
 * called for each access to one of these arrays.
 * --Zane C. Motteler (13 January 2000)
 */

#include "rt.h"

Integer
    *intcode,
    *ic,
    *forstck,
    *lbstack,
    *looptyp,
    *loopctl,
    *fnfwa,
    *argfwa,
    *arglen,
    *lpnext;
Integer
    nextcode,
    mxcode,
    codebgn,
    mdl,
    forstptr,
    lbstptr,
    mxdep,
    nestlevl,
    lpdepth,
    nextlab;
logical
    set1flg;

static logical ltrue = (logical) 1;

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *
 *      Written by Zane C. Motteler
 *              July, 1986
 *
 *      This subroutine is called whenever a label for a forward jump
 *      is emitted; It checks all jumps in the intermediate code
 *      and replaces the label of their destination with actual
 *      location in the intermediate code.
 *
 *      Revised by Zane C. Motteler
 *              June, 1988
 *      Eliminate references to the INT instruction (which was used to
 *      implement the obsolete EVENT command) and to servflg (used to tell
 *      whether the obsolete SERVE command had been seen).
 *      Later bug fix: INT is now used to translate the BREAK instruction,
 *      which must be handled differently from a JMP because it does its
 *      own unnesting.
 *      Eliminate references to the LOOPCK instruction (our only self-
 *      altering instruction.)
 *               August, 1989
 *      replace LOOPJ with DOTEST
 *                January, 2000
 *       Rewrite in C.
 */
static void getlbls(Integer lbl, Integer lblno)
{
    int i, incr;
    integer ci, ci1;

    for (i = codebgn - 1; i < nextcode - 1; i += incr) {
        ci = intcode[i];
        ci1 = intcode[i + 1];
        if (ci == JMP || ci == JF || ci == DOTEST || ci == INT)
            if (lblno == ci1 - 1000000)
                intcode[i + 1] = lbl - i - 1;
        if (ci < 0)
            incr = 3;
        else if (ci == DOTEST || ci == JF || ci == JMP || ci == INT ||
                 ci == LINE || ci == NEST || ci == UNNEST)
            incr = 2;
        else
            incr = 1;
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *      Programmed by Zane C. Motteler
 *              August, 1986
 *      Purpose: emit a one-word operation or value.
 *      Revised by Zane C. Motteler
 *              January, 1994
 *      turn off line suppression if it's on and we emit a non-LINE opcode.
 *      takes care of the weird case endf;x=y or some such
 *       Revised by Zane C. Motteler
 *               January, 2000
 *       written in C for use in this module only
 */

void Lremit(Integer value)
{
    if (linsuprs && value != LINE)
        linsuprs = (logical) 0;
    intcode[nextcode - 1] = value;
    nextcode++;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Initialize the intermediate code block.
 */
void Codeinit(void)
{
    codebgn = 1;
    nextcode = codebgn;
    mxcode = MAXCODE;
    Osallotn((Address *) (&intcode), mxcode, "PERM|CODEINIT:intcode");
    mdl = MD;
    Osallotn((Address *) (&forstck), mdl, "PERM|CODEINIT:forstck");
    Osallotn((Address *) (&lbstack), mdl, "PERM|CODEINIT:lbstack");
    lbstptr = 0;
    forstptr = 0;
    nestlevl = 0;
    mxdep = MAXDEP;
    Osallotn((Address *) (&looptyp), mxdep, "PERM|CODEINIT:looptyp");
    Osallotn((Address *) (&loopctl), mxdep, "PERM|CODEINIT:loopctl");
    Osallotn((Address *) (&fnfwa), mxdep, "PERM|CODEINIT:fnfwa");
    Osallotn((Address *) (&argfwa), mxdep + 1, "PERM|CODEINIT:argfwa");
    Osallotn((Address *) (&arglen), mxdep + 1, "PERM|CODEINIT:arglen");
    Osallotn((Address *) (&lpnext), 2 * mxdep, "PERM|CODEINIT:lpnext");
    argfwa[0] = ufcnptr;
    arglen[0] = 0;
    lpdepth = 0;
    nextlab = 1;
    set1flg = (logical) 0;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Places a string table number into the intcode block */

void Strtocod(Integer ivalue, Integer ioffset)
{
    intcode[nextcode + ioffset] = ivalue;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *      Programmed by Zane C. Motteler
 *              July, 1986
 *      Purpose: At the end of a then clause, evaluate a label for the
 *               upcoming else clause, and insert an unconditional jump
 *               to the end of the if structure.
 *      Revised by Zane C. Motteler
 *              1 September 1993
 *      Purpose: Insert a line no. opcode after the JMP so basis won't
 *               lose track of where it is. (Only if in user function).
 *       Revised by Zane C. Motteler
 *               January 2000
 *       Purpose: Rewrite in C.
 */
void Lrelse(void)
{
    Integer lbl, lblno;

    lbl = nextcode + 2;         /* insert label for else clause */
    lblno = lbstack[lbstptr - 1];
    getlbls(lbl, lblno);
    lbstptr--;                  /* label used, pop stack        */
    Lremit(JMP);
    Lremit(lbstack[lbstptr - 1] + 1000000);
    if (fdepth > 0) {
        Lremit(LINE);
        Lremit(ftextlno(fdepth - 1));
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *      Programmed by Zane C. Motteler
 *              July, 1986
 *      Purpose: Increment nesting level and set up NEXT and BREAK
 *               labels for loop statements.
 *      Revised by Zane C. Motteler
 *              August 30, 1993
 *      Purpose: emit a LINE opcode at each jump target in a function
 *       Reprogrammed by Zane C. Motteler in C (January 2000)
 */
void Lrloops(Integer t)
{
    Parnest(&t);
    Lremit(NEST);
    Lremit(t);
    lpdepth++;
    /* target of next: */
    lpnext[(lpdepth - 1) * 2 + NEXT_TOK - 1] = nextcode;
    if (fdepth > 0) {
        Lremit(LINE);
        Lremit(ftextlno(fdepth - 1));
    }
    /* target of break: */
    lpnext[(lpdepth - 1) * 2 + BREAK_TOK - 1] = nextlab;
    nextlab++;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *      Programmed by Zane C. Motteler
 *              July, 1986
 *      Purpose: Do all the bookkeeping at the end of a loop:
 *      Evaluate the break label
 *      Decrement the nesting level
 *      Revised by Zane C. Motteler
 *              August 30, 1993
 *      Purpose: Emit a LINE opcode to make sure Basis knows line no.
 *               at end of loop--must occur after UNNEST since instr after
 *               UNNEST is target of a BREAK
 *       Rep-rogrammed by Zane C. Motteler in C (January 2000)
 */
void Lrlpend(void)
{
    Integer lblno;

    lblno = lpnext[(lpdepth - 1) * 2 + BREAK_TOK - 1];
    /* break label */
    getlbls(nextcode, lblno);
    /* pop stacks */
    nestlevl--;
    Lremit(UNNEST);
    Lremit(1);
    if (fdepth > 0) {
        Lremit(LINE);
        Lremit(ftextlno(fdepth - 1));
    }
    lpdepth--;
    if (nestlevl == 0)
        nextlab = 1;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *      Programmed by Zane C. Motteler
 *              September 1986
 *      Purpose: generate flow of control at the end of a logical if.
 *              The top of lbstack is the label number of the conditional
 *              "jump false" that was emitted at the end of the IF
 *              expression. This is backpatched with the current location.
 *              lbstack and looptype are popped (lbstack is popped twice,
 *              the lower label being used only for a block IF--this system
 *              does not know what kind of IF we have when the two entries
 *              are pushed).
 *       Reprogrammed by Zane C. Motteler in C (January 2000)
 */
void Lrp800(void)
{
    Integer lbl, lblno;

    /* a label is emitted at the end of a logical if,  *
     * which means that all jumps to this label must   *
     * be filled in with the next code location.       */
    lbl = nextcode;
    lblno = lbstack[lbstptr - 1];
    getlbls(lbl, lblno);
    /* pop stacks */
    lbstptr -= 2;
    nestlevl--;
    Lremit(UNNEST);
    Lremit(1);
    if (nestlevl == 0)
        nextlab = 1;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *      Programmed by Zane C. Motteler
 *              September 1986
 *      Purpose: Take care of flow of control at end of block IF. Backpatches
 *              two labels. The top one on the stack is (if nonzero) the label
 *              jumped to by the last "jump false" (this only occurs when the
 *              block ends with an ELSEIF, not with an else); the second one on
 *              the stack is the target of all "jump"s at the end of THEN
 *              clauses. When done, lbstack is popped two and looptype is popped
 *              one.
 *      Revised (ZCM 1/13/94) emit a line number opcode if we are in a
 *              function.
 *       Reprogrammed by Zane C. Motteler in C (January 2000)
 */
void Lrp801(void)
{
    Integer lbl, lblno;

    /* target of last JF if any */
    lbl = nextcode;
    if (lbstack[lbstptr - 1] != 0) {
        lblno = lbstack[lbstptr - 1];
        getlbls(lbl, lblno);
    }
    /* target of all JMPs (successful then clauses) */
    lblno = lbstack[lbstptr - 2];
    getlbls(lbl, lblno);
    /* pop stacks */
    lbstptr -= 2;
    nestlevl--;
    if (fdepth > 0) {
        Lremit(LINE);
        Lremit(ftextlno(fdepth - 1));
    }
    Lremit(UNNEST);
    Lremit(1);
    if (nestlevl == 0)
        nextlab = 1;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *      Programmed by Zane C. Motteler
 *              September 1986
 *      Purpose: Take care of flow of control at the end of logical expression
 *              controlling the execution of an IF and a WHILE. Emits a "jump
 *              on false" to the top label on lbstack (IF) or to the BREAK
 *              address on lpnext stack (WHILE).
 *       Reprogrammed in C by Zane C. Motteler
 *               January 2000
 */
void Lrp803(void)
{
    Lremit(JF);
    if (looptyp[nestlevl - 1] == IF_STRUCT)
        Lremit(lbstack[lbstptr - 1] + 1000000);
    else
        Lremit(lpnext[2 * (lpdepth - 1) + BREAK_TOK - 1] + 1000000);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *      Programmed by Zane C. Motteler
 *              September 1986
 *      Purpose: Emit the JF to NEXT address on the value of the UNTIL
 *              clause of a DO..UNTIL.
 *       Reprogrammed by Zane C. Motteler in C (January 2000)
 */

void Lrp810(void)
{
    Lremit(JF);
    Lremit(lpnext[2 * (lpdepth - 1) + NEXT_TOK - 1] - nextcode + 1);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *      Programmed by Zane C. Motteler
 *              September 1986
 *      Rewritten by Zane C. Motteler
 *              August 1989
 *      Purpose: Set the NEXT address for a controlled DO to be the DOTEST
 *              instruction. Emit SET1 (which puts the loop control parameters
 *              into the stack at runtime when the loop is entered);  and emit
 *              DOTEST, which is set to jump to the BREAK label on lpnext stack.
 *              Revision: 8/30/93, ZCM if inside a function, emit a LINE directive so
 *                Basis can keep track of line numbers inside functions.
 *       Rewritten in C by Zane C. Motteler
 *               January 2000
 *
 *      This eliminates the separate dostack.
 */

void Lrp814(void)
{
    lpnext[(lpdepth - 1) * 2 + NEXT_TOK - 1] = nextcode + 1;
/*NEXT needs to be reset to after computations to initialize the loop*/
    Lremit(SET1);
    if (fdepth > 0) {
        Lremit(LINE);
        Lremit(ftextlno(fdepth - 1));
    }
    Lremit(DOTEST);
    Lremit(lpnext[(lpdepth - 1) * 2 + BREAK_TOK - 1] + 1000000);
    looptyp[nestlevl - 1] = DO_DEF;

}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *      Programmed by Zane C. Motteler
 *              September 1986
 *      Purpose: Emit the code which at runtime creates a stack element
 *              for the constant 1. This is for the default increment of a
 *              DO loop, in the case none is specified.
 *      Revised by Zane C. Motteler
 *              June 1988
 *      Revision: The location of 1 in the "values" array is now placed
 *              in intcode(nextcode+1). ntoken, formerly stored here, was
 *              never used; so it now contains a pointer to the value of the
 *              constant (in this case, 1) which does not need to be recomputed
 *              each time it is referred to.
 *      Revised by Zane C. Motteler
 *              July 1989
 *      Revision: alter format of 5-word instruction so word 4 contains token
 *              length and word 5 contains the actual token.
 *      Revision: (ZCM 8/13/1999) Instruction is now 3 words:
 *              word 1 is action, word 2 is index into values, word 3
 *              is index into string table.
 *       Reprogrammed by Zane C. Motteler in C
 *               January, 2000
 */
void Lrp816(void)
{
    static char str[] = "1";

    /* emit code to create a stack element with 1 on it */
    Lremit(-101);
    /* put 1 into values array; conversion no longer necessary */
    Lremit(Find(1, 1));         /* Location of value */
    Lremit(StrgFind(str));
    Lremit(FACLIT);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *      Programmed by Zane C. Motteler
 *              September 1986
 *
 *      Creates database entry for the function now that the defn
 *      is complete.  Necessary info about the name, etc. was
 *      saved by lrvact under the influence of funcflag.
 *
 *      The nestlevel is popped and the code and tokens made permanent.
 *
 *      A return statement with no argument is issued so as to return
 *      NULL if an endf is reached at run-time
 *
 *      Revised by Zane C. Motteler
 *              July 1989
 *      Dynamic space is no longer needed for tokens since they are embedded
 *      in variable-length instructions.
 *      Revised by Zane C. Motteler
 *                 January, 1994
 *      Set suppression of next LINE if we've just seen endf<peopl>
 *       Reprogrammed by Zane C. Motteler in C
 *                  January, 2000
 */
Integer Lrp823(void)
{
    Integer *newcode;
    Address anewcode;

    NameType *newnametype;
    Address anewnametype;

#define newname(i) newnametype[i].names
#define newtype(i) newnametype[i].types
    int i, stop;

    /* emit code as if there was a "return" statement. */
    Lremit(NULLRET);            /* push a null on the stack */
    Lremit(RET);                /* perform the return       */
    if (fdepth > 1)
        linsuprs = (logical) 1;
    /* create space for the newly created function and its tokens, and move them */
    ufcnlen = nextcode - fnfwa[nestlevl - 1];
    Osallotn(&anewcode, ufcnlen, "LRP823:anewcode");
    newcode = (Integer *) anewcode;
    for (i = fnfwa[nestlevl - 1]; i <= nextcode - 1; i++)
        newcode[i - fnfwa[nestlevl - 1]] = intcode[i - 1];
    ufcnstrt = (Address) newcode;
    /* do the same for names and types of args */
    Osallotn(&anewnametype, (arglen[nestlevl] + 1) * sizeof(NameType),
             "LRP823:anewnametype");
    newnametype = (NameType *) anewnametype;
    for (i = 1; i <= arglen[nestlevl] + 1; i++) {
        newtype(i - 1) = funpart(i + argfwa[nestlevl] - 2);
        strcpy(newname(i - 1), funparn(i + argfwa[nestlevl] - 2));
    }
    ufcnname = anewnametype;
    if (ufcnargc < umnargs)
        umnargs = ufcnargc;
    /* make a database entry in user database */
    if (Rtfcneu(&ltrue) == 0)
        return (0);
    /* restore nextcode to previous value */
    nextcode = fnfwa[--nestlevl];
    i = nestlevl;
    stop = 0;
    while (i > 0 && !stop) {
        if (looptyp[i - 1] == FUNCTION_COMPILING) {
            stop = 1;
            ufcnptr = argfwa[i];
            ufcnargc = arglen[i] - 1;
        }
        i--;
    }
    if (!stop) {
        ufcnptr = argfwa[0];
        ufcnargc = arglen[0];
    }
    return (OK);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Zane C. Motteler (January 2000)
 * Put here to get certain common references out of lr.m.
 */
void Lrp824(void)
{
    cfuncdep++;
    /* End special treatment of names: */
    funcflag = (logical) 0;
    argfwa[nestlevl] = ufcnptr;
    arglen[nestlevl] = 1 + ufcnargc;
    ufcnptr = ufcnptr + 1 + ufcnargc;
    Mcunnest();
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 * Action 123 pops the break or next level off the stack.
 * It is used at compile time, not runtime, and can build
 * up the stack needlessly. It always occurs *after* an
 * INT has been emitted. So the INT needs to be moved up,
 * and the 123 placed right before it.
 */
void Lrp123(void)
{
    intcode[nextcode - 1] = intcode[nextcode - 2];
    intcode[nextcode - 2] = intcode[nextcode - 3];
    intcode[nextcode - 3] = 123;
    nextcode++;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *      Programmed by Zane C. Motteler
 *              August, 1986
 *      Purpose: emit the flow of control code at the end of <forout>, the
 *              expression in the FOR statement that controls termination.
 *      Revision: (8/30/93, ZCM) emit a LINE directive to help Basis keep track
 *                of line numbers in loops
 *       Reprogrammed by Zane C. Motteler in C
 *               January 2000
 */
 /* jump on false to break address (loop terminates): */
void Lrpfor(void)
{
    Lremit(JF);
    Lremit(lpnext[(lpdepth - 1) * 2 + BREAK_TOK - 1] + 1000000);
    /* otherwise jump around lstmt to body of loop: */
    Lremit(JMP);
    Lremit(nextlab + 1000000);
    /* address of lstmt is preserved on stack: */
    lbstack[lbstptr] = nextcode;
    /* for eventual address of body of loop: */
    lbstack[lbstptr + 1] = nextlab;
    /* new NEXT address for for: */
    lpnext[(lpdepth - 1) * 2 + NEXT_TOK - 1] = nextcode;
    if (fdepth > 0) {
        Lremit(LINE);
        Lremit(ftextlno(fdepth - 1));
    }
    lbstptr += 2;
    nextlab++;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *      Programmed by Zane C. Motteler
 *              August, 1986
 *      Purpose: emit the flow of control at the end of lstmt in a FOR--
 *              lstmt is logically performed after the body of the for,
 *              which is physically later in the code, so this is the
 *              true end of the loop and must branch back to the NEXT address.
 *       Reprogrammed by Zane C. Motteler in C
 *               January 2000
 */
void Lrpfor2(void)
{
    Integer lbl, lblno;

    Lremit(JMP);
    Lremit(forstck[forstptr - 1] - nextcode + 1);
    forstptr--;
    /* this last is to fill in the address of the JF at the end *
     * of <forout>, the FOR loop control.                       */
    lbl = nextcode;
    lblno = lbstack[lbstptr - 1];
    getlbls(lbl, lblno);
    lbstptr--;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Handle the production forinit ::= assignlist | <empty>
 */
void Lrpfor3(void)
{
    forstptr += 1;
    forstck[forstptr - 1] = nextcode;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *      Programmed by Zane C. Motteler
 *                      July,1986
 *      Purpose:        emit the code for an unconditional jump out of
 *                      the proper nesting level when a break or next is
 *                      encountered. Note that at level 0, a break or next
 *                      is ignored, and an attempt to break or next to
 *                      a nonexistent level (negative or too high) is also
 *                      ignored.
 *                      Bug fix, June 1988: Use the old INT instruction for breaks.
 *                      A break does its own unnesting, so should jump to the
 *                      instruction after the loop's UNNEST.
 *       Reprogrammed by Zane C. Motteler in C (January 2000)
 */
void Setbknx(Integer brknxt, Integer outno)
{
    if (brknxt != 0) {
        if (brkflg) {
            brkflg = (logical) 0;
            Lremit(UNNEST);
            Lremit(outno);
            Lremit(INT);
            Lremit(lpnext[(brknxt - 1) * 2 + BREAK_TOK - 1] + 1000000);
        } else if (nextflg) {
            nextflg = (logical) 0;
            Lremit(UNNEST);
            Lremit(outno - 1);
            Lremit(JMP);
            Lremit(lpnext[(brknxt - 1) * 2 + NEXT_TOK - 1] - nextcode + 1);
        }
    } else if (nextflg || brkflg) {
        /* no action if not in a loop deep enough */
        nextflg = (logical) 0;
        brkflg = (logical) 0;
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Zane C. Motteler (January 2000) Trying to isolate label stuff
 * in this routine.
 */

void Lrp113(void)
{
    Lrelse();
    lbstack[lbstptr++] = 0;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Zane C. Motteler (January 2000) Trying to isolate label stuff
 * in this routine.
 */
void Lrp114(void)
{
    Lrelse();
    lbstack[lbstptr++] = nextlab++;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *      Programmed by Zane C. Motteler
 *              September, 1986
 *      Purpose: Called when the word "function" is seen
 *              1. Push FUNCTION_COMPILING on looptype stack
 *              2. Set FUNCFLAG to indicate start of name collection
 *              3. Set ufcnargc, ufcnstrt and emit a FCNENTER
 *      Revised by Zane C. Motteler
 *              July, 1989
 *      Remove creation of a new block of tokens (now embedded in code)
 *      Revised by Zane C. Motteler
 *              August, 1993
 *      Initialize for collection of function text at compile time
 *      Revised by Zane C. Motteler
 *              September 9, 1993
 *      call macro nest routine (will suppress macro expansion on
 *      function name and arguments)
 *              September 17, 1993
 *      when parsing a function recursively, start saving text
 *              January 10-11, 1994
 *      add variable firstf. Set to TRUE when keyword 'function'
 *      is seen (signals lxgchr to save contents of pushback stack,
 *      this time only; pushback stack could contain macro-expanded
 *      text or a blank character which occurred after 'function').
 *              May 9, 1994
 *      eliminate firstf--part of function text fix.
 *       Reprogrammed by Zane C. Motteler in C (January 2000)
 */
Integer Lrv107(Integer recur)
{
    static integer func_c = FUNCTION_COMPILING;

    funcflag = (logical) 1;
    ufcnargc = -1;              /* haven't even seen name of it yet */
    umnargs = MAXCFARGS;
    Parnest(&func_c);
    fnfwa[nestlevl - 1] = nextcode;
    ftextlno(fdepth) = 0;       /* line number so far ZCM 8/93 */
    if (fdepth == 0) {
        ftextptr(0) = 0;
        /* outermost function declaration:
         * don't insert word "function" if it's nested--we are
         * already putting characters one at a time into buffer so
         * it will already be there.
         */
        if (textpos + 8 > textsize) {
            /* double text buffer size */
            Oschangn((Address *) &functext, 2 * textsize, textsize,
                     "function text block");
            textsize *= 2;
        }
        strncpy(((char *) functext) + textpos, "function", 8);
        textpos += 8;
    } else                      /* we have to move ftextptr back to start of 'function' */
        ftextptr(fdepth) = textpos - 8;
    fdepth++;
    /* calculate what base of sp is for compiled statements inside the function
     *  function <funcdes> <eos> <stlist> endf
     *  <stlist> ::= null | <stlist> <stmt> <eos>
     *   sp points to the word function so stmt gets started at sp+4
     */
    Lremit(FCNENTER);
    ftextlno(fdepth - 1) += 1;
    Lremit(LINE);
    Lremit(ftextlno(fdepth - 1));       /* emit Line no. opcode ZCM 8/93 */
    Mcnest();
    return (OK);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *      Programmed by Zane C. Motteler
 *              September, 1986
 *      Purpose: Push IF on looptype stack. Emit a NEST instruction.
 *              Push two labels on lbstack: the first to be target of all
 *              JMPs at end of THEN clauses; the second to be target of
 *              the next JF, updated each time an ELSEIF is seen, and
 *              dummied on an ELSE clause.
 *       Reprogrammed by Zane C. Motteler in C (January 2000)
 */
void Lrv112(void)
{
    static integer if_s = IF_STRUCT;

    Parnest(&if_s);
    Lremit(NEST);
    Lremit(IF_STRUCT);
    lbstack[lbstptr] = nextlab; /* target of all JMPs in if (end of thens) */
    lbstack[lbstptr + 1] = nextlab + 1; /* target of next JF */
    nextlab += 2;
    lbstptr += 2;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *      Programmed by Zane C. Motteler
 *              June 1988
 *      Purpose: Handle the flow of control upon encountering an ENDDO in the
 *              input, namely emit a JMP to the NEXT address.
 *              Set semerr if we are in the wrong context for an ENDDO.
 *       Reprogrammed by Zane C. Motteler in C (January 2000)
 */

void Lrv115(void)
{
    static char msg[] = "lrv115: ENDDO occurs in the wrong context.";

    if (looptyp[nestlevl - 1] == DO_DEF || looptyp[nestlevl - 1] == DO_IND) {
        Lremit(JMP);
        Lremit(lpnext[(lpdepth - 1) * 2 + NEXT_TOK - 1] - nextcode + 1);
    } else {
        Remark(msg, strlen(msg));
        semerr = ERR;
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *      Programmed by Zane C. Motteler
 *              June 1988
 *      Purpose: Handle the flow of control upon encountering an ENDFOR in the
 *              input, namely, emit a JMP to the "update code."  Set semerr if
 *              we are in the wrong context for an ENDFOR.
 *       Reprogrammed by Zane C. Motteler in C (January 2000)
 */
void Lrv116(void)
{
    static char msg[] = "lrv116: ENDFOR occurs in the wrong context.";

    if (looptyp[nestlevl - 1] == FOR_LOOP) {
        Lremit(JMP);
        /* label for "update code" sits atop the label stack.  *
         * jump to lstmt, not to NEXT, at end of FOR loop.     */
        Lremit(lbstack[lbstptr - 1] - nextcode + 1);
        lbstptr--;
    } else {
        Remark(msg, strlen(msg));
        semerr = ERR;
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *      Programmed by Zane C. Motteler
 *              June 1988
 *      Purpose: Handle the flow of control upon encountering an ENDWHILE in the
 *              input, namely, emit a JMP to the NEXT address. Set semerr if
 *              we are in the wrong context for an ENDWHILE.
 *       Reprogrammed by Zane C. Motteler in C (January 2000)
 */
void Lrv121(void)
{
    static char msg[] = "lrv121: ENDWHILE occurs in the wrong context.";

    if (looptyp[nestlevl - 1] == WHILE_LOOP) {
        Lremit(JMP);
        Lremit(lpnext[(lpdepth - 1) * 2 + NEXT_TOK - 1] - nextcode + 1);
    } else {
        Remark(msg, strlen(msg));
        semerr = ERR;
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *      Programmed by Zane C. Motteler
 *              September, 1986
 *      Purpose: Create an integer on top of the stack.
 *              This is a compile-time routine, so ic is always pointing
 *              at intcode, where lryact will go to execute.
 *      Revised by Zane C. Motteler
 *              June, 1988
 *      This routine used to be much more complex, when the language
 *              contained event service.
 *      Revised by Zane C. Motteler
 *              August 1999
 *      No longer has a stack pointer argument. lryact builds its
 *      results on the runtime stack.
 *       Reprogrammed by Zane C. Motteler in C (January 2000)
 */
void Lrvint(void)
{
    Integer next3;

    next3 = nextcode - 3;
    ic = intcode;
    Lryact(&next3);
    nextcode = next3;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

Integer Igetic(Integer pc)
{
    return ic[pc - 1];
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Written by Zane C. Motteler
 *            2 February 2000
 * Purpose: Make sure there is enough code space.
 */
void Codechk(void)
{
    char errmsg[] = "execuser: too much code space: a megaword.";

    if (nextcode > mxcode - 20) {
        if (mxcode > 1000000) {
            BasError(errmsg, strlen(errmsg));
            Kaboom(0);
        }
        mxcode += MAXCODE;
        Oschangn((Address *) &intcode, mxcode, mxcode - MAXCODE,
                 "intcode block");
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 * increase the nesting level by 1; store icode in looptyp
 *
 * Revision 7/89 (ZCM) -- no more token array
 * Rewritten in C 2/4/2000 (ZCM)
 */
void Parnest(Integer *icode)
{
    nestlevl += 1;
    if (nestlevl > mxdep) {
        mxdep += MAXDEP;
        Oschangn((Address *) &looptyp, mxdep, mxdep - MAXDEP, "loop type");
        Oschangn((Address *) &loopctl, mxdep, mxdep - MAXDEP,
                 "loop control");
        Oschangn((Address *) &fnfwa, mxdep, mxdep - MAXDEP,
                 "function fwa");
        Oschangn((Address *) &argfwa, mxdep, mxdep - MAXDEP,
                 "argument fwa");
        Oschangn((Address *) &arglen, mxdep, mxdep - MAXDEP,
                 "argument length");
        Oschangn((Address *) &lpnext, 2 * mxdep, 2 * (mxdep - MAXDEP),
                 "loop next");
    }
    looptyp[nestlevl - 1] = *icode;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *      pop the stack of DO controls as appropriate
 *      if we are exiting any DO_DEF loops, release the control variable
 *       Translated into C from exec.m (ZCM 2/4/2000)
 */
void Parunnst(Integer pc)
{
    Integer ic2;
    int i;

    ic2 = ic[pc];
    for (i = nestlevl; i >= nestlevl - ic2 + 1; i--)
        if (looptyp[i - 1] == DO_DEF) {
            Cllit(loopctl[i - 1]);
            rsp -= 1;           /* Get rid of loop control stack element */
        }
    nestlevl -= ic2;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *      Programmed by Zane C. Motteler
 *              July, 1986
 *      Purpose:        given a break or a next to a given level,
 *                      find the appropriate index into lpnext for
 *                      fetching the next or break address
 */
Integer Bnlevel(Integer *outno)
{
    Integer i, toutno, noutno;
    static char msg[] =
        "bnlevel: It is illegal to BREAK out of a function.";

    if (*outno > lpdepth || *outno < 0)
        return 0;               /* Treat it as a NOP */
    else {
        toutno = *outno;
        noutno = *outno;
        /* increase nesting level by 1 for each if */
        for (i = nestlevl; i >= 1; i--) {
            if (looptyp[i - 1] == IF_STRUCT)
                *outno += 1;
            else if (looptyp[i - 1] == FUNCTION_TYPE) {
                BasError(msg, strlen(msg));
                Kaboom(0);
            } else {
                toutno -= 1;
                if (toutno == 0)
                    break;
            }
        }
        return (lpdepth - noutno + 1);
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

void Getpbch(Integer *recur, Integer *lxptr)
{
    int r, i;

    if (nestlevl > 0) {
        r = 0;
        for (i = nestlevl; i >= 1; i--)
            if (looptyp[nestlevl - 1] == PARSTR) {
                r = 1;
                break;
            }
        if (r) {
            *recur = 1;
            Lxgptr(lxptr);
        }
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Check for UNTIL in correct context (ZCM 2/4/2000)
 */

Integer Ckuntil(void)
{
    static char msg[] = "lrvact: UNTIL occurs in incorrect context.";

    if (looptyp[nestlevl - 1] != DO_IND) {
        BasError(msg, strlen(msg));
        return ERR;
    }
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* print remark regarding next source of input. (ZCM 2/4/2000).
 */
void Nextfil(char *fname)
{
    char msg[FILENAMESIZE + 21];
    static char msg1[] = "Resuming input from ";
    int i, r;

    strcpy(msg, msg1);
    r = 0;
    for (i = nestlevl; i >= 1; i--)
        if (looptyp[nestlevl - 1] == PARSTR) {
            r = 1;
            break;
        }
    if (r)
        strcat(msg, "PARSESTR string");
    else
        strcat(msg, fname);
    Remark(msg, strlen(msg));
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

void Decrnest(void)
{
    nestlevl -= 1;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Compute and print the nesting prompt (ZCM 2/4/2000)
 */
Integer Nestpr(void)
{
    char prompt[18];
    int i, iupper, nmac;

    nmac = Mcnum();
    if (nestlevl + nmac > 0) {
        if (nmac == 0)
            strcpy(prompt, ">");
        else
            strcpy(prompt, "}");
        iupper = nestlevl + nmac < 15 ? nestlevl + nmac : 15;
        for (i = 2; i <= iupper; i++)
            strcat(prompt, ">");
        strcat(prompt, " ");
        Osprompt(prompt, strlen(prompt));
        return 1;
    } else
        return (0);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Reinitialize certain variables after error.
 * (Zane C. Motteler 2/16/2000)
 */
void Cparoops(void)
{
    lxsave = (logical) 1;
    lxskip   = 0;
    codebgn  = 1;
    nextcode = codebgn;
    nestlevl = 0;
    xuser    = 0;
    tickflag = (logical) 0;
    tclose   = (logical) 0;
    tickpos  = 0;
    lbstptr  = 0;
    forstptr = 0;
    set1flg  = (logical) 0;
    lpdepth  = 0;
    funcdep  = 0;
    cfuncdep = 0;
    fdepth   = 0;
    firstf   = (logical) 0;
    linsuprs = (logical) 0;
    textpos  = 0;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
