/*
 * dbaux.c
 *
 * Basis Code Development System
 * Copyright 1986 The Regents of the University of California
 *
 * FILE raux.c: Consists of those database routines which do not
 * explicitly access the database. They either indirectly access
 * it via access functions, or they are used by the database routines
 * to do dog work.
 *       Extracted from the old rtd.m.(ZCM 8/20/97)
 *       Reprogrammed as raux.c(ZCM 10/1999)
 *
 */

#include "rt.h"

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* UTCODSTR - convert type code to a string, e.g. INTEGER -> "integer"
 *          -   icode    = integer type code(<0 means character*(-icode))
 */

void Utcodstr(Integer icode, char *utcod)
{
    char lstr[10];

    if (icode < 0) {
        sprintf(lstr, "%ld", (long) -icode);
        strcpy(utcod, "character*");
        strcat(utcod, lstr);
        return;
    }

    switch (icode) {
    case NULL_TYPE:
        strcpy(utcod, "null");
        return;
    case INTEGER:
        strcpy(utcod, "integer");
        return;
    case REAL:
        strcpy(utcod, "real");
        return;
    case COMPLEX:
        strcpy(utcod, "complex");
        return;
    case DOUBLE:
        strcpy(utcod, "double precision");
        return;
    case DOUBLE_C:
        strcpy(utcod, "double complex");
        return;
    case LOGICAL:
        strcpy(utcod, "logical");
        return;
    case NAME:
        strcpy(utcod, "name");
        return;
    case GROUP:
        strcpy(utcod, "group");
        return;
    case LHS:
        strcpy(utcod, "left-hand side");
        return;
    case RANGE:
        strcpy(utcod, "range");
        return;
    case STRUCTURE:
        strcpy(utcod, "structure");
        return;
    case CHAMELEON:
        strcpy(utcod, "chameleon");
        return;
    case FUNCTION_TYPE:
        strcpy(utcod, "function");
        return;
    case FWA:
        strcpy(utcod, "word address");
        return;
    case STRING_ADDRESS:
        strcpy(utcod, "String");
        return;
    case INDIRECT:
        strcpy(utcod, "indirect");
        return;
    case FILE_VAR:
        strcpy(utcod, "variable in file");
        return;
    default:
        strcpy(utcod, "type??");
        return;
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* RTNOFRGN - is this a foreign package?
 *          - it is an error if service requested for a foreign variable
 */

Integer Rtnofrgn(char *service, Integer jvar, Integer ndb)
{
    static char msg[] =
        "Routine called does not work for variables in foreign packages:";

    if (jvar > 0)
        return OK;

    BasError(msg, strlen(msg));
    BasError(service, strlen(service));

    return ERR;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* PARNAME - Return the name corresponding to a stack entry.
 *         - Parname knows about every kind of stack element.
 *         - Function value is length of name(no trailing blanks)
 */

Integer Parname(Integer *si, char *parnam)
{
    Integer nc, ndb, jvar;
    static char msg[] = "parname: name expected, not supplied";

    if (si[TC - 1] == LHS) {
        if (si[N - 1] == UNKNOWN) {
            /* it is still basically NAME */
            StrgGet(si[PTR - 1], parnam);
            nc = Utgetcl(parnam);
        } else if (si[N - 1] == 0) {
            /* it wasn't found and isn't UNKNOWN???   */
            /* I think this should never happen...pfd */
            strcpy(parnam, "?");
            nc = 1;
        } else {
            /* It's been parfetched so it's known */
            ndb = si[N - 1];
            jvar = si[PTR - 1];
            Rtgvname(jvar, ndb, parnam);
            nc = Utgetcl(parnam);
        }
    } else if (si[TC - 1] == NAME || si[TC - 1] < 0) {
        /* we have three cases */
        Parnams(si, parnam, &nc);
    } else if (si[TC - 1] == FUNCTION_TYPE) {
        /* same as a variable */
        if (si[N2 - 1] > 0)
            /* user function */
            ndb = USERDB;
        else
            ndb = si[N - 1];
        jvar = si[PTR - 1];
        Rtgvname(jvar, ndb, parnam);
        nc = Utgetcl(parnam);
    } else if (si[TC - 1] == NULL_TYPE) {
        BasError(msg, strlen(msg));
        strcpy(parnam, "Stack corrupted!");
        nc = strlen(parnam);
    } else if (si[NAMESLOT - 1] == 0) {
        strcpy(parnam, "?????");
        nc = 5;
    } else {
        Rtcrack(si[NAMESLOT - 1], &jvar, &ndb);
        Rtgvname(jvar, ndb, parnam);
        nc = Utgetcl(parnam);
    }

    return (nc);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* PARFIND - find a variable
 *
 * input: package number npac, or zero to search current stack
 *        variable name
 * output : ndb number of database
 *          jvar nonzero if name is a variable
 *          jvar 0(and function returns ERR) if not found
 *          parfind = OK if found
 * performs recursive search if variable found is of type indirect
 *
 */

Integer Parfind(Integer npack, char *name, Integer *jvar, Integer *ndb,
                Integer *tc, Integer kslot)
{
    char ename[VARNAMESIZE + 1], vname[VARNAMESIZE + 1], msg[80];
    int lname;
    Address fwa;
    Integer ndim, dum[7];

    *ndb = npack;

    lname = strlen(name);
    strcpy(ename, name);
    strcpy(vname, name);

    *jvar = Rtlookup(ename, ndb, tc, kslot);
    if (*jvar == 0) {
        return (ERR);
    }

    if (*tc == INDIRECT) {
        Rtxdb(*jvar, *ndb, &fwa, &ndim, dum, dum, dum, LIMITED);

        strncpy(vname, (char *) fwa, VARNAMESIZE - 1);
        vname[VARNAMESIZE - 1] = '\0';
        if (!strcmp(ename, vname)) {
            sprintf(msg, "parfind: %s  is indirect but points to itself!",
                    name);
            Kaboom(0);
        }

        return Rtfinder(vname, jvar, ndb, tc, "indirect reference");
    }
    return (OK);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* PARFINDG - find a group
 * input: package number npac, or zero to search current stack
 *       groupname name
 * output : db filled up with database of package
 *          jvar positive if name is a group
 *          ndb set to package in which name found
 *          jvar 0(and function returns ERR) if not found
 *          parfindg = OK if found
 * Returns exact match; if none found tries for partial match
 */

Integer Parfindg(Integer npack, char *name, Integer *jvar, Integer *ndb)
{

    *ndb = npack;
    *jvar = Rtlookg(name, ndb);

    return *jvar != 0 ? OK : ERR;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* PARFINDS - find a name
 * find the NAME described by stack element si
 * same as parfind with si replacing npac and name
 * works by calling parfind
 */

Integer Parfinds(Integer *si, Integer *jvar, Integer *ndb, Integer *tc,
                 Integer *nc, char *name, Integer lname)
{
    Integer npac, rval;
    char vname[256];

    npac = si[N - 1];

/* If *nc == 0, name is to be filled in; lname is amount of
 * space available in name
 */
    if (*nc == 0) {
        INIT_BLANK_STRING(vname, lname + 1);
        *nc = Parname(si, vname);
    } else {
        memcpy(vname, name, *nc);
        vname[*nc] = '\0';
    }

/* si(N1) contains string table index in NAME stack elements */
    rval = Parfind(npac, vname, jvar, ndb, tc, si[N1 - 1]);

/* copy and blank fill the name to return it to FORTRAN */
    memcpy(name, vname, *nc);

    if (lname > *nc)
        memset(name + (*nc), (int) ' ', lname - (*nc));

    return rval;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* RTSPLIT - split name into components
 * split name into package number and name
 * name might be of the form name.name, .name , ..name or name
 * returns OK or ERR if name badly formed
 * .name means search just the locals
 * ..name means search just the globals
 */

static Integer Rtsplit(char *name, Integer *npackage, char *vname)
{
    char *idot;
    int j, leng, dotpos;
    char pkg[PNSIZE + 1];

/* Get trailing blanks out of name. */
    for (j = strlen(name) - 1; j > 0; j--)
        if (name[j] != ' ')
            break;
    name[j + 1] = '\0';

    idot = strchr(name, '.');
    if (idot == (char *) NULL) {
        *npackage = 0;
        strcpy(vname, name);
        return (OK);
    }

    dotpos = idot - name;
    leng = strlen(name);
    j = strspn(name, " ");
    if (dotpos == leng - 1 || j > leng) {
        *npackage = 0;
        strcpy(vname, name);
        return (OK);
    }

    if (dotpos == j) {
        *npackage = LOCALDB;
        if (idot[1] == '.') {   /* safe to do since dotpos < leng */
            idot += 1;
            dotpos += 1;
            *npackage = USERDB;
        }
    } else {
        strncpy(pkg, name + j, dotpos - j);
        pkg[dotpos - j] = '\0';
        *npackage = Glbpknux(pkg);
        if (*npackage == 0) {
            strcpy(vname, name);
            return (OK);
        }
    }

    idot += 1;
    j = strspn(idot, " ");
    strcpy(vname, idot + j);

    return (OK);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* RTFINDER - To find jvar, ndb, and type corresponding to name.
 *          - Returns OK if name makes sense, ERR otherwise
 *          - IF OK returned, sets jvar, db, type
 *          - If name not found, jvar = 0(silently unless complain .ne. " ")
 *          - and returns ERR.  complain otherwise is name of the caller.
 */

Integer Rtfinder(char *name, Integer *jvar, Integer *ndb, Integer *type,
                 char *complain)
{
    Integer npackage, ierr;
    char vname[VARNAMESIZE + 1], cbell, msg[80];

    ierr = Rtsplit(name, &npackage, vname);
    if (ierr != ERR) {
        if (Parfind(npackage, vname, jvar, ndb, type, -1) == ERR) {
            if (Parfindg(npackage, vname, jvar, ndb) == OK) {
                *type = GROUP;
                return OK;
            }
        } else
            return OK;
    }

/* bad name or package */
    if (ierr == ERR) {
        cbell = bell == 1 ? (char) 7 : ' ';
        sprintf(msg, "%crtfinder: badly formed name or unknown package:",
                cbell);
        BasError(msg, strlen(msg));
        BasError(name, strlen(name));
    }

/* unknown name */
    if (strncmp(complain, " ", 1) && complain[0] != '\0') {
        cbell = bell == 1 ? (char) 7 : ' ';
        sprintf(msg, "%cUnknown name: %s.", cbell, name);
        BasError(msg, strlen(msg));
        sprintf(msg, "Called from routine named %s.", complain);
        BasError(msg, strlen(msg));
    }

    return ERR;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* RTZAPALL - delete user database(implements 'forget')
 *
 *       Reprogrammed by Paul Dubois, July 1988
 *       Purpose: Delete everything in the user database
 *       Moved here and renamed 7/91
 *       changes in varnam stuff 7/91
 *       Reprogrammed in C by Zane C. Motteler, 10/05/1999
 */

void Rtzapall()
{
    int n;
    Integer jvar;

    n = Rtgnvar(USERDB);
    for (jvar = n; jvar >= 1; jvar--)
        if (!Rtisdead(jvar, USERDB))
            Rtforget(jvar, USERDB);

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CSETLAST - sets a limit on the highest dimension's subscript */

Integer Csetlast(char *name, Integer irec)
{
    Integer jvar, tc, ndb;
    static char msg[] =
        "setlast: cannot call with a group or function name.";

    if (Rtfinder(name, &jvar, &ndb, &tc, "setlast") == ERR)
        return ERR;

    if (tc == GROUP || tc == FUNCTION_TYPE) {
        BasError(msg, strlen(msg));
        BasError(name, strlen(name));
        return ERR;
    }

    /* this service not available to foreigners */
    if (Rtnofrgn("setlast", jvar, ndb) == ERR) {
        BasError(name, strlen(name));
        return ERR;
    }

    if (Rtsetdl(jvar, ndb, irec) == ERR) {
        BasError(name, strlen(name));
        return ERR;
    }

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* RTINNAME - insert a name into the database without reserving space
 *          - It had to be factored out to handle the case of target
 *          - variables which inherit properties of their sources
 *          - (autovar flag on). A later call to parendow reserves
 *          - the space for these variables.
 *
 * stack element format: svar(Ni) is to be the starting subscript, and
 * svar(Mi) is to be the upper subscript
 * svar(NS) number of subscripts
 */

Integer Rtinname(Integer ndb, Integer tc, Integer *svari)
{
    Integer ndim, lcomma, jvar, idum, i, inamlm1, nc;
    Address iptr;
    char vname[VARNAMESIZE + 1];
    char dimsto[DIMSTRSIZE + 1];
    char ilow[16], ihi[16];
    char typstr[TYPESIZE + 1];

    inamlm1 = NAML - 1;
    nc = abs(svari[inamlm1]);

/* this is an index to a negative characte length */

/* 1. Get name
 *    Blank out the receptor first
 */
    INIT_BLANK_STRING(vname, nc + 1);

    idum = Parname(svari, vname);

/* 2.  Pack up dimsto to describe desired dimensioning */
    ndim = svari[NS - 1];
    dimsto[0] = '(';
    dimsto[1] = '\0';
    lcomma = 0;                 /* trick to prevent leading comma in string */

    for (i = 0; i < ndim; i++) {
        if (lcomma != 0)
            strcat(dimsto, ",");
        lcomma = 1;
        sprintf(ihi, "%ld", (long) svari[M1 - 1 + i]);

        if (svari[N1 - 1 + i] == 1)
            strcat(dimsto, ihi);
        else {
            sprintf(ilow, "%ld", (long) svari[N1 - 1 + i]);
            strcat(dimsto, ilow);
            strcat(dimsto, ":");
            strcat(dimsto, ihi);
        }
    }
    strcat(dimsto, ")");

/* dimsto now contains something like(4:7,3,...)
 * fix for scalar
 */
    if (ndim == 0)
        strcpy(dimsto, "scalar");

/* 3. create variable as dynamic with type tc
 * this version does not set units or comment field
 * first get one word to be the pointer
 */
    idum = sizeof(Address) / sizeof(Integer);
    Osallotn(&iptr, idum, "RTINNAME:iptr");

/* next add the variable to the variable database */
    Utcodstr(tc, typstr);
    jvar = Rtvare(ndb, vname, iptr, 2, typstr, dimsto, " ");

    return jvar;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CEXISTS - does a variable exist?
 * return 1/0 depending on whether or not a name exists
 * Called by the old FORTRAN 'exists' function.
 */

Integer Cexists(char *name)
{
    Integer jvar, ndb, tc;

    if (Rtfinder(name, &jvar, &ndb, &tc, " ") == ERR)
        return 0;
    else if (jvar == 0)
        return 0;
    else
        return 1;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CPROTECT - protect all user variables
 *          - gives all current User variables the attribute readonly
 */

void Cprotect()
{
    Integer nvar, jvar;

    nvar = Rtgnvar(USERDB);
    for (jvar = 1; jvar <= nvar; jvar++)
        Rtvattre(jvar, USERDB, "readonly");

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CSETLIMIT - set limits of a variable */

Integer Csetlimit(char *name, char *limits)
{
    Integer jvar, ndb, tc;
    static char msg1[] =
        "setlimit: cannot call with a group or function name.", msg2[] =
        "Limiting string too long, ignored.";

    if (Rtfinder(name, &jvar, &ndb, &tc, "setlimit") == ERR)
        return ERR;

    if (tc == GROUP || tc == FUNCTION_TYPE) {
        BasError(msg1, strlen(msg1));
        BasError(name, strlen(name));
        return ERR;
    }

/* this service not available to foreigners */
    if (Rtnofrgn("setlimit", jvar, ndb) == ERR) {
        BasError(name, strlen(name));
        return ERR;
    }

    if (strlen(limits) > DIMSTRSIZE) {
        BasError(msg2, strlen(msg2));
        return ERR;
    }

    Rtslimst(jvar, ndb, StrgFind(limits));

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CSETACT - set assignment action string of a variable */

Integer Csetact(char *name, char *action)
{
    Integer jvar, ndb, tc;
    static char msg1[] =
        "setact: cannot call with a group or function name.", msg2[] =
        "Assignment-action string too long, ignored.";

    if (Rtfinder(name, &jvar, &ndb, &tc, "setact") == ERR)
        return ERR;

    if (tc == GROUP || tc == FUNCTION_TYPE) {
        BasError(msg1, strlen(msg1));
        BasError(name, strlen(name));
        return ERR;
    }

/* this service not available to foreigners */
    if (Rtnofrgn("setact", jvar, ndb) == ERR) {
        BasError(name, strlen(name));
        return ERR;
    }

    if (strlen(action) > DIMSTRSIZE) {
        BasError(msg2, strlen(msg2));
        return ERR;
    }

    Rtsnastr(jvar, ndb, StrgFind(action));

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CCOMMENT - set comment to variable or group
 * ---Comment adds a comment to a variable or function or group in a database.
 * -- name can be of the form pkg.name
 */

void Ccomment(char *name, char *comm)
{
    Integer jvar, ndb, tc;

    if (Rtfinder(name, &jvar, &ndb, &tc, "comment") == ERR)
        return;

/* this service not available to foreigners */
    if (Rtnofrgn("comment", jvar, ndb) == ERR) {
        BasError(name, strlen(name));
        return;
    }

    if (tc == GROUP)
        Rtcomeg(jvar, ndb, comm);
    else
        Rtcomev(jvar, ndb, comm);

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* PARFINDA - find up to ten occurrences of 'name' in the db
 *        Tries to find all occurrences of variables named 'name',
 *        up to a maximum of ten. It returns the values in the
 *        arrays jvar, ndb, tc, pri sorted by priority.
 *        Written by ZCM August 1997, rewritten in C Oct. 1999.
 */

static Integer Parfinda(char *name, Integer *jvar, Integer *ndb,
                        Integer *tc, Integer *pri, Integer *no)
{
    char vname[VARNAMESIZE + 1], msg[80];
    Integer i, j, tmp, dum[7], ndim, exchange;
    Address fwa;

    Rtlooka(name, jvar, ndb, tc, pri, no);
    if (*no == 0) {
        return (ERR);
    }

/* Check for indirects */
    for (i = 0; i < *no; i++) {
        if (tc[i] == INDIRECT) {
            Rtxdb(jvar[i], ndb[i], &fwa, &ndim, dum, dum, dum, LIMITED);
            strncpy(vname, (char *) fwa, VARNAMESIZE - 1);
            /* There may be trailing blanks, so remove them */
            for (j = VARNAMESIZE - 2; j >= 0; j--)
                if (vname[j] != ' ') {
                    vname[j + 1] = '\0';
                    break;
                }
            if (!strcmp(name, vname)) {
                sprintf(msg,
                        "parfinda: '%s' is indirect but points to itself!",
                        vname);
                BasError(msg, strlen(msg));
                Kaboom(0);
            }
            if (Rtfinder(vname, &(jvar[i]), &(ndb[i]), &(tc[i]),
                         "indirect reference") == ERR)
                return ERR;
        }
    }

    if (*no > 1) {
        /* sort by priorities(bubble sort, highest first) */
        /* bubble sort is fast enough for ten items or less */
        for (i = *no - 1; i >= 1; i--) {
            exchange = 0;
            for (j = 0; j <= *no - 2; j++) {
                if (pri[j] < pri[j + 1]) {
                    exchange = 1;
                    tmp = pri[j];
                    pri[j] = pri[j + 1];
                    pri[j + 1] = tmp;
                    tmp = jvar[j];
                    jvar[j] = jvar[j + 1];
                    jvar[j + 1] = tmp;
                    tmp = tc[j];
                    tc[j] = tc[j + 1];
                    tc[j + 1] = tmp;
                    tmp = ndb[j];
                    ndb[j] = ndb[j + 1];
                    ndb[j + 1] = tmp;
                }
            }
            if (!exchange)
                break;
        }
    }

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* RTFINDA - find more or less all variables named 'name'
 *         - To find jvar, ndb, type, and pri corresponding to
 *     the first ten or fewer occurrences of name in the database.
 *     Returns OK if name makes sense, ERR otherwise.
 *     IF OK returned, sets jvar, db, type, pri arrays and no
 *    (the number of entries).
 *     If name not found, no and jvar(0) == 0 (silently unless complain<>" ")
 *     and returns ERR.  complain otherwise is name of the caller.
 */

Integer Rtfinda(char *name, Integer *jvar, Integer *ndb, Integer *type,
                Integer *pri, Integer *no, char *complain)
{
    Integer ierr, npackage;
    char vname[VARNAMESIZE + 1];

/* split name into package number and name */
    if (Rtsplit(name, &npackage, vname) == OK && npackage == 0) {
        if (Parfinda(vname, jvar, ndb, type, pri, no) == ERR) {
            if (Parfindg(npackage, vname, jvar, ndb) == OK) {
                type[0] = GROUP;
                *no = 1;
                return OK;
            }
        } else
            return OK;
    }

/* we end up here if we need to call rtfinder */
    ierr = Rtfinder(name, jvar, ndb, type, complain);
    *no = ierr == ERR ? 0 : 1;

    return ierr;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* RTGETVAR - look up a variable and extract its properties */

Integer Rtgetvar(char *vname, Address *fwa, Integer *tc, Integer *ndim,
                 Integer *ilow, Integer *ihi)
{
    Integer icol[7];
    Integer jvar, ndb;
    static char msg[] = "ERROR rtgetvar:  name is a function or group";

/* look up variable */
    if (Rtfinder(vname, &jvar, &ndb, tc, "rtgetvar") == ERR)
        return ERR;

    if (*tc == FUNCTION_TYPE || *tc == GROUP) {
        BasError(msg, strlen(msg));
        BasError(vname, strlen(vname));
        return ERR;
    }

/* .. extract properties
 * don't know what the access will be assume the read of now
 * later rtgetvar should have an access argument, I think...PFD
 */
    Rtxdb(jvar, ndb, fwa, ndim, ilow, ihi, icol, LIMITED);

    if (*tc == NULL_TYPE)
        *ndim = 0;

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* RTGETANY - */

Integer Rtgetany(char *name, char *package, Integer rwaccess,
                 Integer *jvar, Integer *ndb, Address *fwa, Integer *tc,
                 Integer *ndim, Integer *ilow, Integer *ihi, Integer *icol,
                 logical *varflg)
{

    char vname[VARNAMESIZE + 1], name2[VARNAMESIZE + 1], pkg[PNSIZE + 1];
    char evalstr[MAXSTRING];
    char *pevalstr = evalstr;   /* needed to pass a static char array to Fortran */
    Integer leval, jentry, npack;

    *varflg = (logical) 1;
    strcpy(pkg, package);
    strcpy(name2, name);        /* same as name.  able to use in concat operation. */

    jentry = StrgFind(name);

    if (IhashFind(mactable, jentry) > 0) {

/* macro */
        sprintf(evalstr, "{%s chameleon %s = %s()}", pkg, vname, name2);
        leval = strlen(evalstr);
        Parsestr((Address *) &pevalstr, &leval);

        npack = Glbpknum(pkg);

/* The new -1 argument tells parfind to actually get the name */
        if (Parfind(npack, name2, jvar, ndb, tc, -1) == ERR)
            return ERR;

        *varflg = (logical) 0;
    } else {
        if (Rtsplit(name, ndb, vname) == ERR)
            return ERR;

        if (Rtfinder(name, jvar, ndb, tc, " ") == ERR)
            return ERR;

        if (*tc == FUNCTION_TYPE) {
            sprintf(evalstr, "{%s chameleon %s = %s()}", pkg, vname,
                    name2);
            leval = strlen(evalstr);
            Parsestr((Address *) &pevalstr, &leval);

            npack = Glbpknum(pkg);

/* The new -1 argument tells parfind to actually get the name */
            if (Parfind(npack, name2, jvar, ndb, tc, -1) == ERR)
                return ERR;
            *varflg = (logical) 0;
        }
    }

    Rtxdb(*jvar, *ndb, fwa, ndim, ilow, ihi, icol, rwaccess);

    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* RTISMAC - see if name is a macro */

void Rtismac(char *name, char *newname, Integer *macindx)
{
    Integer firstlet, lastlet, jentry, lstr;

    firstlet = strspn(name, " ");
    for (lastlet = strlen(name) - 1; lastlet >= firstlet; lastlet--)
        if (name[lastlet] != ' ') {
            break;
        }
    if (lastlet < firstlet) {
        lastlet = firstlet;
    }
    name[lastlet + 1] = '\0';

    *macindx = 0;

/* First check if it is a macro, unless name is enclosed in {}'s. */
    strcpy(newname, name + firstlet);
    if (name[firstlet] == '{' && name[lastlet] == '}') {
        lstr = lastlet - firstlet - 1;
        strncpy(newname, name + firstlet + 1, lstr);
        newname[lstr] = '\0';
    } else {
        /* check if it's a macro */
        jentry = StrgFind(name);
        if (IhashFind(mactable, jentry) > 0)
            *macindx = jentry;
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
