/*
 * dbg.c
 *
 * Basis Code Development System
 * This file contains what used to be in dbg.m--a complete debugging
 * suite for Basis.
 * Copyright 1986 The Regents of the University of California
 */

#include "rt.h"

Integer jout;

char
    laststr[101],
    parnam[VARNAMESIZE + 1];

extern void
    F77_ID(prttrc_, prttrc, PRTTRC) (Integer *unit, Integer *tc,
                                     Integer *ne, Address *ad);

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* LROPCODE - converts number numbers to mnemonics */

char *Lropcode(number)
{
    switch (number) {
    case 1:
        return "ID->LHS";       /* <factor> ::= <lhs> */
    case 2:
        return "LHS(args)";     /* <lhs> ::= <lhs> <args> */
    case 3:
        return "ASSIGN";
    case 4:
        return "DISPLAY GRP";
    case 5:
        return "DISPLAY EXPR";
    case 6:
        return "CALL F";        /* call <lhs> <args> */
    case 7:
        return "F COMMAND ARGS";
    case 8:
        return "FETCH VARIABLE";
    case 9:
        return "&LHS";
    case 10:
        return "BEGIN RHS";

/* named numbers that need to be coordinated with the BNF file */
    case FACLIT:
        return "CREATE LITERAL";        /* 11 */

    case SET1:
        return "SET1";
    case NEST:
        return "NEST";
    case UNNEST:
        return "UNNEST";
    case LOOPSTRT:
        return "LOOPSTRT";
    case FCNENTER:
        return "FUNCTION";
    case RET:
        return "RET";
    case NULLRET:
        return "RET NULL";
    case 19:
        return "RET VALUE";
    case 101:
        return "EXPLIST";
    case 102:
        return "OR";
    case 103:
        return "AND";
    case 104:
        return "NOT";
    case 105:
        return "LESS";
    case 106:
        return "LESSEQ";
    case 107:
        return "GRTR";
    case 108:
        return "GRTREQ";
    case 109:
        return "EQUAL";
    case 110:
        return "NOTEQ";
    case 115:
        return "ADD";
    case 116:
        return "SUBTRACT";
    case 117:
        return "PLUS";
    case 118:
        return "MINUS";
    case 119:
        return "TIMES";
    case 120:
        return "STARBANG";
    case 121:
        return "DIVIDE";
    case 122:
        return "DOT_PROD";
    case 123:
        return "POP STACK";
    case 124:
        return "CONCAT";
    case 125:
        return "SLSHBANG";
    case 126:
        return "POWER";

/* 127,128 void */

    case 129:
        return "[EXPLIST]";
    case 130:
        return "( LEXP ) -> ATOM";
    case 131:
        return "REDUCE ` <rterm> `";
    case 132:
        return "STACK FRAME(FCN)";
    case 133:
        return "STACK FRAME(CMD)";
    case 134:
        return "<RANGE>->X:Y";
    case 135:
        return "PUSH NULL";
    case 136:
        return "<LHS>-><FACTOR>";
    case 138:
        return "<RANGE>->X:Y:Z";
    case 143:
        return "name.name -> <ID>";
    case 144:
        return "ERROR";
    case 145:
        return "ERROR";
    case LINE:
        return "LINE_NO";
    case 230:
        return "LIST";
    case 234:
        return "LIST_ID";
    case 235:
        return "LIST_GRP";
    case 369:
        return "INP_TERM";
    case 370:
        return "OUT_LINE";
    case 371:
        return "ASSIGNIO";
    case 372:
        return "TO_TTY";
    case 374:
        return "OUTPUT";
    case 375:
        return "NEW LINE";
    case 376:
        return "INPUT";
    case 377:
        return "INPUT";
    case 378:
        return "SKIPLINE";
    case 379:
        return "SKIPLINE";
    case 380:
        return "END_INP";
    case 400:
        return "DECLARE INTEGER";
    case 401:
        return "DECLARE REAL";
    case 402:
        return "DECLARE LOGICAL";
    case 403:
        return "DECLARE COMPLEX";
    case 404:
        return "DECLARE CHAMELEON";
    case 405:
        return "DECLARE CHARACTER*N";
    case 406:
        return "DECLARE CHARACTER";
    case 407:
        return "DECLARE INDIRECT";
    case 408:
        return "DECLARE RANGE";
    case 409:
        return "DECLARE DOUBLE";
    case 422:
        return "DECLARE REAL8";
    case 423:
        return "DECLARE COMPLEX8";
    case 410:
    case 411:
        return "CREATE VARIABLE";
    case 412:
        return "SCALAR DECLARE";
    case 413:
        return "ARRAY DECLARE";
    case 414:
        return "INITIAL VALUE";
    case 415:
        return "NO INITIAL VALUE";
    case 416:
        return "DECL DOUBLE COMPLEX";
    case 417:
        return "POP SCOPE & TYPE";
    case 420:
        return "DECLARE SCOPE";
    case 421:
        return "DEFAULT_SCOPE";
    case 600:
        return "LOOP INCREMENT";
    case 602:
        return "SAVE_GRP";
    case 603:
        return "FORGET";
    case 604:
        return "FORGET ID";
    case 605:
        return "READ";
    case 606:
        return "LOOP START VAL";
    case 607:
        return "LOOP END VAL";
    case 608:
        return "APPEND";
    case 701:
        return "OR EQUAL";
    case 702:
        return "AND EQUAL";
    case 703:
        return "PLUS EQUAL";
    case 704:
        return "MINUS EQUAL";
    case 705:
        return "TIMES EQUAL";
    case 706:
        return "DIVIDE EQUAL";
    case 707:
        return "POWER EQUAL";
    default:
        return "ERROR";
    }

    return (NULL);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* LRXPRT - print out a standard one-word intermediate instruction.
 *        - Only the action code pointer is given, except for
 *        - the SET1 instruction, which will dynamically set the do
 *        - loop control statement immediately ahead.
 */

void Lrxprt(Integer unit, Integer pc, Integer rsp1, Integer rsp2,
            Integer rlinkp1, Integer rlinkp2)
{
    char msg[MAXOUT + 1];       /* to store output text */
    Integer ic1;

    ic1 = ic[pc - 1];
    if (ic1 == FCNENTER)
        sprintf(msg,
                "%5ld:%7ld   Enter function, set up actual parameters.",
                (long) pc, (long) ic1);
    else if (ic1 == SET1)
        sprintf(msg, "%5ld:%7ld   Will set do controls at = %5ld.",
                (long) pc, (long) ic1, (long) (pc + 1));
    else if (ic1 == LINE || ic1 == NEST || ic1 == UNNEST)
        sprintf(msg, "%5ld:%7ld   %s %5ld.",
                (long) pc, (long) ic1, Lropcode(ic1), (long) ic[pc]);
    else
        sprintf(msg, "%5ld:%7ld   %s.",
                (long) pc, (long) ic1, Lropcode(ic1));
    Baswline(&unit, msg, strlen(msg));
    if (!partrace)
        return;

    sprintf(msg, "                rsp before %4ld, rsp after %4ld.",
            (long) rsp1, (long) rsp2);
    Baswline(&unit, msg, strlen(msg));
    if (ic1 == 132 || ic1 == 133 || ic1 == 2 || ic1 == 7 ||
        (ic1 == 136 && rlinkp1 != rlinkp2)) {

/* Set up stack frame--these alter rlinkp. */
        if (rlinkp1 != 0)
            sprintf(msg,
                    "                rlinkp is now %ld, pointing at %ld.",
                    (long) rlinkp1, (long) rlinkp2);
        else
            sprintf(msg, "                rlinkp is now %ld.",
                    (long) rlinkp1);
        Baswline(&unit, msg, strlen(msg));
    }

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* LRVPRT - Print out informative comments about terminal actions
 *        -(primarily those which build stack elements)
 */

void Lrvprt(Integer unit, Integer pc, Integer rsp1, Integer rsp2)
{
    Integer ic1, action;
    char keyword[32], *toktext, msg[MAXOUT + 1];
    Integer ival;
    float rval;
    real8 dval;

    ic1 = ic[pc - 1];
    action = -ic1;
    toktext = FccMakeString(NULL, MAXSTRING, 1);
    StrgGet(ic[pc + 1], toktext);
    toktext[StrgLen(ic[pc + 1])] = '\0';
    switch (action) {
    case 100:
        strcpy(keyword, "name");
        sprintf(msg, "%5ld:%7ld   PUSH TOKEN '%s'(%s).",
                (long) pc, (long) ic1, toktext, keyword);
        break;

    case 101:
        strcpy(keyword, "integer constant");
        ival = values[ic[pc] - 1];
        sprintf(msg, "%5ld:%7ld   PUSH TOKEN '%s'(%s). value = %ld.",
                (long) pc, (long) ic1, toktext, keyword, (long) ival);
        break;

    case 102:
        strcpy(keyword, "real constant");
        rval = ((float *) values)[ic[pc] - 1];
        sprintf(msg, "%5ld:%7ld   PUSH TOKEN '%s'(%s). value = %7e.",
                (long) pc, (long) ic1, toktext, keyword, (double) rval);
        break;

    case 103:
        strcpy(keyword, "complex constant");

#  ifdef HAS_DOUBLE_COMPLEX
        dval = dvalues[ic[pc] - 1];
        sprintf(msg, "%5ld:%7ld   PUSH TOKEN '%s'(%s). value = %7e.",
                (long) pc, (long) ic1, toktext, keyword, dval);
#  else
        rval = ((float *) values)[ic[pc] - 1];
        sprintf(msg, "%5ld:%7ld   PUSH TOKEN '%s'(%s). value = %7e.", pc,
                ic1, toktext, keyword, (double) rval);
#  endif
        break;

    case 104:
        strcpy(keyword, "double constant");
        dval = dvalues[ic[pc] - 1];
        sprintf(msg, "%5ld:%7ld   PUSH TOKEN '%s'(%s). value = %7e.",
                (long) pc, (long) ic1, toktext, keyword, dval);
        break;

    case 105:
        strcpy(keyword, "Group name");
        sprintf(msg, "%5ld:%7ld   PUSH TOKEN '%s'(%s).",
                (long) pc, (long) ic1, toktext, keyword);
        break;

    case 106:
        strcpy(keyword, "string");
        sprintf(msg, "%5ld:%7ld   PUSH TOKEN '%s'(%s).",
                (long) pc, (long) ic1, toktext, keyword);
        break;

    default:
        strcpy(keyword, "oops");
    }

    Baswline(&unit, msg, strlen(msg));
    if (partrace) {
        sprintf(msg, "                rsp before %4ld, rsp after %4ld.",
                (long) rsp1, (long) rsp2);
        Baswline(&unit, msg, strlen(msg));
    }

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* LRDOPRT - Print explanatory information about a loop control action
 *         - about to take place.
 */

void Lrdoprt(Integer unit, Integer pc, Integer rsp1, Integer rsp2)
{
    Integer ic1, ic2, jump;
    char errmsg[MAXLINE + 1];
    Address value;
    Integer ival;

    ic1 = ic[pc - 1];
    ic2 = ic[pc];
    if (ic2 > 1000000) {
        jump = ic2 - 1000000;
        sprintf(errmsg,
                "%5ld:%7ld          DO loop, DOTEST to be backpatched at %5ld",
                (long) pc, (long) ic1, (long) jump);
        Baswline(&unit, errmsg, strlen(errmsg));
    } else {
        jump = pc + ic2;        /* convert from relative */
        sprintf(errmsg, "%5ld:%7ld   DO loop, DOTEST instruction to %5ld.",
                (long) pc, (long) ic1, (long) jump);
        Baswline(&unit, errmsg, strlen(errmsg));
    }

/* we are executing, so values have been filled in */
    if (execflag) {
        ival = loopctl[nestlevl - 1];
        value = Getaddr(ival);
        ival = *((Integer *) value);
        sprintf(errmsg,
                "                       DO variable location is %p; value is %8ld.",
                value, (long) ival);
        Baswline(&unit, errmsg, strlen(errmsg));
        sprintf(errmsg,
                "                       DO loop limit is           %8ld.",
                (long) ssl(N1, rsp));
        Baswline(&unit, errmsg, strlen(errmsg));
        sprintf(errmsg,
                "                       DO loop increment is    %8ld.",
                (long) ssl(M1, rsp));
        Baswline(&unit, errmsg, strlen(errmsg));
        sprintf(errmsg,
                "                       rsp before %4ld, rsp after %4ld.",
                (long) rsp1, (long) rsp2);
        Baswline(&unit, errmsg, strlen(errmsg));
    } else {
/* we are not executing, so don't give spurious limits */
        sprintf(errmsg,
                "                       --controls not initialized, not executing at the moment.");
        Baswline(&unit, errmsg, strlen(errmsg));
    }

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* LRFLPRT - Print informative trace of flow of control instructions. */

void Lrflprt(Integer unit, Integer pc, Integer *incr, Integer rsp1,
             Integer rsp2, Integer rlinkp1, Integer rlinkp2)
{
    Integer j, ic1;
    char jump[8], msg[MAXOUT + 1];

    ic1 = ic[pc - 1];
    switch (ic1) {
    case JMP:
        strcpy(jump, "JMP");
        *incr = 2;
        break;
    case INT:
        strcpy(jump, "INT");
        *incr = 2;
        break;
    case JF:
        strcpy(jump, "JF ");
        *incr = 2;
        break;
    case DOTEST:
        Lrdoprt(unit, pc, rsp1, rsp2);
        *incr = 2;
        return;
    case RET:
        strcpy(jump, "RET");
        *incr = 1;
        break;
    case NULLRET:
        strcpy(jump, "NULLRET");
        *incr = 1;
        break;
    default:
        strcpy(jump, "OOP");
        *incr = 2;
        break;
    }

    j = ic[pc];
    if (j < 1000000)
        j += pc;
    else if (j > 1000000)
        j = 1000000 - j;

    if (ic1 == INT)
        j += 2;

    if (ic1 != JF && ic1 != RET)
        sprintf(msg, "%5ld:%7ld   %s to location %5ld.",
                (long) pc, (long) ic1, jump, (long) j);
    else if (ic1 == JF)
        sprintf(msg, "%5ld:%7ld   %s to location %5ld.",
                (long) pc, (long) ic1, jump, (long) j);
    else
        sprintf(msg, "%5ld:%7ld   %s(return from current function)",
                (long) pc, (long) ic1, jump);

    Baswline(&unit, msg, strlen(msg));
    if (!partrace)
        return;

    sprintf(msg, "                rsp before %4ld, rsp after %4ld.",
            (long) rsp1, (long) rsp2);
    Baswline(&unit, msg, strlen(msg));

    if (ic1 == RET) {
        if (rlinkp1 != 0)
            sprintf(msg,
                    "                rlinkp is now %ld, pointing at %ld.",
                    (long) rlinkp1, (long) rlinkp2);
        else
            sprintf(msg, "                rlinkp is now %ld.",
                    (long) rlinkp1);
        Baswline(&unit, msg, strlen(msg));
    }

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* LPTRACE - To describe depth and type of nesting as an aid
 *         - in tracking down semantic errors.
 */

void Lptrace(Integer pc)
{
    Integer ifuncdep, iveseen, i, idum, j;
    logical infunc;
    char msg2[OUTLINE + 1];
    static char
        msg1[] = "Here is the information I have on where you were:",
        msg3[] = "the problem(maybe) or call to whereami.",
        msg4[] = "The most recent call to parsestr was on this string:";

    if (!execflag)
        return;

    ifuncdep = 0;
    Baswline(&jout, msg1, strlen(msg1));
    iveseen = 0;
    infunc = (logical) 0;
    for (i = 1; i <= nestlevl; i++) {
        switch (looptyp[i - 1]) {
        case IF_STRUCT:
            strcpy(msg2, "IF statement containing");
            break;
        case WHILE_LOOP:
            strcpy(msg2, "WHILE statement containing");
            break;
        case FOR_LOOP:
            strcpy(msg2, "FOR statement containing");
            break;
        case DO_IND:
        case DO_DEF:
            strcpy(msg2, "DO statement containing");
            break;
        case FUNCTION_TYPE:
            infunc = (logical) 1;
            strcpy(parnam, "?");
            idum = 1;
            for (j = ifuncdep; j <= funcdep; j++)
                if (Fstkgeti(SPFN, j) > 0) {
                    INIT_BLANK_STRING(parnam, VARNAMESIZE);

                    idum = Parname(&(ssl(1, Fstkgeti(SPFN, j))), parnam);
                    parnam[idum] = '\0';
                    break;
                }
            ifuncdep = j + 1;
            sprintf(msg2, "A call to %s  containing", parnam);
            break;
        case EXECUSER:
            strcpy(msg2, "A call to execuser executing ");
            break;
        case PARSTR:
            iveseen = 1;
            strcpy(msg2, "A call to parsestr(recursive parsing)");
            break;
        default:
            strcpy(msg2, "--Uh, oh, unknown structure.");
        }
        Baswline(&jout, msg2, strlen(msg2));
    }

    if (nestlevl > 0)
        Baswline(&jout, msg3, strlen(msg3));

    if (infunc) {
        sprintf(msg2,
                "The trouble is on or near line number %ld of function %s.",
                (long) Fstkgeti(LINENO, funcdep), parnam);
        Baswline(&jout, msg2, strlen(msg2));
    }
    if (iveseen > 0) {
        Baswline(&jout, msg4, strlen(msg4));
        Baswline(&jout, laststr, strlen(laststr));
    }

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* SLPTRACE - This routine saves some information for lptrace */

void Slptrace(char *str, integer nstr)
{
    Integer n;

    n = nstr > 100 ? 100 : nstr;
    strncpy(laststr, str, n);
    laststr[n] = '\0';

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* DBGTRACE - Prints out a trace of a single executed instruction, and the
 *          - runtime stack pointer before and after
 */

void Dbgtrace(Integer unit, Integer pc, Integer rsp1, Integer rsp2,
              Integer rlinkp1, Integer rlinkp2)
{
    Integer incr;

    if (ic[pc - 1] < 0)
        Lrvprt(unit, pc, rsp1, rsp2);
    else if (ic[pc - 1] < JMP && ic[pc - 1] != RET)
        Lrxprt(unit, pc, rsp1, rsp2, rlinkp1, rlinkp2);
    else
        Lrflprt(unit, pc, &incr, rsp1, rsp2, rlinkp1, rlinkp2);

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* LRPRINT - dissamble intermediate code from location start
 *         - until stop.  If stop = 0, stop at next FCNENTER
 *         - Uses lrvprt, prdoprt, lrflprt, and lrxprt to format and print
 *         - the various types of intermediates
 */

void Lrprint(Integer unit, Integer start, Integer stop)
{
    Integer oldpart, pc, limit, locexec, ic1, incr;
    static char msg1[] = "   pc   opcode        operation",
        msg2[] = "***************************";

    oldpart = partrace;
    partrace = (logical) 0;
    Remark(msg1, strlen(msg1));

    pc = start;
    if (stop == 0)
        limit = codebgn;
    else
        limit = stop;

    locexec = execflag;         /* fool mother nature */
    execflag = (logical) 0;
    while (pc <= limit) {
        Ruthere();
        ic1 = ic[pc - 1];
        if (ic1 < 0) {
            Lrvprt(unit, pc, 0, 0);
            pc += 3;
        } else if (ic1 < JMP && ic1 != RET) {
            Lrxprt(unit, pc, 0, 0, 0, 0);
            if (ic1 == NEST || ic1 == UNNEST || ic1 == LINE)
                pc += 2;
            else
                pc += 1;
        } else {
            Lrflprt(unit, pc, &incr, 0, 0, 0, 0);
            pc += incr;
        }
        if (stop == 0 && ic1 == FCNENTER)
            break;
    }

    execflag = locexec;
    Remark(msg2, strlen(msg2));
    Remark(" ", 1);
    partrace = oldpart;

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* DPRSE - print a stack element (debug) */

void Dprse(Integer *si)
{
    Integer i, tc, ptr, nml, ns, ne;
    Integer ndb;
    char name[VARNAMESIZE], type[VARNAMESIZE];
    char *pn;
    void *ad;
    extern void Utcodstr(Integer tc, char *type);

    memset(name, ' ', VARNAMESIZE);

    si--;

    ndb = si[N];
    tc = si[TC];
    ptr = si[PTR];
    nml = si[NAML];
    ns = si[NS];

    if (tc == NULL_TYPE) {
        strcpy(name, "NULL");
        strcpy(type, "NULL");
        pn = name;
        ne = 0;
    } else {
        Utcodstr(tc, type);
        ne = Parlen(&si[N]);
        ad = Paraddr(&si[N]);

        if (tc == FUNCTION_TYPE) {
            Parname(&si[N], name);
            pn = strtok(name, " ");
            printf("   %s %s entry %ld in %ld\n",
                   type, pn, (long) ptr, (long) ndb);
        } else {
            if ((0 < nml) && (nml < MAXPAK + 3))
                Parname(&si[N], name);

            if ((name[0] == '?') || (name[0] == ' ') ||
                (strcmp(name, "Bad stack element.") == 0)) {
                strcpy(name, "none");
                pn = name;
            } else
                pn = strtok(name, " ");
        }
    }

    printf("   %s %s[%ld] @ %px entry %ld in %ld\n",
           type, pn, (long) ne, ad, (long) ptr, (long) ndb);
    printf("  TC    %ld\n", (long) tc);
    printf("  NAML  %ld\n", (long) nml);
    printf("  NS    %ld\n", (long) ns);

    printf("  DIMS  ");
    for (i = 0; i < 7; i++)
        printf(" (%ld, %ld, %ld)",
               (long) si[N1 + i], (long) si[M1 + i], (long) si[I1 + i]);
    printf("\n");

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* DPRST - print a range of stack elements (debug) */

void Dprst(Integer i1, Integer i2)
{
    Integer ei;

    for (ei = i1; ei <= i2; ei++) {
        printf("-------------------------------------------------\n");
        printf("Stack element #%ld:\n", (long) ei);

        Dprse(&ssl(N, ei));

        printf("-------------------------------------------------\n");
    }

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* LRFTRMK - mark a function with a trace flag */

void Lrftrmk(char *name, int flag, int *value)
{
    int *pi;
    char lname[MAXLINE];
    char *pn;
    static HASHTAB *funtab = NULL;

    if (flag == 3) {
        if (funtab != NULL) {
            SC_rl_hash_table(funtab);
            funtab = NULL;
        }
    } else {
        if (funtab == NULL)
            funtab = SC_make_hash_table(HSZLARGE, NODOC);

        strncpy(lname, name, MAXLINE);
        pn = strtok(lname, " ");

        pi = (int *) SC_def_lookup(pn, funtab);
        switch (flag) {
        case 1:
            if (pi == NULL) {
                pi = FMAKE(int, "LRFTRMK:pi");

                SC_install(name, pi, SC_INTEGER_S, funtab);
            }

            *pi = *value;
            break;

        case 2:
            *value = (pi == NULL) ? FALSE : *pi;
            break;
        }
    }

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* LRPFCALL - print a function call part */

static void Lrpfcall(Integer ei)
{
    Integer tc, nml, ndb, ptr, ne, fty;
    char name[VARNAMESIZE], type[VARNAMESIZE];
    char *pn;
    void *ad;
    static Integer unit = 6;
    extern void Utcodstr(Integer tc, char *type);

    memset(name, ' ', VARNAMESIZE);

    tc = ssl(TC, ei);
    nml = ssl(NAML, ei);
    fty = ssl(N2, ei);

    if (tc == NULL_TYPE)
        printf("   NULL\n");
    else {
        Utcodstr(tc, type);
        ad = Paraddr(&ssl(N, ei));
        ne = Parlen(&ssl(N, ei));

        if (tc == FUNCTION_TYPE) {
            Parname(&ssl(N, ei), name);
            pn = strtok(name, " ");
            printf("Entering function %s with:\n", pn);
        } else {
            ndb = ssl(N, ei);
            ptr = ssl(PTR, ei);
            if ((0 < ndb) && (ndb < MAXPAK + 3) &&
                (0 < ptr) && (ptr <= Rtgnvar(ndb)) &&
                (0 < nml) && (nml < MAXPAK + 3) &&
                ((tc == LHS) || (tc == NAME) || (tc < 0) ||
                 (tc == NULL_TYPE) || (nml == 0)))
                Parname(&ssl(N, ei), name);

            if ((name[0] == '?') || (name[0] == ' '))
                pn = NULL;
            else if (strcmp(name, "Bad stack element.") == 0)
                pn = NULL;
            else
                pn = strtok(name, " ");

            if (pn == NULL) {
                if (ne < 2)
                    printf("   %s =", type);
                else
                    printf("   %s[%ld] =", type, (long) ne);
            } else {
                if (ne < 2)
                    printf("   %s %s =", type, pn);
                else
                    printf("   %s %s[%ld] =", type, pn, (long) ne);
            }

            if (tc == LHS)
                printf(" unevaled expression\n");
            else
                F77_ID(prttrc_, prttrc, PRTTRC) (&unit, &tc, &ne, &ad);
        }
    }

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* LRFDOTR - trace a function call */

void Lrfdotr(void)
{
    int i, ifp, trace;
    Integer tc;
    char name[VARNAMESIZE];

    INIT_BLANK_STRING(name, VARNAMESIZE);

    ifp = rlinkp - 1;

    tc = ssl(TC, ifp);
    if (tc == FUNCTION_TYPE) {
        Parname(&ssl(N, ifp), name);

        Lrftrmk(name, 2, &trace);

        if (trace) {
            Lrpfcall(ifp);

            for (i = rlinkp + 1; i <= rsp; i++)
                Lrpfcall(i);
        }
    }

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* UNTRACE - untrace a function */

void Untrace(char *name)
{
    int value;

    value = FALSE;
    Lrftrmk(name, 1, &value);

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* TRACE - trace a function call */

void Trace(char *name)
{
    int value;

    value = TRUE;
    Lrftrmk(name, 1, &value);

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* ERRPRT - In the event of a deeply buried semantic error, this
 *        - routine attempts to give maximum context information:
 *        - what were the most recent operations attempted,
 *        - and what function calls got us here.
 */

void Errprt(void)
{
    Integer pc, sp, ic1, i, len;
    char *opcode, lhs[73];
    char cprobname[33];
    static char msg[] = "unknown(please report this message)",
        msg1[] =
        "The error occurred in the assignment or append statement: ",
        msg2[] =
        "The following lines contain clues(not facts) about the r. h. s.",
        msg3[] = "Error occurred in non-assignment statement.",
        msg4[] = "The following lines contain clues to the error.",
        msg5[] = "Some or all may be irrelevant to your problem.",
        msg6[] = "-- Stack information unavailable since debug = no.",
        msg7[] = "-- Retrying with debug = yes may be helpful.";

    if (!execflag || !strncmp(bastrace, "none", 4))
        return;

    if (ic == (Address) NOTSET)
        ic = intcode;

    pc = oldpc;
    sp = nsp;
    strncpy(cprobname, probname, 32);
    cprobname[32] = '\0';

    if (!strncmp(bastrace, " ", 1)) {
        if (!strncmp(cprobname, " ", 1)) {
            strcpy(cprobname, "wasblank");
        } else {
            len = strcspn(cprobname, " ");
            if (len == 0)
                len = 32;
            cprobname[len] = '\0';
        }
        memset(bastrace, (int) ' ', FILENAMESIZE);
        strcpy(bastrace, cprobname);
        strcat(bastrace, ".000.err");
        ((char *) bastrace)[strlen(bastrace)] = ' ';
    }

    Basnxtsq(bastrace, bastrace, FILENAMESIZE, FILENAMESIZE);

    ic1 = ic[pc - 1];
    opcode = Lropcode(ic1);
    Ffopen(&jout);
    Lptrace(pc);
    if (debug == 1)
        if (inassign) {
            if (lxp[assignsp - 2] <= 0)
                strcpy(lhs, msg);
            else {
                strncpy(lhs, nxp[assignsp - 2], lxp[assignsp - 2]);
                lhs[lxp[assignsp - 2]] = '\0';
            }
            strcat(lhs, " = expression");
            Baswline(&jout, msg1, strlen(msg1));
            Baswline(&jout, lhs, strlen(lhs));
            Baswline(&jout, msg2, strlen(msg2));
        } else {
            Baswline(&jout, msg3, strlen(msg3));
            Baswline(&jout, msg4, strlen(msg4));
            Baswline(&jout, msg5, strlen(msg5));
            for (i = sp; i >= 1 && i >= sp - 10; i--) {
                if (lxp[i - 1] <= 0 || lxp[i - 1] > COMMENTSIZE)
                    continue;
                Baswline(&jout, nxp[i - 1], lxp[i - 1]);
            }
    } else {
        Baswline(&jout, msg6, strlen(msg6));
        Baswline(&jout, msg7, strlen(msg7));
    }

    sprintf(lhs,
            "Parser's action number = %ld(%s), program counter = %ld.",
            (long) ic1, opcode, (long) pc);
    Baswline(&jout, lhs, strlen(lhs));

    Basclose(&jout);

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

void Whereami(void)
{
    integer iout;

    iout = jout;
    jout = STDERR;

    Lptrace(0);
    jout = iout;

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
