
      subroutine dfitpol(xindt,ydept,ndata,nterms,weight,coeff,rsd2
     1 ,wrkspc,job,ierr)
      double precision xindt(ndata),ydept(ndata),coeff(nterms)
      double precision wrkspc(ndata,1),weight(1)
c
c fits polynomial to data; that is, finds coeff(1),...,
c coeff(nterms) for which ydept = coeff(1) + coeff(2)*xindt +
c ...+ coeff(nterms)*xindt**(nterms-1) approximately - i.e.,
c minimizes rsd2, the sum of squares of the residuals.  if
c weight.ne.0 then weight is a vector of weights for the
c ndata input data points (xindt,ydept).  calling program pro-
c vides work space wrkspc, a vector of dimension at least
c nterms*ndata + ndata + nterms + 1.  if job.ne.0 then on output
c residuals will be in wrkspc(1),...,wrkspc(ndata).  ierr=0
c iff no error found.    routines called:
c  linpack  dqrdc, dqrsl
c  blas     ddot
c
c internal variables:
      integer i,j,info
      double precision dummy,rmax,rmin
c data is first put in wrkspc as follows:
c
c   y(1)     nterms  1          x(ndata)      ... x(ndata)**(nterms-1)
c   y(2)     1       x(1)       x(1)**2       ... qraux(1)
c   y(3)     1       x(2)       x(2)**2       ... qraux(2)
c     .      .         .            .         ...     .
c     .      .         .            .         ...     .
c   y(ndata) 1       x(ndata-1) x(ndata-1)**2 ...
c
c where x,y are short for xindt,ydept and qraux is
c used in dqrdc and dqrsl.
c
c
c should data be weighted?
c
      if (weight(1) .ne. 0) go to 20
c
c  unweighted data. copy ydept into working variable and
c prepare matrix of powers of xindt to be unweighted.
c
      do 10 i = 1,ndata
         wrkspc(i,1) = ydept(i)
   10    wrkspc(i+1,2) = 1.
      go to 40
c
c weighted data. put weighted ydept into working variable and
c prepare matrix of powers of xindt to be weighted.
c
   20 do 30 i = 1,ndata
         wrkspc(i,1) = ydept(i)*weight(i)
   30    wrkspc(i+1,2) = weight(i)
c
c construct matrix of powers of xindt.
c
   40 do 60 j = 3, nterms+1
         do 50 i = 1,ndata
   50       wrkspc(i+1,j) = wrkspc(i+1,j-1)*xindt(i)
   60    continue
c
c decompose matrix of powers of xindt.
c
      call dqrdc(wrkspc(2,2),ndata,ndata,nterms,wrkspc(2,nterms+2),
     10,dummy,0)
c
c find lower bound for condition number as ratio of min and max
c diagonal element of upper triangular part r of decomposed
c matrix.
c
      ierr = 0
      rmin = abs(wrkspc(2,2))
      rmax = rmin
      do 70 i = 3, nterms + 1
         rmin = dmin1(rmin,dabs(wrkspc(i,i)))
   70    rmax = dmax1(rmax,dabs(wrkspc(i,i)))
c
c if condition number less than 1.e4, problem is ok. go on to 90.
c
      if(rmax.le.(rmin*1.e4)) go to 90
c
c ill-conditioned problem. set error flag and then go on.
c
      if(rmin.eq.0..or.rmax.eq.0.) go to 80
      ierr = int(log10(rmin) - log10(rmax))
      go to 90
c
c r has zero diagonal element. let dsqrsl set error flag.
c
   80 continue
c
c are residuals needed?
c
   90 if(job.eq.0) go to 100
c
c solve for coeff and compute residuals.
c
      call dqrsl(wrkspc(2,2),ndata,ndata,nterms,wrkspc(2,nterms+2),
     1  wrkspc(1,1),dummy,wrkspc(1,1),coeff,wrkspc(1,1),dummy,110,
     1  info)
c
c compute sum of squares of residuals and go to finish up.
c
      rsd2 = ddot(ndata,wrkspc(1,1),1,wrkspc(1,1),1)
      go to 110
c
c solve for coeff without fully computing residuals.
c
  100 call dqrsl(wrkspc(2,2),ndata,ndata,nterms,wrkspc(2,nterms+2),
     1  wrkspc(1,1),dummy,wrkspc(1,1),coeff,dummy,dummy,100,info)
c
c compute sum of squares of residuals = sum of squares of last
c ndata-nterms components of qty, which dsqrsl outputs.
c
      rsd2 = ddot(ndata-nterms,wrkspc(nterms+1,1),1,
     1 wrkspc(nterms+1,1),1)
c
c update error flag, save nterms for use by refitp, and return.
c
  110 wrkspc(1,2) = nterms
      ierr = ierr + info
      return
      end
