/*
 * dkspc.c
 *
 */

#include "rt.h"

/*
 * Define or not the HasQuota macro to choose whether to have the
 * diskspace function make a system(3) call to a special quota
 * program.  For an example of such a special program, see the perl
 * script "quota" located in the scripts subdirectory of this package.
 */
/*#define HasQuota*/
#ifdef HasQuota
#   ifndef QuotaProg
#       if defined(YMP)|defined(XMP)
#           define QuotaProg "/u2/h/busby/pub/remquota "
#       else
#           define QuotaProg "/env/bin/quota -r "
#       endif
#   endif
#endif

#ifdef SGI
/* Figure out which version of SGI O.S. we've got...            */
#   ifdef _SVR4_SOURCE
#       define SGI_R4
#   else
#       define SGI_R3
#   endif
#endif

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

float C_dkspc(p)
char *p;
{

/* The SunOS model */
/* BTW, I don't know how BSD4.[123] computed disk free space.  There is
   no statfs() system call in those versions of Unix.  Oh, well. */
#if defined(SUN4)|defined(HP700)|defined(LINUX)
#   include <sys/vfs.h>
#   define Statfs statfs
#   define Blk_sz f_bsize
#   define Blk_fr f_bavail
#   define St_arg p,&sb
#endif

/* The IBM model */
#if defined(RS6000)
#   include <sys/statfs.h>
#   define Statfs statfs
#   define Blk_sz f_bsize
#   define Blk_fr f_bavail
#   define St_arg p,&sb
#endif

/* The SVR4 model */
#if defined(SOL)|defined(SGI_R4)|defined(CS2)|defined(AXP)|defined(IRIX64)
#   include <sys/statvfs.h>
#   define Statfs statvfs
#   define Blk_sz f_frsize
#   define Blk_fr f_bavail
#   define St_arg p,&sb
#endif

/* The SVR3 model */
#if defined(YMP)|defined(SGI_R3)|defined(C90)|defined(J90)|defined(CRAY2)|defined(XMP)
#   include <sys/statfs.h>
#   undef Statfs
#   undef Blk_sz
#   undef Blk_fr
#   undef St_arg
#   define Statfs statfs
#   define Blk_sz f_bsize
#   define Blk_fr f_bfree
#   define St_arg p,&sb,sizeof(sb),0
#endif

    struct Statfs sb;
    float quo_val = 1.0e30;     /* Hope that's big enough for awhile */
    float sta_val;

#ifdef HasQuota
    FILE *fp;
    char *qp;

#ifdef PCK_SCORE
    qp = FMAKE_N(char, 1 + strlen(QuotaProg) + strlen(p), "C_DKSPC:qp");
#else
    qp = (char *) malloc(1 + strlen(QuotaProg) + strlen(p));
#endif

    strcpy(qp, QuotaProg);
    strcat(qp, p);
    if ((fp = popen(qp, "r")) != (FILE *) 0) {
        if (fscanf(fp, "%f", &quo_val) != 1) {
            quo_val = 1.0e30;
        }
        pclose(fp);
    }
#endif

    while (isspace(*p))
        ++p;
    if (Statfs(St_arg) == 0) {
        sta_val = sb.Blk_fr * ((float) sb.Blk_sz / 1048576.0);
    } else {
        sta_val = (float) -1.0;
    }
    return ((sta_val < quo_val) ? sta_val : quo_val);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

#ifdef TEST
/* Typical compile line: "cc -o tester -DHasQuota -D$CPU -DTEST thisfile.c" */
/* Usage: "./tester . /tmp /usr/local" - Then compare with df           */
main(a, b)
int a;
char *b[];
{
    float C_dkspc();

    while (*++b)
        printf("%-20s: %.3f MB\n", *b, C_dkspc(*b));
    exit(0);
}
#endif

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
