# Basis Code Development System
# Copyright 1986 The Regents of the University of California

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c BASWLINE - write out a line, and check for interrupt

      subroutine baswline(unit, msg)
      integer unit
      character*(*) msg

      call baspline(unit, msg)
      call ruthere

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PTEXTW - wrapper for graphics ptext if you have it

      subroutine ptextw(msg)
      character*(*) msg

ifdef([NO_GRAPHICS],[
      call remark(msg)
],[
      call ptext(msg)
])

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c BAASPLINE - write out a line, but don't check for interrupt
c           - if baspecho has been called with a positive unit number, 
c           - echo to it.

      subroutine baspline(iunit, msg)

Use(Multiprocessing)
Use(Basplc)

      character*(*) msg
      integer iunit, utgetcl, il

      il = utgetcl(msg)
      if (iunit == 100) then
         call ptextw(msg(1:il))
      else

c ... multiprocessing modification
         if (~(nostdout & iunit == STDOUT)) then
            write(iunit, '(a)') msg(1:il)
         endif
      endif

      if (iecho > 0) then
         if ((iunit <> STDOUT & iunit <> STDERR & iunit <> 100) |
             (iecho == iunit)) return

         if (iecho == 100) then
            call ptextw(msg(1:il))
         else

c ... No test needed.(iecho cannot be STDOUT--see baspecho.)
            write(iecho, '(a)', err=100) msg(1:il)
         endif
      endif
      return

100   continue
      iecho = 0

      call baserror('baspline: write failure on unit set by baspecho.')
      call baserror('          baspecho disabled.')
      call kaboom(0)

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c BASPECHO - begin echo to iunit
c          - call with iunit <=0 to disable

      subroutine baspecho(iunit)

Use(Basplc)

      logical isopen
      external setiotcl

      if (iunit==100) then
         iecho = 100
         return
      endif

      if (iunit <= 0) then
         iecho = 0
         return
      endif

      if (iunit==STDOUT | iunit==STDERR) then
         iecho = 0
         call remark('baspecho: called with stdout or stderr; ignored.')
         return
      endif

      inquire(unit=iunit, opened=isopen)
      if (isopen) then
         iecho = iunit
         call setiotcl(iunit, FALSE) #don't close on error
      else
         iecho = 0
         call baserror('baspecho: called with unit number of non-open unit.')
         call kaboom(0)
      endif

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EDTATTR - print out the attributes of a LISTed variable. This was
c         - formerly part edtlstvl(Programmed by Paul Dubois) but was
c         - turned into a subroutine so that it could be called to list
c         - the attributes of functions and groups as well.

      subroutine edtattr(jvar, ndb, unit)
      integer jvar, ndb, unit

      Varname attr
      Ch(OUTLINE) attrlist
      integer rtnattr, utgetcl
      logical rtdattr
      external rtdattr, rtnattr, rtxattr, utgetcl, utcat, rtxnattr
      integer nattl, numattr, i, nnext

### write list of attributes
        nattl = 0
        attrlist = BLANK
        numattr = rtnattr()
        do i = 1, numattr
                call rtxnattr(i, nattr)
                if (rtdattr(jvar, ndb, nattr)) then
                        call rtxattr(nattr, attr)
                        if (nattl == 0) then
                                attrlist = "Attributes: "
                                nattl = utgetcl(attrlist)
                                nattl = nattl + mod(99-nattl, 9)
                        endif
                        nnext = utgetcl(attr)
                        if (nattl+nnext+1 > OUTLINE) then
                                call baswline(unit, attrlist(1:nattl))
                                attrlist = BLANK
                                attrlist = attr(1:nnext)
                                nattl = nnext + 1
                        else
                                call utcat(attrlist, nattl, attrlist, nattl, 
                                        attr, nnext, " ", 1)
                        endif
                        nattl = nattl + mod(99-nattl, 9)
                endif
        enddo
        if (nattl > 0) call baswline(unit, attrlist(1:nattl))
      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EDTLSTG - this routine is called to list the attributes of a requested
c         - group(jgrp), in the callers rt data base.  this is accomplished
c         - by setting Pointers to the callers(db).
c         - This function is also called by the PRT routines.
c         - jgrp    = index of group variables to be listed
c         - db   = callers run time data base manager
c         - unit    = unit number for output device
c         - dgvmap  = index map to group variables
c         - jvar    = index of variable to be listed

      integer function edtlstg(jgrp, ndb, unit)
      integer ndb, utgetcl
      integer imkstr
      integer jgrp, unit, jvar
      integer lvar, lname

      Varname name
      Ch(REMARKSIZE) remark
      character*16 nch
      Ch(OUTLINE) msg
      integer rtgnvar1, j1, j2, j
      logical rtisdead
      external rtgnvar1, rtisdead

      external imkstr, utgetcl, rtxcomg
      integer rtxcomg

#        set group attributes for message line
      edtlstg = YES
      call rtlstg(jgrp, jvar, ndb, name)

#        get actual number of "live" variables
      j1 = rtgnvar1(jgrp, ndb)
      j2 = rtgnvar1(jgrp+1, ndb)-1
      do j = j1 , j2
         if (rtisdead(j, ndb)) jvar = jvar - 1
      enddo

#        convert number of variables(jvar) to characters
      lvar = imkstr(jvar, nch)

#        setup message for one line group attributes
      lname = utgetcl(name)
      msg = "Group: "//name(:lname)//"  Num Vars: "//nch(:lvar)

      call baswline(unit, " ")
      call baswline(unit, msg)

      do i=1, 250
        if (rtxcomg(jgrp, ndb, i, remark) == ERR)break
        call baswline(unit, "     "//remark)
      enddo

      call edtattr(jgrp, -ndb, unit)

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EDTLSTGC - this routine is called to list the attributes of all variables
c          - in the callers rt data base whose group index is(jgrp).  this
c          - is accomplished by looping over all the variables in group(jgrp)
c          - and calling edtlstv to list the attributes of each variable.
c          - jgrp    = index of group variables to be listed
c          - db   = callers run time data base manager
c          - unit    = unit number for output device
c          - dgvmap  = index map to group variables
c          - jvar    = index of variable to be listed

      subroutine edtlstgc(jgrp, ndb, unit)
      integer ndb
      integer jgrp, unit, jvar
      integer edtlstg, edtlstv, edtlstf, edtvhl, rtgnvar1 , rtgtc, tc
      external edtlstg, edtlstv, edtlstf, edtvhl, rtgvname, rtgnvar1 , rtgtc
      external rtisdead
      logical rtisdead
      Varname vname

# call EDTLSTG to list group attributes and check function
# value to continue printing
      if (edtlstg(jgrp, ndb, unit) == NO) go to 200

# list info about each variable by looping over vars in group
# but call EDTVHL to print variable headline and test function
# value to continue printing
      if (edtvhl(unit) == NO) go to 200
      
      j1 = rtgnvar1(jgrp, ndb)
      j2 = rtgnvar1(jgrp+1, ndb)-1

      do jvar=j1 , j2

# call EDTLSTV to list variable attributes and test function
# value to continue printing
         tc = rtgtc(jvar, ndb)
         if (tc = FUNCTION_TYPE) then
            if (~rtisdead(jvar, ndb))then
               call rtgvname(jvar, ndb, vname)
               if (edtlstf(
               vname, jvar, ndb, unit)  <>  YES) break
            endif
         else
            if (~rtisdead(jvar, ndb))then
               call rtgvname(jvar, ndb, vname)
               if (edtlstv(vname, jvar, ndb, tc, unit)  <>  YES)
               break
            endif
         endif
      enddo

 200  continue
      
      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EDTLSTF - print the name and comment for a function
c         -  jvar   = function index in db
c         -  db     = callers run time data base manager
c         -  unit   = unit number for print out

      function edtlstf(name, jvar, ndb, unit)

Use(Functions)

      character*(*) name
      integer jvar, unit, ndb
      Ch(OUTLINE*3) msg   #to hold output line
      character*(DIMSTRSIZE) size  #to hold size output
      character*(COMMENTSIZE) comment
      integer icol, lsize, lcom, lmsg, i
      integer ftyp, namindx
      integer dimno
      integer edtlstf
      Ch(16) ilow, ihi
      integer ivalueo, gfunpart
      external ivalueo, gfunpart
      integer utgetcl, imkstr, rtgndimst, rtgftyp, rtgndim
      external utgetcl, imkstr, rtxcom, rtgndimst, rtgftyp, rtgndim
      logical rtxcom
      
# set pointers to database variables
# extract info from database
      ftyp = rtgftyp(jvar, ndb)
      namindx = rtgndim(jvar, ndb)
      msg = name
      lmsg = utgetcl(msg)
      if (~ rtxcom(jvar, ndb, 1, comment))comment = " "
      lcom = utgetcl(comment)
      
# add dimsto and an indication of type
      call baswline(unit, " ")
      dimno = rtgndimst(jvar, ndb)
      if (dimno == 0) then

# compiled function, need to reconstitute dimension string
         call edtdim(msg , lmsg , jvar , ndb)
      else
         call strgget(dimno, size)
         lsize = utgetcl(size) + 1
         call utcat(msg, lmsg, msg, lmsg, size, lsize, BLANK, 0)
         if (lsize == DIMSTRSIZE+1) then
            msg(lmsg-1:lmsg) = '.)'
            do icol = lmsg-2, 1, -1
               if (msg(icol:icol) == ', ') break
               msg(icol:icol) = '.'
            enddo
         endif
      endif
      
      ilow = "function "; n1 = 9; n2 = 4
      if (ftyp = 0) then
         ihi = "builtin  "; n2 = 8
      elseif (ftyp >0) then
         ihi = "user-defined "; n2 = 13
      else
         if (gfunpart(namindx) == NULL_TYPE) then
            ilow="subroutine "; n1=11; n2=0; ihi=" "
         else
            call utcodstr(gfunpart(namindx), ihi)
            n2 = utgetcl(ihi) + 1
         endif
      endif
      
# start comment in column 20 if possible
      if (msg ~= " ") call baswline(unit, msg(1:79))
      icol = 158
      if (lmsg > 158) then
         do icol = 158, 81, -1
            if (msg(icol:icol)=' ' | msg(icol:icol)=', ') break
         enddo
      endif
      
      if (msg(80:icol) ~= " ") call baswline(unit, msg(80:icol))
      if (lmsg > 158) 
      call baswline(unit, "                    "//msg(icol+1:187))
      call baswline(unit, "    " // ihi // ilow)
      do i = 1, lcom, 72
         icol = min(i + 71, lcom)

# five blanks in front so all comments line up
         call baswline(unit, "     "//comment(i:icol))
      enddo
      
      return(YES)
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EDTDIM - reconstitute the calling sequence of a compiled
#                     function or subroutine. This is necessary because
#                     the dimension string is parsed to extract its
#                     information, and then discarded. This routine
#                     will return no more than DIMSTRSIZE characters
#                     and will return only the parameter names; the
#                     LIST command always prints the names and types
#                     of parameters later in the same output.
#         str :        String into which the result will be stored.
#         strsize :    Number of characters in the string so far
#         jvar , ndb : Uniquely identify the item in the database.

      subroutine edtdim(str , strsize , jvar , ndb)

Use(Functions)

      Ch(*) str
      integer strsize , jvar , ndb

      integer utgetcl, strgetcl, ivalueo, gfunpart
      external utgetcl, rtgargs, strgetcl, ivalueo, gfunpart

      integer namindx
      integer minargs , maxargs , nchar , iarg , namlen, itype
      Varname fname

# extract database information
      call rtgargs(jvar, ndb, minargs, maxargs, namindx)
      
      nchar = 0
      strsize = strsize + 1
      str(strsize : strsize) = "("               # open paren
      itype = 0
      do iarg = 2 , 1 + maxargs
         if (itype == STRING_ADDRESS) then
# Skip fake argument right after a String
            itype = 0
            next
         endif
         call gfunparn(namindx + iarg - 1, fname)
         namlen = utgetcl(fname)
# insert ellipsis if next name would make string too long
         if (nchar + namlen + 2 > DIMSTRSIZE) then
            str(strsize + 1 : strsize + 5) = ", ...)"
            strsize = strsize + 5
            return
         endif
# insert semicolon at very start if all args are optional
         if (iarg == 2 & minargs == 0) then
            strsize = strsize + 1
            str(strsize : strsize) = ";" 
         endif
         if (iarg > 2) then
# insert semicolon or comma after preceding name, if any
            strsize = strsize + 1
            if (iarg == minargs + 2) then
# semicolon if variable no. of arguments
# and preceding was the last required one
               str(strsize : strsize) = ";"
            else
# comma otherwise
               str(strsize : strsize) = ", "
            endif
         endif
         str(strsize + 1:) = fname(1:namlen)
         strsize = strsize + namlen
         itype = gfunpart (namindx + iarg - 1)
      enddo
      
      strsize = strsize + 1
      str(strsize : strsize) = ")"     # terminating paren
      
      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EDTLSTV - print the name and comment for a variable
#        name of the variable
#        jvar      = variable index
#        ndb     = callers run time data base manager
#        tc        = variable type
#        unit      = unit number for print out

      function edtlstv(name, jvar, ndb, type, unit)

Use(Stacks)

      Ch(OUTLINE) msg   #to hold output line
      character*(DIMSTRSIZE) size, dimsto #to hold size output
      character*(*) name
      character*(COMMENTSIZE) comment
      integer lsize, lcom, lmsg, type, itghi(7), itgorgn(7), ndim, ilen
      Address fwa
      integer icol(7), lcomma, i, n2, n1
      integer edtlstv
      integer jvar, unit, ndb
      Ch(16) ilow, ihi
      character comma
      integer utgetcl, imkstr
      external utgetcl, imkstr, rtxcom
      logical rtxcom

# set pointers to database variables
# extract info from database
      call rtxdb(jvar, ndb, fwa, ndim, itgorgn, itghi, icol, INFO)
      if (~ rtxcom(jvar, ndb, 1, comment)) then
         if (name == "$") then
            comment = nxp(rsp)
         else
            comment = " "
         endif
      endif
      lcom = utgetcl(comment)

# size contains either(a) nothing if dimen=0
#                    (b) "*dimsto" if variable is dynamic but pointer
#                               not set
#                    (c)(ilow:ihi, jlow:jhi, ..) for dimensioned vars
# if the origin is 1, the ilow: is omitted.
      if (name == "$") then
         size = "shape: (scalar) "    ## don't include text "(scalar) " as
         lsize = 7                    ## part of msg unless dim == 1
      else
         size = name
         lsize = utgetcl(size)
      endif
      
      if (fwa == NOTSET) then
         call rtxdso(jvar, ndb, dimsto)
         if (dimsto <> "scalar") then
            msg = size(1:lsize) // dimsto(1:utgetcl(dimsto)) // \
            " **unallocated**"
         else
            msg = size(1:lsize) // " **unallocated scalar**"
         endif
         size = msg
         lsize = utgetcl(size) + 10
      elseif (ndim == 0)   ## add blank space
         lsize = lsize+1
         if (name == "$") lsize = 16  ## size = "shape:(scalar) "
      else
         msg = size(1:lsize) // "("
         size = msg
         lsize = lsize + 1
         comma=","
         lcomma = 0
         do i = 1, ndim
            if (itgorgn(i) == 1 | name=="$") then
               ilen = itghi(i) - itgorgn(i) + 1
               n2 = imkstr(ilen, ihi)
               call utcat(size, lsize, size, lsize, comma, lcomma, ihi, n2)
            else
               n1 = imkstr(itgorgn(i), ilow)
               n2 = imkstr(itghi(i), ihi)
               call utcat(size, lsize, size, lsize, comma, lcomma, ilow, n1)
               call utcat(size, lsize, size, lsize, ":", 1, ihi, n2)
            endif
            lcomma = 1 #put in a comma after the first one
         enddo
         call utcat(size, lsize, size, lsize, ")", 1, BLANK, 1)
      endif

# add the size to the name
 75   continue
      if (name == "$") then
         msg = comment(1:lcom)
         lmsg = max(lcom+3, 19)       ## add a few blanks
         comment = size(1:lsize)
         lcom = lsize
      else
         msg = size(1:lsize)
         lmsg = max(lsize, 19) # start comment in column 20 if possible
      endif
      if (lmsg + lcom <= OUTLINE) then
         call utcat(msg, lmsg, msg, lmsg, comment, lcom, BLANK, 0)
         call baswline(unit, msg(1:lmsg))
      else
         call baswline(unit, msg(1:lmsg))
         call baswline(unit, "     "//comment)
      endif

      return(YES)
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EDTLSTFL - this routine prints a long edit of the attributes of function(jvar)
#        as they exist in the callers rt data base.  this is accomplished
#        by setting up Pointers to the(ndb) variable attribute arrays
#        jvar      = variable index
#        ndb     = callers run time data base manager
#        unit      = unit number for print out

      function edtlstfl(name, jvar, ndb, unit)

Use(Functions)
Use(Actions)
Use(Lrh1)

      integer edtlstfl
      character*(*) name
      integer iarg, jvar, unit, ndb, maxargs, minargs
      Address namptr, fwa, lic, afunpar
      integer codelen, ftyp, namindx, lname
      character*(TYPESIZE) type
      character*(COMMENTSIZE) comment
      character*(MAXLINE) msg
      Address fname  # Address of funpar struct
      Varname vname, pkgname, grpname
      integer nv1, nv2
      integer  edtlstf, utgetcl, rtgndim, strgetcl, rtgextra, jextra, ivalueo
      external edtlstf, rtxcom, utgetcl, rtgndim, strgetcl, rtgextra, ivalueo
      external gfunpart, afunpar, getic, setic
      integer gfunpart, nvar
      Address getic
      logical rtxcom
      character*1 reply
      
      nv1 = 24
      
# call EDTLSTF to print short edit of function attributes
# and check function value to continue printing
      
      if (jvar < 0) then
         msg = name(1:utgetcl(name))//" is a variable in a file."
         call baswline(unit, msg)
         return(YES)
      endif
      if (edtlstf(name, jvar, ndb, unit) .eq. NO) return(NO)
      do i = 2, 250
         if (~ rtxcom(jvar, ndb, i, comment))break
         call baswline(unit, "     "// comment)
      enddo
      
# get variable attributes for message out
      call rtxdbf(jvar, ndb, ftyp, minargs, maxargs, npack, njvar)
      if (ftyp < 0) then
         jextra = rtgextra(jvar, ndb)
      else
         jextra = 0
      endif
      
      write(msg, 1400) minargs - jextra
      call baswline(unit, msg)
      write(msg, 1401) maxargs - jextra
      call baswline(unit, msg)
      
# builtin
      if (ftyp = 0) then
         call glbpknam(ndb, pkgname)
         write(msg, 1407) pkgname(1:utgetcl(pkgname))
         call baswline(unit, msg)
         
# get group and package names
      elseif (ftyp < 0) then
         call rtlstg(-ftyp, nvar, ndb, grpname)
         call glbpknam(ndb, pkgname)
         
         write(msg, 1402) pkgname(1:utgetcl(pkgname)), 
     &        grpname(1:utgetcl(grpname))
         call baswline(unit, msg)
         
# user-defined
      else
         call rtguinfo(jvar, ndb, fwa, codelen, namptr)
         write(msg, 1403) fwa 
         call baswline(unit, msg)
         write(msg, 1404) codelen
         call baswline(unit, msg)
      endif
      
      if (ftyp > 0) then
         fname = namptr
      else
         namindx = rtgndim(jvar, ndb)
         fname = afunpar(namindx)
      endif
      
# GOTCHA: this is probably wrong if VARNAMESIZE is not 24
      call baswline(unit, " NAME                     TYPE")
      
# user
      if (ftyp > 0) then
         type = "varies"
         do iarg = 1, 1 + maxargs
            call gfuparn2(fname, iarg, vname)
            lname = utgetcl(vname)
            msg = vname
            nv2 = max(nv1, lname) + 1
            msg(nv2:) = "  " // type
            call baswline(unit, msg)
         enddo
         
# compiled
      elseif (ftyp < 0) then
         type = " "
         do iarg = 1, 1 + maxargs
            msg = " "
            if (type == "String") then
# Leave out integer argument right after String
               type = " "
            else
               call gfuparn2(fname, iarg, vname)
               call utcodstr(gfunpart(namindx - 1 + iarg), type)
               lname = utgetcl(vname)
               msg = vname
               nv2 = max(nv1, lname) + 1
               if (type == "String") then
                  msg(nv2:) = "  character*500"
               else
                  msg(nv2:) = "  " // type
               endif
               call baswline(unit, msg)
            endif
         enddo
      else
         lname = utgetcl(name)
         msg = name
         nv2 = max(nv1, lname) + 1
         msg(nv2:) = " varies"
         call baswline(unit, msg)
      endif
      
      call edtattr(jvar, ndb, unit)
      if (ftyp > 0) then
         reply = ' '
         call basget("Dump text of function?(y|n) ", reply)
         if ((reply == "y") |(reply == "Y")) then
            call utftxout(unit, jvar, ndb)
         endif
         reply = ' '
         call basget("Dump intermediate code?(y|n) ", reply)
         if ((reply == "y") |(reply == "Y")) then
            lic = getic()
            call setic(fwa)
            call lrprint(unit, 1, codelen)
            call setic(lic)
         endif
      endif
      
      return(YES)
      
 1400 format("Minimum number of arguments:", i6)
 1401 format("Maximum number of arguments:", i6)
 1402 format("This is a compiled function from package ", 
      a, ", Group ", a, ".")
 1403 format("User-defined function, begins at absolute address ", i12)
 1404 format("Function consists of ", i6, " words of intermediate code.")
 1405 format(1x, a, 2x, a16)
 1406 format(a1)
 1407 format("This is a built-in function from package ", a, ".")
      
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EDTLSTVA - this routine prints a list of all packages and their
c          - priorities which contain the given name, up to a limit
c          - of ten

      integer function edtlstva(name, jvar, ndb, tc, pr, no, unit)
      Ch(*) name
      integer jvar(10), ndb(10), tc(10), pr(10), no, unit

Use(Controls)

      Varname type
      Ch(PNSIZE) pkgname
      integer i, nc
      integer utgetcl, rtgtc, rtgpri
      external utgetcl, utcodstr, rtgtc, rtgpri, glbpknam
      Ch(80) msg

      if (edtfcnmod .eq. 0) then
         msg = " "
         call utsprint(1, 1, 80, msg,
     &                 52, "Here are the %d highest priority occurrences of %.*s",
     &                 no, utgetcl(name), name)
         call baswline(unit, msg)

         call baswline(unit, "Priority    Package Name    Type")
         call baswline(unit, "--------    ------------    ----")

         do i = 1, no
            type = " "
            msg  = " "
            call utcodstr(rtgtc(jvar(i), ndb(i)), type)
            call glbpknam(ndb(i), pkgname)

            nc           = utgetcl(type) + 1
            type(nc:nc)  = char(0)
            pkgname(8:8) = char(0)
            call utsprint(1, 1, 80, msg,
     &                    28, "%5ld       %.*s         %.*s",
     &                    rtgpri(ndb(i)), 8, pkgname, nc, type)
            call baswline(unit, msg)
         enddo

      else
         msg = " "
         call utsprint(1, 1, 80, msg,
     &                 52, "Here are the %d highest priority occurrences of %.*s",
     &                 no, utgetcl(name), name)
         call baswline(unit, msg)

         call baswline(unit, "Package Name    Type")
         call baswline(unit, "------------    ----")

         do i = 1, no
            type = " "
            msg  = " "
            call utcodstr(rtgtc(jvar(i), ndb(i)), type)
            call glbpknam(ndb(i), pkgname)

            nc           = utgetcl(type) + 1
            type(nc:nc)  = char(0)
            pkgname(8:8) = char(0)
            call utsprint(1, 1, 80, msg,
     &                    17, " %.*s        %.*s",
     &                    8, pkgname, nc, type)
            call baswline(unit, msg)
         enddo

      endif

      return(YES)
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------
      
c EDTLSTVL - this routine prints a long edit of the attributes of variable(jvar)
#        as they exist in the callers rt data base.  this is accomplished
#        by setting up Pointers to the(ndb) variable attribute arrays
#        jvar      = variable index
#        ndb     = callers run time data base manager
#        itype     = variable type
#        unit      = unit number for print out

      integer function edtlstvl(name, jvar, ndb, itype, unit)
      character*(*) name
      integer limstr
      integer jvar, unit, ndb, itghi(7), itgorgn(7), icol(7), itype, gsize
      Address fwa
      character*(TYPESIZE) type
      character*(DIMSTRSIZE) dimst
      character*(COMMENTSIZE) comment
      character*(UNITSIZE) units
      character*80 msg
      external rtgunits
      Varname grp
      character*6 pkg
      integer actstr, rtgnastr
      external rtgnastr, rtischam, rtisdyn
      integer edtlstv, utgetcl, rtgnvar1, rtgwords, rtgndimst, rtglimst
      Address truncadr
      external edtlstv, truncadr, utgetcl, rtxcom, rtgnvar1, rtggname, rtgwords, 
      rtgndimst, rtglimst
      logical rtxcom, rtischam, rtisdyn

#        call EDTLSTV to print short edit of variable attributes
#        and check function value to continue printing
      if (edtlstv(name, jvar, ndb, itype, unit) .eq. NO) return(NO)
      do i=2, 250
         if (~ rtxcom(jvar, ndb, i, comment))break
         call baswline(unit, "     "// comment)
      enddo
      do igrp = 1, 1000
         if (rtgnvar1(igrp, ndb) > jvar) break
      enddo
      call rtggname(igrp-1, ndb, grp)
      gsize = utgetcl(grp)
      call glbpknam(ndb, pkg)
      if (jvar > 0) then
         write(msg, 99) pkg, grp(1:gsize), rtgwords(jvar, ndb)
 99      format('Package = ', a6, ', group = ', a, ', storage = ', i8)
      else
         write(msg, 98) pkg, grp(1:gsize)
 98      format('Package = ', a6, ', group = ', a, ', storage = foreign')
      endif
      call baswline(unit, msg)

# write list of attributes
      call edtattr(jvar, ndb, unit)
      
# get variable attributes for message out
        call rtxdb(jvar, ndb, fwa, ndim, itgorgn, itghi, icol, INFO)

        if (jvar > 0) then
           call strgget(rtgndimst(jvar, ndb), dimst)
           ndimst = utgetcl(dimst)
           write(msg, 900) dimst(1:ndimst)
 900       format("Original dimension: ", a)
           call baswline(unit, msg)
           limstr = rtglimst(jvar, ndb)
           if (limstr > 0) then
              call strgget(limstr, dimst)
              ndimst = utgetcl(dimst)
              write(msg, 899) dimst(1:ndimst)
              call baswline(unit, msg)
 899          format("Limited dimension string: ", a)
           elseif (limstr < 0) then
              write(msg, 898)(-limstr)-1
              call baswline(unit, msg)
 898          format("Last dimension limited to length ", i6)
           endif
           actstr = rtgnastr(jvar, ndb)
           if (actstr > 0) then
              call strgget(actstr, dimst)
              ndimst = utgetcl(dimst)
              write(msg, 1899) dimst(1:ndimst)
              call baswline(unit, msg)
 1899         format("Assignment-action string: ", a)
           endif
           call utcodstr(itype, type)
           if (rtischam(jvar, ndb)) then
              write(msg, 901) type
           else
              write(msg, 902) type
           endif
           call baswline(unit, msg)
 901       format("Chameleon, current type: ", a)
 902       format("Type: ", a)
#     
           if (rtisdyn(jvar, ndb)) then
              if (fwa == NOTSET) then
                 write(msg, 910)
              else
                 write(msg, 920) fwa
              endif
           else
              fwa = truncadr(fwa)
              write(msg, 930) fwa
 910          format("Dynamic variable, pointer not set.")
 920          format("Dynamic variable, currently points to ", i16)
 930          format("Address: ", i16)
           endif
           call baswline(unit, msg)
#     
           call rtgunits(jvar, ndb, units)
           if (units == " ") units = "None."
           write(msg, 940) units
           call baswline(unit, msg)
 940       format("Units: ", a)
        else
           call utcodstr(itype, type)
           write(msg, 902) type
           call baswline(unit, msg)
           write(msg, 950)
           call baswline(unit, msg)
 950       format("Foreign variable, no other info available.")
        endif

        return(YES)
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EDTVHL - this routine when called prints to(unit) a headline
#        for either the long variable attribute list or the short
#        attribute list.
#        unit      = unit number for printout of headline

      function edtvhl(unit)
      integer unit, edtvhl

# write out headline for variable attribute list
      call baswline(unit, " ")
      call baswline(unit, "Variable(size)      Comment")
      call baswline(unit, "---------------------------")

      return(YES)
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EDTMCNAM - to list the names of all declared macros
*            ius       =:  the us(unit specifier) to write to

      subroutine edtmcnam(ius)
      integer ius

      external mcedtnam

      call mcedtnam(ius)

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EDTMCBDY - to list the definition of the given macro
*            jdef      =:  index of the macro to list the definition of
*            argflg    =:  TRUE if macro declared with arguments, else FLASE
*            ius       =:  the us(unit specifier) to write to

      subroutine edtmcbdy(jdef1, jdef2, argflg, ius)
      integer jdef1, jdef2, ius
      logical argflg

      external mcedtbdy

      call mcedtbdy(jdef1, jdef2, argflg, ius);

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EDTVNAME - print out all the variables, by group, if iflag = YES
c          - print out just group names, if iflag = NO

      subroutine edtvname(fd, ndb, iflag)
      integer ndb, rtgtvar
      external rtgtvar
      Filedes fd

Use(Stacks)

      if (ndb == 0) then
         if (rtgtvar() == 0) then
            call baswline(fd, "There are no User or Local variables.")
         else
            call edtvnam1(LOCALDB, fd, iflag)
            call edtvnam1(USERDB, fd, iflag)
         endif

      else
         call edtvnam1(ndb, fd, iflag)
      endif

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EDTVNAM1 - do the real work for EDTVNAME

      subroutine edtvnam1(ndb, fd, iflag)

      Filedes fd
      integer ndb, iflag
      integer jlo, jhi, i, j
      integer utgetcl, rtgnvar, rtgnvar1, rtgtc, rtgngrp
      external rtggname, rtgvname, rtisdead
      logical rtisdead
      external utgetcl, glbpknam, rtxcomg, rtgnvar, rtgnvar1, rtgtc, rtgngrp
      integer rtxcomg
      character*(6) pkg
      character*(OUTLINE) msg
      character*(REMARKSIZE) comment
      Varname vnam
      integer ngrp

      if (rtgnvar(ndb) = 0) return

      call glbpknam(ndb, pkg)

      write(msg, 111) pkg
      call baswline(fd, " ")
      call baswline(fd, msg)

      ngrp = rtgngrp(ndb)
      do i=1, ngrp  #for each group
         call rtggname(i , ndb , msg)
         nmsg = utgetcl(msg)

         if (rtxcomg(i, ndb, 1, comment) == ERR) comment = " "

         ifun = index(comment, "and functions")
         if (ifun > 0) comment(ifun:ifun+12) = " "

         call utcat(msg, nmsg, msg, nmsg, ": ", 2, comment, len(comment))
         call baswline(fd, " ")
         call baswline(fd, msg)

         jlo = rtgnvar1(i, ndb)
         jhi = rtgnvar1(i+1, ndb)-1
         if (jhi < jlo | iflag=NO) next

         msg = " "
         nmsg = 0
         do j=jlo, jhi
            if (~rtisdead(j, ndb))then
               call rtgvname(j, ndb , vnam)
               nnext = utgetcl(vnam)
               if (nmsg+nnext+1 > OUTLINE) then
                  call baswline(fd, msg)
                  nmsg = 0
                  msg = " "
               endif
               if (rtgtc(j, ndb) <> FUNCTION_TYPE) call utcat(msg, nmsg, msg, 
               nmsg, vnam, nnext, " ", 1)
               nmsg = nmsg + mod(99-nmsg, 9)
            endif
         enddo
         call baswline(fd, msg)
      enddo

      return
 111  format("----- Package ", a, " -----")
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

