
c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

#     NAME OF ROUTINE...utfcopy
#
#     AUTHOR OF ROUTINE...Paul Dubois Oct. 18, 1984
#
#     PURPOSE...
#
#        copy file whose unit specifier is fin onto unit fout
#
#     GLOSSARY OF IMPORTANT VARIABLES...
#
#        fin      = unit specifier of source which is assumed open
#        fout     = unit specifier of sink, which is already created

       subroutine utfcopy (fin,fout)
        Ch(MAXLINE) buf
        Filedes fin, fout
        integer utgetcl
	external utgetcl

        do
           read(fin,100,err=900,end=901) buf
           write(fout,100,err=900) buf(1:utgetcl(buf))
        enddo

900     continue
        call remark("utfcopy: io error.")

901     continue
        return

100     format(a)
        end
        

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

# SBASGET - to allow basis itself to prompt for, and receive, character input

        character*MAXLINE function sbasget(prompt)
        character*(*) prompt
        character*MAXLINE line

        integer basget
        external basget

        line = ' '
        if (basget(prompt, line) < 0) return(" ")

        return(line)
        end


c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

# BASISMAC - obtain and push macro files to be read at startup
# bug fix--ZCM 9/2/94 clear name to blanks right at the start
# (sometimes uninitialized, the new strg stuff broke on it)
# ZCM 7/1999 use access functions to get at macfile information

        subroutine basismac
        Filename name, mname, getmacf
        integer getnmacf, im
        external getnmacf, getmacf
        integer ospush
	logical osfind
        external ospush, osfind

        name=" "
        do im = getnmacf(), 1, -1
           mname=getmacf(im)
           if(mname(1:1)=="$") then
              call getenv(mname(2:),name)
              mname=name
           endif
           if(mname == " ") next
           if(mname(1:1)==".") then
              if(osfind(mname,name,"i")) then
		if(ospush(name," ",0)==ERR) then
			call baderr('Trouble reading '//name)
		endif
              endif
           else
		if( ospush(mname," ",0)==ERR) then
        		call remark("Couldn't read startup file:")
        		call remark(mname)
		endif
           endif
        enddo

        return
        end


c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

#        Create an output text file; do not close on error recovery
#        Altered by Zane C. Motteler
#        6 March 1995
#        Purpose: Will not add a file to the list if it is already
#                 on the list. Keeps a count of the number of times
#                 a name has been used. This gives a much shorter
#                 printout at the end.

        subroutine outfile (fd, tag)
        integer fd
        character*(*) tag

	integer basopen, streq1f,utgetcl, giotabn, found, basfile
	external basopen, streq1f,utgetcl, giotabn, basfile
        Filename f
Use(IO)
	call glbhspn(f)
        fd = basopen(f, "w")
        iotabcl(fd) = FALSE
        found = basfile (f, tag)

        # (ZCM 11/11/1999) The following is so that a file name can be
        # retrieved knowing the file descriptor; mainly used in hst tests.
        if (found ~= 0) then
           iotabfi (fd) = found
        else
           iotabfi (fd) = giotabn ()
        endif

        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

#       Return the filename corresponding to the given file descriptor.
#       Used primarily by the hst tests.

        function outfilen (iunit)
        Filename outfilen
        integer iunit
        external giotabf

Use(IO)

        if (iotabfi (iunit) <= 0) return ("?? Unknown file name ??")
        call giotabf (iotabfi (iunit), outfilen)

        return 
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

# the unit specifier currently active for output, usually the terminal
# but modified by 'output to ' command

        function iooutus()
        integer iooutus

Use(IO)

        return(iusout)
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

#     NAME OF ROUTINE...oscurfd
#
#     AUTHOR OF ROUTINE...Paul Dubois, 10/24/84
#
#     PURPOSE...
#
#         return the unit specifier of the current input file
#
#     GLOSSARY OF IMPORTANT VARIABLES...
#
#        fds      = stack of 'ioc's
#        istk     = stack pointer for above stacks

      subroutine oscurfd (fd,n)

Use(Infiles)

      integer n
      Filedes fd

      fd = fds(istk)
      n  = nextlin(istk)

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

#     NAME OF ROUTINE...ospop
#
#     AUTHOR OF ROUTINE...Paul Dubois, Oct 11, 1984
#
#     PURPOSE...
#
#    Pop the current file off the read stack

      subroutine ospop

Use(Infiles)

      pblinl(istk) = 0   #clear pushed back line

      if (istk > 1 ) then  #close file and pop stack
         call basclose(fds(istk))
         istk = istk - 1
      endif

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

#     NAME OF ROUTINE...ospush
#
#     AUTHOR OF ROUTINE...Paul Dubois, Oct 11, 1984
#               Revised 3/15/85 to improve read command
#
#     PURPOSE...
#
#    Push a new file onto the read stack
#    We save the remainder of a line from the current file
#
#     GLOSSARY OF IMPORTANT VARIABLES...
#
#    fname    = name of file to start reading
#   lintex    = text of remainder of line (if any)
#   linlen    = length of line to save

	function ospush (fname,lintex,linlen)
	character*(*) fname
        character*(*) lintex
        integer linlen
	integer ospush

Use(Controls)
Use(IO)
Use(Infiles)
	integer fd, basopen, utgetcl
	external basopen, utgetcl
        Filename msg

	if(istk == MAXINSTACK) then
        	call remark("ospush: Input files nested too deeply")
	        return(ERR)
	endif
	fd = basopen(fname,"r")
# skip nskipr records, checking for possible errors.
	do isk = 1, nskipr
	  read (fd,100,err=900,end=900)
	enddo
  100  format(a1)
# save remainder of line -- will recover on a pop
	pblinl(istk) = linlen
	if(linlen > 0) then
        	pblin(istk) = lintex(1:linlen)
	endif
# push
	istk = istk + 1
	fds(istk) = fd
	fnames(istk) = actualf  #changed from fname; actualf set by basopen
	nextlin(istk) = nskipr + 1
	nskipr = 0
	return(OK)
# ERROR return
  900   write(msg, 901) fname(1:utgetcl(fname)), nskipr
  901   format("Failed attempting to restore file ",a," to line ",i6)
        call remark(msg)
        call basclose(fd)
        nskipr = 0
        return(ERR)
	end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

# OSERRPOP

	subroutine oserrpop

Use(IO)
Use(Infiles)
	integer i, utgetcl, ncare
	external utgetcl
	logical stillopn
        Filename msg

# Save the stack
	call osstores

# unwind the input stack
	do
		if(istk > 1) then
                   ncare=min(80, utgetcl(fnames(istk)))
		   write(msg,100) fnames(istk)(1:ncare),
				nextlin(istk)-1
                   call remark(msg)
                endif
	        call ospop
	until(istk == 1)
#close anything opened by basopen rather than outfile 
        do i = 1, 99
		if(iotable(i)<=0) next
		if(iotabcl(i)) then
			inquire(unit=i,opened=stillopn)
			if(stillopn) call basclose(i)
		endif
	enddo

	return
100 format('Last line read from ',a,' was line number ',i6)
	end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

# GETFILEN - returns the name of the current file being read
#          - by the "read" statement

      Filename function getfilen()

Use(Infiles)

      return(fnames(istk))
      end


c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

# IOTABSET - mark things in iotable as -1 if permanently unavailable
#          - when freeus gives away a unit, mark it 1.

      subroutine iotabset(ius)
      integer ius

Use(IO)

      if (ius == -100) then
         iotable(TTY)    = -1
         iotable(STDIN)  = -1
         iotable(STDOUT) = -1
         if (STDERR > 0) iotable(STDERR)=-1

      elseif (iotable(ius) == 0) then
         iotable(ius) = -1

      else
         call baderr('ERROR: trying to reserve an I/O unit already reserved')
      endif

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

      subroutine freeus(ius)
      integer ius
      logical isopen, iounitc

Use(IO)

   10 do i = 1, 99
        if(iotable(i) == 0) then
          ius = i
          iotable(ius) = 1
	  iotabcl(ius) = FALSE
          return
        endif
      enddo

c ... haven't found an available unit
c ... check to see if anything has become available.
      iounitc = FALSE
      do i = 1, 99
	if(iotable(i)== -1) next
        inquire(unit=i, opened=isopen)
        if(~ isopen ) then
          iotable(i) = 0
          iounitc = TRUE
        endif
      enddo

      if (iounitc ) then
        go to 10
      else
        call baderr("You have all 99 I/O units in use. I'm doomed!")
      endif

      return
      end      

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

! GIVEUPUS - inverse of freeus
c this routine performs the inverse of freeus: freeus/open, giveupus/close.
c freeus and giveupus can be called in conjunction with f90 open and close.
c note that giveupus has character pointer errors if compiled in f90.

      subroutine giveupus(unit)
      integer unit
Use(Inout)
      integer i
      external setiotab

      call setiotab(unit, 0)
      do i = 2, MAXINP
        if(inunit(i).eq.unit) inunit(i)=0
      enddo
      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

# OSSTORES - store the input file stack

      subroutine osstores

Use(Infiles)

      do i = 1, istk
        istko = istk
        fnameso(i) = fnames(i)
        pblino(i) = pblin(i)
        pblinlo(i) = pblinl(i)
        nextlino(i) = nextlin(i)
      enddo

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

# OSRESUME - resume reading a file after an error which occurred
#          - during a file read

      subroutine osresume(nrec, fname)
      character*(*) fname
      integer utgetcl, i, k, idummy, ospush
      external utgetcl, ospush

Use(Controls)
Use(Infiles)

      lfname = utgetcl(fname)
      if(istko <= istk) then
# not really a resume, it is a read and start at line
        nskipr = max(nrec - 1, 0)
        idummy = ospush(fname(1:lfname)," ",0)
        return
      endif
      i = istko
# Check if no fname on command line.
      if(fname <> ' ') then
# Check for invalid fname.
        do i = istk+1, istko
          if(fname(1:lfname) == fnameso(i)) go to 10
          k = index(fnameso(i),fname(1:lfname))
          if( k > 1) then
             if(fnameso(i)(k:)=fname(1:lfname)) go to 10
          endif
        enddo
# not really a resume, it is a read and start at line
        nskipr = max(nrec - 1, 0)
        idummy = ospush(fname(1:lfname)," ",0)
        return
      endif
   10 jstk = istk
      istko = i
# Reset file stack.
      do i = istk+1, istko-1
        fnames(i) = fnameso(i)
        nskipr = nextlino(i) - 1
        idummy = ospush(fnames(i), " ", 0)
        nextlin(i) = nextlino(i)
      enddo
      if(nrec > 0) then
        nskipr = nrec - 1
      else
        nskipr = nextlino(istko) - 2
      endif
      idummy = ospush(fnameso(istko), " ", 0)
      do i = jstk+1, istko-1
        pblin(i) = pblino(i)
        pblinl(i) = pblinlo(i)
      enddo
      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

      subroutine basflush(iunit)
      integer iunit

ifelse(SYSTEM,RS6000,[define flush ifelse([$1],,flush_,flush_($1))])
ifelse(SYSTEM,HP700,,[external flush])
      call flush(iunit)

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

# create file for send edits to a file.

        subroutine outputto(s1,s2)

Use(IO)
        character*80 timedate, gtimdat
        external gtimdat
        character*(*) s1, s2
        integer basopen, utgetcl
        external basopen, utgetcl
        Filename fname

        if( s1 == "to") then
           fname = s2
        else
           fname = s1
        endif

        if(fname == " ") fname = "tty"
        if(fname == "terminal") fname = "tty"
        if(fname == outpfile) then
                call remark("outputto warning: already output is to ")
                call remark(fname(1:utgetcl(fname)))
                return
        endif

        if(iusout <> STDOUT & iusout <> 100) then
                call basclose(iusout)
               iusout = -1
                outpfile = "tty"
        endif
        iusout = STDOUT

        if(fname == "graphics") then
                iusout = 100
        elseif(fname == "tty") then
                iusout = STDOUT
        else
                iusout=basopen(fname,"w")
                if(iusout==ERR) then
                   call baserror("outputto: couldn't create ")
                   call baserror(fname(1:utgetcl(fname)))
                   call kaboom(0)
                   return
                endif
                timedate = gtimdat()
# (FNF 8/04/97) Changed "write(iusout," to "call baspline(iusout,".
                call baspline(iusout, timedate(1:utgetcl(timedate)))
        endif
        outpfile = fname

        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

# display group whose name is in stack element si
# ss(N,rsp) = package number or 0
# si(TC,rsp) = - number of characters in name
# (ZCM 8/11/1999) sp is no longer an argument. Not needed with separate stacks.

        function pardodig()
        integer pardodig
        Varname grpnam
        character*PNSIZE pkg
        integer ngrp,npkg,n
        integer edit, utgetcl, parname
        external edit, utgetcl, parname

Use(Stacks)
Use(IO)

        ngrp = parname(ss(1,rsp), grpnam)
        n = ss(N,rsp)
        call parkill(ss(1,rsp))
        if(n == 0) then
                return( edit(iusout,grpnam(1:ngrp)))
        else
                call glbpknam(n,pkg)
                npkg = utgetcl(pkg)
                return( edit(iusout,pkg(1:npkg)//"."//grpnam(1:ngrp)))
        endif
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

#       Altered by ZCM 7/91 (moved database references to rtd)

        subroutine parsetuf(value)
        integer value #actually logical, but want to work if civic calls

Use(Controls)

        if(value==0) then
           usrfirst=FALSE
        else
           usrfirst=TRUE
        endif

        if(usrfirst) then
                call rtsetuf (2) #will use prihigh value from Database
        else
                call rtsetuf (1)
        endif

        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

        subroutine setiotab (us, val)
        integer us, val

Use(IO)

        iotable(us) = val

        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

        subroutine setiotcl (fd, val)
        integer fd
        logical val

Use(IO)

        iotabcl(fd) = val

        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

        integer function getius ()

Use(IO)

        return (iusout)
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

        subroutine setius (val)
        integer val

Use(IO)

        iusout = val

        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

        Filename function getoutp ()

Use(IO)

        return (outpfile)
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

        subroutine setoutp (fname)
        Filename fname

Use(IO) 

        outpfile = fname

        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

        subroutine setactf (fname)
        character *(*) fname

Use(IO)

        actualf = fname

        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

