c
c FORMAT.M - routines to write numbers into strings
c

c--------------------------------------------------------------------------

c                      NUMBER TO STRING ROUTINES

c--------------------------------------------------------------------------

c PRASTR - format N Addresses in VAR into string DST

      subroutine prastr(dst, type, iof, id, n, var)
      implicit none
      character*(*) dst, type
      integer iof, id(*), n
      Address var(n)

      integer i, last, nc, ndc, mew, ieg, eiw, sio, fw
      character*24 ei
      character*2 fmt

      integer utgetcl
      external utgetcl

      mew  = id(iof+1)
      ieg = id(13)
      sio = id(14)
      ndc = len(dst)
      last = 0

      fmt = '%'//type
    
      do i = 1, n
         if (i == n) last = 1
         eiw = len(ei)
         call utsprint(1, 1, eiw, ei, 2, fmt, var(i))
	 eiw = utgetcl(ei)
	 fw = ieg + mew	                # constant field width
	 if (sio == 1) fw = ieg + eiw	# or variable field width
         call utsprint(0, last, ndc, dst, 3, '%*s', fw, ei)
      enddo

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PRCHSTR - format N characters in VAR into string DST

      subroutine prchstr(dst, type, iof, id, n, var)
      implicit none
      character*(*) dst, type
      integer iof, id(*), n
      # GOTCHA - the width is required because a character is not passed in
      character*(128) var

      integer ln, i, j, last, nc, ndc, ns, fw, c, n1, n2, ieg
      character*OUTLINE fmt
      character*10 nfc, mfc
      character*MAXSTRING s, blnk
Use(Controls)

      integer utgetcl
      external utgetcl

      ln  = -id(2)
      ns  = id(9)
      fw  = id(iof+1)
      ieg = id(13)
      fmt = " "
      blnk = " "

      ndc  = len(dst)
      last = 0

c ... handle decimal
      if (type .eq. "d") then
         do i = 1, n
            j = 1 + (i-1)*ln
            s = var(j:j+ln-1)
#           s(ln+1:ln+1) = char(0)
            if (i .eq. n) then
               last = 1
            endif
            if (ns .eq. 1) then
               fmt = '  "%-*s"'
            else
               fmt = '%-*s'
            endif
            nc = utgetcl(fmt)
            n1 = ln
            if(autotrod) n1 = utgetcl(s(1:ln))
            call utsprint(0, last, ndc, dst, nc, fmt, n1, s)
            if (autotrod .and. i .lt. n) then
               ! blank fill after string
               n2 = ln - n1
               call utsprint(0, last, ndc, dst, 3, '%*s', n2, blnk)
            endif
         enddo

c ... handle octal and hex differently
      else
         call utsprint(1, 1, OUTLINE, fmt, 6, "%%%d%c", ieg+fw, type)
         nc = utgetcl(fmt)

         do i = 1, n
            j = 1 + (i-1)*ln
            s = var(j:j+ln)
            do j = 1, ln
               if ((i .eq. n) .and. (j .eq. ln)) then
                  last = 1
               endif
               c = ichar(s(j:j))
               call utsprint(0, last, ndc, dst, nc, fmt, c)
            enddo
         enddo
      endif

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

# Glossary of terms
# Element: what gets formatted by one pass through the do-loop:
#    A) one integer, logical, address, real, or double;
#    B) or one range specifier;
#    C) or one complex number (two reals or doubles, comma-separated).
# mew : maximum element width, as set by utfmtdesc
# eiw : width of the i'th element.  May vary per-element.
# ei  : the i'th element
# ieg : inter-element gap
# sio : flag indicating whether this is a streamio operation
# fw  : overall width of the field to print one element, including the ieg

c PRISTR - format N integers in VAR into string DST

      subroutine pristr(dst, type, iof, id, n, var)
      implicit none
      character*(*) dst, type
      integer iof, id(*), n
      integer var(n)

      integer i, last, nc, ndc, mew, ieg, eiw, sio, fw
      character*24 ei
      character*2 fmt

      integer utgetcl
      external utgetcl

      mew  = id(iof+1)
      ieg = id(13)
      sio = id(14)
      ndc = len(dst)
      last = 0

      fmt = '%'//type

      do i = 1, n
         if (i == n) last = 1
         eiw = len(ei)
         call utsprint(1, 1, eiw, ei, 2, fmt, var(i))
	 eiw = utgetcl(ei)
	 fw = ieg + mew	                # constant field width
	 if (sio == 1) fw = ieg + eiw	# or variable field width
         call utsprint(0, last, ndc, dst, 3, '%*s', fw, ei)
      enddo

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PRLSTR - format N logicals in VAR into string DST

      subroutine prlstr(dst, type, iof, id, n, var)
      implicit none
      character*(*) dst, type
      integer iof, id(*), n
      logical var(n)

      integer i, last, ndc, ieg, mew
      character*8 c

      integer utgetcl
      external utgetcl

      mew = id(iof+1)
      ieg = id(13)

      ndc  = len(dst)
      last = 0
      do i = 1, n
         if (i == n) last = 1
         if (type == "d") then
	    c = "F"
            if (var(i)) c = "T"
         else
	    c = "0"
            if (var(i)) c = "1"
         endif
	 c(2:2) = char(0)
         call utsprint(0, last, ndc, dst, 3, "%*s", ieg+mew, c(1:1))
      enddo

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PRRNGSTR - format N ranges in VAR into string DST

      subroutine prrngstr(dst, type, iof, id, n, var)
      implicit none
      character*(*) dst, type
      integer iof, id(*), n
      integer var(n)

      integer i, last, nc, ndc, ieg
      integer i1, i2, i3
      character*OUTLINE fmt

      integer utgetcl
      external utgetcl

      fmt = " "
      ndc = len(fmt)
      ieg = id(13)
      call utsprint(1, 1, ndc, fmt, 20, "%%%d%c:%%%d%c:%%%d%c",
     &              ieg+id(iof+1), type, id(iof+2), type, id(iof+3), type)
      nc = utgetcl(fmt)

      ndc  = len(dst)
      last = 0
      do i = 1, n
         if (i .eq. n) then
            last = 1
         endif
         i1 = var(3*i-2)
         i2 = var(3*i-1)
         i3 = var(3*i)
         call utsprint(0, last, ndc, dst, nc, fmt, i1, i2, i3)
      enddo

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PRRSTR - format N float/reals in VAR into string DST

      subroutine prrstr(dst, type, iof, id, n, var)
      implicit none
      character*(*) dst, type
      integer iof, id(*), n
      real var(n)

      integer i, last, ndc, fw, dw, ieg, mew, sio, eiw
      character*48 ei
      character*5 fmt

      integer utgetcl
      external utgetcl

      mew = id(iof+1)
      dw  = id(iof+2)
      ieg = id(13)
      sio = id(14)

      if ((type .eq. "o") .or. (type .eq. "X")) then
         fmt = '%'//type
      else
         fmt = '%*.*'//type
      endif

      ndc  = len(dst)
      last = 0
      do i = 1, n
         if (i == n) last = 1
	 eiw = len(ei)
         if ((type .eq. "o") .or. (type .eq. "X")) then
            call utsprint(1, 1, eiw, ei, 2, fmt, var(i))
         else
            call utsprint(1, 1, eiw, ei, 5, fmt, mew, dw, var(i))
         endif
	 eiw = utgetcl(ei)
	 fw = ieg + mew
	 if (sio == 1) fw = ieg + eiw
         call utsprint(0, last, ndc, dst, 3, '%*s', fw, ei)
      enddo

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PRDSTR - format N doubles in VAR into string DST

      subroutine prdstr(dst, type, iof, id, n, var)
      implicit none
      character*(*) dst, type
      integer iof, id(*), n
      double precision var(n)

      integer i, last, nc, ndc, fw, dw, i1, i2, ieg, sio, eiw, mew, fmtw
      integer idv(2)
      double precision v1
      character *24 fmt
      character *48 ei
      character*5 tbuffer

      integer utgetcl
      external utgetcl

      if (id(8) .eq. 1) then
	 i1 = 1
	 i2 = 2
      else
	 i1 = 2
	 i2 = 1
      endif

      mew = id(iof+1)
      dw  = id(iof+2)
      ieg = id(13)
      sio = id(14)

      ndc  = len(dst)
      last = 0
      do i = 1, n
         if (i == n) last = 1
	 eiw = len(ei)
         v1 = var(i)
         if ((type .eq. "o") .or. (type .eq. "X")) then
            call mvalue(loc(v1), 1, INTEGER, idv, 2)
            fmtw = len(fmt)
            call utsprint(1, 1, fmtw, fmt, 13, "%%%d%c %%%d%c",
     &                 mew, type, mew, type)
	    fmtw = utgetcl(fmt)
	    call utsprint(1, 1, eiw, ei, fmtw, fmt, idv(i1), idv(i2))
         else
# GOTCHA : zero fault complains about a Bad Memory Write without tbuffer.
	    tbuffer = '%*.*'//type
	    call utsprint(1, 1, eiw, ei, len(tbuffer), tbuffer, mew, dw, v1)
         endif
	 eiw = utgetcl(ei)
	 fw = ieg + id(4)
	 if (sio == 1) fw = ieg + eiw
	 call utsprint(0, last, ndc, dst, 3, '%*s', fw, ei)
      enddo

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PRCSTR - format N complex numbers in VAR into string DST

      subroutine prcstr(dst, type, iof, id, n, var)
      implicit none
      character*(*) dst, type
      integer iof, id(*), n
      complex var(n)

      integer i, last, nc, ndc, fw, dw, ieg, mew, sio, eiw, fmtw
      integer idv(2)
      complex v1
      real vr, vi
      character*48 fmt
      character*48 ei

      integer utgetcl
      external utgetcl

      mew = id(iof+1)
      dw  = id(iof+2)
      ieg = id(13)
      sio = id(14)
      fmt = " "

      ndc  = len(dst)
      last = 0
      do i = 1, n
         if (i == n) last = 1
	 eiw = len(ei)
	 fmtw = len(fmt)
         if ((type .eq. "o") .or. (type .eq. "X")) then
            call mvalue(loc(var(i)), 1, INTEGER, idv, 2)
            call utsprint(1, 1, fmtw, fmt,
     &         13, "%%%d%c,%%%d%c", mew, type, mew, type)
	    fmtw = utgetcl(fmt)
	    call utsprint(1, 1, eiw, ei, fmtw, fmt, idv(1), idv(2))
         else
            v1 = var(i)
            vr = real(v1)
            vi = aimag(v1)
	    fmt = '%*.*' // type // ',%*.*' // type
	    fmtw = utgetcl(fmt)
	    call utsprint(1, 1, eiw, ei, fmtw, fmt, mew, dw, vr, mew, dw, vi)
         endif
	 eiw = utgetcl(ei)
	 fw = ieg + id(4)
	 if (sio == 1) fw = ieg + eiw
	 call utsprint(0, last, ndc, dst, 3, '%*s', fw, ei)
      enddo

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PRDCSTR - format N double precision complex numbers in VAR into string DST

      subroutine prdcstr(dst, type, iof, id, n, var)
      implicit none
      character*(*) dst, type
      integer iof, id(*), n
      double complex var(n)

ifelse(HAS_DOUBLE_COMPLEX,YES,[

      integer i, last, ndc, fw, dw, i1, i2, i3, i4, ieg
      integer mew, sio, eiw, fmtw
      integer idv(4)
      double complex v1
      double precision vr, vi
      character*48 fmt
      character*96 ei

      integer utgetcl
      external utgetcl

      if (id(8) .eq. 1) then
	 i1 = 1
	 i2 = 2
	 i3 = 3
	 i4 = 4
      else
	 i1 = 2
	 i2 = 1
	 i3 = 4
	 i4 = 3
      endif

      mew = id(iof+1)
      dw  = id(iof+2)
      ieg = id(13)
      sio = id(14)
      fmt = " "

      ndc  = len(dst)
      last = 0
      do i = 1, n
         if (i == n) last = 1
	 eiw = len(ei)
	 fmtw = len(fmt)
         if ((type .eq. "o") .or. (type .eq. "X")) then
            call mvalue(loc(var(i)), 1, INTEGER, idv, 4)
            call utsprint(1, 1, fmtw, fmt,
     &                 27, "%%%d%c %%%d%c,%%%d%c %%%d%c",
     &                 mew, type, mew, type, mew, type, mew, type);
	    fmtw = utgetcl(fmt)
            call utsprint(1, 1, eiw, ei, fmtw, fmt,
     &                    idv(i1), idv(i2), idv(i3), idv(i4))
         else
	    v1 = var(i)
            vr = dble(v1)
            vi = dimag(v1)
	    fmt = '%*.*' // type // ',%*.*' // type
	    fmtw = utgetcl(fmt)
	    call utsprint(1, 1, eiw, ei, fmtw, fmt, mew, dw, vr, mew, dw, vi)
         endif
	 eiw = utgetcl(ei)
	 fw = ieg + id(4)
	 if (sio == 1) fw = ieg + eiw
	 call utsprint(0, last, ndc, dst, 3, '%*s', fw, ei)
      enddo

])

      return
      end

c--------------------------------------------------------------------------

c                               OTHERS

c--------------------------------------------------------------------------

        function itostr(i)
        character*(*) itostr
        integer i

        character*24 nch

        integer imkstr
        external imkstr

        i = imkstr(i,nch)

        return(nch(1:i))
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

#     NAME OF ROUTINE...imkstr
#
#     AUTHOR OF ROUTINE...Paul Dubois 9/14/84
#
#     PURPOSE...
#
#        convert k to characters, store in str...return length occupied
#
#     GLOSSARY OF IMPORTANT VARIABLES...
#
#        k...the integer to be converted
#        str...the string in which to store the result
#        imkstr...return number of characters

        integer function imkstr(k, str)
        integer k
        character*(*) str

        integer i
        character*(24) nch

        write(nch, 10) k
10      format(i24)

        i = 1
        call utskipbl(nch, i)

        str = nch(i:)

        return(24-i+1)
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

        integer function amkstr(a, str)
#       Same as above, except a is an Address
        Address a
        character*(*) str
        character*(24) nch
        integer i

        write(nch, 10) a
10	format(i24)

	i = 1
	call utskipbl(nch, i)

	str = nch(i:)

	return(24-i+1)
	end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------
