/*
 * FUNC.C
 *
 */

#include "rt.h"

#define clooptyp ( (Integer *) looptyp)

Function_Stack *funcstk;        /* stack        */
NameType *funpar;               /* array of names & types */
Integer funmax = 0;

int funcdep;                    /* stack pointer */

/* depth of function nesting at compile time--used to inhibit execution: */
int cfuncdep;
int fund = 0;                   /* dynamic size of funcstk */

/* compile time nesting depth: */
int fdepth = 0;

/* buffer where text of function is accumulated while parsing: */
char *functext;
Integer textsize = MINFUNC;     /* initial size of function text buffer */
Integer textpos = 0;            /* initial position in text buffer      */
logical linsuprs = (logical) 0; /* whether or not to suppress a LINE opcode */

/* whether to save function text in lxgchr or not (set false on
 * the first parse of a command string) (ZCM 9/12/94):                   */
logical lxsave = (logical) 1;

/* number of characters of delimiter to skip after a command
 * string argument when putting text in a function (ZCM 9/13/94):        */
Integer lxskip = 0;
logical firstf = (logical) 0;   /* set TRUE at start of function parse  */

/* used when compiled function calls a user function--signals lrexec
 * to execute even though nestlevl might not be 0:                       */
Integer xuser;
Integer tickflag;               /* true => inside a `` expression (Z 8/97) */
Integer tclose;                 /* set when close tick is seen (Z 8/97)    */
Integer tickpos;                /* position in code on first ` (Z 8/97)    */
Integer *tb;
char *nb;

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

Address Getic(void)
{
    return ic;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

void Setic(Address icset)
{
    ic = icset;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

Address Getnb(void)
{
    return nb;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

void Setnb(Address nbset)
{
    nb = nbset;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

Address Gettb(void)
{
    return tb;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

Integer Gfuncdep(void)
{
    return funcdep;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

void Settb(Address tbset)
{
    tb = tbset;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

void Ftxtmark(Integer linelen)
{
    if (fdepth > 0) {
        ftextend(fdepth - 1) = linelen;
        ftextbeg(fdepth - 1) = 0;
    }
    lineno += 1;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 * at this point, we have a text character. put it into the
 * saved function text if we are parsing through a function.
 */
void Ftxtput(Integer a)
{
    if (fdepth > 0 && a == STRINGMARK)
        lxsave = !lxsave;
    if (fdepth > 0 && lxsave && a != STRINGMARK && lxskip == 0) {
        if (textpos + 1 >= textsize) {
            /* double text buffer size */
            Oschangn((Address *) &functext, 2 * textsize, textsize,
                     "function text");
            textsize *= 2;
        }
        functext[textpos++] = (char) a;
    } else if (a != STRINGMARK && lxsave && lxskip > 0)
        lxskip -= 1;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Insert a line number if we're in a function */

void Ftxtline(void)
{
    if (fdepth > 0) {
        if (!linsuprs) {
            ftextlno(fdepth - 1) += 1;
            Lremit(LINE);
            Lremit(ftextlno(fdepth - 1));
        } else
            linsuprs = (logical) 0;
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* if we are parsing function text, move the text pointer back one
 * don't back off text pointer if lxsave is false
 * ZCM 9/12/94
 */

void Ftxtback(void)
{
    if (fdepth > 0 && lxsave)
        textpos -= 1;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

Integer Gtextpos(Integer toklen)
{
    if (fdepth > 0 && lxsave)
        return textpos - toklen;
    else
        return textpos;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

Integer Gfdepth(void)
{
    return fdepth;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

void Stextpos(Integer value)
{
    if (fdepth > 0)
        textpos = value;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

void Slxskip(Integer value)
{
    if (fdepth > 0)
        lxskip = value;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

void Fstkinit(void)
{
    Integer size;

    funcdep = 0;
    cfuncdep = 0;
    fund = FUNCDEP + 1;
    size = sizeof(Function_Stack) / sizeof(Integer) +
        (sizeof(Function_Stack) % sizeof(Integer) ? 1 : 0);
    Osallotn((Address *) &funcstk, size * fund, "PERM|FSTKINIT:fncstk");
    spfn(0) = 0;
    retadd(0) = 1;
    Nextcode(0) = 1;
    locdepth(0) = 0;
    linenumber(0) = 0;
    retbgn(0) = intcode;
    namebgn(0) = funpar;
    typebgn(0) = funpar;
    firstf = 0;
    linsuprs = 0;
    textpos = 0;
    textsize = MINFUNC;
    size =
        textsize / sizeof(Integer) + (textsize % sizeof(Integer) ? 1 : 0);
    Osallotn((Address *) &functext, size, "PERM|FSTKINIT:functext");
    fdepth = 0;
    lxsave = (logical) 1;
    lxskip = (logical) 0;
    xuser = 0;
    tickflag = 0;
    tclose = 0;
    tickpos = 0;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

void Deccfdep(void)
{
    cfuncdep--;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

void Fstkpsh(Integer s, Integer r1, Integer l2, Address t)
{
    Integer size;

    funcdep += 1;
    if (funcdep > fund) {
        fund += FUNCDEP;
        size = sizeof(Function_Stack) / sizeof(Integer) +
            (sizeof(Function_Stack) % sizeof(Integer) ? 1 : 0);
        Oschangn((Address *) &funcstk, fund * size,
                 (fund - FUNCDEP) * size, "function stack");
    }
    spfn(funcdep) = s;
    retadd(funcdep) = r1;
    Nextcode(funcdep) = nextcode;
    locdepth(funcdep) = Rtgloc();
    linenumber(funcdep) = l2;
    retbgn(funcdep) = ic;
    namebgn(funcdep) = nb;
    typebgn(funcdep) = t;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *      Programmed by Zane C. Motteler;   revised 1/87 Dubois
 *              September, 1986
 *      Purpose: To execute the RET instruction (return from user function)
 *      Revised Zane C. Motteler
 *              July, 1989
 *      Changes in funcstk due to elimination of token arrays
 *      Revised Zane C. Motteler
 *              July, 1991
 *      Moved all Database references to rtd.
 *
 *      Revised Zane C. Motteler
 *              August 1999
 *      No longer need sp parameter since runtime stack is now separate.
 */
Integer Lrxret(Integer *pcnext)
{
    static char msg[] = "Error: return statement outside function";

    if (funcdep == 0) {
        BasError(msg, strlen(msg));
        return ERR;
    }
    /* replace function name on stack with value */
    Parcopy(&(ssl(1, rsp)), &(ssl(1, rlinkp - 1)));

    /* (set pc to new location counter value and */
    /* set all bases to calling values)          */
    *pcnext = retadd(funcdep);
    ic = retbgn(funcdep);
    nb = namebgn(funcdep);
    tb = typebgn(funcdep);
    nextcode = Nextcode(funcdep);
    Rtsloc(locdepth(funcdep));
    /* remove stack frame */
    Rtpoploc();
    /* pop funcstk */
    funcdep -= 1;
    /* calculate runtime nesting level (RETURN could be nested */
    /* inside structured statements in the function)           */

    while (clooptyp[nestlevl - 1] != FUNCTION_TYPE)
        nestlevl--;
    nestlevl--;
    return (OK);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *      Programmed by Zane C. Motteler
 *              August, 1986
 *      Purpose: checks to make sure stacks are adequate, and if not,
 *              allocates more space for them, up to a ridiculous
 *              limit.
 *      Revised by Zane C. Motteler
 *              July, 1989
 *      No more dynamic space for token text
 *      Bug fix by Zane C. Motteler
 *              December, 1990
 *      Purpose: If intcode is reallocated, and a pointer to it has been
 *              saved on funcstk, make sure that pointer is changed too.
 */
Integer Ckstacks(void)
{
    static char
        msg1[] = "ckstacks: too much code space: a megaword.",
        msg2[] = "ckstacks: nesting too deep: 20000. Tight loop.";
    int i;
    Address oldcode;

    /* check code space */
    if (nextcode > mxcode - 20) {
        if (mxcode > 1000000) {
            BasError(msg1, strlen(msg1));
            return (ERR);
        }
        oldcode = intcode;
        mxcode += MAXCODE;
        Oschangn((Address *) &intcode, mxcode, mxcode - MAXCODE,
                 "intcode block");
        /* Don't leave any old pointers around */
        if (funcdep >= 1)
            for (i = 1; i <= funcdep; i++)
                if (retbgn(i) == oldcode)
                    retbgn(i) = intcode;
    }
    /* check label and for-stack space */
    if (lbstptr > mdl - 5 || forstptr > mdl - 5) {
        if (mdl > 20000) {
            BasError(msg2, strlen(msg2));
            return ERR;
        }
        mdl += MD;
        Oschangn((Address *) &lbstack, mdl, mdl - MD, "label stack");
        Oschangn((Address *) &forstck, mdl, mdl - MD, "for stack");
    }
    /* check nesting and loop space */
    if (nestlevl > mxdep - 5 || lpdepth > mxdep - 5) {
        if (mxdep > 20000) {
            BasError(msg2, strlen(msg2));
            return ERR;
        }
        mxdep += MAXDEP;
        Oschangn((Address *) &looptyp, mxdep, mxdep - MAXDEP, "loop type");
        Oschangn((Address *) &loopctl, mxdep, mxdep - MAXDEP,
                 "loop control");
        Oschangn((Address *) &fnfwa, mxdep, mxdep - MAXDEP,
                 "function fwa");
        Oschangn((Address *) &argfwa, mxdep, mxdep - MAXDEP,
                 "argument fwa");
        Oschangn((Address *) &arglen, mxdep, mxdep - MAXDEP,
                 "argument length");
        Oschangn((Address *) &lpnext, mxdep, mxdep - MAXDEP, "loop next");
    }
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

Integer Fstkgeti(Integer comp, Integer depth)
{
    switch (comp) {
    case SPFN:
        return spfn(depth);
    case RETADD:
        return retadd(depth);
    case NEXTCODE:
        return Nextcode(depth);
    case LOCDEPTH:
        return locdepth(depth);
    case LINENO:
        return linenumber(depth);
    }
    return (-1);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

Address Fstkgetp(Integer comp, Integer depth)
{
    switch (comp) {
    case RETBGN:
        return retbgn(depth);
    case NAMEBGN:
        return namebgn(depth);
    case TYPEBGN:
        return typebgn(depth);
    }
    return (NULL);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

void Lrdoline(Integer *pc, Integer *pcnext)
{
    if (funcdep > 0)
        linenumber(funcdep) = ic[*pc];
    if (baslit)
        Basltest(ic[*pc]);
    *pcnext += 1;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Save current environment before calling a user function via
 * execuser. Moved here from execuser to isolate references
 * to funcstk. (Zane C. Motteler 1/24/2000)
 */
void Fstksav(char *uname)
{
    Integer size;

    funcdep += 1;
    if (funcdep > fund) {
        fund += FUNCDEP;
        size = sizeof(Function_Stack) / sizeof(Integer) +
            (sizeof(Function_Stack) % sizeof(Integer) ? 1 : 0);
        Oschangn((Address *) funcstk, fund * size, (fund - FUNCDEP) * size,
                 "function stack");
    }
    spfn(funcdep) = -1;
    retbgn(funcdep) = ic;
    namebgn(funcdep) = nb;
    locdepth(funcdep) = Rtgloc();
    /* Save environment of the code that's currently executing in intcode block */
    Rtsloc(0);
    Lremit(codebgn);
    Lremit(nestlevl);
    Lremit(assignsp);
    if (inassign)
        Lremit(1);
    else
        Lremit(2);
    Lremit(xuser);
    codebgn = nextcode;
    /* Now set up a function call in intcode */
    Lremit(-100);               /* name */
    Lremit(Mcname());
    Lremit(StrgFind(uname));
    Lremit(1);                  /* ID -> LHS      */
    Lremit(136);                /* LHS -> FACTOR  */
    retadd(funcdep) = nextcode;
    Nextcode(funcdep) = nextcode;
    xuser = nestlevl;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Restore the current environment after calling a user function via
 * execuser. Moved here from execuser to isolate references
 * to funcstk.
 * (Zane C. Motteler 1/24/2000)
 */
void Fstkrst(void)
{
    /* Restore the environment to what it was */
    nextcode = codebgn - 5;
    codebgn = intcode[nextcode - 1];
    Rtpoploc();
    nestlevl = intcode[nextcode];
    assignsp = intcode[nextcode + 1];
    inassign = (logical) (intcode[nextcode + 2] == 1);
    xuser = intcode[nextcode + 3];
    execflag = (logical) 1;
    ic = retbgn(funcdep);
    nb = namebgn(funcdep);
    tb = typebgn(funcdep);
    Rtsloc(locdepth(funcdep));
    funcdep -= 1;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *      Name of Routine: parestor
 *      Author: Zane C. Motteler
 *      Date: September, 1987
 * 
 *      Purpose: This routine restores the environment after a single
 *              parser call, or after a series of calls, if the user wants
 *              to parse a string that is too long for parstr, in multiple
 *              calls to lrpars.
 *      Revised: Zane C. Motteler
 *      Date: July, 1989
 *              funcstk changes due to elimination of token arrays
 *       Revised: Zane C. Motteler
 *       Date: January, 2000
 *               Translated to C and moved here to isolate funcstk refs.
 */
void Parestor(void)
{
    nextcode = Nextcode(funcdep);
    codebgn = intcode[nextcode - 1];
    Rtpoploc();
    nestlevl = intcode[nextcode];
    assignsp = intcode[nextcode + 1];
    inassign = (logical) (intcode[nextcode + 2] == 1);
    xuser = intcode[nextcode + 3];
    execflag = (logical) 1;
    ic = retbgn(funcdep);
    nb = namebgn(funcdep);
    tb = typebgn(funcdep);
    Rtsloc(locdepth(funcdep));
    funcdep -= 1;
    nestlevl -= 1;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *      Name of Routine: parsetup
 *      Author: Zane C. Motteler
 *              September, 1987
 *      Purpose: This routine saves the environment prior to one or a
 *              series of calls to lrpars. Lrpars parses a string, or a
 *              series of substrings. Then parestor must be called to
 *              put everything right again.
 *      Revised: Zane C. Motteler
 *      Date: July, 1989
 *              funcstk changes due to elimination of token arrays
 *
 *      Revised 7/91 (ZCM) Database references -> rtd
 *       Revised 1/2000 (ZCM) Rewritten in C to isolate funcstk refs
 */
Integer Parsetup(void)
{
    Integer size;
    static Integer parstr = PARSTR;

    /* Create a dummy local database for parstr and also nest it */
    Rtgrpe(LOCALDB, "Parstr");
    Parnest(&parstr);
    funcdep += 1;
    if (funcdep > fund) {
        fund += FUNCDEP;
        size = sizeof(Function_Stack) / sizeof(Integer) +
            (sizeof(Function_Stack) % sizeof(Integer) ? 1 : 0);
        Oschangn((Address *) funcstk, fund * size, (fund - FUNCDEP) * size,
                 "function stack");
    }
    spfn(funcdep) = -2;         /* signal to paroops & debug */
    retadd(funcdep) = nextcode + 1;     /* (8/13/1999[ZCM] changed from 2 */
    retbgn(funcdep) = ic;
    namebgn(funcdep) = nb;
    typebgn(funcdep) = tb;
    Nextcode(funcdep) = nextcode;
    locdepth(funcdep) = Rtgloc();
    /*  Save runtime environment in intcode block */
    Lremit(codebgn);
    Lremit(nestlevl);
    Lremit(assignsp);
    if (inassign)
        Lremit(1);
    else
        Lremit(0);
    Lremit(xuser);
    codebgn = nextcode;
    xuser = nestlevl;
    return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *      pick up function values and store in database (ZCM, 7/91)
 *      allocate space for text, move it, save pointer (ZCM, 8/93)
 *      ignore anything in buffer after "endf" (ZCM, 1/94)
 *      if no endf, save buffer less 1 character (ZCM, 1/94)
 */
void Rtufcn(Address *fwa1, Integer *i2, Integer *i3, Integer *i4,
            Integer *i5, Address *fwa6, Integer *i8, Address *fwa9,
            logical hastext)
{
    char *text;
    Integer textstrt, oldi8, size;
    logical done;

    *fwa1 = ufcnstrt;
    *i2 = ufcnptr;
    *i3 = umnargs;
    *i4 = ufcnargc;
    *i5 = ufcnlen;
    *fwa6 = ufcnname;
    if (fdepth > 0 && hastext) {
        /* we are parsing function text in input */
        *i8 = textpos - ftextptr(fdepth - 1);
        /* Move end of text pointer back to last 'endf' */
        done = (logical) 0;
        textstrt = ftextptr(fdepth - 1);
        while (!done) {
            /* move backwards till we find an "f" */
            oldi8 = *i8 - 1;
            while (functext[textstrt + *i8] != 'f' &&
                   functext[textstrt + *i8] != 'F') {
                *i8 -= 1;
                /* Gone too far? (should never happen) */
                if (*i8 <= 13)
                    break;
            }
            if (*i8 <= 13) {
                *i8 = oldi8;
                break;
            }
            /* is it preceded by "end"? */
            done = (functext[textstrt + *i8 - 3] == 'e' &&
                    functext[textstrt + *i8 - 2] == 'n' &&
                    functext[textstrt + *i8 - 1] == 'd') ||
                (functext[textstrt + *i8 - 3] == 'E' &&
                 functext[textstrt + *i8 - 2] == 'N' &&
                 functext[textstrt + *i8 - 1] == 'D');
            /* if not, keep looking */
            if (!done)
                *i8 -= 1;
        }
        *i8 += 1;
        size = *i8 / sizeof(Integer) + (*i8 % sizeof(Integer) ? 1 : 0);
        Osallotn((Address *) &text, size, "RTUFCN:text");
        *fwa9 = (Address) text;
        textpos = ftextptr(fdepth - 1);
        /* Move function text to permanent place */
        strncpy(text, functext + textpos, *i8);
        if (fdepth > 1) {
            firstf = (logical) 1;       /* in some cases a char has been pushed back */
            ftextbeg(fdepth - 2) = ftextbeg(fdepth - 1);
            ftextend(fdepth - 2) = linlen;
        }
        fdepth -= 1;
    } else {
        *i8 = 0;
        *fwa9 = (Address) 0;
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *      Name of function: LXBACK
 *      Argument: backtick, logical, TRUE if the current token
 *         is a backtick.
 *      Called from: lxmac, if a backtick is seen
 *                      lrpact, on reduction of ` <rterm> ` to <dname>
 *                      after reserved word function
 *      Author: Zane C. Motteler, 11-13 August 1997
 *      Purpose: To implement computed names for functions.
 *      Background: Computed names are allowed in other Basis
 *         contexts, and were requested as function names.
 *      Caveat: This goes to show how far we will go to
 *         satsfy our users. This routine is triggered by
 *         a lexical action (seeing a backtick) and a parser
 *         action (131) but actually causes code to execute
 *         in order to compute the function name. This is
 *         major heresy in an interpreter where every effort
 *         has been made to keep the lexical analysis and
 *         parsing actions separate from code execution.
 *      Implementation: The problem is that while a function
 *         is being parsed, its intermediate code is stored
 *         but not executed. Yet for a computed name to be
 *         evaluated, its code must be executed. I did this
 *         as follows:
 *              when the opening backtick is seen, mark the spot
 *              in the intermediate code where the code for
 *              generating the computed name will begin.
 *              when the closing backtick is seen, leave a
 *              souvenir (tclose).
 *              when action 131 is emitted and we are in a
 *              function name context, execute the intermediate
 *              code from the starting spot marked above to
 *              the end, to generate a NAME stack element. Then
 *              call lxvcons on the name's behalf so it will be
 *              recorded as the function's name.
 *      Restrictions:
 *         backticks may not be nested in function names.
 *         functions with computed names can't be declared
 *              inside a loop.
 *      Revision:
 *         (ZCM 13 August 1999)
 *         No longer uses the parser stack pointer, since
 *         the parser and runtime stacks have been de-linked.
 *       Revision:
 *          (ZCM 25 January 2000)
 *          Rewritten in C.
 */
Integer Lxback(logical backtick)
{
    Integer pctemp, nesttemp, Tokname, lname, iact, four;
    char fname[VARNAMESIZE + 1];

    if (!tclose && backtick) {
        tickflag = !tickflag;
        if (tickflag)
            /* mark the spot in the code where the expression begins.
             * The name has to be computed before evaluation can
             * continue.
             */
            tickpos = nextcode;
        else if (!tickflag)
            tclose = 1;
        return DONE;
    } else if (tclose && !backtick) {
        tclose = 0;
        /* execute the code to create the name
         * got to fool mother nature a bit to force it to execute.
         */
        pctemp = codebgn;
        codebgn = tickpos;
        nesttemp = nestlevl;
        nestlevl = xuser;
        if (Lrexec() == ERR)
            return ERR;
        nestlevl = nesttemp;
        codebgn = pctemp;
        four = 4;
        Lrvlup(&Tokname, "name", &four);

        INIT_BLANK_STRING(fname, VARNAMESIZE);

        lname = Parname(&(ssl(1, rsp)), fname);
        Parrel(&(ssl(1, rsp)));
        rsp -= 1;               /* pop the stack */
        iact = 100;
        return (Lxvcons(&Tokname, fname, &lname, &iact));
    } else
        return OK;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

logical Gtickflg(void)
{
    return tickflag;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *  save name in the funparn stack, type in the funpart stack
 *  permanent installation
 *  nb: only called for builtins and compiled, so true funparn and
 *  funpart are used, not dynamically created ones
 *  Changed: ZCM 9/1999 -- now called with nameindx rather than name
 *  so it can easily be called from C.
 *  Rewritten in C by Zane C. Motteler 2/8/2000
 */
Integer Funparin(Integer nameindx, Integer type)
{
    char name[VARNAMESIZE + 1];
    int i;
    Integer size;

    INIT_BLANK_STRING(name, VARNAMESIZE);

    StrgGet(nameindx, name);
    i = ufcnptr;
    ufcnptr += 1;
    if (i > funmax) {
        if (funmax == 0) {
            funmax = 500;
            size = 1 + sizeof(NameType) / sizeof(Integer) +
                (sizeof(NameType) % sizeof(Integer) ? 1 : 0);
            Osallotn((Address *) &funpar, funmax * size,
                     "PERM|FUNPARIN:funpar");
        } else {
            funmax += 200;
            size = 1 + sizeof(NameType) / sizeof(Integer) +
                (sizeof(NameType) % sizeof(Integer) ? 1 : 0);
            Oschangn((Address *) &funpar, funmax * size,
                     (funmax - 200) * size, "name/type block");
        }
    }
    strcpy(funparn(i - 1), name);
    funpart(i - 1) = type;
    return (i);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *  save name in the funparn stack, type in the funpart stack
 *  uses ufcnargc; does not reset ufcnptr
 *  since funparin / funpars can be called before parinit, must initialize
 *  stacks here if required
 *  called only for user functions during definition, so true funparn stack
 *  is used.
 *  Rewritten in C by Zane C. Motteler 2/8/2000
 */
void Funpars(char *name, Integer type)
{
    integer i;
    Integer size;

    ufcnargc += 1;
    i = ufcnptr + ufcnargc;
    if (i > funmax) {
        if (funmax == 0) {
            funmax = 500;
            size = sizeof(NameType) / sizeof(Integer) +
                (sizeof(NameType) % sizeof(Integer) ? 1 : 0);
            Osallotn((Address *) &funpar, funmax * size, "FUNPARS:funpar");
        } else {
            funmax += 200;
            size = sizeof(NameType) / sizeof(Integer) +
                (sizeof(NameType) % sizeof(Integer) ? 1 : 0);
            Oschangn((Address *) &funpar, funmax * size,
                     (funmax - 200) * size, "name/type block");
        }
    }
    strcpy(funparn(i - 1), name);
    funpart(i - 1) = type;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

Integer Gfunpart(Integer i)
{
    /* Access function for types */
    return funpart(i - 1);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

void Gfunparn(Integer i, char *vname)
{
    /* access function for names */
    Integer vlen;

    vlen = strlen(vname);
    if (vlen < VARNAMESIZE)
        strcpy(vname, funparn(i - 1));
    else
        strncpy(vname, funparn(i - 1), VARNAMESIZE - 1);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* General version of above: access name component in a NameType
 * array that is not permanent.
 */
void Gfuparn2(NameType * parn, Integer i, char *vname)
{
    Integer vlen;

    vlen = strlen(vname);
    if (vlen < VARNAMESIZE)
        strcpy(vname, parn[i - 1].names);
    else
        strncpy(vname, parn[i - 1].names, VARNAMESIZE - 1);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* access function for addresses of funpar entries */

Address Afunpar(Integer i)
{
    return ((Address) (funpar + i - 1));
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

 /*
  * Print out error message for j-th argument of a function.
  * Written by Zane C. Motteler 2/8/2000
  */
Integer Funcerr(Integer iptf, Integer j)
{
    char msg[100];

    sprintf(msg, "Function %s: argument %ld is wrong type.",
            funparn(iptf - 1), (long) j);
    BasError(msg, strlen(msg));
    return ERR;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
