/*
 * GLMMCHK.C - do global memory leak checking
 *
 */

#include "rt.h"

#if !defined(SOLARIS) && !defined(AIX)

static int
    glmmnb;

#endif

static int
    glmmfl;

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* GLMMINIT - initialize the global memory monitoring
 *          - at the very beginning of the session
 *          - this is intended for leak checking primarily
 */

void F77_ID(glmminit_, glmminit, GLMMINIT) (FIXNUM *pg) {
    char *p;

    glmmfl = *pg;

    if (glmmfl == 1) {
        printf("Starting Basis global memory monitoring\n");
        fflush(stdout);

        p = FMAKE(char, "GLMMINIT:p");

#if !defined(SOLARIS) && !defined(AIX)
        {
            char msg[MAXLINE];
	    int mo, dy, yr;

	    strcpy(msg, SC_version_string);
	    mo = SC_stoi(SC_firsttok(msg, "."));
	    dy = SC_stoi(SC_firsttok(msg, "."));
	    yr = SC_stoi(SC_firsttok(msg, "."));

/* for a version later than February 2001 use the new call */
            if ((0 < yr) && (yr <= 92) && (2 < mo))
	       glmmnb = SC_mem_monitor(-1, -2, "glob", msg);

/* for a version earlier use the old call */
	    else
	       glmmnb = SC_mem_monitor(0, -2, "glob", msg);
        }
#endif
    }

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* GLMMCHK - report on global memory at the very end of the session
 *         - this is intended for leak checking primarily
 */

void F77_ID(glmmchk_, glmmchk, GLMMCHK) (void) {

    if (glmmfl == 1) {
        Rtsetxit();

/* free all the user variables */
        Rtzapall();

/* free any remaining space associated with LOCALDB */
        Rtclean();

/* free all the internal interpreter variables */
        Rtkill();

/* cleanup any trace residue */
        Lrftrmk(NULL, 3, NULL);

        printf("Ending Basis global memory monitoring\n");

#if !defined(SOLARIS) && !defined(AIX)
        {
            char msg[MAXLINE];

            glmmnb = SC_mem_monitor(glmmnb, -2, "glob", msg);
        }
#endif
    }

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
