/*
 * IOTAB.C
 *
 * Outfiles group moved here from FORTRAN to facilitate handling Dynamics
 * (Zane C. Motteler 2/16/2000)
 */

#include "rt.h"

static Integer iotabn = 0;      /* Number of output files. */

typedef struct {
    Integer iotabr;             /* iotabr(i) is the number of times  *
                                 * the file iotabf(i) has been used. */
    char iotabf[FILENAMESIZE + 1];      /* Names of output files */
    char iotabc[COMMENTSIZE + 1];       /* Comments about output files. */
} IoTab;

static IoTab *iotab;

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* BASFILE -
 *       Register a name for an output file labeled tag when reported at end.
 *       Altered by Zane C. Motteler
 *       6 March 1995
 *       Purpose: Will not add a file to the list if it is already
 *                on the list. Keeps a count of the number of times
 *                a name has been used. This gives a much shorter
 *                printout at the end.
 *       Moved to C (ZCM 2/16/2000)
 */

Integer Basfile(char *f, char *tag)
{
    int i;
    Integer size, found;

    found = 0;
    for (i = 1; i <= iotabn; i++) {
        if (!strcmp(iotab[i - 1].iotabf, f)) {
            found = i;
            iotab[i - 1].iotabr += 1;
            break;
        }
    }
    if (!found) {
        size =
            sizeof(IoTab) / sizeof(Integer) +
            (sizeof(IoTab) % sizeof(Integer)
             ? 1 : 0);
        Oschangn((Address *) &iotab, (iotabn + 1) * size, iotabn * size,
                 "BASFILE:iotab");
        iotabn += 1;
        strcpy(iotab[iotabn - 1].iotabc, tag);
        strcpy(iotab[iotabn - 1].iotabf, f);
        iotab[iotabn - 1].iotabr = 1;
    }
    return found;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

void Osdeliotab(void)
{
    Osfreen((Address *) &iotab, "ODDELIOTAB:iotab");
    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* OSOUTFLS - inform user about the output files create by calls to outfile */

void Osoutfls(void)
{
    char msg[FILENAMESIZE + 1], times[15];
    int i, lenc, lenm;

    if (iotabn > 0) {
        sprintf(msg, "Output files:");
        Remark(msg, strlen(msg));
    }

    for (i = 1; i <= iotabn; i++) {
        strcpy(msg, iotab[i - 1].iotabf);

        if (strlen(msg) + 14 <= FILENAMESIZE) {
            if (iotab[i - 1].iotabr == 1)
                sprintf(times, " (1 time): ");
            else
                sprintf(times, " (%ld times): ",
                        (long) iotab[i - 1].iotabr);
            strcat(msg, times);
            lenm = strlen(msg);
            lenc = min(FILENAMESIZE - lenm, strlen(iotab[i - 1].iotabc));
            if (lenc >= 0) {
                strncat(msg, iotab[i - 1].iotabc, lenc);
                msg[lenm + lenc] = '\0';
            }
        }
        Remark(msg, strlen(msg));
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* GIOTABN - access to iotabn */

Integer Giotabn(void)
{
    return iotabn;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* GIOTABF - return the specified filename */

void Giotabf(Integer i, char *fn)
{
    strcpy(fn, iotab[i - 1].iotabf);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
