/*
 *LIT.C
 *
 */

#include "rt.h"

/* Literal tables moved here from group Literals in par.v */
Integer mxlit = 0;
Address *literal;
Integer *locks;
Integer killme[LASTTYPE + 2];
Integer litcnt[10], litcntc[10], lastcnt[10];

/* Space for literal constant tables */
Integer *values, *nextptr, *nextptd;
real8 *dvalues;
Integer valavail, valavaid, maxvals, maxvald;

/* N. B. FORTRAN indexing is from 1, so in most routines 1 is subtracted
 * from a subscript that comes in. killme used to be indexed beginning
 * with -1, so its subscript will always have 1 added to it. When a
 * literal table entry is found, its index + 1 is returned to FORTRAN.
 */

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* GETADDR - Return the address resident in the ith slot of the literal table. */

Address Getaddr(Integer i)
{
    return (literal[i - 1]);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* GETLOCK - Return the value of the lock on literal s(PTR). */

Integer Getlock(Integer *s)
{
    return (locks[s[PTR - 1] - 1]);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* LITERALCHECK - print out the value of literal and locks */

void LiteralCheck(void)
{
    printf("literal: %p; locks: %p.\n", literal, locks);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* GETKILL - Return the value of killme for stack element si */

Integer Getkill(Integer *si)
{
    return (killme[max(-1, si[TC - 1]) + 1]);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* INITKILL - initialize the killme array. Also the literal table
 *          - status arrays.
 */

void Initkill()
{
    int i;

    for (i = 0; i < LASTTYPE + 2; i++)
        killme[i] = 1;
    for (i = 0; i < 10; i++) {
        litcnt[i] = 0;
        litcntc[i] = 0;
        lastcnt[i] = 99;
    }
    killme[NULL_TYPE + 1] = 0;
    killme[GROUP + 1] = 0;
    /* the following two used to be 0 until new stack
     * element format was adopted.[ZCM 8/3/95] */
    killme[NAME + 1] = 1;
    killme[FWA + 1] = 1;
    killme[STRING_ADDRESS + 1] = 1;
    killme[FUNCTION_TYPE + 1] = 0;
    killme[LHS + 1] = 0;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CLLITTAB - clears the literal table */

void Cllittab()
{
    int i;

    for (i = 0; i < mxlit; i++) {
        if (locks[i] == UNLOCKED)
            Osfreen((Address *) &(literal[i]), "locks");
        locks[i] = EMPTY;
        literal[i] = (Address) NOTSET;
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* CLLIT - clears one literal entry */

void Cllit(Integer i)
{
    literal[i - 1] = (Address) NOTSET;
    locks[i] = EMPTY;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* PARNAMS - get the name of a NAME or string stack element. */

void Parnams(Integer *si, char *parnam, Integer *nc)
{
    Integer stc, snaml, sptr;

    stc = si[TC - 1];
    snaml = si[NAML - 1];
    sptr = si[PTR - 1];

/* we have three cases  */
    if (stc == NAME) {

/* its name is in table */
        if (snaml >= 0) {
            StrgGet(sptr, parnam);
            *nc = StrgLen(sptr);
        }

        else if ((snaml < 0) && (sptr > 0) && (sptr <= mxlit)) {

/*  it's in the literal table (we sure hope) */
            if (literal[sptr - 1] > (Address) 0) {
                *nc = min(-snaml, strlen(parnam));
                memcpy(parnam, literal[sptr - 1], *nc);
                parnam[*nc] = '\0';
            }

/* Something clobbered? */
            else {
                strcpy(parnam, "??");
                *nc = 2;
            }
        }

/* inconsistency in name information */
        else {
            strcpy(parnam, "???");
            *nc = 3;
        }
    }

    else if (stc < 0) {

/* in string table */
        if (snaml <= 0) {
            StrgGet(sptr, parnam);
            *nc = StrgLen(sptr);
        }

/* pointer is in literal table (we hope) */
        else if (sptr > 0 && sptr <= mxlit) {
            if (literal[sptr - 1] > (Address) 0) {
                *nc = -stc;
                memcpy(parnam, literal[sptr - 1], *nc);
                parnam[*nc] = '\0';
            }

/* Something clobbered? */
            else {
                strcpy(parnam, "????");
                *nc = 4;
            }
        }

/* Something clobbered? */
        else {
            strcpy(parnam, "?????");
            *nc = 5;
        }
    }

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* LITALLOT - allot space for literals */

void Litallot()
{
    int i;
    Integer litsize;

    valavail = 131;
    valavaid = 131;

/* Allocate space for literal constant table (integers, reals, complexes) */
    maxvals = MAXVALS;
    maxvald = MAXVALS;

    Osallotn((Address *) &values, maxvals, "PERM|LITALLOT:values");
    Osallotn((Address *) &dvalues,
             maxvald * sizeof(real8) / sizeof(Integer),
             "PERM|LITALLOT:dvalues");
    Osallotn((Address *) &nextptr, maxvals, "PERM|LITALLOT:nextptr");
    Osallotn((Address *) &nextptd, maxvald, "PERM|LITALLOT:nextptd");

    for (i = 1; i <= maxvals; i++)
        nextptr[i - 1] = NOT_THERE;

    for (i = 1; i <= maxvald; i++)
        nextptd[i - 1] = NOT_THERE;

    mxlit = MAXLIT;
    litsize = mxlit * sizeof(Address) / sizeof(Integer);

    Osallotn((Address *) &literal, litsize, "PERM|LITALLOT:literal");
    Osallotn((Address *) &locks, mxlit, "PERM|LITALLOT:locks");

    for (i = 0; i < mxlit; i++) {
        literal[i] = (Address) NOTSET;
        locks[i] = EMPTY;
    }

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* PARREL - release storage corresponding to stack element s.
 *        - not done if it's a literal string
 *        - all strings are in permanent table
 */

void Parrel(Integer *s)
{
    int i;
    char msg[80];

    if (s[TC - 1] == NULL_TYPE || (s[TC - 1] < 0 && s[NAML - 1] == 0))
        return;

    /* 11/10/95 ZCM: don't attempt to release something in string table. */
    if (s[TC - 1] == NAME && s[NAML - 1] > 0)
        return;
    i = s[PTR - 1];
    if (i <= 0 || i > mxlit)
        return;                 /* I don't know what to do if out of range */
    if (literal[i - 1] <= (Address) 0) {
        if (literal[i - 1] != (Address) NOTSET) {
            BasError("parrel: bad stack element.", 26);
            exit(0);
        } else {
            return;             /* somebody else released it & I lost track. */
        }
    }
    sprintf(msg, "literal %d", i);
    if (locks[i - 1] == UNLOCKED)
        Osfreen((Address *) &(literal[i - 1]), msg);
    literal[i - 1] = (Address) NOTSET;
    locks[i - 1] = EMPTY;
    s[PTR - 1] = 0;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* PARKILL - release storage for a stack element if necessary. */

void Parkill(Integer *s)
{

/* it's a string literal, nothing to kill */
    if (s[TC - 1] < 0 && s[NAML - 1] == 0)
        return;

    if (killme[max(-1, s[TC - 1]) + 1]) {
        if (s[PTR - 1] == 0)
            return;
        if (literal[s[PTR - 1] - 1] > (Address) 0)
            Parrel(s);
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* PARFREE - release storage pointed at by literal(i). */

void Parfree(Integer i)
{
    char msg[80];

    /* I don't know what to do if out of range */
    if (i <= 0 || i > mxlit)
        return;
    if (literal[i - 1] <= (Address) 0) {
        if (literal[i - 1] != (Address) NOTSET) {
            Remark("parrel: bad entry in literal table.", 35);
            exit(1);
        } else {                /* somebody else released it & I lost track. */
            return;
        }
    }
    sprintf(msg, "literal %ld", (long) i);
    if (locks[i - 1] == UNLOCKED)
        Osfreen((Address *) &(literal[i - 1]), msg);
    literal[i - 1] = (Address) NOTSET;
    locks[i - 1] = EMPTY;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* PARTEMP - get temporary space of n words for parser
 *         - To enlarge dynamic space available for literal table
 *         - if it is full
 *         -    n        = number of words needed
 *         -    partemp  = index into literal table where address is put
 */

Integer Partemp(Integer n)
{
    int found;
    Integer i, j;
    Integer oldlit, newlit;
    char msg[80];

    found = 0;
    for (i = 0; i < mxlit; i++) {
        if (literal[i] == (Address) NOTSET) {
            found = 1;
            break;
        }
    }
    if (!found) {
        /* Need to allocate more space in literal table */
        i = mxlit;
        oldlit = i * sizeof(Address) / sizeof(Integer);
        mxlit += MAXLIT;
        newlit = mxlit * sizeof(Address) / sizeof(Integer);
        Oschangn((Address *) &literal, newlit, oldlit, "literal table");
        Oschangn((Address *) &locks, mxlit, i, "locks");
        for (j = i; j < mxlit; j++) {
            literal[j] = (Address) NOTSET;
            locks[j] = EMPTY;
        }
    }
    /* allocate 1 word if request was for zero */
    j = 1;
    sprintf(msg, "literal %ld", (long) (i + 1));
    Osallotn((Address *) &(literal[i]), n < 1 ? j : n,
             "PERM|PARTEMP:literal[i]");
    locks[i] = UNLOCKED;
    if (i < 10) {
        litcnt[i]++;
        lastcnt[i] = 0;
    }
    return (i + 1);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* BASLTEST - This routine, when called, prints out valuable information
 *          - about literal table usage, and can be very helpful in
 *          - tracking down memory manipulation errors.
 */

void Basltest(Integer line)
{
    integer i, found;
    char message[MAXLINE + 1];
    char lastrout[9];

    found = 0;
    for (i = 0; i < mxlit; i++) {
        if (literal[i] == (Address) NOTSET) {
            found = 1;
            break;
        }
    }
    if (!found) {
        sprintf(message,
                "Literal table completely full (%ld) at line %ld.",
                (long) mxlit, (long) line);
        Remark(message, strlen(message));
        return;
    }
    sprintf(message, "Literal table begins with %ld entries at line %ld.",
            (long) i, (long) line);
    Remark(message, strlen(message));
    if (i >= 1) {
        sprintf(message, "Important diagnostics below:");
        Remark(message, strlen(message));
        for (i = 0; i < 10; i++) {
            if (litcnt[i] != 0 || litcntc[i] != 0) {
                if (lastcnt[i] == 0)
                    strcpy(lastrout, "partemp");
                else if (lastcnt[i] == 1)
                    strcpy(lastrout, "partempc");
                else
                    strcpy(lastrout, "unknown");
                sprintf(message,
                        "Entry: %ld allocated %ld times by partemp,",
                        (long) (i + 1), (long) litcnt[i]);
                Remark(message, strlen(message));
                sprintf(message,
                        "       %ld times by partempc, last by %s.",
                        (long) litcntc[i], lastrout);
                Remark(message, strlen(message));
            }
        }
    }
    found = 0;
    while (i < mxlit) {
        if (literal[i] != (Address) NOTSET) {
            found = 1;
            break;
        }
        i++;
    }
    if (!found)
        return;
    sprintf(message, "The following additional entries are in use:");
    Remark(message, strlen(message));
    while (i < mxlit) {
        if (literal[i] != (Address) NOTSET) {
            sprintf(message, "Entry %ld Contains 0x%p",
                    (long) (i + 1), literal[i]);
            Remark(message, strlen(message));
        }
        i++;
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* BASLT10 - Check the integrity of the first ten words of the literal
 *         - table. Check especially to see if overwritten with ascii.
 */

void Baslt10(void)
{
    int i, j;
    char msg[MAXOUT + 1];
    char diag[9];

    for (i = 0; i < 10; i++) {
        if (locks[i] != EMPTY && locks[i] != LOCKED &&
            locks[i] != UNLOCKED) {
            sprintf(msg, "locks[%d] has corrupted value: %ld.", i,
                    (long) locks[i]);
            Remark(msg, strlen(msg));
        }
        if (locks[i] == EMPTY ||
            (locks[i] != EMPTY && locks[i] != LOCKED &&
             locks[i] != UNLOCKED)) {
            if (literal[i] != (Address) NOTSET) {
                sprintf(msg,
                        "literal[%d] appears to have corrupted value: %p.",
                        i, literal[i]);
                Remark(msg, strlen(msg));
                strncpy(diag, (char *) (&(literal[i])), sizeof(Address));
                for (j = 0; j < sizeof(Address); j++)
                    if (diag[j] < ' ' || diag[j] > '~')
                        break;
                if (j > 0) {
                    diag[j] = '\0';
                    sprintf(msg, "literal[%d] overwritten by ascii: '%s'.",
                            i, diag);
                    Remark(msg, strlen(msg));
                }
            }
        }
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* PARTEMPC - Find an empty entry in the literal table and set it to point at
 *          - fwa. This prevents storage space from being allocated for <lhs>'s
 *          - until and unless it is needed (for assignment, parcoerc'ion,
 *          - result of an operation, etc.).
 */

Integer Partempc(Address fwa)
{
    int j;
    int found;
    Integer i, oldlit, newlit;

    found = 0;
    for (i = 0; i < mxlit; i++) {
        if (literal[i] == (Address) NOTSET) {
            found = 1;
            break;
        }
    }
    if (!found) {
        /* Need to allocate more space in literal table */
        i = mxlit;
        oldlit = i * sizeof(Address) / sizeof(Integer);
        mxlit += MAXLIT;
        newlit = mxlit * sizeof(Address) / sizeof(Integer);
        Oschangn((Address *) &literal, newlit, oldlit, "literal table");
        Oschangn((Address *) &locks, mxlit, i, "locks");
        for (j = i; j < mxlit; j++) {
            literal[j] = (Address) NOTSET;
            locks[j] = EMPTY;
        }
    }
    literal[i] = fwa;
    locks[i] = LOCKED;
    if (i < 10) {
        litcntc[i]++;
        lastcnt[i] = 1;
    }
    return (i + 1);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* PARGETI - Allocate space for an integer, copy n to that space, and
 *         - return the address
 */

Address Pargeti(Integer n)
{
    Address temp;

    Osallotn(&temp, 1, "PARGETI:temp");
    memcpy(temp, &n, sizeof(Integer));

    return (temp);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* PARGETA - Allocate space for an address, copy fwa to that space, and
 *         - return the address
 */

Address Pargeta(Address fwa)
{
    Address lfwa;
    Integer oneaddress;

    oneaddress = sizeof(Address) / sizeof(Integer);
    Osallotn(&lfwa, oneaddress, "PARGETA:lfwa");

    memcpy(lfwa, &fwa, sizeof(Address));

    return (lfwa);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* PARADDR - Return the address of the object described by stack element s. */

Address Paraddr(Integer *s)
{

/* There's nothing there! */
    if (s[TC - 1] == NULL_TYPE)
        return (Address) NOTSET;

/* Use strgaddr for strings */
    if ((s[TC - 1] < 0 && s[NAML - 1] == 0) || s[TC - 1] == NAME)
        return StrgAddr(s[PTR - 1]);

/* String variable with address in literal table--we hope!
 * First check s (PTR) for integrity
 */
    if (s[TC - 1] < 0 && s[NAML - 1] > 0) {
        if (s[PTR - 1] > 0 && s[PTR - 1] <= mxlit)
            return (literal[s[PTR - 1] - 1]);
        else
            return (Address) NOTSET;
    }

/* Address of anything else is in literal table (we hope!)
 * First check s (PTR) for integrity
 */
    if (s[PTR - 1] > 0 && s[PTR - 1] <= mxlit)
        return (literal[s[PTR - 1] - 1]);
    else
        return (Address) NOTSET;

}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* PARSAVE - save a string token into memory space
 *         -   ttokn    = string text
 *         -   ltokn    = token length
 */

Integer Parsave(char *ttokn, Integer ltokn)
{
    int i;

    i = Partemp(ltokn / sizeof(int) + (ltokn % sizeof(int) ? 1 : 0));
    memcpy(literal[i - 1], ttokn, ltokn);

    return (i);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* PARLEN - calculate number of elements in a matrix represented
 *        - by a stack element
 */

Integer Parlen(Integer *s)
{
    Integer len;
    int i;

    if (s[TC - 1] == NULL_TYPE)
        return 0;

    len = 1;
    for (i = 0; i < s[N - 1]; i++)
        len *= s[N1 - 1 + i];

    return len;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* PARGET - get storage needed by a stack entry s
 *        - changed by ZCM 5/7/95 to reflect new stack structure.
 *        - a string literal has type code < 0, and the NAML element points at
 *        - where it is in the string table already, so there is no need to use
 *        - the literal table.
 *        - changed 6/9/95 by ZCM. if NAML is 0 set it to 1 and get the space.
 *        - The string has to be copied out of the string table and maybe blank
 *        - extended, since blanks are trimmed off strings put into the
 *        - string table.
 */

void Parget(Integer *s)
{
    Integer nelt, nbytes, nwords;

    if (s[TC - 1] < 0 && s[NAML - 1] == 0)
        s[NAML - 1] = 1;

    nelt = Parlen(s);
    nbytes = nelt * Utbytesin(s[TC - 1]);
    nwords = nbytes / sizeof(Integer) + (nbytes % sizeof(Integer) ? 1 : 0);

    s[PTR - 1] = Partemp(nwords);

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* PARGETC - when the storage for a <lhs> is consecutive, use this routine to
 *         - set s(PTR) properly. See partempc for details.
 */

void Pargetc(Integer *s, Address fwa)
{
    s[PTR - 1] = Partempc(fwa);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* HASH - hash a number, to get a subscript into the values & nextptr tables */

static Integer Hash(Integer number)
{
    return abs(number + number / 2048) % 131 + 1;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* FIND - Find a number, or if not there, place it, in the values table. In
 *      - any case, return with its index into the table. The comments in the
 *      - routine explain the process in detail.
 *      - prelim is the value actually hashed; in equals number if number is
 *      - an integer, but is something else if number is real.
 */

Integer Find(Integer prelim, Integer number)
{
    Integer h, i;

    h = Hash(prelim);

/* nextptr(h) is always NOT_THERE if the hash table entry is empty, in which
 * case we can place the number there and return with that index value.
 */
    if (nextptr[h - 1] == NOT_THERE) {

/* no number with this hash value in table.
 * means no entry chained to this one.
 */
        nextptr[h - 1] = NULL_TYPE;
        values[h - 1] = number;
        return h;
    }

/* otherwise there are one or more entries possibly chained together.
 * Search the chain and return if the value is there.
 */
    while (number != values[h - 1] && nextptr[h - 1] != NULL_TYPE)
        h = nextptr[h - 1];

    if (number == values[h - 1])
        return h;

/* number was not found, so place it in an overflow bucket at the end
 * of the hash chain. allocate more space if it was all used up.
 */
    valavail += 1;

/* if out of space */
    if (valavail > maxvals) {

/* allocate MAXVALS more space, and initialize all the new nextptrs
 * to NOT_THERE
 */
        maxvals += MAXVALS;
        Oschangn((Address *) &values, maxvals, maxvals - MAXVALS,
                 "values block");
        Oschangn((Address *) &nextptr, maxvals, maxvals - MAXVALS,
                 "next real");
        for (i = maxvals - MAXVALS + 1; i <= maxvals; i++)
            nextptr[i - 1] = NOT_THERE;
    }

    nextptr[h - 1] = valavail;
    values[valavail - 1] = number;
    nextptr[valavail - 1] = NULL_TYPE;

    return valavail;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* DFIND - Find a dnumber, or if not there, place it, in the dvalues table.
 *       - In any case, return with its index into the table. The comments
 *       - in the routine explain in the process in detail.
 *       - prelim is the value actually hashed; in equals dnumber if dnumber
 *       - is an integer, but is something else if dnumber is double.
 */

Integer Dfind(Integer prelim, real8 dnumber)
{
    Integer h, i;

    h = Hash(prelim);
/* nextptd(h) is always NOT_THERE if the hash table entry is empty, in which
 * case we can place the dnumber there and return with that index value.
 */
    if (nextptd[h - 1] == NOT_THERE) {

/* no number with this hash value in table.
 * means no entry chained to this one.
 */
        nextptd[h - 1] = NULL_TYPE;
        dvalues[h - 1] = dnumber;
        return h;
    }

/* otherwise there are one or more entries possibly chained together.
 * Search the chain and return if the value is there.
 */
    while (dnumber != dvalues[h - 1] && nextptd[h - 1] != NULL_TYPE)
        h = nextptd[h - 1];

    if (dnumber == dvalues[h - 1])
        return h;

/* dnumber was not found, so place it in an overflow bucket at the end
 * of the hash chain. allocate more space if it was all used up.
 */
    valavaid += 1;

/* if out of space */
    if (valavaid > maxvald) {

/* allocate MAXVALS more space, and initialize all the new nextptrs
 * to NOT_THERE
 */
        maxvald += MAXVALS;
        Oschangn((Address *) &dvalues,
                 maxvald * sizeof(real8) / sizeof(Integer),
                 (maxvald - MAXVALS) * sizeof(real8) / sizeof(Integer),
                 "doubles block");
        Oschangn((Address *) &nextptd, maxvald, maxvald - MAXVALS,
                 "next double");

        for (i = maxvald - MAXVALS + 1; i <= maxvald; i++)
            nextptd[i - 1] = NOT_THERE;
    }

    nextptd[h - 1] = valavaid;
    dvalues[valavaid - 1] = dnumber;
    nextptd[valavaid - 1] = NULL_TYPE;

    return valavaid;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

Integer Gmemtr(void)
{
    return memtrace;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

Integer Fmnint(Address ax, Integer tc)
{
    Integer *ix;

    ix = (Integer *) ax;
    if (tc == 1)
        return *ix;

    else if (*ix != 0)
        return *ix;

/* On DEC, big endian means choose second word most of the time. */
    else
        return ix[1];

}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
