# Basis Code Development System
# Copyright 1986 The Regents of the University of California

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

* Routine:   MCDODEF
* Author:    Zane C. Motteler
*
* Function:  To store the name and definition of a DEFINE type macro.
*            This routine is rewritten from the original by Susan
*            Hockett, to use the string and hash table facilities
*            written by Paul Dubois.
*
* Arguments:
*
*            parstr    := string determining where the parser is parsing from
*            nstr      := length of parstr
*            strptr    := pointer into parstr
*            recur     := nonzero if recursion is taking place
*            lxptr     := old untop pointer
*
* Externals:
*       lxgtok , strgfind , lxgnbtok , remark , kaboom ,
*       mcbufbod , strgfinz

        subroutine mcdodef(parstr,nstr,strptr,recur,lxptr)
Use(Macros)
Use(Lrhcoms)
        parameter ( inamlen = 128 )
        character * (inamlen) name
        integer nstr
        Address parstr
        integer strptr,recur,lxptr
        integer bptr
        integer lxgtok , strgfind , strgfinz, ltok
        external lxgtok , strgfind , lxgnbtok , remark , kaboom ,
                 mcbufbod , strgfinz

        call lxgnbtok(ntoken,ttoken,ltoken, parstr,nstr,strptr,recur,lxptr)
        if(ntoken <> tokname & ntoken <> tokgname) then
           if (ctyp(ichar(ttoken(1:1))) <> ALPHA) then
              call remark('define followed by "'//ttoken(1:ltoken)//'", not name')
              call kaboom(0)
           endif
        endif
        ltok = ltoken
        name = ttoken ( 1 : ltok )

## --  eatup rest of line as definition

        call lxgnbtok(ntoken, ttoken, ltoken, parstr,nstr,strptr,recur,lxptr)
        bptr = 0 # no of characters stored in body so far
        while(ntoken <> tokcr | ttoken(ltoken:ltoken) = ";")
           if (ntoken == tokstrng) call lxstr(ntoken, ttoken, ltoken)
           call mcbufbod(ttoken, ltoken, bptr)
           if (lxgtok(ntoken, ttoken, ltoken, parstr,nstr,strptr,recur,lxptr) <>
     >            OK) call kaboom(1)
        endwhile

        call mcinstal(name(1:ltok), bptr, FALSE)
        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

* Routine:   MCDOMDEF
* Author:    Zane C. Motteler, August 25, 1994
*
* Function:  to store the name and definition of a MDEF type macro
*            This routine is rewritten from the original by Susan
*            Hockett, to use the string and hash table facilities
*            written by Paul Dubois.
*
* Arguments:
*
*            parstr    := string determining where the parser is parsing from
*            nstr      := length of parstr
*            strptr    := pointer into parstr
*            recur     := nonzero if recursion is taking place
*            lxptr     := old untop pointer
*
* Externals:
*       lxgtok, lxgnbtok, remark, kaboom, mcbufbod, mcinstal

        subroutine mcdomdef(parstr,nstr,strptr,recur,lxptr)
        Use(Macros)
        Use(Lrhcoms)
        parameter ( inamlen = 128 )
        character * ( inamlen ) name
        Address parstr
        integer parencnt,strptr,recur,lxptr,ltok
        logical argflg
        integer lxgtok, bptr
        external lxgtok, lxgnbtok, remark, kaboom, mcbufbod, mcinstal

        argflg = FALSE
        call lxgnbtok(ntoken, ttoken, ltoken, parstr,nstr,strptr,recur,lxptr)
        if (ctyp(ichar(ttoken(1:1))) <> ALPHA) then
           call remark('define followed by "'//ttoken(1:ltoken)//'", not name')
           call kaboom(0)
        endif
        ltok = ltoken
        name = ttoken(1:ltok)

        call lxgnbtok(ntoken, ttoken, ltoken, parstr,nstr,strptr,recur,lxptr)
        if (ntoken == toklparn) then
           call lxgnbtok(ntoken, ttoken, ltoken, parstr,nstr,strptr,recur,lxptr)
           if (ntoken <> tokrparn) then
              call remark('mdef has missing ")"')
              call kaboom(0)
           endif
           argflg = TRUE
           call lxgnbtok(ntoken, ttoken, ltoken, parstr,nstr,strptr,recur,lxptr)
        endif

        if (ntoken <> tokequal) then
           call remark('mdef has missing equal sign (=)')
           call kaboom(0)
        endif

# eatup rest of definition until outermost mdef
        bptr = 0 # no of characters stored in body so far
        if (lxgtok(ntoken, ttoken, ltoken, parstr,nstr,strptr,recur,lxptr) <> OK)
     >       call kaboom(1)
        mdefcnt = 1  #depth of nesting
        parencnt = 0
        do
           if (ntoken == toklcurl) then
              expand = expand+1
           elseif (ntoken == tokrcurl) then
              expand = expand - 1
           elseif (ntoken == toklparn) then
              parencnt = parencnt + 1
           elseif (ntoken == tokrparn) then
              parencnt = parencnt - 1
           elseif (expand==0 & parencnt==0
     1        & (ttoken(1:ltoken)=="mdef" | ttoken(1:ltoken)=="MDEF")) then
              mdefcnt = mdefcnt + 1
           elseif (expand==0 & parencnt==0
     1        & (ttoken(1:ltoken)=="mend" | ttoken(1:ltoken)=="MEND")) then
              mdefcnt = mdefcnt - 1
              if (mdefcnt == 0) break
           endif
           call mcbufbod(ttoken, ltoken, bptr)
           if (lxgtok(ntoken, ttoken, ltoken, parstr,nstr,strptr,recur,lxptr)
     >          <> OK) call kaboom(1)
        enddo

        call mcinstal(name(1:ltok), bptr, argflg)

        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

* Routine:   MCDOIF
* Author:    Zane C. Motteler
*            August 26, 1994
*
* Function:  to determine if two arguments are equivalent and to return
*            the correct selection based on this test.
*            Rewritten from Susan Hockett's original to use character*1
*            arrays instead of integer arrays, and to take full
*            advantage of recursion.
*
* Arguments:
*
*            toktyp   := the type of the token (macro name IFELSE)
*            parstr   := string determining where parser is parsing from
*            nstr     := length of parstr
*            strptr    := pointer into parstr
*            recur     := nonzero if recursion is taking place
*            lxptr     := old untop pointer
*
* Externals:
*            kaboom, mcrdargs, mcsinfo, remark

        subroutine mcdoif(toktyp, parstr,nstr,strptr,recur,lxptr)

        Use(Macros)
        integer toktyp,strptr,recur,lxptr, strgetch, streqch
        external strgetch, streqch
        Address parstr
        integer a1len, a2len, a3len, a1indx, a2indx, nargs, i
        logical argflg, expflg
        integer argptrs ( 10 ) # indices tell where arguments start
                               # in argtext

## -- if first argument equals second argument, then push back third
## -- argument, else fourth (if fourth arg is present)

        nargs = 0
        argflg = TRUE
        expflg = TRUE     # expand all macros in IFELSE test
        call mcrdargs(toktyp, argflg, expflg, argptrs, nargs, parstr,nstr,
		    strptr,recur,lxptr) 
        # read first 2 args
        if (nargs <> 2) then
                call remark('ERROR:  wrong number of arguments for IFELSE test')
                call kaboom(0)
        endif

        expflg = FALSE   # don't expand macros in IFELSE selections
        call mcrdargs(tokrparn,argflg,expflg,argptrs, nargs,parstr,nstr,
		  strptr,recur,lxptr)  
        # read last 1-2 args
        if (nargs > 4 | nargs < 3) then
                call remark
     1             ('ERROR:  wrong number of arguments for IFELSE selection')
                call kaboom(0)
        endif

        a1len = argptrs(2) - argptrs(1)
        a2len = argptrs(3) - argptrs(2)

        a1indx = argptrs(1)
        # eliminate white space at start and finish of first two arguments
        while (a1len > 0 & ctyp(strgetch (argtext, a1indx-1)) == WHITE)
                a1indx = a1indx + 1
                a1len = a1len - 1
        endwhile
        while (a1len > 0 & ctyp(strgetch (argtext, a1indx+a1len-2)) == WHITE)
                a1len = a1len -1
        endwhile
        a2indx = argptrs(2)
        while (a2len > 0 & ctyp(strgetch (argtext, a2indx-1)) == WHITE)
                a2indx = a2indx + 1
                a2len = a2len - 1
        endwhile
        while (a2len > 0 & ctyp(strgetch (argtext, a2indx+a2len-2)) == WHITE)
                a2len = a2len -1
        endwhile

        if (a1len == a2len) then
                do i=0, a1len-1
                        if (streqch (argtext, a1indx+i-1,
                                     argtext, a2indx+i-1) == 0) goto 50
                enddo
## arg1 == arg2, choose third argument
                if ( nargs == 3 ) then
                   a3len = argtnext - argptrs(3)
                else
                   a3len = argptrs(4) - argptrs(3)
                endif
                call mcsinfo (argtext, argptrs(3), a3len)
                argtnext = argptrs(1) # restore argtnext to old value
                return
        endif

## arg1 <> arg2, choose fourth argument if there is one.
 50     continue
        if (nargs == 4) then
                call mcsinfo (argtext, argptrs(4), argtnext - argptrs(4))
        endif

        argtnext = argptrs(1) # restore argtnext to old value

        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

* Routine:   MCSINFO
* Author:    Susan Hockett
*            Revised by Zane C. Motteler to use character*1 arrays
*            Revised (ZCM 3/1999) to use Address/length paradigm
*
* Function:  to store the given information on the PUSH BACK STACK
*
* Arguments:
*
*            info     =:  the information to push back.  The information
*                         is stored in a character*1 array pointed at by
*                         Address info.
*            offset   =:  the offset of the first character in info
*                         (1-based in deference to FORTRAN)
*            linfo    =:  the number of characters of information
*
* Externals:
*            lxpushm (ZCM, 5/9/94)
*
* Comments:
*            the information is pushed back one character at a time,
*            with the last character pushed first and the first
*            character pushed last.

        subroutine mcsinfo(info, offset, linfo)
Use(Macros)   # zcm 8/26/94
        integer linfo, offset, strgetch
        external strgetch
        Address info
        integer i

        do i = 1, linfo
                call lxpushm(char(strgetch(info, (offset-1) + linfo - i))) # zcm 3/25/1999
        enddo
        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

* Routine:   MCEVAL
* Author:    Susan Hockett
*            Extensively rewritten by Zane C. Motteler 8/26/94
*            to use character*1 arrays and take full advantage
*            of recursion.
*
* Function:  to expand (and evaluate) the named macro
*
* Arguments:
*
*            toktyp   =:  the type of token (integer value)
*            token    =:  the bcd text of the token (macro name)
*            toklen   =:  the length in characters of the token (macro name)
*            macflg    := TRUE if token is a macro name, else FALSE
*            parstr    := string determining where the parser is parsing from
*            nstr      := size of parstr
*            strptr    := pointer into parstr
*            recur     := nonzero if recursion is taking place
*            lxptr     := old untop pointer
*
* Externals:
*            mcdodef, mcdomdef, mcdoif, mcdoundf, strgfind, ihashfind,
*            ihashget, strglen, strggetz, mcrdargs, mcsinfo
*
* Comments:
*            DEFINE, MDEF, and UNDEFINE are the names of predefined
*            macros
*
* Revised:
*            Zane C. Motteler
*            August, 1989
*            Improved recursion; add parameters strptr, recur, lxptr
*
*            Zane C. Motteler 
*            May 4, 1994
*            If a macro name occurs in the definition of a user function,
*            demarcate where the expansion is placed on the stack, and
*            when finished, replace the macro name with the expansion
*            text.

        subroutine mceval(toktyp, token, toklen, macflg, parstr,nstr,
	strptr,recur, lxptr)
        Use(Macros)
        Use(Functions)
        Use(Hash)
        character*(*) token
#character parstr(nstr)
        Address parstr, strloc
        integer toktyp, toklen,strptr,recur,lxptr, macstart
        logical macflg
        logical argflg, expflg, mcparck
        integer ano, k, nargs, jdef, start, defend
        integer bottom # starting index of macro def on stack
        integer defsize, alen
        # argptrs are startying indices of arguments in argtext
        # argstrt is what argtnext was upon entry, so it can be restored.
        integer argptrs(10), argtstrt # zcm 8/26/94
        integer strgfind, ihashfind, ihashget, strglen, strgetch,
           streq1f, gtextpos, gfdepth
        external mcdodef, mcdomdef, mcdoif, mcdoundf, strgfind, ihashfind,
                 ihashget, strglen, strggetz, mcrdargs, mcsinfo, strgetch,
                 streq1f,strloc, gtextpos, gfdepth, stextpos, mcparck

## -- check if the given token is the name of a macro
        nargs = 0
        argtstrt = argtnext

        macflg = FALSE
        jdef = 0
        if (token(1:toklen) == "define" | token(1:toklen) == "DEFINE") then
                call mcdodef(parstr,nstr,strptr,recur,lxptr)
        elseif (token(1:toklen) == "mdef" | token(1:toklen) == "MDEF") then
                call mcdomdef(parstr,nstr,strptr,recur,lxptr)
        elseif (token(1:toklen) == "ifelse" | token(1:toklen) == "IFELSE") then
                call mcdoif(toktyp, parstr,nstr,strptr,recur,lxptr)
        elseif (token(1:toklen)=="undefine" | token(1:toklen)=="UNDEFINE") then
                call mcdoundf(parstr,nstr,strptr,recur,lxptr)
        else
                # see if it's a user macro
                jdef = strgfind ( token(1:toklen) )         #1. look up the name
                jentry1 = ihashfind ( mactable , jdef )
                                                  #2. Is it in macro hashtable?
                if ( jentry1 <= 0 ) return        #3. If not, return
                jentry2 = ihashget ( mactable , jentry1 )
                                                  #4. Get entry for body
                if ( jentry2 < 0 ) return        #5. It's been undefined
## -- jentry2 now is the entry number for the macro body
                if ( jentry2 > 0 ) then
                   defsize = strglen ( jentry2 ) + 1 #6. Get length of body
                   call strggeta ( jentry2 , macbody , defsize )
                                                     #7. Retrieve body
                                                     # (extra char for null)
                else # macro must have a blank body
                   defsize = 1
                   call smovftoa (macbody, 0, " " // char (0), 0, 2)
                endif

## -- save the where the macro starts so it can be replaced
## -- if lxsave is not on, macro name isn't in function text.
                macstart = gtextpos (toklen)

## -- initialize data structures needed to evaluate macro and
## -- read the arguments of the macro (if it has any)

                expflg = FALSE
                argflg = mcparck ( jdef )
                nargs = 0
                call mcrdargs(toktyp, argflg, expflg, argptrs, nargs, parstr,
                              nstr,strptr, recur,lxptr)

## -- strip off beginning and ending blanks and newline characters from body

                start = 1
                while(ctyp(strgetch (macbody, start - 1))==WHITE |
                      ctyp(strgetch (macbody, start - 1))==NEWLINE)
                        start = start+1
                endwhile

                defend = defsize-1
                 while(ctyp(strgetch (macbody, defend - 1))==WHITE |
                      ctyp(strgetch (macbody, defend - 1))==NEWLINE)
                        defend = defend - 1
                endwhile

# do replacement on text
# now scan text backwards looking for $n -- notation for the nth argument

# mark first character on stack of macro expansion
                if ( gfdepth () > 0 ) then
                   call lxgptr ( bottom )
                   bottom = bottom + 1
                endif


                for(k = defend, k>start, k = k-1)
                   if (streq1f (macbody, k-2, 1, "$", 0, 1) == 0) then
                      call mcsinfo(macbody,k,1)
                   else # $n or $* or $- or $$
                      if (streq1f (macbody, k-1, 1, "*", 0, 1) == 1) then
                         for(ano=nargs,ano>0,ano=ano-1)
                            if ( ano < nargs ) then
                               alen = argptrs ( ano + 1 ) - argptrs ( ano )
                            else
                               alen = argtnext - argptrs ( ano )
                            endif
                            call mcsinfo(argtext, argptrs(ano),alen)
                            if(ano>1) call mcsinfo(strloc(COMMA),1,1)
                         endfor
                         k = k - 1 #skip over dollar sign
                      elseif (streq1f (macbody, k-1, 1, "-", 0, 1) == 1) then
                         for(ano=nargs,ano>1,ano=ano-1)
                            if ( ano < nargs ) then
                               alen = argptrs ( ano + 1 ) - argptrs ( ano )
                            else
                               alen = argtnext - argptrs ( ano )
                            endif
                            call mcsinfo(argtext, argptrs(ano),alen)
                            if(ano>2) call mcsinfo(strloc(COMMA),1,1)
                         endfor
                         k = k - 1 #skip over dollar sign
                      else #look for digit
                         ano = strgetch (macbody, k - 1) - ZERO
                         if(ano >=0 & ano <=9) then
                            if ( ano <= nargs) then
                               if ( ano < nargs ) then
                                  alen = argptrs ( ano + 1 ) - 
                                         argptrs ( ano )
                               else
                                  alen = argtnext - argptrs ( ano )
                               endif
                               call mcsinfo(argtext, argptrs(ano),alen)
                            endif
                            k = k - 1 #skip over dollar sign
                         else  #dollar other, put the other
                            call mcsinfo(macbody,k,1)
                         endif
                      endif
                   endif
                
                endfor loop on replacement text
                if (k == start) call mcsinfo(macbody,k,1)   #do last character
                # If we are defining a function, then this text should replace
                # the token text in the function text buffer. Move the
                # textpos pointer back where the macro began.
                call stextpos (macstart)

        endif
        macflg = TRUE  ## being evaluated
        argtnext = argtstrt #restore argtnext

        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

* Routine:   MCRDARGS
* Author:    Susan Hockett
* Revision:  Zane C. Motteler 8/29/94
*            Uses character*1 arrays; new argument argptrs
*
* Function:  to read the argument list following a macro name
*
* Arguments:
*
*            atoktyp  =:  the type of token (integer value)
*            argflg   =:  FALSE if no argument list is to be read,
*                         TRUE if possible argument list follows.
*            expflg   =:  TRUE if arguments are to be immediately
*                         expanded, FALSE if arguments are to be
*                         expanded only when used.
*            argptrs   := subscripts into argtext telling where each
*                         argument's text starts
*            nargs     := the number of arguments read
*            parstr    := string determining where the parser is parsing from
*            nstr      := the length of parstr
*            strptr    := pointer into parstr
*            recur     := nonzero if recursion is taking place
*            lxptr     := old untop pointer
*
* Externals:
*            lxpeek, lxgtok, lxgnbtok, remark, kaboom, mceval, mcbufarg
* Comments:
*            if ATOKTYP == TOKREF, then the (macro) name was followed
*            by a left parenthesis (by lexer definition)
*
* Revised:
*            Zane C. Motteler
*            August, 1989
*            Improved recursion; add parameters strptr, recur, lxptr

        subroutine mcrdargs(atoktyp, argflg, expflg, argptrs, nargs, parstr,
                            nstr,strptr,recur,lxptr)
        Use(Macros)
	Use(Lrhcoms)
        logical argflg, expflg
        integer atoktyp, nargs, parencnt, strptr,recur,lxptr, argptrs(10)
        Address parstr
        logical macflg, newarg
        integer lxgtok, lxpeek
        external lxpeek, lxgtok, lxgnbtok, remark, kaboom, mceval, mcbufarg

        macflg = .false.
        parencnt = 0

## -- if "(" does not follow macro name or if macro is defined not to
## -- have an argument list, then return since no argument list to read

        if (~argflg) return
## check if ( is next
        if (lxpeek(LPAREN, parstr,nstr,strptr,recur,lxptr)<>OK) return      ## no "("
        call lxgnbtok(ntoken, ttoken, ltoken, parstr,nstr,strptr,recur,lxptr)
        if (ntoken <> toklparn) then
                call remark(
                    'INTERNAL ERROR: Expecting "(" following macro name')
                call kaboom(0)
        endif

        parencnt = 1
        newarg = TRUE
        nargs = nargs + 1
        argptrs ( nargs ) = argtnext

        do
           if (newarg) then     ### ignore leading blanks and newlines
              do
                 call lxgnbtok(ntoken, ttoken, ltoken, parstr,
                               nstr,strptr,recur,lxptr)
              until (ntoken <> tokcr | ttoken(ltoken:ltoken) == ";")
           else
              if (lxgtok(ntoken, ttoken, ltoken, parstr,nstr,strptr,recur,
                         lxptr) <> OK)
     1              call kaboom(1)
           endif


           if (ntoken == tokrcurl) then
              if(expand < 1) then
                 call remark(
     1             'ERROR: Extra right curly brace in argument list.')
                 call kaboom(0)
              endif
              expand = max(expand - 1, 0)
           elseif (ntoken == tokrparn) then
              parencnt = parencnt - 1
              if (parencnt == 0) break
           endif

# start collecting arguments

           newarg = FALSE
           if (ntoken==tokcomma & parencnt==1 & expand==0) then   # new arg
              nargs = nargs + 1
              if ( nargs > 10 ) then
                 call remark ("mcrdargs: macro has too many arguments.")
                 call kaboom ( 0 )
              endif
              argptrs ( nargs ) = argtnext
              newarg = TRUE

## if expflg, then check all names not in {}'s to see if it is a macro
## if yes, expand macro, else buffer original name

           elseif(expflg & expand==0 &
                 ((ntoken==tokname ) | (ntoken==tokgname)))then
              call mceval(ntoken, ttoken, ltoken, macflg, parstr,
                          nstr,strptr,recur,lxptr)
              if (~macflg) call mcbufarg(ttoken, ltoken)

           else                   ## buffer everything else
              call mcbufarg(ttoken, ltoken)
           endif

           if(ntoken==toklcurl)  expand = expand + 1
           if(ntoken==toklparn)  parencnt = parencnt + 1

        enddo           # end of gettok call loop

        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

* Routine:   MCCOMSTR
* Author:    Susan Hockett
* Revision:  Zane C. Motteler August 29, 1994
*            Changes due to use of Dubois string/hash routines
*
* Function:  to expand all the macros in an COMMAND string
*
* Arguments:
*
*            toktyp   =:  the type of token (integer value)
*            token    =:  the bcd text of the token (first token in
*                         fcommand string)
*            toklen   =:  the length in characters of the token
*            parstr    := string determining where the parser is parsing from
*            nstr      := length of parstr
*            strptr    := pointer into parstr
*            recur     := nonzero if recursion is taking place
*            lxptr     := old untop pointer
*
* Externals:
*            lxpeek, lxgtok, mclrv122, lxgchr, mcbufarg,
*            mceval, lxstr, mcsinfo, kaboom
*
* Comments:
*            After the entire COMMAND string is lexed and all the
*            macros are expanded, then a marker is put at the beginning
*            and end of the string, and the string is relexed with the
*            entire string constituting one COMSTR token.
*            Revisions by ZCM September 12-13, 1994:
*            If we are inside a function, the string has already
*            been deposited in the function text; the STRINGMARK
*            character becomes a signal for lxgchr not to put it
*            there a second time.
*            Since the string is followed on unstak by a delimiter,
*            we set lxskip to the length of this delimiter, since
*            it, too, is already in the function text. This is only
*            esthetically unpleasing if the delimiter was white space,
*            but disastrous if it was any other delimiter.
*
*            Blanks and commas are delimiters of the string and tokcr
*            is a terminator (unless these tokens are protected).
*
*            This routine returns ERR if the token is to be discarded,
*            else returns OK.  Only delimiters and terminators are kept.
*
* Revised:
*            Zane C. Motteler
*            August, 1989
*            Improved recursion; add parameters strptr, recur, lxptr
*            September, 1989
*            Add command_l (eat up the rest of the line as a string)
*            October, 1989
*            Alter to ignore delimiters inside parentheses as is done
*            by mccomexp	
*            November 1989
*            Alter to ignore delimiters inside square brackets as is
*            done my mccomexp

      integer function mccomstr(toktyp, token, toklen, parstr,
     &                          nstr, strptr, recur, lxptr)
      integer toktyp
      character*(*) token
      integer toklen
      Address parstr
      integer nstr, strptr, recur, lxptr

      Use(Macros)
      Use(Lrhcoms)
      Use(Functions)

      logical lxdltst, macflg, gmactype
      integer argindx, pkflg, tokflg,a, idone
      integer lxgtok, lxpeek, utgetcl, gcomtyp
      external lxpeek, lxgtok, mclrv122, lxgchr, mcbufarg, gcomtyp
      external mceval, lxstr, mcsinfo, kaboom, utgetcl, slxskip, gmactype

c ... test if token is an argument delimiter
      if (lxdltst(toktyp)) then
         if ((toktyp .eq. tokwhite) & delimflg) then
            delimflg = FALSE
            return(ERR)

c ... change current delimiter type to comma (for line cont.)
         elseif ((lastntok .eq. tokwhite) & delimflg) then
            lastntok = tokcomma
            delimflg = TRUE
            if (gcomtyp(comarg) .eq. LINCOM) lflag = 1
            return(ERR)

c ... current token is a delimiter, and delimflg is false;
c ... or else delimflg is true and
c ... neither the current nor the previous token are white space
         else 
            comflg = comflg - 1
            comarg = comarg + 1
            if (comarg > mxcomsz) then
               call mclrv122(comstrtk, utgetcl(comstrtk),
     &                       max(mxcomsz+COMINC,comarg))
            endif
            delimflg = TRUE

c ... let grammar think it saw a comma
            if (toktyp .ne. tokwhite) then
               toktyp = tokcomma  

c ... unfortunately, if this token is white space and we have
c ... an "l" type argument (the rest of the line) we can't
c ... set lflag because the next nonblank token could be a
c ... delimiter. If it isn't, we'll catch it below and back
c ... it out, in a manner of speaking.
c ... this is only a problem on an argument after the first.
               if (gcomtyp(comarg) .eq. LINCOM) lflag = 1

            elseif (toktyp .eq. tokwhite) then
               pkflg = lxpeek("#",parstr,nstr,strptr,recur,lxptr)

c ... discard whitespace if preceding a "#" or EOF
               if ((pkflg .eq. OK) | (pkflg .eq. DONE)) return(ERR)
            endif
            return(OK)
         endif

      elseif (toktyp .eq. tokwhite) then
         pkflg = lxpeek("#",parstr,nstr,strptr,recur,lxptr)

c ... discard whitespace if preceding a "#" or EOF
         if ((pkflg .eq. OK) | (pkflg .eq. DONE)) then
            return(ERR)
         endif

c ... check for line continuation
      elseif (toktyp .eq. tokcr) then
         if ((token(toklen:toklen) .ne. ";") & (lastntok .eq. tokcomma)) then
            delimflg = TRUE
            return(ERR)
         else
            comflg = 0
            comarg = 0
            delimflg = FALSE
            return(OK)
         endif

c ... handle EOF
      elseif (toktyp .eq. -1) then
         toktyp = tokcr
         comflg = 0
         comarg = 0
         delimflg = FALSE
         return(OK)
      endif

      if (delimflg & (comarg > 1) & (gcomtyp(comarg) .eq. LINCOM)) then

c ... there is more than one argument, and the last one is "l" type, and
c ... it starts with the current token since the current token is not a
c ... delimiter. Put a quote in front of the current token, gather in the
c ... rest of the current line, put a quote after it, and change its
c ... type to tokstrng.
         idone = lxgchr(a, parstr, nstr,strptr, recur, lxptr)
         while (a ~= peol & a ~= petx)
            toklen = toklen + 1
            token(toklen:toklen) = char(a)
            idone = lxgchr(a, parstr, nstr,strptr, recur, lxptr)
         endwhile

         call lxpush(a)
         toktyp   = tokcmstr
         delimflg = FALSE
         return(OK)
      endif

c ... argument is part of an extended string; parse until delimiter is found
      delimflg = FALSE
      argindx  = argtnext
      call mcbufarg(char(STRINGMARK), 1)

      ntoken = toktyp
      ttoken = token
      ltoken = toklen
      while ((ntoken .ne. tokcr) &
     &       (~lxdltst(ntoken) | (expand > 0) | (exprparn > 0) |
     &       (exprbrak > 0)))
         if (ntoken .eq. toklparn) then
            exprparn = exprparn + 1

         elseif (ntoken .eq. tokrparn) then
            exprparn = exprparn - 1
         endif

         if (ntoken .eq. toklbrak) then
            exprbrak = exprbrak + 1

         elseif (ntoken .eq. tokrbrak) then
            exprbrak = exprbrak - 1
         endif

         if (ntoken .eq. toklcurl) then
            expand = expand + 1

         elseif (ntoken .eq. tokrcurl) then
            expand = expand - 1

         elseif ((expand .eq. 0) & (gmactype(comarg)) &
     &           ((ntoken .eq. tokname) | (ntoken .eq. tokgname))) then
            call mceval(ntoken, ttoken, ltoken, macflg, parstr,
     &                  nstr,strptr,recur,lxptr)
            if (~macflg) call mcbufarg(ttoken, ltoken)

         else
            if ((ntoken .eq. tokstrng) & (exprbrak .eq. 0)) then
               call lxstr(ntoken,ttoken,ltoken)
            endif
            call mcbufarg(ttoken, ltoken)
         endif

         tokflg = lxgtok(ntoken, ttoken, ltoken, parstr,nstr,strptr,recur,
     &                   lxptr)

         if (tokflg .eq. ERR) call kaboom(1)
         if (tokflg .eq. DONE) break
      endwhile

      call mcbufarg(char(STRINGMARK), 1)

      if (tokflg .ne. DONE) then
         call mcbufarg(ttoken, ltoken)
         call slxskip (ltoken)
      endif

      call mcsinfo(argtext, argindx, argtnext-argindx)
      argtnext = argindx

      return(ERR)
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

* Routine:   MCCOMEXP
* Author:    Susan Hockett
*
* Function:  to determine if each token in a COMMAND expression should
*            be kept or discarded
*
* Arguments:
*
*            toktyp   =:  the type of token (integer value)
*            token    =:  the bcd text of the token in a COMMAND expression
*            toklen   =:  the length in characters of the token
*            parstr    := string determining where the parser is parsing from
*            nstr      := length of parstr
*            strptr    := pointer into parstr
*            recur     := nonzero if recursion is taking place
*            lxptr     := old untop pointer
*
* Externals:
*            lxconlin, lxpeek
*
* Comments:
*            whitespace is to be discarded if it is unprotected and
*            is within parentheses.
*
*            This routine returns OK if token is to be kept by LXMAC, else
*            ERR if token is to be thrown away
*
* Revised:
*            Zane C. Motteler
*            August, 1989
*            Improved recursion; add parameters strptr, recur, lxptr

        function mccomexp(toktyp,token,toklen,parstr,nstr,strptr,recur,lxptr)
        integer mccomexp
        Use(Macros)
	Use(Lrhcoms)
        integer toktyp, toklen,strptr,recur,lxptr
        character*(*) token
#character parstr(nstr)
        Address parstr
        integer retflg, pkflg
        integer lxconlin, lxpeek
        logical lxdltst
        external lxconlin, lxpeek

        retflg = OK
        if (toktyp == toklparn) then
                exprparn = exprparn + 1
        elseif (toktyp == tokrparn) then
                exprparn = exprparn - 1
        elseif (toktyp == toklbrak) then
		exprbrak = exprbrak + 1
        elseif (toktyp == tokrbrak) then
		exprbrak = exprbrak - 1
        endif

        if (lxdltst(toktyp) & exprparn==0 & exprbrak==0) then
            if (toktyp == tokwhite) then
                pkflg = lxpeek("#",parstr,nstr,strptr,recur,lxptr)
				# this whitespace is not a delimiter; ignore it.
                if (delimflg | pkflg==OK | pkflg==DONE) then
  	   	   delimflg = FALSE
                   return(ERR)
                endif
   	    elseif ((lastntok == tokwhite) & delimflg) then 
		    # set current delimiter type to tokcomma (for line cont.).
		lastntok = tokcomma
  	  	    # this whitespace is not a delimiter; ignore it.
   	        delimflg = TRUE
                return(ERR)
            endif

            comflg = comflg - 1
            comarg = comarg + 1
                   # let grammar think it saw a comma or space
  	    if (toktyp<>tokwhite)  toktyp = tokcomma 
            delimflg = TRUE
        elseif(toktyp==tokwhite) then
                retflg = ERR    #this whitespace is not a delimiter
        else
                if (toktyp == tokcr) then
                        retflg = lxconlin(toktyp, token, toklen)  ## check for
                        if (retflg == ERR) then## line continuation
			   				delimflg = TRUE
			   				return(ERR)
						else
                           comflg = 0
                           comarg = 0
						endif
                elseif (toktyp == -1) then
                        comflg = 0
                        comarg = 0
                        retflg = OK
                endif
                delimflg = FALSE
        endif

        return(retflg)
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

* Routine:   MCDOUNDF
* Author:    Susan Hockett
*
* Function:  to delete a macro definition
*
* Arguments:
*
*            parstr    := string determining where the parser is parsing from
*            strptr    := pointer into parstr
*            recur     := nonzero if recursion is taking place
*            lxptr     := old untop pointer
*
* Externals:
*            kaboom, lxgtok, mcundef, remark
*
* Revised:
*            Zane C. Motteler
*            August, 1989
*            Improved recursion; add parameters strptr, recur, lxptr

        subroutine mcdoundf(parstr,nstr,strptr,recur,lxptr)
        Use(Macros)
	Use(Lrhcoms)
        Address parstr
        integer lxgtok,strptr,recur,lxptr,l
        external lxgtok

        if (lxgtok(ntoken, ttoken, ltoken,
                   parstr,nstr,strptr,recur,lxptr) <> OK) call kaboom(1)
        while (ntoken <> tokcr)
           if (ntoken==tokname | ntoken==tokgname ) then
              call mcundef(ttoken, ltoken)
           elseif (ntoken<>tokwhite & ntoken<>tokcomma) then
              call remark(
     1              'UNDEFINE statement must be blank or comma delimited')
              call kaboom(0)
           endif
	   l = lxgtok(ntoken, ttoken, ltoken, parstr,nstr,strptr,recur,lxptr) 
           if (l == ERR) then
               call kaboom(1)
           elseif (l == DONE) then
               break
           endif
        endwhile
        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

* Routine:   MCNEST
* Author:    Zane C. Motteler / 9 September 1993
*
* Function:  increases the apparent macro depth by 1.
*            Called, for instance, when a function is declared, to prevent
*            its name and arguments from being expanded.
*
* Arguments:
*
*            none
*
* Externals:
*            none

	subroutine mcnest
	Use(Macros)
	expand = expand + 1
	return
	end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

* Routine:   MCUNNEST
* Author:    Zane C. Motteler / 9 September 1993
*
* Function:  decreases the apparent macro depth by 1.
*            Called, for instance, after a function name and arguments have
*            been processed, turning macro expansion back on if depth was 1.
*
* Arguments:
*
*            none
*
* Externals:
*            none

	subroutine mcunnest
	Use(Macros)
	expand = expand - 1
	return
	end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

* Routine:   MCNOLKUP
* Author:    Zane C. Motteler / 9 September 1993
*
* Function:  Tells whether a name should be looked up as a macro or not.
*            Basically, f could be defined as a macro and also as a function.
*            list f should print out the macro expansion of f.
*               (in this case nameflg <> 0)
*            list {f} should print out the functional information on f.
*               (in this case nameflg <> 0 and expand > 0)
*
* Arguments:
*
*            none
*
* Externals:
*            none

	logical function mcnolkup()
	Use(Macros)
	if ( nameflg <> 0 & expand > 0 ) then
		return ( TRUE )
	else
		return ( FALSE )
	endif
	end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

* Routine:   MCBUFBOD
* Author:    Zane C. Motteler August 1994
*
* Function:  Stores the next component of a macro body
*
* Arguments:
*
*            cout    =:  the text to store (cout is of type character)
*            n       =:  the length in characters of the text to store
*            bptr    =:  index into macbody array
*
* Externals:
*            oschange

      subroutine mcbufbod(cout, n , bptr)
      Ch(*) cout
      integer n, bptr
Use(Macros)
      external oschangn
      # make sure buffer is big enough for this macro
      while ( bptr + n > mcbodsz )
         call oschangn ( macbody , mcbodsz + MAXCHARR / \
            CHAR_PER_WORD + 1 , mcbodsz , "macbody")
         mcbodsz = mcbodsz + MAXCHARR / CHAR_PER_WORD + 1
      endwhile
      call smovftoa (macbody, bptr, cout, 0, n)
      bptr = bptr + n
      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

* Routine:   MCBUFARG
* Author:    Susan Hockett
* Revision:  Zane C. Motteler August 29, 1994
*            Arguments are stored in character*1 array
*
* Function:  stores the argument (or argument fragment)
*
* Arguments:
*
*            cout    =:  the text to store (cout is of type character)
*            n       =:  the length in characters of the text to store
*
* Externals:
*            oschange
*
* Comments:
*            The argument can be buffered in several pieces.  arguments
*            are stored in the global argtext; the beginning subscripts
*            of each argument are stored by the routine which calls this one.
*
*            Argument COUT is of type character.

        subroutine mcbufarg(cout,n)
        Use(Macros)
        integer n
        character*(*) cout
        integer newsize
        external oschangn

        if(argtnext+n>=argtsize) then
                newsize = argtsize + ((n+ARGINC-1)/ARGINC)*ARGINC
                call oschangn(argtext, newsize, argtsize, "argtext")
                argtsize = newsize
        endif
        call smovftoa (argtext, argtnext -1, cout, 0, n)
        argtnext = argtnext + n
        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c MCCHRNAM - return the name of the given macro
c            name     =:  name of the given macro
c            buf       := the array in which to store the macro name
c                         (as characters)
c            bufmax   =:  maximum size of the buffer to store the name
c            indx     =:  index (1-origin) in the buffer at which to store name
c            newindx   := index (1-origin) of the first character following
c                         the name just put in the buffer
c
c Comments:
c            the name is returned as a character string.  However it
c            is internally stored as integers with
c            one character (right justified, 0 filled) per word.
c
c            Returns an error if the name can not fit in the buffer.  The
c            name is not stored unless INDX is 1, i.e. name length > bufmax
c            and stored at beginning of the buffer.
c
c            Argument BUF must be an array of characters, not a
c            character string, i.e. defined as "character BUF(some_size)".

        function mcchrnam(name, buf, bufmax, indx, newindx)
        integer mcchrnam
        Ch(*) name
        character*(*) buf
        integer bufmax, indx, newindx

        Use(Macros)

        integer i, n, retflg, namlen

        retflg = OK
        if (indx + len(name)-1 > bufmax) then
           retflg = ERR
           if (indx > 1) return(retflg)
        endif

        namlen = min(bufmax, len(name))
        do i = 1 , namlen
           n = indx + i
           buf(n:n) = name(i:i)
        enddo

        newindx = indx + namlen

        return(retflg)
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

# I removed mcchrdln here 8/31/94--replaced by strggetn (ZCM)
# I removed mcdomac and mcdefmac here 8/25/94--nobody seemed to call them (ZCM)

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

* Routine:   MCCHRDEF
* Author:    Susan Hockett
*
* Function:  to store a macro definition.
*            This is usually called from FORTRAN (e. g., act.m)
*            to define a more or less permanent macro. 
*            The similar mcchrdef_ is automatically
*            called from the Basis command line, but it's easier
*            just to use define or mdef on the command line.
*
* Arguments:
*
*            mcname    =:  the ASCII text (characters) of the macro name
*            mcdef     =:  the ASCII text (characters) of the macro definition
*            argflg    =:  TRUE if macro can have arguments, else FALSE
*
* Externals:
*
*            mcinstal
*
* Revised:
*            Zane C. Motteler
*            August 1994
*            macros now use the Dubois string and hash tables.
*            March 1999
*            split this off from mcchrdef since the way dynamic strings
*            are passed has been changed. mppl will change all calls
*            to mcchrdef in .m files, to calls to this function.

        subroutine mcchrdef(mcname, mcdef, argflg)
        character*(*) mcname, mcdef
        logical argflg
        integer deflen, utgetcl
        external mcinstal, utgetcl
Use(Macros)

# mcinstal assumes the macro is in macbody, so we'd better put it there
        deflen = len(mcdef)
        call smovftoa (macbody, 0, mcdef, 0, deflen)
        call mcinstal(mcname(1:utgetcl(mcname)), deflen, argflg)

        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

        subroutine mcchrdef_(mcname, lmcname, mcdef, lmcdef, argflg)
        Address mcname, mcdef
        integer lmcname, lmcdef, utgetcl
        external utgetcl
        logical argflg
        Varname vmcname

Use(Macros)

        vmcname = " "

        call smovatof(vmcname, 0, mcname, 0, min (lmcname, len (vmcname)))
        call smovatoa(macbody, 0, mcdef, 0, lmcdef)

        call mcinstal(vmcname(1:utgetcl(vmcname)), lmcdef, argflg)

        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

*       MCINSTAL
*       Reprogrammed by : Zane C. Motteler
*       Date            : August 1994
*       Purpose         : macros are now put into the Dubois string and
*                         hash tables. See explanations below.

        subroutine mcinstal(name,bptr,argflg)
        Ch(*) name
        integer bptr
        logical argflg
Use(Macros)
Use(Hash)
        integer jname, jbody, jdummy
        integer ihashput, strgfind, strgfina, utgetcl
        external ihashput, strgfind, strgfina, utgetcl

## -- install definition in table:
## -- put the name and body in the string table, then, using the
## -- index for the name as key, put the body index into the macro
## -- hash table (table 0) as its value.
## -- strgfina allows sending an array of character*1 to C as a
## -- standalone string, using the Fcc type 'character'. (the Fortran
## -- 500-character limit strikes again.)
## -- No need to see if there was a previous definition; strings always
## -- stay around, and ihashput simply replaces previous key with a
## -- new one.
        jname = strgfind ( name(1:utgetcl(name)) )
        
# bptr is the actual length of the string to store. strgfina appends
# a null at the next position.
        jbody = strgfina ( macbody , bptr )

        call defmchk (jname, argflg)

        jdummy = ihashput ( mactable , jname , jbody )

        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

# removed mcnewptr here 8/31/94 (ZCM)

# removed mcoldptr here 8/31/94 (ZCM)

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

#       MCUNDEF
#       revised by ZCM 8/91 -- uses C routines to do name stuff
#       revised by ZCM 8/94 -- uses Dubois hash and string tables

        subroutine mcundef(cname, lname)
# take out a macro definition
# change characters to integers and call mcunstal
        Use(Hash)
        character*(*) cname
        integer jentry
        integer ihashfind, strgfind
        external ihashfind, strgfind, mcunstal
#
#
        jentry = strgfind(cname(1:lname))
        call mcunstal(ihashfind(mactable,jentry),jentry)
        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

#       MCUNSTAL
#       revised by ZCM 8/91 -- uses C routines to do name stuff
#       revised by ZCM 8/94 -- uses Dubois hash and string tables
#          Much simpler than it used to be. Nothing gets physically
#          removed. 

        subroutine mcunstal(ientry,jentry)
Use(Macros)
Use(Hash)
        integer ientry,         # ptr to hashtable entry to be removed
                jentry,         # index of the macro's name
                jdummy

        external ihashput
        integer ihashput
#
# if it doesn't exist, do nothing
        if(ientry > 0) then
# remove the definition
           jdummy = ihashput ( mactable , jentry , -1 ) ## ZCM 8/94 remove value
           call clrdefm (jentry)
        endif
        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

* Routine:   MCVATTRE
* Author:    Susan Hockett
*
* Function:  to tag a macro with the given attribute
*
* Arguments:
*
*            macindx  =:  the index in the string table of the name
*                         of the macro to tag
*            attr     =:  the name of the attribute to tag the macro with
*
* Externals:
*            rtchattr
*

       subroutine mcvattre(macindx, attr)
       Use(Macros)

       integer macindx
       external rtmacat
       character*(*) attr
       Address gmacatra
       external gmacatra

       call rtchattr(gmacatra(macindx), attr)
       return
       end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

*      MCSERV
* Revised:
*            Zane C. Motteler
*            August 1994
* Purpose:
*            Uses the Dubois string/hash tables

### Created 11/89 by SMH
### mcserv: service for attributes
# attr: an attribute name
# actor: a subroutine
# kactor: flag specifying the nature of actor's calling sequence
# param: a parameter to be passed to actor
# mcserv cycles over the macros which have the given attribute set.


       subroutine mcserv (attr, actor, kactor, param)

       Use(Macros)
       Use(Hash)
       character*(*) attr
       external actor
       integer namlen, utgetcl
       logical bastestattr
       external nstrgs, strglen, bastestattr, utgetcl
       integer kactor, param, ns, nstrgs, strglen
       integer i, numattr
       character*40 macname
       integer rtattrlu, rtmacat
       external rtmacat

       call mcactor(actor,kactor," ",attr,param,0)
       ns = nstrgs ( )
       do i = 1 , ns
          if ( ihashfind ( mactable , i-1 ) > 0 ) then # it is a macro name
             call strgget ( i-1 , macname )
             namlen = utgetcl(macname)
             numattr = rtattrlu(attr)
             if (bastestattr(rtmacat(i-1), numattr)) then
                if ( strglen(i-1) > 40 ) \
                   call remark(
                      "mcserv:  macro name truncated to 40 characters")
                call mcactor(actor,kactor,macname,attr,param,1)
             endif
          endif
       enddo
       call mcactor(actor,kactor," ",attr,param,2)
       return
       end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

       subroutine mcactor(actor,kactor,mcname, attr,param,istage)
       external actor
       integer kactor, param, istage
       character*(*) mcname, attr

       select kactor
       case 1:
          call actor(mcname, attr, param, istage)
       default:
          call sorry("mcactor: kactor value not implemented")
       endselect
       return
       end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

#       Revision (ZCM - 9/89) -- local variable carg replaces comarg, because
#       in the case when blank is a delimiter and there is also a user
#       delimiter following it, comarg has already been incremented, so the
#       user delimiter may (in some cases) not be recognized.
#       Revision (ZCM - 11/89) -- in the special case when we have yet to see
#       the first argument, and it's a string, we want to skip white space
#       in front of it, rather than taking that white space as part of the
#       token. In this case comarg will be 1 and delimflg will be true and
#       ntok will be tokwhite. 
# Revision 4/1/96
# All the command logic processing had been using rtsattr/rtrattr, part of
# the attribute package, because they happened to manipulate bits in a word.
# This was broken when I change the attribute handling.
# We are left with the funny business of subtracting 1 from everything to
# get a bit position, and I would just change the constants in par.v but
# there is a funny place where a variable is used and I think there may be
# a tie-in between the delimiter values and some token numbers, so rather
# than figure it out I'm leaving it be. -- Paul Dubois

        function lxdltst(ntok)

        logical lxdltst
        integer ntok, carg, gdlimtyp, dlim, ilt, nc
        Varname s, t
        external gdlimtyp

        Use(Macros)
	Use(Lrhcoms)

        integer utgetcl
        external utgetcl

        if (comarg == 1 & delimflg & ntok == tokwhite) return(TRUE)

        carg = comarg
        if ((delimflg & lastntok == tokwhite) |                 \
            (delimflg & ntok == tokwhite)) carg = carg - 1

# This function tests whether a token is a delimiter in an argument list
        dlim = gdlimtyp (carg)
        if ((ntok==tokcomma & btest(dlim,DLIM_CM-1)) |
     1      (ntok==tokwhite & btest(dlim,DLIM_WS-1)) |
     1      (ntok==tokatsym & btest(dlim,DLIM_AT-1)) |
     1      (ntok==tokequal & btest(dlim,DLIM_EQ-1))) then
            return(TRUE)
        endif

# Check to see if token is a user-defined delimiter
# GOTCHA: this can hardly be the best fix but it is not clear
# where ttoken is being initialized and the last character is
# not set which foils the comparison
        nc  = VARNAMESIZE - 2
        ilt = ltoken
        s   = ttoken(1:ilt)
        do i = 1, 10
           t = userdlim(i)
           if (t(1:1) .eq. char(0)) then
              break
           else
              if (btest(dlim, i-1) .and. (s(1:nc) .eq. t(1:nc))) then
                 return(TRUE)
              endif
           endif
        enddo

        return(FALSE)
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

#       mcpar -- routine to initialize macro database
#
#       part of changes needed to encapsulate Macro references in mac.m
#       moved here from act.m
#       Zane C. Motteler July, 1991
#   Altered by ZCM 8/94 to allocate space for macro body text and argument
#   text.

	subroutine mcpar
Use(Macros)

        argtsize = ARGINC
        call osallotn(argtext, argtsize, "PERM|MCPAR:argtext ") # added 8/94
        call defminit ()
        call osallotn(macbody, mcbodsz, "PERM|MCPAR:macbody ") # added 8/94

        call lrvlup(tokname,'name',4)
        call lrvlup(tokgname,'Groupname',9)
        call lrvlup(tokcr,'cr',2)
        call lrvlup(tokcarat, '^',1)
        call lrvlup(tokstrng,'string',6)
        call lrvlup(tokcmstr, 'fcomstr', 7)
        call lrvlup(tokint,'integer-constant',16)
        call lrvlup(tokoct,'octal-constant',14)
        call lrvlup(tokhex,'hex-constant',12)
        call lrvlup(tokrparn,')',1)
        call lrvlup(tokrcurl,'}',1)
        call lrvlup(tokrbrak,']',1)
        call lrvlup(tokwhite,'whitespace',10)
        call lrvlup(tokatsym,'@',1)
        call lrvlup(tokexcla,'!',1)
        call lrvlup(tokback, '`',1)
        call lrvlup(tokend,'END',3)  #this is used in parsestr
# set up automatic line continuation on following symbols
        call lrvlup(tokcontu(1),'(',1)
        toklparn = tokcontu(1)
        call lrvlup(tokcontu(2),',',1)
        tokcomma = tokcontu(2)
        call lrvlup(tokcontu(3),'+',1)
        call lrvlup(tokcontu(4),'-',1)
        call lrvlup(tokcontu(5),'*',1)
        call lrvlup(tokcontu(6),'/',1)
        call lrvlup(tokcontu(7),'[',1)
        toklbrak = tokcontu(7)
        call lrvlup(tokcontu(8),'!',1)
        call lrvlup(tokcontu(9),'&',1)
        call lrvlup(tokcontu(10),'|',1)
        call lrvlup(tokcontu(11),'<',1)
        call lrvlup(tokcontu(12),'>',1)
        call lrvlup(tokcontu(13),'=',1)
        tokequal = tokcontu(13)
        call lrvlup(tokcontu(14),'==',2)
        call lrvlup(tokcontu(15),'~',1)
        call lrvlup(tokcontu(16),'<=',2)
        call lrvlup(tokcontu(17),'>=',2)
        call lrvlup(tokcontu(18),'{',1)
        toklcurl = tokcontu(18)
        call lrvlup(tokcontu(19),'<<',2)
        call lrvlup(tokcontu(20),'>>',2)
        call lrvlup(tokcontu(21),':=',2)
        tokcontn = 21   #max value 25 currently
        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c MCNUM - function returning nesting depth of macros
c       - put here to encapsulate Macros references
#
	integer function mcnum ( )
Use(Macros)
	return(mdefcnt)
	end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c MCCTYP - initializes ctyp array in Macros (formerly in act.m)

	subroutine mcctyp
	integer i
Use(Macros)
Use(Lrh1)
        do i = 65, 90                  ## UPPER case letters
                ctyp(i) = ALPHA
        enddo
        do i= 97, 122                  ## lower case letters
                ctyp(i) = ALPHA
        enddo
        ctyp(36) = ALPHA               ## $
        ctyp(37) = ALPHA               ## %

        ctyp(32) = WHITE               ## blank
        ctyp(9) = WHITE                ## tab
        ctyp(10) = NEWLINE             ## linefeed
        ctyp(peol) = NEWLINE           ## \n
		return
	end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c MCEDTNAM - lists names of all declared macros 
c          - moved here from edt.m to encapsulate macro references

    subroutine mcedtnam(ius)

Use(Macros)
Use(Hash)
      integer ius
      integer nmsg, ok, namlen,jmac, jbody
      character*80 msg
      integer nstrgs, mcchrnam, ns, i, ihashfind, utgetcl
      external nstrgs, mcchrnam, ihashfind, utgetcl
      logical frstname
      Varname mname
 
      ns       = nstrgs()
      frstname = TRUE
      nmsg     = 1
      msg      = " "
 
      do i = 1, ns
         jmac = ihashfind(mactable, i-1)
         if (jmac > 0) then
            jbody = ihashget(mactable, jmac)
         else
            jbody = -1
         endif
         if (jbody >= 0) then # it is a macro name
            if (frstname) then
               call baswline(ius,"    user-defined macro names:")
               frstname = FALSE
            endif
            call strgget(i-1, mname)
            namlen = utgetcl(mname)
            if (mcchrnam(mname(1:namlen), msg, 80, nmsg, nmsg) == ERR) then
               call baswline(ius,msg)
               call ruthere
               msg = " "
               ok  = mcchrnam(mname(1:namlen), msg, 80, 1, nmsg)
            endif
            nmsg = nmsg+1 + mod(99-nmsg,9)
         endif
      enddo

      if (frstname) then
         call baswline(ius,"   there are no user-defined macros.")
         return
      endif

      call baswline(ius,msg)

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c MCEDTBDY - lists the definition of the given macro
c          - moved here from edt.m to encapsulate macro references

    subroutine mcedtbdy(jdef1, jdef2, argflg, ius)

Use(Macros)
Use(Lrh1)  #zcm 8/30/94 -- for peol char.

      integer jdef1, jdef2, ius, utgetcl, n, nattl, nnext, rtlattr, rtnattr
      logical argflg, bastestattr, mcparck
      Varname rattr
      external utgetcl, utcat, rtlattr, rtxattr, strglen, bastestattr, mcparck
      character*80 msg
      character * OUTLINE attrlist
      integer defindx, strglen, rtmacat
       external rtmacat

      msg = " "
      call strggetn(jdef1,msg,80,0,nnext,peol)
      if (nnext>=80) then
          call remark(
            "edtmcbdy:  macro name is over 80 characters(truncated).")
      endif
      call baswline(ius, msg)
      if (mcparck(jdef1)) then
          call baswline(ius, "    user-defined macro:  defined with arguments")
      else
          call baswline(ius,"    user-defined macro:  defined without arguments")
      endif
      call baswline(ius, "     definition is the following:")
      defindx = 0
      while(defindx < strglen(jdef2))
          msg = " "
          call strggetn(jdef2, msg, 80, defindx, defindx,peol)
          call baswline(ius, msg)
          call ruthere
      endwhile

      if (rtmacat(jdef1) ~= 0)then
*     MACRO HAS ATTRIBUTES:
	attrlist = "Attributes: "
	nattl = utgetcl(attrlist)
	nattl = nattl + mod(99-nattl, 9)
	do n = 1, rtnattr()
        if (bastestattr(rtmacat(jdef1),n))then
           nnext = rtlattr(n)
           if (nattl+nnext+1 > OUTLINE)then
          call baswline(ius,attrlist(1:nattl))
          call rtxattr(n, attrlist)
          nattl = nnext + 1
           else
          call rtxattr(n, rattr)
          call utcat(attrlist, nattl, attrlist, nattl, rattr,
          nnext, " ", 1)
           endif
           nattl = nattl + mod(99 - nattl, 9)
        endif
	enddo
	if (nattl > 0) call baswline(ius,attrlist(1:nattl))
      endif
      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c MCUDLIM - function which returns one of the user-defined delimiters
c	  - Put here by ZCM 7/91 to remove Macros reference from fcn.m

	Varname function mcudlim ( i )
	integer i

Use(Macros)

	return(userdlim(i))
	end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c MCSDLIMN - subroutine which sets delimnum

	subroutine mcsdlimn ( n )
	integer n

Use(Macros)

	delimnum = n
	return
	end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c MCSDLIM - subroutine which sets a user delimiter

	subroutine mcsdlim ( name, n )
	Varname name
	integer n

Use(Macros)

	userdlim(n) = name
	return
	end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c MCOOPS - reinitializes macro stuff after a kaboom

	subroutine mcoops

Use(Macros)

        argtnext = 1 # added 8/94
        mdefcnt = 0
        nameflg = 0
        comflg = 0
        comarg = 0
        exprparn = 0
        exprbrak = 0
        delimflg = FALSE
        expand = 0

	return
	end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

      subroutine mcscomtk(ttokn)
      character*(*) ttokn
      Use(Macros)

      comstrtk = ttokn

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c MCLRV122 - Parse the string of argument and delimiter designators which
c          - follows 'command_'. This revises a routine written by Brian
c          - Bonnlander which was originally a part of lrvact.
c          - This version uses bit positions in a word to define whether a
c          - particular delimiter is enabled or disabled: positions 0-9 for
c          - user delimiters 1-10; and 11-14 for white space,. comma, at, or
c          - equal, respectively. 

      subroutine mclrv122(ttokn,ltokn, minsiz)
Use(Macros)
      character*(*) ttokn
      integer ltokn, minsiz
      integer i, argcnt, istate, lparen, ic, ch, iact

      if (minsiz > mxcomsz) call mccomchg (minsiz)
         
      comflg = mxcomsz
      comarg = 1
      call mccomclr ()

      if(ltokn == 7)then
#        the token is 'command' alone, so we default to white space and comma
#        as delimiters, and expressions with macro expansion as arguments
         call mccomdef (comflg)
         delimflg = TRUE
         exprparn = 0
         exprbrak = 0
         return
      endif

      i = 9
#     the lexical analyzer has already ensured that the command string is
#     syntactically correct, so it has to have at least one character and
#     start at position 9.
      argcnt = 0
      istate = 1
      lparen = 0

      while(i <= ltokn)
         ic = ichar(ttokn(i:i)) - 39
         ch = ctype(ic)
         iact = atable(istate, iabs(ch))
         istate = stable(istate, iabs(ch))
         select(iact)
         case 1:
#           setting delimiters
            call mccomdel (argcnt, ch)
         case 2:
#           command_l
            if(argcnt == 0)then
               lflag = 1
               comflg = 1
            endif
            argcnt = argcnt + 1
            call smactyp (0, LINCOM, FALSE, argcnt)
            delimflg = TRUE
            exprparn = 0
            exprbrak = 0
            return
         case 3:
#           an argument is coming along
            call mcarg (argcnt, comflg, ch)
         case 4:
#           left paren
            if(lparen > 0)then
               call remark("mclrv122: too pany left parens in command string.")
               call kaboom(0)
            endif
            lparen = argcnt + 1
         case 5:
            call remark("mclrv122: incorrect command syntax.")
            call kaboom(0)
         case 6:
            break
         endselect
         i = i + 1
      endwhile

      call mctloop (lparen, argcnt)

      delimflg = TRUE
      exprparn = 0
      exprbrak = 0

      return

      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

#
#	mcwhite -- function to return token no. for white space
#	put here to help isolate Macros references (ZCM--7/91)
#
	integer function mcwhite ( )
Use(Macros)
	return ( tokwhite )
	end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

#
#	mccoml -- function to handle the command_l; moved here from lx.m
#	to help encapsulate Macros references. (ZCM, 7/91)
#
	integer function mccoml ( a , l )
	integer a, l
Use(Macros)
Use(Lrh1)
#       if we're in a command_l situation, the entire line gets enclosed
#       in quotes. lflag = 1 tells us to push back the current character
#       and return a quote instead (start of argument); lflag = 2 and peol
#       tells us to push back peol and return a quote (end of line and argu-
#       ment.
        if(lflag ~= 0)then
           if(lflag == 1)then
              if(l == DONE)then
#                empty argument is error
                 lflag = 0
                 return(ERR)
              endif
              lflag = 2
           elseif(a == peol)then
              lflag = 0
           else
              return(l)
           endif
           call lxpush(a)
           a = ichar('"')
           return(OK)
        endif
        return(l)
	end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

* Routine:   ISMAC
* Author:    Zane C. Motteler September 9, 1994
*
* Function:  To tell whether a name is the name of a macro.
*            returns 0 if it is not; returns jentry for
*            the macro body if it is.
*
* Arguments:
*            name =: the name to be looked up

      integer function ismac ( name )
      Ch ( * ) name
      Use(Hash)
      integer strgfind, ihashfind, ihashget, utgetcl
      external strgfind, ihashfind, ihashget, utgetcl
      integer jentry
      jentry = strgfind ( name(1:utgetcl(name)) )
      jentry = ihashfind(mactable,jentry)
      if ( jentry <= 0 ) return ( 0 )
      jentry = ihashget ( mactable, jentry )
      if ( jentry < 0 ) return ( 0 )
      return ( jentry )
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

* Routine:   LXMAC
* Author:    Susan Hockett
*
* Function:  to possibly change or reject the given token.  This allows
*            macros to be processed.  If the token is rejected, then ERR
*            is returned.  Normally, OK is returned.
*
* Arguments:
*
*            token    =: the type of token (integer value)
*            toktext  =: the bcd text of the token
*            lentok   =: the length in characters of the token
*            parstr   =: string determining where parser is parsing from
*            strptr   =: pointer into parstr
*            recur    =: signal whether recursion is active
*            lxptr    =: old UNTOP pointer
*
* Externals:
*            lxconlin, mccomexp, mccomstr, mceval
* Revised:
*            Zane C. Motteler -- August 1989
*            To incorporate recursion improvements and new parameters
*                 strptr, lxptr and recur
* Moved :
*            Zane C. Motteler -- July 1991
*            To help encapsulate all Macro processing in mac.m

        function lxmac(token,toktext,lentok,parstr,nstr,strptr,recur,lxptr)

# see if it's a macro, push macro definition onto unput stack
# handle automatic line continuation
        integer lxmac,lentok,token,strptr,recur, lxptr
        character*(*) toktext
#   character parstr(nstr)
        Address parstr
        integer retflg
        logical macflg
        integer lxconlin, mccomexp, mccomstr, utgetcl, lxback, ierr, gcomtyp
        external lxconlin, mccomexp, mccomstr, utgetcl, lxback, gcomtyp
        integer gfuncdep
        external gfuncdep
Use(Macros)
Use(Functions)
#### ----
      if (token == -1 & gfuncdep () == 0) then
         # The token is an ETX. discard it and ask for another.
         return (ERR)
      endif
      # process backtick if necessary 
      if (funcflag & token == tokback) then
         ierr = lxback (TRUE)
         if (ierr == ERR) return (ERR)
         if (ierr == DONE) return (OK)
      endif
      if (comflg > 0) then
         if (comarg > mxcomsz) call mclrv122(comstrtk, utgetcl(comstrtk),
                                             max(mxcomsz+COMINC,comarg))
         if (token == tokcarat) then
            call rcomtyp (comarg)
            return(ERR)
         elseif (token == tokcmstr) then
            call smovftof(toktext, 0, toktext, 1, lentok-2)
            lentok = lentok - 2
            return(OK)
         elseif (gcomtyp(comarg)==EXPRCOM) then
            retflg = mccomexp(token, toktext, lentok,parstr,nstr,strptr,recur, lxptr)
            if (retflg == ERR | token==tokwhite) return(retflg)
         elseif (gcomtyp(comarg)==STRCOM|gcomtyp(comarg)==LINCOM) then
            retflg=mccomstr(token, toktext, lentok, parstr,nstr,strptr,recur,lxptr)
            return(retflg)
         endif
      endif

      if( token == tokwhite) then
         return(ERR)
      elseif( token == tokstrng ) then
         call lxstr(token,toktext,lentok)
      elseif( token == -1 ) then
         token = tokcr
         return(DONE)
      elseif( token == tokcr ) then  #can be lf, ;, or \n
         retflg = lxconlin(token, toktext, lentok) # do line-continuation check
         if (retflg == OK) nameflg = 0          ## turn off protection for names
         return(retflg)
      elseif( token == toklcurl ) then
         expand = expand + 1
         if (expand == 1) return(ERR)
      elseif( token == tokrcurl ) then
         if( expand > 0) then
            expand = expand - 1
            if (expand == 0) return(ERR)
         endif
      elseif( token == tokname | token == tokgname |
              ctyp(ichar(toktext(1:1))) == ALPHA) then
        if( expand > 0 | nameflg <> 0) return(OK)
        if( expand < 0) then
           expand = 0 # one word protec
           return(OK)
        endif
        call mceval(token, toktext, lentok, macflg, parstr,nstr,strptr,recur,lxptr)
        if (macflg) then
           call ruthere
           return(ERR)
        endif    
      endif
      return ( OK )
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

#
#	mccont -- test for a continuation token
#	moved here from lx.m to help encapsulate all macro processing
#	(ZCM 7/91)
#
	integer function mccont ( lastntok )
	integer i, lastntok
Use(Macros)
        do i=1, tokcontn
                if(lastntok = tokcontu(i)) return(ERR)
        enddo
        return(OK)
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

#
#	gmcctyp -- function returns member of ctyp array
#	Created to help encapsulate all macro processing in mac.m
#	(ZCM 7/91)
#
	integer function gmcctyp ( i )
	integer i
Use(Macros)
	return(ctyp(i))
	end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

#
#	mc119 -- routine to force string collection
#	moved here from lx.m to encapsulate Macros references in mac.m
#	(ZCM 7/91)
#
	subroutine mc119
Use(Macros)
        integer dlim
	comflg = 1
	comarg = 1
        if (comarg > mxcomsz) call mccomchg(max(COMINC, comarg))
	# we expect a space before the string
	delimflg = TRUE
        dlim = 0
        dlim = ibset(dlim,DLIM_WS-1)
        call smactyp (dlim, STRCOM, TRUE, 1)
	return
	end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

#   
#   mc120 -- routine to protect list names from macro expansion
#   moved here from lx.m to encapsulate Macros references in mac.m
#   (ZCM 7/91)   
#
        subroutine mc120
Use(Macros)
	nameflg = 1     #protect names from macro expansion
	return
	end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

#
#   mc121 -- routine to stop name protection; called after a list
#   command has been reduced. (ZCM 12/13/95)
#
        subroutine mc121
Use(Macros)
    nameflg = 0      #stop protecting names from macro expansion
    return
    end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

#
#	The next three routines simply check to see if the token is 
#   int, octal, or hex. Placed here to remove Macro stuff from lx.m.
#
	logical function mcint(n)
	integer n
Use(Macros)
	return(n .eq. tokint)
	end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

    logical function mcoct(n) 
    integer n 
Use(Macros) 
    return(n .eq. tokoct)  
    end 

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

    logical function mchex(n)  
    integer n  
Use(Macros)  
    return(n .eq. tokhex)   
    end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

#
#	The next two routines return the token numbers for 'end' and 'name'
#	Placed here to remove macro references from ref.m.
#	(ZCM, 7/91)
#
	integer function mcendt()
Use(Macros)
	return(tokend)
	end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

    integer function mcname()
Use(Macros)
    return(tokname)
    end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

