c   subroutine / function  cot
c
c
CC+------------------------------------------------------------------+CC
CC+								     +CC
CC+		   Lawrence Livermore National Laboratory	     +CC
CC+								     +CC
CC+	   Mathematics and Statistics Division Software Library	     +CC
CC+								     +CC
CC+------------------------------------------------------------------+CC
CC+								     +CC
CC+  The SLATEC Mathematical Program Library is issued by	     +CC
CC+  the following:						     +CC
CC+								     +CC
CC+	  Air Force Weapons Laboratory, Albuquerque		     +CC
CC+	  Lawrence Livermore National Laboratory, Livermore	     +CC
CC+	  Los Alamos National Scientific Laboratory, Los Alamos	     +CC
CC+	  National Bureau of Standards, Washington		     +CC
CC+	  Oak Ridge National Laboratory, Oak Ridge		     +CC
CC+	  Sandia National Laboratories, Albuquerque		     +CC
CC+	  Sandia National Laboratories, Livermore		     +CC
CC+								     +CC
CC+  Please refer questions regarding this software to the	     +CC
CC+  MSD consulting office, Extension 3-2976.			     +CC
CC+								     +CC
CC+								     +CC
CC+		 * * * * * N O T I C E * * * * *		     +CC
CC+								     +CC
CC+  This material was prepared as an account of work sponsored	     +CC
CC+  by the United States Government.  Neither the United	     +CC
CC+  States Government, nor the Department of Defense, nor	     +CC
CC+  any of their employees, makes any warranty, expressed	     +CC
CC+  or implied, or assumes any legal liability or		     +CC
CC+  responsibility for the accuracy, completeness, or		     +CC
CC+  usefulness of any information, apparatus, product,		     +CC
CC+  or process disclosed, or represents its use would not	     +CC
CC+  infringe upon privately owned rights.			     +CC
CC+								     +CC
CC+------------------------------------------------------------------+CC
*deck cot
      function cot(x)
c***begin prologue  cot
c***date written   770601   (yymmdd)
c***revision date  861211   (yymmdd)
c***category no.  c4a
c***keywords  library=slatec(fnlib),
c	      type=single precision(cot-s dcot-d ccot-c),cotangent,
c	      degrees,elementary function,trigonometric
c***author  fullerton, w., (lanl)
c***purpose  computes the cotangent.
c***description
c
c cot(x) calculates the cotangent of the real argument x.  x is in
c units of radians.
c
c series for cot	on the interval	 0.	     to	 6.25000d-02
c					 with weighted error   3.76e-17
c					  log weighted error  16.42
c				significant figures required  15.51
c				     decimal places required  16.88
c***references	(none)
c***routines called  csevl,orinits,r1mach,baderr
c***end prologue  cot
      dimension cotcs(8)
      save cot cs, pi2rec, nterms, xmax, xsml, xmin, sqeps
      data cot cs( 1) /	   .2402591609 8295630e0 /
      data cot cs( 2) /	  -.0165330316 01500228e0 /
      data cot cs( 3) /	  -.0000429983 91931724e0 /
      data cot cs( 4) /	  -.0000001592 83223327e0 /
      data cot cs( 5) /	  -.0000000006 19109313e0 /
      data cot cs( 6) /	  -.0000000000 02430197e0 /
      data cot cs( 7) /	  -.0000000000 00009560e0 /
      data cot cs( 8) /	  -.0000000000 00000037e0 /
      data pi2rec / .01161977236 75813430 e0 /
      data nterms, xmax, xsml, xmin, sqeps / 0, 4*0.0 /
c***first executable statement	cot
      if (nterms.ne.0) go to 10
      nterms = orinits (cotcs, 8, 0.1*r1mach(3))
      xmax = 1.0/r1mach(4)
      xsml = sqrt (3.0*r1mach(3))
      xmin = exp ( amax1(alog(r1mach(1)), -alog(r1mach(2))) + 0.01)
      sqeps = sqrt (r1mach(4))
c
 10   y = abs(x)
      if (abs(x).lt.xmin) call baderr ( 'cot	 abs(x) is zero or so sm
     1all cot overflows')
      if (y.gt.xmax) call baderr (  'cot     no precision because abs(x)
     1 is big')
c
c carefully compute y * (2/pi) = (aint(y) + rem(y)) * (.625 + pi2rec)
c = aint(.625*y) + rem(.625*y) + y*pi2rec  =  aint(.625*y) + z
c = aint(.625*y) + aint(z) + rem(z)
c
      ainty = aint (y)
      yrem = y - ainty
      prodbg = 0.625*ainty
      ainty = aint (prodbg)
      y = (prodbg-ainty) + 0.625*yrem + y*pi2rec
      ainty2 = aint (y)
      ainty = ainty + ainty2
      y = y - ainty2
c
      ifn = amod (ainty, 2.)
      if (ifn.eq.1) y = 1.0 - y
c
      if (abs(x).gt.0.5 .and. y.lt.abs(x)*sqeps) call baderr (	    'cot
     1	   answer lt half precision, abs(x) too big or x near n*pi (n.ne
     2.0)')
c
      if (y.gt.0.25) go to 20
      cot = 1.0/x
      if (y.gt.xsml) cot = (0.5 + csevl (32.0*y*y-1., cotcs, nterms)) /y
      go to 40
c
 20   if (y.gt.0.5) go to 30
      cot = (0.5 + csevl (8.0*y*y-1., cotcs, nterms)) / (0.5*y)
      cot = (cot**2 - 1.0) * 0.5 / cot
      go to 40
c
 30   cot = (0.5 + csevl (2.0*y*y-1., cotcs, nterms)) / (0.25*y)
      cot = (cot**2 - 1.0) * 0.5 / cot
      cot = (cot**2 - 1.0) * 0.5 / cot
c
 40   if (x.ne.0.) cot = sign (cot, x)
      if (ifn.eq.1) cot = -cot
c
      return
      end
      
c   subroutine / function  dcot
c
c
CC+------------------------------------------------------------------+CC
CC+								     +CC
CC+		   Lawrence Livermore National Laboratory	     +CC
CC+								     +CC
CC+	   Mathematics and Statistics Division Software Library	     +CC
CC+								     +CC
CC+------------------------------------------------------------------+CC
CC+								     +CC
CC+  The SLATEC Mathematical Program Library is issued by	     +CC
CC+  the following:						     +CC
CC+								     +CC
CC+	  Air Force Weapons Laboratory, Albuquerque		     +CC
CC+	  Lawrence Livermore National Laboratory, Livermore	     +CC
CC+	  Los Alamos National Scientific Laboratory, Los Alamos	     +CC
CC+	  National Bureau of Standards, Washington		     +CC
CC+	  Oak Ridge National Laboratory, Oak Ridge		     +CC
CC+	  Sandia National Laboratories, Albuquerque		     +CC
CC+	  Sandia National Laboratories, Livermore		     +CC
CC+								     +CC
CC+  Please refer questions regarding this software to the	     +CC
CC+  MSD consulting office, Extension 3-2976.			     +CC
CC+								     +CC
CC+								     +CC
CC+		 * * * * * N O T I C E * * * * *		     +CC
CC+								     +CC
CC+  This material was prepared as an account of work sponsored	     +CC
CC+  by the United States Government.  Neither the United	     +CC
CC+  States Government, nor the Department of Defense, nor	     +CC
CC+  any of their employees, makes any warranty, expressed	     +CC
CC+  or implied, or assumes any legal liability or		     +CC
CC+  responsibility for the accuracy, completeness, or		     +CC
CC+  usefulness of any information, apparatus, product,		     +CC
CC+  or process disclosed, or represents its use would not	     +CC
CC+  infringe upon privately owned rights.			     +CC
CC+								     +CC
CC+------------------------------------------------------------------+CC
*deck dcot
      double precision function dcot(x)
c***begin prologue  dcot
c***date written   770601   (yymmdd)
c***revision date  861211   (yymmdd)
c***category no.  c4a
c***keywords  library=slatec(fnlib),
c	      type=double precision(cot-s dcot-d ccot-c),cotangent,
c	      degrees,elementary function,trigonometric
c***author  fullerton, w., (lanl)
c***purpose  compute the cotangent function.
c***description
c
c dcot(x) calculates the double precision trigonometric cotangent
c for double precision argument x.  x is in units of radians.
c
c series for cot	on the interval	 0.	     to	 6.25000e-02
c					 with weighted error   5.52e-34
c					  log weighted error  33.26
c				significant figures required  32.34
c				     decimal places required  33.85
c***references	(none)
c***routines called  d1mach,dcsevl,orinitds,baderr
c***end prologue  dcot
      double precision x, cotcs(15), ainty, ainty2, pi2rec, sqeps,
     1	xmax, xmin, xsml, y, yrem, prodbg,  dint, dcsevl, d1mach
      save cot cs, pi2rec, nterms, xmax, xsml, xmin, sqeps
      data cot cs(  1) / +.2402591609 8295630250 9553617744 970 d+0    /
      data cot cs(  2) / -.1653303160 1500227845 4746025255 758 d-1    /
      data cot cs(  3) / -.4299839193 1724018935 6476228239 895 d-4    /
      data cot cs(  4) / -.1592832233 2754104602 3490851122 445 d-6    /
      data cot cs(  5) / -.6191093135 1293487258 8620579343 187 d-9    /
      data cot cs(  6) / -.2430197415 0726460433 1702590579 575 d-11   /
      data cot cs(  7) / -.9560936758 8000809842 7062083100 000 d-14   /
      data cot cs(  8) / -.3763537981 9458058041 6291539706 666 d-16   /
      data cot cs(  9) / -.1481665746 4674657885 2176794666 666 d-18   /
      data cot cs( 10) / -.5833356589 0366657947 7984000000 000 d-21   /
      data cot cs( 11) / -.2296626469 6464577392 8533333333 333 d-23   /
      data cot cs( 12) / -.9041970573 0748332671 9999999999 999 d-26   /
      data cot cs( 13) / -.3559885519 2060006400 0000000000 000 d-28   /
      data cot cs( 14) / -.1401551398 2429866666 6666666666 666 d-30   /
      data cot cs( 15) / -.5518004368 7253333333 3333333333 333 d-33   /
      data pi2rec / .01161977236 7581343075 5350534900 57 d0 /
      data nterms, xmax, xsml, xmin, sqeps /0, 4*0.d0 /
c***first executable statement	dcot
      if (nterms.ne.0) go to 10
      nterms = orinitds (cotcs, 15, 0.1*sngl(d1mach(3)) )
      xmax = 1.0d0/d1mach(4)
      xsml = dsqrt (3.0d0*d1mach(3))
      xmin = dexp (dmax1(dlog(d1mach(1)), -dlog(d1mach(2))) + 0.01d0)
      sqeps = dsqrt (d1mach(4))
c
 10   y = dabs(x)
      if (y.lt.xmin) call baderr (  'dcot    dabs(x) is zero or so small
     1 dcot overflows')
      if (y.gt.xmax) call baderr ( 'dcot    no precision because dabs(x)
     1 is big')
c
c carefully compute y * (2/pi) = (aint(y) + rem(y)) * (.625 + pi2rec)
c = aint(.625*y) + rem(.625*y) + y*pi2rec  =  aint(.625*y) + z
c = aint(.625*y) + aint(z) + rem(z)
c
      ainty = dint (y)
      yrem = y - ainty
      prodbg = 0.625d0*ainty
      ainty = dint (prodbg)
      y = (prodbg-ainty) + 0.625d0*yrem + pi2rec*y
      ainty2 = dint (y)
      ainty = ainty + ainty2
      y = y - ainty2
c
      ifn = dmod (ainty, 2.0d0)
      if (ifn.eq.1) y = 1.0d0 - y
c
      if (dabs(x).gt.0.5d0 .and. y.lt.dabs(x)*sqeps) call baderr ( 'dcot
     1	  answer lt half precision, abs(x) too big or x near n*pi (n.ne.
     20)')
c
      if (y.gt.0.25d0) go to 20
      dcot = 1.0d0/x
      if (y.gt.xsml) dcot = (0.5d0 + dcsevl (32.0d0*y*y-1.d0, cotcs,
     1	nterms)) / y
      go to 40
c
 20   if (y.gt.0.5d0) go to 30
      dcot = (0.5d0 + dcsevl (8.d0*y*y-1.d0, cotcs, nterms))/(0.5d0*y)
      dcot = (dcot*dcot-1.d0)*0.5d0/dcot
      go to 40
c
 30   dcot = (0.5d0 + dcsevl (2.d0*y*y-1.d0, cotcs, nterms))/(.25d0*y)
      dcot = (dcot*dcot-1.d0)*0.5d0/dcot
      dcot = (dcot*dcot-1.d0)*0.5d0/dcot
c
 40   if (x.ne.0.d0) dcot = dsign (dcot, x)
      if (ifn.eq.1) dcot = -dcot
c
      return
      end

*deck orinits
      function orinits(os,nos,eta)
c***begin prologue  orinits
c***date written   770401   (yymmdd)
c***revision date  861211   (yymmdd)
c***category no.  c3a2
c***keywords  library=slatec(fnlib),
c	      type=single precision(orinits-s orinitds-d),chebyshev,
c	      initialize,orthogonal polynomial,orthogonal series,series,
c	      special functions
c***author  fullerton, w., (lanl)
c***purpose  initializes an orthogonal series so that it defines the
c	     number of terms to carry in the series to meet a specified
c	     error.
c***description
c
c initialize the orthogonal series so that orinits is the number of terms
c needed to insure the error is no larger than eta.  ordinarily, eta
c will be chosen to be one-tenth machine precision.
c
c	      input arguments --
c os	 array of nos coefficients in an orthogonal series.
c nos	 number of coefficients in os.
c eta	 requested accuracy of series.
c***references	(none)
c***routines called  baderr
c***end prologue  orinits
      dimension os(nos)
c***first executable statement	orinits
      if (nos.lt.1) call baderr
     >( 'orinits   number of coefficients lt 1')
c
      err = 0.
      do 10 ii=1,nos
	i = nos + 1 - ii
	err = err + abs(os(i))
	if (err.gt.eta) go to 20
 10   continue
c
 20   if (i.eq.nos) call baderr ( 'orinits   eta may be too small')
      orinits = i
c
      return
      end

*deck orinitds
      function orinitds(dos,nos,eta)
c***begin prologue  orinitds
c***date written   770601   (yymmdd)
c***revision date  861211   (yymmdd)
c***category no.  c3a2
c***keywords  library=slatec(fnlib),
c	      type=double precision(orinits-s orinitds-d),chebyshev,
c	      initialize,orthogonal polynomial,orthogonal series,series,
c	      special functions
c***author  fullerton, w., (lanl)
c***purpose  initializes the d.p. properly normalized orthogonal
c	     polynomial series to determine the number of terms needed
c	     for specific accuracy.
c***description
c
c initialize the double precision orthogonal series dos so that orinitds
c is the number of terms needed to insure the error is no larger than
c eta.	ordinarily eta will be chosen to be one-tenth machine precision
c
c	      input arguments --
c dos	 dble prec array of nos coefficients in an orthogonal series.
c nos	 number of coefficients in dos.
c eta	 requested accuracy of series.
c***references	(none)
c***routines called  baderr
c***end prologue  orinitds
c
      double precision dos(nos)
c***first executable statement	orinitds
      if (nos.lt.1) call baderr
     >( 'orinitds  number of coefficients lt 1')
c
      err = 0.
      do 10 ii=1,nos
	i = nos + 1 - ii
	err = err + abs(sngl(dos(i)))
	if (err.gt.eta) go to 20
 10   continue
c
 20   if (i.eq.nos) call baderr ( 'orinitds  eta may be too small')
      orinitds = i
c
      return
      end

*deck csevl
      function csevl(x,cs,n)
c***begin prologue  csevl
c***date written   770401   (yymmdd)
c***revision date  861211   (yymmdd)
c***category no.  c3a2
c***keywords  library=slatec(fnlib),
c	      type=single precision(csevl-s dcsevl-d),chebyshev,
c	      special functions
c***author  fullerton, w., (lanl)
c***purpose  evaluate the n-term chebyshev series cs at x.
c***description
c
c evaluate the n-term chebyshev series cs at x.	 adapted from
c r. broucke, algorithm 446, c.a.c.m., 16, 254 (1973). also see fox
c and parker, chebyshev polynomials in numerical analysis, oxford press,
c page 56.
c
c	input arguments --
c x    value at which the series is to be evaluated.
c cs   array of n terms of a chebyshev series.	in eval-
c      uating cs, only half the first coefficient is summed.
c n    number of terms in array cs.
c***references	(none)
c***routines called  baderr
c***end prologue  csevl
c
       dimension cs(1)
c***first executable statement	csevl
       if(n.lt.1) call baderr( 'csevl	number of terms le 0')
c        Corrected erroneous baderr call (FNF 12/17/96)
       if(n.gt.1000) call baderr ( 'csevl   number of terms gt 1000')
c      if(n.gt.1000) call baderr ( 'csevl   number of terms gt 1000',
c    1	 31,3,2)
       if (x.lt. -1.0 .or. x.gt. 1.0) call baderr( 'csevl   x outside (-
     11,+1)')
c
       b1=0.
       b0=0.
       twox=2.*x
       do 10 i=1,n
       b2=b1
       b1=b0
       ni=n+1-i
       b0=twox*b1-b2+cs(ni)
 10    continue
c
       csevl = 0.5 * (b0-b2)
c
       return
      end

*deck dcsevl
      double precision function dcsevl(x,a,n)
c***begin prologue  dcsevl
c***date written   770401   (yymmdd)
c***revision date  861211   (yymmdd)
c***category no.  c3a2
c***keywords  library=slatec(fnlib),
c	      type=double precision(csevl-s dcsevl-d),chebyshev,
c	      special functions
c***author  fullerton, w., (lanl)
c***purpose  evaluate the double precision n-term chebyshev series a
c	     at x.
c***description
c
c evaluate the n-term chebyshev series a at x.	adapted from
c r. broucke, algorithm 446, c.a.c.m., 16, 254 (1973).
c w. fullerton, c-3, los alamos scientific laboratory.
c
c	input arguments --
c x    double precision value at which the series is to be evaluated.
c a    double precision array of n terms of a chebyshev series.	 in
c      evaluating a, only half of the first coefficient is summed.
c n    number of terms in array a.
c***references	(none)
c***routines called  baderr
c***end prologue  dcsevl
c
       double precision a(n),x,twox,b0,b1,b2
c***first executable statement	dcsevl
       if(n.lt.1)call baderr( 'dcsevl  number of terms le 0')
       if(n.gt.1000) call baderr ( 'dcsevl  number of terms gt 1000')
       if ((x.lt.-1.d0) .or. (x.gt.1.d0)) call baderr ( 'dcsevl	 x outsi
     1de (-1,+1)')
c
       twox = 2.0d0*x
       b1 = 0.d0
       b0=0.d0
       do 10 i=1,n
	 b2=b1
	 b1=b0
	 ni = n - i + 1
	 b0 = twox*b1 - b2 + a(ni)
 10    continue
c
       dcsevl = 0.5d0 * (b0-b2)
c
       return
      end


