/*
 * mem.c
 *
 */

#include "rt.h"

static logical
    enabled = (logical) 0;      /* Is small object cache enabled? */

#ifdef PCK_SCORE

/*--------------------------------------------------------------------------*/

/*                       SCORE MEMORY MANAGER ROUTINES                      */

/*--------------------------------------------------------------------------*/

/* get space from the SCORE memory manager */

void Osallot(Address *ptr, Integer n)
{
    char msg[79];
    static char
        bmsg[] = "Memory has problems before allocation.",
        amsg[] = "Memory has problems after  allocation.";
    long ierr;

    if (memtrace == YES) {
        if (n > 1)
            sprintf(msg, "osallot: seeking %ld words.", (long) n);
        else
            sprintf(msg, "osallot: seeking 1 word.");
        Baspline(&iusout, msg, strlen(msg));
        ierr = SC_mem_chk(3);
        if (ierr < 0)
            Baspline(&iusout, bmsg, strlen(bmsg));
    }
    *ptr = (Address) SC_alloc(max(n, 1), sizeof(Integer), "Osallot:ptr");
    if (*ptr == (Address) NULL) {
        sprintf(msg, "osallot: error # %d allocating %ld words of memory.",
                ERR, (long) n);
        BasError(msg, strlen(msg));
        *ptr = (Address) NOTSET;
        Kaboom(0);
    }
    if (memtrace == YES) {
        if (n != 1)
            sprintf(msg, "........ %ld words at %p.", (long) n, *ptr);
        else
            sprintf(msg, "........ 1 word at %p.", *ptr);
        Baspline(&iusout, msg, strlen(msg));
        ierr = SC_mem_chk(3);
        if (ierr < 0)
            Baspline(&iusout, amsg, strlen(amsg));
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* get space from the SCORE memory manager
 * Same as Osallot but called with name
 */

void Osallotn(Address *ptr, Integer n, char *name)
{
    char msg[79];
    static char
        bmsg[] = "Memory has problems before allocation.",
        amsg[] = "Memory has problems after  allocation.";
    char myname[80];
    int namelen;
    long ierr;

    if (memtrace == YES) {
        namelen = min(strcspn(name, " "), 79);
        strncpy(myname, name, namelen);
        myname[namelen] = '\0';
        if (n > 1)
            sprintf(msg, "osallot: seeking %ld words for %s.", (long) n,
                    myname);
        else
            sprintf(msg, "osallot: seeking 1 word for %s.", myname);
        Baspline(&iusout, msg, strlen(msg));
        ierr = SC_mem_chk(3);
        if (ierr < 0)
            Baspline(&iusout, bmsg, strlen(bmsg));
    }
    *ptr = (Address) SC_alloc(max(n, 1), sizeof(Integer), name);
    if (*ptr == (Address) NULL) {
        sprintf(msg, "osallot: error # %d allocating %ld words of memory.",
                ERR, (long) n);
        BasError(msg, strlen(msg));
        *ptr = (Address) NOTSET;
        Kaboom(0);
    }
    if (memtrace == YES) {
        if (n != 1)
            sprintf(msg, "........ %ld words at %p.", (long) n, *ptr);
        else
            sprintf(msg, "........ 1 word at %p.", *ptr);
        Baspline(&iusout, msg, strlen(msg));
        ierr = SC_mem_chk(3);
        if (ierr < 0)
            Baspline(&iusout, amsg, strlen(amsg));
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* get space from the SCORE memory manager
 * Same as Osallotn but called with pointer to pointer to pointer
 */

void Osallot1(Address *ptr, Integer n, char *name)
{
    Address *myptr;

    myptr = (Address *) (*ptr);
    Osallotn(myptr, n, name);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

void Osmemchk(char *who_called)
{
    long ierr;
    char msg[80];

    ierr = SC_mem_chk(3);
    if (ierr < 0) {
        sprintf(msg,
                "osmemchk: called from %s; memory has problems.",
                who_called);
        Baspline(&iusout, msg, strlen(msg));
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Return space to SCORE memory manager */
void Osfree(Address *ptr)
{
    static char
        bmsg[] = "Trouble before memory freed.",
        amsg[] = "Trouble after memory freed.",
        cmsg[] = "....... success!";
    char msg[80];
    long ierr;

    if (*ptr == (Address) NOTSET)
        return;
    if (memtrace == YES) {
        sprintf(msg, "osfree: trying to free %p.", *ptr);
        Baspline(&iusout, msg, strlen(msg));
        ierr = SC_mem_chk(3);
        if (ierr < 0)
            Baspline(&iusout, bmsg, strlen(bmsg));
    }
    SC_free(*ptr);
    if (memtrace == YES) {
        Baspline(&iusout, cmsg, strlen(cmsg));
        ierr = SC_mem_chk(3);
        if (ierr < 0)
            Baspline(&iusout, amsg, strlen(amsg));
    }
    *ptr = (Address) NOTSET;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Return space to SCORE memory manager
 * Same as Osfree but is given name
 */

void Osfreen(Address *ptr, char *name)
{
    static char
        bmsg[] = "Trouble before memory freed.",
        amsg[] = "Trouble after memory freed.",
        cmsg[] = "....... success!";
    char msg[80];
    long ierr;
    char myname[80];
    int namelen;

    if (*ptr == (Address) NOTSET)
        return;
    if (memtrace == YES) {
        namelen = min(strcspn(name, " "), 79);
        strncpy(myname, name, namelen);
        myname[namelen] = '\0';
        sprintf(msg, "osfreen: trying to free %s at %p.", myname, *ptr);
        Baspline(&iusout, msg, strlen(msg));
        ierr = SC_mem_chk(3);
        if (ierr < 0)
            Baspline(&iusout, bmsg, strlen(bmsg));
    }
    SC_free(*ptr);
    if (memtrace == YES) {
        Baspline(&iusout, cmsg, strlen(cmsg));
        ierr = SC_mem_chk(3);
        if (ierr < 0)
            Baspline(&iusout, amsg, strlen(amsg));
    }
    *ptr = (Address) NOTSET;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* toggle small cache allocator on or off */

void Oscache(logical flag)
{
    if (flag)
        SC_configure_mm(csize, cnumber, cnumber, 1.1);
    enabled = flag;
    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Supposed to check integrity of small object cache, I guess */

void Oscachet()
{
    int ierr;
    char msg[120];
    static char
        emsg[] = "Small object cache is currently enabled.",
        dmsg[] = "Small object cache is currently disabled.",
        omsg[] = "Cache appears to be OK.",
        bmsg[] = "Cache appears to be damaged.",
        msg0[] = "No memory chunks have been allocated.",
        msg01[] = "1 memory chunk has been allocated.",
        msg1[] = "Forward and backward counts differ.",
        msg2[] = "There is a NULL pointer in the chain.",
        msg3[] = "The link count exceeds the number of blocks allocated.",
        msg4[] = "Checking freed blocks....";

    if (enabled) {
        Baspline(&iusout, emsg, strlen(emsg));
    } else {
        Baspline(&iusout, dmsg, strlen(dmsg));
    }
    ierr = SC_mem_trace();
    if (ierr >= 0) {
        Baspline(&iusout, omsg, strlen(omsg));
        if (ierr == 0)
            Baspline(&iusout, msg0, strlen(msg0));
        else if (ierr == 1)
            Baspline(&iusout, msg01, strlen(msg01));
        else {
            sprintf(msg, "%d memory chunks have been allocated.", ierr);
            Baspline(&iusout, msg, strlen(msg));
        }
    } else {
        Baspline(&iusout, bmsg, strlen(bmsg));
        if (ierr == -1)
            Baspline(&iusout, msg1, strlen(msg1));
        else if (ierr == -2)
            Baspline(&iusout, msg2, strlen(msg2));
        else if (ierr == -3)
            Baspline(&iusout, msg3, strlen(msg3));
        else {
            sprintf(msg, "Cache has unknown error number %d.", ierr);
            Baspline(&iusout, msg, strlen(msg));
        }
    }
    Baspline(&iusout, msg4, strlen(msg4));
    ierr = SC_mem_chk(2);
    sprintf(msg, "%d memory chunks have been freed.", ierr);
    Baspline(&iusout, msg, strlen(msg));
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Change space from SCORE memory manager */
void Oschange(Address *ptr, Integer newlen, Integer oldlen)
{
    char msg[79];
    static char
        bmsg[] = "Memory has problems before allocation.",
        amsg[] = "Memory has problems after  allocation.";
    long ierr;

    if (oldlen <= 0) {
        Osallot(ptr, newlen);
        return;
    }
    if (memtrace == YES) {
        sprintf(msg, "oschange: %p from %ld to %ld words.", *ptr,
                (long) oldlen, (long) newlen);
        Baspline(&iusout, msg, strlen(msg));
        ierr = SC_mem_chk(3);
        if (ierr < 0)
            Baspline(&iusout, bmsg, strlen(bmsg));
    }
    *ptr = SC_realloc(*ptr, (long) newlen, (long) sizeof(Integer));
    if (*ptr == (Address) NULL) {
        sprintf(msg,
                "......... error # %d allocating %ld words of memory.",
                ERR, (long) newlen);
        BasError(msg, strlen(msg));
        *ptr = (Address) NOTSET;
        Kaboom(0);
    }
    if (memtrace == YES) {
        sprintf(msg, "......... new block of %ld starts at %p.",
                (long) newlen, *ptr);
        Baspline(&iusout, msg, strlen(msg));
        ierr = SC_mem_chk(3);
        if (ierr < 0)
            Baspline(&iusout, amsg, strlen(amsg));
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Change space from SCORE memory manager
 * Same as oschange but has name as arg
 */
void Oschangn(Address *ptr, Integer newlen, Integer oldlen, char *name)
{
    char msg[79];
    static char
        bmsg[] = "Memory has problems before allocation.",
        amsg[] = "Memory has problems after  allocation.";
    long ierr;
    char myname[80];
    int namelen;

    if (oldlen <= 0) {
        Osallotn(ptr, newlen, name);
        return;
    }
    if (memtrace == YES) {
        namelen = min(strcspn(name, " "), 79);
        strncpy(myname, name, namelen);
        myname[namelen] = '\0';
        sprintf(msg, "oschange: %s at %p from %ld to %ld words.", myname,
                *ptr, (long) oldlen, (long) newlen);
        Baspline(&iusout, msg, strlen(msg));
        ierr = SC_mem_chk(3);
        if (ierr < 0)
            Baspline(&iusout, bmsg, strlen(bmsg));
    }
    *ptr = SC_realloc(*ptr, (long) newlen, (long) sizeof(Integer));
    if (*ptr == (Address) NULL) {
        sprintf(msg,
                "......... error # %d allocating %ld words of memory.",
                ERR, (long) newlen);
        BasError(msg, strlen(msg));
        *ptr = (Address) NOTSET;
        Kaboom(0);
    }
    if (memtrace == YES) {
        sprintf(msg, "......... new block of %ld starts at %p.",
                (long) newlen, *ptr);
        Baspline(&iusout, msg, strlen(msg));
        ierr = SC_mem_chk(3);
        if (ierr < 0)
            Baspline(&iusout, amsg, strlen(amsg));
    }
}

/*--------------------------------------------------------------------------*/

#else

/*--------------------------------------------------------------------------*/

/* non-SCORE memory manager data */
static Address scalslo = (Address) NOTSET;
static Address scalshi = (Address) NOTSET;

/* avail == -2 means not initialized yet; avail == -1 means full. */
static int avail = -2;          /* Next available item    */

/* scalsn [i] if >= 0 is index of next in chain; -1 means end of chain */
static int scalsn[cnumber];     /* Available list         */
static int scals[csize][cnumber];       /* The data itself        */

/*--------------------------------------------------------------------------*/

/*                     NON-SCORE MEMORY MANAGER ROUTINES                    */

/*--------------------------------------------------------------------------*/

void Osmemchk(char *who_called)
{
    static char
        msg1[] =
        "Osmemchk: service only available from SCORE memory manager.";
    char msg2[80];

    BasError(msg1, strlen(msg1));
    sprintf(msg2, "--called from %s.", who_called);
    BasError(msg2, strlen(msg2));
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Get space from the system memory manager */

void Osallot(Address *ptr, Integer n)
{
    int nextav;
    char msg[79];

    if (enabled) {
        if (n < csize & avail >= 0) {
            *ptr = &(scals[0][avail]);
            nextav = scalsn[avail];
            scalsn[avail] = -1;
            avail = nextav;
            return;
        }
    }

    *ptr = (Address) malloc(sizeof(Integer) * (n));
    if (*ptr == (Address) NULL) {
        sprintf(msg, "osallot: error # %d allocating %d words of memory.",
                ERR, n);
        BasError(msg, strlen(msg));
        *ptr = (Address) NOTSET;
        Kaboom(0);
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Get space from the system memory manager
 */

void Osallotn(Address *ptr, Integer n, char *name)
{

    int nextav;
    char msg[79];
    char myname[80];
    int namelen;

    if (enabled) {
        if (n < csize & avail >= 0) {
            *ptr = &(scals[0][avail]);
            nextav = scalsn[avail];
            scalsn[avail] = -1;
            avail = nextav;
            return;
        }
    }

    *ptr = (Address) malloc(sizeof(Integer) * (n));
    if (*ptr == (Address) NULL) {
        namelen = min(strcspn(name, " "), 79);
        strncpy(myname, name, namelen);
        myname[namelen] = '\0';
        sprintf(msg,
                "osallot: error # %d allocating %d words of memory for %s.",
                ERR, n, myname);
        BasError(msg, strlen(msg));
        *ptr = (Address) NOTSET;
        Kaboom(0);
    }
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Return space to system memory manager.
 */

void Osfree(Address *ptr)
{
    int j;
    static char
        msg1[] = "Small element cache bad attempt to free memory.",
        msg2[] = "Most likely the same pointer freed twice.";
    char errmsg[OUTLINE];

    if (*ptr == (Address) NOTSET)
        return;
    if (*ptr <= scalshi)
        if (*ptr >= scalslo) {
            /* Space is in small object cache */
            j = (*ptr - scalslo) / (csize * LOCS_PER_WORD);
            if (scalsn[j] != -1) {
                Remark(msg1, strlen(msg1));
                Remark(msg2, strlen(msg2));
                sprintf(errmsg,
                        "osfree: error # %d deallocating memory at 0x%x.",
                        j, ptr);
                BasError(errmsg, strlen(errmsg));
                Kaboom(0);
            }
            scalsn[j] = avail;
            avail = j;
            *ptr = (Address) NOTSET;
            return;
        }
    /* Space is in memory manager memory */
    free(*ptr);
    *ptr = (Address) NOTSET;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Return space to system memory manager.
 */
void Osfreen(Address *ptr, char *name)
{
    int j;
    static char
        msg1[] = "Small element cache bad attempt to free memory.",
        msg2[] = "Most likely the same pointer freed twice.";
    char errmsg[OUTLINE];
    char myname[80];
    int namelen;

    if (*ptr == (Address) NOTSET)
        return;
    if (*ptr <= scalshi)
        if (*ptr >= scalslo) {
            /* Space is in small object cache */
            j = (*ptr - scalslo) / (csize * LOCS_PER_WORD);
            if (scalsn[j] != -1) {
                Remark(msg1, strlen(msg1));
                Remark(msg2, strlen(msg2));
                namelen = min(strcspn(name, " "), 79);
                strncpy(myname, name, namelen);
                myname[namelen] = '\0';
                sprintf(errmsg,
                        "osfree: error # %d deallocating memory for %s at 0x%x.",
                        j, myname, ptr);
                BasError(errmsg, strlen(errmsg));
                Kaboom(0);
            }
            scalsn[j] = avail;
            avail = j;
            *ptr = (Address) NOTSET;
            return;
        }
    /* Space is in memory manager memory */
    free(*ptr);
    *ptr = (Address) NOTSET;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Change space from system memory manager.
 */
void Oschange(Address *ptr, Integer newlen, Integer oldlen)
{
    Address *oldptr;

    if (oldlen <= 0) {
        Osallot(ptr, newlen);
        return;
    }

    oldptr = ptr;
    Osallot(ptr, newlen);
    memcpy(ptr, oldptr, min(oldlen, newlen));
    Osfree(oldptr);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Change space from system memory manager */
void Oschangn(Address *ptr, Integer newlen, Integer oldlen, char *name)
{
    Address *oldptr;

    if (oldlen <= 0) {
        Osallot(ptr, newlen);
        return;
    }

    oldptr = ptr;
    Osallotn(ptr, newlen, name);
    memcpy(ptr, oldptr, min(oldlen, newlen));
    Osfree(oldptr);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Turn small object allocator on or off */
void Oscache(logical flag)
{
    int i;

    if (flag) {
        if (avail == -2) {
            /* This is the first enabling */
            avail = 0;
            for (i = 0; i < cnumber - 1; i++)
                scalsn[i] = i + 1;
            /* end of chain results in avail being -1 */
            scalsn[cnumber - 1] = -1;
            scalslo = &(scals[0][0]);
            scalshi = &(scals[0][cnumber - 1]);
            /* Fill up a word to detect overwrites */
            for (i = 0; i < cnumber; i++)
                scals[csize - 1][i] = -12341234;
        }
    }
    enabled = flag;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* Check integrity of small object cache */
void Oscachet()
{
    char msg[120];
    static char
        emsg[] = "Small object cache is currently enabled.",
        dmsg[] = "Small object cache is currently disabled.";
    int jfree, i;

    if (enabled)
        Baspline(&iusout, emsg, strlen(emsg));
    else
        Baspline(&iusout, dmsg, strlen(dmsg));

    for (i = 0; i < cnumber; i++)
        if (scals[csize - 1][i] != -12341234) {
            sprintf(msg, "Small object cache object %d at %x overstrided.",
                    i, &(scals[0][i]));
            Baspline(&iusout, msg, strlen(msg));
        }
    i = avail;
    jfree = 0;
    while (i != -1 && i != -2) {
        if (scalsn[i] < -1 || scalsn[i] >= cnumber) {
            sprintf(msg,
                    "Small object cache damaged at slot %d, crash soon!",
                    i);
            Baspline(&iusout, msg, strlen(msg));
            return;
        }
        jfree += 1;
        i = scalsn[i];
    }
    sprintf(msg, "Small object cache linking ok, %d of &d slots free.",
            jfree, cnumber);
    Baspline(&iusout, msg, strlen(msg));
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

#endif
