/*
 * NAMEME.C
 *
 */

#include "rt.h"

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 * Return a program's "realname" and "realpath".  The "argv0" argument
 * is typically argv[0].  The function skips over any symbolic links on
 * the way to the final name of the executable.  It uses the PATH
 * environment variable to find the executable if "argv0" is not an
 * absolute pathname.  After the final pathname to the executable is
 * decided upon, "realname" is set to the last component of the
 * pathname, and "realpath" to its prefix.
 */
void Nameme0(char *argv0, char *realname, char *realpath)
{
    int findname();
    char *getenv(), *cp, *pv0[256], **pv, path[1024];
    void pathsplit();

    pathsplit(pv0, getenv("PATH"));
    if (strchr(argv0, '/') == 0) {      /* We have a relative pathname */
#ifdef DEBUG
        fprintf(stderr, "Relative pathname search begins ...\n");
#endif
        for (pv = pv0; *pv; pv++) {
#ifdef DEBUG
            fprintf(stderr, "Current PATH component is %s\n", *pv);
#endif
            sprintf(path, "%s/%s", *pv, argv0);
            if (findname(path) == 0)
                break;
        }
    } else {                    /* We have an absolute pathname */
#ifdef DEBUG
        fprintf(stderr, "Absolute pathname: %s\n", argv0);
#endif
        strcpy(path, argv0);
        (void) findname(path);
    }

    for (pv = pv0; *pv; pv++)
#ifdef PCK_SCORE
        SFREE(*pv);
#else
        free(*pv);
#endif

    cp = strrchr(path, '/');
    strcpy(realname, 1 + cp);
    *cp = '\0';
    strcpy(realpath, (*path == '\0') ? "/" : path);
#ifdef DEBUG
    fprintf(stderr, "Realname found as: %s\n", realname);
    fprintf(stderr, "Realpath found as: %s\n", realpath);
#endif
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *  Given a colon-separated list of path components in "path", generate
 * a vector of path components.  Null components are returned as '.'.
 * The returned vector is null terminated.
 */
void pathsplit(char **pv, char *path)
{
    int nc;
    char *p = path, *p2;

#ifdef DEBUG
    fprintf(stderr, "Initial PATH: %s\n", path);
#endif
    while (*p != '\0') {
        if (*p == ':') {
            if (p == path || *(p + 1) == ':' || *(p + 1) == '\0') {     /* Null entry */

#ifdef PCK_SCORE
                *pv = FMAKE_N(char, 2, "PATHSPLIT:pv");
#else
                *pv = (char *) malloc(2);
#endif

                strcpy(*pv, ".");
                pv++;
                p++;
            } else {
                p2 = ++p;
                while (*p2 != ':' && *p2 != '\0')
                    ++p2;
                nc = (p2 - p) + 1;

#ifdef PCK_SCORE
                *pv = FMAKE_N(char, nc + 1, "PATHSPLIT:pv");
#else
                *pv = (char *) malloc(p2 - p + 1 + 1);
#endif
                strncpy(*pv, p, nc);
                (*pv)[nc - 1] = '\0';
                pv++;
                p = p2;
            }
        } else {
            p2 = p;
            while (*p2 != ':' && *p2 != '\0')
                ++p2;
            nc = (p2 - p) + 1;

#ifdef PCK_SCORE
            *pv = FMAKE_N(char, nc + 1, "PATHSPLIT:pv");
#else
            *pv = (char *) malloc(p2 - p + 1 + 1);
#endif
            strncpy(*pv, p, nc);
            (*pv)[nc - 1] = '\0';
            pv++;
            p = p2;
        }
    }
    *pv = (char *) NULL;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/*
 *  Given a pathname, determine if it is a symbolic link.  If so,
 * continue searching to the ultimate terminus - there may be
 * more than one link.  Use the error value to determine when the
 * terminus is reached, and to determine if the pathname really
 * exists.  Then stat it to determine whether it's executable.
 * Return 0 for an executable, errno otherwise.  Note that 'p'
 * *must* have at least one '/' character - it does by construction
 * in this program.  The contents of the array pointed to by 'p'
 * are changed to the actual pathname if findname is successful.
 */
int findname(char *p)
{
    int n;
    char buf[1024], *cp;
    extern int errno;
    struct stat sbuf;

    while ((n = readlink(p, buf, 1024)) > 0) {
#ifdef DEBUG
        fprintf(stderr, "Readlink read %d chars at: %s\n", n, p);
#endif
        if (buf[0] == '/') {    /* Link is an absolute path */
            strncpy(p, buf, n);
            p[n] = '\0';
#ifdef DEBUG
            fprintf(stderr, "Link is absolute: %s\n", p);
#endif
        } else {                /* Link is relative to its directory; make it absolute */
            cp = 1 + strrchr(p, '/');
            strncpy(cp, buf, n);
            cp[n] = '\0';
#ifdef DEBUG
            fprintf(stderr, "Link is relative: %s\n\tTotal path: %s\n", cp,
                    p);
#endif
        }
    }

/* SGI machines return ENXIO instead of EINVAL Dubois 11/92 */
    if (errno == EINVAL || errno == ENXIO) {
#ifdef DEBUG
        fprintf(stderr, "%s may be the one ...", p);
#endif
        if ((stat(p, &sbuf) == 0) && S_ISREG(sbuf.st_mode)) {
#ifdef DEBUG
            fprintf(stderr, "regular file\n");
#endif
            return (access(p, X_OK));
        }
    }
#ifdef DEBUG
    fprintf(stderr, "not executable\n");
#endif
    return (errno ? errno : -1);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
