c
c PRT.M - routines for printing variables and other runtime state
c
c Copyright 1986 The Regents of the University of California
c

define IRPART 20
# Strings get formatted here by prepending an "index range part".  It
# varies in length, but 12-16 characters is typical.  To preserve the
# ability to print strings that are MAXSTRING long, the buffer needs
# to be a bit longer.  Hope the various Fortran compilers buy this.

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c UTGETPREL - get the elements to print in the next line
c           - return IR and VAR

      subroutine utgetprel(ir, var, pvar, var1, ityp, ni, lc, inc, nel)
      implicit none
      integer ir
      double precision var(*)
      Address pvar
      double precision var1(*)
      integer ityp, ni, lc, inc, nel

Use(Controls)

      integer ok, jr, kr

      integer mcompare
      external mcompare

      ir = 1
      call mvalue(pvar, lc, ityp, var, ni)

      if (compress .eq. YES) then
         jr = inc*ir + lc
         kr = jr + inc
         while (kr .lt. nel)
            call mvalue(pvar, jr, ityp, var1, ni)

            ok = mcompare(var, var1, ityp, ni)
            if (ok .ne. 0) return

            ir = ir + 1
            jr = kr
            kr = kr + inc
         endwhile
      endif

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c UTGETPRER - get a row elements to print in the next line
c           - return IR and VAR

      subroutine utgetprer(ir, var, pvar, var1, ityp,
     &                     row, ni, nn, lr, lc, nel)
      implicit none
      integer ir
      double precision var(*)
      Address pvar
      double precision var1(*)
      integer ityp, row, ni, nn, lr, lc, nel

Use(Controls)

      integer ok

      integer mcompare
      external mcompare

      ir = 1
      call mvaluer(pvar, nn, ityp, var, ni, row)

      if (compress .eq. YES) then
         while (lr+ir .le. nel)
            nn = (lc - 1)*row + lr + ir
            call mvaluer(pvar, nn, ityp, var1, ni, row)

            ok = mcompare(var, var1, ityp, ni)
            if (ok .ne. 0) return

            ir = ir + 1
         endwhile
      endif

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c UTMKRN - prepare the range part of a line of text

      subroutine utmkrn(msg, ismx, inf, inl, inle, ins, ir, lp)
      implicit none
      character*(*) msg
      integer ismx, inf, inl, ins, inle, ir, lp

      integer i, mc, fw, ine
      character*OUTLINE fmt

      integer utgetcl
      external utgetcl

c ... form " (inf:inl:ins) "
      if (ins .ne. 1) then
         if (ins .gt. 1) then
            ine = min(inle, inl)
         else
            ine = max(inle, inl)
         endif

         call utsprint(1, 1, OUTLINE, msg, 11, " (%d:%d:%d)", inf, ine, ins)

c ... form " (inf:inl) "
      elseif (ir .ne. 1) then
         call utsprint(1, 1, OUTLINE, msg, 8, " (%d:%d)", inf, inl)

c ... form " (inf) "
      else
         call utsprint(1, 1, OUTLINE, msg, 5, " (%d)", inf)
      endif

c ... pad out to lp
      fw = ismx + 1
      mc = utgetcl(msg)
      mc = min(mc, lp-1)
      do i = mc+1, lp
         msg(i:i) = " "
      enddo
      msg(i:i) = char(0)

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c UTPRCOLINF - print out the column info heading

      subroutine utprcolinf(unit, corigin, cinc, lf, ll, fw, lp)
      implicit none
      integer unit, corigin, cinc, lf, ll, fw, lp

      integer i, id, mc, nc, last
      character*(OUTLINE) msg, fmt

      integer utgetcl
      external utgetcl

      fmt = " "
      call utsprint(1, 1, OUTLINE, fmt, 7, "  %%%dd", fw)
      nc = utgetcl(fmt)

ifelse(SYSTEM,IRIX64|AXP|SOL|HP700|LINUXA,[
      call utsprint(1, 1, OUTLINE, msg, 11, ' row\\col =')
],[
      call utsprint(1, 1, OUTLINE, msg, 11, " row\\\\col =")
])

c ... pad out to lp
      mc = 11
      mc = min(mc, lp-1)
      do i = mc+1, lp
         msg(i:i) = " "
      enddo
      msg(i:i) = char(0)

      last = 0
      do i = lf, ll
         if (i .eq. ll) then
            last = 1
         endif
         id = corigin + (i-1)*cinc
         call utsprint(0, last, OUTLINE, msg, nc, fmt, id)
      enddo

      call baswline(unit, msg)

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c UTFMTDESC - load up descriptive info for printing various data types
c           - in sundry formats
c           - NOTE: almost everything that you might want to do to change
c           - the look of the output should be done right here

      subroutine utfmtdesc(id, ftyp, unit,
     &                     cf, cl, cs, rf, rl, rs, ityp, pvar)
      implicit none
      integer id(*)
      character ftyp
      integer unit, cf, cl, cs, rf, rl, rs, ityp
      Address pvar

Use(Controls)

      integer ln, lx, maxlen, epl, fw
      integer lf, ll, ls, lp, re, ncl, ncv
      integer i1n, i1x, i2n, i2x, i3n, i3x
      integer m1, m2, m3
      integer intwid, rngwid
      real dpop, dphp
      character cm*24, ieff*9

      integer imkstr
      external imkstr

# "-1.23456e+00" is 12 characters long.  Five characters are fuzz, leaving 7.
# That is, Everything Except The Fraction is (at least) 7 characters long.
define EETF 7

      dpop = 1.107309365        #  1.0/log10(8)
      dphp = 0.8304820235       #  1.0/log10(16)

      fuzz = min0(max0(fuzz, 0), 16)
      re   = rf + (rl-1)*rs
      lf   = imkstr(rf, ieff)
      ll   = imkstr(re, ieff)
      ls   = imkstr(rs, ieff)

      lp  = lf + ll + ls + 4
      lp  = min(lp, rngespce)
      lp  = max(lp, 12)
      ncl = OUTLINE
      ncv = ncl - lp

      intwid = CHAR_PER_WORD
      if (ityp .eq. INTEGER) then
         call cminmax(pvar, cl*rl, i1x, i1n)
         if (cs >= 1) then
            i1n = min0(i1n, cf)
            i1x = max0(i1x, (cl-1)*cs + cf)
         else
            i1x = max0(i1x, cf)
            i1n = min0(i1n, (cl-1)*cs + cf)
         endif
         ln     = imkstr(i1n, cm)
         lx     = imkstr(i1x, cm)
         intwid = max(ln, lx)
      elseif (ityp .eq. RANGE) then
         call cminmaxr(pvar, cl*rl, i1x, i1n, i2x, i2n, i3x, i3n)
         ln     = imkstr(i1n, cm)
         lx     = imkstr(i1x, cm)
         m1     = max(ln, lx)
         ln     = imkstr(i2n, cm)
         lx     = imkstr(i2x, cm)
         m2     = max(ln, lx)
         ln     = imkstr(i3n, cm)
         lx     = imkstr(i3x, cm)
         m3     = max(ln, lx)
         rngwid = max(m1, m2)
      endif

c ... octal format
      if (formdesc = 'oct') then
         ftyp   = "o"
         maxlen = 8*CHAR_PER_WORD / 3
         if (mod(8*CHAR_PER_WORD,3) ~= 0) maxlen = maxlen + 1

         if (ityp .lt. 0) then
            maxlen = 3
            fw     = maxlen

         elseif (ityp .eq. LOGICAL) then
            maxlen = 1
            fw     = maxlen

         elseif (ityp .eq. RANGE) then
            m1 = rngwid*dpop + 1
            m2 = m1
            m3 = m1
            fw = m1 + m2 + m3 + 2

         elseif (ityp .eq. REAL) then
            fw = maxlen

         elseif (ityp .eq. DOUBLE) then
            fw = 2*maxlen + 1

         elseif (ityp .eq. COMPLEX) then
            fw = 2*maxlen + 1

         elseif (ityp .eq. DOUBLE_C) then
            fw = 4*maxlen + 3

         else
            maxlen = intwid*dpop + 1
            fw     = maxlen
         endif

c ... hexadecimal format
      elseif (formdesc = 'hex') then
         ftyp   = "X"
         maxlen = 2*CHAR_PER_WORD

         if (ityp .lt. 0) then
            maxlen = 3
            fw     = maxlen

         elseif (ityp .eq. LOGICAL) then
            maxlen = 1
            fw     = maxlen

         elseif (ityp .eq. RANGE) then
            m1 = rngwid*dphp + 1
            m2 = m1
            m3 = m1
            fw = m1 + m2 + m3 + 2

         elseif (ityp .eq. REAL) then
            fw = maxlen

         elseif (ityp .eq. DOUBLE) then
            fw = 2*maxlen + 1

         elseif (ityp .eq. COMPLEX) then
            fw = 2*maxlen + 1

         elseif (ityp .eq. DOUBLE_C) then
            fw = 4*maxlen + 3

         else
            maxlen = intwid*dphp + 1
            fw     = maxlen
         endif

c ... decimal format
      else
         if (ityp .lt. 0) then
            ftyp   = "d"
            maxlen = -ityp + 2
            fw     = maxlen

         elseif (ityp .eq. INTEGER) then
            ftyp   = "d"
            maxlen = intwid
            fw     = maxlen

         elseif (ityp .eq. LOGICAL) then
            ftyp   = "d"
            maxlen = 1
            fw     = maxlen

         elseif (ityp .eq. RANGE) then
            ftyp = "d"
            fw   = m1 + m2 + m3 + 2

         elseif (ityp .eq. FWA) then
            ftyp   = "d"
            maxlen = intwid
            fw     = maxlen

         elseif (ityp .eq. REAL) then
            ftyp   = "E"
            maxlen = fuzz + EETF
            fw     = maxlen

         elseif (ityp .eq. COMPLEX) then
            ftyp   = "E"
            maxlen = fuzz + EETF
            fw     = 2*maxlen + 1

         elseif (ityp .eq. DOUBLE) then
            ftyp   = "D"
            maxlen = fuzz + EETF
            fw     = maxlen

         elseif (ityp .eq. DOUBLE_C) then
            ftyp   = "D"
            maxlen = fuzz + EETF
            fw     = 2*maxlen + 1
         endif
      endif

c ... GOTCHA: must have at least one entry per line, but
c ... a character string longer than OUTLINE-lp will blow 
c ... the memory
c ... NOT clear at the moment what to do, when it blows
c ... we'll obtain more info to decide
      epl = ncv/(fw + 2)
      epl = max(epl, 1)

      id(1) = unit
      id(2) = ityp
      id(3) = re                  # last effective index
      id(4) = fw                  # full width of a single entry
      id(5) = epl                 # entries per line (ieg spaces between)
      id(6) = max(lf, ll)         # width of index range entry
      id(7) = lp                  # pad between indices and values
      id(8) = netnormord          # byte ordering flag
      id(9) = 1                   # flag to print strings with quotes
      id(13) = 2		  # ieg, inter-element gap
      id(14) = 0		  # stream i/o flag

      if (ityp .eq. RANGE) then
         id(10) = m1
         id(11) = m2
         id(12) = m3

      elseif ((ityp .eq. REAL) .or. (ityp .eq. DOUBLE) .or.
     &        (ityp .eq. COMPLEX) .or. (ityp .eq. DOUBLE_C)) then
         id(10) = maxlen
         id(11) = fuzz

      else
         id(10) = maxlen

      endif

Undefine([EETF])
      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PRSTR - dispatch printing to correct type

      subroutine prstr(msg, ftyp, off, id, ni, var)
      implicit none
      character*(*) msg
      character ftyp
      integer off, id(*), ni
      double precision var(*)

      integer ityp

      ityp = id(2)

      if (ityp .lt. 0) then
         call prchstr(msg, ftyp, off, id, ni, var)

      elseif (ityp .eq. INTEGER) then
         call pristr(msg, ftyp, off, id, ni, var)

      elseif (ityp .eq. RANGE) then
         call prrngstr(msg, ftyp, off, id, ni, var)

      elseif (ityp .eq. LOGICAL) then
         call prlstr(msg, ftyp, off, id, ni, var)

      elseif (ityp .eq. REAL) then
         call prrstr(msg, ftyp, off, id, ni, var)

      elseif (ityp .eq. DOUBLE) then
         call prdstr(msg, ftyp, off, id, ni, var)

      elseif (ityp .eq. COMPLEX) then
         call prcstr(msg, ftyp, off, id, ni, var)

      elseif (ityp .eq. DOUBLE_C) then
         call prdcstr(msg, ftyp, off, id, ni, var)

      elseif (ityp .eq. FWA) then
         call prastr(msg, ftyp, off, id, ni, var)

      endif

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PRTTRC - print something for trace
c        -
c        -   UNIT     = unit number for write
c        -   TYPE     = variable type code
c        -   PVAR     = ptr variable to be written

      subroutine prttrc(unit, type, ne, pvar)
      implicit none
      integer unit, type, ne
      Address pvar

Use(Controls)

      integer ir, nc, fw
      integer id(20)
      character ftyp
      character*VARNAMESIZE name
      character*Evaluate(MAXSTRING+IRPART) msg
      double precision var(128), var1(128)

      integer utgetcl
      external utgetcl

      call utfmtdesc(id, ftyp, unit, 1, 1, 1, 1, ne, 1,
     &               type, pvar)

c ... get the data into var
      call utgetprel(ir, var, pvar, var1, type, ne, 1, 1, 1)

c ... make the initial part of the line
      call utsprint(1, 0, OUTLINE, msg, 1, " ")

c ... make the part of the line with the values
      call prstr(msg, ftyp, 9, id, ne, var)

c ... print the line
      call baswline(unit, msg)

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PRT0DV - print a scalar variable
c        -
c        -   UNIT     = unit number for write
c        -   TYPE     = variable type code
c        -   PVAR     = ptr to scalar variable to be written
c        -   VNAME    = variable name
c        -   COMM     = variable comment character string

      subroutine prt0dv(unit, type, pvar, vname, comm)
      implicit none
      integer unit, type
      Address pvar
      character*(*) vname, comm

Use(Controls)

      integer ir, nc, fw
      integer id(20)
      character ftyp
      character*VARNAMESIZE name
      character*Evaluate(MAXSTRING+IRPART) msg
      double precision var(128), var1(128)

      integer utgetcl
      external utgetcl

      msg = " "

      call utfmtdesc(id, ftyp, unit, 1, 1, 1, 1, 1, 1, type, pvar)

      name = " "
      if (vname .eq. "$") then
         name = comm
      elseif ((vname .eq. "<?>") .or. (vname .eq. "?")) then
         if (comm .ne. " ") then
            name = comm
         else
            name = vname
         endif
      else
         name = vname
      endif
      nc = utgetcl(name)

c ... check whether name expression is too long
      fw = id(4)
      if (nc + fw + 3 .gt. OUTLINE) then
         nc = OUTLINE - fw - 5
         if (nc .gt. 12) then
            name(nc-3:nc-3) = "."
            name(nc-2:nc-2) = "."
            name(nc-1:nc-1) = "."
            name(nc:)       = " "
         endif
      endif
      nc = utgetcl(name)

c ... now print a scalar
      call utgetprel(ir, var, pvar, var1, type, 1, 1, 1, 1)

c ... make the initial part of the line
      call utsprint(1, 0, OUTLINE, msg, 7, "%.*s = ", nc, name)

c ... make the part of the line with the value
      call prstr(msg, ftyp, 9, id, 1, var)

c ... print the line
      call baswline(unit, msg)

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PRT1DV - print a 1d array variable
c        - print a maximum of EPL elements per line
c        - the row count is printed down the left side of the page
c        - and is the index count of the first variable in each row
c        -
c        -   UNIT  unit number for write
c        -   PVAR  ptr to one dimensional variable to be written
c        -   RF    index of first entry
c        -   RL    index of last effective entry
c        -   RS    step by which the data was gathered

      subroutine prt1dv(unit, type, pvar, rf, rl, rs)
      implicit none
      integer unit, type
      Address pvar
      integer rf, rl, rs

Use(Controls)

      integer re, epl, ifw
      integer lf, lc, ni, ll, kk, ir, lp
      integer id(20)
      character*Evaluate(MAXSTRING+IRPART) msg
      character ftyp
      double precision var(128), var1(128)

      msg = " "

      call utfmtdesc(id, ftyp, unit, 1, 1, 1, rf, rl, rs,
     &               type, pvar)

      re  = id(3)
      epl = id(5)
      ifw = id(6)
      lp  = id(7)

c ... now print 1D array
      lc = 1
      lf = rf
      while (lc .le. rl)
         kk = min0(lc+epl-1, rl)
         ni = kk - lc + 1

c ... print a line of elements with current element count at the
c ... start of each line
         call utgetprel(ir, var, pvar, var1, type, ni, lc, epl, rl)

c ... prepare a line
         ll = lf + (ir*epl - 1)*rs

c ... make the index range part of the line
         call utmkrn(msg, ifw, lf, ll, re, rs, ir, lp)

c ... make the part of the line with the value
         call prstr(msg, ftyp, 9, id, ni, var)

c ... print the line
         call baswline(unit, msg)

         lc = lc + ir*epl
         lf = lf + ir*epl*rs
      endwhile

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PRT2DV - print a 2d array variable
c        - print a maximum of EPL elements per line
c        - and the entire row count down the page
c        - If the total column count is greater then EPL,
c        - the EPL column by row print block is repeated with the column
c        - count changing until the column count is exhausted
c        -
c        -   UNIT  unit number for write
c        -   PVAR  ptr to two dimensional variable to be written
c        -   ROW   row dimension of variable
c        -   CF    index of first for column dimension
c        -   RF    index of first for row dimension
c        -   CL    index of last effective column value
c        -   RL    index of last effective row value
c        -   CS    column step by which the data was gathered
c        -   RS    row step by which the data was gathered

      subroutine prt2dv(unit, type, pvar,
     &                  cf, cl, cs, row, rf, rl, rs)
      implicit none
      integer unit, type
      Address pvar
      integer cf, cl, cs, row
      integer rf, rl, rs

Use(Controls)

      integer ir, ic, lr, lc, kk, ni, nn, nc
      integer epl, maxlen, ifw, lp
      integer id(20)
      character*Evaluate(MAXSTRING+IRPART) msg
      character ftyp
      double precision var(128), var1(128)

      integer maxcols

      integer utsizeof
      external utsizeof

      msg = " "

c ... GOTCHA: we KNOW we will be printing at most OUTLINE -(" row\col ="+4) 
c ...characters in the column header so we should not use anything largert 
c ...in the call to utprcolinf, otherwise, we get an error, as when we try 
c ...printing a 2-d 256 character array "maxlen" gets too big

      maxcols = OUTLINE - (11 + 4)

      call utfmtdesc(id, ftyp, unit,
     &               cf, cl, cs, rf, rl, rs,
     &               type, pvar)

      maxlen = id(4) 
      epl    = id(5)
      ifw    = id(6)
      lp     = id(7)

c ... now print the two-dimensional array var(row,*) with a
c ... maximum of epl columns per page.  Also print the column
c ... indices across the page and the row indices down the page.

c ... GOTCHA: don't let the column space exceed the space on a single line
      if(maxlen .gt. maxcols) maxlen = maxcols
 
      do lc = 1, cl, epl
         kk = min0(lc+epl-1, cl)
         ni = kk - lc + 1

c ... print current column count across top of output page
         call utprcolinf(unit, cf, cs, lc, kk, maxlen, lp)

c ... loop over rows and print rows of (epl) elements or less
         lr = 1
         while (lr .le. rl)
            nn = (lc - 1)*row + lr

            call utgetprer(ir, var, pvar, var1, type,
     &                     row, ni, nn, lr, lc, rl)

            ic = rf + (lr-1)*rs
            nc = rf + (lr+ir-2)*rs

c ... print a line.
            call utmkrn(msg, ifw, ic, nc, nc, rs, ir, lp)

c ... make the part of the line with the value
            call prstr(msg, ftyp, 9, id, ni, var)

c ... print the line
            call baswline(unit, msg)

            lr = lr + ir
         endwhile
      enddo

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PRTNDV - print a 3D - 7D array variable
c        - run a nested DO LOOP over all possible dimensions, and
c        - call the 2D PRT routines to print the inner 2D plane
c        -
c        -   UNIT     unit number for write
c        -   PVAR     ptr to variable to be printed
c        -   TYPE     type of variable; real, integer, etc.
c        -   JDIM     number of dimensions in variable array
c        -   ITGDIMS  array of integer array dimensions
c        -   ITGORGN  array of integer array origins
c        -   ITGEFF   array of integer array effective dims
c        -   INCRS    increments along dimensions by which data was accessed

      subroutine prtndv(unit,pvar,type,jdim,itgdims,itgorgn,itgeff,incrs)
      integer unit
      Address pvar
      integer type, jdim, itgdims(*), itgorgn(*), itgeff(*), incrs(*)

      integer i, j7, j6, j5, j4, j3, j, jj
      integer row, rorg, rinc, corg, reff, ceff, cinc
      integer bpi, np, ni, incvar
      integer jdx(7), jf(7), jl(7), js(7)
      real col
      Ch(OUTLINE) msg

      integer utsizeof, imkstr
      external utsizeof, imkstr

      bpi = utsizeof(1, type)*LOCS_PER_WORD
      ni  = bpi
      np  = itgdims(1)*itgdims(2)
      do i = 1, jdim
         lf = itgorgn(i)
         ll = itgeff(i)
         ls = incrs(i)
         ni = ni*itgdims(i)

         jf(i) = lf
         jl(i) = lf + (ll - 1)*ls
         js(i) = ls
      enddo

      do i = jdim+1, 7
         jf(i) = 1
         jl(i) = 1
         js(i) = 1
      enddo

c ... now setup to run a nest of DO loops over all possible dimensions
c ... and save current dimension index in array JDX

      incvar = bpi*np
      jj     = 0

      do j7 = jf(7), jl(7), js(7)
         jdx(7) = j7

        do j6 = jf(6), jl(6), js(6)
           jdx(6) = j6

          do j5 = jf(5), jl(5), js(5)
             jdx(5) = j5

            do j4 = jf(4), jl(4), js(4)
               jdx(4) = j4

              do j3 = jf(3), jl(3), js(3)
                 jdx(3) = j3

c ... first construct a character index to label the 2-Dimensional
c ... plane to be printed out by the appropriate PRT subroutine

c ... the label will be of the form: INDEX(row,col,i,j....jdim)
                 call utsprint(1, 0, OUTLINE, msg,
     &                         29, "               INDEX(row, col")
                 do j = 3, jdim
                    call utsprint(0, 0, OUTLINE, msg, 4, ", %d", jdx(j))
                 enddo
                 call utsprint(0, 1, OUTLINE, msg, 1, ")")
                 call baswline(unit, msg)

c ... setup to call appropriate 2D print routine, set row,
c ... column, origin, and effective attributes
                 row  = itgdims(1)
                 rorg = itgorgn(1)
                 reff = itgeff(1)
                 rinc = incrs(1)

                 col  = itgdims(2)
                 corg = itgorgn(2)
                 ceff = itgeff(2)
                 cinc = incrs(2)

                 call prt2dv(unit, type, pvar+jj,
     &                       corg, ceff, cinc, row, rorg, reff, rinc)

                 jj = jj + incvar

                 if (jj .lt. ni) then
                    call baswline(unit,
     &                   "               ------------------------------")
                 endif

              enddo
            enddo
          enddo
        enddo
      enddo
      
      return
      end

c--------------------------------------------------------------------------

c                        HIGH LEVEL PRINT ROUTINES

c--------------------------------------------------------------------------

c     NAME OF ROUTINE...PRTGVAR
c
c     AUTHOR OF ROUTINE...Ron Litterst   10/2/84
c
c     PURPOSE...
c
c        This routine is called to print the values of all variables
c        in the caller's rt data base whose group index is (jgrp).  This
c        is accomplished by looping over all the variables in group (jgrp)
c        and calling prtvar to print the values of each variable.
c
c     GLOSSARY OF IMPORTANT VARIABLES...
c
c        jgrp    = index of group variables to be printed
c        db      = caller's run time data base manager
c        unit    = unit number for output device
c        dgvmap  = index map to group variables
c        jvar    = index of variable to be printed
c
c Altered by ZCM 7/91: encapsulate database references in rtd

      subroutine prtgvar(jgrp, ndb, unit)
      integer ndb,idum,prtvar
      integer edtlstg, jgrp, jvar, i, j1, j2, unit, si(7)

      integer rtgnvar1, rtgtc, tc
      Varname vnam
      external edtlstg, edtlstv, prtvar, rtgnvar1, rtgtc, rtgvname, rtisdead
      logical rtisdead #ZCM 7/91

c ... call EDTLSTG to list group attributes and check function value
c ... to continue printing

      if (edtlstg(jgrp,ndb,unit) .ne. NO) then

c ... print info about each variable by looping over vars in group
c ... before print of each variable print variable short attribute
c ... list for heading

         j1 = rtgnvar1(jgrp, ndb)
         j2 = rtgnvar1(jgrp+1, ndb)-1

         do i = 1,7
            si(i) = 1
         enddo

         do jvar = j1, j2
            call ruthere
            if (rtisdead(jvar, ndb)) next
            call rtgvname(jvar, ndb, vnam)
            tc = rtgtc(jvar, ndb)
            if (tc .ne. FUNCTION_TYPE) then
               idum = prtvar(vnam, jvar, ndb, tc, si, unit)
            endif
         enddo
      endif

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PRTVAR - this routine prints all the values of variable JVAR as
c        - they exist in the caller's rt data base.  This is accomplished
c        - by setting up Pointers to the (db) variable attribute arrays
c        - and scalar
c        -   JVAR    variable index
c        -   DB      caller's run time data base manager
c        -   INCRS   increments used along each dimension to gather the data
c        -   UNIT    unit number for print out
c        -   VARNAM  name of variable

      integer function prtvar(varnam, jvar, ndb, type, incrs, unit)
      character*(*) varnam
      integer jvar, ndb, type, incrs(*), unit

Use(Controls)
Use(Stacks)

      integer chars, rorigin, corigin
      integer coleff, roweff, dimen, row, orgvar, eff, inc, cinc, rinc
      integer itgdims(7), itgeff(7), itgorgn(7), itginc(7)
      Address fwa
      Address pvar
      Address chvar
      character*(OUTLINE) msg
      Ch(COMMENTSIZE) comm
      Ch(TYPESIZE) utcod
      integer  edtlstv
      external edtlstv, rtxcom
      logical rtxcom

c ... extract info from database
      call rtxdb(jvar, ndb, fwa, dimen, itgorgn, itgeff, itgdims, LIMITED)

c ... squeeze dimension of length 1
      nsubs = dimen
      dimen = 0
      do i = 1, nsubs
         if (itgorgn(i) .ne. itgeff(i)) then
            dimen = dimen + 1
            itgorgn(dimen) = itgorgn(i)
            itgeff(dimen)  = itgeff(i)
            itgdims(dimen) = itgdims(i)
            itginc(dimen)  = incrs(i)
         endif
      enddo

c ... set local pointer variable VAR with address of variable to be printed
      pvar = fwa

c ... test if dynamic variable has unset pointer
      if (pvar .eq. NOTSET) then
         call utsprint(1, 1, OUTLINE, msg,
     &                 45, "%.*s is a dynamic variable with unset pointer",
     &                 utgetcl(varnam), varnam)
         call baswline(unit,msg)
         return(ERR)

      endif

c ... print a known type
      if ((type .lt. 0)       .or. (type .eq. INTEGER) .or.
     &    (type .eq. LOGICAL) .or. (type .eq. RANGE) .or.
     &    (type .eq. REAL)    .or. (type .eq. DOUBLE) .or.
     &    (type .eq. COMPLEX) .or. (type .eq. DOUBLE_C) .or.
     &    (type .eq. FWA)     .or. (type .eq. STRUCTURE)) then

         if (~rtxcom(jvar,ndb,1,comm)) comm = " "

c ... test if this is a structure
         if (type .eq. STRUCTURE) then
            call prtstru(varnam, pvar, comm, unit)
            return(OK)
         endif

c ... Next, test if variable to be printed is a scalar, if it is
c ... then call PRT0DV to print one line variable print
         if (dimen .eq. 0) then
            if (varnam .eq. "$") comm = nxp(rsp)
            call prt0dv(unit, type, pvar, varnam, comm)
            return(OK)

         elseif (type .eq. FWA) then
            call remark("Printing out array of address type not implemented.")
            return(ERR)
         endif

c ... first print variable heading by calling EDTLSTV and then
c ... test for 1D or 2D variable print, also check function value
c ... to continue or stop printing

c ...convert from high subscript form to length
         do i = 1, dimen
            itgeff(i) = itgeff(i) - itgorgn(i) + 1
         enddo
         
         if (edtlstv(varnam, jvar, ndb, type, unit) .ne. NO) then

c ... print 1d array
            if (dimen .eq. 1) then
               call prt1dv(unit, type, pvar, itgorgn(1), itgeff(1), itginc(1))

c ... print 2d array
            elseif (dimen .eq. 2) then
               row     = itgdims(1)
               rorigin = itgorgn(1)
               roweff  = itgeff(1)
               rinc    = itginc(1)
               corigin = itgorgn(2)
               coleff  = itgeff(2)
               cinc    = itginc(2)
               call prt2dv(unit, type, pvar,
     &                     corigin, coleff, cinc, row, rorigin, roweff, rinc)
            
c ... print multi-dimensional, 3 thru N, array
            else
               call prtndv(unit, pvar, type, dimen,
     &                     itgdims, itgorgn, itgeff,itginc)
            endif
         endif

         return(OK)

c ... handle unknown types
      else
          call utcodstr(type, utcod)
          call utsprint(1, 1, OUTLINE, msg,
     &                  45, "%.*s cannot be printed since it has type %.*s",
     &                  utgetcl(varname), varname, utgetcl(utcod), utcod)
          call baswline(unit,msg)

          return(ERR)
      endif

      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c LIST - edit list requests

      integer function list(ius, name)

      Use(Hash)

      character*(*) name
      integer  ius, jdef1, lname, ientry, jdef2, ierr
      integer jvar(USERDB), ndb(USERDB), tc(USERDB), pr(USERDB), no
      logical argflg, mcnolkup
      character*(OUTLINE) errmsg

      integer rtfinder, utgetcl,strgfind,
              ihashfind, ihashget, edtlstvl, edtlstfl, edtlstva,
              rtfinda
      external rtfinder, utgetcl, mcnolkup, strgfind,
              ihashfind, ihashget, edtlstvl, edtlstfl, edtlstva,
              rtfinda

      lname = utgetcl(name)
      jdef2 = -1

c ... find name in string table; a nonzero
c ... entry in hash table 0 means it's a macro; the
c ... value there is the index of the macro body.
      if (~mcnolkup()) then
         jdef1  = strgfind(name)
         ientry = ihashfind(mactable, jdef1)
         if (ientry > 0) jdef2 = ihashget(mactable, ientry)
      endif

      if (jdef2 .ge. 0) call edtmcbdy(jdef1,jdef2, argflg, ius)

      ierr = rtfinda(name, jvar, ndb, tc, pr, no, " ")
      if ((jdef2 .lt. 0) .and. ((ierr .eq. ERR) .or. (no .eq. 0))) then
         errmsg = name(1:utgetcl(name)) // " is an unknown name."
         call baserror(errmsg)
         return (OK)
      elseif ((ierr .eq. ERR) .or. (no .eq. 0)) then
         return (OK)
      endif

      if (jdef2 .ge. 0) then
         call baswline (ius,
     &                  "------------------------------------------")
      endif

c ... only one name, so proceed as usual
      if (no .eq. 1) then
         if (tc(1) .eq. GROUP ) then
            call edtlstgc(jvar(1), ndb(1), ius)
         elseif (tc(1) .eq. FUNCTION_TYPE) then
            if (edtlstfl(name, jvar(1), ndb(1), ius) .eq. NO) return(ERR)
         else
            if (edtlstvl(name, jvar(1), ndb(1), tc(1), ius) .eq. NO) return(ERR)
         endif

c ... more than one name, so print out informative list.
      else
         if (edtlstva(name, jvar, ndb, tc, pr, no, ius) .eq. NO) return(ERR)
      endif

      return(OK)
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c LISTLIST - report the things available to the list command

      subroutine listlist(ius)
      integer ius,iunit

      iunit = ius

      if (ius .eq. 100) iunit = STDERR

      call baspline(iunit,"list options")
      call baspline(iunit,"------------")
      call baspline(iunit,"list                    [print the list options]")
      call baspline(iunit,"list par.Attributes")
      call baspline(iunit,"list [pkg.]functions")
      call baspline(iunit,"list Groupname")
      call baspline(iunit,"list [pkg.]groups")
      call baspline(iunit,"list idname")
      call baspline(iunit,"list macros")
      call baspline(iunit,"list packages")
      call baspline(iunit,"list [pkg.]variables")
      call baspline(iunit," ")
      call baspline(iunit,
        "NOTE:Groupname is the name of a group in any package on the")
      call baspline(iunit,
        "     current search stack.")
      call baspline(iunit,
        "NOTE:Groupname can be abbreviated.")
      call baspline(iunit,
        "NOTE:idname is the name of a function, macro, or variable in any")
      call baspline(iunit,
        "     package on the current search stack.")
      call baspline(iunit,
        "NOTE:list groups, functions, and variables list local and user")
      call baspline(iunit,
        "     created groups, functions and variables respectively ")
      call baspline(iunit,
        "     unless pkg. is utilized.")
      call baspline(iunit,
        "NOTE:list pkg.functions lists the built-in and compiled functions")
      call baspline(iunit,
        "     in the database for that package.")

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EDIT - edit print requests

      function edit(ius, name)
      integer edit
      character*(*) name
      integer ius, jvar, ndb, tc, si(7)
      integer rtfinder,prtvar
      external rtfinder,prtvar

      if (rtfinder(name, jvar, ndb, tc, "edit") = ERR) return(ERR)

      do i=1,7
         si(i) = 1
      enddo

      if (tc = GROUP) then
         call prtgvar(jvar, ndb, ius)
      elseif (tc = FUNCTION_TYPE) then
         call edtlstf(name,jvar,ndb,ius)
      else
         return(prtvar(name,jvar, ndb, tc, si, ius))
      endif

      return(OK)
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c GETVALUE - put the value of the real variable, NAME into VALUE

      function getvalue(name, value)

      integer getvalue
      character*(*) name
      integer ndb, jvar, type, ndim, ilow(7), ihi(7), icol(7)
      real value, rvalue
      integer rtfinder
      external rtfinder, rvalue
      Address pvar

c input name has the form pkg.name, .name, or name
      if (rtfinder(name, jvar, ndb, type, "getvalue/setvalue") .eq. ERR)
        return(ERR)

      if (type <> REAL) then
        call remark("getvalue argument not real")
        call remark(name)
        return(ERR)
      endif

      call rtxdb(jvar,ndb,pvar,ndim,ilow,ihi,icol,LIMITED)

      if (pvar = NOTSET) then
         call remark(
         "getvalue: variable is dynamic with unset pointer.")
         return(ERR)
      endif

      value = rvalue (pvar)

      return(OK)
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c SETVALUE - put the value of the real variable, NAME into VALUE

      function setvalue(name, value)
      integer setvalue
      character*(*) name
      integer ndb, jvar, type, ndim, ilow(7), ihi(7),
              icol(7)
      real value
      integer rtfinder
      external rtfinder
      Address pvar

c input name has the form pkg.name, .name, or name

      if ( rtfinder(name, jvar, ndb, type, "setvalue/setvalue") .eq. ERR)
        return(ERR)

      if (type <> REAL) then
        call remark("setvalue argument not real")
        call remark(name)
        return(ERR)
      endif

      call rtxdb(jvar,ndb,pvar,ndim,ilow,ihi,icol,LIMITED)

      if ( pvar = NOTSET) then
         call remark(
         "setvalue: variable is dynamic with unset pointer.")
         return(ERR)
      endif

      call rset (pvar, value)

      return(OK)
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c UTSTRMEM - label the member of a structure

      subroutine utstrmem(unit, itype)
      implicit none
      integer unit, itype

      character*OUTLINE text

      if (itype .lt. 0) then
         call utsprint(1, 1, OUTLINE, text,
     &                 19, " Character array = ")

      elseif (itype .eq. INTEGER) then
         call utsprint(1, 1, OUTLINE, text,
     &                 17, " Integer array = ")

      elseif (itype .eq. REAL) then
         call utsprint(1, 1, OUTLINE, text,
     &                 14, " Real array = ")

      elseif (itype .eq. DOUBLE) then
         call utsprint(1, 1, OUTLINE, text,
     &                 16, " Double array = ")

      elseif (itype  .eq.  COMPLEX) then
         call utsprint(1, 1, OUTLINE, text,
     &                 17, " Complex array = ")

      elseif (itype .eq. DOUBLE_C) then
         call utsprint(1, 1, OUTLINE, text,
     &                 24, " Double complex array = ")

      else
         call utsprint(1, 1, OUTLINE, text,
     &                 34, " Structure member has bad type (%d)", itype)
      endif

      call baswline(unit,text)

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PRTSTRU - print a structure
c         -   STRNAM  name of structure
c         -   ISTR    structure to be printed (note that what is passed is
c         -           the fwa of the structure, not its stack element)
c         -   COMM    a comment
c         -   UNIT    output unit number

        subroutine prtstru(strnam, fwa, comm, unit)
        character*(*) strnam, comm
        character*OUTLINE text
        integer unit, nelt
        Address fwa
        Address pistr
        integer corigin, ceff, row, rorigin, reff, cinc, rinc
        integer idims(7), ieff(7), iorgns(7), incs(7)
        integer jbase, ipiecod, itype, ndim, nchars
        Address pvar

       call utsprint(1, 1, OUTLINE, text, 
     &               11, "%.*s   %.*s",
     &               utgetcl(strnam), strnam, utgetcl(comm), comm)

       pistr = fwa
       nelt  = ivalueo(pistr, 3)

       call utsprint(1, 1, OUTLINE, text, 
     &               27, " Structure has %3d elements", nelt)
       call baswline(unit, text)

c ... begin main loop on number of elements in structure
       for (jbase = 3; i=1 , i<= nelt, jbase=jbase+ivalueo(pistr, jbase+2);i=i+1)

           call baswline(unit, " ")
           call utsprint(1, 1, OUTLINE, text, 14, " Element # %3d", i)
           call baswline(unit, text)

           if (ivalueo(pistr, jbase+1) .eq. STRUCTURE) then
              call utsprint(1, 1, OUTLINE, text, 
     &                      46, "... a structure occupying %d words of storage.", 
     &                      ivalueo (pistr, jbase+2))
              call baswline(unit, text)
              next
           endif

           ipiecod = ivalueo(pistr, jbase+1)
           call strcodtx(ipiecod, text)
           call baswline(unit, text)

           itype = ivalueo(pistr, jbase+3)
           ndim  = ivalueo(pistr, jbase+4)
           if (ndim = 0) then

c ... scalar print
              pvar = fwa + (jbase + 18) * CHAR_PER_WORD

              call utstrmem(unit, itype)
              call prt0dv(unit, itype, pvar, " Scalar", "")

c ... 1-d array print
           elseif (ndim .eq. 1) then
              rinc    = 1
              reff    = ivalueo(pistr, jbase+5)
              rorigin = ivalueo(pistr, jbase+12)
              pvar    = fwa + (jbase + 18)*CHAR_PER_WORD

              call utstrmem(unit, itype)
              call prt1dv(unit, itype, pvar, rorigin, reff, rinc)

c ... 2-d array print
           elseif (ndim .eq. 2) then
              reff    = ivalueo (pistr, jbase+5)
              row     = reff
              rorigin = ivalueo (pistr, jbase+12)
              rinc    = 1
              ceff    = ivalueo (pistr, jbase+6)
              corigin = ivalueo (pistr, jbase+13)
              cinc    = 1
              pvar    = fwa + (jbase + 18) * CHAR_PER_WORD
              
              call utstrmem(unit, itype)
              call prt2dv(unit, itype, pvar,
     &                    corigin, ceff, cinc, row, 
     &                    rorigin, reff, rinc)

c ... multivariate print
           elseif (ndim >= 2) then
              do j = 1, 7
                 idims(j)  = ivalueo (pistr, jbase+4+j)
                 ieff(j)   = ivalueo (pistr, jbase+4+j)
                 iorgns(j) = ivalueo (pistr, jbase+11+j)
                 incs(j)   = 1
              enddo
              
              pvar = fwa + (jbase + 18) * CHAR_PER_WORD
              
              call utstrmem(unit, itype)
              call prtndv(unit, pvar, itype, jdim, idims, iorgns, ieff, incs)
           endif
       endfor

       return
       end
      
c--------------------------------------------------------------------------
c--------------------------------------------------------------------------
      
      subroutine decimal
      Use(Controls)
      formdesc = '   '
      return
      end
      
c--------------------------------------------------------------------------
c--------------------------------------------------------------------------
      
      subroutine oct
      Use(Controls)
      formdesc = 'oct'
      return
      end
      
c--------------------------------------------------------------------------
c--------------------------------------------------------------------------
      
      subroutine hex
      Use(Controls)
      formdesc = 'hex'
      return
      end
      
c--------------------------------------------------------------------------
c--------------------------------------------------------------------------
      
c PARDOLI - perform list command for the top runtime stack element ss(., rsp)

      integer function pardoli(si)

      Use(Stacks)

      Ch(128) name
      character*PNSIZE pkg
      integer n1, npac, npkg
      integer list, utgetcl, parname, getius
      external list, utgetcl, parname, getius

      n1 = parname(ss(1, rsp), name)

c ... list packages
      if ((name .eq. "packages") .or. (name .eq. "PACKAGES")) then
         call glblpack

c ... list macros
      elseif ((name .eq. "macros") .or. (name .eq. "MACROS")) then
         call edtmcnam(getius())

      else
         npac = ss(N, rsp)

c ... list groups
         if ((name .eq. "groups") .or. (name .eq. "GROUPS")) then
            call edtvname(getius(), npac, NO)

c ... list functions
         elseif ((name .eq. "functions") .or. (name .eq. "FUNCTIONS")) then
            call pardoluf(npac)

c ... list variables
         elseif ((name .eq. "variables") .or. (name .eq. "VARIABLES")) then
            call edtvname(getius(), npac, YES)

         else
            call parkill(ss(1, rsp))
            if (npac .eq. 0) then
               return(list(getius(), name(1:n1)))

            else
               call glbpknam(npac, pkg)
               npkg = utgetcl(pkg)
               return(list(getius(), pkg(1:npkg)//"."//name(1:n1)))
            endif
         endif
      endif

c ... cleanup
      call parkill(ss(1, rsp))

      return(OK)
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PARDOLUF - list the package's functions

        subroutine pardoluf(ndb)
        integer nv, rtgnvar, rtgtc
        Varname vname
        external rtgnvar, rtgtc, rtgvname, rtisdead
        integer ndb
        logical nofuncs, rtisdead
        character*(OUTLINE) msg

        nofuncs = TRUE
        if (ndb .eq. 0) then

c ... user database only
           nv = rtgnvar(USERDB)
           do i = 1, nv
              if (rtgtc(i, USERDB) .eq. FUNCTION_TYPE) then
                 nofuncs = FALSE
                 call rtgvname (i, USERDB, vname)
                 if (~rtisdead(i, USERDB)) then
                    call edtlstf(vname, i, USERDB, iooutus())
                 endif
              endif
           enddo

           if (nofuncs) then
              call baswline(iooutus(), 
     &                      "   No user functions have been defined.")
           endif
        else
           nv = rtgnvar(ndb)
           do i = 1, nv
              if (rtgtc(i, ndb) .eq. FUNCTION_TYPE) then
                 nofuncs = FALSE
                 call rtgvname(i, ndb, vname)
                 if (~rtisdead(i, ndb)) then
                    call edtlstf(vname, i, ndb, iooutus())
                 endif
              endif
           enddo

           if (nofuncs) then
              call utsprint(1, 1, OUTLINE, msg,
     &                      45, "   No functions for db #%d have been defined.",
     &                      ndb)
              call baswline(iooutus(), msg)
           endif
        endif

        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c ... NOTE: this function does NOT appear to be used anywhere

      subroutine sav1dvdc (unit, var, eff)

Use(Controls)

ifelse(HAS_DOUBLE_COMPLEX, YES, [

      double complex var(*)
      integer eff, kk, lc, leng, maxepl, unit
      character ifc*2, jfc*2, kfc*1, wline*64, msg*(OUTLINE)
      character*1 s(2)
      integer imkstr
      external imkstr

      leng = imkstr(fuzz, ifc)
      leng = imkstr(fuzz+8, jfc)
      maxepl = min0(2, 78/(2*fuzz+19))

      do lc = 1, eff, maxepl
         wline="(e  .  e3, a1, e  .  e3, 'i' (', 'e  .  e3, a1, e  .  e3, 'i'), ', ')"
         wline(3:4) = jfc
         wline(6:7) = ifc
         wline(15:16) = jfc
         wline(18:19) = ifc
         wline(32:33) = jfc
         wline(35:36) = ifc
         wline(44:45) = jfc
         wline(47:48) = ifc

         kk   = min0(lc+maxepl-1, eff)
         leng = imkstr(kk-lc, kfc)
         wline(26:26) = kfc
         if (kk .eq. eff) wline(58:58) = ']'

         s(lc) = '+'
         if (dimag(var(lc)) < 0.) s(mod(lc-1, maxepl)+1) = '-'
         if (dimag(var(lc)) .eq. 0.) wline(14:21) = "f2.0   "

         s(lc+1) = '+'
         if (dimag(var(lc+1)) < 0.) s(mod(lc+1-1, maxepl)+1) = '-'
         if (dimag(var(lc+1)) .eq. 0.) wline(43:50) = "f2.0    "
         if (wline(26:26) .eq. '0')wline(25:64)="]')"

         write(msg, wline) \
         (dble(var(i)), s(mod(i-1, maxepl)+1), abs(dble(dimag(var(i)))) , i=lc, kk)

         call baswline(unit, msg)
      enddo

])

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

