# Tue Oct 15 10:26:22 PDT 1991
# Original author: Lee Busby
# This file implements a shell around the ranf family of random
# number routines and is designed to run on the Cray.  It is
# compatible with the set of routines of the same name in ltsslib/ranf.c.
# Routines getmult and setmult added by Fred Fritsch 8/21/97.
# Added "external ranget" to getranf. (FNF 6/06/98)

# Reset the RNG, using ranset
      subroutine seedranf(s)
      integer s
      call ranset(s)
ifdef([RAN_DEBUG],[
      write(*,'(a,z18)') "SEEDRANF set ",s
])
      end

# Set the RNG state, using semantics identical to rnfmix, e.g.:
#   s < 0  ==>  Use the default seed value.
#   s = 0  ==>  Set a "random" value for the seed from the system clock.
#   s > 0  ==>  Set seed directly (all 48 bits).
# Return the new state in the integer array "st".
      subroutine mixranf(s,st)
      integer s,st(2),newseed,rnfmix
      newseed = rnfmix(s)
      st(1) = mod(newseed,2**32)
      st(2) = (newseed - st(1))/2**32
ifdef([RAN_DEBUG],[
      write(*,'(a,2z18)') "MIXRANF returns ",st(2),st(1)
])
      end

# Get a 48 bit integer from ranget.  Store the low 32 bits into
# st(1) and the high 16 bits in st(2).
      subroutine getranf(st)
      integer st(2)
      integer seed,ranget
      external ranget
      seed = ranget(0)
      st(1) = mod(seed,2**32)
      st(2) = (seed - st(1))/2**32
ifdef([RAN_DEBUG],[
      write(*,'(a,2z18)') "GETRANF returns ",st(2),st(1)
])
      end

# Pack the two parts of the 48 bit seed back into one
# (64 bit) word, and then call ranset.
      subroutine setranf(st)
      integer st(2)
      integer seed
      seed = st(2)*2**32 + st(1)
ifdef([RAN_DEBUG],[
      write(*,'(a,i24,a,z18,a)')
          "SETRANF setting ",seed," = ",seed,"(hex)"
])
      call ranset(seed)
      end

      subroutine getmult(mt)
# Get a 48 bit multiplier from rnmuset.  Store the low 32 bits into
# mt(1) and the high 16 bits in mt(2).
      integer mt(2)
      integer mult,rnmuset,zero
#      Caution: rnmuset changes the value of its argument.
      zero = 0
      mult = rnmuset(zero)
      mt(1) = mod(mult,2**32)
      mt(2) = (mult - mt(1))/2**32
ifdef([RAN_DEBUG],[
      write(*,'(a,2z18)') "GETMULT returns ",mt(2),mt(1)
])
      end

      subroutine setmult(mt)
# Pack the two parts of the 48 bit multiplier back into one
# (64 bit) word, and then call rnmuset.
      integer mt(2)
      integer defmult,mult,rnmuset
      data defmult /Z'2875A2E7B175'/

      mult = mt(2)*2**32 + mt(1)
ifdef([RAN_DEBUG],[
      write(*,'(a,i24,a,z18,a)')
          "SETMULT input = ",mult," = ",mult,"(hex)"
])
      if (mult.ne.0) then
#        If input multiplier is nonzero, make sure it's odd.
         if (mod(mult,2).eq.0) mult=mult+1
      else
C        Reset to default multiplier.
         mult = defmult
      endif

ifdef([RAN_DEBUG],[
      write(*,'(a,i24,a,z18,a)')
          "SETMULT setting ",mult," = ",mult,"(hex)"
])
      if (rnmuset(mult).ne.0) then
         call baserror("SETMULT failed; multiplier unchanged.")
      endif

      end

# Note that the following function need not be provided here, because
# it is part of Mathlib on the Cray.
#     real function ranf()
