
c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c EXECUSER - A compiled function has called a user
c          - function. This routine saves the current
c          -  environment on the intcode stack, generates
c          -  code for the function call, executes it,
c          -  and then restores the environment.
c       MAINTENANCE NOTES : execuser does not call the parser recursively and
c           hence does not need to save the entire parser environment as
c           parsestr does. execuser creates and then executes a function
c           call in the intcode block. It must thus preserve more of the
c           execution-time environment than parstr does.

      subroutine execuser(f)
      Ch(*) f
      integer utgetcl, funchk, mcname, rtgloc, utsizeof, strgfind, idum
      integer rtgrpe, ivalueo
      external utgetcl, funchk, mcname, rtgloc, utsizeof, strgfind
      external rtgrpe, ivalueo
      Varname uname

Use(Actions)
Use(Functions)
Use(Compiled_functions)
Use(Stacks)

      Address curenv(ENVSIZE)

# Turn on symbolic
      symbolic=.true.

# 0. Is there a function available to call ???
      uname=f
      if (funchk(uname)==ERR) then
         call kaboom(0)
      endif

# 1. Is there enough code space?
      call codechk()

# 1a.Save the environment of the current function--its name and type and
#    the arguments' names and types
      curenv(1) = cfcnargc
      do i = 1, cfcnargc
         curenv(2 + i) = cfcnarg(i)
      enddo
      curenv(3 + cfcnargc) = cfcnres
      curenv(4 + cfcnargc) = fargnum

# 2. Create a dummy local database for execuser and also nest it
      idum = rtgrpe(LOCALDB,"Execuser")
      call parnest(EXECUSER)
      call fstksav(uname)

# 3. Execute the call and hence the function
      call lrexec()

# 4. Restore the environment to what it was
      call fstkrst()

# 5. Unnest this execuser call
      call decrnest()

# 6.Restore the rest of the environment
      cfcnargc = curenv(1)
      do i = 1, cfcnargc
         cfcnarg(i) = curenv(2 + i)
      enddo

      cfcnres = curenv(3 + cfcnargc)
      fargnum = curenv(4 + cfcnargc)

# The following gets rid of a null stack element left behind when
# execuser was called from FORTRAN. It is restured in execuser_,
# which is the path by which interpreted code gets here.
      rsp = rsp - 1 

      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PARSTR

      subroutine parstr(str, user)
#      character*(*) str
      Address str

# -- for backward compatibility
      call parsestr(str)
      
      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PARSESTR - This routine may be called either from Basis or from
c          - compiled code, with a string argument. It saves the environment
c          - and recursively calls the parser, which parses the string. In most
c          - cases(i. e., if the string is a complete statement and not a function),
c          - it will execute the resulting intermediate code. If the statement is a
c          - function, its code and data space will be generated dynamically and
c          - the code and data moved there.
c          - Parstr is intended for parsing and/or executing a single string of
c          - no more than 500 characters in length.
c          - Rewritten to call parsestr_ with an Address and length.

        subroutine parsestr(str)

        Ch(*) str
        Address strloc
        external strloc

        call parsestr_(strloc(str), len(str))

        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

c PARSESTR_ - To clean up recursive calls to the parser. Before
c           - these changes, it was not possible to call parstr
c           - from compiled functions, because the environment
c           - was saved too early. Now the environment is not
c           - saved at all unless the parser is recursively called
c           - and it is saved at as low a level as possible.
c           - All environment saving is done right here in
c           - parstr, which is what makes it possible to be
c           - called from anywhere, e. g., a builtin function or
c           - the graphics.

      subroutine parsestr_(str, nstr)

Use(Actions)
Use(Controls)
Use(Functions)
Use(Stacks)
Use(Lrhcoms)

      Address str
      integer nstr

# For local stackframe storing of the environment:
      Address curenv(ENVSIZE)
      integer i

# Save current parser info here
      integer parptr, sltok, sntok, strptr, lxptr
      character * 500 sttok
      integer nstate, parserr

      external parsetup, lrpars, lrerr, parestor, utgetcl, mcendt,
      gstkptr, sstkptr
      integer parsetup, lrpars, lrerr, utgetcl, mcendt, gstkptr
      integer recur
      logical retflg, xflag

# Turn on symbolic
      symbolic=.true.
 10   continue
      if (partrace) then
         call baserror("PARSESTR: parsing")
         call strgprt(str, nstr)
      endif

# Save info about this function call here
      curenv(1) = cfcnargc
      do i = 1, cfcnargc
         curenv(2 + i) = cfcnarg(i)
      enddo
      curenv(3 + cfcnargc) = cfcnres
      curenv(4 + cfcnargc) = fargnum
      semerr = parsetup()
      
# put away parser info
      xflag = execflag
      parptr = gstkptr()
      sltok = ltoken
      sntok = ntoken
      sttok = ttoken
      strptr = 1
      call lxgptr( lxptr )
      ttoken = "END"
      ntoken = mcendt()
      ltoken = 3
      call lrpush(MLRINT, ntoken)

# save some info for lptrace
      call slptrace(str, nstr)
      if (semerr==OK) then
         recur = 1
         strptr = 1
         do
            semerr  = lrpars(recur,str,nstr,strptr,lxptr,nstate,retflg)
            parserr = semerr        ## local var so remembered recursively
            if (semerr == ERR) call kaboom(0)
            if (retflg) call lrexec()   ## could recursiv. call parsestr
            recur = 2
         until(parserr == DONE)  ## test parserr so work correct recursively
         semerr = OK
      else
         semerr = lrerr(10, str, nstr, strptr, recur)
      endif
      
# bring back parser info
      execflag = xflag
      call sstkptr(parptr)

      ltoken = sltok
      ntoken = sntok
      ttoken = sttok

#(ZCM 12/14/95) Upon return from debugger, if ttoken is a
# blank, then Basis manages to build a token that starts
# with a blank, which is illegal. This fixes the problem.
      if (ttoken(1:ltoken)==" ") ltoken = 0

      call lxsptr(lxptr)
      call parestor()

      cfcnargc = curenv(1)
      do i = 1, cfcnargc
         cfcnarg(i) = curenv(2 + i)
      enddo

      cfcnres = curenv(3 + cfcnargc)
      fargnum = curenv(4 + cfcnargc)
      
      return
      end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

        subroutine parselng_(str, l1str, lstr)
        Address str
        integer l1str, lstr

        call parsestr_(str, lstr)

        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

        subroutine parselng(sstr, lstr)
        character *(*) sstr
        integer lstr
        Address strloc
        external strloc

        call parsestr_(strloc(sstr), lstr)

        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

        subroutine parse(str, nstr)
        Ch(*) str(nstr)
        integer nstr
        Address strloc
        external strloc

        call parse_(strloc(str), len(str(1)), nstr)

        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

        subroutine parse_(str,lstr,nstr)
        Address str
	integer lstr, nstr
# parse an array of character strings, treating array boundaries as newlines
        Address Point(s)
	integer sx(SS_WIDTH)
        Address paraddr
	integer utgetcl
	external utgetcl

	m = (lstr+1)*(nstr)
	sx(TC) = -1
	sx(N) = 1
	sx(N1) = m
	sx(M1) = 1
	sx(I1) = 1
        sx(NAML) = 1 # since in literal table(ZCM 4/10/95)
	call parget(sx)
	Point(s) = paraddr(sx)
	do i=1, nstr
	   call smovatoa(Point(s),(i-1)*(lstr+1),str,(i-1)*lstr,lstr)
           call smovftoa(Point(s),i*(lstr+1)-1,";",0,1)
	enddo

        call parselng_(Point(s),1,m)
	call parrel(sx)

	return
	end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

        subroutine parsecnt(str,nstr)
# routine parsecnt is almost identical to parse, except that the ";" is not
# inserted between messages.

	integer nstr
        Address strloc
        external strloc
 	character*(*) str(nstr)

        call parselng_(strloc(str(1)), len(str(1)), len(str(1))*nstr)

        return
        end

c--------------------------------------------------------------------------
c--------------------------------------------------------------------------

