/*
 * RT.H - header for runtime directory
 *
 */

#ifndef HDR_RT
#define HDR_RT

#ifdef PACT
# define PCK_COMPLEX
# include <pml.h>
#else
# include <sys/types.h>
# include <sys/stat.h>
# include <float.h>
# include <limits.h>
# include <math.h>
# include <stdio.h>
# include <errno.h>
# include <stdlib.h>
# include <unistd.h>
# include <string.h>
# include <ctype.h>
# include <setjmp.h>
# include <signal.h>
# if defined(SGI) || defined(IRIX64)
#  include <sys/signal.h>
# endif
# ifdef AXP
#  include <alpha/machlimits.h>
# endif
# include <time.h>
#endif

#include <assert.h>
#include <sys/utsname.h>
#include <sys/param.h>
#include <sys/times.h>
#include <pwd.h>

#undef MAXPATH
#include "par_.h"
#include "Basis.h"
#include "func.h"
#include "strg.h"
#include "multi.h"

/*--------------------------------------------------------------------------*/

/*                                   MACROS                                 */

/*--------------------------------------------------------------------------*/

#define YES     1
#define DEFAULT -999999
#define NStrg   2999

#ifndef WORD_BIT
#define WORD_BIT (8*sizeof(int))
#endif

#ifdef LINUX
# undef LONG_BIT
# define LONG_BIT sysconf(_SC_LONG_BIT)
# undef WORD_BIT
# define WORD_BIT sysconf(_SC_WORD_BIT)
#endif

#define ssl(i, j)    ss[SSWIDTH*((j) - 1) + (i) - 1]

#undef NOTSET
#if defined (SGI)
# define NOTSET ((Address) 0)
#else
# define NOTSET ((Address) -8192)
#endif

#ifndef abs
# define abs(x) ((x) > 0? (x) : - (x))
#endif

#undef max
#define max(n, m) ((n) > (m) ? (n) : (m))

#undef min
#define min(x, y) ((x) > (y) ? (y) : (x))

#define INIT_BLANK_STRING(_s, _n)                                           \
  memset(_s, ' ', _n);                                                      \
  (_s)[(_n) - 1] = '\0'

/*--------------------------------------------------------------------------*/

/*                          VARIABLE DECLARATIONS                           */

/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/

/*                          FUNCTION DECLARATIONS                           */

/*--------------------------------------------------------------------------*/

/* ACCESS.C declarations */

extern void
 Cobjcpy(Integer *dst, Integer *src);


/* ATTR.C declarations */

extern logical
 Rtdattr(Integer jvar, Integer ndb, Integer nattr),
 Rtattrin(char *attr);

extern Integer
 getreadonly(void);

extern void
 Rtvattre(Integer jvar, Integer ndb, char *attr),
 Rtsattr(Integer *m, Integer n),
 Rtinscod(Integer codewrd),
 Rtdelcod(void);


/* CMAC.C declarations */

extern Integer
 Rtmacat(Integer k);


/* CEXEC.C declarations */

extern void
 Lryact(integer *pc);


/* DBAUX.C declarations */

extern Integer
 Rtfinder(char *name, Integer *jvar, Integer *ndb,
	  Integer *type, char *complain),
 Rtinname(Integer ndb, Integer tc, Integer *svari),
 Rtlookg(char *group, Integer *ndb),
 Rtnofrgn(char *service, Integer jvar, Integer ndb),
 Utgetcl(char *name);

extern void
 Rtismac(char *name, char *newname, Integer *macindx),
 Rtzapall(void);


/* DBG.C declarations */

extern void
 Dbgtrace(Integer unit, Integer pc, Integer rsp1, Integer rsp2,
	  Integer rlinkp1, Integer rlinkp2),
 Errprt(void),
 Trace(char *name),
 Untrace(char *name),
 Lrfdotr(void),
 Lrftrmk(char *name, int flag, int *value);


/* FUNC.C declarations */

extern Integer
 Funparin(Integer nameindx, Integer type);

extern void
 Rtufcn(Address *fwa1, Integer *i2, Integer *i3, Integer *i4,
	Integer *i5, Address *fwa6, Integer *i8, Address *fwa9,
	logical hastext);


/* ERRREC.C declarations */

extern void
 Kaboom(Integer idum),
 Ruthere(void);


/* IHASH.C declarations */

extern Integer
 IhashFind(Integer jtable, Integer k),
 IhashPut(Integer jtable, Integer k, Integer v),
 IhashGet(Integer jtable, Integer jentry);

extern void
 FreeNodes(void);


/* IOTAB.C declarations */

extern void
 Osdeliotab(void);


/* LIT.C declarations */

extern Integer
 Parlen(Integer *s),
 Partempc(Address fwa),
 Find(Integer prelim, Integer number);

extern void
 Parkill(Integer *s),
 Parget(Integer *s),
 Parrel(Integer *s),
 Parnams(Integer *si, char *parnam, Integer *nc);

extern Address
 Paraddr(Integer *s);


/* MACH.C declarations */

extern Integer
 I1mach(Integer *i);


/* MEM.C declarations */

extern void
 Osfreen(Address *ptr, char *name),
 Osallotn(Address *ptr, Integer n, char *name),
 Oschangn(Address *ptr, Integer newlen, Integer oldlen, char *name),
 Osmemchk(char *who_called);


/* PARMOVE.C declartions */

extern Integer
 Utbytesin(Integer tc);


/* RDB.C declartions */

extern logical
 Rtisdyn(Integer jvar, Integer ndb),
 Rtischam(Integer jvar, Integer ndb),
 Rtisdead(Integer jvar, Integer ndb);

extern Integer
 Rtlookup(char *var, Integer *ndb, Integer *tc, Integer intable),
 Rtfcneu(logical *hastext),
 Rtgnvar(Integer ndb),
 Rtgnvari(Integer ndb),
 Rtgndim(Integer jvar, Integer ndb),
 Rtgvattr(Integer jvar, Integer ndb),
 Rtggattr(Integer jgrp, Integer ndb),
 Rtsetdl(Integer jvar, Integer ndb, Integer irec),
 Rtvare(Integer ndb, char *vname, Address addr, Integer dynflg,
	char *type, char *dimsto, char *units),
 Glbpknum(char *pn),
 Glbpknux(char *pn),
 Rtgetverb(void);

extern void
 Rtcomeg(Integer jvar, Integer ndb, char *comment),
 Rtcomev(Integer jvar, Integer ndb, char *comment),
 Rtcrack(Integer key, Integer *jvar, Integer *ndb),
 Rtforget(Integer jvar, Integer ndb),
 Rtgvname(Integer jvar, Integer ndb, char *name),
 Rtlooka(char *var, Integer *jvar, Integer *ndb, Integer *tc,
	 Integer *pri, Integer *no),
 Rtsgattr(Integer ndb, Integer ngrp, Integer iattr),
 Rtslimst(Integer jvar, Integer ndb, Integer ival),
 Rtsnastr(Integer jvar, Integer ndb, Integer ival),
 Rtsvattr(Integer jvar, Integer ndb, Integer iattr),
 Rtxdb(Integer jvar, Integer ndb, Address *fwa, Integer *ndim,
       Integer *ilow, Integer *ihi, Integer *icol, Integer access),
 Rtfin(Integer ies),
 Rtclean(void),
 Rtkill(void),
 Rtsetxit(void),
 Glblpackn(Integer ndb, Integer flag),
 Glblpack(void);


/* SPRINT.C declartions */

extern int
 Isnetnormord(void);


/* STRG.C declartions */

extern unsigned int
 StrgHash(char *s);

extern Integer
 StrEQ(char *string1, integer offset1, integer length1,
       char *string2, integer offset2, integer length2),
 StrgFind(char *s);

extern void
 StrgGet(integer no, char *string),
 StrgDone(void);


#endif
