/*
 * sprint.c - provide a FORTRAN callable sprintf
 *
 */

#include <score.h>
#include "par_.h"
#include "Basis.h"
extern void Kaboom(Integer idum);

extern int utstrlen(char *s, int nx);

#define STRSLOP 5
/* A user may want to print a MAXLINE long string.  Basis will format
 * it with two leading spaces, surround it with double quotes, and we
 * may want to tack on a trailing NULL here.  Hence STRSLOP.
 */

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* ISNETNORMORD - return TRUE iff this machine has network normal byte
 *              - ordering
 *              - in PDB terms TRUE -> NORMAL and FALSE -> REVERSE
 */

int Isnetnormord(void)
{
    int nno;
    union ucsil {
        unsigned char c[8];
        int i;
    } bo;

    bo.i = 1;
    nno = (bo.c[0] != 1);

    return (nno);
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* BSPRINT - worker for formatting numerical types into strings */

static void bsprint(int availmem, char *dst, char *fmt, va_list SC_VA_VAR)
{
    int c, nc;
    char local[STRSLOP+MAXLINE], s[STRSLOP+MAXLINE], sbuf[STRSLOP+MAXLINE];
    char *le, *lb, *pt;
    float *fv;
    double *dv;
    FIXNUM *lv;
    F77_string sv;

    sbuf[0] = '\0';

    while (TRUE) {
        for (pt = local; (((c = *fmt++) != '%') && (c != '\0')); pt++) {
            if (c == '\\') {
                switch (c = *fmt++) {
                case 't':
                    *pt = '\t';
                    break;
                case 'r':
                    *pt = '\r';
                    break;
                case 'n':
                    *pt = '\n';
                    break;
                default:
                    *pt = c;
                    break;
                }
            } else {
                *pt = c;
            }
        }
        *pt = '\0';
        strcat(sbuf, local);

        if (c == '\0')
            break;

/* copy from the % to the type specifier to get the format descriptor for
 * this item
 */
        le = strpbrk(fmt, "sdouxXfeEgGcD%");
        local[0] = '%';
        for (lb = &local[1]; le != fmt; *lb++ = *fmt++) ;
        fmt++;
        *lb++ = *le;
        *lb = '\0';

/* jump on the type spec to pull the correct arg type off the stack */
        switch (*le) {
        case 's':
            if (strchr(local, '*')) {
                lv = SC_VA_ARG(FIXNUM *);
                nc = *lv;
                sv = SC_VA_ARG(F77_string);
                SC_FORTRAN_STR_C(s, sv, nc);
                (void) utstrlen(s, nc);
                sprintf(sbuf + strlen(sbuf), local, nc, s);
            } else {
                nc = 128;
                sv = SC_VA_ARG(F77_string);
                SC_FORTRAN_STR_C(s, sv, nc);
                nc = utstrlen(s, nc);
                sprintf(sbuf + strlen(sbuf), local, s);
            }

            break;

        case 'c':
            sv = SC_VA_ARG(F77_string);
            SC_FORTRAN_STR_C(s, sv, 1);
            strncat(sbuf, s, 1);
            break;

        case 'i':
        case 'X':
        case 'x':
        case 'o':
        case 'd':
        case 'u':
            lv = SC_VA_ARG(FIXNUM *);
            sprintf(sbuf + strlen(sbuf), local, (long) *lv);
            break;

        case 'f':
        case 'e':
        case 'E':
        case 'g':
        case 'G':
            if (strchr(local, '*')) {   /* Assume "%*.*". */
                FIXNUM *lv1 = SC_VA_ARG(FIXNUM *);
                FIXNUM *lv2 = SC_VA_ARG(FIXNUM *);
                int nc1 = *lv1, nc2 = *lv2;

                fv = SC_VA_ARG(float *);
                sprintf(sbuf + strlen(sbuf), local, nc1, nc2,
                        (double) *fv);
            } else {
                fv = SC_VA_ARG(float *);
                sprintf(sbuf + strlen(sbuf), local, (double) *fv);
            }
            break;

        case 'D':
            if (strchr(local, '*')) {   /* Assume "%*.*". */
                FIXNUM *lv1 = SC_VA_ARG(FIXNUM *);
                FIXNUM *lv2 = SC_VA_ARG(FIXNUM *);
                int nc1 = *lv1, nc2 = *lv2;
                dv = SC_VA_ARG(double *);
                lb--;
                *lb = 'E';
                sprintf(sbuf + strlen(sbuf), local, nc1, nc2, *dv);
            } else {
                dv = SC_VA_ARG(double *);
                lb--;
                *lb = 'E';
                sprintf(sbuf + strlen(sbuf), local, *dv);
            }

/* fix errant SGI E format for doubles - substitute D for E */
            lb = sbuf + strlen(sbuf);
            while (--lb != sbuf) {
                if (*lb == 'E') {
                    *lb = 'D';
                    break;
                }
            }

/* make D-00 be D+00 */
            lb = sbuf + strlen(sbuf);
            while (--lb != sbuf) {
                if (*lb == '-') {
                    *lb = '+';
                    break;
                } else if (*lb != '0') {
                    break;
                }
            }

            break;

        case '%':
            strcat(sbuf, "%");
            break;
        }
    }
    if (strlen(sbuf) > availmem) {
        char msg[STRSLOP+MAXLINE];

        sprintf(msg,
                "Error in bsprint: string length (%d) exceeds buffer (%d)\n",
                (int) strlen(sbuf), availmem);
        BasError(msg, strlen(msg));
        sprintf(msg, "Portion of string was: %s\n", sbuf);
        BasError(msg, strlen(msg));
        Kaboom(0);
    }
    strcat(dst, sbuf);

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/

/* UTSPRINT - interpret C style format specifcation and get values
 *          - from arg list to fulfill them
 *          - return a string with the result
 */

void F77_ID(utsprint_, utsprint, UTSPRINT)
    (FIXNUM *pi, FIXNUM *pf, FIXNUM *pnd, F77_string d,
     FIXNUM *pnf, F77_string f, ...)
{
    int nd, nf, init, fin, availmem;
    char forms[STRSLOP+MAXLINE];
    char *fm, *dst, *fmt, *pd;

    fm = SC_F77_C_STRING(f);
    dst = SC_F77_C_STRING(d);

    nd = *pnd;
    nf = *pnf;
    init = *pi;
    fin = *pf;

    fmt = forms;
    strncpy(fmt, fm, nf);
    fmt[nf] = '\0';

    if (init) {
        memset(dst, ' ', nd);
        dst[0] = '\0';
        pd = dst;
    } else {
        pd = dst + strlen(dst);
    }

    availmem = dst - pd + nd - 1;

    SC_VA_START(f);

    bsprint(availmem, pd, fmt, SC_VA_VAR);

    SC_VA_END;

    if (fin)
        dst[strlen(dst)] = ' ';

    return;
}

/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
