#!/bin/env python
#
# Mac replacement
# Process a Variable Description File (VDF) into several output files.
#
#
# TODO:
#   1) add a type for each object (variable, function, builtin, command)
#   2) print from a dictionary to a single template
#   3) rename typename? (reorder)
# done  4) integerate Typenode into Ranknode
#   5) test 'language' at package level
#   6) implement ignore_language
#   7) implement 'SYSTEM'
#   8) decide how to parse 'character*(8)'
#      right now it get parsed as character * expr
#      should it be:              character * ( expr )
#
# F90 note:
#  'local' attribute in a group implies an include file, not a module
#
'''
Implement the user interface to the Variable Description File (VDF)
reading routines. It then creates output based on the VDF file and
command line options.
'''

_debug = 1

import sys

try:
    import configured
except ImportError:
    print "vast: configured.py not found"
    sys.exit()

########################################
#
#	Runtime options
#
class MACoptions:
    def __init__(self, optlist=[]):
        import arglist
        
        # long options
        self.cpu       = configured.HostCpu

        # Internal options
        self.fp        = None

        # short, long, has_value, default_value, val_cb, description
        args = arglist.Arguments(
            arglist.Option('b', None, 0, 0, None,
                           'Turn on debuggin output to STDERR'),
            arglist.Option('c', None, 0, 0, None,
                           'Generate a Crontigner C++ interface'),
            arglist.Option('d', None, 1, None, str,
                           'Name of documention file'),
            arglist.Option('e', None, 0, 0, None,
                           'Echo the input to STDERR'),
            arglist.Option('h', None, 0, 0, None,
                           'Ignore "hidden" attribute'),
            arglist.Option('i', None, 0, 0, None,
                           'Ignore "language" directive'),
            arglist.Option('l', None, 0, 0, None,
                           'Set the output line length to 80'),
            arglist.Option('m', None, 1, None, str,
                           'Name of generated mppl macro file'),
            arglist.Option('n', None, 0, 0, None,
                           'Flag for NVDFs'),
            arglist.Option('p', None, 0, 0, None,
                           'Print the variable names to STDOUT'),
            arglist.Option('s', None, 0, 0, None,
                           'Turn off report'),
            arglist.Option('x', None, 0, 0, None,
                           'Do not enter comments into database'),
            arglist.Option('w', None, 1, None, str,
                           'Name of handler file'),
            arglist.Option('y', None, 1, None, str,
                           'Name of runtime package connections file'),
            arglist.Option('r4', None, 0, 0, None,
                           'Use 32-bit reals'),
            arglist.Option('r8', None, 0, 0, None,
                           'Use 64-bit reals (default)'),
            arglist.Option('std', None, 0, 0, None,
                           'Do not use cray pointers'),
            arglist.Option('blockdata', None, 0, 0, None,
                           'Create block data'),
            arglist.Option('noblockdata', None, 0, 0, None,
                           'Do not create block data'),
            arglist.Option('wantc', None, 0, 0, None,
                           'Create C interface stuff'),
            arglist.Option('xmitid', None, 0, 0, None,
                           '  '),
            arglist.Option('H', 'help', 0, 0, None,
                           'Help message'),
            arglist.Option('f90', None, 0, 0, None,
                           'Junk'),

#            arglist.Option('incdir', None, 1, '.', str,
            arglist.Option('incdir', None, 1, None, str,
                           'Directory to put created f90 include files'),
            arglist.Option('ptrprefix', None, 1, 'p', str,
                           'Prefix for cray pointers'),
            )
                           
        args.process(sys.argv[1:])

        for opt in args.badopts:
            sys_opt = opt[1:]
            if sys_opt in ("SUN4","HP700","YMP","C90","J90","RS6000","CRAY2",
                           "IRIX64","GENERIC","LINUX", "SGI","VP2000","SOL",
                           "AXP","CS2","XMP","ULTRIX","VAX","MAC","MIPS"):
                self.cpu = sys_opt
            else:
                sys.stdout.write("Illegal option: %s\n" % opt)
                sys.exit(1)

        self.debug = args.b
        self.crotinger = args.c
        self.docfilename = args.d
        self.echo_input = args.e
        self.ignore_hidden = args.h
        self.ignore_language = args.i
        self.line_length = (72, 80)[args.l]
        self.mac_file = args.m
        self.nvdf = args.n
        self.r4 = args.r4
        self.print_variables = args.p
        self.print_report = not args.s
        self.doc_in_database = not args.x
        self.ww_file = args.w
        self.yy_file = args.y
        self.std = args.std
        self.incdir = args.incdir
        self.ptrprefix = args.ptrprefix

        if not args.blockdata and not args.noblockdata:
            self.blockdata = 1
        elif args.blockdata and args.noblockdata:
            raise RuntimeError, \
                  'mac: only one of the options -blockdata and -noblockdata should be specified.'

#        self.dump()

        if not args.nonopts or args.H or args.help:
            sys.stdout.write('Usage: mac inputfile [-b] [-d docname] [-e] [-h] [-m macname] [-p] [-s] [-x] [-y yname] -CPU\n')
            sys.exit(0)

        if len(args.nonopts) > 1:
            sys.stderr.write('mac: more than one input file specified; will use first.\n')

        self.vdf = args.nonopts[0]
        
    def dump(self):
        for (attr, value) in self.__dict__.items():
            print '%s : %s' % (attr, str(value))

########################################
#
#
def mac():
    import basis.preprocess as preprocess
    import basis.vdf as vdf
    import basis.mppl as mppl

    opts = MACoptions()

    filename = opts.vdf
    # check if file exists

    filter = preprocess.SystemFilter(filename, opts.cpu)

    pkg = vdf.read_vdf(filter, filename)
#    vdf.dump_vdf_nodes(pkg)

    # reset real variable types to 32-bits
    if opts.r4:
        var_type = vdf.type_table['real']
        var_type.decl  = 'real'
        var_type.ctype = 'real4'
        var_type.mppl_type = 'real'

        var_type = vdf.type_table['complex']
#        var_type.decl = 'real'
        var_type.ctype = 'real4'

#        var_type = vdf.type_table['default_real']
#        var_type.decl  = 'real'
#        var_type.ctype = 'real4'
#        var_type.mppl_type = 'real'

    opts.package_name = pkg.name

    if opts.docfilename:
        mppl.PkgDump_Doc(pkg, opts)
        if opts.print_report:
            print 'Wrote document file', opts.docfilename

    mppl.PkgDump_PreProcess(pkg, opts)

    if opts.debug:
        import rdp.parsedebug
        rdp.parsedebug.print_tree(pkg)
    mppl.PkgDump_Cname(pkg, opts)

    if opts.mac_file:
        out_name = opts.mac_file
    else:
        out_name = 'mac' + opts.package_name
    opts.fp = open(out_name, 'w')
    mppl.PkgDump_MPPL(pkg, opts)
    opts.fp.close
    if opts.print_report:
        print 'Wrote file containing definitions of your groups,', out_name

    mppl.gen_code_mppl(pkg, opts)

    if opts.print_report:
        print '%s: %s has %d groups, %d variables, %d functions' % \
              (filename, opts.package_name, pkg.ng, pkg.nv, pkg.nf)

    if opts.crotinger:
        import basis.crot as crot
        crot.gen_code_crot(pkg, opts)
    else:
        import basis.fcc as fcc
        fcc.gen_code_c(pkg, opts)
        fcc.gen_code_fcc(pkg, opts)

    if opts.incdir:
        import basis.mac90 as mac90
        mac90.vdf_using_cray_pointers(pkg, opts)

        #mac90.f90ize_types()
        #    mac90.vdf_to_f90(pkg)

########################################
#
#
if __name__ == '__main__':
    mac()

