#!/bin/env python
#
"""
Implements a simple form of acquisition.

Use ExplicitAcquisition as a mixin to replace the getattr function.
The acquire class variable defines which attributes to attempt to acquire.
The _of instance variable defines who the container for this class is.

"""

class ExplicitAcquisition:
    acquire = []
    _of = None
    def __getattr__(self, name):
        try:
            if name in self.__class__.__dict__['acquire']:
                of = self.__dict__['_of']
                return getattr(of, name)
        except KeyError:
            pass
        raise AttributeError, "'%s' instance has no attribute '%s'" % \
              (self.__class__.__name__, name)

if __name__ == '__main__':
    class Test:
        def dump(self, label='dump'):
            print label
            print "a = ", self.a
            print "b = ", self.b
            print "c = ", self.c
    
    class A(Test):
        def __init__(self):
            self.a = 1
            self.b = 1
            self.c = 1
            self.d = 1

    class B(ExplicitAcquisition, Test):
        acquire = ['a']
        def __init__(self):
            self.b = 2
            self.c = 2

    class C(ExplicitAcquisition, Test):
        acquire = ['a', 'b']
        def __init__(self):
            self.c = 3

    a = A()
    b = B()
    b._of = a
    
    c = C()
    c._of = b

    print 'set 1'
    a.dump('a')
    b.dump('b')
    c.dump('c')

    print 'set 2'
    b.a = 2
    b.c = 22  # C will not acquire
    a.dump('a')
    b.dump('b')
    c.dump('c')

    print 'set 3'
    c.a = 4
    c.b = 4
    c.c = 4
    a.dump('a')
    b.dump('b')
    c.dump('c')

    print c.d



'''
Test inherited acquistion:

class Abstract_VariableNode(acquisition.ExplicitAcquisition):
    acquire = ['ftype', 'common', 'limited']
    def __init__(self):
        pass

class VariableNode(Abstract_VariableNode):
    def __init__(self):
        Abstract_VariableNode.__init__(self, name)

class GroupNode:
    def __init__(self):
        self.ftype     = None
        self.limited   = None
        self.common    = None   # name of common block

    def add_variable(self, variable):
        variable._of = self  # acquisiton: variable of group
        self.append(variable)

g = GroupNode()
v = VariableNode()
g.add_variable(v)

# does not work, will not inherit acquire
print v.ftype 
'''
