#
# basistypes.py
#
import os
from string import join
from rdp.parsetools import AST
import acquisition


import f90
# instead of subclassing all of the class, just assume they're the same
# unless explicitly subclassed.
from f90 import *

# XXX - move here from vast.py

class Parameter(AST):
    def __init__(self, name, expr_str=None, value=None):
        AST.__init__(self, 'parameter')
        self.name = name
        self.expr_str = expr_str
        self.value = value

class Define(AST):
    def __init__(self, name):
        AST.__init__(self, 'define')
        self.name = name

########################################
#
#
class BasisProgram(AST):    #class symtab: #(py_methods):
    def __init__(self, name):
	AST.__init__(self, 'program')
	self.name = name
	self.object = None
	self.package_list = []
	self.module_dict = {}
	# XXX - singleton
#	global_sym.append(self)

########################################
#
#
class BasisPackage(AST):
    """A Basis Package.

    parameters and groups are separtated to allow them to be processed
    individually. getChildren will return both.
    """
    def __init__(self, name):
	AST.__init__(self, 'package')
	self.current = os.path.abspath(os.curdir)
	self.paths = ['.']  # always search the current directory
	self.name  = name
	self.package_dict = {}
	self.dict_list = []
	self.symtab    = {}
        self.comments = []

        self.parameters = []
        self.groups = []

    def getChildren(self):
        return self.parameters + self.groups

    def add_group(self, group):
        self.groups.append(group)
    
# used with VDF files

########################################
#
#
class Pcomment(AST):
    def __init__(self, *args):
	AST.__init__(self, 'pcomment')
        self.contents = args

########################################
#
#
class LiteralText(AST):
    def __init__(self, *args):
	AST.__init__(self, 'literaltext')
        self.contents = args

########################################
#
#
class ArgumentNode(AST):
    def __init__(self, dummy_arg, ftype=None, addr=0):
	AST.__init__(self, 'argument')
        self.dummy_arg = dummy_arg
        self.ftype     = ftype
        self.separator = ''
        if addr:
            self.addr  = addr  # '&'
        else:
            self.addr  = ''

########################################
#
#
class ArgumentsNode(AST):
    def __init__(self, *args):
	AST.__init__(self, 'arguments')
        self.optional = 0
        self.args_str = '()'

    def num_args(self):
        return len(self._kids)
    
    def has_string(self):
        '''return true if any argument is a string.'''
        for rank in self:
            if rank.ftype and rank.ftype.name == 'string':
                return 1
        return 0

########################################
#
#
class VDF_Variable(f90.FortranVariable, acquisition.ExplicitAcquisition):
    acquire = ['common', 'limited', 'units']
    def __init__(self, name, dim_spec=None, init=None, type_spec=None):
        f90.FortranVariable.__init__(self, name, dim_spec, init, type_spec)
        self.compileas   = None
        self.limited     = None
        self.plus_attrs  = []
        self.minus_attrs = []
#        self.init        = None
        self.alias       = None
        # self.units       = None # acquire
        self.comments  = []

        # set with set_dimstr
        self.dim_str     = ''
        self.dims        = []  ##### FortranArrayShape() - allows empty iterations

    def get_type(self):
        '''extract the type_spec and the type_var.'''
        return (self.type_spec, self.type_spec.attrs.var_type)

########################################
#
#
class Abstract_VariableNode(AST, acquisition.ExplicitAcquisition):
    '''This abstract call is used to contain attributes common to
    variables, externals, builtins, and commands. It should not be
    instanceated.
    '''

    def __init__(self, name, *args):
        self.name        = name
        #self.object      = None
        # self.limited     = None # acquired
        self.compileas   = None
        #        self.shape       = None
        # self.ftype       = None # acquired
        self.plus_attrs  = []
        self.minus_attrs = []
        self.init        = None
        self.alias       = None
        # self.units       = None # acquire
        self.comments  = []

        # set with set_dimstr
        self.dim_str     = ''
        
########################################
#
#
class VDF_ExternalNode(Abstract_VariableNode):
#    acquire = ['ftype']
    def __init__(self, name, *args):
        Abstract_VariableNode.__init__(self, name)
	AST.__init__(self, 'vdf_external')

        self.units = 'none'  # units do not apply, default to 'none'

########################################
#
#
class VDF_BuiltinNode(Abstract_VariableNode):
    acquire = ['ftype']
    def __init__(self, name, *args):
        Abstract_VariableNode.__init__(self, name)
	AST.__init__(self, 'builtin')

        self.units = None
        self.dim_str = ''

########################################
#
#
class VDF_CommandNode(Abstract_VariableNode):
    acquire = ['ftype']
    def __init__(self, name, *args):
        Abstract_VariableNode.__init__(self, name)
	AST.__init__(self, 'command')

########################################
#
#
class GroupNode(AST):
    def __init__(self, name, *args):
	AST.__init__(self, 'group')
        self.name      = name
        self.ftype     = None   # XXX-YYY
        self.type_spec = None
        self.kind      = None
        self.comments  = []
        self.limited   = None
        self.compileas = None
        self.units     = 'none'
        self.common    = None   # name of common block
        self.language  = 'fortran'
        self.attrs     = []
        self.hidden    = 0
        self.no_names  = 0
        
        self.local     = 0
        self.variables = []
        self.literal   = []

    def add_variable(self, variable):
        variable._of = self  # acquisiton: variable of group
        self._kids.append(variable)
        self.variables.append(variable)

########################################
#
#
#class UserTypeNode(AST):
#    def __init__(self, name, ftype):
#	AST.__init__(self, 'user_type')
#        self.name = name
#        self.ftype = ftype
        
########################################
#
#
class FortranType(f90.FortranType):
    "Description of a Fortran type."
    def __init__ (self, name, kind=None, length=None, derived=0, decl=None,
                  ctype=None, character=0, vdf_type=None,
                  mppl_type=None):
	f90.FortranType.__init__(self, name, kind, length, derived, decl)
        self.ctype = ctype
        self.character = character
        if vdf_type is None:
            # default vdf_type to the same as the fortran type
            self.vdf_type = self.decl
        else:
            self.vdf_type = vdf_type
        if mppl_type is None:
            # default vdf_type to the same as the fortran type
            self.mppl_type = self.decl
        else:
            self.mppl_type = mppl_type

########################################
#
#
class PackageNode(AST):
    def __init__(self, *args):
	AST.__init__(self, 'package')

########################################
#
#
class Visit_VDF_Node(f90.Visit_F90_Node):
    '''Print out the tree.

    useful for debugging.
    '''

    def visitBasisPackage(self, node):
        self.writeto("Package: %s\n" % node.name)
        self.doindent()
        self.writeto("user types\n")
        for unode in node.userorder:
            self.visit(unode)
        for child in node.getChildren():
            self.visit(child)
        self.deindent()

    def visitParameter(self, node):
        if node.value is node.expr_str:
            self.writeto('parameter %s = %s\n' % (node.name, node.expr_str))
        else:
            self.writeto('parameter %s = %s # %s\n' % (
                node.name, node.expr_str, node.value))
        # don't visit children here

    def visitDefine(self, node):
        if node.value is node.expr_str:
            self.writeto('define %s = %s\n' % (node.name, node.expr_str))
        else:
            self.writeto('define %s = %s # %s\n' % (
                node.name, node.expr_str, node.value))
        # don't visit children here

    def visitLiteralText(self, node):
        self.writeto('literal: %s\n' % node.contents)

    def visitGroupNode(self, node):
        self.writeto('Group:%s\n' % node.name)
        self.doindent()
        self.writeto('local=%(local)d, common=%(common)s, hidden=%(hidden)d, compileas=%(compileas)s, language=%(language)s\n' %  node.__dict__)
        self.writeto('type=%(ftype)s, kind=%(kind)s\n' % node.__dict__)
        if node.attrs:
            self.writeto(' attrs=%s\n' % join(node.attrs, ' '))
#        for line in node.literal:
#            self.writeto(line)
        for child in node.getChildren():
            self.visit(child)
        self.deindent()

    def helper_attrs(self, node):
        self.writeto('plus_attrs=%s\n' % join(node.minus_attrs, ' '))
        self.writeto('minus_attrs=%s\n' % join(node.plus_attrs, ' '))

    def visitVDF_Variable(self, node):
        self.writeto('Variable: %s\n' % (node.name))
        self.doindent()
        self.writeto(node.f90_decl() + '\n')
        self.writeto('init=%s\n' % node.init)
        self.writeto('alias=%(alias)s\n' % node.__dict__)
        self.writeto('common: %s\n' % node.common)
        self.helper_attrs(node)
        for child in node.getChildren():
            self.visit(child)
        self.deindent()

    def visitVDF_ExternalNode(self, node):
        self.writeto('VDF_ExternalNode: %s\n' % node.name)
        self.doindent()
        self.visit(node.type_spec)
#        self.writeto('init=%s\n' % node.init)
#        self.writeto('alias=%(alias)s\n' % node.__dict__)

        if node.args:
            for arg in args:
                self.visit(arg)
#####            Visit_VDF_Node(node.args, self.icount)
        else:
            self.writeto('No arguments')

        self.helper_attrs(node)
        for child in node.getChildren():
            self.visit(child)
        self.deindent()

    def visitVDF_BuiltinNode(self, node):
        self.writeto('Builtin: %s\n' % node.name)
        self.doindent()
        self.helper_attrs(node)
        for child in node.getChildren():
            self.visit(child)
        self.deindent()

    def visitVDF_CommandNode(self, node):
        self.writeto('Command: %s\n' % node.name)
        self.doindent()
        self.helper_attrs(node)
        for child in node.getChildren():
            self.visit(child)
        self.deindent()

    def visitTypeSpec(self, node):
        self.writeto(node.f90_decl() + '\n')

    def visitArraySpec(self, node):
        self.writeto(node.f90_decl() + '\n')
        # don't visit children here

    def visitArguments(self, node):
        self.writeto('Shape: opt=%s\n' % node.optional)
        self.doindent()
        for child in node.getChildren():
            self.visit(child)
        self.deindent()

    def visitArgument(self, node):
        self.writeto('%s:%s\n' % (node.dummy_arg, node.ftype.decl))
    
    def visitType(self, node):
        self.writeto('TypeNode: %s\n' % node.name)
        self.doindent()
        self.writeto('decl=%(decl)s  vdf_type=%(vdf_type)s\n' % node.__dict__)
        self.deindent()

def dump_vdf_nodes(ast, fp=None):
    walk(ast, Visit_VDF_Node(fp))

########################################
#
#
class Visit_VDF_Code:

    def visitPackage(self, node):
        print node.name
        for child in node.getChildren():
            self.visit(child)
        
    def visitComment(self, node):
        for comment in node.contents:
            print "#%s" % str(comment)
        for child in node.getChildren():
            self.visit(child)

    def visitPcomment(self, node):
        for comment in node.contents:
            print "$%s" % str(comment)
        for child in node.getChildren():
            self.visit(child)

    def visitLiteralText(self, node):
        for text in node.contents:
            print "%%%s" % str(text)
        for child in node.getChildren():
            self.visit(child)

    def visitGroupNode(self, node):
        print "*** Group %s:" % (node.name)
        for child in node.getChildren():
            self.visit(child)

    def visitVDF_Variable(self, node):
        if node.dims:
            dims = str(node.dims)
        else:
            dims = ''
        print "%s%s %s" % (node.name, dims, node.ftype.ftype)
        for child in node.getChildren():
            self.visit(child)

def print_vdf_nodes(ast, fp=None):
    walk(ast, Visit_VDF_Code(fp))
    
