#
# crot.py - Crotinger interface
#
import macio
import sys
from string import join
from rdp.visitor import walk

# The next array is those types handled by the Crotinger conventions
# I will add to this as I gain understanding of what he does
# and as I figure out how to do additional types.
jims_type = {
    'integer':'Integer',
    'Filedes':'Integer',
    'integer(Size4)':'Integer',
    'real(Size4)':'Real',
    'complex':'Complex',
    'double':'Double',
    'double precision':'Double',
    'real(Size8)':'Double',
    'logical':'Logical',
    'Address':'Address',
    'type(address)':'Address',
    'string':'Character'
    }

jims_type['real'] = 'Real'

# the following is indexed into by the number of commas in a
# dimension specification (or that number - 1 in the case of character)
array_type = [
    'Vector',
    'Matrix',
    'Array3',
    'Array4' 
    ]


class C_Mixin:
    def get_dim_list(self, variable):
        # size will be the total linear size of the array
        # dimlist will be of the form lower1,upper1,...
        type_spec = variable.type_spec
        var_type = type_spec.attrs.var_type
        shape = []
        current = []
        if var_type.decl == 'complex':
            shape   = ['2']
            current = ['1', '2']
        if variable.dim_spec:
            shape   = variable.dim_spec.get_size(shape)
            current = variable.dim_spec.get_dim_list(current)
            ncommas = variable.dim_spec.rank() - 1
        else:
            ncommas = -1
        if var_type.character:
            if var_type.length and var_type.length != 1:
                shape.append(var_type.length)
                current.append('1')
                current.append(str(var_type.length))
                ncommas += 1
        if shape:
            size = '[%s]' % join(shape, '*')
            dimlist = join(current, ',')
        else:
            size = ''
            dimlist = ''
        return (size, dimlist, ncommas)


########################################
#
#
#
class PkgVisit_Crotinger(C_Mixin):
    #
    # this will put all of the defines at the end of the file
    # instead of with the matching struct.  Is to so it will
    # match mac-perl.
    #
    def __init__(self, options):
        self.options = options

        wrapper = macio.Template()
        wrapper.dotH1 = []
        wrapper.dotH2 = []
        wrapper.dotC = []
        wrapper.cppgrpdefs = []
        wrapper.crotdefs = []
        wrapper.sync_body = []
        wrapper.defines = []
        self.wrapper = wrapper
        self.ddd = {}

    def visitBasisPackage(self, package):
        # usertypes become typedefs in C, C++. Hope this is correct.
        for unode in package.userorder:
            j = jims_type.get(unode.decl)
            if j:
                self.wrapper.dotH1.append('typedef %s %s;' %
                                          (j, unode.vdf_type))
        self.wrapper.package_name = package.name
        self.wrapper.PACKAGE_NAME = package.name.upper()
        self.ddd['pname'] = package.name
        self.current_package = package

        for child in package.getChildren():
            self.visit(child)

        hname = package.name + '.h'
        fp = open(hname, 'w')
        self.wrapper.write(fp, stringers.dotH1)
        fp.close()

        cname = package.name + 'Base.cc'
        fp = open(cname, 'w')
        # XXX-XXX for a linefeed to compare
        if not self.wrapper.sync_body_list:
            self.wrapper.sync_body_list += '\n'
        self.wrapper.write(fp, stringers.basecc)
        fp.close()
        if self.options.print_report:
            sys.stdout.write(
                'Wrote c++ header files: %s & %s\n' % (hname, cname))

    def visitParameter(self, node):
        self.wrapper.defines.append('#define %s %s' % (node.name, node.value))

    def visitDefine(self, node):
        self.wrapper.defines.append('#define %s (%s)' % (node.name, node.value))

    def visitGroupNode(self, group):
        ddd = self.ddd
        ddd['gname'] = group.name
        wrapper = self.wrapper

        wrapper.dotH2.append('\n// Group %(gname)s\n' % ddd)

        wrapper.crotdefs.append('\n/* Group %(gname)s */' % ddd)

        wrapper.dotC.append('\n// Group %(gname)s\n' % ddd)
       
        self.cppgrpdefs = []

        self.Ccommons = {}
        self.current_group = group

        for child in group.getChildren():
            self.visit(child)

        cppgrpdefs = self.cppgrpdefs

        Ccommons = self.Ccommons
        foo = '''
extern "C" struct {
%(Ccommons_list)s
} %(cmn_name)s;'''
        
        Ccommon_names = self.Ccommons.keys()
        Ccommon_names.sort()
        for Ccommon_name in Ccommon_names:
            if not Ccommons[Ccommon_name]: continue
            ddd['cmn_name'] = Ccommon_name
            ddd['Ccommons_list'] = join(Ccommons[Ccommon_name], '\n')
            self.wrapper.dotC.append(foo % ddd)
        # XXX-XXX extra linefeeds to match
        if cppgrpdefs:
            self.wrapper.dotC.append('\n' + join(cppgrpdefs, '\n') + '\n')
        else:
            self.wrapper.dotC.append('\n')

    def visitVDF_ExternalNode(self, variable):
        wrapper = self.wrapper

        ddd = self.ddd
        ddd['nns_u'] = variable.name.upper()
        ddd['nns_l'] = variable.name.lower()
        ddd['name'] = variable.name

        # handle functions/subroutines for Crotinger interface
        if variable.object in ('subroutine', 'csubroutine'):
            cpp_vartype = 'void'
        else:
            cpp_vartype = jims_type[variable.type_spec.attrs.var_type.decl]
        ddd['cpp_vartype'] = cpp_vartype

        if not cpp_vartype:
            sys.stderr.write("WARNING: c++ can't process $vartype c-function $names[$i]\n")
            sys.stderr.write("     so it wasn't added to files $package_name.h & ${package_name}Base.cc\n")
            return

        if variable.object == 'function':
            typ = 'function'
        elif variable.object == 'subroutine':
            typ = 'subroutine'
        elif variable.object == 'cfunction':
            typ = 'c-function'
        else:
            typ = 'c-subroutine'
        ddd['typ'] = typ

        wrapper.crotdefs.append(
            '#define %(pname)s_%(name)s FF_ID(%(nns_l)s, %(nns_u)s)          /* %(typ)s */' % ddd)

        proto = self.protoC(variable.args)
        ddd['proto0'] = proto[0]
        ddd['proto1'] = proto[1]
        ddd['proto2'] = proto[2]
        wrapper.dotH2.append('  static %(cpp_vartype)s %(name)s %(proto0)s;' % ddd)
        if typ[0:2] == 'c-':
            typ = 'c++ ' + typ[2:]
            if variable.object == 'cfunction':
                body = '      return %(pname)s::%(name)s%(proto2)s;' % ddd
            else:
                body = '      %(pname)s::%(name)s%(proto2)s;' % ddd
            ddd['typ'] = typ
            ddd['body'] = body
            foo = '''
// %(name)s is a %(typ)s
extern "C" {
  %(cpp_vartype)s %(pname)s_%(name)s%(proto1)s
    {
%(body)s
    }
};'''
            wrapper.dotC.append(foo % ddd)
        else:
            typ = 'fortran ' + typ
            if variable.object == 'function':
                body = '      return ::%(pname)s_%(name)s%(proto2)s;' % ddd
            else:
                body = '      ::%(pname)s_%(name)s%(proto2)s;' % ddd
                #                    '      ${package_name}::$names[$i]$proto[2];' ;
            ddd['typ'] = typ
            ddd['body'] = body
            foo = '''
// %(name)s is a %(typ)s
extern "C" %(cpp_vartype)s %(pname)s_%(name)s%(proto0)s;
%(cpp_vartype)s %(pname)s::%(name)s%(proto1)s
  {
%(body)s
  }'''
            wrapper.dotC.append(foo % ddd)

    def protoC(self, args):
        # Accepts as argument the string specifying names and possibly argument
        # types for a function or subroutine in C++ to be called from FORTRAN
        # (or vice versa).
        # Turns it into appropriate strings for a Crotinger interface file.
        if args.num_args() == 0:
            return ( '( void )' , '()' , '()' )

        result0 = []
        result1 = []
        result2 = []
        swarn = 0

        for v in args:
            if v.ftype:
                if v.ftype.character and not swarn:
                    sys.stderr.write(
                        "(mac) WARNING: c++ can't handle argument of type string\n")
                    swarn = 1
                else:
                    # XXX-XXX more correct would be to use ftype.decl
                    #  but we use vdf_type to match mac's output
                    cpt = jims_type[v.ftype.vdf_type]
                    if not cpt:
                        sys.stderr.write(
                            "mac: can't interface a parameter of type %s.\n" %
                            v.ftype.decl)
                        sys.exit(1)
                result0.append(' %s &' % cpt)
                result1.append(' %s &%s' % (cpt, v.dummy_arg))
            elif v.dummy_arg[0] in 'ijklmn':
                # default to integer
                result0.append(' Integer &')
                result1.append(' Integer &%s' % v.dummy_arg)
            elif v.dummy_arg[0] in 'abcdefghopqrstuvwxyz':
                # default to real
                result0.append(' Real &')
                result1.append(' Real &%s' % v.dummy_arg)
            else:
                raise RuntimeError, \
                      "protoC: %s starts with an illegal character." % v
            result2.append(' %s' % v.dummy_arg)
        return ('(%s )' % join(result0, ','),
                '(%s )' % join(result1, ','),
                '(%s )' % join(result2, ','))

    def visitVDF_Variable(self, variable):
        wrapper = self.wrapper
        cppgrpdefs = self.cppgrpdefs

        ddd = self.ddd
        ddd['nns_u'] = variable.common.upper()
        ddd['nns_l'] = variable.common.lower()
        ddd['name'] = variable.name

        if variable.type_spec.attrs.var_type.character:
            cpp_vartype = 'Char'  # XXX-
        else:
            try:
                cpp_vartype = jims_type[variable.type_spec.attrs.var_type.decl]
            except KeyError:
                raise RuntimeError, \
                      'mac: %s is not valid for sharing with C++.' % \
                      variable.type_spec.attrs.var_type.decl
        ddd['cpp_vartype'] = cpp_vartype

        size, dimlist, ncommas = self.get_dim_list(variable)
        ddd['size'] = size
        ddd['dimlist'] = dimlist
            
        if ncommas >= 0:
            ddd['array'] = array_type[ncommas]

        wcmn = '%s_%s' % (variable.common.upper(), self.current_group.name)
        ddd['common'] = wcmn

        if not variable.alias:
            Ccommons = self.Ccommons
            if not Ccommons.get(wcmn):
                Ccommons[wcmn] = []
                wrapper.crotdefs.append(
                    '#define %(common)s FCB_ID(%(nns_l)s, %(nns_u)s)' % ddd)

        type_spec = variable.type_spec
        var_type = type_spec.attrs.var_type
        if type_spec.dynamic():
            # BEGIN treatment of pointer variables
            #            if variable.ftype.character:
            #                ddd['cpp_vartype'] = 'Char'
            Ccommons[wcmn].append(
                '  %(cpp_vartype)s *%(name)s;' % ddd)

            if ncommas == -1:
                cppgrpdefs.append(
                    '%(cpp_vartype)s* &%(pname)s::%(name)s = %(common)s.%(name)s;' % ddd)
                wrapper.dotH2.append(
                    '  static %(cpp_vartype)s* &%(name)s;' % ddd)
            else:
                cppgrpdefs.append(
                    '%(array)s<%(cpp_vartype)s> %(pname)s::%(name)s(false);' % ddd)
                wrapper.dotH2.append(
                    '  static %(array)s<%(cpp_vartype)s> %(name)s;' % ddd)
                wrapper.sync_body.append(
                    '  %(name)s.init(%(dimlist)s, %(common)s.%(name)s);' % ddd)

        elif var_type.character:
            # character types or usertypes which are -character--
            #if ( $vartype =~ /\*\(?([^)]+)\)?$/o ) {
            #   $ncommas += 1 if $1 ne '1' ; #add a dimension if more than *1
            #  }
            #            wrapper.dotC.append(stringers.dot_char % ddd)
            # XXX - this oddness is carried over from mac
            #    Char vs char for named or not common blocks
            if self.current_group.common:
                Ccommons[wcmn].append(
                    '  char %(name)s%(size)s;' % ddd)
            else:
                Ccommons[wcmn].append(
                    '  Char %(name)s%(size)s;' % ddd)

            if ncommas == -1:
                cppgrpdefs.append(
                    'Char  &%(pname)s::%(name)s = %(common)s.%(name)s;' %
                    ddd)
                wrapper.dotH2.append(
                    '  static Char  &%(name)s;' % ddd)
            else:
                cppgrpdefs.append(
                    '%(array)s<Char> %(pname)s::%(name)s(%(dimlist)s, %(common)s.%(name)s);' % ddd)
                wrapper.dotH2.append(
                    '  static %(array)s<Char> %(name)s;' % ddd)

        else:
            # BEGIN treatment of non-character, non-pointer variables
            #            cpp_vartype = jims_type[variable.type_spec.attrs.var_type.decl]
            #            if not cpp_vartype:
            #                raise RuntimeError, '$vartype is not valid for sharing with C++'
            Ccommons[wcmn].append(
                '  %(cpp_vartype)s %(name)s%(size)s;' % ddd)

            if ncommas == -1:
                cppgrpdefs.append(
                    '%(cpp_vartype)s  &%(pname)s::%(name)s = %(common)s.%(name)s;' % ddd)
                wrapper.dotH2.append(
                    '  static %(cpp_vartype)s  &%(name)s;' % ddd)
            else:
                cppgrpdefs.append(
                    '%(array)s<%(cpp_vartype)s> %(pname)s::%(name)s(%(dimlist)s, %(common)s.%(name)s);' % ddd)
                wrapper.dotH2.append(
                    '  static %(array)s<%(cpp_vartype)s> %(name)s;' % ddd)

def PkgDump_Crotinger(ast, options):
    walk(ast, PkgVisit_Crotinger(options))

########################################
#
#
def gen_code_crot(pkg, opts):
    PkgDump_Crotinger(pkg, opts)

########################################
#
# This class defined a bunch of strings.
# They are placed here to avoid cluttering stuff up too much
# and to avoid confusing the emacs python mode.
class stringers:
######################################################################

    dot_char = '''
extern "C" struct {
   Char %(name)s%(size)s;
} %(common)s;
'''

######################################################################
    dotH1 = '''/* FILE: %(package_name)s.h */

#if !defined(__%(package_name)s__)
#define __%(package_name)s__

#include <basconf.h>

#if defined(__cplusplus)
#if !defined(__Basis__)
#include "Basis.h"
#endif
#if !defined(__Vector__)
#include "Vector.h"
#endif
#if !defined(__Array3__)
#include "Array3.h"
#endif
#if !defined(__Array4__)
#include "Array4.h"
#endif
#if !defined(__Character__)
#include "Character.h"
#endif
#if !defined(__CCLComplex__)
#include "CCLComplex.h"
#endif
#endif

#if !defined(__cplusplus)
#include "SysConfig.h"
#endif

/* Definitions / Parameters */

#define Pi 3.14159265358979323
%(defines_list)s%(crotdefs_list)s
#if defined(__cplusplus)

// %(package_name)s package class definition

class %(package_name)s {
public:
%(dotH2_list)s
#if defined(UseCTLPackage)
  static Integer gen(Vector<Integer> &optlist, Integer &nopt);
  static Integer exe(Vector<Integer> &optlist, Integer &nopt);
  static Integer fin(Vector<Integer> &optlist, Integer &nopt);
#endif

  static void sync();
};

#endif /* __cplusplus */

/* Run-controller package stuff: */

#if defined(UseCTLPackage)
#define %(package_name)s_%(package_name)sgen FF_ID(%(package_name)sgen, %(PACKAGE_NAME)sGEN)
#define %(package_name)s_%(package_name)sexe FF_ID(%(package_name)sexe, %(PACKAGE_NAME)sEXE)
#define %(package_name)s_%(package_name)sfin FF_ID(%(package_name)sfin, %(PACKAGE_NAME)sFIN)
#endif


#endif  /* __%(package_name)s__ */

'''

######################################################################

    basecc = '''/* FILE: %(package_name)sBase.cc */

#if !defined(__%(package_name)s__)
#include "%(package_name)s.h"
#endif

#if defined(UseCTLPackage)
  extern "C" Integer %(package_name)s_%(package_name)sgen(Integer &, Integer &);
  extern "C" Integer %(package_name)s_%(package_name)sexe(Integer &, Integer &);
  extern "C" Integer %(package_name)s_%(package_name)sfin(Integer &, Integer &);
#endif


%(dotC_list)s// Sync function initializations

void %(package_name)s::sync()
{
%(sync_body_list)s}

// Run-controller package stuff:

#if defined(UseCTLPackage)

extern "C" {
  Integer %(package_name)s_%(package_name)sgen(Integer &optlist, Integer &nopt)
    {
      Vector<Integer> optlist1(32, &optlist);
      return %(package_name)s::gen(optlist1, nopt);
    }
  Integer %(package_name)s_%(package_name)sexe(Integer &optlist, Integer &nopt)
    {
      Vector<Integer> optlist1(32, &optlist);
      return %(package_name)s::exe(optlist1, nopt);
    }
  Integer %(package_name)s_%(package_name)sfin(Integer &optlist, Integer &nopt)
    {
      Vector<Integer> optlist1(32, &optlist);
      return %(package_name)s::fin(optlist1, nopt);
    }
};

#endif  /* ifdef(UseCTLPackage) */
'''
