#
# fcc.py - Fortran Calls C
#
import macio
import string
from string import join
from rdp.visitor import walk


########################################
#
#
#
class PkgVisit_Cinterface:
    #
    # this will put all of the defines at the end of the file
    # instead of with the matching struct.  Is to so it will
    # match mac-perl.
    #
    def __init__(self, options, nametab):
        self.options = options
        self.nametab = nametab

        wrapper = macio.Template()
        wrapper.h1 = []
        wrapper.H1 = []
        wrapper.doth2 = []
        wrapper.dotH2 = []
        wrapper.dotC  = []
        self.wrapper = wrapper
        self.ddd = {}

    def visitBasisPackage(self, package):
        self.someC    = 0
        self.someCh   = 0
        self.someCPP  = 0
        self.someCPPh = 0
        for group in package.groups:
            if group.language == 'C':
                self.someC = 1
                self.someCh = 1
            elif group.language == 'C++':
                self.someCPP = 1
                self.someCPPh = 1

        # usertypes become typedefs in C, C++. Hope this is correct.
        for unode in package.userorder:
            if unode.ctype and unode.ctype != 'char':    ###XXX-YYY and not unode.dynamic:
                line = 'typedef %(ctype)s %(vdf_type)s;' % unode.__dict__
                if self.someCh:
                    self.wrapper.h1.append(line)
                if self.someCPPh:
                    self.wrapper.H1.append(line)

#        self.ddd['pname'] = package.name
        self.wrapper.pname = package.name

        for child in package.getChildren():
            self.visit(child)

        opts = self.options

        if opts.nvdf:
            file_prefix = os.path.splitext(self.options.mac_file)[0]
        else:
            file_prefix = self.options.package_name

        if self.someCh:
            file_to_open = file_prefix + '_.h'
            fp = open(file_to_open, 'w')
            self.wrapper.write(fp, stringers.h_file)
            fp.close()
            if opts.print_report:
                print 'Wrote header file,', file_to_open

#        if self.someC:
#            file_to_open = file_prefix + '.h'
#            self.c1 = open(file_to_open, 'w')

        if self.someCPPh:
            file_to_open = file_prefix + '_.H'
            fp = open(file_to_open, 'w')
            self.wrapper.write(fp, stringers.H_file)
            fp.close()
            if opts.print_report:
                print 'Wrote header file,', file_to_open

        if self.someCPP:
            file_to_open = file_prefix + 'CBase.C'
            fp = open(file_to_open, 'w')
            self.wrapper.write(fp, stringers.C_file)
            fp.close()
            if opts.print_report:
                print 'Wrote C++ interface file,', file_to_open

    def visitParameter(self, node):
        self.wrapper.h1.append('#define %s %s' % (node.name, node.value))
        self.wrapper.H1.append('#define %s %s' % (node.name, node.value))

    def visitDefine(self, node):
        self.wrapper.h1.append('#define %s (%s)' % (node.name, node.value))
        self.wrapper.H1.append('#define %s (%s)' % (node.name, node.value))

    def c_comment(self, cmmt):
        # C style comments
        return '/* %s */' % cmmt
        
    def cpp_comment(self, cmmt):
        # C++ style comments
        return '// %s' % cmmt
        
    def visitGroupNode(self, group):
        wrapper = self.wrapper
        ddd = self.ddd
        ddd['gname'] = group.name
        
        if group.local:
            return
        elif group.language == 'C':
            ddd['lang'] = ''
            self.header = wrapper.h1
            self.doth2  = wrapper.doth2
            self.do_comment = self.c_comment
        elif group.language == 'C++':
            ddd['lang'] = '"C" '
            self.header = wrapper.H1
            self.doth2  = wrapper.dotH2
            self.do_comment = self.cpp_comment
        else:
            return

        self.Ccommons = {}
        self.doth1 = []
        #        doth2 = []

        for child in group.getChildren():
            self.visit(child)

        # XXX-XXX convert to self.comment('Group %s' % group.name)
        ddd = self.ddd
        Ccommons = self.Ccommons

        self.header.append('\n/* Group %(gname)s */' % ddd)

        if group.common:
            ddd['comment'] = self.do_comment(
                'The FORTRAN (user-defined) common block looks like a C struct with mangled name')
        else:
            ddd['comment'] = self.do_comment(
                'The FORTRAN common block looks like a C struct with mangled name')

        common_names = Ccommons.keys()
        common_names.sort()
        for common_name in common_names:
            if not Ccommons[common_name]: continue
            ddd['common_name'] = common_name
            ddd['ucommon_name'] = common_name.upper()
            ddd['lcommon_name'] = common_name.lower()
            ddd['ccommons'] = join(Ccommons[common_name], '\n')
            self.header.append(stringers.struct % ddd)
        self.header.append('')


            #        header.write(join(doth1,'\n'))
            #        header.write('\n')
        # header.write(join(doth2,'\n'))
        # header.write('\n')

    def visitVDF_Variable(self, variable):
        wrapper = self.wrapper
        Ccommons = self.Ccommons

        (type_spec, var_type) = variable.get_type()
        if not Ccommons.get(variable.common):
            Ccommons[variable.common] = []

        cpp_type = var_type.ctype
        if cpp_type is None:
            print "00000:", variable.name, var_type.name, var_type.kind, var_type.length
            raise RuntimeError, \
                  'mac: %s is not valid for sharing with C or C++.' \
                  % var_type.decl

        ### XXX-XXX see C_Mixin
        # size, dimlist = self.get_dim_list(variable)
        # size will be the total linear size of the array
        # dimlist will be of the form lower1,upper1,...
        shape = []
        current = []
        if var_type.decl == 'complex':
            shape   = ['2']
            current = ['1', '2']
        if variable.dim_spec:
            shape   = variable.dim_spec.get_size(shape)
            current = variable.dim_spec.get_dim_list(current)
        if shape:
            size = '[%s]' % join(shape,'*')
            dimlist = join(current,',')
        else:
            size = ''
            dimlist = ''

        if variable.alias:
            al = string.translate(variable.alias,
                                  string.maketrans('()','[]'))
            if al[-1] == ']':
                al = al[:-1] + '-1]'  # adjust for 0-based indexing
            self.doth2.append('#define %s %s' % (variable.name, al))
        else:
            self.doth2.append('#define %s cb_%s.%s' %
                         (variable.name, variable.common, variable.name))
            if type_spec.dynamic():
                Ccommons[variable.common].append('  %s *%s;' %
                                                 (cpp_type, variable.name))
            elif var_type.character:
                if var_type.length:
                    size += '[%s]' % var_type.length
                Ccommons[variable.common].append('  char %s%s;' %
                                                 (variable.name, size))
            else:
                Ccommons[variable.common].append('  %s %s%s;' %
                                                 (cpp_type, variable.name, size))

def PkgDump_Cinterface(ast, options, nametab):
    walk(ast, PkgVisit_Cinterface(options, nametab))
    
########################################
#
#
#
class PkgVisit_Fcc:
    '''Determine final variable attributes using group inheritance.
    And compute some other attributes.'''
    def __init__(self, options, fcc):
        self.options = options
        self.fcc = fcc

    def visitGroupNode(self, group):
        if group.language in ('C', 'C++'):
            for child in group.getChildren():
                self.visit(child)

    def visitVDF_ExternalNode(self, variable):
        if variable.object == 'subroutine':
            line = ['subroutine ']
        elif variable.object == 'function':
            line = ['%s ' % variable.type_spec.attrs.var_type.mppl_type]
        elif variable.object in ('csubroutine', 'cfunction'):
            raise RuntimeError, \
                  "mac: '%s' not allowed without '-c' option." % variable.object
        else:
            raise RuntimeError, \
                  "mac: '%s' unknown" % variable.object

        if variable.alias:
            line.append(variable.alias)
        else:
            line.append(string.capitalize(variable.name.lower()))

        line.append(self.proto(variable.args))

        self.fcc.append(join(line, ''))

    def proto(self, args):
        # Accepts as argument the string specifying names and possibly argument
        # types for a function or subroutine in C++ to be called from FORTRAN.
        # Turns it into an appropriate string of types for a .fcc file.
        if not args:
            return '()'

        result = []
        for v in args:
            result.append(v.addr + v.ftype.vdf_type)
        return '(%s)' % join(result, ',')

def PkgDump_Fcc(ast, options, fcc):
    walk(ast, PkgVisit_Fcc(options, fcc))

########################################
#
#
def gen_code_c(pkg, opts):
    nametab = {}
    PkgDump_Cinterface(pkg, opts, nametab)

########################################
#
#
def gen_code_fcc(pkg, opts):
    fcc = []
    PkgDump_Fcc(pkg, opts, fcc)

    if fcc:
        if opts.nvdf:
            file_prefix = os.path.splitext(opts.mac_file)[0]
        else:
            file_prefix = opts.package_name
        file_to_open = file_prefix + 'Wrap.fcc'
        fp = open(file_to_open, 'w')
        fp.write('\n')
        fp.write(join(fcc, '\n'))
        fp.write('\n')
        fp.close()
        if opts.print_report:
            print 'Wrote C/C++ wrapper file,', file_to_open

########################################
#
# This class defined a bunch of strings.
# They are placed here to avoid cluttering stuff up too much
# and to avoid confusing the emacs python mode.
# XXX - convert to template files
##
class stringers:
######################################################################
    
    h_file = '''/* FILE: %(pname)s_.h */

#if !defined(__%(pname)s__)
#define __%(pname)s__

#include <basconf.h>
#include "Fcc.h"

/* Definitions / Parameters */

#define Pi 3.14159265358979323
%(h1_list)s%(doth2_list)s
#endif /* __%(pname)s__ */
'''

######################################################################
    H_file = '''/* FILE: %(pname)s_.H */

#if !defined(__%(pname)s__)
#define __%(pname)s__

#include <basconf.h>
#include "Fcc.h"

/* Definitions / Parameters */

#define Pi 3.14159265358979323
%(H1_list)s%(dotH2_list)s
#endif /* __%(pname)s__ */
'''

######################################################################
    C_file = '''/* FILE: %(pname)sCBase.C */
#if !defined(__%(pname)s__)
#include "%(pname)s_.H"
#endif

%(dotC_list)s'''

######################################################################
    struct = '''
%(comment)s
typedef struct {
%(ccommons)s
} %(common_name)s_group;

#define cb_%(common_name)s FCB_ID(%(lcommon_name)s, %(ucommon_name)s)
extern %(lang)s%(common_name)s_group cb_%(common_name)s;'''



