#
# mppl.py -
#
from string import join, split
import macio

from cStringIO import StringIO
from rdp.visitor import walk

########################################
#
#
class Mppl_Mixin:
    # Now an attribute of group
#    def process_group(self, group):
#        if (not group.hidden or self.options.ignore_hidden) \
#           and not group.local:
#            self.current_group = group
#            return 1
#        else:
#            return 0

    def dump_list(self, list, prefix, suffix, continuation, indent, this_sub):
        """Dump a list to standard output.
        split if line is over 72 characters.
        """

        next = indent * ' '

        line = [prefix]
        nline = len(prefix)
        if list:
            if nline + len(list[0]) + 1 > 71:
                this_sub.append(prefix)
                line = [next]
                nline = len(next)
            line.append(list[0])
            nline += len(list[0])
        
        for name in list[1:]:
            if nline + 1 > 71:
                line.append(continuation)
                this_sub.append(join(line,''))
                line = [next]
                nline = len(next)
            line.append(',')
            nline += 1
            
            if nline + len(name) + 1 > 71:
                line.append(continuation)
                this_sub.append(join(line,''))
                line = [next]
                nline = len(next)
            line.append(name)
            nline += len(name)

        if nline + 1 > 71:
            this_sub.append(join(line,''))
            line = [next]
        line.append(suffix)
        this_sub.append(join(line,''))

########################################
#
#
class PkgVisit_PreProcess:
    '''Count members and check flags.'''

    def __init__(self, options):
        self.options = options

    def visitBasisPackage(self, package):
        self.package = package

        package.nv  = 0 # number of variables
        package.nf  = 0 # number of functions

        self.ng  = 0 # number of groups

        self.nhg = 0 # non-hidden groups
        self.nhv = 0 # non-hidden variables

        for child in package.getChildren():
            self.visit(child)

        package.ng  = self.ng

        package.nhg = self.nhg
        package.nhv = self.nhv

    def visitGroupNode(self, group):
        '''Count number of variables and functions in each group'''
        
        if (not group.hidden or self.options.ignore_hidden) \
           and not group.local:
            group.process = 1
        else:
            group.process = 0

        self.nv  = 0 # number of variables
        self.nf  = 0 # number of functions

        for child in group.getChildren():
            self.visit(child)

        group.no_names = self.nv + self.nf
        self.ng += 1
        group.number = self.ng

        if group.process:
            self.nhg += 1
            self.nhv += group.no_names

        group.nv  = self.nv
        group.nf  = self.nf

        self.package.nv += group.nv
        self.package.nf += group.nf

    def visitVDF_Variable(self, node):
        self.nv += 1

    def visitVDF_ExternalNode(self, node):
        self.nf += 1

    def visitVDF_BuiltinNode(self, node):
        self.nf += 1

def PkgDump_PreProcess(ast, options):
    walk(ast, PkgVisit_PreProcess(options))

########################################
#
#
class PkgVisit_Doc(Mppl_Mixin):
    def __init__(self, options):
        self.options = options

    def visitBasisPackage(self, package):
        self.fp = open(self.options.docfilename, 'w')
        self.fp.write('\n                      Package %s\n' %
                      (package.name))
        for child in package.getChildren():
            self.visit(child)
        self.fp.close()

    def visitGroupNode(self, group):
        if group.hidden or group.local:
            # if not group.process:
            return
        fp = self.fp
        fp.write("\n                      %s\n" % (group.name))
        fp.write("                      ------------------------\n\n")
        #       if ( $first_comment && $current_name eq "" && $docfile &&
        #           ! $hidden[$ng-1] && ! $local[$ng-1] ) ;
        for comment in group.comments:
            fp.write('          %s\n' % comment)
        fp.write("                      ------------------------\n")
        for child in group.getChildren():
            self.visit(child)

    def visitVDF_Variable(self, variable):
        fp = self.fp
        fp.write('%s ' % variable.name)
        shortdim = self.breakup(variable.dim_str)
        fp.write(shortdim)
        fp.write('\n')

        for comment in variable.comments:
            fp.write('%s %s\n' % (variable.name, comment))

    def visitVDF_ExternalNode(self, node):
        fp = self.fp
        fp.write('%s ' % node.name)
        shortdim = self.breakup(node.args.args_str)
        fp.write(shortdim)
        fp.write('\n')

        for comment in node.comments:
            fp.write('%s %s\n' % (node.name, comment))

    def visitVDF_BuiltinNode(self, node):
        fp = self.fp
        fp.write('%s ' % node.name)
        shortdim = self.breakup(node.args_str)
        fp.write(shortdim)
        fp.write('\n')

        for comment in node.comments:
            fp.write('%s %s\n' % (node.name, comment))

    def breakup(self, dimstring):
        # take a dimension string and break it up into multiple lines.
        # this is out of deference to those people who have functions
        # with 10,000 arguments and build a DOCFILE1.
        # (ZCM 2/6/96) OK, I have changed the cutoff point to approximately
        # 500 characters, for those people who want functions with lots
        # of arguments but fairly clean printouts from 'LIST'.
        return dimstring

    
#   local ( $dimstring ) = @_ ;
#   return ( $dimstring ) if length($dimstring) <= 450 ;
#   local ( $newstring , $thisline ) = ( "" , "          (" ) ;
#   $dimstring =~ s/[()]//go ;   # remove parens
#   while ( $dimstring ne "" ) {
#       $dimstring =~ s/([^,]+,?)//o ;
#       if ( length($thisline) + length($1) > 490 ) {
#           $newstring .= $thisline . "\n" ;
#           $thisline = $current_name . 
#              substr ( "            " , 0 , 10 - length ( $current_name )) ;
#          }
#       $thisline .= $1 ;
#      }
#   $newstring .= $thisline if length($thisline) > 10 ;
#   $newstring =~ s/          //o ;
#   return ( $newstring . ")" ) ;
#  }


def PkgDump_Doc(ast, options):
    walk(ast, PkgVisit_Doc(options))
        
########################################
#
#
class PkgVisit_Cname:
    '''Determine the common block name for each variable.'''

    def __init__(self, options=None):
        self.options = options
        # The purpose of the next array is to recognize standard FORTRAN types
        # and give the last digit of the common block number.
        # We can expand to 3 further types (7, 8, and 9) if necessary.
        self.std_types = {
            'integer': 0,
            'integer(Size4)':0,
            'logical':1,
            'real(Size4)':2,
            'real(Size8)':3,
            'double':3,
            'double precision':3,
            'complex':4,
            'Address':5,
            'Pointer':6,
            'longint':7,
            }
        if options is None:  # XXX-XXX - make options optional?
            self.std_types['real'] = 3
        elif self.options.r4:
            self.std_types['real'] = 2
        else:
            self.std_types['real'] = 3

    def visitBasisPackage(self, node):
        self.ncmnpref = node.name # common name prefix for std variables in this group
        self.cmnno    = 0         # for sequential numbering of common blocks
        self.chcmnno  = 10000     # for character and usertypes
        # (character and user types are each put into separate
        # common blocks for alignment purposes, and we want
        # them numbered separately from regular variables so
        # that adding a variable to a group won't change the
        # numbering of regular common blocks.)
        for child in node.getChildren():
            self.visit(child)

    def visitGroupNode(self, group):
        self.common = {}
        for child in group.getChildren():
            self.visit(child)
        self.cmnno += 1

    def visitVDF_Variable(self, variable):
        # If a common name is defined for the group, give it to all variables
        if not variable.common:
            type_spec = variable.type_spec
            var_type = type_spec.attrs.var_type

            if type_spec.dynamic():
                vartype = 'Pointer'
            else:
                vartype = var_type.mppl_type

            cname = self.common.get(vartype)
            if not cname:
                # Note: 0 is a legal response from std_types, thus true/false
                # is not a valid test (can not tell 0 from None)
                try:
                    sf = self.std_types[vartype]
                except KeyError:
                    cname = '%s%d' % (self.ncmnpref, self.chcmnno)
                    self.chcmnno += 1
                else:
                    cname = '%s%d%d' % (self.ncmnpref, self.cmnno, sf)
                if not var_type.character:
                    self.common[vartype] = cname
            variable.common = cname

def PkgDump_Cname(ast, options):
    walk(ast, PkgVisit_Cname(options))

########################################
#
#
#
class PkgVisit_MPPL:
    '''Dump the MPPL macros to define a package'''

    def __init__(self, options):
        self.options = options

    def visitBasisPackage(self, package):
        self.package = package
        fp = self.options.fp
        #        for (utype, unode) in package.usertypes.items():
        for unode in package.userorder:
            #            if unode.ftype != unode.full_type: #XXX-XXX
            if unode.define:
                fp.write('define %s %s\n' % (unode.name, unode.mppl_type))
                fp.write('define _%s %s\n' % (unode.name, unode.mppl_type))
        for child in package.getChildren():
            self.visit(child)

    def visitLiteralText(self, node):
        fp = self.options.fp
        fp.write('%s\n' % node.contents)

    def visitParameter(self, node):
        self.options.fp.write('define %s %s\n' % (node.name, node.value))

    def visitDefine(self, node):
        self.options.fp.write('define %s %s\n' % (node.name, node.value))

    def visitGroupNode(self, group):
        fp = self.options.fp

        #        print MACPKG1 $machdr if $language[$ng-1] == $FORTRAN ||
        #        $language[$ng-1] == $CPLUSPLUS ||
        #        $language[$ng-1] == $C ;
        fp.write('define([Use%(name)s],[Immediate([c Group %(name)s])\\\n' %
                 group.__dict__)
        for line in group.literal:
            fp.write(line + '\n')

        # XXX - move dynamics as declare['Address']
        dynamics = []
        commons = {}
        declare = {}   # indexed by type
        declared = {}  # indexed by name
        aliased  = {}

        literals = []
        index = 0
        last_variable = 0
        
        for variable in group.getChildren():
            index += 1
            if variable.type == 'literaltext':
                literals.append((index, variable.contents))
            else:
                last_variable = index

        # print all literal before the last variable together
        for (index, literal) in literals:
            if index < last_variable:
                fp.write('\n'.join(literal)+'\n')


        for variable in group.variables:
            if variable.type != 'variable_decl': continue
            if not commons.get(variable.common):
                commons[variable.common] = []
# XXX - 2.0
#            commons.setdefault(variable.common, [])

            declared[variable.name] = 1
            type_spec = variable.type_spec
            var_type  = type_spec.attrs.var_type

            if type_spec.dynamic():
                # Save up dynamic statements to be put out at end of group.
                # Thus any other declarations, including dynamic dimension
                # variables, can be put out first, as some compilers require.
                if var_type.character:
                    # Dynamic characters now handled as Address type, though
                    # they go in database as usual.
                    dynamics.append('      Address %s' % variable.name)
                    commons[variable.common].append(variable.name)
                elif self.options.std:
                    dynamics.append('      Address %(name)s' %
                                   variable.__dict__)
                    commons[variable.common].append(variable.name)
                else:
                    if variable.dim_spec:
                        shape = variable.dim_spec.get_shape()
                        if len(shape) == 1:
                            dim2 = ', %s' % shape[0]
                        else:
                            dim2 = ', [%s]' % join(shape,',')
                    else:
                        dim2 = ''
                    dynamics.append('      Dynamic(%s, %s%s)' %
                                    (variable.name, var_type.mppl_type, dim2))
                    commons[variable.common].append('Point(%s)' % variable.name)

            elif var_type.character:
                # XXX-XXX - remove the write here in favor of assigning
                # to a variable. Keep for now to match perl-mac.
                # This section can also be collapsed into the next clause
                fp.write('      %s %s%s\n' %
                         (var_type.mppl_type, variable.name, variable.dim_str))
##                fp.write('      common /%(common)s/ %(name)s\n' %
##                         variable.__dict__)
                commons[variable.common].append(variable.name)
            else:
                # BEGIN treatment of non-character, non-pointer variables
                # all other types (non-character, non_pointer):
                # for each variable the user specifies, we add it to a line in the
                # associative array for that type.
                vartype = var_type.mppl_type
##                if not declare.get(vartype):
##                    declare[vartype] = []
                ddd = declare.setdefault(vartype, [])

                # Now put the variable in the appropriate FORTRAN type declaration:
##                declare[vartype].append(variable.name + variable.dim_str)
                ddd.append(variable.name + variable.dim_str)

                # XXX - this kludge forces each line out as it is full
                # This is what mac currently does.
                # However, remove them and it will still work, just the
                # resulting lines will be group together.
                line = '      %s %s' % \
                       (vartype, join(declare[vartype], ', '))
                if len(line) > 72:
                    line = '      %s %s\n' % \
                           (vartype, join(declare[vartype][:-1], ', '))
                    fp.write(line)
                    del declare[vartype][:-1]
                # XXX -

                if variable.alias:
                    # if I have an alias, see if the alias is already in common
                    # (need to remove dimensions, if any, to get bare name)
                    my_alias = split(variable.alias, '(', 1)[0]
                    # figure out if the alias has been previously declared
                    if not declared.has_key(my_alias):
                        # It's apparently not in commons, so put it there
                        # unless of course this is a local group
                        if not group.local:
                            commons[variable.common].append(my_alias)
                        aliased[my_alias] = 1
                    # we have an alias, put out declaration and then
                    # equivalence statement

                    line = '      %s %s\n' % \
                           (vartype, join(declare[vartype], ', '))
                    declare[vartype] = []
                    fp.write(line)
                    # XXX if ($cpu eq "SOL" && ! $f90) {
                    #               print MACPKG1 "      ifelse(SYSTEM,SUNOS, static $names[$i])\n" ;
                    fp.write('      equivalence(%s,%s)\n' %
                             (variable.name, variable.alias))

                elif aliased.has_key(variable.name):
                    # if I am already in common, aliased[names]
                    # will have been set and now should be cleared
                    del aliased[variable.name]
                else:
                    # just a regular guy so put me in common
                    commons[variable.common].append(variable.name)

        self.dump_dict(declare, '      %s ')

        if dynamics:
            fp.write(join(dynamics, '\n'))
            fp.write('\n')

        if not group.local:
            self.dump_dict(commons, '      common /%s/ ')

        # print all literal after the last variable
        for (index, literal) in literals:
            if index > last_variable:
                fp.write('\n'.join(literal)+'\n')

#        for child in group.getChildren():
#            self.visit(child)
        self.options.fp.write('Immediate([c End of %(name)s])\\\n])\n' % group.__dict__)

    def dump_dict(self, dict, prefix):
        '''Dump a dictionary to standard output.
        split if line is over 72 characters.'''
        fp = self.options.fp
        keys = dict.keys()
        keys.sort()
        for key in keys:
            header = prefix % (key)
            line = []
            nline = len(header)
            for n in dict[key]:
                if nline + len(n) + 2 > 72:
                    fp.write(header + join(line, ', ') + '\n')
                    line = []
                    nline = len(header)
                # if other items on line, include length of separator (", ")
                if line:
                    nline = nline + len(n) + 2
                else:
                    nline = nline + len(n)
                line.append(n)
            if line:
                fp.write(header + join(line, ', ') + '\n')

#    def add_character_common:

           # character types or usertypes which are -character--
           # we put out the declaration statement right away.
	   # If this is not a named common, we also put out a common block now.
           # If this is a named common, we hope the user was smart
           # enough to have only a single character declaration in it
	   # or at least know what their doing.


#    def visitVDF_Variable(self, variable):
#        if variable.character:
            

        
#        if not variable.object:
#            self.add_variable(variable)
#        elif variable.object in ('function', 'subroutine',
#                                     'cfunction', 'csubroutine'):
#            self.add_function(variable)
#        elif variable.object == 'command':
#            self.add_command(variable)
#        elif variable.object == 'builtin':
#            self.add_builtin(variable)
#        else:
            # should never get here--this would be a name with
            # an unallowed object designation
            #die "mac: $names[$i] has unknown object designation." ;
#            raise RuntimeException, "mac: $names[$i] has unknown object designation." ;
    
def PkgDump_MPPL(ast, options):
    walk(ast, PkgVisit_MPPL(options))

########################################
#
#
#
class PkgVisit_jvar(Mppl_Mixin):
    # Avoid emitting jvar and/or i000addr declarations
    # and getting a compiler warning, if these variables are not needed.

    def __init__(self, options, wrapper):
        self.options = options
        self.wrapper = wrapper

    def visitBasisPackage(self, package):
        self.jvar     = ''
        self.i000addr = ''
        for child in package.getChildren():
            self.visit(child)
        self.wrapper.jvar = self.jvar
        self.wrapper.i000addr = self.i000addr

    def visitGroupNode(self, group):
        if not group.process:
            return
        if self.jvar and self.i000addr:
            # already found,
            return
        for child in group.getChildren():
            self.visit(child)

    def visitVDF_Variable(self, node):
        if node.dims:
            self.i000addr = '      Address i000addr'

    def visitVDF_ExternalNode(self, node):
        self.jvar = ', jvar'

def PkgDump_jvar(ast, options, wrapper):
    walk(ast, PkgVisit_jvar(options, wrapper))

########################################
#
#
#
class PkgVisit_Init(Mppl_Mixin):
    '''Write code to initialize variables.'''
    
    def __init__(self, options, wrapper):
        self.options = options
        self.wrapper = wrapper

    def visitBasisPackage(self, node):
        self.wrapper.package_name = node.name
        self.wrapper.block_external = '      external %siyiyi' % node.name

        # now we put out a Use statement for each group in the database.
        lines = []
        for group in node.groups:
            if group.local:
                continue
            if group.language in ('fortran', 'C++', 'C'):
                 lines.append('Use(%(name)s)' % group.__dict__)
        self.wrapper.use_all = lines

        # go through groups one-by-one looking for variables
        inits   = []
        initptr = []

        lines = StringIO()
        dat = '      data '
        datlen  = 72 - len(dat)
        datlist = ''

        for group in node.groups:
##            self.fp.write('c     Group %(name)s\n' % group.__dict__)
            if group.no_names == 0: continue # make sure it has variables
            if group.local: continue
            # look at variables in this group
            for variable in group.variables:
                if variable.type != 'variable_decl': continue
                # get the variable's type (we need to know if it's dynamic)
                (type_spec, var_type) = variable.get_type()

                if type_spec.dynamic():
                    if var_type.character or self.options.std:
                        initptr.append('      %s = NOTSET' % \
                                       variable.name)
                    else:
                        initptr.append('      Point(%s) = NOTSET' % \
                                       variable.name)

                elif variable.init: # non-pointer variable with initial value
                    var = '%(name)s/%(init)s/' % variable.__dict__

                    if not datlist:
                        # push first variable on if list is empty
                        datlist = var
                    elif len(datlist) + 1 + len(var) <= datlen:
                        # if there's room for it on nonempty list, put it on with comma
                        datlist += ',' + var
                    else:
                        # otherwise print out the list and start a new one
                        lines.write(dat)
                        lines.write(datlist)
                        lines.write('\n')
                        datlist = var

                    if len(datlist) > datlen:
                        # we've got a really long list, print it on multiple lines.
                        lines.write(dat)
                        lines.write(variable.name)
                        lines.write('\n     &')
                        datlist = '/%s/' % variable.init
                        j = 0
                        # finds the next occurrence of '&'
                        # after the current position of j
                        while j >= 0:
                            # if there aren't more than 66 characters left after '&',
                            # there is no need to subdivide further
                            j1 = j + 67
                            if j1  > len(datlist) + 1:
                                break
                            # is there a comma anywhere after the ampersand
                            # and before the current position?
                            if datlist[j:j1].find(',') >= 0:
                        	# if so, back up to the very last one and insert a break
                                j1 = datlist[j:j1].rfind(',') + j + 1
                            else:
                                # if there are no preceding commas, we probably have
                                # a very long piece of data, e. g., a string, and we'll
                                # have to break on the 66th character.
                                j1 -= 1
                            lines.write(datlist[j:j1])
                            lines.write('\n     &')
                            j = j1
                        lines.write(datlist[j:])
                        lines.write('\n')
                        datlist = ''

            # print anything that's left over.
            if datlist:
                lines.write(dat)
                lines.write(datlist)
                lines.write('\n')
                datlist = ''

        self.wrapper.ptr_inits = initptr

        self.wrapper.data_stmts = lines.getvalue()

    def visitGroupNode(self, node):
        pass

    def visitVDF_Variable(self, node):
        pass
    
def PkgDump_Init(ast, options, wrapper):
    walk(ast, PkgVisit_Init(options, wrapper))

########################################
#
#
#
class Segs:
    def __init__(self, number, fnames):
        # fnames is a list of tuples - (function name, external name)
        #
        self.number = number
        self.fnames = fnames

    def __int__(self):
        return self.number

########################################
#
#
#
class PkgVisit_Database(Mppl_Mixin):
    '''Write code to define variables to database. a.k.a the yy file'''
    
    def __init__(self, options, wrapper):
        self.options = options
        self.wrapper = wrapper

        self.separator = '/*' + 74*'-' + '*/'
        self.wrapper.separator = self.separator

    def visitBasisPackage(self, package):
        wrapper = self.wrapper
        wrapper.package_name = package.name
        wrapper.nhg = package.nhg
        wrapper.nhv = package.nhv

        self.dbcr = []         # database creation code
        self.sub_code = []     # wrapper subroutine code for functions w/ strings
        self.c_wrappers = []

        self.wrapper.dbcr = self.dbcr
        self.wrapper.sub_code = self.sub_code
        self.wrapper.c_wrappers = self.c_wrappers

        self.isegs = []
        self.ggg = 0

        self.nbf = 0
        self.ddd = {}
        self.lines = []

        for child in package.getChildren():
            self.visit(child)

        # put out the rest of the routine and continue
        #($newdocfilename = $docfilename) =~ /([^\/]+$)/ ;
        #$newdocfilename = $1 ;

        if self.options.docfilename:
            import os.path
            file = os.path.basename(self.options.docfilename)
            self.dbcr.append('      call rtsetdoc(drtdm,\"%s\")' % file)

        # to do the function parxpf, we have to look at isegs.
        wrapper = self.wrapper
        wrapper.xpf_func = []
        if self.isegs:
            lines = []
            else_kw = ''
            for iseg in self.isegs:
                lines.append('      %sif(iseg.eq.%d) then' %
                            (else_kw, iseg))
                lines.append('         call %sxp%d(name1234)' %
                            (package.name, iseg))
                else_kw = 'else'
            lines.append("      else")
            lines.append("         call baderr('%(name)sxpf: impossible event')" % package.__dict__)
            lines.append("      endif")
            wrapper.xpf_ifblock = lines
        else:
            wrapper.xpf_ifblock = ["      call baderr('%(name)sxpf: impossible event (no such function '//name5678//').')" % package.__dict__]

        # now put out each separate routine for each group with functions
        fcn_wrapper = macio.Template()
        fcn_wrapper.package_name = package.name
        for iseg in self.isegs:
            fcn_wrapper.iseg = iseg
            lines = []
            for (name, fwrapper, funcname) in iseg.fnames:
                lines.append('      external [%s], [%s]' % (fwrapper, funcname))
            fcn_wrapper.external = lines
            else_kw = ''
            lines = []
            for (name, fwrapper, funcname) in iseg.fnames:
                lines.append("      %sif(name1234 = '%s') then"
                         % (else_kw, name))
                lines.append('         call parexecf([%s], [%s])' %
                             (fwrapper, funcname))
                else_kw = 'else'
            fcn_wrapper.ifblock = lines
            wrapper.xpf_func.append(fcn_wrapper.expand(stringers.group_xp))

        if self.nbf == 0: # no builtins; put out default pkgbfcn
            wrapper.pkg_bfcn = stringers.database_bfcn % package.__dict__
        else:
            wrapper.pkg_bfcn = ''

    def visitGroupNode(self, group):
        if not group.process:
            return
        if group.language not in ('fortran', 'C++', 'C'):
            import sys
            sys.stderr.write("mac: group %s not processed;" % (group.name) +
                             " its language option not implemented yet.\n" +
                             "     Variables and functions are included in " +
                             "the following count, however.\n")
            return

        # The next variable tells whether we send out C++ stuff
        # nothing happens if group is hidden (unless ignore_hidden is on)
        # or if group is local.
        #        gg = g + 1 ;
        
        # array fnames will contain names of function objects
        self.fnames = []
        
        dbcr = self.dbcr
        dbcr.append('      jgrp=rtgrpe(drtdm,"%s")' % group.name)
        dbcr.append('      if(jgrp = 0) go to 999')

        for attribute in group.attrs:
            dbcr.append('      call rtgattre(jgrp,drtdm,Quote(%s))' \
                     % attribute)
        if self.options.doc_in_database:
            for comment in group.comments:
                dbcr.append('      call rtcomeg(jgrp, drtdm,\n"%s")' \
                            % comment)

        for child in group.getChildren():
            self.visit(child)

        # array isegs will contain group numbers that contain compiled functions
        self.ggg += 1
        if self.fnames:
            self.isegs.append(Segs(self.ggg, self.fnames))
            
    def helper_attrs(self, variable):
        dbcr = self.dbcr
        # if name has + or - attributes, take care of it
        for attr in variable.plus_attrs:
            dbcr.append('      call rtvattre(i0001234,drtdm,Quote(+%s))' %
                     attr)
        for attr in variable.minus_attrs:
            dbcr.append('      call rtvattre(i0001234,drtdm,Quote(-%s))' %
                     attr)
        # if name has comments, enter into database
        if self.options.doc_in_database:
            for comment in variable.comments:
                dbcr.append('      call rtcomev(i0001234, drtdm,\n"%s")' %
                            comment)

    def visitVDF_Variable(self, variable):
        dbcr = self.dbcr
        #-------------VARIABLE--------------------------
        if self.options.print_variables:
            print variable.name
            # AAAAAAAA - ftype.dynamic
####        vartype = variable.ftype
        dim     = variable.dim_str # XXX-XXX
        type_spec = variable.type_spec
        var_type = type_spec.attrs.var_type
        dynamic = type_spec.dynamic()

        vartype = var_type.mppl_type

        ### &continue_dims ;
        n = variable.alias or variable.name  ####$alias[$i] ? $alias[$i] : $names[$i] ;
        if var_type.character and dynamic:
            vartype = '_' + vartype
            p = 'TRUE'
        elif dynamic:
            vartype = '_' + vartype
            p = 'TRUE'
            if not self.options.std:
                n = "Point(%s)" % n
        else:
            p = "FALSE"

        if dim:
            dbcr.append('      i000addr = varadr ( %s )' % n)
            address = 'i000addr'
        else:
            address = 'varadr(%s)' % n
            dim = 'scalar'

        dbcr.append('      i0001234=rtvare(drtdm,"%s",%s,%s,Quote(%s),'
                 % (variable.name, address, p, vartype))
        dbcr.append('Quote(%s), "%s")' % (dim, variable.units))
        dbcr.append('      if(i0001234 = 0) go to 999')
        
        # if name is dynamic with initial value, do rtdynset
        quotes = '"' + "'"
        if dynamic and variable.init:
            # XXX-XXX this is very messy
#            if variable.character or variable.init[0] in quotes:
            if variable.type_spec.attrs.var_type.character or \
               variable.init[0] in quotes:
                dbcr.append('      call rtdynstc(i0001234,drtdm,%(init)s)'
                         % variable.__dict__)
            else:
                dbcr.append('      call rtdynset(i0001234,drtdm,%(init)s)'
                         % variable.__dict__)
        # if name has limits set, enter into database
        lmt = variable.limited
        if lmt:
            dbcr.append('      if (setlimit("%s.%s",Quote(%s)) ~= OK) goto 999'
                     % (self.wrapper.package_name, variable.name,
                        lmt.explicit_shape()))
        self.helper_attrs(variable)

    def visitVDF_ExternalNode(self, variable):
        # give a function a type
        if variable.object in ('function', 'cfunction'):
            var_type = variable.type_spec.attrs.var_type
            vartype = var_type.mppl_type
            cpp_vartype = var_type.ctype
            char_func = var_type.character
            # character functions must have a wrapper
            if cpp_vartype is None:
                cpp_vartype = 'void'
            elif char_func:
                cpp_vartype = 'void'
            elif vartype == 'complex':
                # XXX - complex does not really map to a C type
                cpp_vartype = 'void'

        else:
            vartype = 'null'
            cpp_vartype = 'void'
            char_func = 0

        package_name = self.wrapper.package_name # XXX-XXX
        handler = "%s_%s_handler" % (package_name, variable.name)
        f_wrapper = "%s_%s_wrapper" % (package_name, variable.name)

        special_string = 0
        if variable.name == 'execuser':
            # not really a character function, but needs
            # additional code in wrapper
            char_func = 0
            make_f_wrapper = 1
            handler_calls  = f_wrapper
        elif variable.name in ('parselng', 'parse',
                               'parsestr', 'mcchrdef'):
            # We don't do so for parse, parsestr, and parselng,
            # however, because the possiblity of extra long string
            # arrays has made us rewrite all three.
            # (later)--also mcchrdef.
            make_f_wrapper = 0;
            char_func = 0;
            handler_calls = variable.name + '_'
            special_string = 1
        elif vartype == 'null':
            # subroutines never have wrapper (except execuser)
            make_f_wrapper = 0
            handler_calls  = variable.name
        elif cpp_vartype == 'void':
            # functions that have no corresponding C type need
            # another wrapper
            make_f_wrapper = 1
            handler_calls  = f_wrapper
        else:
            make_f_wrapper = 0
            handler_calls  = variable.name

        if make_f_wrapper:
            # Now emit code for this routine.
            self.sub_code.append(self.make_wrapper_function(
                variable, char_func, f_wrapper))
        self.fnames.append((variable.name, handler, handler_calls))



        ##########
        """Create a handler for a compiled fortran function.

        The purpose of this function is to handle string arguments correctly
        by dereferencing the address as well as passing down the magical
        length argument.

        """
        callvarnames = []
        otherargs = []
        varnames = []
        stringlens = []

        this_c_sub = []

        varnames.append('%s (*func)()' % cpp_vartype)
        if char_func:
            varnames.append('char *result')
            varnames.append('integer *result_len')

            callvarnames.append('result /* character result */')
            callvarnames.append('result_len')

            stringlens.append('_result  /* len of result */')
        elif vartype != 'null':
            # functions
            if cpp_vartype == 'void':
                varnames.append('void *result')
                callvarnames.append('result /* result */')
            else:
                varnames.append('%s *result' % cpp_vartype)

        varnames.append('void **cfcnarg')

        ii = 1
        for arg in variable.args:
            name = arg.dummy_arg
            if arg.ftype.name == 'string':
                if special_string:
                    callvarnames.append('cfcnarg[%d]  /* %s */' %
                                        (ii, name))
                    ii += 1
                    callvarnames.append('cfcnarg[%d]  /* len of %s */' %
                                        (ii, name))
                else:
                    callvarnames.append('*(char **) cfcnarg[%d]  /* %s */' %
                                        (ii, name))
                    ii += 1
                    stringlens.append('*(int *) cfcnarg[%d]  /* len of %s */' %
                                        (ii, name))
            else:
                callvarnames.append('cfcnarg[%d]  /* %s */' %
                                    (ii, name))
            ii += 1
            
        if char_func:
            varnames.append('int _result')

        # Now emit code for this routine.

        # C handler for complied functions
        this_c_sub.append(self.separator)
        this_c_sub.append('\n/* Handler for compiled routine %s. */' % variable.name )
#        this_c_sub.append('/* %s */' % variable.args_str)
        line = 'void FF_ID(%s, %s) (' % (handler.lower(), handler.upper())
        self.dump_list(varnames, line, ')', '', 4, this_c_sub)

        this_c_sub.append('{')

        if cpp_vartype == 'void':
            line = '    func('
        else:
            line = '    *result = func('

        callvarnames += stringlens
        if callvarnames:
            line += '\n        ' + ',\n        '.join(callvarnames)
        line += ');'
        this_c_sub.append(line)
        this_c_sub.append('    return;')
        this_c_sub.append('}\n')
        this_c_sub.append(self.separator)

        self.c_wrappers.append(join(this_c_sub, '\n'))
        
        ##########
        #                       # comment out for functions/subroutines since
        #                       # dimension string will be sent in installments
        #                       # &continue_dims ;
        #                       $unt = $units[$i] || $gunits[$g] || "none" ;
        #                       $unt =~ s/\[|\]//g ;
        
        # Now a loop to enter all the dimension information
        # flag if the first argument is optional
        #
        # XXX-this oddness is requied because of a null optional argument
        # dim.optional==1, yet dim.rank()==0
           
        if variable.args.optional:
            dims = ['"(;"']
            add_end = 1
        elif variable.args.num_args() == 0:
            dims = ['"()"']
            add_end = 0
        else:
            dims = ['"("']
            add_end = 1
           
#        if not variable.args: #### XXX-XXX
#            dims = ['"()"']
#        elif variable.dims.optional:
#            dims = ['"(;"']
#        else:
#            dims = ['"("']
        for arg in variable.args:
            if arg.ftype.name == 'string':
                dims.append('Quote(%(dummy_arg)s:String,)' % arg.__dict__)
                dims.append('Quote(l_%(dummy_arg)s_:integer%(separator)s)'
                            % arg.__dict__)
            else:
                dims.append('Quote(%s:%s%s)' %
                            (arg.dummy_arg, arg.ftype.mppl_type,
                             arg.separator))
        if add_end:
            dims.append('")"')

        dbcr = self.dbcr
        dbcr.append('      jvar = rtfcne1(drtdm, "%s",jgrp,Quote(%s),"%s")'
                 % (variable.name, vartype, variable.units))
        dbcr.append('      if(jvar == 0) go to 999')
        for ss in dims:
            dbcr.append('      i0001234 = rtfcne2(jvar, drtdm , %s)' % (ss))
            dbcr.append('      if(i0001234 == 0) go to 999')
        # now do the final dimension parsing
        dbcr.append('      i0001234 = rtfcne3(jvar, drtdm)')
        dbcr.append('      if(i0001234 == 0) go to 999')
        self.helper_attrs(variable)

    def make_wrapper_function(self, variable, char_func, f_wrapper):
        """Create a wrapper for a fortran compiled function.

        This would be necessary for fortran functions that return types that
        are not handled by C.  For example, functions that return character
        strings or complex types.
        """
        callvarnames = []
        otherargs = []
        varnames = []

        this_sub = []

        # give a function a type
        if variable.object in ('function', 'cfunction'):
            vartype = variable.type_spec.attrs.var_type.mppl_type
        else:
            vartype = None

        # The wrapped routine is always an external
        otherargs.append((variable.name, 'external'))

        if char_func:
            # Declare type of function
            otherargs.append((variable.name, vartype))

            # Character functions pass down the result and length
            result = variable.name + '_res'
            varnames.append(result)
            otherargs.append((result, 'character*(*)'))

            len_result = 'l_%s_res' % variable.name
            varnames.append(len_result)
            otherargs.append((len_result, 'integer'))

        elif vartype is not None:
            # Declare type of function
            otherargs.append((variable.name, vartype))

            # Non-character functions pass down a result
            result = variable.name + '_res'
            varnames.append(result)
            otherargs.append((result, vartype))

        for arg in variable.args:
            name = arg.dummy_arg
            if arg.ftype.name == 'string':
                otherargs.append((name, 'character*(*)'))
                callvarnames.append(name)
                varnames.append(name)
            else:
                otherargs.append((name, arg.ftype.mppl_type))
                callvarnames.append(name)
                varnames.append(name)

        this_sub.append('\n#     Wrapper routine for compiled routine %s.' % (variable.name))
        line = "      subroutine %s(" % f_wrapper
        self.dump_list(varnames, line, ')', '\\', 6, this_sub)
#        this_sub.append('      implicit none')

        if variable.name == 'execuser':
            this_sub.append('      Use(Stacks)')
        for (name, type) in otherargs:
            if type == 'double':
                type = 'double precision'
            this_sub.append('      %s %s' % (type, name))
        if vartype:
            line = '      %s_res = %s(' % (variable.name, variable.name)
        else:
            line = '      call %s(' % (variable.name)
        self.dump_list(callvarnames, line, ')', '\\', 6, this_sub)
        if char_func:
            this_sub.append('      %s = len(%s_res)' %
                            (len_result, variable.name))
        if variable.name == 'execuser':
            this_sub.append('      rsp = rsp + 1;')
        this_sub.append('      return\n      end\n')
######            variable.name = variable.name + '_'
        return join(this_sub, '\n')

    def visitVDF_CommandNode(self, node):
        pass
    	#-------------COMMAND---------------------------
        # WILL BE FILLED IN LATER--NEED TO ALTER BASIS

    def visitVDF_BuiltinNode(self, node):
        #-------------BUILTIN---------------------------
        self.nbf += 1
        #                       $dim = $dimstr[$i] || "()" ;
        #                       $unt = $units[$i] ;
        #                       $unt =~ s/^\[|\]$//g || die
        #                           "mac: You did not specify number of parameters " .
        #                           "for $names[$i]." ;
        dbcr = self.dbcr
        dbcr.append('      i0001234 = rtfcneb(drtdm, "%s","%s", Quote(%s))'
                 % (node.name, node.args_str, node.units))
        dbcr.append('      if(i0001234 = 0) go to 999')
        self.helper_attrs(node)

    
def PkgDump_Database(ast, options, wrapper):
    walk(ast, PkgVisit_Database(options, wrapper))

########################################
#
#
def gen_code_mppl(pkg, opts):

    wrapper = macio.Template()
    
    PkgDump_jvar(pkg, opts, wrapper)
    PkgDump_Init(pkg, opts, wrapper)
    PkgDump_Database(pkg, opts, wrapper)

    if opts.yy_file:
        file_to_open = opts.yy_file
    else:
        file_to_open = 'yy' + opts.package_name
    fp = open(file_to_open, 'w')
    fp.write(wrapper.expand(stringers.pkg_init0))
    if opts.blockdata:
        fp.write(wrapper.expand(stringers.block_data))
        wrapper.data_stmts = []
    else:
        wrapper.block_external = ''
    fp.write(wrapper.expand(stringers.pkg_data))
    fp.write(wrapper.expand(stringers.pkg_dbcr))
    fp.close()

    if opts.print_report:
        print 'Wrote database interface routines into file', file_to_open

    if opts.ww_file:
        file_to_open = opts.ww_file
    else:
        file_to_open = 'ww' + opts.package_name
    fp = open(file_to_open, 'w')
    fp.write("/* Character variables come in as bare address + length. */\n")
    fp.write("#include <basconf.h>\n")
    fp.write("#include <Fcc.h>\n\n")
    if len(wrapper.c_wrappers_list):
        fp.write(wrapper.separator)
        fp.write("\n")
        fp.write(wrapper.c_wrappers_list)
        fp.write(wrapper.separator)
        fp.write("\n")
    fp.close()
    
    if opts.print_report:
        print 'Wrote compiled function handlers into file', file_to_open

########################################
#
# This class defined a bunch of strings.
# They are placed here to avoid cluttering stuff up too much
# and to avoid confusing the emacs python mode.
# XXX - convert to template files
##
class stringers:
########################################
    pkg_init0  = r'''      subroutine %(package_name)sinit0
c initializes a Package
      integer drtdm
      common /%(package_name)srtdm/ drtdm
      Ch(32) lngn
      integer glbpknum,glbstat,izverbos
      external glbpknum,glbstat,izverbos
      drtdm = glbpknum("%(package_name)s")
      if (glbstat(drtdm) .ne. SLEEPING) return
      call glblngn(drtdm,lngn)
      if(izverbos(0).gt.0) call remark('Initializing '//lngn)
c initialize our run-time database manager
      call %(package_name)sdata
      call %(package_name)swake
c set our status
      call glbsstat(drtdm,UP)
c call user's initialization routine
      call glbinit(drtdm)
      return
      end
define([Use_All_Groups_%(package_name)s],[
%(use_all_list)s])
'''

########################################
    block_data  = r'''      block data %(package_name)siyiyi
# Replace pointer statements with Address types
define([Dynamic],[Address Point([$1])])
Use_All_Groups_%(package_name)s
%(data_stmts)s
      end
# restore definition from mppl.BASIS
define([Dynamic],[\
[$2] [$1] ifelse([$3],,,([$3]));\
pointer(Point([$1]),[$1])\
])
'''

########################################
    pkg_data  = r'''      subroutine %(package_name)sdata
Use_All_Groups_%(package_name)s
%(block_external)s
      integer iyiyiyi
      iyiyiyi = 1
%(data_stmts_list)s
%(ptr_inits_list)s
      return
       end
'''

########################################
    pkg_dbcr  = r'''      subroutine %(package_name)sdbcr(drtdm)
      integer drtdm
      call rtdbcr(drtdm,%(nhg)s,%(nhv)s)
      return
       end
      subroutine %(package_name)swake
      integer drtdm,i0001234, jgrp %(jvar)s
      integer rtgrpe, rtvare, rtfcne1, rtfcne2, rtfcne3, rtfcneb, setlimit
      external rtgrpe, rtvare, rtfcne1, rtfcne2, rtfcne3, rtfcneb, setlimit
      common /%(package_name)srtdm/ drtdm
Use_All_Groups_%(package_name)s

%(i000addr)s
      external varadr
      Address varadr


      call %(package_name)sdbcr(drtdm)
%(dbcr_list)s        return
999   continue
      call baderr("Error initializing variable in database")
        end
%(sub_code_list)s      subroutine %(package_name)sxpf(iseg,name1234)
      character*(*) name1234
      integer iseg
      character*8 name5678
      name5678=name1234
%(xpf_ifblock_list)s      return
      end
%(xpf_func_list)s%(pkg_bfcn)s'''

########################################
    database_bfcn = '''      function %(name)sbfcn(ss,sp,nargs,name1234,sx)
      integer %(name)sbfcn,ss(SS_WIDTH,1),sp,nargs
      integer sx(SS_WIDTH)
      character*(*) name1234
      character*8 name5678
      name5678=name1234
      call remark('No such builtin function ('//name5678//') in %(name)s')
      call baderr(name1234)
      return(ERR)
      end
'''

########################################
    group_xp = '''      subroutine %(package_name)sxp%(iseg)d(name1234)
      character*(*) name1234
%(external_list)s      character*8 name5678
      name5678=name1234
%(ifblock_list)s      else
         call baderr('%(package_name)sxp%(iseg)d: impossible event: '//name1234)
      endif
      return
      end'''
