#!/bin/env python

'''
preprocess.py - read a file and filter lines.

Read the input, assuming that the ARGV array contains names of input
files. diff the input according to $cpu, i. e., $cpu is a global
variable set to the character string of the CPU for which we are
processing the file.

Lines beginning with the reserved word SYSTEM in column 1 specify
conditional processing depending on CPU.  A SYSTEM line containing a
list of CPU's not including $cpu, or containing -$cpu, causes the
following lines to be skipped. A later SYSTEM line containing $cpu or
+$cpu will cause lines to be processed again. Processing will also
begin again when we switch to a new file.  This routine assumes that
lines containing CPU specifications do not contain any other useful
information.

Must pass in the architecture type.

For example:

  SYSTEM SUN4 HP700 SOL

specifies that the lines which follow will be procesed only for the
three architectures specified.  A subsequent specification such as

  SYSTEM +SGI

will add SGI to the list of architectures for which the following
lines will be specified.  Similarly -CPU for one of the allowd cpu's
mentioned above would remove that particular cpu from the list of
allowd ones.  Finally the reserved word ALL will process lines which
follow for all architectures know to mac.

The primary interface is readline() to return the next line.

It also keeps track of line numbers since the user of this will never see
some of the lines, we must report them
'''

class SystemFilter:
    def __init__(self, filename, cpu):
        self.fp = open(filename)
        self.cpu = cpu
        self.plus_cpu = '+' + cpu
        self.minus_cpu = '-' + cpu
        self.keep = 1
        self.linenum = 0

    def readline(self):
        while 1:
            self.linenum += 1
            line = self.fp.readline()
            if len(line) == 0:
                break
            if line[:6] == 'SYSTEM':
                systems = line.split()[1:]
                if 'ALL' in systems:
                    self.keep = 1
                elif self.cpu in systems:
                    self.keep = 1
                elif self.minus_cpu in systems:
                    self.keep = 0
                else:
                    # at this point, we know ALL or current cpu were
                    # not on this line
                    # if any other CPU occurs without + or -, disable this one
                    # if any other CPU occurs with + or -, don't change keep
                    for system in systems:
                        if system[0] != '-' and system[0] != '+':
                            self.keep = 0
                            break
                continue
            if self.keep:
                break
        return line

########################################
#
#
if __name__ == '__main__':
    import sys
    fp = SystemFilter(sys.argv[1], 'RS6000')

    while 1:
        line = fp.readline()
        if len(line) == 0:
            break
        print 'OUTPUT:', line,
        

        
