#!/bin/env python
#
#
#
# TODO:
# make mustbe and have global
#
# constant types - interpret the r4 options by changing one type ('real')
# constant keywords (use id() and is  'keyword is kw.integer')
#
# longint
#
# replace lexer
#   recognize 'double', 'double precision', and 'double complex'
#
# Decide if quotes should be include in the attribute of quoted strings
#  but remember, C wants double for strings
#

debug = 0

import copy
import string
from string import join

from basistypes import *
#import frame as f90
import ccode

#import sys
import rdp.parsetools as parsetools
Token = parsetools.Token

########################################
# Variables
#

keywords = [
    'precision',
#    'Size2',
#    'Size4',
#    'Size8',
#    'Size16',
    'builtin',
    'command',
    'compileas',
    'define',
    'function',
    'Group',
    'hidden',
    'language',
    'cfunction',
    'csubroutine',
    'limited',
    'local',
    'subroutine',
    'usertype',
    ]

# create a node for each type
#
# Some types do not map to c types
#

# map between fortran and C
cdecl = ccode.Declaration  # save some space in the following table

type_table = {
    'integer':           FortranType('integer',       ctype='integer'),

    'integer*2':         FortranType('integer', length='2'),
    'integer*4':         FortranType('integer', length='4',  ctype='integer'),
    'integer*8':         FortranType('integer', length='8'),
    'integer*16':        FortranType('integer', length='16'),

    'integer(Size2)':    FortranType('integer', 'Size2'),
    'integer(Size4)':    FortranType('integer', 'Size4',  ctype='integer'),
    'integer(Size8)':    FortranType('integer', 'Size8'),
    'integer(Size16)':   FortranType('integer', 'Size16'),

    'real':              FortranType('double precision', ctype='real8',
                                     vdf_type='real'),

    'real*2':            FortranType('real', length='2'),
    'real*4':            FortranType('real', length='4',  ctype='real4'),
    'real*8':            FortranType('real', length='8',  ctype='real8'),
    'real*16':           FortranType('real', length='16'),

    'real(Size2)':       FortranType('real', 'Size2'),
    'real(Size4)':       FortranType('real', 'Size4',  ctype='real4'),
    'real(Size8)':       FortranType('real', 'Size8',  ctype='real8'),
    'real(Size16)':      FortranType('real', 'Size16'),

    'complex':           FortranType('complex',           ctype='real8'),
    'complex(Size2)':    FortranType('complex', 'Size2'),
    'complex(Size4)':    FortranType('complex', 'Size4',  ctype='complex4'),
    'complex(Size8)':    FortranType('complex', 'Size8',  ctype='complex8'),
    'complex(Size16)':   FortranType('complex', 'Size16', ctype='complex16'),

    'logical':           FortranType('logical', ctype='integer'),

    'double':            FortranType('double precision', ctype='real8',
                                     vdf_type='double'),
    'double precision':  FortranType('double precision', ctype='real8'),
    'double complex':    FortranType('double complex'),

    'character':         FortranType('character', ctype='char', character=1),

    'Ch':                FortranType('character', ctype='char', character=1,
                                     vdf_type='Ch', mppl_type='Ch'),

    'Filedes':           FortranType('Filedes'),
    'Address':           FortranType('Address', decl='type(address)',
    	                             ctype='Address',
                                     vdf_type='Address', mppl_type='Address'),
    'longint':           FortranType('longint'),
    }

# native types, along with their pointer equivalents
# Note how Filedes, Address, Ch are considered native
# along with the usual integer, real, logical, and character
type_tokens = {
    'Filedes':1,
    'Address':1,
    'longint':1,
    'character':1,
    'Ch':1,
    'complex':1,
    'double':1,
    'integer':1,
    'real':1,
    'logical':1,

    '_Filedes':1,
    '_Address':1,
    '_longint':1,
    '_character':1,
    '_Ch':1,
    '_complex':1,
    '_double':1,
    '_integer':1,
    '_real':1,
    '_logical':1,
    }

user_types = {
    'Varname': FortranType('character', length='VARNAMESIZE',
                           ctype='char', character=1,
                           vdf_type='Varname', mppl_type='Varname'),
    'Filename': FortranType('character', length='FILENAMESIZE',
                            ctype='char', character=1,
                            vdf_type='Filename', mppl_type='Filename'),
    }


user_types['_Varname'] = user_types['Varname']
user_types['_Filename'] = user_types['Filename']

int_node = type_table['integer']
real_node = type_table['real']

type_var  = 1
type_char = 2
type_int  = 3
type_real = 4

symbol_table = {}
symbol_types = {}

# These symbols were implicitly defined by mppl
symbol_types['WORDSIZE'] = type_int
symbol_types['CHAR_PER_WORD'] = type_int
symbol_types['LOCS_PER_WORD'] = type_int
symbol_types['FILENAMESIZE'] = type_int
symbol_types['VARNAMESIZE'] = type_int

########################################
#
#
class vdflex:
    """A lexical analyzer class for Variable Description Files."""

    def __init__(self, instream=None, infile=None):
        if instream:
            self.instream = instream
            self.infile = infile
        else:
            # XXX- is this useful?  linenumbers will not work
            self.instream = sys.stdin
            self.infile = None

        self.idstart = string.letters + '_'
        self.idchars = self.idstart + string.digits

        self.whitespace = ' \t\r\n'
        self.quotes = '\'"'
        self.lineno = 1
        self.debug = 0
        self.token = ''
        self.linenum = 0
        self.column = 0
        self.endcolumn = 0
        self.lines = ['line numbers start at 1']
        if self.debug:
            print 'vdflex: reading from %s, line %d' \
                  % (self.instream, self.lineno)
        self.next_char()

    def position(self):
        return (self.infile, self.linenum, self.column)

    def next_char(self):
        # XXX - force a newline before an EOF?
        if self.column == self.endcolumn:
            self.line = self.instream.readline()
            self.lines.append(self.line)
            self.linenum = self.instream.linenum
            self.column = 0
            self.endcolumn = len(self.line)
            if self.endcolumn == 0:
                self.peek = '\x04'
            else:
                self.peek = self.line[0]
                self.column = 1
        else:
            self.peek = self.line[self.column]
            self.column += 1
        return self.peek

    def get_token(self):
        """Read a token from the input stream.

        This will find multicharacter tokens as well.
        """

	value = None
        peek = self.peek
        # remember where this token starts
        # This is more important for multiline tokens like comments
        self.token_linenum = self.linenum
        self.token_column = self.column
        if peek == '\x04':
            tok = 'EOF'
        elif peek == '\n':
            tok = 'newline'
            self.next_char()
        elif peek in ' \t':
            while 1:
                peek = self.next_char()
                if peek not in ' \t':
                    break
            tok = 'space'
        elif peek == '*':
            peek = self.next_char()
            if peek == '*':
                peek = self.next_char()
                if peek == '*':
                    while 1:
                        peek = self.next_char()
                        if peek != '*':
                            break
                    tok = 'newgroup'
                else:
                    tok = '**'
            else:
                tok = '*'
        elif peek == ':':
            peek = self.next_char()
            if peek == ':':
                tok = '::'
                self.next_char()
            else:
                tok = ':'
        elif peek == '[':
            # XXX - balance?
            value = ''
            while 1:
                peek = self.next_char()
                if peek == ']':
                    break
                value += peek
            self.next_char()
            tok = 'mppl_macro'
        elif peek == '#':
            value = ' '  # XXX-space is for mac compatiblity
            while 1:
                peek = self.next_char()
                if peek == '\n':
                    break
                value += peek
            self.next_char()
            tok = 'comment'
        elif peek == '$':
            value = ''
            while 1:
                peek = self.next_char()
                if peek == '\n':
                    break
                value += peek
            self.next_char()
            tok = 'pcomment'
        elif peek == '%':
            value = ''
            while 1:
                peek = self.next_char()
                if peek == '\n':
                    break
                value += peek
            self.next_char()
            tok = 'literaltext'
        elif peek in ('"', "'"):
            end = peek
            value = peek
            while 1:
                peek = self.next_char()
                value += peek
                if peek == end:
                    break
            self.next_char()
            if end == '"':
                tok = 'double_quoted'
            else:
                tok = 'single_quoted'
        elif peek == '\\':
            peek = self.next_char()
            if peek == '\n':
                tok = 'newline'
                self.next_char
            else:
                tok = '\\'
        elif peek in string.digits:
            value = peek
            while 1:
                peek = self.next_char()
                if peek not in string.digits:
                    break
                value += peek
            if peek in ('.', 'e', 'E', 'd', 'D'):
                tok = 'float'
                if peek == '.':
                    while 1:
                        value += peek
                        peek = self.next_char()
                        if peek not in string.digits:
                            break
                if peek in ['e', 'E', 'd', 'D']:
                    reset = self.column - 1
                    exp = peek
                    peek = self.next_char()
                    if peek in ['+', '-']:
                        exp += peek
                        peek = self.next_char()
                    if peek in string.digits:
                        while 1:
                            exp += peek
                            peek = self.next_char()
                            if peek not in string.digits:
                                break
                        value += exp
                        tok = 'float'
                    else:
                        tok = 'int'
                        self.column = reset
                        self.next_char()
            else:
                tok = 'number'
        elif peek in self.idstart:
            value = peek
            while 1:
                peek = self.next_char()
                if peek not in self.idchars:
                    break
                value += peek
            tok = 'id'

        else:
            tok = peek
            self.next_char()
        return (tok, value)

    def next_token(self):
        """Screen tokens.
        Throw out white space and preprocessor lines.
        convert some id's into keywords
        """
        while 1:
            (tok, value) = self.get_token()
            if tok == 'space':
                continue
            if tok == 'newline':
                continue
            break
        if tok == 'id':
            if value in keywords:
                tok = Token(value, value)
            elif type_tokens.get(value):
                tok = Token('type', value)
            elif user_types.get(value):
                tok = Token('user_type', value)
            else:
                tok = Token('id', value)
            tok.is_id = 1
        else:
            tok = Token(tok, value)
            tok.is_id = 0
                
        tok.linenum = self.token_linenum
        tok.column = self.token_column

        return tok
        
    def error_leader(self, infile=None, lineno=None):
        """Emit a C-compiler-like, Emacs-friendly error-message leader."""
        if not infile:
            infile = self.infile
        if not lineno:
            lineno = self.lineno
        return "\"%s\", line %d: " % (infile, lineno)

########################################
#
#
class Expr_Parser(parsetools.Expr_Parser):

    def __init__(self):
        LEFT = parsetools.Expr_Table.LEFT
        RIGHT = parsetools.Expr_Table.RIGHT
        table = parsetools.Expr_Table(LEFT, LEFT, RIGHT, RIGHT)
        
        table.append('+',         0, 2, self.binary)
        table.append('-',         0, 2, self.binary)

        table.append('*',         1, 2, self.binary)
        table.append('/',         1, 2, self.binary)

        table.append('**',        2, 2, self.binary)

        table.append('-',         3, 1, self.unary_minus)
        table.append('+',         3, 1, self.unary_plus)
        table.append('type',      3, 1, self.unary_type)
#        table.append('integer',   3, 1, self.unary)
#        table.append('real',      3, 1, self.unary)
#        table.append('character', 3, 1, self.unary)

        parsetools.Expr_Parser.__init__(self, table)

    def unary_plus(self, k):
        tt = self.token.type
        self.next_token()
        right = self.expr(k + self.table.associativity[k])
        return right

    def unary_minus(self, k):
        tt = self.token.type
        self.next_token()
        right = self.expr(k + self.table.associativity[k])
        node = AST(tt)
        node.expr_str = '%s%s' % (tt, right.expr_str)
        node.expr_py = '- %s' % right.expr_py
        node.expr_type = right.expr_type
        node.append(right)
        return node

    def unary_type(self, k):
        tt = self.token.attr
        self.next_token()
        right = self.expr(k + self.table.associativity[k])
        node = AST(tt)
        node.expr_str = '%s %s' % (tt, right.expr_str)
        if tt == 'integer':
            node.expr_str = 'int(%s)' % right.expr_py
            node.expr_py = 'int(%s)' % right.expr_py
            node.expr_type = type_int
        elif tt == 'real':
            node.expr_str = 'real(%s)' % right.expr_py
            node.expr_py = 'float(%s)' % right.expr_py
            node.expr_type = type_real
        elif tt == 'character':
            # use repr to get the quotes
            node.expr_py = 'repr(%s)' % right.expr_py
            node.expr_type = type_char
        else:
            self.croak("unexpected token: "+ tt)
        node.append(right)
        return node

    def binary(self, k, left):
        node = AST(self.token.type)
        self.next_token()
        right = self.expr(k + self.table.associativity[k])
        node.append(left)
        node.append(right)
        node.expr_str = '%s%s%s' % (left.expr_str, node.type, right.expr_str)
        node.expr_py = '%s %s %s' % (left.expr_py, node.type, right.expr_py)
        node.expr_type = max(left.expr_type, right.expr_type)
        return node

    def factor(self):
##        print "factor:", self.token
        tt = self.token.type
        if tt == '(':
            self.next_token()
            node = self.expr(0)
            node.expr_str = '(%s)' % node.expr_str
            node.expr_py = '(%s)' % node.expr_py
            self.demand(')')
        elif tt == 'single_quoted':
            node = AST(self.token.type)
            node.attr = self.token.attr
            node.expr_str = node.attr
            node.expr_py = node.expr_str
            node.expr_type = type_char
        elif tt == 'double_quoted':
            node = AST(self.token.type)
            node.attr = self.token.attr
            node.expr_str = node.attr
            node.expr_py = node.expr_str
            node.expr_type = type_char
        elif tt == 'number':
            node = AST(self.token.type)
            node.attr = self.token.attr
            node.expr_str = node.attr
            node.expr_py = node.expr_str
            node.expr_type = type_int
        elif tt == 'float':
            node = AST(self.token.type)
            node.attr = self.token.attr
            node.expr_str = node.attr
            node.expr_py = node.expr_str
            node.expr_type = type_real
        elif tt == 'id':
            node = AST(self.token.type)
            node.attr = self.token.attr
            node.expr_str = node.attr
            node.expr_py = node.expr_str
            try:
                node.expr_type = symbol_types[node.attr]
            except KeyError:
                node.expr_type = type_var
        else:
            self.croak("unexpected token: "+ tt)
    
        self.next_token()
        return node

########################################
#
#
class DimsParser(parsetools.Parser, Expr_Parser):
    '''Parse dimension and type specifiactions.

    After encountering a dimension expression while parsing the VDF file,
    save all of the tokens, then actually parse them when we know what
    we are looking for.

    p = DimsParser()
    while 1:
      p.append()

    p.start()
    p.p_dimensions()   or p.p_arguments()
    '''

    def __init__(self):
        parsetools.Parser.__init__(self)
        Expr_Parser.__init__(self)
        self.location = ('', 0, 0)

    def __len__(self):
        return len(self.tokens)

    def start(self):
        self.next_token()

        # XXX - inherit?
    def next_token(self):
        '''Return the next token'''
        try:
            token = self.tokens.pop()
        except IndexError:
            token = Token('EOF')
        self.token = token
        return token.type

    def set_position(self, location):
        self.location = location

    def position(self):
        return self.location

    def p_dimensions(self):
        '''Parse a dimension string.
        
        <array_spec>          ::= <explict_shape_spec_list> |
                                  <assumed_size_spec>
        <explicit_shape_spec> ::= [<lower_bound> : ] <upper_bound>
        <assumed_size_spec>   ::= [<explicit_shape_spec_list> ,]
                                    [<lower_bound> :] *
        '''

        have = self.have
        mustbe = self.mustbe

        mustbe('(')
        
        shape = FortranArraySpec()
        
        while 1:
            lower = None
            if have('*'):
                upper = FortranArrayBound('*')
            else:
                upper = FortranArrayBound(self.parse_expr().expr_str)

            if have(':'):
                lower = upper
                if have('*'):
                    upper = FortranArrayBound('*')
                else:
                    upper = FortranArrayBound(self.parse_expr().expr_str)

            attr = FortranArrayShape(lower, upper)
            shape.append(attr)
            if not have(','):
                break
        mustbe(')')
        return shape

    def p_arguments(self):
        '''Parse an argument list.

        <arg> ::=  id [: [&|*] ( <type> | <utype> | string ) ]
        '''

        have = self.have
        mustbe = self.mustbe

        mustbe('(')

        arg_str = ['(']

        args = ArgumentsNode()
        if have(';'):
            args.optional = 1
            arg_str.append(';')
        
        while 1:
            if have(')'):
                arg_str.append(')')
                break

            # Any id is ok here, even 'keywords' or 'types'
            if self.token.is_id:
                dummy_arg = self.token.attr
                arg_str.append(dummy_arg)
                self.next_token()
            else:
                self.croak(("syntax error, expecting id' "
                            "(actually saw %s=%s)") %
                           (self.token.type, self.token.attr))
#            dummy_arg = mustbe('id')
            addr = 0

            if have(':'):
                arg_str.append(':')
                if have('&'):
                    addr = '&'
                    arg_str.append('&')
                elif have('*'):
                    addr = '*'
                    arg_str.append('*')

                # XXX-XXX 
                if have('id'):
                    # usually string, address, range but could be anything
                    ftype = FortranType(self.token_attr)
                elif have('user_type'):
                    # XXX-YYY
                    ftype = self.p_user_type().attrs.var_type
                elif mustbe('type'):
                    ftype = self.p_type().attrs.var_type
                arg_str.append(ftype.vdf_type)
            else:
                if dummy_arg[0] in 'ijklmnIJKLNM':
                    ftype = int_node
                    addr = '&'
                else:
                    ftype = real_node
                    addr = '&'

            attr = ArgumentNode(dummy_arg, ftype, addr)
            args.append(attr)
            if have(','):
                attr.separator = ','
                arg_str.append(',')
                continue
            if have(';'):
                attr.separator = ';'
                arg_str.append(';')
                continue

###            break
        args.args_str = ''.join(arg_str)
        return args

    def p_user_type(self):
        '''Extract the fortran type out of the user type and create
        a FortranTypeSpec.'''

	typespec = self.token_attr # recognized token
        if typespec[0] == '_':
            basetype = typespec[1:]
            dynamic = 1
        else:
            basetype = typespec
            # None indicates that the value is not used, 0 is different
            dynamic = None

        var_type = user_types[basetype]
        return FortranTypeSpec(var_type=var_type, pointer=dynamic)

    def p_type(self):
        '''Parse a type.

        We have seen a word at the beginning of a type, and need to pick up
        any qualifiers.
        '''
	have = self.have
        mustbe = self.mustbe

	typespec = self.token_attr # recognized token
        if typespec[0] == '_':
            basetype = typespec[1:]
            dynamic = 1
        else:
            basetype = typespec
            # None indicates that the value is not used, 0 is different
            dynamic = None

        if basetype == 'Address':
            ftype = type_table[basetype]

        elif basetype == 'Filedes':
            ftype = type_table[basetype]

        elif basetype == 'longint':
            ftype = type_table[basetype]
            
        elif basetype == 'logical':
            ftype = type_table[basetype]

        elif basetype == 'Ch':
            if have('('):
                length = self.parse_expr().expr_str
                mustbe(')')
                ftype = FortranType(basetype, length=length, ctype='char',
                                    character=1)
                ftype.decl = 'character*(%s)' % length
                ftype.vdf_type = 'Ch(%s)' % length
                ftype.mppl_type = 'Ch(%s)' % length
            else:
                ftype = type_table[basetype]

        elif basetype == 'double':
            # note: 'precision' is a keyword, yet 'complex' is a type
            if have('precision'):
                ftype = type_table['double precision']
            elif have('type', 'complex'):
                ftype = type_table['double complex']
            else:
                ftype = type_table['double']

        # for complex, integer and real
        elif basetype == 'character':
            if have('*'):
                # must be a * expression, so evaluate
                if have('('):
                    length = self.parse_expr().expr_str
                    decl = 'character*(%s)' % length
                    self.mustbe(')')
                elif have('id'):
                    length = self.token_attr
                    decl = 'character*' + length
                elif mustbe('number'):
                    length = self.token_attr
                    decl = 'character*' + length
                ftype = FortranType('character', length=length,
                                    ctype='char', character=1)
                # This is set explicitly to preserve the difference between
                # character*8 and character*(8)
                ftype.decl = decl
                ftype.vdf_type = decl
                ftype.mppl_type = decl
            else:
                ftype = type_table['character']

        # for complex, integer, real, and logical
        elif have('*'):
            length = mustbe('number')
            decl = '%s*%s' % (basetype, length)
            try:
                ftype = type_table[decl]
            except KeyError:
                # XXX- stuff like real*3 is not really useful but will work
                ftype = FortranType(basetype, length=length)

        elif have('('):
            # We must assume that we are building a dimension.
            # If the upper limit of the first element is 'magic', then
            # it is really a kind specification.
            # It makes more sense to have dimensions specified close to the name,
            # but in the interest of compatibility with older .v files, ....

            # Peek ahead at the next token, if it is  <id>=Size?, then we are
            # dealing with a kind specifier

            if self.token.type == 'id' and \
               self.token.attr in ('Size2', 'Size4', 'Size8', 'Size16'):
                # recognize things like: real(Size2), integer(Size8)
                kind = self.token.attr
                self.next_token()
                mustbe(')')
                desc = '%s(%s)' % (basetype, kind)
                ftype = type_table[desc]
            else:
                kind = None
                self.push_token(self.token)   # push look ahead
                self.token = Token('(')       # reset current token
                ftype = type_table[basetype]

        # basic type - real, integer, ...
        else:
            ftype = type_table[basetype]

        return FortranTypeSpec(var_type=ftype, pointer=dynamic)

    def p_builtin_args(self):
        '''Parse builtin argument list.

        Actually, no parsing is done, just concatenate the arguments.
        '''
        # ones(n [,m,...])        builtin [1-7] 
        # inf(x,[y,...])  builtin [1-ARBITRARY]
        # shape(expr, [n1, ...,]) builtin [1-8]

        out = []

        tt = self.token.type
	while tt != 'EOF':
            if tt == 'id':
                out.append(self.token.attr)
            elif tt == 'number':
                out.append(self.token.attr)
            elif tt == 'mppl_macro':
                # remove imbedded spaces
                out.append(self.token.attr.replace(' ', ''))
            elif tt == 'user_type':
                out.append(self.token.attr)
            elif tt == 'type':
                out.append(self.token.attr)
            else:
                out.append(tt)
            tt = self.next_token()

        return ''.join(out)

########################################
#
#
class VDF(DimsParser):
    '''
    xyz-list  means xyz [, xyz]
    
    The grammar we are parsing is:
    <start>        ::= <package> <parameters> [<group>]*
    <comments>     ::= [<comment> | <pcomment> | <literaltext>]*
    <package>      ::= <comments> <id> [<comments> | <usertype> ]*
    <usertype>     ::= usertype id             |
                       usertype id id          |
                       usertype id <type>      |
                       usertype id <utype>     |
                       usertype id - character
    <parameters>   ::= [{] [<statement> | <comments>]* [}]
    <statement>    ::= <identifer> = <expression> [,] |
                       define <id> <macro>
    <group>        ::= <group_id> <id> [<g_attribute>]* : <comments>
                       [<variable> <comments>]*
    <group_id>     ::= newgroup [Group] | Group
    <g_attribute>  ::= <id>                   |
                       local                  |
                       hidden                 |
                       / <id> /               |
                       compileas   ( <spec> ) |
                       language STRING
    <variable>     ::= <id> [<v_attributes>]* <comments>
    <v_attribute>  ::= + <id>                 |
                       - <id>                 |
                       ( <array_spec_list> )  |
                       [ <units> ]            |
                       / <init_value_list> /  |
                       compileas ( <spec> )   |
                       limited   ( <spec> )   |
                       STRING                 |
                       <utype>                 |
                       <type>

    <init_value>   :: = [ number *]
                        [id | quoted_string | ([+|-] number | float)

    <type>         ::= <type1> [ ( <spec> ) ]
                       <type2>
                       <type3> [* <spec> | ( <spec> ) ]
    <type1>        ::= Ch | _Ch
    <type2>        ::= (double | _double) [ complex | precision ]
    <type3>        ::= character | _character |
                       integer   | _integer   |
                       real      | _real      |
                       logical   | _logical

    <array_spec>   ::= [;] [ <expr> : ] <expr>
       upper_bound    ::= <expr>
       lower_bound    ::= <expr> : <expr>
       assumed_length ::= *
       arg_spec       ::= <id> : var_type
       arg_addr_spec  ::= <id> : & ( <id> | <var_type> | <utype> )
    
    Assume the correct token before entering procedure
    '''

    def __init__(self, input):
        DimsParser.__init__(self)
        self.input = input

    def next_token(self):
        '''Return the next token'''
        if self.tokens:
            token = self.tokens.pop()
        else:
            token = self.input.next_token()
            if token is None:
                token = Token('EOF')
        self.token = token
        return token.type

    def position(self):
        return self.input.position()

    def start(self):
        self.next_token()
        return self.p_start()

    def p_start(self):
        have = self.have

        # discard leading comments
        while 1:
            if have('comment'):
                pass
            elif have('pcomment'):
                pass
            elif have('literaltext'):
                pass  # XXX-XXX warn about ignoring leading literal text
            else:
                break

        package = self.p_package()
        self.p_parameters(package)
        while self.token.type != 'EOF':
            self.p_group(package)
        return package

    def p_package(self):
        name = self.mustbe('id')
        node = BasisPackage(name)
        node.type_table = type_table
        node.userorder = []
        have = self.have
        while 1:
            if have('comment'):
                node.comments.append(self.token_attr)
            elif have('pcomment'):
                pass
            else:
                break
        return node

    def p_usertype(self, package):
        name = self.mustbe('id')

        if self.have('id'):
            # This is similar to a macro definition; 'utypename' will be
            # treated like a legal FORTRAN type name, but replaced by 'id'
            node = FortranType(name, vdf_type=self.token_attr)
            node.define = 1
        elif self.have('user_type'):
            # usertype foo mychar
            uname = self.token_attr
            node = FortranType(name, decl=uname)
            node.define = 1
            # lookup the user type and copy declaration.
            unode = user_types[uname]
            node.decl = unode.decl
            node.character = unode.character
        elif self.have('type'):
            # usertype mychar character*45
            type_spec = self.p_type()
            var_type = type_spec.attrs.var_type
#            if type_spec.dynamic():
#                uname = '_' + var_type.name
#            else:
#                uname = var_type.name
            node = FortranType(name, var_type.kind, var_type.length,
                               ctype=var_type.ctype,
                               decl=var_type.decl,
                               vdf_type=name)
#            node.vdf_type = name # XXX-YYY must copy!
            node.define = 1
        elif self.have('-'):
            # This type of definition is used to specify that 'utypename'
            # is to be used as if it were a legal FORTRAN character type
            # (but otherwise unspecified)
            ta = self.mustbe('type', 'character')
            node = FortranType(name, ctype='char', character=1)
            node.define = 0
        else:
            # Allows the identifier 'utypename' to be used as if it were a
            # legal FORTRAN type.
            node = FortranType(name)
            node.define = 0

        _name = '_' + node.name

        user_types[name]  = node
        user_types[_name] = node
        package.userorder.append(node)
##XXX-XXX        package.userorder.append(_node)

    def p_parameters(self, package):
        parameters = package.parameters
        curley = 0

        have = self.have
        mustbe = self.mustbe
        right = None
        
        while 1:
            if have('comment'):
                # XXX-XXX add to the package
                pass
            elif have('pcomment'):
                pass
            elif have('literaltext'):
                node = LiteralText(self.token_attr)
                parameters.append(node)
            elif self.have('usertype'):
                self.p_usertype(package)
            elif have('{'):
                curley += 1
            elif have('}'):
                curley += 1
            elif have('define'):
                mustbe('id')
                name = self.token_attr
                if have('mppl_macro'):
                    node = Define(name)
                    node.expr_str = self.token_attr
                    node.value = self.token_attr
                else:
                    node = Define(name)
                    right = self.parse_expr()
                parameters.append(node)
            elif have('id'):
                name = self.token_attr
                node = Parameter(name)
                mustbe('=')
                right = self.parse_expr()
                parameters.append(node)
                have(',')  # optional comma
            else:
                break

            # evaluate the right hand expression
            if right:
                node.append(right)
                node.expr_str = right.expr_str
                node.expr_type = right.expr_type
                symbol_types[name] = right.expr_type
                try:
                    value = eval(right.expr_py, symbol_table)
                    symbol_table[name] = value
                    # we must provide our own 'repr' since it will
                    # use single quotes for strings and we need double
                    # (To define C strings)
#                    node.value = repr(value)
                    if type(value) == type(''):
                        node.value = '"%s"' % value
                    else:
                        node.value = value
                except NameError, msg:
                    # If a name in the expression is undefined, just use the
                    # unexpanded string
                    node.value = right.expr_str
                node.expr_str = right.expr_str
                right = None
        # XXX-XXX error check on {}'s
        return parameters
    
    def p_group(self, package):
        self.p_group_id()
        name = self.mustbe('id')
        group = GroupNode(name)
        package.add_group(group)
        have = self.have
        mustbe = self.mustbe

        tt = self.token.type
        while tt != ':':
            tt = self.token.type
            self.token_attr = self.token.attr  # XXX - remove with have()
            if tt == 'id':
                group.attrs.append(self.token.attr)
                self.next_token()
            elif tt == 'local':
                self.next_token()
                group.local = 1
                group.attrs.append('local')
            elif tt == 'hidden':
                self.next_token()
                group.hidden = 1
                group.attrs.append('hidden')
            elif tt == '/':
                self.next_token()
                if have('/'):
                    group.common = ' '  # blank common
                else:
                    group.common = mustbe('id')
                    mustbe('/')
            elif tt == 'compileas':
                self.next_token()
                # group.compileas = 
            elif tt == 'limited':
                self.next_token()
                # group.limited = 
            elif tt == 'language':
                self.next_token()
                if have('single_quoted'):
                    attr = self.token_attr[1:-1]
                else:
                    attr = mustbe('double_quoted')[1:-1]
                group.language = attr
            elif tt == 'user_type':
                group.ftype = self.p_user_type()
                self.next_token()
                if debug:
                    print "Group %s being typed as %s." % (
                        group.name, group.ftype.decl)
            elif tt == 'type':
                self.next_token()
                group.ftype = self.p_type()
                if debug:
                    decl = group.ftype.attrs.var_type.decl
                    print "Group %s being typed as %s." % (
                        group.name, decl)
            else:
                break
        mustbe(':')

        # trailing comments and literals are associated with this group
        while 1:
            if have('comment'):
                # change double quotes in comments to singles, since comments
                # are entered into the database as quoted strings.
                comment = self.token_attr
                comment = comment.replace('"', "'")
                comment = comment.replace('\t', ' ')
                group.comments.append(comment)
            elif have('pcomment'):
                pass
            elif have('literaltext'):
                group.append(LiteralText(self.token_attr))
            else:
                break

        while self.token.type == 'id':
            self.p_variable(group)
    
    def p_group_id(self):
        if self.have('newgroup'):
            self.have('Group')
        else:
            self.mustbe('Group')
    
    def p_variable(self, group):
        '''Parse an identifier.

        It is a variable by default, until we see a subroutine/function/
        builtin/command keyword. So we collect information about it in a
        dictionary, then at the end decide what it is we are actually building.
        Part of the information we collect is the dimension tokens. We then
        reparse the tokens when we know that we are looking for.
        '''
        # This is all caused by the grammar not being LL(1)
        # We actually need lots of look ahead.
        # 
        name = self.mustbe('id')
        if debug:
            print "creating name entry for '%s'." % name
#        variable = VariableNode(name)
        have = self.have
        mustbe = self.mustbe

        object = 'variable'
        type_spec = None
        plus_attrs = []
        minus_attrs = []
        comments = []
        options = {}
        dparser = DimsParser()
        dparser.set_position(self.input.position())

        while 1:
            tt = self.token.type
            self.token_attr = self.token.attr  # XXX - remove with have()

            if tt == '+':
                self.next_token()
                attr = mustbe('id')
                plus_attrs.append(attr)
            elif tt == '-':
                self.next_token()
                attr = mustbe('id')
                minus_attrs.append(attr)
            elif tt == 'mppl_macro':
                options['units'] = self.token.attr
                self.next_token()
            elif tt == '/':
                self.next_token()
                options['init'] = self.p_init_value()
                mustbe('/')
            elif tt == '(':
                balance = 0
                while 1:
                    if tt ==  '(':
                        balance += 1
                    if tt ==  ')':
                        balance -= 1
                    dparser.unshift_token(self.token)
                    if balance == 0:
                        break
                    tt = self.next_token()
                self.next_token()
            elif tt == 'compileas':
                self.next_token()
                options['compileas'] = self.p_dimensions()
            elif tt == 'limited':
                self.next_token()
                options['limited'] = self.p_dimensions()
            elif tt == 'single_quoted':
                options['alias'] = self.token.attr[1:-1]
                self.next_token()
            elif tt == 'double_quoted':
                options['alias'] = self.token.attr[1:-1]
                self.next_token()
            elif tt == 'user_type':
                type_spec = self.p_user_type()
                self.next_token()
                if debug:
                    decl = type_spec.attrs.var_type.decl
                    print "Gave usertype %s to variable %s." % (decl, name)
            elif tt == 'type':
                self.next_token()
                type_spec = self.p_type()
                if debug:
                    decl = type_spec.attrs.var_type.decl
                    print "Variable %s being typed as %s." % (name, decl)
#                    if dims:
#                        print ("Just added dimensions %s to variable '%s'." %
#                               (dims, name))

            elif tt == 'builtin':
                object = tt
                self.next_token()
            elif tt in ('subroutine', 'function',
                        'csubroutine', 'cfunction'):
                object = tt
                options['object'] = object
                self.next_token()
            elif tt == 'comment':
                # trailing comments are associated with this variable
                # change double quotes in comments to singles, since comments
                # are entered into the database as quoted strings.
                comment = self.token_attr
                comment = comment.replace('"', "'")
                comment = comment.replace('\t', ' ')
                comments.append(comment)
                self.next_token()
            elif tt == 'pcomment':
                self.next_token()
            else:
                break

        dparser.start()

        if object == 'variable':
            if type_spec:
                pass # explicitly set
            elif group.ftype:
#                type_spec = copy.deepcopy(group.ftype)
                type_spec = group.ftype
            elif name[0] in 'ijklmnIJKLNM':
                type_spec = FortranTypeSpec(var_type=int_node)
            else:
                type_spec = FortranTypeSpec(var_type=real_node)

            variable = VDF_Variable(name, type_spec=type_spec)
            #variable = VDF_Variable(name, dim_spec, init, type_spec)
            #            variable.ftype = type_spec  # XXX-YYY

            if dparser:
                dim_spec = dparser.p_dimensions()
                options['dims'] = dim_spec
                # XXX-YYY
                options['dim_str'] = dim_spec.explicit_shape()
                # dimensions can live in the type_spec or the variable
                # here, we choose variable
#                type_spec.attrs.dimension = dim_spec
                variable.dim_spec = dim_spec

        elif object == 'builtin':
            #"mac: You did not specify number of parameters for $names[$i]." ;
            variable = VDF_BuiltinNode(name)

            if dparser:
                options['args_str'] = dparser.p_builtin_args()
            else:
                options['args_str'] = '()'

        elif object == 'command':
            variable = VDF_CommandNode()

        else:
            variable = VDF_ExternalNode(name)
            if type_spec:
                t = type_spec.attrs.var_type
            elif name[0] in 'ijklmnIJKLNM':
                type_spec = FortranTypeSpec(var_type=int_node)
            else:
                type_spec = FortranTypeSpec(var_type=real_node)
            variable.type_spec = type_spec

            if dparser:
                args = dparser.p_arguments()
            else:
                args = ArgumentsNode()
            options['args'] = args

        variable.__dict__.update(options)
        variable.plus_attrs = plus_attrs
        variable.minus_attrs = minus_attrs
        variable.comments = comments
        group.add_variable(variable)
            
        while 1:
            if have('literaltext'):
                group.append(LiteralText(self.token_attr))
            else:
                break

    def p_init_value(self):
        """slurp everything till the ending /

        parse [ (number|id [ ( expr )) *]
              [id | quoted_string | ([+|-] number | float)
        """
        tokens = []
        have = self.have
        # true if we have a possible count (number or id)
        have_count = 0

        while 1:
            if self.token.type == '/':
                break
            if self.token.attr:
                tokens.append(self.token.attr)
            else:
                tokens.append(self.token.type)
            self.next_token()

#        while 1:
#            have_sign = 0
#            if have('id'):
#                tokens.append(self.token_attr)
#                have_count += 1
#            elif have('single_quoted'):
#                tokens.append(self.token_attr)
#            elif have('double_quoted'):
#                tokens.append(self.token_attr)
#            else:
#                if have ('+'):
#                    have_sign = 1
#                    tokens.append('+')
#                elif have ('-'):
#                    have_sign = 1
#                    tokens.append('-')
#
#                if have('number'):
#                    if have_sign == 0:
#                        have_count += 1
#                    tokens.append(self.token_attr)
#                elif have('float'):
#                    tokens.append(self.token_attr)
#
#            # XXX-XXX perhaps improve error checking here
#            if have_count == 1:
#                if have('*'):
#                    tokens.append('*')
#                    # we have the [number *] part, loop back and get the value
#                    continue
#                
#            if have(','):
#                tokens.append(',')
#                have_count = 0
#                continue
#                
#            break;
        return join(tokens, '')

#    def build_dimensions(self):
#        # having seen a left parenthesis/bracket, we now build the rest of the
#        # declaration, keeping parentheses/brackets balanced.
#        if debug:
#            print "----> Entering build_dimensions <----"
#   $token_number == $LPAR && ( $pdepth = 1 ) ||
#      $token_number == $LBKT && ( $bdepth = 1 ) ;
#   while ( $bdepth ) { # inside brackets, anything goes
#       # -- but, in the interest of sanity, I want it complete on one line.
#       $current_line =~ s/^([^\[\]]*)([\[\]])//o ;
#       die "mac: brackets out of balance: $bdepth now open.\n" .
#           "     quantities in brackets should be complete on one line."
#           if ! $2 ;
#       $txt .= $1 . $2 ;
#       $bdepth += ( $2 eq "[" ) ? 1 : -1 ;
#      }
#   while ( $pdepth ) {
#       &get_nonblank_token ( "dimension/parameter/type size specification" , 0 ) ;
#       &debug ;
#       $bdepth += ( $token_number == $LBKT ? 1 : -1 ) if ( $token_number == $LBKT ||
#                                                           $token_number == $RBKT ) ;
#       $token_text = "" if ( $token_number == $LBKT && $bdepth == 1 ||
#                             $token_number == $RBKT && $bdepth == 0 ) ;
##XXX       # expand any macros in dimension string
#       $token_text = $value{$token_text}
#           if ( !$defining_group && $token_number == $NAME &&
#                !$bdepth && $defined{$token_text} ) ;
#       $txt .= $token_text ;
#       $token_number == $LPAR && $pdepth ++ ||
#           $token_number == $RPAR && $pdepth -- ;
#      }
#$debug && print STDERR +
#      ">----Just built '$txt'.----<\n" ;
#   return $txt ;


########################################
#
#
def read_vdf(fp, filename):
    scanner = vdflex(fp, filename)

    p = VDF(scanner)
    ast = p.start()

    if debug:
        dump_vdf_nodes(ast)

    return ast

########################################
#
#
if __name__ == '__main__':
    import parsedebug

#    filename = "example7.in"
    filename = "testaa"
    filename = "testbb"
    ast = read_vdf(filename)

#    parsedebug.PrintTree1(ast, ['name', 'attr'])

    dump_vdf_nodes(ast)

# XXX -     f90.print_fortran_code(ast)


