#!/bin/env python

import string

import exceptions
#import sys

########################################
#
#
class Token:

    def __init__(self, type, attr=None):
	self.type = type
	self.attr = attr
#	if self.debug > 1:
#	    print "CML token: ", `self`
    #  __cmp__		required for GenericParser, required for
    #			GenericASTMatcher only if your ASTs are
    #			heterogeneous (i.e., AST nodes and tokens)
    #  __repr__		recommended for nice error messages in GenericParser
    #  __getitem__	only if you have heterogeneous ASTs
    def __repr__(self):
	    if self.type == "EOF":
		return "EOF"
	    elif self.attr != None:
		return self.type + "=" + `self.attr`
            else:
                return self.type
    def __cmp__(self, o):
        return cmp(self.type, o)
    def __getitem__(self, i):
        raise IndexError

########################################
#
#  Minimal AST class.
#
class AST:
    def __init__(self, type):
        self.type = type
        self._kids = []

    #
    #  Not all these may be needed, depending on which classes you use:
    #
    #  __getitem__		GenericASTTraversal, GenericASTMatcher
    #  __len__		GenericASTBuilder
    #  __setslice__		GenericASTBuilder
    #  __cmp__		GenericASTMatcher
    #
    def __getitem__(self, i):
        return self._kids[i]
#   def __len__(self):
#	return len(self._kids)
    def __setslice__(self, low, high, seq):
        self._kids[low:high] = seq
    def __cmp__(self, o):
        return cmp(self.type, o)
    def __str__(self):
        return self.type

## Is causes endless problems when I do  'if node:'
## because it evaluates to 0, if len is 0, but the node exits.
##
##    def __len__(self):
##        return len(self._kids)

    def __delitem__(self, i):
        del self._kids[i]

    def __getslice__(self, i, j):
        return self._kids[i:j]

    def error(self, msg):
        raise RuntimeError, msg

    def append(self, node):
        # add a single child to the node
        self._kids.append(node)

    def insert(self, index, node):
        self._kids.insert(index, node)

    def add_kids(self, nodes):
        for node in nodes:
            self._kids.append(node)

    def set_kids(self, kids):
        self._kids = kids

    def assignkids(self, node):
        # XXX - Implement behavior similar to Parser.nonterminal
        # "Skip allocation a new node if there's only one child"
        #if (len(node._kids) == 1):
        #    self._kids = node._kids[0]
        #else:
        #    self._kids = node._kids
        self._kids = node._kids

    def assignkid(self, node):
        # replace kids with a new node
        self._kids = [node]

    def getChildren(self):
        return self._kids

########################################
#
#
from UserDict import UserDict

class AttrDict(UserDict):
    """Access a dictionary using setattr and getattr."""

    def __init__(self, dict={}):
        UserDict.__init__(self)
        self.update(dict)

    def __setattr__(self, name, value):
        if name == 'data':
            self.__dict__[name] = value
        else:
            self.data[name] = value

    def __getattr__(self, name):
        if self.data.has_key(name):
            return self.data[name]
        return 0

    def decl(self, container, handlers, order=None):
	"""format the attributes for a fortran declaration list. """
        if order is None:
            order = self.handlers.keys()
	list = []
	for attr in order:
	    value = self.data.get(attr)
	    if value is not None:
                handler = handlers[attr]
                if handler is None:
		    list.append(attr)
                else:
                    result = handler(container, attr, value)
                    if result is not None:
                        list.append(result)
	return string.join(list, ', ')

########################################
#
#
class Lexer:
    def decimal(self, value):
        while 1:
            value += peek
            peek = self.next_char()
            if peek not in string.digits:
                break
        return value

    def exponent(self, value):
        if peek in ['e', 'E', 'd', 'D']:
            reset = self.column - 1
            exp = peek
            peek = self.next_char()
            if peek in ['+', '-']:
                exp += peek
                peek = self.next_char()
            if peek in string.digits:
                while 1:
                    exp += peek
                    peek = self.next_char()
                    if peek not in string.digits:
                        break
                    value += exp
                    tok = 'float'
            else:
                tok = 'int'
                self.column = reset
                self.next_char()

        
########################################
#
#
class ParserError(exceptions.Exception):

    def __init__(self, args=None):
        self.args = args

########################################
#
#
class Parser:
    """Recursive Descent Parser.
    """

    def __init__(self):
        self.tokens = []
        self.recovering = 0

    def start(self):
        self.next_token()

    def push_token(self, token):
        '''Add a token to the end of the pushback list (first off).'''
        self.tokens.append(token)

    def unshift_token(self, token):
        '''Add a token to the front of the pushback list (last off).'''
        self.tokens.insert(0, token)

    def tokens_str(self):
        '''Return the token stack as a string.'''

        list = []
        for token in self.tokens:
            if token.attr:
                list.append(token.attr)
            else:
                list.append(token.type)
        if self.token.attr:
            list.append(self.token.attr)
        else:
            list.append(self.token.type)
        list.reverse()
        return ''.join(list)

    def next_token(self):
        '''Return the next token.'''
        try:
            token = self.tokens.pop()
        except IndexError:
            token = Token('EOF')
        self.token = token
        return token.type

    def consume(self):
        '''Skip to next token, when the current one is used.'''
        self.token_attr = self.token.attr
        self.prev_token = self.token
        self.next_token()

    def demand(self, type, attr=None):
	'''Require a given token or token type, croak if we dont get it.'''
        if self.recovering:
            if attr is None:
                while self.token.type != type:
                    self.next_token()
            else:
                while self.token.type != type or self.token.attr != attr:
                    self.next_token()
            self.recovering = 0

        token = self.token
        if token.type == "EOF":
            self.croak("premature EOF")
        elif attr is not None and token.attr != attr:
            self.croak("syntax error, saw `%s' while expecting `%s'" %
                       (token, attr))
        elif token.type != type:
            if token.attr:
                self.croak(("syntax error, expecting token of type `%s' "
                            "(actually saw %s=%s)") %
                           (type, token.type, token.attr))
            else:
                self.croak(("syntax error, expecting token of type `%s' "
                            "(actually saw %s)") % (type, token.type))
        else:
            return token.attr

    def mustbe(self, type, attr=None):
        '''calls demands, then consumes the token'''
        attr = self.demand(type, attr)
        self.token_attr = self.token.attr
        self.prev_token = self.token
        self.next_token()
        return attr
    
    def have(self, type, attr=None):
        if attr is None and self.token.type == type:
            self.token_attr = self.token.attr
            self.prev_token = self.token
            self.next_token()
            return 1
        elif self.token.attr == attr and self.token.type == type:
            self.token_attr = self.token.attr
            self.prev_token = self.token
            self.next_token()
            return 1
        else:
            return 0
        
    def havelist(self, *types):
        for type in types:
            if self.token.type == type:
                self.token_attr = self.token.attr
                self.prev_token = self.token
                self.next_token()
                return 1
        return 0
        
    def have_attrlist(self, type, attrs):
        if self.token.type == type and self.token.attr in attrs:
            self.token_attr = self.token.attr
            self.prev_token = self.token
            self.next_token()
            return 1
        else:
            return 0
        
    def error_leader(self, infile=None, lineno=None):
        "Emit a C-compiler-like, Emacs-friendly error-message leader."
        location = self.position()
        if not infile:
            infile = location[0]
        if not lineno:
            lineno = location[1]
        return "\"%s\", line %d: " % (infile, lineno)

    def complain(self, str, lineno=None):
	# Report non-fatal parse error; format like C compiler message.
	raise ParserError(self.error_leader(lineno) + str)
#	raise ParserError(self.error_leader(lineno) + str)
#	sys.stderr.write(self.error_leader(lineno) + str + "\n")
	errors = 1

    def croak(self, str, lineno=None):
	# Report a fatal parse error and die
        if not self.recovering:
            self.recovering = 1
            self.complain(str, lineno)
#            raise RuntimeError, 'development stop'  #### XXX-XXX
#	sys.exit(1)

    def position(self):
        '''Return the (filename, line number, column number)'''
        return ('', 0, 0)
    
########################################
#
#
class Expr_Node:

    def __init__(self, token, level, noperands, func=None):
        self.token = token
        self.level = level
        self.noperands = noperands
        self.func = func
        self.assoc = 0

########################################
#
#
class Expr_Table:

    RIGHT = 0
    LEFT = 1
    def __init__(self, *associativity):
        self.nlevels = len(associativity)
        self.associativity = associativity
        self.precedence = []
        for node in associativity:
            self.precedence.append([])

    def append(self, token, level, noperands, func=None):
        node = Expr_Node(token, level, noperands, func)
        self.precedence[level].append(node)
        node.assoc = self.associativity[level]

########################################
#
#
class Expr_Parser:
    """Compact Recursive-descent Parsing of Expressions.

    From Software-Practice and Experience, vol 15, p1205-1212 (1985)
    David R. Hanson

    A simple technique for parsing expressions using recursive descent that
    avoids the usual proliferation of procedures that occurs when recursive
    descent is used to parse expressions.  By taking advantage of the
    similarity of the productions describing expressions in most languages,
    the n+1 procedures usually required to parse expressions with n precedence
    levels can be replaced with a table and two procedures.
    """

    def __init__(self, table):
        self.table = table

    def parse_expr(self):
        return self.expr(0)

    def expr(self, k):
        if k == self.table.nlevels:
            node = self.factor()
        else:
            # prefix
            op = self.isop(k, 1)
            if op:
                node = op.func(k)
            else:
                node = self.expr(k+1)

            # infix
            op = self.isop(k, 2)
            while op:
                node = op.func(k, node)
                op = self.isop(k, 2)

            # postfix
        return node
            
    def isop(self, k, n):
        for op in self.table.precedence[k]:
            if self.token == op.token and n == op.noperands:
                return op
        return None

    def factor(self):
        self.next_token()
        return None

########################################
#
#
#
class Dump_Mixin:

    def __init__(self):
        self.max_line = 80
        self.current_len = 0
#        import cStringIO
#        self.fp = cStringIO.StringIO

    
    def set_indent(self, num):
        self.icount = num
        self.indent = ' ' * num

    def doindent(self):
        self.icount = self.icount + 4
        self.indent = ' ' * self.icount

    def deindent(self):
        self.icount = self.icount - 4
        self.indent = ' ' * self.icount

    def continue_line(self, out):
        return out

    def writeto(self, line):
        '''Write a line of output.'''

        if type(line) is type(()):
            for s in line:
                out = '%s%s' % (self.indent, s)
                if self.current_len > self.max_line:
                    out = self.continue_line(out)
        else:
            out = '%s%s' % (self.indent, line)
            if len(out) > self.max_line:
                out = self.continue_line(out)
        self.fp.write(out)
        self.current_len = 0

    def append_to_line(self, line):
        self.fp.write(line)
        self.current_len += len(line)

    def new_line(self):
        '''write a line without any indent'''
        self.fp.write('\n')
        
#    def start_line(self):
#        self.fp.write('\n%s' % self.indent)
#        self.current_len = self.icount

########################################
#
#
if __name__ == '__main__':
    
    handlers = {'restart':None,
                'dump':None,
                'read':None,
                'data':None}

    try:
        ddd = AttrDict(handlers)
    except RuntimeError, msg:
        print msg

    del handlers['data']

    ddd = AttrDict(handlers)
    ddd.restart = 1
    print "restart=", ddd.restart
    print "dump=", ddd.dump
    ddd.read = 1

    print ddd.decl()
    
    try:
        ddd.bad = 4
    except AttributeError, msg:
        print msg
        
