$TESTOPTIONS: -c
ctr
#///////////////////////////////////////////////////////////////////////////
#//
#// FILE NAME
#//    $Id: ctr.v,v 1.1 2001/07/16 20:53:08 taylor Exp $
#//
#// AUTHOR
#//    Scott Haney, LLNL, (510) 423-6308
#//
#// CREATED
#//    February 3, 1992
#//
#// DESCRIPTION
#//    Variable description file for Corsica's core transport package.
#//
#///////////////////////////////////////////////////////////////////////////

#///////////////////////////////////////////////////////////////////////////
#//
#// DEFINITIONS
#//
#///////////////////////////////////////////////////////////////////////////

{
  Ntemps     =  2             # Maximum index of temperatures.
  Nparts     =  10            # Maximum index of particles.
  Ndata      =  6             # Maximum amount of particle/heat flux data.
  Nscalings  =  6             # Number of 0-D scaling laws.
  Nconfig    =  Nparts+2      # Size of configuration array.

  GLFNKY     = 20             # Max no. ky's in glf2d group Callglf2d_com
}


#///////////////////////////////////////////////////////////////////////////
#//
#// GROUPS
#//
#///////////////////////////////////////////////////////////////////////////

******** Transport_Control:
# Parser callable functions for the ctr package.

genTrStorage                       csubroutine
  # Makes storage for the transport module.

newTrPlasma                        csubroutine
  # Resets the plasma into a completely "clean" state.

checkTrEqns                        csubroutine
  # Checks to make sure that the current euqation set makes sense.

configureTrEqn(iEqn: integer, iFlags: integer) csubroutine
  # Specifies solution technique and perhaps initial cond. for an equation...
  # iEqn -- (input) which equation; choices are iElec, ..., iLzTor in Torflux
  # iFlags -- (both input and stored into eqnsConfig?)

setTrInitConds                     csubroutine
  # Establishes initial conditions for transport. ...
  # Possible to specify [qtor, iotaTor], [entor, tempTor], and densTor.

computeTrProfiles                  csubroutine
  # Computes profile based on fitting parameters.

setBCs                             csubroutine
  # Sets the boundary conditions for the equations being solved.

computeTrCoefs                     csubroutine
  # Computes transport coefficients.

computeTrParms                     csubroutine
  # Computes plasma parameters like tau_E's, averages, etc.

advanceTrEqns                      csubroutine
  # Computes matrices needed to advance the transport.

setTrModel(iModel: integer, iDefault: integer) csubroutine
  # Resets the transport model.

makeTrProfile(u: real, u0: real, a: real, b: real, r: real, n: real)
  csubroutine
  # Calculates a profile u(rhobar) according to Nevins' form.

testTr(in: real, out: real)        csubroutine
  # Generic testing routine for the ctr package.

ip_chi(rlt: real, rln: real, rlne: real, q: real, kappa: real,
  shat: real, zth: real, nbeam: real, tau: real, eps: real,
  gnu: real, beta: real, g_perp: real, rmajor: real, rho_i: real,
  v_ti: real, rho_e: real, v_te: real, etg: real, rlte: real,
  rltcrit: real, rltcritz: real, chi_0: real, gfac: real,
  chi_i: real, chi_e: real)   subroutine
  # Calculates chi_i and chi_e for KDBH transport model.

evalSimple                        csubroutine
  # evaluate chiSimple, diffSimple and diff_LzSimple.

evalNeo                           csubroutine
  # evaluate chiNeo, etc., by first setting either chiCH, chiHS, or ..., ...
  # depending on which neo-classical model has been selected (see neoModel).

evalCH                            csubroutine
  # evaluate chiCH and diffCH, etc., from Chang-Hinton.

evalRLW                           csubroutine
  # evaluate chiRLW, diffRLW, and pinchRLW.

evalGLF                           csubroutine
  # evaluate chiGLF, diffGLF

evalHS                            csubroutine
  # evaluate chiHS, diffHS, pinchHS, etc., from Hirshman-Sigmar.

evalNclass                        csubroutine
  # evaluate chiNclass, etc., from the Nclass neo-classical module.

evalKDBH                          csubroutine
  # evaluate chiKDBH, etc., the IFS/PPPL transport model.

evalUserTransport                 csubroutine
  # Call the user supplied routine to evaluate the transport....
  # To use this function, write a routine that evaluates chi,
  # diff, etc., and then assign the name of this function to
  # the global character string userTransportFunction.
  # As the userTransportFunction directly sets diff, chi, etc.,
  # this is no different from calling computeTrCoefs, but it is
  # retained for compatibility with the other models.

glfinit1_0                        subroutine
  # Set parameters for GLF23 comprehensive transport model; ...
  # for use with Gary Smith's Basis glf2d version 1.0 03/25/96.

glftest1_0                        subroutine
  # Set test-drive input for GLF23 comprehensive transport model; ...
  # for use with Gary Smith's Basis glf2d version 1.0 03/25/96.

glf2d1_0                          subroutine
  # Compute transport coefficients according to the GLF23 model. ...
  # This is Gary Smith's Basis glf2d version 1.0 03/25/96.  See also vGLF.

glf2d                             subroutine
  # Compute transport coefficients according to the GLF23 model, ...
  # NTCC version 1.4. See also vGLF.

$callglf2d( leigen: integer, jshoot: integer, jmm: integer,
$  jmaxm: integer, itport_pt: integer, 
$  te_m: real, ti_m: real, ne_m: real,
$  ni_m: real, ns_m: real, i_grad: integer, idengrad: integer,
$  zpte_in: real, zpti_in: real, zpne_in: real, 
$  zpni_in: real, angrotp_exp: real, egamma_exp: real, gamma_p_exp: real, 
$  vphi_m: real, vpar_m: real, vper_m: real, zeff_exp: real, 
$  bt_exp: real, rho: real, arho_exp: real, gradrho_exp: real,
$  gradrhosq_exp: real, rmin_exp: real, rmaj_exp: real,
$  rmajor_exp: real, zimp_exp:real, amassimp_exp:real,
$  q_exp: real, shat_exp: real, alpha_exp: real, elong_exp: real, 
$  amassgas_exp: real, alpha_e: real, x_alpha: real, 
$  i_delay: integer, diffnem: real,
$  chietem: real, chiitim: real, 
$  etaphim: real, etaparm: real, etaperm: real, exchm: real, diff_m: real, 
$  chie_m: real, chii_m: real, 
$  etaphi_m: real, etapar_m: real, etaper_m: real, exch_m: real, 
$  egamma_m: real, egamma_d: real,
$  gamma_p_m: real, anrate_m: real, anrate2_m: real,
$  anfreq_m: real, anfreq2_m: real)      subroutine
$ ^^^ 1.4 NTCC Lib version. list Transport_Control with this: bus error
$$ VVV Aug 2000 Ntc code version. "
$callglf2d( leigen_arg:integer, nroot_arg:integer,
$  jshoot_arg:integer, jmm_arg:integer, 
$  jmaxm_arg:integer, itport_pt_arg:integer, irotstab_arg:integer,
$  te_m_arg:real, ti_m_arg:real, ne_m_arg:real,
$  ni_m_arg:real, ns_m_arg:real, i_grad_arg:integer, idengrad_arg:integer,
$  zpte_in_arg:real, zpti_in_arg:real, zpne_in_arg:real,
$  zpni_in_arg:real, angrotp_exp_arg:real, egamma_exp_arg:real,
$  gamma_p_exp_arg:real,
$  vphi_m_arg:real, vpar_m_arg:real, vper_m_arg:real, zeff_exp_arg:real,
$  bt_exp_arg:real, bt_flag_arg:integer,
$  bteff_exp_arg:real, rho_arg:real, arho_exp_arg:real, gradrho_exp_arg:real,
$  gradrhosq_exp_arg:real, rmin_exp_arg:real, rmaj_exp_arg:real,
$  rmajor_exp_arg:real, zimp_exp_arg:real, amassimp_exp_arg:real,
$  q_exp_arg:real, shat_exp_arg:real, alpha_exp_arg:real, elong_exp_arg:real,
$  amassgas_exp_arg:real, alpha_d_arg:real, alpha_e_arg:real, x_alpha_arg:real,
$  i_delay_arg:integer, diffnem_arg:real,
$  chietem_arg:real, chiitim_arg:real,
$  etaphim_arg:real, etaparm_arg:real, etaperm_arg:real, exchm_arg:real,
$  diff_m_arg:real, chie_m_arg:real, chii_m_arg:real,
$  etaphi_m_arg:real, etapar_m_arg:real, etaper_m_arg:real, exch_m_arg:real,
$  egamma_m_arg:real, egamma_d_arg:real,
$  gamma_p_m_arg:real, anrate_m_arg:real, anrate2_m_arg:real,
$  anfreq_m_arg:real, anfreq2_m_arg:real, xkymax_m_arg:real,
$  xkymax2_m_arg:real) subroutine

  # Calculates chi_i, chi_e, diff, diff_Lz for GLF23 transport model ...
  #  --fills common /glf/ with argument values and calls glf2d.
  # See source for callglf2d.f for variable documentation.

callglf2d_wrap() subroutine
  # wrapper to call callglf2d. See ctr.v for callglf2d arguments.

getOmegaExB(ifield: integer)      csubroutine
  # get ExB shear, starting from toroidal rotation frequency,
  # E_r, or E_r/RB_p depending on value of ifield.  
  # See group ErShearFields for allowed values of ifield.


******* Transport_Hooks:
# Basis code hooks used by transport.

userSource_hooks_n			integer / 0 /
  # no. of hooks to evaluate user-supplied sources.

diagnostic_hooks_n			integer / 0 /
  # no. of hooks to calculate diagnostics.

iterdiag_hooks_n			integer / 0 /
  # no. of hooks to calculate diagnostics within iter loop.

history_hooks_n				integer / 0 /
  # no. of hooks to customize history collection

profiles_prehooks_n 			integer / 0 /
  # no. of pre-hooks for computeTrProfiles

profiles_posthooks_n 			integer / 0 /
  # no. of post-hooks for computeTrParams

transCoefs_prehooks_n 			integer / 0 /
  # no. of pre-hooks for computeTrCoefs

transCoefs_posthooks_n 			integer / 0 /
  # no. of post-hooks for computeTrCoefs

parameters_prehooks_n 			integer / 0 /
  # no. of pre-hooks for computeTrParms

parameters_posthooks_n 			integer / 0 /
  # no. of post-hooks for computeTrParms

equations_prehooks_n			integer / 0 /
  # no. of pre-hooks for advanceTrEqns

equations_posthooks_n			integer / 0 /
  # no. of post-hooks for advanceTrEqns


******* Transport_Flags:
# Flags used to select various options in the transport calculation.

$$ Flags and variables used to select the transport model...

iSimple                            integer / 1 /
  # Simple (parabolic chi,D) transport model selector; see currentModel.
iNeo                               integer / 2 /
  # Neo-classical transport model selector; see currentModel.
iRLW                               integer / 3 /
  # Rebut-Lallia-Watkins critical temperature gradient model selector.
iKDBH                              integer / 4 /
  # Kotschenreuther-Dorland-Beer-Hammett transport model selector.
iGLF                               integer / 5 /
  # GLF (Waltz) transport model selector.
iUser                              integer / 6 /
  # User supplied (via userTransportFunction) model...
  # To use this option, write a basis-function that evaluates chi,
  # diff, etc., and then assign the name of this function to
  # the global character string userTransportFunction.

currentModel                +input integer
  # Present transport model being used. Choices include...
  #   iSimple       - simple model (polynomial transport coeff's);
  #                   see chi,diff,diff_LzData
  #   iNeo          - neo-classical model; see also neoModel
  #   iRLW          - Rebut-Lallia-Watkins critical temp. grad. model
  #   iKDBH         - Kotschenreuther-Dorland-Beer-Hammett model
  #   iGLF          - GyroLandauFluid model of Waltz
  #   iUser         - user sets chi,diff,pinch in script userTransportFunction
  #
  # See also suppress_Simple, suppress_PS, and suppress_Neo.

neoWarn                     +input logical / TRUE /
  # neoWarn=false turns off warning message in ctr::evalNclass & evalHS

$$ Flags used to add classical-PS and neo-classical transport to other models

suppress_Simple             +input integer / 1 /
  # When currentModel is neither iSimple, iNeo, nor iUser: then ...
  # iff suppress_Simple=0, Simple transport is added to diff, chi, etc.

suppress_Neo                +input integer / 0 /
  # When currentModel is neither iNeo nor iUser: then neo-classical ...
  # transport is automatically added to diff, etc., iff suppress_Neo = 0.

suppress_PS                 +input integer / 0 /
  # Non-zero to suppress Classical/PS transport which otherwise is added to all

$$ Flags used to control the transport numerics...

useMassLumping                     logical / TRUE /
  # True if mass-lumping is to be used.
useBSplineInterp                   logical / TRUE /
  # True if B-Splines are to be used to interpolate H-profiles.
nonConvDirichletBC                 logical / TRUE /
  # True--> forces Dirichlet directly
goodEndDerivative                  logical / TRUE /
  # True--> add the residual of the last test function to the residual...
  #	    of the second to the last. This is effective only when using
  #         Dirichlet BC and nonConvDirichletBC=TRUE.
fluxEnforceBC                      logical / FALSE /
  # True--> the y and hr eqns use the boundary condition as a constraint...
  #         to replace the residual of the last test function.
hr_BCFlag                          integer /0/
  # 0 Boundary condition for hrTor is jparTor_edge...
  # 1 Boundary condition for hrTor is  jtTor_edge
  # Note that this forces the j..Tor to maintain its edge value

$$ Flags used to indicate the solution method.

elementType                        integer / 2 /
  # Specifies the type of finite element to use...
  #    == iLinearFE (4)  => linear element
  #    == iBSplineFE (2) => cubic element

eqnsConfig(Nconfig)                integer
  # Array holding configuration information for the transport equations.

iNoEqn                             integer / 0 /
  # Selector indicating a quantity is not being evolved.
iZeroD                             integer / 1 /
  # Selector indicating a quantity is evolved using a 0-D equation.
iBSplineFE                         integer / 2 /
  # Selector indicating a quantity is evolved using B-spline Finite elements.
iLinearFE                          integer / 4 /
  # Selector indicating a quantity is evolved using linear Finite elements.

$$ Flags used to control the nature of the flux evolution equation...

iSpecifyIotaIC                     integer / 8 /
  # Flag indicating that iotaTor gives the initial condition in the iota eqn...
iSpecifyQIC                        integer / 16 /
  # Flag indicating that qtor gives the initial condition in the iota eqn...

$$ Flags used to control the nature of the entropy evolution equation...

iSpecifyEntropyIC                  integer / 8 /
  # Flag indicating that entor gives the initial condition in the mu eqn...
iSpecifyTempIC                     integer / 16 /
  # Flag indicating that tempTor gives the initial condition in the mu eqn...

$$ Miscellaneous flags to turn off various physics models

useUGradP                          integer / 1 /
  # Flag specifying whether the u.p' term is kept in the heat eqn...
  # (0 == false, 1 == true)

magnShld                           integer / 1 /
  # 1: calculate magnetic shielding, 0 don't.

$$ 0-D energy confinement scaling selectors....

iNeoAlc                            integer / 1 /
  # Neo-Alcator scaling index.
iDIIID                             integer / 2 /
  # DIIID ohmic scaling index.
iITER_P                            integer / 3 /
  # ITER-89 power law scaling index.
iITER_O                            integer / 4 /
  # ITER-89 offset-linear scaling index.
iRebutLallia                       integer / 5 /
  # Rebut-Lallia scaling index.
iITER_PMin                         integer / 6 /
  # ITER power, neo-alcator composite scaling index.

$$ Flags controlling the operation of the sawtooth model...

useSawtoothModel                   integer / 0 /
  # Flag to determine whether sawtooth model is applied...
  # If useSawtoothModel <> 0, then the sawtooth model
  # is called immediately after the "parameters" are
  # calculated by the plasma.
  # Four options are available: iNoSawtooth, iUserSawtooth,
  # iChiSawtooth, iSmoothSawtooth.
  # Also, all models call the function "fix_sigTor" to fix up
  # the conductivity. However a later call to toroid overwrites
  # this and thus "fix_sigTor" is also called from do_Toroid.

iNoSawtooth                        integer / 0 /
  # No sawtooth model (DEFAULT).
iUserSawtooth                      integer / 1 /
  # Calls the basis function "user_sawtooth_model".
iChiSawtooth                       integer / 2 /
  # Sets chi = chi(q==1) * chiEnhanceFac for q < 1.
iSmoothSawtooth                    integer / 3 /
  # Like iChiSawtooth, only smooth transition to chi_max.

$$ Misc.

boundaryFluxHack                   integer / 0 /
  # Use boundary condition to calculate fluxes at last grid point.

fixed_dens                         logical /FALSE/ +input
 # Fix nTor 
fixed_y                            logical /FALSE/ +input
 # Fix yTor
fixed_mu                           logical /FALSE/ +input
 # Fix muTor
fixed_lz                           logical /TRUE/  +input
 # Fix lzTor

******* Transport_Dimensions:
# Array dimensions.

nrho                               integer
  # Size of the rhobar grid.

nigpts                             integer / 4 /
  # Number of gauss points per interval.

nfps                               integer
  # Number of fitting parameters.

nparts                             integer / Nparts /
  # Number of particle species.

ntemps                             integer / Ntemps /
  # Number of temperatures

nhterms                            integer
  # Number of terms in the DiffusionEqn class's equation model.

iElecIon                           integer  /3/
  # off-diagonal

iIonElec                           integer  /4/
  # off-diagonal

******* Flux_Transport:
# Variables relevant to rotational transform & magnetic flux evolution.
thetaFlux      /1.0/               real
  # implicit fraction for flux

torfDot                            real [1/sec]
  # (Time derivative of torf) / torf.

psiAxis                            real [Wb]
  # Poloidal flux at the magnetic axis.
psiAxisDot                         real [Wb/sec]
  # Time derivative of poloidal flux at the magnetic axis.
psiEdge                            real [Wb]
  # Poloidal flux at plasma edge

ipPrEdge                           real [MA]
  / 0.0 /
  # Derivative of ipTor at edge...
  # Used in spline fit. If not zero, should probably use
  # the value 777 to force the use of not-a-knot fitting.

jtTor_edge                         real [A/M]
  # Boundary condition for hrTor if hr_BCFlag=1
  # This is used only when withHyperR == true, and the coefficient
  # for Hyper-Resistivity is non-zero at the edge.

jparTor_edge                       real [A/M^2/T]
  # Boundary condition for hrTor if hr_BCFlag=0
  # This is used only when withHyperR == true, and the coefficient
  # for Hyper-Resistivity is non-zero at the edge.

yBC_(3)                            real
  /0,1.0,1.0/
  # Boundary conditions for the y equations...
  # BC(1) y' + BC(2) y = BC(3), where y = yTor, y' = yPrTor
  # Normally these are calculated from the plasma
  # current: y(1) = Mu0 * c3wall * 1e6 * plascur / TwoPi./ c23

yBC_old(3)                         real

c23Dot(nrho)                      _real
  # Time derivative of c2 * c3.

ipTor(nrho)                       _real [MA]
  # Current contained inside a flux surface.
ipPrTor(nrho)                     _real [MA]
  # Rhobar derivative of current in a flux surface.

strTor(nrho)                      _real
  # internal stretching for flux equations str=rhobar**(lstrTor):...
  # rhobar is no longer dimensionless toroidal flux
lstrTor            / 1 /
  # stretching power
unstrTor(nrho)                    _real
  # see strTor
strPrTor(nrho)                    _real
  # rho derivative of strTor
rhostr(nrho)                      _real
  # equals strTor/strPrTor=rhobar/lstrTor

iotaTor(nrho)                     _real
  # Rotational transform on the rhobar grid.
iotaPrTor(nrho)                   _real
  # Derivative of iotaTor on the rhobar grid.

yTor(nrho)                        _real [Wb]
  # Dependent flux variable on the rhobar grid...
  # y = torf * iota.
yPrTor(nrho)                      _real [Wb]
  # "Rhobar" derivative of y on the rhobar grid now yTor+rhobar*yPrTor...
  # Necessary if internally stretched then rhobar is the independent
  # but it is still the same physical quantity, ,sorry LLL
yPrPrTor(nrho)                    _real [Wb]
  # 2nd Rhobar derivative of y on the rhobar grid.
yTorDot(nrho)                     _real
  # time-derivative of yTor

hrTor(nrho)                       _real [A/Wb^2]
  # Hyper Resistivity term is the flux equation
hrPrTor(nrho)                     _real [A/Wb^2]
  # "Rhobar" derivative of hrTor.
hrPrPrTor(nrho)                   _real [A/Wb^2]
  # 2nd Rhobar derivative of hrTor on the rhobar grid.

qExp(nrho)                        _real
  # experimental q from EFIT or better

pExp( nrho)                       _real [pascals]
  # MHD pressure from EFIT or better

jniTor(nrho)                      _real [A/m]
  # <J . B> / <Btor / R> from all non-inductive sources (CD,bootstrap ...) ...
  # list jtTor for fluxCord=2 option
jICTor(nrho)                      _real [A/m]
  # <J . B> / <Btor / R> from ion cyclotron current drive...
  # list jtTor for fluxCord=2 option
jLHTor(nrho)                      _real [A/m]
  # <J . B> / <Btor / R> from lower hybrid current drive...
  # list jtTor for fluxCord=2 option
jECTor(nrho)                      _real [A/m]
  # eccd from all sources...
  # list jtTor for fluxCord=2 option
juserTor(nrho)                    _real [A/m]
  # <J . B> / <Btor / R> piece from user...
  # list jtTor for fluxCord=2 option

factTor(nrho)                     _real
  # electro shielding factor from Start Cordey

yHProfs(nrho, nhterms, 4)         _real
  # Factors in flux diffusion equation.
yHProfs_old(nrho, nhterms, 4)     _real

withHyperR			   logical / FALSE /
  # TRUE means hyper-resistivity included.

******* Entropy_Transport:
# Variables relevant to rotational electron & ion entropy evolution.

thetaMu      /1.0/                 real
  # implicit fraction for entropy

convMult                           real  / 2.5 /
  # Factor multiplying the particle flux in the entropy equation.

tempVolAvg(Ntemps)                 real [keV]
  # Volume-averaged temperatures.
tempNVolAvg(Ntemps)                real [keV]
  # Density-weighted-volume-averaged temperatures.

heatSrcTot(Ntemps)                 real [MW]
  # Net source terms.

pUGradPTot(Ntemps)                 real [MW]
  # Total power due to u . Grad(p) term

tempBC(3, Ntemps)                  real
  # Temperature boundary conditions for the entropy equations...
  # BC(1) <grad-T> + BC(2) T = BC(3), where T = tempTor,
  # < . > = flux-surface-average, and <grad-T> = c4*tempPrTor.

tempBC_old(3, Ntemps)              real

muBC_(3, 2*Ntemps)                 real
  # Entropy boundary conditions for the entropy equations...
  # BC(1) mu' + BC(2) mu = BC(3), w/ mu = muTor, mu' = muPrTor.
  # Normally these are calculated from the tempBC's.

muBC_old(3, 2*Ntemps)              real

pUGradP(nrho, Ntemps)             _real [MW/m^3]
  # Power due to u . Grad(p)

muTor(nrho, Ntemps)               _real
  # Dependent entropy variable on the rhobar grid...
  # mu = entropy / torf^5/3 = N_20 T_keV (V' / torf)^2/3.
muPrTor(nrho, Ntemps)             _real
  # Rhobar derivative of mu on the rhobar grid.
muPrPrTor(nrho, Ntemps)           _real
  # 2nd rhobar derivative of mu on the rhobar grid.
muTorDot(nrho, 2)                 _real
  # time-derivative of muTor
muToTemp(nrho, 2, Ntemps)         _real
  # Conversion factor...
  #   tempTor   = muToTemp(,1,) * muTor  (+ Tiny)
  #   tempPrTor = muToTemp(,1,) * muPrTor + muToTemp(,2,) * muTor

vpr23(nrho)                       _real
  # (V'/torf)**(2/3)

gammaPS(nrho,2)                   _real [1/sec]
  # gammaPS = c4* <Pf_Schl_pflux>/densTor 

chi(nrho, 2*Ntemps)               _real [m^2/sec]
  # Diffusive heat flux = -n chi Grad-T. ...
  # Includes the convective heat flux proportional to T'.

fluxH(nrho, Ntemps)               _real [keV/m^2/sec]
  # Heat flux exclusive of chi, remaining convective flux, and ...
  # pinchMu term.
  # corrects a multitude of sins when neglecting off-diagonal terms.

pinchH(nrho, Ntemps)              _real [m/sec]
  # Pinch heat flux = pinchMu * densTor * T.

heatSrc(nrho, Ntemps)             _real [MW/m^3]
  # Heat source in entropy equations.
heatFluxCoefs(nrho, 3, 2*Ntemps)  _real
  # Coefficients used to construct the heat fluxes...
  # heatFlux = -hfc(1) dmu/drho + hfc(2) mu + hfc(3).
heatFlux(nrho, Ntemps)            _real [MW/m^3]
  # Surface-averaged (|grad rho| * heat fluxes) in entropy equations.

entropyTor(nrho, Ntemps)          _real
  # Entropy densities, n_20 * T_keV * (V')^5/3 on the rhobar grid.

muHProfs(nrho, nhterms, 2*Ntemps) _real
  # Factors in the entropy diffusion equations.

muHProfs_old(nrho, nhterms, 2*Ntemps) _real

******* Auxiliary_Power:
# Auxiliary power source terms in entropy equations.

pAuxTot(Ntemps)                    real [MW]
  # Total auxiliary heating to all species.

pAuxCoefs(3, Ntemps)               real
  # Coefficients controlling auxiliary heating deposition.

pAux(nrho, Ntemps)                _real [MW/m^3]
  # Auxiliary heating power density.


******* Drag_Power:
# Drag source/sink terms in entropy equations.

pDragTot                           real [MW]
  # Total ion-electron drag power.

pDrag(nrho)                       _real [MW/m^3]
  # Ion-electron drag power density.

dragCoef(nrho)                    _real [MW/m^3/keV]
  # Coefficient multiplying the temperature difference in the drag term.


******* Fusion_Power:
# Fusion power source terms.

pFusDDElecTot                      real [MW]
  # Volume integrated DD fusion power to electrons.
pFusDDIonTot                       real [MW]
  # Volume integrated DD fusion power to ions.
pNeutDDTot                         real [MW]
  # Volume integrated 2.45 MeV neutron power from DD reactions.
pFusDDTot                          real [MW]
  # Volume integrated DD fusion power.

fPromptLoss                        real / 0.0 /
  # Fraction of fast alpha particles lost from plasma.

pFusDTElecTot                      real [MW]
  # Volume integrated DT fusion power to electrons.
pFusDTIonTot                       real [MW]
  # Volume integrated DT fusion power to ions.
pNeutDTTot                         real [MW]
  # Volume integrated 14.1 MeV neutron power from DT reactions.
pFusDTTot                          real [MW]
  # Volume integrated DT fusion power.

sigmavDDn(nrho)                   _real [m^3/sec]
  # DD fusion reaction (n, He3) rate parameter.
sigmavDDp(nrho)                   _real [m^3/sec]
  # DD fusion reaction (p, T) rate parameter.
rxnRateDDn(nrho)                  _real [#/sec/m^3]
  # DD fusion reaction (n, He3) rate density.
rxnRateDDp(nrho)                  _real [#/sec/m^3]
  # DD fusion reaction (p, T) rate density.
pFusDDElec(nrho)                  _real [MW/m^3]
  # DD fusion power density to electrons.
pFusDDIon(nrho)                   _real [MW/m^3]
  # DD fusion power density to ions.
pNeutDD(nrho)                     _real [MW/m^3]
  # 2.45 MeV neutron power density.
fastPressDD(nrho)                 _real [Pa]
  # Fast ion (He3 + T + p) pressure from DD reactions.

sigmavDT(nrho)                    _real [m^3/sec]
  # DT fusion reaction rate parameter.
rxnRateDT(nrho)                   _real [#/sec/m^3]
  # DT fusion reaction rate density.
pFusDTElec(nrho)                  _real [MW/m^3]
  # DT fusion power density to electrons.
pFusDTIon(nrho)                   _real [MW/m^3]
  # DT fusion power density to ions.
pNeutDT(nrho)                     _real [MW/m^3]
  # 14.1 MeV neutron power density.
fastPressDT(nrho)                 _real [Pa]
  # Fast alpha pressure from DT reactions.

rxnRateBtDDn(nrho)		  _real [#/sec/m^3]
  # DD fusion reaction (n, He3) rate density, beam-thermal.
rxnRateBtDT(nrho)		  _real [#/sec/m^3]
  # DT fusion reaction rate density, beam-thermal.
pNeutBtDD(nrho)                   _real [MW/m^3]
  # 2.45 MeV neutron power density, beam-thermal.
pNeutBtDT(nrho)                   _real [MW/m^3]
  # 14.1 MeV neutron power density, beam-thermal.
pNeutBtDDTot                       real [MW]
  # Volume integrated 2.45 MeV neutron power from DD reactions, beam-thermal.
pNeutBtDTTot                       real [MW]
  # Volume integrated 14.1 MeV neutron power from DT reactions, beam-thermal.

rxnRateDDnTot			   real [#/sec]
  # Volume integrated DD fusion reaction (n, He3) rate.
rxnRateDTTot			   real [#/sec]
  # Volume integrated DT fusion reaction rate.
rxnRateBtDDnTot			   real [#/sec]
  # Volume integrated DD fusion reaction (n, He3) rate, beam-thermal.
rxnRateBtDTTot			   real [#/sec]
  # Volume integrated DT fusion reaction rate, beam-thermal.

******* Ohmic_Power:
# Ohmic power source terms.

pOhmicTot                          real [MW]
  # Volume integrated ohmic heating power.

pOhmic(nrho)                      _real [MW/m^3]
  # Ohmic heating power density.

******* RF_Power:
pRFTot                           real [MW]
  # Volume-integrated ech power.

pRF(nrho)                         _real [MW/m^3]
  # ech power density

******* Runaway_Electrons:
# Runaway electron terms

tau_run(nrho)                     _real [sec]
# Growth time of runaway current

source_run(nrho)                  _real [amps/m^2/sec]
# ambient source for runaways

ecrit(nrho)                       _real [volts/m]
# crticical electric field

run_away
 +flag                           logical /FALSE/ +input

******* Radiation_Power:
# Radiation power source terms.

rWall                              real / 0.8 /
  # Wall reflection coefficient.

pBLRRadTot                         real [MW]
  # Total bremsstrahlung, line, and recombination radiation power.

pSyncTot                           real [MW]
  # Total synchrotron radiation power.

pBLRRad(nrho)                     _real [MW/m^3]
  # Bremsstrahlung, line, and recombination radiation power density.

pSync(nrho)                       _real [MW/m^3]
  # Synchrotron radiation power density.


******* Density_Transport:
# Variables relevant to density evolution.

thetaN      /1.0/               real
  # implicit fraction for density

particleSrcTot(Nparts)             real [10^20/sec]
  # Integrated particle sources in density equations.

mass(Nparts)                       real [kg]
  # Masses of the ion species...
  # Must initialize in transport.cc due to %*!@$ mppl bug.

densVolAvg(Nparts)                 real [10^20/m^3]
  # Volume-averaged densities for all particle species.

ionMassAvg                         real
  # Density-weighted mass average for all particle species.

etaDivertor(Nparts)                real
  / 0, 0, 1.0, 1.0, 1.0, 0.98, 0, 0, 0, 0 /
  # Recycling efficiency at the divertor

alphaRecycle(Nparts)               real
  / 0, 0, 0.1, 0.1, 0.1, 0.1, 0, 0, 0, 0 /
  # Fraction of neutrals that are recycled to the core

recycleCoef(Nparts)                real
  / 0, 0, 1.0, 1.0, 1.0, 0.98, 0, 0, 0, 0 /
  # Recycling coefficient for thermal particles species.

gasPuffSrc(Nparts)                 real / Nparts*0.0 /
  # Rate at which gas is being puffed into the SOL.

gasPuffCoef(Nparts)                real / Nparts*0.0 /
  # Recycling coefficient for the puffed gas.

recycleWidth                       real / 0.025 /
  # Scale length for recycling source as fraction of minor radius.

densBorass                         real [10^20/m^3]
  # Limit on average electron density according to Borass.

densGreenwald                      real [10^20/m^3]
  # Limit on average electron density according to Greenwald.

densBC(3, Nparts)                 real
  # Density (densTor) boundary conditions for the nV' equations...
  # BC(1) <grad-n> + BC(2) n = BC(3), where n = densTor,
  # < . > = flux-surface-average, and <grad-n> = c4*densPrTor.

nBC_(3, Nparts)                   real
  # nV' boundary conditions for the nV' equations...
  # BC(1) N' + BC(2) N = BC(3), w/ N = nTor, N' = nPrTor
  # Normally these are calculated from the densBC's.

nTor(nrho, Nparts)                _real
  # Dependent density variable on the rhobar grid...
  # N = n_20 V' / torf^2.
nPrTor(nrho, Nparts)              _real
  # Rhobar derivative of N on the rhobar grid.
nPrPrTor(nrho, Nparts)            _real
  # 2nd rhobar derivative of N on the rhobar grid.
nTorDot(nrho, Nparts)             _real
  # time-derivative of nTor.
massDensPrTor(nrho)               _real [kg/m^3]
  # rhobar derivative of Mass (ion) density.
mn_over_charge(nrho)              _real [kg/m^3]
  # sum over ions of m_i n_i/chargeNum(i)
mnPr_over_charge(nrho)            _real [kg/m^3]
  # sum over ions of  m_i (d/drho)n_i/chargeNum(i)

nToDens(nrho, 2)                  _real
  # Conversion factor...
  #   densTor   = nToDens(,1) * nTor
  #   densPrTor = nToDens(,1) * nPrTor + nToDens(,2) * nTor

recycleSrc(nrho, Nparts)          _real
  # The particle source due to recycling.

implicitRecycleSrc(nrho, Nparts)  _real
  # Used internally to handle the recycling source.

diff(nrho, Nparts)                _real [m^2/sec]
  # Diffusive particle flux = -diff Grad-n

pinch(nrho, Nparts)               _real [m/sec]
  # Pinch particle flux = pinch n.

fluxN(nrho, Nparts)               _real [10^20/m^2/sec]
  # Flux of particles exclusive of pinch and diff terms ...
  # corrects a multitude of sins when neglecting off-diagonal terms

particleSrc(nrho, Nparts)         _real [10^20/m^3/sec]
  # Particle sources in density equations.

particleFluxCoefs(nrho, 3, Nparts) _real
  # Coefficients used to construct the particle fluxes ...
  # particleFlux = -pfc(1) dN/drho + pfc(2) N + pfc(3).

particleFlux(nrho, Nparts)        _real [10^20/m^3/sec]
  # Surface-averaged (|grad rho| * particle fluxes) in density equations.

sumPartFlux(nrho)                 _real [10^20/m^3/sec]
  # Sum of surface-averaged (|grad rho| * particle fluxes) in ion density
  # equations.  Calculated and used in toroidal momentum eq. class

nHProfs(nrho, nhterms, Nparts)    _real
  # Factors in the density diffusion equations.


******* Turbulent_Transport  input:
# Inputs for turbulent transport models (in general)

shat(nrho)                        _real [none]
  # (rho / q) * dq/d(rho)
taui(nrho)                        _real [none]
  # T_i / T_e


******* Simple_Transport_Input input:
# Inputs for the "Simple" transport model. See also suppress_Neo, suppress_PS.

chiData(Ndata, Ntemps)            _real
  # Data used to parameterize the thermal diffusivity...
  #
  #              a^2   (1 + c2 rhobar + c4 rhobar^c5)
  #   chi = c1  -----  ------------------------------
  #             Tau_E            (1 + c2)
  #
  # where cN (N = 1, 5) = chiData(N,J), J = {iIon, iElec},
  # and Tau_E is calculated from a scaling law. The particular
  # scaling law depends on chiData(3,), which can take on one
  # of the following values:
  #
  #    $$ 0-D energy confinement scaling selectors....
  #
  #    iNeoAlc                            integer / 1 /
  #      # Neo-Alcator scaling index.
  #    iDIIID                             integer / 2 /
  #      # DIIID ohmic scaling index.
  #    iITER_P                            integer / 3 /
  #      # ITER-89 power law scaling index.
  #    iITER_O                            integer / 4 /
  #      # ITER-89 offset-linear scaling index.
  #    iRebutLallia                       integer / 5 /
  #      # Rebut-Lallia scaling index.
  #    iITER_PMin                         integer / 6 /
  #      # ITER power, neo-alcator composite scaling index.

diffData(Ndata, Nparts)           _real
  # Data used to parameterize particle diffusion coefficients...
  #
  #                  a^2   (1 + d2 rhobar)
  #      diff = d1  -----  ---------------
  #                 Tau_E     (1 + d2)
  #
  # where d1, d2 = diffData(1,iSpecies), diffData(2,iSpecies)
  # and Tau_E is calculated from a scaling law. The particular
  # scaling law depends on chiData(3,), which can take on one
  # of the following values:
  #
  #    $$ 0-D energy confinement scaling selectors....
  #
  #    iNeoAlc                            integer / 1 /
  #      # Neo-Alcator scaling index.
  #    iDIIID                             integer / 2 /
  #      # DIIID ohmic scaling index.
  #    iITER_P                            integer / 3 /
  #      # ITER-89 power law scaling index.
  #    iITER_O                            integer / 4 /
  #      # ITER-89 offset-linear scaling index.
  #    iRebutLallia                       integer / 5 /
  #      # Rebut-Lallia scaling index.
  #    iITER_PMin                         integer / 6 /
  #      # ITER power, neo-alcator composite scaling index.

diff_LzData(Ndata)                _real
  # Data used to parameterize angular momentum diffusion coefficients...
  #
  #                  a^2   (1 + l2 rhobar + l4 rhobar^l5))
  #      diff_Lz = l1  -----  ---------------
  #                 Tau_E     (1 + l2)
  #
  # where lN (N = 1, 5) = diff_LzData(N),
  # and Tau_E is calculated from a scaling law. The particular
  # scaling law depends on chiData(3,), which can take on one
  # of the following values:
  #
  #    $$ 0-D energy confinement scaling selectors....
  #
  #    iNeoAlc                            integer / 1 /
  #      # Neo-Alcator scaling index.
  #    iDIIID                             integer / 2 /
  #      # DIIID ohmic scaling index.
  #    iITER_P                            integer / 3 /
  #      # ITER-89 power law scaling index.
  #    iITER_O                            integer / 4 /
  #      # ITER-89 offset-linear scaling index.
  #    iRebutLallia                       integer / 5 /
  #      # Rebut-Lallia scaling index.
  #    iITER_PMin                         integer / 6 /
  #      # ITER power, neo-alcator composite scaling index.


******* Simple_Transport_Output:
# Transport coefficients for the "Simple" transport model ...
# See also suppress_Neo and suppress_PS.

chiSimple(nrho,Ntemps)             _real [m^2/sec]
  # Simple transport model chi. See also chiData.

diffSimple(nrho,Nparts)            _real [m^2/sec]
  # Simple transport model diff. See also diffData.

diff_LzSimple(nrho)                _real [m^2/sec]
  #  Simple transport model diff_Lz. See also diff_LzData.


******* Neo_Transport:
# Neo-classical transport coefficients (= chiCH, chiHS, or ... (see neoModel))

diffNeo(nrho,Nparts)               _real [m^2/sec]
  # Neo-classical diff.  = diffCH, diffHS, or ... (see neoModel).

pinchNNeo(nrho,Nparts)             _real [m/sec]
  # Neo-classical pinch. pinchNNeo = pinchNHS, or ... (see neoModel).

fluxNNeo(nrho,Nparts)              _real [10^20/m^2/sec]
  # Neo-classical fluxN. fluxNNeo = fluxNHS, or ... (see neoModel).

chiNeo(nrho,2*Ntemps)              _real [m^2/sec]
  # Neo-classical chi.  = chiCH, chiHS, or ... (see neoModel).

pinchHNeo(nrho,Ntemps)             _real [m/sec]
  # Neo-classical pinchH. pinchHNeo = pinchHHS, or ... (see neoModel).

fluxHNeo(nrho,Ntemps)              _real [keV/m^2/sec]
  # Neo-classical fluxH. fluxHNeo = fluxHHS, or ... (see neoModel).

diff_LzNeo(nrho)                   _real [m^2/sec]
  # Neo-classical angular momentum diff (for all neoModel's).


******* CH_Transport:
# Transport coefficients for the Chang-Hinton neo-classical model

chiCH(nrho, Ntemps)              _real [m^2/sec]
  # Chang-Hinton neo-classical chi.

diffCH(nrho)                     _real [m^2/sec]
  # Chang-Hinton neo-classical diff.
vWarePinch(nrho)                 _real [m/sec]
  # Chang-Hinton neo-classical pinch (currently ignored).


******* RLW_Transport_Input:
# Input parameters for the RLW transport model ...
# See also suppress_Simple, suppress_PS, and suppress_Neo.

rebutHElec                        real / 1.0 /
  # chi(,iElec) = rebutHElec * (chi + chiRLW)

rebutHIon                         real / 1.0 /
  # chi(,iIon)  = rebutHIon * (chi + chiRLW)

rebutDiffCoef(Nparts)             real / Nparts*1.0 /
  # diff  = rebutDiffCoef * (diff + diffRLW)...
  # pinch = rebutDiffCoef * (pinch + pinchRLW)
  # See also suppress_Simple, suppress_PS, and suppress_Neo.

rebutDiff_LzCoef                  real / 1.0 /
  # diff_Lz  = rebutDiff_LzCoef * (diff_Lz + diff_LzRLW)

chifact1                          real / 2.0 /
  # coefficient of RLW chi.

critTeCoef                        real / 6.0 /
  # coefficient of grad Te_crit.

chiIECoef                         real / 2.0 /
  # "Old" RLW model for chi_i...
  # For old RLW model, use chiIECoef = 2, chiIECoef2 = 0.
  # For new RLW model, use chiIECoef = 0, chiIECoef2 = 2.

chiIECoef2                        real / 0.0 /
  # New RLW model (with rho-star effect) for chi_i...
  # For old RLW model, use chiIECoef = 2, chiIECoef2 = 0.
  # For new RLW model, use chiIECoef = 0, chiIECoef2 = 2.

diffOverChi                       real / 1.0 /
  # ratio of diffRLW to chiRLW(,iElec)

pinchCoef                         real / 0.3 /
  # coefficient of pinchRLW (default 0.3).


******* RLW_Transport_Output:
# Transport coefficients for the RLW transport model ...
# See also suppress_Simple, suppress_PS, and suppress_Neo.

chiRLW(nrho,Ntemps)               _real [m^2/sec]
  # RLW chi.

diffRLW(nrho)                     _real [m^2/sec]
  # RLW diff ...
  # (RLW uses the same diffusion coefficient for all species.)

pinchRLW(nrho)                    _real [m/sec]
  # pinch for the RLW model = chargeNum(iSpecies) * pinchRLW.

diff_LzRLW(nrho)                  _real [m^2/sec]
  # RLW model angular momentum diff ...
  # diff_LzRLW = diff_LzCoef*chiRLW*(<B_pol^2>/<B^2>)^.5

gradTe_crit(nrho)                 _real
  # Critical temperature gradient used in some models, like RLW.

chiAnomalous(nrho)                _real
  # RLW diffusivity before the "trigger" is factored in.

******* HS_Transport_Output:

chiHS(nrho,2*Ntemps)             _real [m**2/sec]
  # Hirshman-Sigmar neo-classical chi.

diffHS(nrho,Nparts)              _real [m**2/sec]
  # Hirshman-Sigmar neo-classical diff.

pinchNHS (nrho,Nparts)           _real [m/sec]
  # Hirshman-Sigmar neo-classical pinch.

pinchHHS(nrho,Ntemps)            _real [m/sec]
  # Hirshman-Sigmar neo-classical pinchH.

fluxNHS(nrho,Nparts)             _real [10^20/m^2/sec]
  # Hirshman-Sigmar neo-classical fluxN.

fluxHHS(nrho,Ntemps)             _real [keV/m^2/sec]
  # Hirshman-Sigmar neo-classical fluxH.


******* Nclass_Transport:

diffNclass(nrho,Nparts)          _real [m^2/sec]
  # Nclass neo-classical diff.

pinchNNclass(nrho,Nparts)        _real [m/sec]
  # Nclass neo-classical pinch.

fluxNNclass(nrho,Nparts)         _real [10^20/m^2/sec]
  # Nclass neo-classical fluxN.

chiNclass(nrho,2*Ntemps)         _real [m^2/sec]
  # Nclass neo-classical chi.

pinchHNclass(nrho,Ntemps)        _real [m/sec]
  # Nclass neo-classical pinchH.

fluxHNclass(nrho,Ntemps)         _real [keV/m^2/sec]
  # Nclass neo-classical fluxH.


******* KDBH_Transport_Input:
# Inputs for energy transport model of Kotschenreuther, et al. ...
# See also suppress_Simple, suppress_PS, and suppress_Neo.

shatmin                  +input    real [none] /.5/
  # Sub. ip_chi uses shatmin if the magnetic shear shat < shatmin.

rmajLTi(nrho)                     _real [none]
  # Rmajor / L_Ti

rmajLNi(nrho)                     _real [none]
  # Rmajor / L_Ni

rmajLNe(nrho)                     _real [none]
  # Rmajor / L_Ne

z_effKDBH(nrho)                   _real [none]
  # = (z_eff*n_e - chargeNum(iNB)**2*nNBTor ) / (n_e-chargeNum(iNB)*nNBTor) ...
  # Dorland in ip_chi.m says to use (n_hydrogenic+36*n_C) / (n_e-nbeam).

eps_tmp(nrho)                     _real [none]
  # inverse aspect ratio

gnuKDBH(nrho)                     _real [none]
  # collisionality parameter

ionGyroRad(nrho)                  _real [m]
  # thermal ion gyroradius

ionVth(nrho)                      _real [m/s]
  # ion thermal velocity

geomfac(nrho)                     _real [none]
  # <|grad rho|> / <|grad rho|**2> * d rho / d r_minor


******* KDBH_Transport_Output  output:
# Outputs for energy transport model of Kotschenreuther, et al. ...
# See also suppress_Simple, suppress_PS, and suppress_Neo.

chiKDBH(nrho,Ntemps)              _real [m^2/sec]
  # KDBH model chi.

diffKDBH(nrho)                    _real [m^2/sec]
  # KDBH model diff. diffKDBH = chiKDBH(,iIon) per Dorland's suggestion.

diff_LzKDBH(nrho)                 _real [m^2/sec]
  # KDBH model angular momentum diff ...
  # diff_LzKDBH = diff_LzCoef*chiKDBH*(<B_pol^2>/<B^2>)^.5

rmajLTcrit(nrho)                  _real [none]
  # critical Rmajor / L_Ti

rmajLTcritz(nrho)                 _real [none]
  # critical Rmajor / L_Tz (carbon impurity)

gamKDBH(nrho)                     _real [rad/sec]
  # fit to the maximum growth rate (seems to be deuterium branch only)


******* GLF_Transport_Input1_0  input:
# Version 1.0 inputs for transport model of R. E. Waltz ...
# See also suppress_Simple, suppress_PS, and suppress_Neo.

eigen_solver_gf1_0                    integer
  # =0 for NAG eigensolver, =1 for cgg
nroot_gf1_0                           integer
  # number of equations
iflagin_gf1_0(20)                     integer
  # control flags (see comments in subroutine glf2d)
xparam_gf1_0(20)                      real
  # control parameters (see comments in subroutine glf2d)
xky0_gf1_0                            real [none]
  # k_theta*rho_s; k_theta= nq/r; normally 0.3
rms_theta_gf1_0                       real [none]
  # width of phi**2 mode function for best fit near pi/3
rlti_gf1_0                            real [none]
  # a/L_Ti, where  a/L_f= sqrt(kappa) a d ln f / d rho
rlte_gf1_0                            real [none]
  # a/L_Te
rlne_gf1_0                            real [none]
  # a/L_ne
rlni_gf1_0                            real [none]
  # a/L_ni
dil_gf1_0                             real [none]
  # dilution (1.-ni_0/ne_0)
xdil_m1_0                             real [none]
  # factor (in interval [0,1]) to turn on dilution
taui_gf1_0                            real [none]
  # Ti/Te
rmin_gf1_0                            real [none]
  # r/a
rmaj_gf1_0                            real [none]
  # Rmaj/a
q_gf1_0                               real [none]
  # safety factor
xnu_gf1_0                             real [none]
  # nu_ei/(c_s/a)
cxnu1_0                               real [none]
  # factor (in interval [0,1]) to turn on trapped-elec. collisionality
xshift1_0                             real [none]
  # factor (in interval [0,1]) to turn on Shafranov shift stabilization
betae_gf1_0                           real [none]
  # neTe/(B**2/(8pi)); 0 is electrostatic
cbetae1_0                             real [none]
  # 1.e-6 for electrostatic; 1. to turn finite beta on fully
shat_gf1_0                            real [none]
  # dlnr/drho; used only for parallel dynamics part
alpha_gf1_0                           real [none]
  # local shear parameter or MHD pressure grad (s-alpha diagram)
elong_gf1_0                           real [none]
  # local elongation or kappa
xwell_gf1_0                           real [none]
  # amount of magnetic well xwell*min(alpha,1)
park_gf1_0                            real [none]
  # a control parameter (=1 or 0) to turn on or off parallel motion;...
  # 0.405 best at zero beta and 2.5x larger at high beta..see iflagin(3)
ghat_gf1_0                            real [none]
  # a control parameter (=1 or 0) to turn on or off curvature drift
gchat_gf1_0                           real [none]
  # a control parameter (=1 or 0) to turn on or off div ExB motion
adamp_gf1_0                           real [none]
  # radial mode damping exponent (1/4 < adamp < 3/4);...
  # 0.25 from direct fit of simulations varying radial mode damping
  # but 0.75 is better fit to rlti dependence
alpha_star_gf1_0                      real [none] / 0. /
  # O(1-3)  gyrobohm breaking coef for diamg. rot. shear
gamma_star_gf1_0                      real [none]
  # ion diamagnetic rot shear rate in units of c_s/a
alpha_e_gf1_0                         real [none] / 0. /
  # O(1-3)  doppler rot shear coef
gamma_e_gf1_0                         real [none]
  # doppler rot shear rate in units of c_s/a
alpha_mode_gf1_0                      real [none] / 0. /
  # mode rotational shear strength (=1 means = alpha_e_gf)
gamma_mode_gf1_0                      real [none]
  # shear rate in mode rotation velocity (without Doppler)
alpha_p_gf1_0                         real [none] / 0. /
  # 1.5 = fit for parallel velocity shear effect at rmaj=3 and q=2
gamma_p_gf1_0                         real [none]
  # parallel velocity shear rate (-d v_phi/ drho) in units of c_s/a
xkdamp_gf1_0                          real [none]
  # model damping, normally 0
ikymax_gf1_0                          integer
  # number of ky modes
xkymin_gf1_0                          real [none]
  # low-ky on log scale
xkymax_gf1_0                          real [none]
  # high-ky on log scale
lprint_gf1_0                          integer
  # controls diagnostic print outs from glf2d
amassgas_gf1_0                        real [none]
  # atomic weight of the gas species
cmodel1_0                             real [none]
  # overall transport strength tuning parameter
vGLF                           /0/    integer
  # glf2d version-selector: 0 latest; 1 glf2d1_0


******* GLF_Transport_Output1_0  output:
# Version 1.0 glf2d outputs for transport model of R. E. Waltz ...
# See also suppress_Simple, suppress_PS, and suppress_Neo.

yparam_gf1_0(20)                      real [none]
  # output diagnostics
xkyf_gf1_0                            real [none]
  # ky at maximum growth rate
gamma_gf1_0(4)                        real [none]
  # maximum growth rate for each root
freq_gf1_0(4)                         real [none]
  # frequency at maximum growth rate for each root
phi_norm_gf1_0(4)                     real [none]
  # normalized saturation value for each root
diff_gf1_0                            real [none]
  # normalized plasma diffusivity for ions
chii_gf1_0                            real [none]
  # normalized ion energy diffusivity
chie_gf1_0                            real [none]
  # normalized electron energy diffusivity
exch_gf1_0                            real [none]
  # normalized anomalous e to i energy exchange
eta_par_gf1_0                         real [none]
  # normalized parallel component of toroidal momentum diffusivity
eta_per_gf1_0                         real [none]
  # normalized perpendicular component of toroidal momentum diffusivity
eta_phi_gf1_0                         real [none]
  # normalized toroidal momentum diffusivity
chie_e_gf1_0                          real [none]
  # normalized eta_e-mode electron energy diffusivity
yparam_k_gf1_0(20,20)                 real [none]
  # output diagnostics for each ky
xkyf_k_gf1_0(20)                      real [none]
  # values of (normalized) ky
gamma_k_gf1_0(4,20)                   real [none]
  # growth rate of each mode at each ky
freq_k_gf1_0(4,20)                    real [none]
  # frequency of each mode at each ky
phi_norm_k_gf1_0(4,20)                real [none]
  # normalized saturation value for each root at each ky
diff_k_gf1_0(20)                      real [none]
  # normalized plasma diffusivity for ions at each ky
chii_k_gf1_0(20)                      real [none]
  # normalized ion energy diffusivity at each ky
chie_k_gf1_0(20)                      real [none]
  # normalized electron energy diffusivity at each ky
exch_k_gf1_0(20)                      real [none]
  # normalized anomalous e to i energy exchange at each ky
eta_par_k_gf1_0(20)                   real [none]
  # norm. parallel component of toroidal momentum diffusivity at each ky
eta_per_k_gf1_0(20)                   real [none]
  # norm. perp. component of toroidal momentum diffusivity at each ky
eta_phi_k_gf1_0(20)                   real [none]
  # normalized toroidal momentum diffusivity at each ky
chie_e_k_gf1_0(20)                    real [none]
  # normalized eta_e-mode electron energy diffusivity at each ky
%ifelse(Module,ctrwake|ctrdata|ctriyiyi,[],[
%  Use(GLF_Transport_Output)
%])


******* GLF_Transport_Output  output:
# ctr outputs for transport model of R. E. Waltz ...
# See also suppress_Simple, suppress_PS, and suppress_Neo.

ionSound(nrho)                    _real [m/s]
  # ion sound speed (normalization velocity)
rhoStar(nrho)                     _real [none]
  # rho_s/arho
xky_m(nrho)                       _real [none]
  # dominant k_theta in units of 1/rho_s
anrate_m(nrho)                    _real [none]
  # dominant growth rate in units of cs/arho
anfreq_m(nrho)                    _real [none]
  # dominant frequency in units of cs/arho
chiGLF(nrho,Ntemps)               _real [m^2/sec]
  # GLF model chi.
diffGLF(nrho)                     _real [m^2/sec]
  # GLF model diff ...
  # (GLF uses the same diffusion coefficient for all species.)
diff_LzGLF(nrho)                  _real [m^2/sec]
  # GLF model angular momentum diff ...
  # diff_LzGLF = diff_LzCoef*chiGLF*(<B_pol^2>/<B^2>)^.5


******* Callglf2d_com /glf_ct/:
# common block for used for communications with glf2d by callglf2d, ...
# made available here for direct calls to glf2d from the parser.
# See ctr/glf/glf2d.F for comments on these variables.
# (Groups GLF_Transport_Input1_0 and GLF_Transport_Output1_0 are well
# commented and contain most of the same variables.)
$
$ NB: /glf/ must strictly duplicate the block in ctr/glf/glf.m .
$
zevec_k_gf(2*GLFNKY,12,12)    real
  # warning: type in glf is complex*16 # unused
zomega_k_gf(2*GLFNKY,12)      real
  # warning: type in glf is complex*16 # unused
yparam_k_gf(GLFNKY,GLFNKY)    real +output
gamma_k_gf(1:4,GLFNKY)        real +output
freq_k_gf(1:4,GLFNKY)         real +output
phi_norm_k_gf(1:4,GLFNKY)     real +output
xparam_gf(30)                 real +input
xkyf_k_gf(GLFNKY)             real +output
diff_k_gf(GLFNKY)             real +output
diff_im_k_gf(GLFNKY)          real +output
chii_k_gf(GLFNKY)             real +output
chie_k_gf(GLFNKY)             real +output
exch_k_gf(GLFNKY)             real +output
eta_par_k_gf(GLFNKY)          real +output
eta_per_k_gf(GLFNKY)          real +output
eta_phi_k_gf(GLFNKY)          real +output
chie_e_k_gf(GLFNKY)           real +output
yparam_gf(GLFNKY)             real +output
gamma_gf(1:4)                 real +output
freq_gf(1:4)                  real +output
phi_norm_gf(1:4)              real +output
xky_gf(1:4)                   real +output
xky0_gf                       real +input # defaulted in callglf2d
rms_theta_gf                  real +input # defaulted in callglf2d
rlti_gf                       real +input
rlte_gf                       real +input
rlne_gf                       real +input
rlni_gf                       real +input
rlnimp_gf                     real +input
dil_gf                        real +input
apwt_gf                       real +input
aiwt_gf                       real +input
taui_gf                       real +input
rmin_gf                       real +input
rmaj_gf                       real +input
q_gf                          real +input
xnu_gf                        real +input
betae_gf                      real +input
shat_gf                       real +input
alpha_gf                      real +input
elong_gf                      real +input
xwell_gf                      real +input
park_gf                       real +input
ghat_gf                       real +input # defaulted in callglf2d
gchat_gf                      real +input # defaulted in callglf2d
adamp_gf                      real +input # defaulted in callglf2d
alpha_star_gf                 real +input # defaulted in callglf2d
gamma_star_gf                 real        # computed in callglf2d < call glf2d
alpha_e_gf                    real +input
gamma_e_gf                    real +input # defaulted in callglf2d
alpha_mode_gf                 real +input # defaulted in callglf2d
gamma_mode_gf                 real +input
alpha_p_gf                    real +input # defaulted in callglf2d
gamma_p_gf                    real +input
xkdamp_gf                     real +input
xkyf_gf                       real +output
diff_gf                       real +output
diff_im_gf                    real +output
chii_gf                       real +output
chie_gf                       real +output
exch_gf                       real +output
eta_par_gf                    real +output
eta_per_gf                    real +output
eta_phi_gf                    real +output
chie_e_gf                     real +output
cnorm_gf                      real # input? defaulted in callglf2d
xkymin_gf                     real # input? defaulted in callglf2d
xkymax_gf                     real # input? defaulted in callglf2d
amassgas_gf                   real +input
amassimp_gf                   real +input
zimp_gf                       real +input
iflagin_gf(30)                integer +input
ngrow_k_gf(0:GLFNKY)          integer +output
nroot_gf                      integer +input
lprint_gf                     integer +input
ikymax_gf                     integer +input
i_err                         integer # unused
i_proc                        integer # unused
n_proc                        integer # unused
jeigen                        integer # unused
igfac                         integer # unused
first_order_gf                integer # unused
ipert_gf                      integer +input
#
##eigen_gf                      integer +input # qqq eigen_gf lost from com


******* GLF_Transport_Input  input:
# Inputs for transport model of R. E. Waltz , NTCC version
# See also suppress_Simple, suppress_PS, and suppress_Neo.

bt_flag                          integer / 0 /
  # switch for use of Bt_eff in calculation of rhosda

alpha_d                          real / 0. /
  # switch for diamagnetic term in ExB

alpha_e                          real / 1. /
  # ExB stabilization, 0. off, 1. full on

x_alpha                          real / -1. /
  # MHD alpha, 1. user-supplied, 0 none, -1. theory

alpha_exp(nrho)                 _real
  # user-supplied MHD alpha (e.g. from experiment), used if x_alpha = 1.

leigen                           integer / 0 /
  # eigenvalue solver 0 for cgg (default), 1 for tomsqz

nroot                            integer / 12 /
  # no. roots,8 for no impurity dynamics, 12 for impurity dynamics
  # Should set this consistently with idengrad

irotstab                         integer / 0 /
  # omegaExB: use transp code's values (0); glf23 internal calc (1)

i_grad                           integer / 0 /
  # default 0, for D-V method use i_grad=1 to input gradients

idengrad                         integer / 3 /
  # dilution flag; set to 2 for no impurities, 3 for impurity dynamics
  # Should set this consistently with nroot.

i_delay                          integer / 0 /
  # time delay for ExB shear should be non-zero only
  # once per step and is less than or equal 10


******* GLF_Transport_Output2  output:
# Outputs for transport model of R. E. Waltz ...NTCC version
# See also suppress_Simple, suppress_PS, and suppress_Neo.

diffnem                          real [m^2/sec]
  # ion plasma diffusivity at last grid point glf computes

chietem                          real [m^2/sec]
  # electron ENERGY diffuivity at last grid point glf computes
chiitim                          real [m^2/sec]
  # ion ENERGY diffuivity at last grid point glf computes

etaphim                          real [m^2/sec]
  # toroidal velocity diffusivity at last grid point glf computes

etaparm                          real [m^2/sec]
  # parallel velocity diffusivity at last grid point glf computes

etaperm                          real [m^2/sec]
  # perpendicular velocity diffusivity at last grid point glf computes

exchm                            real [MW/m^3]
  # turbulent electron to ion ENERGY exchange at last grid point glf computes

chie_m(nrho)                     _real [m^2/sec]
  # electron ENERGY diffuivity

chii_m(nrho)                     _real [m^2/sec]
  # ion ENERGY diffuivity 

etaphi_m(nrho)                   _real [m^2/sec]
  # toroidal velocity diffusivity

etapar_m(nrho)                   _real [m^2/sec]
  # parallel velocity diffusivity

etaper_m(nrho)                   _real [m^2/sec]
  # perpendicular velocity diffusivity

exch_m(nrho)                     _real [MW/m^3]
  # turbulent electron to ion ENERGY exchange

egamma_m(nrho)                   _real
  # internally calculated  exb shear rate in units of local csda_m

egamma_d(nrho)                   _real
  # exb shear rate delayed by i_delay steps

gamma_p(nrho)                    _real
  # par. vel. shear rate in units of local csda_m

anrate(nrho)                     _real
  # leading mode rate in unints of local csda_m

anrate2(nrho)                    _real
  # 2nd mode rate in units of local csda_m

anfreq(nrho)                     _real
  # leading mode frequency

anfreq2(nrho)                    _real
  # 2nd mode frequency

xkymax(nrho)                     _real
  # leading mode k_y rho_s

xkymax2(nrho)                    _real
  # 2nd mode k_y rho_s

chiGLFN(nrho,Ntemps)              _real [m^2/sec]
  # GLF model chi.

diffGLFN(nrho)                    _real [m^2/sec]
  # GLF model diff ...
  # (GLF uses the same diffusion coefficient for all species.)

diff_LzGLFN(nrho)                 _real [m^2/sec]
  # GLF model angular momentum diff ...
  # diff_LzGLF = diff_LzCoef*chiGLF*(<B_pol^2>/<B^2>)^.5


******* GLF_Transport_wrapper_temps :
# locals in evalGLF, needed here to pass to callglf2d_wrap().
hydDens             real
impDens             real
arhosquared         real
bt                  real
csdacoef            real
csda                real
rho_over_r          real
te_glf(nrho)       _real
ti_glf(nrho)       _real
ne_glf(nrho)       _real
ni_glf(nrho)       _real
jmm                 integer


******* GLF_Transport_diagnostics   output:
# All of the input variables of the NTCC GLF23 callglf2d that
# are not already in database.  This is for diagnostic
# use only.   callglf2d input quantities are calculated
# in evalGLF.  To call glf23 with your own set of
# inputs, invoke callglf2d (with all of its args.)
# directly from the parser.

neGLF(nrho)                      _real [10^19 m^3/sec]
  # electron density

niGLF(nrho)                      _real [10^19 m^3/sec]
  # ion density

nbeamGLF(nrho)                   _real [10^19 m^3/sec]
  #  beam ion density

zpte_in                          real [keV]
  # log gradient te w.r.t rho

zpti_in                          real [keV]
  # log gradient ti w.r.t rho

zpne_in                          real [10^19 m^3]
  # log gradient ne w.r.t rho

zpni_in                          real [10^19 m^3]
  # log gradient ni w.r.t rho

angrotp_exp(nrho)                _real [s^-1]
  # plasma toroidal angular velocity
  # if itport_pt(4)=0 itport_pt(5)=0

egamma_exp(nrho)                 _real 
  # normalized (to cs/rho) ExB shear rate
  # if itport_pt(4)=-1 itport_pt(5)=0

gamma_p_exp(nrho)                _real 
  # normalized (to cs/rho) vparallel shear rate
  # if itport_pt(4)=-1 itport_pt(5)=0

vphilin(nrho)                    _real [m/s]
  # torodal velocity
  #  if itport_pt(4)=1 itport_pt(5)=0 otherwise output

vpar_m(nrho)                     _real [m/s]
  # parallel velocity m/sec
  # if itport_pt(4)=1 itport_pt(5)=1 otherwise output

vper_m(nrho)                     _real [m/s]
  # 0:jmaxm perp. velocity m/sec
  # if itport_pt(4)=1 itport_pt(5)=1 otherwise output

bt_axis                          real [T]
  # vaccuum axis toroidal field

bteff_exp(nrho)                 _real [T]
  #  effective toroidal field B_toroidal*(rho/r)(drho/dr)

rho_glf(nrho)                   _real [none]
  # normalized square root of toroidal flux 

arho                             real [m]
  # square root of physical toroidal flux/B_0 at rho_glf = 1

gradrho_glf(nrho)               _real [none]
  # arho*<|grad rho_glf|> (dimensionless)

gradrhosq_glf(nrho)             _real [none]
  # arho^2*<(grad rho_glf)^2>  (dimensionless)

rmajor_exp                       real [m]
  # axis major radius

zimpav                           real [none]
  # effective Z of impurity (average over plasma)

amassimpav                       real [none]
  # effective A of impurity (average over plasma)

shat_glf(nrho)                  _real [none]
  # magnetic shear rho d ln q_exp/ d rho

amassgas_glf                     real [none]
  # effective A of hydrogenic gas (average over plasma)

jshoot                           integer 
  # 0 for time-dep code; 1 for shooting code

jmaxm                            integer
  # callglf's dimension number, should be nr grid pts - 1

itport_pt(5)                     integer
  # transport flags, should be (0,1,1,1,0) for our application


******* ToroidalRotation:
omstarax                         real [rad/sec]
  # e_r2/<R Bp> = toroidal omega_* at the magnetic axis

omegaTor(nrho)                  _real [rad/sec]
  # Angular rotation frequency of the flux surface.

omegaPrTor(nrho)                _real [rad/sec]
  # d omegaTor / d rhobar

omegaExB(nrho)                  _real [rad/sec]
  # omegaExB shearing rate

er_outer(nrho)                  _real [V/m]
  # Radial electric field at outer midplane

erbp(nrho)                      _real [V/m^2 T]
  # E_r/RB_p

e_r(nrho)                       _real [V/m]
  # flux-surface averaged radial electric field, used for 
  # iEShearMod = iStaeb

e_r1(nrho)                      _real [V/m]
  # E_r due to toroidal rotation (Staebler shear model only)

e_r2(nrho)                      _real [V/m]
  # E_r due to diamagnetic drift (Steabler shear model only)

******* ErShearFields:
# Integer variables to specify starting field in Er shear calc
#  used in getOmegaExB(ifield).  Allowed ifield values are:

iEr                              integer /31/
# electric field on outer midplane (for Waltz or Hahm-Burrell
#       models) or flux-surf. averaged (for Staebler mdoel)

iErBp                            integer /32/
#  E_r/RBp

iPhiEPr                          integer /33/
# Phi_E'

iOmegTor                         integer /34/
# toroidal rotation frequency


******* ErShearMod:
# integer variable and flags to set model used to calculate OmegaExB
iEShearMod                +input integer /41/
  # flag to pick the prescription for OmegaExB.  Choices are
  #  iWaltz, iHB, or iStaeb

iWaltz                           integer /41/
  # Waltz ExB shear model (uses average quantities), see iEShearMod

iHB                              integer /42/
  # Hahm-Burrell ExB shear model, see iEShearMod

iStaeb                           integer /43/
  # Waltz ExB shear model (uses average quantities), see iEShearMod


******* AngularMomentum_Transport:
# Variables used in the axial angular momentum equation.

lzSrcTot                           real [kg m^2/s^2]
  # Integrated source of axial angular momentum.

omegaBC(3)                         real
  # Toroidal rotation (omegaTor) boundary conditions...
  # BC(1) <grad-omega> + BC(2) omega = BC(3), where omega = omegaTor
  # < . > = flux-surface-average, and <grad-omega> = c4*omegaPrTor.

lzBC_(3)                           real
  # angular momentum lzTor boundary conditions in flux coordinates...
  # BC(1) Lz' + BC(2) Lz = BC(3), w/ Lz = lzTor, Lz' = lzPrTor
  # These are calculated from the lzBC's.

diff_Lz_add                        real [m^2/s] / 0./
  # additive const to use in getting effective diff_lz from chi

chiForLzMult                       real [none] / 1. /
  # multiplier in defining diff_Lz when none in model,
  # diff_Lz_eff = chiForLzMult*(Bp/Bt)*chi(,iIon) + diff_Lz_add

diff_LzCoef                        real [none] /1.0/
  # multiplier on diff_LzRLW,KDBH, etc. models.

lzTor(nrho)                       _real [kg/m/s]
  # Axial angular momentum density on the rhobar grid...
  # lzTor = sum(m_i n_i) <R^2> V' omegaTor.
  # Total angular momentum L_z = sum(rhoWts*lzTor).

lzPrTor(nrho)                     _real
  # Rhobar derivative of lzTor on the rhobar grid.

lzPrPrTor(nrho)                   _real
  # 2nd rhobar derivative of lzTor on the rhobar grid.

lzToOmega(nrho,2)                 _real
  # Conversion factor...
  #   omegaTor   = lzToOmega(,1,) * lzTor  
  #   omegaPrTor = lzToOmega(,1,) * lzPrTor + lzToOmega(,2,) * lzTor

diff_Lz(nrho)                     _real [m^2/s]
  # Angular momentum diffusivity

useChiForDLz                       integer  / 1 /
  # flag to use chi to get effective diff_lz; scripts set to 0 for
  #  models that return a diff_Lz

vpinch_Lz(nrho)                   _real [m/s]
  # Angular momentum pinch velocity

extraLzFlux(nrho)                 _real [kg/s^2]  # / nrho*0./
  # extra (Lz-independent) flux of angular momentum

lzSrc(nrho)                       _real [kg/m/s^2]
  # Source of axial angular momentum.

nbLzSrc(nrho)                     _real [kg/m/s^2]
  # Source of axial angular momentum.

lzFluxCoefs(nrho, 3)              _real
  # Coefficients used to construct the angular momentum fluxes...
  # lzflux = -lzfc(1)*dLz/dr+lzfc(2)*Lz+lzfc(3)

lzFlux(nrho)                      _real [kg/m/s^2]
  # Radial flux of axial angular momentum.

lzHProfs(nrho, nhterms)           _real
  # "Terms" in the axial angular momentum diffusion equation.


******* Pellets:
# Pellet fueling model.

rPellet                            real [m] / 6.0e-3 /
  # Initial pellet radius.

vPellet                            real [m/sec] / 5000 /
  # Pellet velocity.

cPel                               real / 1.0 /
  # Multiplier in pellet ablation rate.

pelDFrac                           real / 0.5 /
  # Fraction of Deuterium in pellet.
pelTFrac                           real / 0.5 /
  # Fraction of Tritium in pellet.
pelHFrac                           real / 0.0 /
  # Fraction of Hydrogen in pellet.
pelSrcTot                        real [10^20/sec] / 0.0 /
  # Rate of total particle input by pellets.

nFuelTot(Nparts)                   real [10^20/sec]
  # Total generic fueling source to all species.

nFuelCoefs(6, Nparts)              real
  # Coefficients controlling generic fueling sources.

nFuel(nrho, Nparts)               _real [10^20/m^3/sec]
  # Generic fueling source density.

hydPelSrc(nrho)                   _real [10^20/m^3/sec]
  # Hydrogen source due to pellets.
deutPelSrc(nrho)                  _real [10^20/m^3/sec]
  # Deuterium source due to pellets.
tritPelSrc(nrho)                  _real [10^20/m^3/sec]
  # Tritium  source due to pellets.


******* Confinement:
# Variables used in calculation of confinement times.

ionStoredEnergy                    real [MJ]
  # Thermal energy stored in ions.
elecStoredEnergy                   real [MJ]
  # Thermal energy stored in ions.
storedEnergy                       real [MJ]
  # Total thermal stored energy.

tau_E                              real [sec]
  # Total energy confinement time.

tauScale(Nscalings)                real [sec]
  # Energy confinement times from various 0-D scalings...
  # iNeoAlc: Neo-Alcator.
  # iDIIID: DIIID ohmic.
  # iITER_P: ITER-89 power law.
  # iITER_O: ITER-89 offset-linear.
  # iRebutLallia: Rebut-Lallia.
  # iITER_PMin: Min(ITER power, Neo-Alcator)

hFacScale(Nscalings)               real
  # Enhancement of tau_E for all of the 0-D scalings.


******* Galerkin:
# Data related to Galerkin finite element solution of transport equations.

yFP(nfps)                         _real
  # Fitting parameters for the flux equation.

hrFP(nfps)                        _real
  # Fitting parameters for the hyper-resistivity term in the flux equation.

lzFP(nfps)                        _real
  # Fitting parameters for the angular momentum equation.

muFP(nfps, Ntemps)                _real
  # Fitting parameters for the entropy equations.

nFP(nfps, Nparts)                 _real
  # Fitting parameters for the density equations.

yCap(nfps, nfps, 4)                  _real
  # Capacity matrix for the flux equation.
yStiff(nfps, nfps, 4)                _real
  # Stiffness matrix for the flux equation.
yLoad(nfps, 4)                       _real
  # Load vector for the flux equation.

muCap(nfps, nfps,2*Ntemps)        _real
  # Capacity matrix for the entropy equations.
muStiff(nfps, nfps,2*Ntemps)      _real
  # Stiffness matrix for the entropy equations.
muLoad(nfps, 2*Ntemps)            _real
  # Load vector for the entropy equations.
muDrag(nfps, nfps, 2*Ntemps)      _real
  # Drag matrix for the entropy equations.

nCap(nfps, nfps, Nparts)          _real
  # Capacity matrix for the density equations.
nStiff(nfps, nfps, Nparts)        _real
  # Stiffness matrix for the density equations.
nLoad(nfps, Nparts)               _real
  # Load vector for the density equations.

lzCap(nfps, nfps)                  _real
  # Capacity matrix for the axial angular momentum equation.
lzStiff(nfps, nfps)                _real
  # Stiffness matrix for the axial angular momentum equation.
lzLoad(nfps)                       _real
  # Load vector for the axial angular momentum equation.


******* Sawtooth_Data:
# Data related to the enhanced transport model used to mock...
# up the time-averaged effects of sawtooth oscillations.

qSawtooth                         real / 1.0 /
  # The value of q(0) at which sawtooth model turns on

sawtoothRadius                    real / 0.0 /
  # The position of the last flux surface to which the model is applied.

chiEnhanceFac                     real / 1.0 /
  # Enhancement parameter for the the thermal conductivity.

chiEnhanceDelta                   real / 0.0 /
  # Distance over which the model turns on.

diffEnhanceFac                    real / 1.0 /
  # Enhancement parameter for the the diffusion coefficients.

diffEnhanceDelta                  real / 0.0 /
  # Distance over which the model turns on.

sigTorEnhanceFrac  		  real / 1.0 /
  # Fraction of resistivity enhancement to apply...
  # The default model for enhanced resistivity sets sigTor
  # inside the q == 1 surface to the value at that surface.
  # If sigEnhanceFraction ("SEF") < 1, then a weighted average
  # is made between this value ("st") and the non-enhanced
  # value ("no_st"):
  #
  #   sigTor = SEF * sigTor_st + (1-SEF) * sigTor_no_st

iSawtoothRadius                   integer / 0 /
  # The grid cell inside which the sawtooth model will be applied.

******* TransStep:

time				  real / 0.0 /
  # During a timestep, this is the time to which we are stepping...
  # Otherwise, it is the time of the last converged timestep.

dt                                real /0.0 /
  # delta time for the time step

time_last                         real / 0.0 /
  # The time at the end of the last successful timestep.


